#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Union, Dict

import serial  # type: ignore
import json


class ArduinoLeonardoDigitalInput:
    """
    ``ArduinoLeonardoDigitalInput`` reads inputs (on pins 0 .. 13) and counts edges (on pins 0, 1, 2 and 3) using pin
    change interrupts on Arduino Leonardo.

    Serial connection: 115200 baud, 8N1.
    """

    def __init__(self, port: str, reset_on_init: bool = True, timeout: Union[int, float] = 0.5) -> None:
        """
        Args:
            port: the com port the Arduino Leonardo listens to running the digital input application.
            reset_on_init=True: reset internal edge counters on initialization.
            timeout=0.5: the timeout for serial communication in seconds.
        """

        self.serial = serial.Serial(
            port=port,
            baudrate=115200,
            parity=serial.PARITY_NONE,
            stopbits=serial.STOPBITS_ONE,
            timeout=timeout,
        )
        if reset_on_init:
            self.reset()

    def get_version(self) -> bytes:
        """
        Get software version.

        Returns:
            bytes: the software version.
        """
        self.serial.flush()
        self.serial.write(b"v")
        version = self.serial.readline()
        return version  # type: ignore

    def get(self) -> Dict[str, int]:
        """
        Get current data, ie. digital input states and edge counts.

        Returns:
            dict: the current data containing digital port states and counter values.
                E.g. {'d0':1,'d1':1,'d2':0,'d3':1,'d4':1,'d5':1,'d6':1,'d7':1,'d8':1,'d9':1,'d10':1,'d11':1,'d12':1,
                'd13':1,'c0':0,'c1':0,'c2':77,'c3':102}
        """

        ret = {}

        commands = [b"g"]

        self.serial.flush()

        for command in commands:
            self.serial.write(command)
            # rawdata = self.serial.read(1024) # takes longer
            raw_data = self.serial.readline()

            data = json.loads(raw_data.decode())

            ret.update(data)

        return ret

    def reset(self) -> None:
        """
        Reset all edge counters.
        """
        self.serial.flush()
        self.serial.write(b"r")
        self.serial.readline()

    def close(self) -> None:
        """
        Close serial connection.
        """
        try:
            self.serial.close()
        except AttributeError:
            pass

    def __del__(self) -> None:
        """
        Destructor closes serial connection.
        """
        self.close()
