#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Type, Literal, Optional
from types import TracebackType

from htf.labjack.register_addresses import RegisterAddresses
from htf.labjack.helpers import locked, transactional
from htf.io.core import LabJackCore, IPv4Address, MacAddress
import threading


class T8(LabJackCore):
    """
    LabJack adapter for the T8.
    """

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        host: Optional[str] = None,
        port: int = 502,
        debuglevel: int = 0,
        timeout: float = 1.0,
        retries: int = 0,
    ) -> None:
        """
        Args:
            name: If name is set the USB device's name is checked against it. If host is None a USB device is opened.
            host: If host is set a Modbus-TCP connection is opened. Name must not be set at the same time.
            port: The Modbus-TCP port to be used
            debuglevel: If set the Modbus packets are printed
            timeout: The timeout for every request
            retries: The number of retries for Modbus communication
        """

        LabJackCore.__init__(
            self,
            name=name,
            host=host,
            port=port,
            product_id=8,
            debuglevel=debuglevel,
            timeout=timeout,
            retries=retries,
        )

    def close(self) -> None:
        self.client.close()

    def _assert_product_id(self, product_id: int) -> None:
        if int(self.product_id) != product_id:
            raise RuntimeError(
                f"The current product id {int(self.product_id)} read from "
                "{self._host} does not match the expected product id for the device (8)!"
            )

    def __enter__(self) -> threading.RLock:
        return self._transaction_lock

    def __exit__(
        self,
        exc_type: Type[BaseException],
        exc_val: BaseException,
        exc_tb: TracebackType,
    ) -> Literal[False]:  # type: ignore
        return False

    @property
    @locked
    def ain0(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN0 (AIN#(0:7)@0)

        Address: 0
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0)  # type: ignore

    @transactional
    def setup_ain0(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN0.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain0_range = range_
        self.ain0_settling_us = settling_time_us
        self.ain0_resolution_index = resolution_index

    @property
    @locked
    def ain1(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN1 (AIN#(0:7)@1)

        Address: 2
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1)  # type: ignore

    @transactional
    def setup_ain1(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN1.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain1_range = range_
        self.ain1_settling_us = settling_time_us
        self.ain1_resolution_index = resolution_index

    @property
    @locked
    def ain2(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN2 (AIN#(0:7)@2)

        Address: 4
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2)  # type: ignore

    @transactional
    def setup_ain2(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN2.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain2_range = range_
        self.ain2_settling_us = settling_time_us
        self.ain2_resolution_index = resolution_index

    @property
    @locked
    def ain3(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN3 (AIN#(0:7)@3)

        Address: 6
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3)  # type: ignore

    @transactional
    def setup_ain3(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN3.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain3_range = range_
        self.ain3_settling_us = settling_time_us
        self.ain3_resolution_index = resolution_index

    @property
    @locked
    def ain4(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN4 (AIN#(0:7)@4)

        Address: 8
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4)  # type: ignore

    @transactional
    def setup_ain4(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN4.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain4_range = range_
        self.ain4_settling_us = settling_time_us
        self.ain4_resolution_index = resolution_index

    @property
    @locked
    def ain5(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN5 (AIN#(0:7)@5)

        Address: 10
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5)  # type: ignore

    @transactional
    def setup_ain5(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN5.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain5_range = range_
        self.ain5_settling_us = settling_time_us
        self.ain5_resolution_index = resolution_index

    @property
    @locked
    def ain6(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN6 (AIN#(0:7)@6)

        Address: 12
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6)  # type: ignore

    @transactional
    def setup_ain6(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN6.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain6_range = range_
        self.ain6_settling_us = settling_time_us
        self.ain6_resolution_index = resolution_index

    @property
    @locked
    def ain7(self) -> float:
        """
        Returns the voltage of the specified analog input.

        Register: AIN7 (AIN#(0:7)@7)

        Address: 14
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7)  # type: ignore

    @transactional
    def setup_ain7(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup AIN7.

        Args:
            range_: the maximum expected voltage to automatically select the best gain
            settling_time_us: the settling time in micro seconds
            resolution_index: see LabJack docs
        """
        self.ain7_range = range_
        self.ain7_settling_us = settling_time_us
        self.ain7_resolution_index = resolution_index

    @property
    @locked
    def temperature0(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE0 (TEMPERATURE#(0:7)@0)

        Address: 600
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE0)  # type: ignore

    @property
    @locked
    def temperature1(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE1 (TEMPERATURE#(0:7)@1)

        Address: 602
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE1)  # type: ignore

    @property
    @locked
    def temperature2(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE2 (TEMPERATURE#(0:7)@2)

        Address: 604
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE2)  # type: ignore

    @property
    @locked
    def temperature3(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE3 (TEMPERATURE#(0:7)@3)

        Address: 606
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE3)  # type: ignore

    @property
    @locked
    def temperature4(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE4 (TEMPERATURE#(0:7)@4)

        Address: 608
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE4)  # type: ignore

    @property
    @locked
    def temperature5(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE5 (TEMPERATURE#(0:7)@5)

        Address: 610
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE5)  # type: ignore

    @property
    @locked
    def temperature6(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE6 (TEMPERATURE#(0:7)@6)

        Address: 612
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE6)  # type: ignore

    @property
    @locked
    def temperature7(self) -> float:
        """
        T8 Only. Returns the temperature of the specified analog input. And saves AIN and temperature inputs for all
        channels.

        Register: TEMPERATURE7 (TEMPERATURE#(0:7)@7)

        Address: 614
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE7)  # type: ignore

    @property
    @locked
    def ain0_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN0_CAPTURE (AIN#(0:7)_CAPTURE@0)

        Address: 650
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_CAPTURE)  # type: ignore

    @property
    @locked
    def ain1_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN1_CAPTURE (AIN#(0:7)_CAPTURE@1)

        Address: 652
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_CAPTURE)  # type: ignore

    @property
    @locked
    def ain2_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN2_CAPTURE (AIN#(0:7)_CAPTURE@2)

        Address: 654
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_CAPTURE)  # type: ignore

    @property
    @locked
    def ain3_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN3_CAPTURE (AIN#(0:7)_CAPTURE@3)

        Address: 656
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_CAPTURE)  # type: ignore

    @property
    @locked
    def ain4_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN4_CAPTURE (AIN#(0:7)_CAPTURE@4)

        Address: 658
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_CAPTURE)  # type: ignore

    @property
    @locked
    def ain5_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN5_CAPTURE (AIN#(0:7)_CAPTURE@5)

        Address: 660
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_CAPTURE)  # type: ignore

    @property
    @locked
    def ain6_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN6_CAPTURE (AIN#(0:7)_CAPTURE@6)

        Address: 662
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_CAPTURE)  # type: ignore

    @property
    @locked
    def ain7_capture(self) -> float:
        """
        T8 Only. Returns the saved voltage of the specified analog input.

        Register: AIN7_CAPTURE (AIN#(0:7)_CAPTURE@7)

        Address: 664
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature0_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE0_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@0)

        Address: 700
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE0_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature1_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE1_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@1)

        Address: 702
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE1_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature2_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE2_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@2)

        Address: 704
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE2_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature3_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE3_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@3)

        Address: 706
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE3_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature4_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE4_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@4)

        Address: 708
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE4_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature5_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE5_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@5)

        Address: 710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE5_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature6_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE6_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@6)

        Address: 712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE6_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature7_capture(self) -> float:
        """
        T8 Only. Returns the saved temperature of the specified analog input.

        Register: TEMPERATURE7_CAPTURE (TEMPERATURE#(0:7)_CAPTURE@7)

        Address: 714
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE7_CAPTURE)  # type: ignore

    @property
    @locked
    def dac0(self) -> float:
        """
        Pass a voltage for the specified analog output.

        Register: DAC0 (DAC#(0:1)@0)

        Address: 1000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DAC0)  # type: ignore

    @dac0.setter
    @locked
    def dac0(self, value: float) -> None:
        """
        Pass a voltage for the specified analog output.

        Register: DAC0 (DAC#(0:1)@0)

        Address: 1000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DAC0, value)

    @property
    @locked
    def dac1(self) -> float:
        """
        Pass a voltage for the specified analog output.

        Register: DAC1 (DAC#(0:1)@1)

        Address: 1002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DAC1)  # type: ignore

    @dac1.setter
    @locked
    def dac1(self, value: float) -> None:
        """
        Pass a voltage for the specified analog output.

        Register: DAC1 (DAC#(0:1)@1)

        Address: 1002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DAC1, value)

    @property
    @locked
    def fio0(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO0 (FIO#(0:7)@0)

        Address: 2000
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO0)  # type: ignore

    @fio0.setter
    @locked
    def fio0(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO0 (FIO#(0:7)@0)

        Address: 2000
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO0, value)

    @property
    @locked
    def fio1(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO1 (FIO#(0:7)@1)

        Address: 2001
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO1)  # type: ignore

    @fio1.setter
    @locked
    def fio1(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO1 (FIO#(0:7)@1)

        Address: 2001
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO1, value)

    @property
    @locked
    def fio2(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO2 (FIO#(0:7)@2)

        Address: 2002
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO2)  # type: ignore

    @fio2.setter
    @locked
    def fio2(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO2 (FIO#(0:7)@2)

        Address: 2002
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO2, value)

    @property
    @locked
    def fio3(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO3 (FIO#(0:7)@3)

        Address: 2003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO3)  # type: ignore

    @fio3.setter
    @locked
    def fio3(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO3 (FIO#(0:7)@3)

        Address: 2003
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO3, value)

    @property
    @locked
    def fio4(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO4 (FIO#(0:7)@4)

        Address: 2004
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO4)  # type: ignore

    @fio4.setter
    @locked
    def fio4(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO4 (FIO#(0:7)@4)

        Address: 2004
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO4, value)

    @property
    @locked
    def fio5(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO5 (FIO#(0:7)@5)

        Address: 2005
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO5)  # type: ignore

    @fio5.setter
    @locked
    def fio5(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO5 (FIO#(0:7)@5)

        Address: 2005
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO5, value)

    @property
    @locked
    def fio6(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO6 (FIO#(0:7)@6)

        Address: 2006
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO6)  # type: ignore

    @fio6.setter
    @locked
    def fio6(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO6 (FIO#(0:7)@6)

        Address: 2006
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO6, value)

    @property
    @locked
    def fio7(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO7 (FIO#(0:7)@7)

        Address: 2007
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO7)  # type: ignore

    @fio7.setter
    @locked
    def fio7(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: FIO7 (FIO#(0:7)@7)

        Address: 2007
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO7, value)

    @property
    @locked
    def eio0(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO0 (EIO#(0:7)@0)

        Address: 2008
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO0)  # type: ignore

    @eio0.setter
    @locked
    def eio0(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO0 (EIO#(0:7)@0)

        Address: 2008
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO0, value)

    @property
    @locked
    def eio1(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO1 (EIO#(0:7)@1)

        Address: 2009
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO1)  # type: ignore

    @eio1.setter
    @locked
    def eio1(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO1 (EIO#(0:7)@1)

        Address: 2009
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO1, value)

    @property
    @locked
    def eio2(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO2 (EIO#(0:7)@2)

        Address: 2010
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO2)  # type: ignore

    @eio2.setter
    @locked
    def eio2(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO2 (EIO#(0:7)@2)

        Address: 2010
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO2, value)

    @property
    @locked
    def eio3(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO3 (EIO#(0:7)@3)

        Address: 2011
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO3)  # type: ignore

    @eio3.setter
    @locked
    def eio3(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO3 (EIO#(0:7)@3)

        Address: 2011
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO3, value)

    @property
    @locked
    def eio4(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO4 (EIO#(0:7)@4)

        Address: 2012
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO4)  # type: ignore

    @eio4.setter
    @locked
    def eio4(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO4 (EIO#(0:7)@4)

        Address: 2012
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO4, value)

    @property
    @locked
    def eio5(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO5 (EIO#(0:7)@5)

        Address: 2013
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO5)  # type: ignore

    @eio5.setter
    @locked
    def eio5(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO5 (EIO#(0:7)@5)

        Address: 2013
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO5, value)

    @property
    @locked
    def eio6(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO6 (EIO#(0:7)@6)

        Address: 2014
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO6)  # type: ignore

    @eio6.setter
    @locked
    def eio6(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO6 (EIO#(0:7)@6)

        Address: 2014
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO6, value)

    @property
    @locked
    def eio7(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO7 (EIO#(0:7)@7)

        Address: 2015
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO7)  # type: ignore

    @eio7.setter
    @locked
    def eio7(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: EIO7 (EIO#(0:7)@7)

        Address: 2015
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO7, value)

    @property
    @locked
    def cio0(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO0 (CIO#(0:3)@0)

        Address: 2016
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO0)  # type: ignore

    @cio0.setter
    @locked
    def cio0(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO0 (CIO#(0:3)@0)

        Address: 2016
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO0, value)

    @property
    @locked
    def cio1(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO1 (CIO#(0:3)@1)

        Address: 2017
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO1)  # type: ignore

    @cio1.setter
    @locked
    def cio1(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO1 (CIO#(0:3)@1)

        Address: 2017
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO1, value)

    @property
    @locked
    def cio2(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO2 (CIO#(0:3)@2)

        Address: 2018
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO2)  # type: ignore

    @cio2.setter
    @locked
    def cio2(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO2 (CIO#(0:3)@2)

        Address: 2018
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO2, value)

    @property
    @locked
    def cio3(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO3 (CIO#(0:3)@3)

        Address: 2019
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO3)  # type: ignore

    @cio3.setter
    @locked
    def cio3(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: CIO3 (CIO#(0:3)@3)

        Address: 2019
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO3, value)

    @property
    @locked
    def mio0(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO0 (MIO#(0:2)@0)

        Address: 2020
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.MIO0)  # type: ignore

    @mio0.setter
    @locked
    def mio0(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO0 (MIO#(0:2)@0)

        Address: 2020
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.MIO0, value)

    @property
    @locked
    def mio1(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO1 (MIO#(0:2)@1)

        Address: 2021
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.MIO1)  # type: ignore

    @mio1.setter
    @locked
    def mio1(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO1 (MIO#(0:2)@1)

        Address: 2021
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.MIO1, value)

    @property
    @locked
    def mio2(self) -> int:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO2 (MIO#(0:2)@2)

        Address: 2022
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.MIO2)  # type: ignore

    @mio2.setter
    @locked
    def mio2(self, value: int) -> None:
        """
        Read or set the state of 1 bit of digital I/O. Also configures the direction to input or output. Read 0=Low
        AND 1=High. Write 0=Low AND 1=High.

        Register: MIO2 (MIO#(0:2)@2)

        Address: 2022
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.MIO2, value)

    @property
    @locked
    def fio_state(self) -> int:
        """
        Read or write the state of the 8 bits of FIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: FIO_STATE

        Address: 2500
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO_STATE)  # type: ignore

    @fio_state.setter
    @locked
    def fio_state(self, value: int) -> None:
        """
        Read or write the state of the 8 bits of FIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: FIO_STATE

        Address: 2500
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO_STATE, value)

    @property
    @locked
    def eio_state(self) -> int:
        """
        Read or write the state of the 8 bits of EIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: EIO_STATE

        Address: 2501
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO_STATE)  # type: ignore

    @eio_state.setter
    @locked
    def eio_state(self, value: int) -> None:
        """
        Read or write the state of the 8 bits of EIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: EIO_STATE

        Address: 2501
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO_STATE, value)

    @property
    @locked
    def cio_state(self) -> int:
        """
        Read or write the state of the 4 bits of CIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: CIO_STATE

        Address: 2502
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO_STATE)  # type: ignore

    @cio_state.setter
    @locked
    def cio_state(self, value: int) -> None:
        """
        Read or write the state of the 4 bits of CIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: CIO_STATE

        Address: 2502
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO_STATE, value)

    @property
    @locked
    def mio_state(self) -> int:
        """
        Read or write the state of the 3 bits of MIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: MIO_STATE

        Address: 2503
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.MIO_STATE)  # type: ignore

    @mio_state.setter
    @locked
    def mio_state(self, value: int) -> None:
        """
        Read or write the state of the 3 bits of MIO in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. The upper 8-bits of this value are inhibits.

        Register: MIO_STATE

        Address: 2503
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.MIO_STATE, value)

    @property
    @locked
    def fio_eio_state(self) -> int:
        """
        Read or write the state of the 16 bits of FIO-EIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written.

        Register: FIO_EIO_STATE

        Address: 2580
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO_EIO_STATE)  # type: ignore

    @fio_eio_state.setter
    @locked
    def fio_eio_state(self, value: int) -> None:
        """
        Read or write the state of the 16 bits of FIO-EIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written.

        Register: FIO_EIO_STATE

        Address: 2580
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO_EIO_STATE, value)

    @property
    @locked
    def eio_cio_state(self) -> int:
        """
        Read or write the state of the 12 bits of EIO-CIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. As of firmware 1.0172, MIO states are included in the upper nibble of
        the CIO byte.

        Register: EIO_CIO_STATE

        Address: 2581
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO_CIO_STATE)  # type: ignore

    @eio_cio_state.setter
    @locked
    def eio_cio_state(self, value: int) -> None:
        """
        Read or write the state of the 12 bits of EIO-CIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written. As of firmware 1.0172, MIO states are included in the upper nibble of
        the CIO byte.

        Register: EIO_CIO_STATE

        Address: 2581
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO_CIO_STATE, value)

    @property
    @locked
    def cio_mio_state(self) -> int:
        """
        Read or write the state of the 12 bits of CIO-MIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written.

        Register: CIO_MIO_STATE

        Address: 2582
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO_MIO_STATE)  # type: ignore

    @cio_mio_state.setter
    @locked
    def cio_mio_state(self, value: int) -> None:
        """
        Read or write the state of the 12 bits of CIO-MIO in a single binary-encoded value. 0=Low AND 1=High. Does
        not configure direction. Reading lines set to output returns the current logic levels on the terminals, not
        necessarily the output states written.

        Register: CIO_MIO_STATE

        Address: 2582
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO_MIO_STATE, value)

    @property
    @locked
    def fio_direction(self) -> int:
        """
        Read or write the direction of the 8 bits of FIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: FIO_DIRECTION

        Address: 2600
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FIO_DIRECTION)  # type: ignore

    @fio_direction.setter
    @locked
    def fio_direction(self, value: int) -> None:
        """
        Read or write the direction of the 8 bits of FIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: FIO_DIRECTION

        Address: 2600
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FIO_DIRECTION, value)

    @property
    @locked
    def eio_direction(self) -> int:
        """
        Read or write the direction of the 8 bits of EIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: EIO_DIRECTION

        Address: 2601
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.EIO_DIRECTION)  # type: ignore

    @eio_direction.setter
    @locked
    def eio_direction(self, value: int) -> None:
        """
        Read or write the direction of the 8 bits of EIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: EIO_DIRECTION

        Address: 2601
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.EIO_DIRECTION, value)

    @property
    @locked
    def cio_direction(self) -> int:
        """
        Read or write the direction of the 4 bits of CIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: CIO_DIRECTION

        Address: 2602
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CIO_DIRECTION)  # type: ignore

    @cio_direction.setter
    @locked
    def cio_direction(self, value: int) -> None:
        """
        Read or write the direction of the 4 bits of CIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: CIO_DIRECTION

        Address: 2602
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CIO_DIRECTION, value)

    @property
    @locked
    def mio_direction(self) -> int:
        """
        Read or write the direction of the 3 bits of MIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: MIO_DIRECTION

        Address: 2603
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.MIO_DIRECTION)  # type: ignore

    @mio_direction.setter
    @locked
    def mio_direction(self, value: int) -> None:
        """
        Read or write the direction of the 3 bits of MIO in a single binary-encoded value. 0=Input and 1=Output. The
        upper 8-bits of this value are inhibits.

        Register: MIO_DIRECTION

        Address: 2603
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.MIO_DIRECTION, value)

    @property
    @locked
    def dio_state(self) -> int:
        """
        Read or write the state of all digital I/O in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. A read of an output returns the current logic level on the terminal, not necessarily the
        output state written. Writes are filtered by the value in DIO_INHIBIT.

        Register: DIO_STATE

        Address: 2800
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_STATE)  # type: ignore

    @dio_state.setter
    @locked
    def dio_state(self, value: int) -> None:
        """
        Read or write the state of all digital I/O in a single binary-encoded value. 0=Low AND 1=High. Does not
        configure direction. A read of an output returns the current logic level on the terminal, not necessarily the
        output state written. Writes are filtered by the value in DIO_INHIBIT.

        Register: DIO_STATE

        Address: 2800
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_STATE, value)

    @property
    @locked
    def dio_direction(self) -> int:
        """
        Read or write the direction of all digital I/O in a single binary-encoded value. 0=Input and 1=Output.
        Writes are filtered by the value in DIO_INHIBIT.

        Register: DIO_DIRECTION

        Address: 2850
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_DIRECTION)  # type: ignore

    @dio_direction.setter
    @locked
    def dio_direction(self, value: int) -> None:
        """
        Read or write the direction of all digital I/O in a single binary-encoded value. 0=Input and 1=Output.
        Writes are filtered by the value in DIO_INHIBIT.

        Register: DIO_DIRECTION

        Address: 2850
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_DIRECTION, value)

    @property
    @locked
    def dio_pulldown_enable(self) -> int:
        """
        T8 Only. This register will enable pulldowns on DIO lines. This is a binary coded value where bit 0
        represent FIO0, and bit 11 represents EIO3, etc. 1 = pulldown enabled, 0 = pulldown disabled. This register
        only affects flex-lines which can be configured as analog or digital. This register is not affected by the
        inhibit register.

        Register: DIO_PULLDOWN_ENABLE

        Address: 2870
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_PULLDOWN_ENABLE)  # type: ignore

    @dio_pulldown_enable.setter
    @locked
    def dio_pulldown_enable(self, value: int) -> None:
        """
        T8 Only. This register will enable pulldowns on DIO lines. This is a binary coded value where bit 0
        represent FIO0, and bit 11 represents EIO3, etc. 1 = pulldown enabled, 0 = pulldown disabled. This register
        only affects flex-lines which can be configured as analog or digital. This register is not affected by the
        inhibit register.

        Register: DIO_PULLDOWN_ENABLE

        Address: 2870
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_PULLDOWN_ENABLE, value)

    @property
    @locked
    def dio_pullup_disable(self) -> int:
        """
        This register will prevent pullups from being enabled on lines set to digital input. This is a binary coded
        value where bit 0 represent FIO0 and bit 11 represents EIO3. 1 = pullup disabled, 0 = pullup enabled. This
        register is not affected by the inhibit register.

        Register: DIO_PULLUP_DISABLE

        Address: 2890
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_PULLUP_DISABLE)  # type: ignore

    @dio_pullup_disable.setter
    @locked
    def dio_pullup_disable(self, value: int) -> None:
        """
        This register will prevent pullups from being enabled on lines set to digital input. This is a binary coded
        value where bit 0 represent FIO0 and bit 11 represents EIO3. 1 = pullup disabled, 0 = pullup enabled. This
        register is not affected by the inhibit register.

        Register: DIO_PULLUP_DISABLE

        Address: 2890
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_PULLUP_DISABLE, value)

    @property
    @locked
    def dio_inhibit(self) -> int:
        """
        A single binary-encoded value where each bit determines whether _STATE, _DIRECTION or _ANALOG_ENABLE writes
        affect that bit of digital I/O. 0=Default=Affected, 1=Ignored.

        Register: DIO_INHIBIT

        Address: 2900
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_INHIBIT)  # type: ignore

    @dio_inhibit.setter
    @locked
    def dio_inhibit(self, value: int) -> None:
        """
        A single binary-encoded value where each bit determines whether _STATE, _DIRECTION or _ANALOG_ENABLE writes
        affect that bit of digital I/O. 0=Default=Affected, 1=Ignored.

        Register: DIO_INHIBIT

        Address: 2900
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_INHIBIT, value)

    @property
    @locked
    def led_comm(self) -> int:
        """
        Sets the state of the COMM LED when the LEDs are set to manual, see the POWER_LED register.

        Register: LED_COMM

        Address: 2990
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LED_COMM)  # type: ignore

    @led_comm.setter
    @locked
    def led_comm(self, value: int) -> None:
        """
        Sets the state of the COMM LED when the LEDs are set to manual, see the POWER_LED register.

        Register: LED_COMM

        Address: 2990
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.LED_COMM, value)

    @property
    @locked
    def led_status(self) -> int:
        """
        Sets the state of the STATUS LED when the LEDs are set to manual, see the POWER_LED register.

        Register: LED_STATUS

        Address: 2991
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LED_STATUS)  # type: ignore

    @led_status.setter
    @locked
    def led_status(self, value: int) -> None:
        """
        Sets the state of the STATUS LED when the LEDs are set to manual, see the POWER_LED register.

        Register: LED_STATUS

        Address: 2991
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.LED_STATUS, value)

    @property
    @locked
    def stream_scanrate_hz(self) -> float:
        """
        Write a value to specify the number of times per second that all channels in the stream scanlist will be
        read. Max stream speeds are based on Sample Rate which is NumChannels*ScanRate. Has no effect when using an
        external clock. A read of this register returns the actual scan rate, which can be slightly different due to
        rounding. For scan rates >152.588, the actual scan interval is multiples of 100 ns. Assuming a core clock of 80
        MHz the internal roll value is (80M/(8*DesiredScanRate))-1 and the actual scan rate is then
        80M/(8*(RollValue+1). For slower scan rates the scan interval resolution is changed to 1 us, 10 us, 100 us, or
        1 ms as needed to achieve the longer intervals.

        Register: STREAM_SCANRATE_HZ

        Address: 4002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_SCANRATE_HZ)  # type: ignore

    @stream_scanrate_hz.setter
    @locked
    def stream_scanrate_hz(self, value: float) -> None:
        """
        Write a value to specify the number of times per second that all channels in the stream scanlist will be
        read. Max stream speeds are based on Sample Rate which is NumChannels*ScanRate. Has no effect when using an
        external clock. A read of this register returns the actual scan rate, which can be slightly different due to
        rounding. For scan rates >152.588, the actual scan interval is multiples of 100 ns. Assuming a core clock of 80
        MHz the internal roll value is (80M/(8*DesiredScanRate))-1 and the actual scan rate is then
        80M/(8*(RollValue+1). For slower scan rates the scan interval resolution is changed to 1 us, 10 us, 100 us, or
        1 ms as needed to achieve the longer intervals.

        Register: STREAM_SCANRATE_HZ

        Address: 4002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_SCANRATE_HZ, value)

    @property
    @locked
    def stream_num_addresses(self) -> int:
        """
        The number of entries in the scanlist.

        Register: STREAM_NUM_ADDRESSES

        Address: 4004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_NUM_ADDRESSES)  # type: ignore

    @stream_num_addresses.setter
    @locked
    def stream_num_addresses(self, value: int) -> None:
        """
        The number of entries in the scanlist.

        Register: STREAM_NUM_ADDRESSES

        Address: 4004
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_NUM_ADDRESSES, value)

    @property
    @locked
    def stream_samples_per_packet(self) -> int:
        """
        Specifies the number of data points to be sent in the data packet. Only applies to spontaneous mode.

        Register: STREAM_SAMPLES_PER_PACKET

        Address: 4006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SAMPLES_PER_PACKET)  # type: ignore

    @stream_samples_per_packet.setter
    @locked
    def stream_samples_per_packet(self, value: int) -> None:
        """
        Specifies the number of data points to be sent in the data packet. Only applies to spontaneous mode.

        Register: STREAM_SAMPLES_PER_PACKET

        Address: 4006
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SAMPLES_PER_PACKET, value)

    @property
    @locked
    def stream_settling_us(self) -> float:
        """
        Time in microseconds to allow signals to settle after switching the mux. Does not apply to the 1st channel
        in the scan list, as that settling is controlled by scan rate (the time from the last channel until the start
        of the next scan). Default=0. When set to less than 1, automatic settling will be used. The automatic settling
        behavior varies by device.

        Register: STREAM_SETTLING_US

        Address: 4008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_SETTLING_US)  # type: ignore

    @stream_settling_us.setter
    @locked
    def stream_settling_us(self, value: float) -> None:
        """
        Time in microseconds to allow signals to settle after switching the mux. Does not apply to the 1st channel
        in the scan list, as that settling is controlled by scan rate (the time from the last channel until the start
        of the next scan). Default=0. When set to less than 1, automatic settling will be used. The automatic settling
        behavior varies by device.

        Register: STREAM_SETTLING_US

        Address: 4008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_SETTLING_US, value)

    @property
    @locked
    def stream_resolution_index(self) -> int:
        """
        The resolution index for stream readings. A larger resolution index generally results in lower noise and
        longer sample times.

        Register: STREAM_RESOLUTION_INDEX

        Address: 4010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_RESOLUTION_INDEX)  # type: ignore

    @stream_resolution_index.setter
    @locked
    def stream_resolution_index(self, value: int) -> None:
        """
        The resolution index for stream readings. A larger resolution index generally results in lower noise and
        longer sample times.

        Register: STREAM_RESOLUTION_INDEX

        Address: 4010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_RESOLUTION_INDEX, value)

    @property
    @locked
    def stream_buffer_size_bytes(self) -> int:
        """
        Size of the stream data buffer in bytes. A value of 0 equates to the default value. Must be a power of 2.
        Size in samples is STREAM_BUFFER_SIZE_BYTES/2. Size in scans is
        (STREAM_BUFFER_SIZE_BYTES/2)/STREAM_NUM_ADDRESSES. Changes while stream is running do not affect the currently
        running stream.

        Register: STREAM_BUFFER_SIZE_BYTES

        Address: 4012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_BUFFER_SIZE_BYTES)  # type: ignore

    @stream_buffer_size_bytes.setter
    @locked
    def stream_buffer_size_bytes(self, value: int) -> None:
        """
        Size of the stream data buffer in bytes. A value of 0 equates to the default value. Must be a power of 2.
        Size in samples is STREAM_BUFFER_SIZE_BYTES/2. Size in scans is
        (STREAM_BUFFER_SIZE_BYTES/2)/STREAM_NUM_ADDRESSES. Changes while stream is running do not affect the currently
        running stream.

        Register: STREAM_BUFFER_SIZE_BYTES

        Address: 4012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_BUFFER_SIZE_BYTES, value)

    @property
    @locked
    def stream_clock_source(self) -> int:
        """
        Controls which clock source will be used to run the main stream clock. 0 = Internal crystal, 2 = External
        clock source on CIO3. Rising edges will increment a counter and trigger a stream scan after the number of edges
        specified in STREAM_EXTERNAL_CLOCK_DIVISOR. T7 will expect external stream clock on CIO3. All other values
        reserved.

        Register: STREAM_CLOCK_SOURCE

        Address: 4014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_CLOCK_SOURCE)  # type: ignore

    @stream_clock_source.setter
    @locked
    def stream_clock_source(self, value: int) -> None:
        """
        Controls which clock source will be used to run the main stream clock. 0 = Internal crystal, 2 = External
        clock source on CIO3. Rising edges will increment a counter and trigger a stream scan after the number of edges
        specified in STREAM_EXTERNAL_CLOCK_DIVISOR. T7 will expect external stream clock on CIO3. All other values
        reserved.

        Register: STREAM_CLOCK_SOURCE

        Address: 4014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_CLOCK_SOURCE, value)

    @property
    @locked
    def stream_auto_target(self) -> int:
        """
        Controls where data will be sent.
        Value is a bitmask.
        bit 0:
        1 = Send to Ethernet 702 sockets,
        bit 1:
        1 = Send to USB,
        bit 4:
        1 = Command-Response mode.
        All other bits are reserved.

        Register: STREAM_AUTO_TARGET

        Address: 4016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_AUTO_TARGET)  # type: ignore

    @stream_auto_target.setter
    @locked
    def stream_auto_target(self, value: int) -> None:
        """
        Controls where data will be sent.
        Value is a bitmask.
        bit 0:
        1 = Send to Ethernet 702 sockets,
        bit 1:
        1 = Send to USB,
        bit 4:
        1 = Command-Response mode.
        All other bits are reserved.

        Register: STREAM_AUTO_TARGET

        Address: 4016
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_AUTO_TARGET, value)

    @property
    @locked
    def stream_num_scans(self) -> int:
        """
        The number of scans to run before automatically stopping (stream-burst). 0 = run continuously. Limit for
        STREAM_NUM_SCANS is 2^32-1, but if the host is not reading data as fast as it is acquired you also need to
        consider STREAM_BUFFER_SIZE_BYTES.

        Register: STREAM_NUM_SCANS

        Address: 4020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_NUM_SCANS)  # type: ignore

    @stream_num_scans.setter
    @locked
    def stream_num_scans(self, value: int) -> None:
        """
        The number of scans to run before automatically stopping (stream-burst). 0 = run continuously. Limit for
        STREAM_NUM_SCANS is 2^32-1, but if the host is not reading data as fast as it is acquired you also need to
        consider STREAM_BUFFER_SIZE_BYTES.

        Register: STREAM_NUM_SCANS

        Address: 4020
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_NUM_SCANS, value)

    @property
    @locked
    def stream_external_clock_divisor(self) -> int:
        """
        The number of pulses per stream scan when using an external clock.

        Register: STREAM_EXTERNAL_CLOCK_DIVISOR

        Address: 4022
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.STREAM_EXTERNAL_CLOCK_DIVISOR
        )

    @stream_external_clock_divisor.setter
    @locked
    def stream_external_clock_divisor(self, value: int) -> None:
        """
        The number of pulses per stream scan when using an external clock.

        Register: STREAM_EXTERNAL_CLOCK_DIVISOR

        Address: 4022
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_EXTERNAL_CLOCK_DIVISOR, value)

    @property
    @locked
    def stream_trigger_index(self) -> int:
        """
        Controls when stream scanning will start. 0 = Start when stream is enabled, 2000 = Start when DIO_EF0
        detects an edge, 2001 = Start when DIO_EF1 detects an edge. See the stream documentation for all supported
        values.

        Register: STREAM_TRIGGER_INDEX

        Address: 4024
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_TRIGGER_INDEX)  # type: ignore

    @stream_trigger_index.setter
    @locked
    def stream_trigger_index(self, value: int) -> None:
        """
        Controls when stream scanning will start. 0 = Start when stream is enabled, 2000 = Start when DIO_EF0
        detects an edge, 2001 = Start when DIO_EF1 detects an edge. See the stream documentation for all supported
        values.

        Register: STREAM_TRIGGER_INDEX

        Address: 4024
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_TRIGGER_INDEX, value)

    @property
    @locked
    def stream_start_time_stamp(self) -> int:
        """
        This register stores the value of CORE_TIMER at the start of the first stream scan. Note that the first
        stream scan happens 1 scan period after STREAM_ENABLE is set to 1.

        Register: STREAM_START_TIME_STAMP

        Address: 4026
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_START_TIME_STAMP)  # type: ignore

    @property
    @locked
    def stream_autorecover_disable(self) -> int:
        """
        When the stream buffer becomes full the stream will be stopped. Same behavior as if STREAM_ENABLE is set to
        zero.

        Register: STREAM_AUTORECOVER_DISABLE

        Address: 4028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_AUTORECOVER_DISABLE)  # type: ignore

    @stream_autorecover_disable.setter
    @locked
    def stream_autorecover_disable(self, value: int) -> None:
        """
        When the stream buffer becomes full the stream will be stopped. Same behavior as if STREAM_ENABLE is set to
        zero.

        Register: STREAM_AUTORECOVER_DISABLE

        Address: 4028
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_AUTORECOVER_DISABLE, value)

    @property
    @locked
    def stream_scanlist_address0(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS0 (STREAM_SCANLIST_ADDRESS#(0:127)@0)

        Address: 4100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS0)  # type: ignore

    @stream_scanlist_address0.setter
    @locked
    def stream_scanlist_address0(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS0 (STREAM_SCANLIST_ADDRESS#(0:127)@0)

        Address: 4100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS0, value)

    @property
    @locked
    def stream_scanlist_address1(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS1 (STREAM_SCANLIST_ADDRESS#(0:127)@1)

        Address: 4102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS1)  # type: ignore

    @stream_scanlist_address1.setter
    @locked
    def stream_scanlist_address1(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS1 (STREAM_SCANLIST_ADDRESS#(0:127)@1)

        Address: 4102
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS1, value)

    @property
    @locked
    def stream_scanlist_address2(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS2 (STREAM_SCANLIST_ADDRESS#(0:127)@2)

        Address: 4104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS2)  # type: ignore

    @stream_scanlist_address2.setter
    @locked
    def stream_scanlist_address2(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS2 (STREAM_SCANLIST_ADDRESS#(0:127)@2)

        Address: 4104
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS2, value)

    @property
    @locked
    def stream_scanlist_address3(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS3 (STREAM_SCANLIST_ADDRESS#(0:127)@3)

        Address: 4106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS3)  # type: ignore

    @stream_scanlist_address3.setter
    @locked
    def stream_scanlist_address3(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS3 (STREAM_SCANLIST_ADDRESS#(0:127)@3)

        Address: 4106
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS3, value)

    @property
    @locked
    def stream_scanlist_address4(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS4 (STREAM_SCANLIST_ADDRESS#(0:127)@4)

        Address: 4108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS4)  # type: ignore

    @stream_scanlist_address4.setter
    @locked
    def stream_scanlist_address4(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS4 (STREAM_SCANLIST_ADDRESS#(0:127)@4)

        Address: 4108
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS4, value)

    @property
    @locked
    def stream_scanlist_address5(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS5 (STREAM_SCANLIST_ADDRESS#(0:127)@5)

        Address: 4110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS5)  # type: ignore

    @stream_scanlist_address5.setter
    @locked
    def stream_scanlist_address5(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS5 (STREAM_SCANLIST_ADDRESS#(0:127)@5)

        Address: 4110
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS5, value)

    @property
    @locked
    def stream_scanlist_address6(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS6 (STREAM_SCANLIST_ADDRESS#(0:127)@6)

        Address: 4112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS6)  # type: ignore

    @stream_scanlist_address6.setter
    @locked
    def stream_scanlist_address6(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS6 (STREAM_SCANLIST_ADDRESS#(0:127)@6)

        Address: 4112
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS6, value)

    @property
    @locked
    def stream_scanlist_address7(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS7 (STREAM_SCANLIST_ADDRESS#(0:127)@7)

        Address: 4114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS7)  # type: ignore

    @stream_scanlist_address7.setter
    @locked
    def stream_scanlist_address7(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS7 (STREAM_SCANLIST_ADDRESS#(0:127)@7)

        Address: 4114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS7, value)

    @property
    @locked
    def stream_scanlist_address8(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS8 (STREAM_SCANLIST_ADDRESS#(0:127)@8)

        Address: 4116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS8)  # type: ignore

    @stream_scanlist_address8.setter
    @locked
    def stream_scanlist_address8(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS8 (STREAM_SCANLIST_ADDRESS#(0:127)@8)

        Address: 4116
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS8, value)

    @property
    @locked
    def stream_scanlist_address9(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS9 (STREAM_SCANLIST_ADDRESS#(0:127)@9)

        Address: 4118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS9)  # type: ignore

    @stream_scanlist_address9.setter
    @locked
    def stream_scanlist_address9(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS9 (STREAM_SCANLIST_ADDRESS#(0:127)@9)

        Address: 4118
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS9, value)

    @property
    @locked
    def stream_scanlist_address10(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS10 (STREAM_SCANLIST_ADDRESS#(0:127)@10)

        Address: 4120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS10)  # type: ignore

    @stream_scanlist_address10.setter
    @locked
    def stream_scanlist_address10(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS10 (STREAM_SCANLIST_ADDRESS#(0:127)@10)

        Address: 4120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS10, value)

    @property
    @locked
    def stream_scanlist_address11(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS11 (STREAM_SCANLIST_ADDRESS#(0:127)@11)

        Address: 4122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS11)  # type: ignore

    @stream_scanlist_address11.setter
    @locked
    def stream_scanlist_address11(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS11 (STREAM_SCANLIST_ADDRESS#(0:127)@11)

        Address: 4122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS11, value)

    @property
    @locked
    def stream_scanlist_address12(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS12 (STREAM_SCANLIST_ADDRESS#(0:127)@12)

        Address: 4124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS12)  # type: ignore

    @stream_scanlist_address12.setter
    @locked
    def stream_scanlist_address12(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS12 (STREAM_SCANLIST_ADDRESS#(0:127)@12)

        Address: 4124
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS12, value)

    @property
    @locked
    def stream_scanlist_address13(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS13 (STREAM_SCANLIST_ADDRESS#(0:127)@13)

        Address: 4126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS13)  # type: ignore

    @stream_scanlist_address13.setter
    @locked
    def stream_scanlist_address13(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS13 (STREAM_SCANLIST_ADDRESS#(0:127)@13)

        Address: 4126
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS13, value)

    @property
    @locked
    def stream_scanlist_address14(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS14 (STREAM_SCANLIST_ADDRESS#(0:127)@14)

        Address: 4128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS14)  # type: ignore

    @stream_scanlist_address14.setter
    @locked
    def stream_scanlist_address14(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS14 (STREAM_SCANLIST_ADDRESS#(0:127)@14)

        Address: 4128
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS14, value)

    @property
    @locked
    def stream_scanlist_address15(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS15 (STREAM_SCANLIST_ADDRESS#(0:127)@15)

        Address: 4130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS15)  # type: ignore

    @stream_scanlist_address15.setter
    @locked
    def stream_scanlist_address15(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS15 (STREAM_SCANLIST_ADDRESS#(0:127)@15)

        Address: 4130
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS15, value)

    @property
    @locked
    def stream_scanlist_address16(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS16 (STREAM_SCANLIST_ADDRESS#(0:127)@16)

        Address: 4132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS16)  # type: ignore

    @stream_scanlist_address16.setter
    @locked
    def stream_scanlist_address16(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS16 (STREAM_SCANLIST_ADDRESS#(0:127)@16)

        Address: 4132
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS16, value)

    @property
    @locked
    def stream_scanlist_address17(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS17 (STREAM_SCANLIST_ADDRESS#(0:127)@17)

        Address: 4134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS17)  # type: ignore

    @stream_scanlist_address17.setter
    @locked
    def stream_scanlist_address17(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS17 (STREAM_SCANLIST_ADDRESS#(0:127)@17)

        Address: 4134
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS17, value)

    @property
    @locked
    def stream_scanlist_address18(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS18 (STREAM_SCANLIST_ADDRESS#(0:127)@18)

        Address: 4136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS18)  # type: ignore

    @stream_scanlist_address18.setter
    @locked
    def stream_scanlist_address18(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS18 (STREAM_SCANLIST_ADDRESS#(0:127)@18)

        Address: 4136
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS18, value)

    @property
    @locked
    def stream_scanlist_address19(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS19 (STREAM_SCANLIST_ADDRESS#(0:127)@19)

        Address: 4138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS19)  # type: ignore

    @stream_scanlist_address19.setter
    @locked
    def stream_scanlist_address19(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS19 (STREAM_SCANLIST_ADDRESS#(0:127)@19)

        Address: 4138
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS19, value)

    @property
    @locked
    def stream_scanlist_address20(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS20 (STREAM_SCANLIST_ADDRESS#(0:127)@20)

        Address: 4140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS20)  # type: ignore

    @stream_scanlist_address20.setter
    @locked
    def stream_scanlist_address20(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS20 (STREAM_SCANLIST_ADDRESS#(0:127)@20)

        Address: 4140
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS20, value)

    @property
    @locked
    def stream_scanlist_address21(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS21 (STREAM_SCANLIST_ADDRESS#(0:127)@21)

        Address: 4142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS21)  # type: ignore

    @stream_scanlist_address21.setter
    @locked
    def stream_scanlist_address21(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS21 (STREAM_SCANLIST_ADDRESS#(0:127)@21)

        Address: 4142
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS21, value)

    @property
    @locked
    def stream_scanlist_address22(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS22 (STREAM_SCANLIST_ADDRESS#(0:127)@22)

        Address: 4144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS22)  # type: ignore

    @stream_scanlist_address22.setter
    @locked
    def stream_scanlist_address22(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS22 (STREAM_SCANLIST_ADDRESS#(0:127)@22)

        Address: 4144
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS22, value)

    @property
    @locked
    def stream_scanlist_address23(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS23 (STREAM_SCANLIST_ADDRESS#(0:127)@23)

        Address: 4146
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS23)  # type: ignore

    @stream_scanlist_address23.setter
    @locked
    def stream_scanlist_address23(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS23 (STREAM_SCANLIST_ADDRESS#(0:127)@23)

        Address: 4146
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS23, value)

    @property
    @locked
    def stream_scanlist_address24(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS24 (STREAM_SCANLIST_ADDRESS#(0:127)@24)

        Address: 4148
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS24)  # type: ignore

    @stream_scanlist_address24.setter
    @locked
    def stream_scanlist_address24(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS24 (STREAM_SCANLIST_ADDRESS#(0:127)@24)

        Address: 4148
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS24, value)

    @property
    @locked
    def stream_scanlist_address25(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS25 (STREAM_SCANLIST_ADDRESS#(0:127)@25)

        Address: 4150
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS25)  # type: ignore

    @stream_scanlist_address25.setter
    @locked
    def stream_scanlist_address25(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS25 (STREAM_SCANLIST_ADDRESS#(0:127)@25)

        Address: 4150
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS25, value)

    @property
    @locked
    def stream_scanlist_address26(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS26 (STREAM_SCANLIST_ADDRESS#(0:127)@26)

        Address: 4152
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS26)  # type: ignore

    @stream_scanlist_address26.setter
    @locked
    def stream_scanlist_address26(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS26 (STREAM_SCANLIST_ADDRESS#(0:127)@26)

        Address: 4152
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS26, value)

    @property
    @locked
    def stream_scanlist_address27(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS27 (STREAM_SCANLIST_ADDRESS#(0:127)@27)

        Address: 4154
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS27)  # type: ignore

    @stream_scanlist_address27.setter
    @locked
    def stream_scanlist_address27(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS27 (STREAM_SCANLIST_ADDRESS#(0:127)@27)

        Address: 4154
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS27, value)

    @property
    @locked
    def stream_scanlist_address28(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS28 (STREAM_SCANLIST_ADDRESS#(0:127)@28)

        Address: 4156
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS28)  # type: ignore

    @stream_scanlist_address28.setter
    @locked
    def stream_scanlist_address28(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS28 (STREAM_SCANLIST_ADDRESS#(0:127)@28)

        Address: 4156
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS28, value)

    @property
    @locked
    def stream_scanlist_address29(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS29 (STREAM_SCANLIST_ADDRESS#(0:127)@29)

        Address: 4158
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS29)  # type: ignore

    @stream_scanlist_address29.setter
    @locked
    def stream_scanlist_address29(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS29 (STREAM_SCANLIST_ADDRESS#(0:127)@29)

        Address: 4158
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS29, value)

    @property
    @locked
    def stream_scanlist_address30(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS30 (STREAM_SCANLIST_ADDRESS#(0:127)@30)

        Address: 4160
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS30)  # type: ignore

    @stream_scanlist_address30.setter
    @locked
    def stream_scanlist_address30(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS30 (STREAM_SCANLIST_ADDRESS#(0:127)@30)

        Address: 4160
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS30, value)

    @property
    @locked
    def stream_scanlist_address31(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS31 (STREAM_SCANLIST_ADDRESS#(0:127)@31)

        Address: 4162
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS31)  # type: ignore

    @stream_scanlist_address31.setter
    @locked
    def stream_scanlist_address31(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS31 (STREAM_SCANLIST_ADDRESS#(0:127)@31)

        Address: 4162
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS31, value)

    @property
    @locked
    def stream_scanlist_address32(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS32 (STREAM_SCANLIST_ADDRESS#(0:127)@32)

        Address: 4164
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS32)  # type: ignore

    @stream_scanlist_address32.setter
    @locked
    def stream_scanlist_address32(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS32 (STREAM_SCANLIST_ADDRESS#(0:127)@32)

        Address: 4164
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS32, value)

    @property
    @locked
    def stream_scanlist_address33(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS33 (STREAM_SCANLIST_ADDRESS#(0:127)@33)

        Address: 4166
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS33)  # type: ignore

    @stream_scanlist_address33.setter
    @locked
    def stream_scanlist_address33(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS33 (STREAM_SCANLIST_ADDRESS#(0:127)@33)

        Address: 4166
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS33, value)

    @property
    @locked
    def stream_scanlist_address34(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS34 (STREAM_SCANLIST_ADDRESS#(0:127)@34)

        Address: 4168
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS34)  # type: ignore

    @stream_scanlist_address34.setter
    @locked
    def stream_scanlist_address34(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS34 (STREAM_SCANLIST_ADDRESS#(0:127)@34)

        Address: 4168
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS34, value)

    @property
    @locked
    def stream_scanlist_address35(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS35 (STREAM_SCANLIST_ADDRESS#(0:127)@35)

        Address: 4170
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS35)  # type: ignore

    @stream_scanlist_address35.setter
    @locked
    def stream_scanlist_address35(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS35 (STREAM_SCANLIST_ADDRESS#(0:127)@35)

        Address: 4170
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS35, value)

    @property
    @locked
    def stream_scanlist_address36(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS36 (STREAM_SCANLIST_ADDRESS#(0:127)@36)

        Address: 4172
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS36)  # type: ignore

    @stream_scanlist_address36.setter
    @locked
    def stream_scanlist_address36(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS36 (STREAM_SCANLIST_ADDRESS#(0:127)@36)

        Address: 4172
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS36, value)

    @property
    @locked
    def stream_scanlist_address37(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS37 (STREAM_SCANLIST_ADDRESS#(0:127)@37)

        Address: 4174
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS37)  # type: ignore

    @stream_scanlist_address37.setter
    @locked
    def stream_scanlist_address37(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS37 (STREAM_SCANLIST_ADDRESS#(0:127)@37)

        Address: 4174
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS37, value)

    @property
    @locked
    def stream_scanlist_address38(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS38 (STREAM_SCANLIST_ADDRESS#(0:127)@38)

        Address: 4176
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS38)  # type: ignore

    @stream_scanlist_address38.setter
    @locked
    def stream_scanlist_address38(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS38 (STREAM_SCANLIST_ADDRESS#(0:127)@38)

        Address: 4176
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS38, value)

    @property
    @locked
    def stream_scanlist_address39(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS39 (STREAM_SCANLIST_ADDRESS#(0:127)@39)

        Address: 4178
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS39)  # type: ignore

    @stream_scanlist_address39.setter
    @locked
    def stream_scanlist_address39(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS39 (STREAM_SCANLIST_ADDRESS#(0:127)@39)

        Address: 4178
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS39, value)

    @property
    @locked
    def stream_scanlist_address40(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS40 (STREAM_SCANLIST_ADDRESS#(0:127)@40)

        Address: 4180
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS40)  # type: ignore

    @stream_scanlist_address40.setter
    @locked
    def stream_scanlist_address40(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS40 (STREAM_SCANLIST_ADDRESS#(0:127)@40)

        Address: 4180
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS40, value)

    @property
    @locked
    def stream_scanlist_address41(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS41 (STREAM_SCANLIST_ADDRESS#(0:127)@41)

        Address: 4182
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS41)  # type: ignore

    @stream_scanlist_address41.setter
    @locked
    def stream_scanlist_address41(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS41 (STREAM_SCANLIST_ADDRESS#(0:127)@41)

        Address: 4182
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS41, value)

    @property
    @locked
    def stream_scanlist_address42(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS42 (STREAM_SCANLIST_ADDRESS#(0:127)@42)

        Address: 4184
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS42)  # type: ignore

    @stream_scanlist_address42.setter
    @locked
    def stream_scanlist_address42(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS42 (STREAM_SCANLIST_ADDRESS#(0:127)@42)

        Address: 4184
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS42, value)

    @property
    @locked
    def stream_scanlist_address43(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS43 (STREAM_SCANLIST_ADDRESS#(0:127)@43)

        Address: 4186
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS43)  # type: ignore

    @stream_scanlist_address43.setter
    @locked
    def stream_scanlist_address43(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS43 (STREAM_SCANLIST_ADDRESS#(0:127)@43)

        Address: 4186
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS43, value)

    @property
    @locked
    def stream_scanlist_address44(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS44 (STREAM_SCANLIST_ADDRESS#(0:127)@44)

        Address: 4188
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS44)  # type: ignore

    @stream_scanlist_address44.setter
    @locked
    def stream_scanlist_address44(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS44 (STREAM_SCANLIST_ADDRESS#(0:127)@44)

        Address: 4188
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS44, value)

    @property
    @locked
    def stream_scanlist_address45(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS45 (STREAM_SCANLIST_ADDRESS#(0:127)@45)

        Address: 4190
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS45)  # type: ignore

    @stream_scanlist_address45.setter
    @locked
    def stream_scanlist_address45(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS45 (STREAM_SCANLIST_ADDRESS#(0:127)@45)

        Address: 4190
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS45, value)

    @property
    @locked
    def stream_scanlist_address46(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS46 (STREAM_SCANLIST_ADDRESS#(0:127)@46)

        Address: 4192
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS46)  # type: ignore

    @stream_scanlist_address46.setter
    @locked
    def stream_scanlist_address46(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS46 (STREAM_SCANLIST_ADDRESS#(0:127)@46)

        Address: 4192
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS46, value)

    @property
    @locked
    def stream_scanlist_address47(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS47 (STREAM_SCANLIST_ADDRESS#(0:127)@47)

        Address: 4194
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS47)  # type: ignore

    @stream_scanlist_address47.setter
    @locked
    def stream_scanlist_address47(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS47 (STREAM_SCANLIST_ADDRESS#(0:127)@47)

        Address: 4194
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS47, value)

    @property
    @locked
    def stream_scanlist_address48(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS48 (STREAM_SCANLIST_ADDRESS#(0:127)@48)

        Address: 4196
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS48)  # type: ignore

    @stream_scanlist_address48.setter
    @locked
    def stream_scanlist_address48(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS48 (STREAM_SCANLIST_ADDRESS#(0:127)@48)

        Address: 4196
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS48, value)

    @property
    @locked
    def stream_scanlist_address49(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS49 (STREAM_SCANLIST_ADDRESS#(0:127)@49)

        Address: 4198
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS49)  # type: ignore

    @stream_scanlist_address49.setter
    @locked
    def stream_scanlist_address49(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS49 (STREAM_SCANLIST_ADDRESS#(0:127)@49)

        Address: 4198
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS49, value)

    @property
    @locked
    def stream_scanlist_address50(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS50 (STREAM_SCANLIST_ADDRESS#(0:127)@50)

        Address: 4200
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS50)  # type: ignore

    @stream_scanlist_address50.setter
    @locked
    def stream_scanlist_address50(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS50 (STREAM_SCANLIST_ADDRESS#(0:127)@50)

        Address: 4200
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS50, value)

    @property
    @locked
    def stream_scanlist_address51(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS51 (STREAM_SCANLIST_ADDRESS#(0:127)@51)

        Address: 4202
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS51)  # type: ignore

    @stream_scanlist_address51.setter
    @locked
    def stream_scanlist_address51(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS51 (STREAM_SCANLIST_ADDRESS#(0:127)@51)

        Address: 4202
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS51, value)

    @property
    @locked
    def stream_scanlist_address52(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS52 (STREAM_SCANLIST_ADDRESS#(0:127)@52)

        Address: 4204
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS52)  # type: ignore

    @stream_scanlist_address52.setter
    @locked
    def stream_scanlist_address52(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS52 (STREAM_SCANLIST_ADDRESS#(0:127)@52)

        Address: 4204
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS52, value)

    @property
    @locked
    def stream_scanlist_address53(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS53 (STREAM_SCANLIST_ADDRESS#(0:127)@53)

        Address: 4206
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS53)  # type: ignore

    @stream_scanlist_address53.setter
    @locked
    def stream_scanlist_address53(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS53 (STREAM_SCANLIST_ADDRESS#(0:127)@53)

        Address: 4206
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS53, value)

    @property
    @locked
    def stream_scanlist_address54(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS54 (STREAM_SCANLIST_ADDRESS#(0:127)@54)

        Address: 4208
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS54)  # type: ignore

    @stream_scanlist_address54.setter
    @locked
    def stream_scanlist_address54(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS54 (STREAM_SCANLIST_ADDRESS#(0:127)@54)

        Address: 4208
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS54, value)

    @property
    @locked
    def stream_scanlist_address55(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS55 (STREAM_SCANLIST_ADDRESS#(0:127)@55)

        Address: 4210
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS55)  # type: ignore

    @stream_scanlist_address55.setter
    @locked
    def stream_scanlist_address55(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS55 (STREAM_SCANLIST_ADDRESS#(0:127)@55)

        Address: 4210
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS55, value)

    @property
    @locked
    def stream_scanlist_address56(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS56 (STREAM_SCANLIST_ADDRESS#(0:127)@56)

        Address: 4212
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS56)  # type: ignore

    @stream_scanlist_address56.setter
    @locked
    def stream_scanlist_address56(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS56 (STREAM_SCANLIST_ADDRESS#(0:127)@56)

        Address: 4212
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS56, value)

    @property
    @locked
    def stream_scanlist_address57(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS57 (STREAM_SCANLIST_ADDRESS#(0:127)@57)

        Address: 4214
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS57)  # type: ignore

    @stream_scanlist_address57.setter
    @locked
    def stream_scanlist_address57(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS57 (STREAM_SCANLIST_ADDRESS#(0:127)@57)

        Address: 4214
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS57, value)

    @property
    @locked
    def stream_scanlist_address58(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS58 (STREAM_SCANLIST_ADDRESS#(0:127)@58)

        Address: 4216
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS58)  # type: ignore

    @stream_scanlist_address58.setter
    @locked
    def stream_scanlist_address58(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS58 (STREAM_SCANLIST_ADDRESS#(0:127)@58)

        Address: 4216
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS58, value)

    @property
    @locked
    def stream_scanlist_address59(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS59 (STREAM_SCANLIST_ADDRESS#(0:127)@59)

        Address: 4218
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS59)  # type: ignore

    @stream_scanlist_address59.setter
    @locked
    def stream_scanlist_address59(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS59 (STREAM_SCANLIST_ADDRESS#(0:127)@59)

        Address: 4218
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS59, value)

    @property
    @locked
    def stream_scanlist_address60(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS60 (STREAM_SCANLIST_ADDRESS#(0:127)@60)

        Address: 4220
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS60)  # type: ignore

    @stream_scanlist_address60.setter
    @locked
    def stream_scanlist_address60(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS60 (STREAM_SCANLIST_ADDRESS#(0:127)@60)

        Address: 4220
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS60, value)

    @property
    @locked
    def stream_scanlist_address61(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS61 (STREAM_SCANLIST_ADDRESS#(0:127)@61)

        Address: 4222
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS61)  # type: ignore

    @stream_scanlist_address61.setter
    @locked
    def stream_scanlist_address61(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS61 (STREAM_SCANLIST_ADDRESS#(0:127)@61)

        Address: 4222
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS61, value)

    @property
    @locked
    def stream_scanlist_address62(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS62 (STREAM_SCANLIST_ADDRESS#(0:127)@62)

        Address: 4224
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS62)  # type: ignore

    @stream_scanlist_address62.setter
    @locked
    def stream_scanlist_address62(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS62 (STREAM_SCANLIST_ADDRESS#(0:127)@62)

        Address: 4224
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS62, value)

    @property
    @locked
    def stream_scanlist_address63(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS63 (STREAM_SCANLIST_ADDRESS#(0:127)@63)

        Address: 4226
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS63)  # type: ignore

    @stream_scanlist_address63.setter
    @locked
    def stream_scanlist_address63(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS63 (STREAM_SCANLIST_ADDRESS#(0:127)@63)

        Address: 4226
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS63, value)

    @property
    @locked
    def stream_scanlist_address64(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS64 (STREAM_SCANLIST_ADDRESS#(0:127)@64)

        Address: 4228
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS64)  # type: ignore

    @stream_scanlist_address64.setter
    @locked
    def stream_scanlist_address64(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS64 (STREAM_SCANLIST_ADDRESS#(0:127)@64)

        Address: 4228
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS64, value)

    @property
    @locked
    def stream_scanlist_address65(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS65 (STREAM_SCANLIST_ADDRESS#(0:127)@65)

        Address: 4230
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS65)  # type: ignore

    @stream_scanlist_address65.setter
    @locked
    def stream_scanlist_address65(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS65 (STREAM_SCANLIST_ADDRESS#(0:127)@65)

        Address: 4230
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS65, value)

    @property
    @locked
    def stream_scanlist_address66(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS66 (STREAM_SCANLIST_ADDRESS#(0:127)@66)

        Address: 4232
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS66)  # type: ignore

    @stream_scanlist_address66.setter
    @locked
    def stream_scanlist_address66(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS66 (STREAM_SCANLIST_ADDRESS#(0:127)@66)

        Address: 4232
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS66, value)

    @property
    @locked
    def stream_scanlist_address67(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS67 (STREAM_SCANLIST_ADDRESS#(0:127)@67)

        Address: 4234
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS67)  # type: ignore

    @stream_scanlist_address67.setter
    @locked
    def stream_scanlist_address67(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS67 (STREAM_SCANLIST_ADDRESS#(0:127)@67)

        Address: 4234
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS67, value)

    @property
    @locked
    def stream_scanlist_address68(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS68 (STREAM_SCANLIST_ADDRESS#(0:127)@68)

        Address: 4236
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS68)  # type: ignore

    @stream_scanlist_address68.setter
    @locked
    def stream_scanlist_address68(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS68 (STREAM_SCANLIST_ADDRESS#(0:127)@68)

        Address: 4236
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS68, value)

    @property
    @locked
    def stream_scanlist_address69(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS69 (STREAM_SCANLIST_ADDRESS#(0:127)@69)

        Address: 4238
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS69)  # type: ignore

    @stream_scanlist_address69.setter
    @locked
    def stream_scanlist_address69(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS69 (STREAM_SCANLIST_ADDRESS#(0:127)@69)

        Address: 4238
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS69, value)

    @property
    @locked
    def stream_scanlist_address70(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS70 (STREAM_SCANLIST_ADDRESS#(0:127)@70)

        Address: 4240
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS70)  # type: ignore

    @stream_scanlist_address70.setter
    @locked
    def stream_scanlist_address70(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS70 (STREAM_SCANLIST_ADDRESS#(0:127)@70)

        Address: 4240
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS70, value)

    @property
    @locked
    def stream_scanlist_address71(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS71 (STREAM_SCANLIST_ADDRESS#(0:127)@71)

        Address: 4242
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS71)  # type: ignore

    @stream_scanlist_address71.setter
    @locked
    def stream_scanlist_address71(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS71 (STREAM_SCANLIST_ADDRESS#(0:127)@71)

        Address: 4242
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS71, value)

    @property
    @locked
    def stream_scanlist_address72(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS72 (STREAM_SCANLIST_ADDRESS#(0:127)@72)

        Address: 4244
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS72)  # type: ignore

    @stream_scanlist_address72.setter
    @locked
    def stream_scanlist_address72(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS72 (STREAM_SCANLIST_ADDRESS#(0:127)@72)

        Address: 4244
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS72, value)

    @property
    @locked
    def stream_scanlist_address73(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS73 (STREAM_SCANLIST_ADDRESS#(0:127)@73)

        Address: 4246
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS73)  # type: ignore

    @stream_scanlist_address73.setter
    @locked
    def stream_scanlist_address73(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS73 (STREAM_SCANLIST_ADDRESS#(0:127)@73)

        Address: 4246
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS73, value)

    @property
    @locked
    def stream_scanlist_address74(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS74 (STREAM_SCANLIST_ADDRESS#(0:127)@74)

        Address: 4248
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS74)  # type: ignore

    @stream_scanlist_address74.setter
    @locked
    def stream_scanlist_address74(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS74 (STREAM_SCANLIST_ADDRESS#(0:127)@74)

        Address: 4248
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS74, value)

    @property
    @locked
    def stream_scanlist_address75(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS75 (STREAM_SCANLIST_ADDRESS#(0:127)@75)

        Address: 4250
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS75)  # type: ignore

    @stream_scanlist_address75.setter
    @locked
    def stream_scanlist_address75(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS75 (STREAM_SCANLIST_ADDRESS#(0:127)@75)

        Address: 4250
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS75, value)

    @property
    @locked
    def stream_scanlist_address76(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS76 (STREAM_SCANLIST_ADDRESS#(0:127)@76)

        Address: 4252
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS76)  # type: ignore

    @stream_scanlist_address76.setter
    @locked
    def stream_scanlist_address76(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS76 (STREAM_SCANLIST_ADDRESS#(0:127)@76)

        Address: 4252
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS76, value)

    @property
    @locked
    def stream_scanlist_address77(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS77 (STREAM_SCANLIST_ADDRESS#(0:127)@77)

        Address: 4254
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS77)  # type: ignore

    @stream_scanlist_address77.setter
    @locked
    def stream_scanlist_address77(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS77 (STREAM_SCANLIST_ADDRESS#(0:127)@77)

        Address: 4254
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS77, value)

    @property
    @locked
    def stream_scanlist_address78(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS78 (STREAM_SCANLIST_ADDRESS#(0:127)@78)

        Address: 4256
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS78)  # type: ignore

    @stream_scanlist_address78.setter
    @locked
    def stream_scanlist_address78(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS78 (STREAM_SCANLIST_ADDRESS#(0:127)@78)

        Address: 4256
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS78, value)

    @property
    @locked
    def stream_scanlist_address79(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS79 (STREAM_SCANLIST_ADDRESS#(0:127)@79)

        Address: 4258
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS79)  # type: ignore

    @stream_scanlist_address79.setter
    @locked
    def stream_scanlist_address79(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS79 (STREAM_SCANLIST_ADDRESS#(0:127)@79)

        Address: 4258
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS79, value)

    @property
    @locked
    def stream_scanlist_address80(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS80 (STREAM_SCANLIST_ADDRESS#(0:127)@80)

        Address: 4260
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS80)  # type: ignore

    @stream_scanlist_address80.setter
    @locked
    def stream_scanlist_address80(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS80 (STREAM_SCANLIST_ADDRESS#(0:127)@80)

        Address: 4260
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS80, value)

    @property
    @locked
    def stream_scanlist_address81(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS81 (STREAM_SCANLIST_ADDRESS#(0:127)@81)

        Address: 4262
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS81)  # type: ignore

    @stream_scanlist_address81.setter
    @locked
    def stream_scanlist_address81(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS81 (STREAM_SCANLIST_ADDRESS#(0:127)@81)

        Address: 4262
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS81, value)

    @property
    @locked
    def stream_scanlist_address82(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS82 (STREAM_SCANLIST_ADDRESS#(0:127)@82)

        Address: 4264
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS82)  # type: ignore

    @stream_scanlist_address82.setter
    @locked
    def stream_scanlist_address82(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS82 (STREAM_SCANLIST_ADDRESS#(0:127)@82)

        Address: 4264
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS82, value)

    @property
    @locked
    def stream_scanlist_address83(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS83 (STREAM_SCANLIST_ADDRESS#(0:127)@83)

        Address: 4266
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS83)  # type: ignore

    @stream_scanlist_address83.setter
    @locked
    def stream_scanlist_address83(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS83 (STREAM_SCANLIST_ADDRESS#(0:127)@83)

        Address: 4266
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS83, value)

    @property
    @locked
    def stream_scanlist_address84(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS84 (STREAM_SCANLIST_ADDRESS#(0:127)@84)

        Address: 4268
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS84)  # type: ignore

    @stream_scanlist_address84.setter
    @locked
    def stream_scanlist_address84(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS84 (STREAM_SCANLIST_ADDRESS#(0:127)@84)

        Address: 4268
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS84, value)

    @property
    @locked
    def stream_scanlist_address85(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS85 (STREAM_SCANLIST_ADDRESS#(0:127)@85)

        Address: 4270
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS85)  # type: ignore

    @stream_scanlist_address85.setter
    @locked
    def stream_scanlist_address85(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS85 (STREAM_SCANLIST_ADDRESS#(0:127)@85)

        Address: 4270
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS85, value)

    @property
    @locked
    def stream_scanlist_address86(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS86 (STREAM_SCANLIST_ADDRESS#(0:127)@86)

        Address: 4272
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS86)  # type: ignore

    @stream_scanlist_address86.setter
    @locked
    def stream_scanlist_address86(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS86 (STREAM_SCANLIST_ADDRESS#(0:127)@86)

        Address: 4272
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS86, value)

    @property
    @locked
    def stream_scanlist_address87(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS87 (STREAM_SCANLIST_ADDRESS#(0:127)@87)

        Address: 4274
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS87)  # type: ignore

    @stream_scanlist_address87.setter
    @locked
    def stream_scanlist_address87(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS87 (STREAM_SCANLIST_ADDRESS#(0:127)@87)

        Address: 4274
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS87, value)

    @property
    @locked
    def stream_scanlist_address88(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS88 (STREAM_SCANLIST_ADDRESS#(0:127)@88)

        Address: 4276
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS88)  # type: ignore

    @stream_scanlist_address88.setter
    @locked
    def stream_scanlist_address88(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS88 (STREAM_SCANLIST_ADDRESS#(0:127)@88)

        Address: 4276
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS88, value)

    @property
    @locked
    def stream_scanlist_address89(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS89 (STREAM_SCANLIST_ADDRESS#(0:127)@89)

        Address: 4278
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS89)  # type: ignore

    @stream_scanlist_address89.setter
    @locked
    def stream_scanlist_address89(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS89 (STREAM_SCANLIST_ADDRESS#(0:127)@89)

        Address: 4278
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS89, value)

    @property
    @locked
    def stream_scanlist_address90(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS90 (STREAM_SCANLIST_ADDRESS#(0:127)@90)

        Address: 4280
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS90)  # type: ignore

    @stream_scanlist_address90.setter
    @locked
    def stream_scanlist_address90(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS90 (STREAM_SCANLIST_ADDRESS#(0:127)@90)

        Address: 4280
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS90, value)

    @property
    @locked
    def stream_scanlist_address91(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS91 (STREAM_SCANLIST_ADDRESS#(0:127)@91)

        Address: 4282
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS91)  # type: ignore

    @stream_scanlist_address91.setter
    @locked
    def stream_scanlist_address91(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS91 (STREAM_SCANLIST_ADDRESS#(0:127)@91)

        Address: 4282
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS91, value)

    @property
    @locked
    def stream_scanlist_address92(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS92 (STREAM_SCANLIST_ADDRESS#(0:127)@92)

        Address: 4284
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS92)  # type: ignore

    @stream_scanlist_address92.setter
    @locked
    def stream_scanlist_address92(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS92 (STREAM_SCANLIST_ADDRESS#(0:127)@92)

        Address: 4284
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS92, value)

    @property
    @locked
    def stream_scanlist_address93(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS93 (STREAM_SCANLIST_ADDRESS#(0:127)@93)

        Address: 4286
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS93)  # type: ignore

    @stream_scanlist_address93.setter
    @locked
    def stream_scanlist_address93(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS93 (STREAM_SCANLIST_ADDRESS#(0:127)@93)

        Address: 4286
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS93, value)

    @property
    @locked
    def stream_scanlist_address94(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS94 (STREAM_SCANLIST_ADDRESS#(0:127)@94)

        Address: 4288
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS94)  # type: ignore

    @stream_scanlist_address94.setter
    @locked
    def stream_scanlist_address94(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS94 (STREAM_SCANLIST_ADDRESS#(0:127)@94)

        Address: 4288
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS94, value)

    @property
    @locked
    def stream_scanlist_address95(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS95 (STREAM_SCANLIST_ADDRESS#(0:127)@95)

        Address: 4290
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS95)  # type: ignore

    @stream_scanlist_address95.setter
    @locked
    def stream_scanlist_address95(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS95 (STREAM_SCANLIST_ADDRESS#(0:127)@95)

        Address: 4290
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS95, value)

    @property
    @locked
    def stream_scanlist_address96(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS96 (STREAM_SCANLIST_ADDRESS#(0:127)@96)

        Address: 4292
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS96)  # type: ignore

    @stream_scanlist_address96.setter
    @locked
    def stream_scanlist_address96(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS96 (STREAM_SCANLIST_ADDRESS#(0:127)@96)

        Address: 4292
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS96, value)

    @property
    @locked
    def stream_scanlist_address97(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS97 (STREAM_SCANLIST_ADDRESS#(0:127)@97)

        Address: 4294
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS97)  # type: ignore

    @stream_scanlist_address97.setter
    @locked
    def stream_scanlist_address97(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS97 (STREAM_SCANLIST_ADDRESS#(0:127)@97)

        Address: 4294
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS97, value)

    @property
    @locked
    def stream_scanlist_address98(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS98 (STREAM_SCANLIST_ADDRESS#(0:127)@98)

        Address: 4296
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS98)  # type: ignore

    @stream_scanlist_address98.setter
    @locked
    def stream_scanlist_address98(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS98 (STREAM_SCANLIST_ADDRESS#(0:127)@98)

        Address: 4296
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS98, value)

    @property
    @locked
    def stream_scanlist_address99(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS99 (STREAM_SCANLIST_ADDRESS#(0:127)@99)

        Address: 4298
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS99)  # type: ignore

    @stream_scanlist_address99.setter
    @locked
    def stream_scanlist_address99(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS99 (STREAM_SCANLIST_ADDRESS#(0:127)@99)

        Address: 4298
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS99, value)

    @property
    @locked
    def stream_scanlist_address100(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS100 (STREAM_SCANLIST_ADDRESS#(0:127)@100)

        Address: 4300
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS100)  # type: ignore

    @stream_scanlist_address100.setter
    @locked
    def stream_scanlist_address100(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS100 (STREAM_SCANLIST_ADDRESS#(0:127)@100)

        Address: 4300
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS100, value)

    @property
    @locked
    def stream_scanlist_address101(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS101 (STREAM_SCANLIST_ADDRESS#(0:127)@101)

        Address: 4302
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS101)  # type: ignore

    @stream_scanlist_address101.setter
    @locked
    def stream_scanlist_address101(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS101 (STREAM_SCANLIST_ADDRESS#(0:127)@101)

        Address: 4302
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS101, value)

    @property
    @locked
    def stream_scanlist_address102(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS102 (STREAM_SCANLIST_ADDRESS#(0:127)@102)

        Address: 4304
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS102)  # type: ignore

    @stream_scanlist_address102.setter
    @locked
    def stream_scanlist_address102(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS102 (STREAM_SCANLIST_ADDRESS#(0:127)@102)

        Address: 4304
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS102, value)

    @property
    @locked
    def stream_scanlist_address103(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS103 (STREAM_SCANLIST_ADDRESS#(0:127)@103)

        Address: 4306
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS103)  # type: ignore

    @stream_scanlist_address103.setter
    @locked
    def stream_scanlist_address103(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS103 (STREAM_SCANLIST_ADDRESS#(0:127)@103)

        Address: 4306
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS103, value)

    @property
    @locked
    def stream_scanlist_address104(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS104 (STREAM_SCANLIST_ADDRESS#(0:127)@104)

        Address: 4308
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS104)  # type: ignore

    @stream_scanlist_address104.setter
    @locked
    def stream_scanlist_address104(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS104 (STREAM_SCANLIST_ADDRESS#(0:127)@104)

        Address: 4308
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS104, value)

    @property
    @locked
    def stream_scanlist_address105(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS105 (STREAM_SCANLIST_ADDRESS#(0:127)@105)

        Address: 4310
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS105)  # type: ignore

    @stream_scanlist_address105.setter
    @locked
    def stream_scanlist_address105(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS105 (STREAM_SCANLIST_ADDRESS#(0:127)@105)

        Address: 4310
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS105, value)

    @property
    @locked
    def stream_scanlist_address106(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS106 (STREAM_SCANLIST_ADDRESS#(0:127)@106)

        Address: 4312
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS106)  # type: ignore

    @stream_scanlist_address106.setter
    @locked
    def stream_scanlist_address106(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS106 (STREAM_SCANLIST_ADDRESS#(0:127)@106)

        Address: 4312
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS106, value)

    @property
    @locked
    def stream_scanlist_address107(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS107 (STREAM_SCANLIST_ADDRESS#(0:127)@107)

        Address: 4314
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS107)  # type: ignore

    @stream_scanlist_address107.setter
    @locked
    def stream_scanlist_address107(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS107 (STREAM_SCANLIST_ADDRESS#(0:127)@107)

        Address: 4314
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS107, value)

    @property
    @locked
    def stream_scanlist_address108(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS108 (STREAM_SCANLIST_ADDRESS#(0:127)@108)

        Address: 4316
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS108)  # type: ignore

    @stream_scanlist_address108.setter
    @locked
    def stream_scanlist_address108(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS108 (STREAM_SCANLIST_ADDRESS#(0:127)@108)

        Address: 4316
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS108, value)

    @property
    @locked
    def stream_scanlist_address109(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS109 (STREAM_SCANLIST_ADDRESS#(0:127)@109)

        Address: 4318
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS109)  # type: ignore

    @stream_scanlist_address109.setter
    @locked
    def stream_scanlist_address109(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS109 (STREAM_SCANLIST_ADDRESS#(0:127)@109)

        Address: 4318
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS109, value)

    @property
    @locked
    def stream_scanlist_address110(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS110 (STREAM_SCANLIST_ADDRESS#(0:127)@110)

        Address: 4320
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS110)  # type: ignore

    @stream_scanlist_address110.setter
    @locked
    def stream_scanlist_address110(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS110 (STREAM_SCANLIST_ADDRESS#(0:127)@110)

        Address: 4320
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS110, value)

    @property
    @locked
    def stream_scanlist_address111(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS111 (STREAM_SCANLIST_ADDRESS#(0:127)@111)

        Address: 4322
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS111)  # type: ignore

    @stream_scanlist_address111.setter
    @locked
    def stream_scanlist_address111(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS111 (STREAM_SCANLIST_ADDRESS#(0:127)@111)

        Address: 4322
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS111, value)

    @property
    @locked
    def stream_scanlist_address112(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS112 (STREAM_SCANLIST_ADDRESS#(0:127)@112)

        Address: 4324
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS112)  # type: ignore

    @stream_scanlist_address112.setter
    @locked
    def stream_scanlist_address112(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS112 (STREAM_SCANLIST_ADDRESS#(0:127)@112)

        Address: 4324
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS112, value)

    @property
    @locked
    def stream_scanlist_address113(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS113 (STREAM_SCANLIST_ADDRESS#(0:127)@113)

        Address: 4326
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS113)  # type: ignore

    @stream_scanlist_address113.setter
    @locked
    def stream_scanlist_address113(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS113 (STREAM_SCANLIST_ADDRESS#(0:127)@113)

        Address: 4326
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS113, value)

    @property
    @locked
    def stream_scanlist_address114(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS114 (STREAM_SCANLIST_ADDRESS#(0:127)@114)

        Address: 4328
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS114)  # type: ignore

    @stream_scanlist_address114.setter
    @locked
    def stream_scanlist_address114(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS114 (STREAM_SCANLIST_ADDRESS#(0:127)@114)

        Address: 4328
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS114, value)

    @property
    @locked
    def stream_scanlist_address115(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS115 (STREAM_SCANLIST_ADDRESS#(0:127)@115)

        Address: 4330
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS115)  # type: ignore

    @stream_scanlist_address115.setter
    @locked
    def stream_scanlist_address115(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS115 (STREAM_SCANLIST_ADDRESS#(0:127)@115)

        Address: 4330
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS115, value)

    @property
    @locked
    def stream_scanlist_address116(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS116 (STREAM_SCANLIST_ADDRESS#(0:127)@116)

        Address: 4332
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS116)  # type: ignore

    @stream_scanlist_address116.setter
    @locked
    def stream_scanlist_address116(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS116 (STREAM_SCANLIST_ADDRESS#(0:127)@116)

        Address: 4332
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS116, value)

    @property
    @locked
    def stream_scanlist_address117(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS117 (STREAM_SCANLIST_ADDRESS#(0:127)@117)

        Address: 4334
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS117)  # type: ignore

    @stream_scanlist_address117.setter
    @locked
    def stream_scanlist_address117(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS117 (STREAM_SCANLIST_ADDRESS#(0:127)@117)

        Address: 4334
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS117, value)

    @property
    @locked
    def stream_scanlist_address118(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS118 (STREAM_SCANLIST_ADDRESS#(0:127)@118)

        Address: 4336
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS118)  # type: ignore

    @stream_scanlist_address118.setter
    @locked
    def stream_scanlist_address118(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS118 (STREAM_SCANLIST_ADDRESS#(0:127)@118)

        Address: 4336
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS118, value)

    @property
    @locked
    def stream_scanlist_address119(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS119 (STREAM_SCANLIST_ADDRESS#(0:127)@119)

        Address: 4338
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS119)  # type: ignore

    @stream_scanlist_address119.setter
    @locked
    def stream_scanlist_address119(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS119 (STREAM_SCANLIST_ADDRESS#(0:127)@119)

        Address: 4338
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS119, value)

    @property
    @locked
    def stream_scanlist_address120(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS120 (STREAM_SCANLIST_ADDRESS#(0:127)@120)

        Address: 4340
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS120)  # type: ignore

    @stream_scanlist_address120.setter
    @locked
    def stream_scanlist_address120(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS120 (STREAM_SCANLIST_ADDRESS#(0:127)@120)

        Address: 4340
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS120, value)

    @property
    @locked
    def stream_scanlist_address121(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS121 (STREAM_SCANLIST_ADDRESS#(0:127)@121)

        Address: 4342
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS121)  # type: ignore

    @stream_scanlist_address121.setter
    @locked
    def stream_scanlist_address121(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS121 (STREAM_SCANLIST_ADDRESS#(0:127)@121)

        Address: 4342
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS121, value)

    @property
    @locked
    def stream_scanlist_address122(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS122 (STREAM_SCANLIST_ADDRESS#(0:127)@122)

        Address: 4344
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS122)  # type: ignore

    @stream_scanlist_address122.setter
    @locked
    def stream_scanlist_address122(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS122 (STREAM_SCANLIST_ADDRESS#(0:127)@122)

        Address: 4344
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS122, value)

    @property
    @locked
    def stream_scanlist_address123(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS123 (STREAM_SCANLIST_ADDRESS#(0:127)@123)

        Address: 4346
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS123)  # type: ignore

    @stream_scanlist_address123.setter
    @locked
    def stream_scanlist_address123(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS123 (STREAM_SCANLIST_ADDRESS#(0:127)@123)

        Address: 4346
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS123, value)

    @property
    @locked
    def stream_scanlist_address124(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS124 (STREAM_SCANLIST_ADDRESS#(0:127)@124)

        Address: 4348
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS124)  # type: ignore

    @stream_scanlist_address124.setter
    @locked
    def stream_scanlist_address124(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS124 (STREAM_SCANLIST_ADDRESS#(0:127)@124)

        Address: 4348
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS124, value)

    @property
    @locked
    def stream_scanlist_address125(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS125 (STREAM_SCANLIST_ADDRESS#(0:127)@125)

        Address: 4350
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS125)  # type: ignore

    @stream_scanlist_address125.setter
    @locked
    def stream_scanlist_address125(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS125 (STREAM_SCANLIST_ADDRESS#(0:127)@125)

        Address: 4350
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS125, value)

    @property
    @locked
    def stream_scanlist_address126(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS126 (STREAM_SCANLIST_ADDRESS#(0:127)@126)

        Address: 4352
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS126)  # type: ignore

    @stream_scanlist_address126.setter
    @locked
    def stream_scanlist_address126(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS126 (STREAM_SCANLIST_ADDRESS#(0:127)@126)

        Address: 4352
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS126, value)

    @property
    @locked
    def stream_scanlist_address127(self) -> int:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS127 (STREAM_SCANLIST_ADDRESS#(0:127)@127)

        Address: 4354
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS127)  # type: ignore

    @stream_scanlist_address127.setter
    @locked
    def stream_scanlist_address127(self, value: int) -> None:
        """
        A list of addresses to read each scan. In the case of Stream-Out enabled, the list may also include
        something to write each scan.

        Register: STREAM_SCANLIST_ADDRESS127 (STREAM_SCANLIST_ADDRESS#(0:127)@127)

        Address: 4354
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_SCANLIST_ADDRESS127, value)

    @property
    @locked
    def stream_out0(self) -> int:
        """
        Include one or more of these registers in STREAM_SCANLIST_ADDRESS#(0:127) to trigger stream-out updates.
        When added to the scan list these do count against the max scan rate just like normal input addresses, but they
        do not return any data in the stream read.

        Register: STREAM_OUT0 (STREAM_OUT#(0:3)@0)

        Address: 4800
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT0)  # type: ignore

    @property
    @locked
    def stream_out1(self) -> int:
        """
        Include one or more of these registers in STREAM_SCANLIST_ADDRESS#(0:127) to trigger stream-out updates.
        When added to the scan list these do count against the max scan rate just like normal input addresses, but they
        do not return any data in the stream read.

        Register: STREAM_OUT1 (STREAM_OUT#(0:3)@1)

        Address: 4801
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT1)  # type: ignore

    @property
    @locked
    def stream_out2(self) -> int:
        """
        Include one or more of these registers in STREAM_SCANLIST_ADDRESS#(0:127) to trigger stream-out updates.
        When added to the scan list these do count against the max scan rate just like normal input addresses, but they
        do not return any data in the stream read.

        Register: STREAM_OUT2 (STREAM_OUT#(0:3)@2)

        Address: 4802
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT2)  # type: ignore

    @property
    @locked
    def stream_out3(self) -> int:
        """
        Include one or more of these registers in STREAM_SCANLIST_ADDRESS#(0:127) to trigger stream-out updates.
        When added to the scan list these do count against the max scan rate just like normal input addresses, but they
        do not return any data in the stream read.

        Register: STREAM_OUT3 (STREAM_OUT#(0:3)@3)

        Address: 4803
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT3)  # type: ignore

    @property
    @locked
    def stream_out0_target(self) -> int:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT0_TARGET (STREAM_OUT#(0:3)_TARGET@0)

        Address: 4040
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_TARGET)  # type: ignore

    @stream_out0_target.setter
    @locked
    def stream_out0_target(self, value: int) -> None:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT0_TARGET (STREAM_OUT#(0:3)_TARGET@0)

        Address: 4040
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_TARGET, value)

    @property
    @locked
    def stream_out1_target(self) -> int:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT1_TARGET (STREAM_OUT#(0:3)_TARGET@1)

        Address: 4042
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_TARGET)  # type: ignore

    @stream_out1_target.setter
    @locked
    def stream_out1_target(self, value: int) -> None:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT1_TARGET (STREAM_OUT#(0:3)_TARGET@1)

        Address: 4042
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_TARGET, value)

    @property
    @locked
    def stream_out2_target(self) -> int:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT2_TARGET (STREAM_OUT#(0:3)_TARGET@2)

        Address: 4044
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_TARGET)  # type: ignore

    @stream_out2_target.setter
    @locked
    def stream_out2_target(self, value: int) -> None:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT2_TARGET (STREAM_OUT#(0:3)_TARGET@2)

        Address: 4044
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_TARGET, value)

    @property
    @locked
    def stream_out3_target(self) -> int:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT3_TARGET (STREAM_OUT#(0:3)_TARGET@3)

        Address: 4046
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_TARGET)  # type: ignore

    @stream_out3_target.setter
    @locked
    def stream_out3_target(self, value: int) -> None:
        """
        Channel that data will be written to. Before writing data to _BUFFER_###, you must write to _TARGET so the
        device knows how to interpret and store values.

        Register: STREAM_OUT3_TARGET (STREAM_OUT#(0:3)_TARGET@3)

        Address: 4046
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_TARGET, value)

    @property
    @locked
    def stream_out0_buffer_allocate_num_bytes(self) -> int:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT0_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@0)

        Address: 4050
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.STREAM_OUT0_BUFFER_ALLOCATE_NUM_BYTES
        )

    @stream_out0_buffer_allocate_num_bytes.setter
    @locked
    def stream_out0_buffer_allocate_num_bytes(self, value: int) -> None:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT0_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@0)

        Address: 4050
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_BUFFER_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def stream_out1_buffer_allocate_num_bytes(self) -> int:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT1_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@1)

        Address: 4052
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.STREAM_OUT1_BUFFER_ALLOCATE_NUM_BYTES
        )

    @stream_out1_buffer_allocate_num_bytes.setter
    @locked
    def stream_out1_buffer_allocate_num_bytes(self, value: int) -> None:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT1_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@1)

        Address: 4052
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_BUFFER_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def stream_out2_buffer_allocate_num_bytes(self) -> int:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT2_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@2)

        Address: 4054
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.STREAM_OUT2_BUFFER_ALLOCATE_NUM_BYTES
        )

    @stream_out2_buffer_allocate_num_bytes.setter
    @locked
    def stream_out2_buffer_allocate_num_bytes(self, value: int) -> None:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT2_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@2)

        Address: 4054
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_BUFFER_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def stream_out3_buffer_allocate_num_bytes(self) -> int:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT3_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@3)

        Address: 4056
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.STREAM_OUT3_BUFFER_ALLOCATE_NUM_BYTES
        )

    @stream_out3_buffer_allocate_num_bytes.setter
    @locked
    def stream_out3_buffer_allocate_num_bytes(self, value: int) -> None:
        """
        Size of the buffer in bytes as a power of 2. Should be at least twice the size of updates that will be
        written and no less than 32. The usable buffer size will be equal to the value of this register minus 2 bytes
        (one 16-bit sample). Before writing data to _BUFFER_###, you must write to _BUFFER_ALLOCATE_NUM_BYTES to
        allocate RAM for the data. Max is 16384.

        Register: STREAM_OUT3_BUFFER_ALLOCATE_NUM_BYTES (STREAM_OUT#(0:3)_BUFFER_ALLOCATE_NUM_BYTES@3)

        Address: 4056
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_BUFFER_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def stream_out0_loop_num_values(self) -> int:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT0_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@0)

        Address: 4060
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_LOOP_NUM_VALUES)  # type: ignore

    @stream_out0_loop_num_values.setter
    @locked
    def stream_out0_loop_num_values(self, value: int) -> None:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT0_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@0)

        Address: 4060
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_LOOP_NUM_VALUES, value)

    @property
    @locked
    def stream_out1_loop_num_values(self) -> int:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT1_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@1)

        Address: 4062
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_LOOP_NUM_VALUES)  # type: ignore

    @stream_out1_loop_num_values.setter
    @locked
    def stream_out1_loop_num_values(self, value: int) -> None:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT1_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@1)

        Address: 4062
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_LOOP_NUM_VALUES, value)

    @property
    @locked
    def stream_out2_loop_num_values(self) -> int:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT2_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@2)

        Address: 4064
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_LOOP_NUM_VALUES)  # type: ignore

    @stream_out2_loop_num_values.setter
    @locked
    def stream_out2_loop_num_values(self, value: int) -> None:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT2_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@2)

        Address: 4064
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_LOOP_NUM_VALUES, value)

    @property
    @locked
    def stream_out3_loop_num_values(self) -> int:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT3_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@3)

        Address: 4066
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_LOOP_NUM_VALUES)  # type: ignore

    @stream_out3_loop_num_values.setter
    @locked
    def stream_out3_loop_num_values(self, value: int) -> None:
        """
        The number of values, from the end of the array, that will be repeated after reaching the end of supplied
        data array.

        Register: STREAM_OUT3_LOOP_NUM_VALUES (STREAM_OUT#(0:3)_LOOP_NUM_VALUES@3)

        Address: 4066
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_LOOP_NUM_VALUES, value)

    @property
    @locked
    def stream_out0_set_loop(self) -> int:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT0_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@0)

        Address: 4070
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_SET_LOOP)  # type: ignore

    @stream_out0_set_loop.setter
    @locked
    def stream_out0_set_loop(self, value: int) -> None:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT0_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@0)

        Address: 4070
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_SET_LOOP, value)

    @property
    @locked
    def stream_out1_set_loop(self) -> int:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT1_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@1)

        Address: 4072
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_SET_LOOP)  # type: ignore

    @stream_out1_set_loop.setter
    @locked
    def stream_out1_set_loop(self, value: int) -> None:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT1_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@1)

        Address: 4072
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_SET_LOOP, value)

    @property
    @locked
    def stream_out2_set_loop(self) -> int:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT2_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@2)

        Address: 4074
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_SET_LOOP)  # type: ignore

    @stream_out2_set_loop.setter
    @locked
    def stream_out2_set_loop(self, value: int) -> None:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT2_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@2)

        Address: 4074
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_SET_LOOP, value)

    @property
    @locked
    def stream_out3_set_loop(self) -> int:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT3_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@3)

        Address: 4076
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_SET_LOOP)  # type: ignore

    @stream_out3_set_loop.setter
    @locked
    def stream_out3_set_loop(self, value: int) -> None:
        """
        Controls when new data and loop size are used. 1=Use new data immediately. 2=Wait for synch. New data will
        not be used until a different stream-out channel is set to Synch. 3=Synch. This stream-out# as well as any
        stream-outs set to synch will start using new data immediately.

        Register: STREAM_OUT3_SET_LOOP (STREAM_OUT#(0:3)_SET_LOOP@3)

        Address: 4076
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_SET_LOOP, value)

    @property
    @locked
    def stream_out0_buffer_status(self) -> int:
        """
        The number of values in the buffer that are not currently being used.

        Register: STREAM_OUT0_BUFFER_STATUS (STREAM_OUT#(0:3)_BUFFER_STATUS@0)

        Address: 4080
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_BUFFER_STATUS)  # type: ignore

    @property
    @locked
    def stream_out1_buffer_status(self) -> int:
        """
        The number of values in the buffer that are not currently being used.

        Register: STREAM_OUT1_BUFFER_STATUS (STREAM_OUT#(0:3)_BUFFER_STATUS@1)

        Address: 4082
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_BUFFER_STATUS)  # type: ignore

    @property
    @locked
    def stream_out2_buffer_status(self) -> int:
        """
        The number of values in the buffer that are not currently being used.

        Register: STREAM_OUT2_BUFFER_STATUS (STREAM_OUT#(0:3)_BUFFER_STATUS@2)

        Address: 4084
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_BUFFER_STATUS)  # type: ignore

    @property
    @locked
    def stream_out3_buffer_status(self) -> int:
        """
        The number of values in the buffer that are not currently being used.

        Register: STREAM_OUT3_BUFFER_STATUS (STREAM_OUT#(0:3)_BUFFER_STATUS@3)

        Address: 4086
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_BUFFER_STATUS)  # type: ignore

    @property
    @locked
    def stream_out0_enable(self) -> int:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT0_ENABLE (STREAM_OUT#(0:3)_ENABLE@0)

        Address: 4090
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_ENABLE)  # type: ignore

    @stream_out0_enable.setter
    @locked
    def stream_out0_enable(self, value: int) -> None:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT0_ENABLE (STREAM_OUT#(0:3)_ENABLE@0)

        Address: 4090
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_ENABLE, value)

    @property
    @locked
    def stream_out1_enable(self) -> int:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT1_ENABLE (STREAM_OUT#(0:3)_ENABLE@1)

        Address: 4092
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_ENABLE)  # type: ignore

    @stream_out1_enable.setter
    @locked
    def stream_out1_enable(self, value: int) -> None:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT1_ENABLE (STREAM_OUT#(0:3)_ENABLE@1)

        Address: 4092
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_ENABLE, value)

    @property
    @locked
    def stream_out2_enable(self) -> int:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT2_ENABLE (STREAM_OUT#(0:3)_ENABLE@2)

        Address: 4094
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_ENABLE)  # type: ignore

    @stream_out2_enable.setter
    @locked
    def stream_out2_enable(self, value: int) -> None:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT2_ENABLE (STREAM_OUT#(0:3)_ENABLE@2)

        Address: 4094
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_ENABLE, value)

    @property
    @locked
    def stream_out3_enable(self) -> int:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT3_ENABLE (STREAM_OUT#(0:3)_ENABLE@3)

        Address: 4096
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_ENABLE)  # type: ignore

    @stream_out3_enable.setter
    @locked
    def stream_out3_enable(self, value: int) -> None:
        """
        When STREAM_OUT#_ENABLE is enabled, the stream out target is generally updated by one value from the stream
        out buffer per stream scan. For example, there will generally be one instance of e.g. STREAM_OUT0 in the stream
        scan list, which will cause one STREAM_OUT0_BUFFER value to be consumed and written to STREAM_OUT0_TARGET for
        every stream scan. The stream scan list could also contain two instances of STREAM_OUT0, in which case two
        values from STREAM_OUT0_BUFFER value would be consumed and written for every stream scan.

        Register: STREAM_OUT3_ENABLE (STREAM_OUT#(0:3)_ENABLE@3)

        Address: 4096
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_ENABLE, value)

    @property
    @locked
    def stream_out0_buffer_f32(self) -> float:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT0_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@0)

        Address: 4400
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_OUT0_BUFFER_F32)  # type: ignore

    @stream_out0_buffer_f32.setter
    @locked
    def stream_out0_buffer_f32(self, value: float) -> None:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT0_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@0)

        Address: 4400
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_OUT0_BUFFER_F32, value)

    @property
    @locked
    def stream_out1_buffer_f32(self) -> float:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT1_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@1)

        Address: 4402
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_OUT1_BUFFER_F32)  # type: ignore

    @stream_out1_buffer_f32.setter
    @locked
    def stream_out1_buffer_f32(self, value: float) -> None:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT1_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@1)

        Address: 4402
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_OUT1_BUFFER_F32, value)

    @property
    @locked
    def stream_out2_buffer_f32(self) -> float:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT2_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@2)

        Address: 4404
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_OUT2_BUFFER_F32)  # type: ignore

    @stream_out2_buffer_f32.setter
    @locked
    def stream_out2_buffer_f32(self, value: float) -> None:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT2_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@2)

        Address: 4404
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_OUT2_BUFFER_F32, value)

    @property
    @locked
    def stream_out3_buffer_f32(self) -> float:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT3_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@3)

        Address: 4406
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.STREAM_OUT3_BUFFER_F32)  # type: ignore

    @stream_out3_buffer_f32.setter
    @locked
    def stream_out3_buffer_f32(self, value: float) -> None:
        """
        Data destination when sending floating point data. Appropriate cal constants are used to convert F32 values
        to 16-bit binary data, and thus each of these values uses 2 bytes of the stream-out buffer. This register is a
        buffer.

        Register: STREAM_OUT3_BUFFER_F32 (STREAM_OUT#(0:3)_BUFFER_F32@3)

        Address: 4406
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.STREAM_OUT3_BUFFER_F32, value)

    @property
    @locked
    def stream_out0_buffer_u32(self) -> int:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT0_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@0)

        Address: 4410
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_BUFFER_U32)  # type: ignore

    @stream_out0_buffer_u32.setter
    @locked
    def stream_out0_buffer_u32(self, value: int) -> None:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT0_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@0)

        Address: 4410
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT0_BUFFER_U32, value)

    @property
    @locked
    def stream_out1_buffer_u32(self) -> int:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT1_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@1)

        Address: 4412
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_BUFFER_U32)  # type: ignore

    @stream_out1_buffer_u32.setter
    @locked
    def stream_out1_buffer_u32(self, value: int) -> None:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT1_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@1)

        Address: 4412
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT1_BUFFER_U32, value)

    @property
    @locked
    def stream_out2_buffer_u32(self) -> int:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT2_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@2)

        Address: 4414
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_BUFFER_U32)  # type: ignore

    @stream_out2_buffer_u32.setter
    @locked
    def stream_out2_buffer_u32(self, value: int) -> None:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT2_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@2)

        Address: 4414
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT2_BUFFER_U32, value)

    @property
    @locked
    def stream_out3_buffer_u32(self) -> int:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT3_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@3)

        Address: 4416
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_BUFFER_U32)  # type: ignore

    @stream_out3_buffer_u32.setter
    @locked
    def stream_out3_buffer_u32(self, value: int) -> None:
        """
        Not used at this time. There are no U32 registers supported in stream-out. This register is a buffer.

        Register: STREAM_OUT3_BUFFER_U32 (STREAM_OUT#(0:3)_BUFFER_U32@3)

        Address: 4416
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_OUT3_BUFFER_U32, value)

    @property
    @locked
    def stream_out0_buffer_u16(self) -> int:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT0_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@0)

        Address: 4420
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT0_BUFFER_U16)  # type: ignore

    @stream_out0_buffer_u16.setter
    @locked
    def stream_out0_buffer_u16(self, value: int) -> None:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT0_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@0)

        Address: 4420
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.STREAM_OUT0_BUFFER_U16, value)

    @property
    @locked
    def stream_out1_buffer_u16(self) -> int:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT1_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@1)

        Address: 4421
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT1_BUFFER_U16)  # type: ignore

    @stream_out1_buffer_u16.setter
    @locked
    def stream_out1_buffer_u16(self, value: int) -> None:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT1_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@1)

        Address: 4421
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.STREAM_OUT1_BUFFER_U16, value)

    @property
    @locked
    def stream_out2_buffer_u16(self) -> int:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT2_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@2)

        Address: 4422
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT2_BUFFER_U16)  # type: ignore

    @stream_out2_buffer_u16.setter
    @locked
    def stream_out2_buffer_u16(self, value: int) -> None:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT2_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@2)

        Address: 4422
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.STREAM_OUT2_BUFFER_U16, value)

    @property
    @locked
    def stream_out3_buffer_u16(self) -> int:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT3_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@3)

        Address: 4423
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_OUT3_BUFFER_U16)  # type: ignore

    @stream_out3_buffer_u16.setter
    @locked
    def stream_out3_buffer_u16(self, value: int) -> None:
        """
        Data destination when sending 16-bit integer data. Each value uses 2 bytes of the stream-out buffer. This
        register is a buffer.

        Register: STREAM_OUT3_BUFFER_U16 (STREAM_OUT#(0:3)_BUFFER_U16@3)

        Address: 4423
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.STREAM_OUT3_BUFFER_U16, value)

    @property
    @locked
    def stream_data_cr(self) -> int:
        """
        Address to read stream data from when operating in C-R mode.

        Register: STREAM_DATA_CR

        Address: 4500
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_DATA_CR)  # type: ignore

    @property
    @locked
    def stream_data_capture_16(self) -> int:
        """
        If a channel produces 32-bits of data the upper 16 will be saved here.

        Register: STREAM_DATA_CAPTURE_16

        Address: 4899
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.STREAM_DATA_CAPTURE_16)  # type: ignore

    @property
    @locked
    def stream_debug_get_self_index(self) -> int:
        """
        Returns the index of the channel. The index is the position in the scan list that the channel occupies.

        Register: STREAM_DEBUG_GET_SELF_INDEX

        Address: 4898
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_DEBUG_GET_SELF_INDEX)  # type: ignore

    @stream_debug_get_self_index.setter
    @locked
    def stream_debug_get_self_index(self, value: int) -> None:
        """
        Returns the index of the channel. The index is the position in the scan list that the channel occupies.

        Register: STREAM_DEBUG_GET_SELF_INDEX

        Address: 4898
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_DEBUG_GET_SELF_INDEX, value)

    @property
    @locked
    def stream_enable(self) -> int:
        """
        Write 1 to start stream. Write 0 to stop stream. Reading this register returns 1 when stream is enabled.
        When using a triggered stream the stream is considered enabled while waiting for the trigger.

        Register: STREAM_ENABLE

        Address: 4990
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.STREAM_ENABLE)  # type: ignore

    @stream_enable.setter
    @locked
    def stream_enable(self, value: int) -> None:
        """
        Write 1 to start stream. Write 0 to stop stream. Reading this register returns 1 when stream is enabled.
        When using a triggered stream the stream is considered enabled while waiting for the trigger.

        Register: STREAM_ENABLE

        Address: 4990
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.STREAM_ENABLE, value)

    @property
    @locked
    def spi_cs_dionum(self) -> int:
        """
        The DIO line for Chip-Select.

        Register: SPI_CS_DIONUM

        Address: 5000
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_CS_DIONUM)  # type: ignore

    @spi_cs_dionum.setter
    @locked
    def spi_cs_dionum(self, value: int) -> None:
        """
        The DIO line for Chip-Select.

        Register: SPI_CS_DIONUM

        Address: 5000
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_CS_DIONUM, value)

    @property
    @locked
    def spi_clk_dionum(self) -> int:
        """
        The DIO line for Clock.

        Register: SPI_CLK_DIONUM

        Address: 5001
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_CLK_DIONUM)  # type: ignore

    @spi_clk_dionum.setter
    @locked
    def spi_clk_dionum(self, value: int) -> None:
        """
        The DIO line for Clock.

        Register: SPI_CLK_DIONUM

        Address: 5001
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_CLK_DIONUM, value)

    @property
    @locked
    def spi_miso_dionum(self) -> int:
        """
        The DIO line for Master-In-Slave-Out.

        Register: SPI_MISO_DIONUM

        Address: 5002
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_MISO_DIONUM)  # type: ignore

    @spi_miso_dionum.setter
    @locked
    def spi_miso_dionum(self, value: int) -> None:
        """
        The DIO line for Master-In-Slave-Out.

        Register: SPI_MISO_DIONUM

        Address: 5002
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_MISO_DIONUM, value)

    @property
    @locked
    def spi_mosi_dionum(self) -> int:
        """
        The DIO line for Master-Out-Slave-In.

        Register: SPI_MOSI_DIONUM

        Address: 5003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_MOSI_DIONUM)  # type: ignore

    @spi_mosi_dionum.setter
    @locked
    def spi_mosi_dionum(self, value: int) -> None:
        """
        The DIO line for Master-Out-Slave-In.

        Register: SPI_MOSI_DIONUM

        Address: 5003
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_MOSI_DIONUM, value)

    @property
    @locked
    def spi_mode(self) -> int:
        """
        The SPI mode controls the clock idle state and which edge clocks the data.
        Bit 1 is CPOL and Bit 0 is CPHA, so CPOL/CPHA for different decimal values:
        0 = 0/0 = b00,
        1 = 0/1 = b01,
        2 = 1/0 = b10,
        3 = 1/1 = b11.
        For CPOL and CPHA explanations, see Wikipedia article:
        https://en.wikipedia.org/wiki/Serial_Peripheral_Interface_Bus.

        Register: SPI_MODE

        Address: 5004
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_MODE)  # type: ignore

    @spi_mode.setter
    @locked
    def spi_mode(self, value: int) -> None:
        """
        The SPI mode controls the clock idle state and which edge clocks the data.
        Bit 1 is CPOL and Bit 0 is CPHA, so CPOL/CPHA for different decimal values:
        0 = 0/0 = b00,
        1 = 0/1 = b01,
        2 = 1/0 = b10,
        3 = 1/1 = b11.
        For CPOL and CPHA explanations, see Wikipedia article:
        https://en.wikipedia.org/wiki/Serial_Peripheral_Interface_Bus.

        Register: SPI_MODE

        Address: 5004
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_MODE, value)

    @property
    @locked
    def spi_speed_throttle(self) -> int:
        """
        This value controls the SPI clock frequency. Pass 0-65535. Default=0 corresponds to 65536 internally which
        results in ~800 kHz. 65500 = ~100 kHz, 65100 = ~10 kHz, 61100 = ~1 kHz, 21000 = ~100 Hz, and 1 = ~67 Hz. Avoid
        setting too low such that the entire transaction lasts longer than the 250 millisecond timeout of the internal
        watchdog timer.

        Register: SPI_SPEED_THROTTLE

        Address: 5005
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_SPEED_THROTTLE)  # type: ignore

    @spi_speed_throttle.setter
    @locked
    def spi_speed_throttle(self, value: int) -> None:
        """
        This value controls the SPI clock frequency. Pass 0-65535. Default=0 corresponds to 65536 internally which
        results in ~800 kHz. 65500 = ~100 kHz, 65100 = ~10 kHz, 61100 = ~1 kHz, 21000 = ~100 Hz, and 1 = ~67 Hz. Avoid
        setting too low such that the entire transaction lasts longer than the 250 millisecond timeout of the internal
        watchdog timer.

        Register: SPI_SPEED_THROTTLE

        Address: 5005
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_SPEED_THROTTLE, value)

    @property
    @locked
    def spi_options(self) -> int:
        """
        Bit 0 is Auto-CS-Disable.
        When bit 0 is 0, CS is enabled.
        When bit 0 is 1, CS is disabled.
        Bit 1:
        0 = Set DIO directions before starting the SPI operations, 1 = Do not set DIO directions.
        Bit 2:
        0 = Transmit data MSB first, 1 = LSB first.
        Bits 4-7:
        This value sets the number of bits that will be transmitted during the last byte of the SPI operation.
        Default is 8, valid options are 1-8.

        Register: SPI_OPTIONS

        Address: 5006
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_OPTIONS)  # type: ignore

    @spi_options.setter
    @locked
    def spi_options(self, value: int) -> None:
        """
        Bit 0 is Auto-CS-Disable.
        When bit 0 is 0, CS is enabled.
        When bit 0 is 1, CS is disabled.
        Bit 1:
        0 = Set DIO directions before starting the SPI operations, 1 = Do not set DIO directions.
        Bit 2:
        0 = Transmit data MSB first, 1 = LSB first.
        Bits 4-7:
        This value sets the number of bits that will be transmitted during the last byte of the SPI operation.
        Default is 8, valid options are 1-8.

        Register: SPI_OPTIONS

        Address: 5006
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_OPTIONS, value)

    @property
    @locked
    def spi_go(self) -> int:
        """
        Write 1 to begin the configured SPI transaction.

        Register: SPI_GO

        Address: 5007
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_GO)  # type: ignore

    @spi_go.setter
    @locked
    def spi_go(self, value: int) -> None:
        """
        Write 1 to begin the configured SPI transaction.

        Register: SPI_GO

        Address: 5007
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_GO, value)

    @property
    @locked
    def spi_num_bytes(self) -> int:
        """
        The number of bytes to transfer. The maximum transfer size is 100 bytes.

        Register: SPI_NUM_BYTES

        Address: 5009
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SPI_NUM_BYTES)  # type: ignore

    @spi_num_bytes.setter
    @locked
    def spi_num_bytes(self, value: int) -> None:
        """
        The number of bytes to transfer. The maximum transfer size is 100 bytes.

        Register: SPI_NUM_BYTES

        Address: 5009
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SPI_NUM_BYTES, value)

    @property
    @locked
    def spi_data_tx(self) -> int:
        """
        Write data here. This register is a buffer.

        Register: SPI_DATA_TX

        Address: 5010
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.SPI_DATA_TX)  # type: ignore

    @spi_data_tx.setter
    @locked
    def spi_data_tx(self, value: int) -> None:
        """
        Write data here. This register is a buffer.

        Register: SPI_DATA_TX

        Address: 5010
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.SPI_DATA_TX, value)

    @property
    @locked
    def spi_data_rx(self) -> int:
        """
        Read data here.

        This register is a buffer.
        Underrun behavior - fill with zeros.

        Register: SPI_DATA_RX

        Address: 5050
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.SPI_DATA_RX)  # type: ignore

    @property
    @locked
    def i2c_sda_dionum(self) -> int:
        """
        The number of the DIO line to be used as the I2C data line.
        Ex:
        Writing 0 will force FIO0 to become the I2C-SDA line.

        Register: I2C_SDA_DIONUM

        Address: 5100
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_SDA_DIONUM)  # type: ignore

    @i2c_sda_dionum.setter
    @locked
    def i2c_sda_dionum(self, value: int) -> None:
        """
        The number of the DIO line to be used as the I2C data line.
        Ex:
        Writing 0 will force FIO0 to become the I2C-SDA line.

        Register: I2C_SDA_DIONUM

        Address: 5100
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_SDA_DIONUM, value)

    @property
    @locked
    def i2c_scl_dionum(self) -> int:
        """
        The number of the DIO line to be used as the I2C clock line.
        Ex:
        Writing 1 will force FIO1 to become the I2C-SCL line.

        Register: I2C_SCL_DIONUM

        Address: 5101
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_SCL_DIONUM)  # type: ignore

    @i2c_scl_dionum.setter
    @locked
    def i2c_scl_dionum(self, value: int) -> None:
        """
        The number of the DIO line to be used as the I2C clock line.
        Ex:
        Writing 1 will force FIO1 to become the I2C-SCL line.

        Register: I2C_SCL_DIONUM

        Address: 5101
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_SCL_DIONUM, value)

    @property
    @locked
    def i2c_speed_throttle(self) -> int:
        """
        This value controls the I2C clock frequency. Pass 0-65535. Default=0 corresponds to 65536 internally which
        results in ~450 kHz. 1 results in ~40 Hz, 65516 is ~100 kHz.

        Register: I2C_SPEED_THROTTLE

        Address: 5102
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_SPEED_THROTTLE)  # type: ignore

    @i2c_speed_throttle.setter
    @locked
    def i2c_speed_throttle(self, value: int) -> None:
        """
        This value controls the I2C clock frequency. Pass 0-65535. Default=0 corresponds to 65536 internally which
        results in ~450 kHz. 1 results in ~40 Hz, 65516 is ~100 kHz.

        Register: I2C_SPEED_THROTTLE

        Address: 5102
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_SPEED_THROTTLE, value)

    @property
    @locked
    def i2c_options(self) -> int:
        """
        Advanced.
        Controls details of the I2C protocol to improve device compatibility.
        bit 0:
        1 = Reset the I2C bus before attempting communication.
        bit 1:
        0 = Restarts will use a stop and a start, 1 = Restarts will not use a stop.
        bit 2:
        1 = disable clock stretching.

        Register: I2C_OPTIONS

        Address: 5103
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_OPTIONS)  # type: ignore

    @i2c_options.setter
    @locked
    def i2c_options(self, value: int) -> None:
        """
        Advanced.
        Controls details of the I2C protocol to improve device compatibility.
        bit 0:
        1 = Reset the I2C bus before attempting communication.
        bit 1:
        0 = Restarts will use a stop and a start, 1 = Restarts will not use a stop.
        bit 2:
        1 = disable clock stretching.

        Register: I2C_OPTIONS

        Address: 5103
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_OPTIONS, value)

    @property
    @locked
    def i2c_slave_address(self) -> int:
        """
        The 7-bit address of the slave device. Value is shifted left by firmware to allow room for the I2C R/W bit.

        Register: I2C_SLAVE_ADDRESS

        Address: 5104
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_SLAVE_ADDRESS)  # type: ignore

    @i2c_slave_address.setter
    @locked
    def i2c_slave_address(self, value: int) -> None:
        """
        The 7-bit address of the slave device. Value is shifted left by firmware to allow room for the I2C R/W bit.

        Register: I2C_SLAVE_ADDRESS

        Address: 5104
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_SLAVE_ADDRESS, value)

    @property
    @locked
    def i2c_num_bytes_tx(self) -> int:
        """
        The number of data bytes to transmit. Zero is valid and will result in a read-only I2C operation.

        Register: I2C_NUM_BYTES_TX

        Address: 5108
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_NUM_BYTES_TX)  # type: ignore

    @i2c_num_bytes_tx.setter
    @locked
    def i2c_num_bytes_tx(self, value: int) -> None:
        """
        The number of data bytes to transmit. Zero is valid and will result in a read-only I2C operation.

        Register: I2C_NUM_BYTES_TX

        Address: 5108
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_NUM_BYTES_TX, value)

    @property
    @locked
    def i2c_num_bytes_rx(self) -> int:
        """
        The number of data bytes to read. Zero is valid and will result in a write-only I2C operation.

        Register: I2C_NUM_BYTES_RX

        Address: 5109
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_NUM_BYTES_RX)  # type: ignore

    @i2c_num_bytes_rx.setter
    @locked
    def i2c_num_bytes_rx(self, value: int) -> None:
        """
        The number of data bytes to read. Zero is valid and will result in a write-only I2C operation.

        Register: I2C_NUM_BYTES_RX

        Address: 5109
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_NUM_BYTES_RX, value)

    @property
    @locked
    def i2c_go(self) -> int:
        """
        Writing to this register will instruct the LabJack to perform an I2C transaction.

        Register: I2C_GO

        Address: 5110
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.I2C_GO)  # type: ignore

    @i2c_go.setter
    @locked
    def i2c_go(self, value: int) -> None:
        """
        Writing to this register will instruct the LabJack to perform an I2C transaction.

        Register: I2C_GO

        Address: 5110
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.I2C_GO, value)

    @property
    @locked
    def i2c_acks(self) -> int:
        """
        A binary encoded value (array of bits) used to observe ACKs from the slave device.

        Register: I2C_ACKS

        Address: 5114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.I2C_ACKS)  # type: ignore

    @i2c_acks.setter
    @locked
    def i2c_acks(self, value: int) -> None:
        """
        A binary encoded value (array of bits) used to observe ACKs from the slave device.

        Register: I2C_ACKS

        Address: 5114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.I2C_ACKS, value)

    @property
    @locked
    def i2c_data_tx(self) -> int:
        """
        Data that will be written to the I2C bus. This register is a buffer.

        Register: I2C_DATA_TX

        Address: 5120
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.I2C_DATA_TX)  # type: ignore

    @i2c_data_tx.setter
    @locked
    def i2c_data_tx(self, value: int) -> None:
        """
        Data that will be written to the I2C bus. This register is a buffer.

        Register: I2C_DATA_TX

        Address: 5120
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.I2C_DATA_TX, value)

    @property
    @locked
    def i2c_data_rx(self) -> int:
        """
        Data that has been read from the I2C bus.

        This register is a buffer.
        Underrun behavior - fill with zeros.

        Register: I2C_DATA_RX

        Address: 5160
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.I2C_DATA_RX)  # type: ignore

    @property
    @locked
    def onewire_dq_dionum(self) -> int:
        """
        The data-line DIO number.

        Register: ONEWIRE_DQ_DIONUM

        Address: 5300
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_DQ_DIONUM)  # type: ignore

    @onewire_dq_dionum.setter
    @locked
    def onewire_dq_dionum(self, value: int) -> None:
        """
        The data-line DIO number.

        Register: ONEWIRE_DQ_DIONUM

        Address: 5300
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_DQ_DIONUM, value)

    @property
    @locked
    def onewire_dpu_dionum(self) -> int:
        """
        The dynamic pullup control DIO number.

        Register: ONEWIRE_DPU_DIONUM

        Address: 5301
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_DPU_DIONUM)  # type: ignore

    @onewire_dpu_dionum.setter
    @locked
    def onewire_dpu_dionum(self, value: int) -> None:
        """
        The dynamic pullup control DIO number.

        Register: ONEWIRE_DPU_DIONUM

        Address: 5301
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_DPU_DIONUM, value)

    @property
    @locked
    def onewire_options(self) -> int:
        """
        Controls advanced features.
        Value is a bitmask.
        bit 0:
        reserved,
        bit 1:
        reserved,
        bit 2:
        1=DPU Enabled 0=DPU Disabled,
        bit 3:
        DPU Polarity 1=Active state is high, 0=Active state is low (Dynamic Pull-Up)

        Register: ONEWIRE_OPTIONS

        Address: 5302
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_OPTIONS)  # type: ignore

    @onewire_options.setter
    @locked
    def onewire_options(self, value: int) -> None:
        """
        Controls advanced features.
        Value is a bitmask.
        bit 0:
        reserved,
        bit 1:
        reserved,
        bit 2:
        1=DPU Enabled 0=DPU Disabled,
        bit 3:
        DPU Polarity 1=Active state is high, 0=Active state is low (Dynamic Pull-Up)

        Register: ONEWIRE_OPTIONS

        Address: 5302
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_OPTIONS, value)

    @property
    @locked
    def onewire_function(self) -> int:
        """
        Set the ROM function to use. 0xF0=Search, 0xCC=Skip, 0x55=Match, 0x33=Read.

        Register: ONEWIRE_FUNCTION

        Address: 5307
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_FUNCTION)  # type: ignore

    @onewire_function.setter
    @locked
    def onewire_function(self, value: int) -> None:
        """
        Set the ROM function to use. 0xF0=Search, 0xCC=Skip, 0x55=Match, 0x33=Read.

        Register: ONEWIRE_FUNCTION

        Address: 5307
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_FUNCTION, value)

    @property
    @locked
    def onewire_num_bytes_tx(self) -> int:
        """
        Number of data bytes to be sent.

        Register: ONEWIRE_NUM_BYTES_TX

        Address: 5308
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_NUM_BYTES_TX)  # type: ignore

    @onewire_num_bytes_tx.setter
    @locked
    def onewire_num_bytes_tx(self, value: int) -> None:
        """
        Number of data bytes to be sent.

        Register: ONEWIRE_NUM_BYTES_TX

        Address: 5308
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_NUM_BYTES_TX, value)

    @property
    @locked
    def onewire_num_bytes_rx(self) -> int:
        """
        Number of data bytes to be received.

        Register: ONEWIRE_NUM_BYTES_RX

        Address: 5309
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_NUM_BYTES_RX)  # type: ignore

    @onewire_num_bytes_rx.setter
    @locked
    def onewire_num_bytes_rx(self, value: int) -> None:
        """
        Number of data bytes to be received.

        Register: ONEWIRE_NUM_BYTES_RX

        Address: 5309
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_NUM_BYTES_RX, value)

    @property
    @locked
    def onewire_go(self) -> int:
        """
        Instructs the device to perform the configured 1-wire transaction.

        Register: ONEWIRE_GO

        Address: 5310
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ONEWIRE_GO)  # type: ignore

    @onewire_go.setter
    @locked
    def onewire_go(self, value: int) -> None:
        """
        Instructs the device to perform the configured 1-wire transaction.

        Register: ONEWIRE_GO

        Address: 5310
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ONEWIRE_GO, value)

    @property
    @locked
    def onewire_rom_match_h(self) -> int:
        """
        Upper 32-bits of the ROM to match.

        Register: ONEWIRE_ROM_MATCH_H

        Address: 5320
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_MATCH_H)  # type: ignore

    @onewire_rom_match_h.setter
    @locked
    def onewire_rom_match_h(self, value: int) -> None:
        """
        Upper 32-bits of the ROM to match.

        Register: ONEWIRE_ROM_MATCH_H

        Address: 5320
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_MATCH_H, value)

    @property
    @locked
    def onewire_rom_match_l(self) -> int:
        """
        Lower 32-bits of the ROM to match.

        Register: ONEWIRE_ROM_MATCH_L

        Address: 5322
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_MATCH_L)  # type: ignore

    @onewire_rom_match_l.setter
    @locked
    def onewire_rom_match_l(self, value: int) -> None:
        """
        Lower 32-bits of the ROM to match.

        Register: ONEWIRE_ROM_MATCH_L

        Address: 5322
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_MATCH_L, value)

    @property
    @locked
    def onewire_rom_branchs_found_h(self) -> int:
        """
        Upper 32-bits of the branches detected during a search.

        Register: ONEWIRE_ROM_BRANCHS_FOUND_H

        Address: 5332
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_BRANCHS_FOUND_H)  # type: ignore

    @property
    @locked
    def onewire_rom_branchs_found_l(self) -> int:
        """
        Lower 32-bits of the branches detected during a search.

        Register: ONEWIRE_ROM_BRANCHS_FOUND_L

        Address: 5334
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_ROM_BRANCHS_FOUND_L)  # type: ignore

    @property
    @locked
    def onewire_search_result_h(self) -> int:
        """
        Upper 32-bits of the search result.

        Register: ONEWIRE_SEARCH_RESULT_H

        Address: 5328
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_SEARCH_RESULT_H)  # type: ignore

    @property
    @locked
    def onewire_search_result_l(self) -> int:
        """
        Lower 32-bites of the search result.

        Register: ONEWIRE_SEARCH_RESULT_L

        Address: 5330
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_SEARCH_RESULT_L)  # type: ignore

    @property
    @locked
    def onewire_path_h(self) -> int:
        """
        Upper 32-bits of the path to take during a search.

        Register: ONEWIRE_PATH_H

        Address: 5324
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_PATH_H)  # type: ignore

    @onewire_path_h.setter
    @locked
    def onewire_path_h(self, value: int) -> None:
        """
        Upper 32-bits of the path to take during a search.

        Register: ONEWIRE_PATH_H

        Address: 5324
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ONEWIRE_PATH_H, value)

    @property
    @locked
    def onewire_path_l(self) -> int:
        """
        Lower 32-bits of the path to take during a search.

        Register: ONEWIRE_PATH_L

        Address: 5326
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ONEWIRE_PATH_L)  # type: ignore

    @onewire_path_l.setter
    @locked
    def onewire_path_l(self, value: int) -> None:
        """
        Lower 32-bits of the path to take during a search.

        Register: ONEWIRE_PATH_L

        Address: 5326
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ONEWIRE_PATH_L, value)

    @property
    @locked
    def onewire_data_tx(self) -> int:
        """
        Data to be transmitted over the 1-wire bus. This register is a buffer.

        Register: ONEWIRE_DATA_TX

        Address: 5340
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.ONEWIRE_DATA_TX)  # type: ignore

    @onewire_data_tx.setter
    @locked
    def onewire_data_tx(self, value: int) -> None:
        """
        Data to be transmitted over the 1-wire bus. This register is a buffer.

        Register: ONEWIRE_DATA_TX

        Address: 5340
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.ONEWIRE_DATA_TX, value)

    @property
    @locked
    def onewire_data_rx(self) -> int:
        """
        Data received over the 1-wire bus.

        This register is a buffer.
        Underrun behavior - buffer is static, old data will fill the extra locations, firmware 1.0225 changes this
        to read zeros.

        Register: ONEWIRE_DATA_RX

        Address: 5370
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.ONEWIRE_DATA_RX)  # type: ignore

    @property
    @locked
    def asynch_enable(self) -> int:
        """
        1 = Turn on Asynch. Configures timing hardware, DIO lines and allocates the receiving buffer.

        Register: ASYNCH_ENABLE

        Address: 5400
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_ENABLE)  # type: ignore

    @asynch_enable.setter
    @locked
    def asynch_enable(self, value: int) -> None:
        """
        1 = Turn on Asynch. Configures timing hardware, DIO lines and allocates the receiving buffer.

        Register: ASYNCH_ENABLE

        Address: 5400
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_ENABLE, value)

    @property
    @locked
    def asynch_baud(self) -> int:
        """
        The symbol rate that will be used for communication. 9600 is typical. Up to 38400 works, but heavily loads
        the T7's processor.

        Register: ASYNCH_BAUD

        Address: 5420
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ASYNCH_BAUD)  # type: ignore

    @asynch_baud.setter
    @locked
    def asynch_baud(self, value: int) -> None:
        """
        The symbol rate that will be used for communication. 9600 is typical. Up to 38400 works, but heavily loads
        the T7's processor.

        Register: ASYNCH_BAUD

        Address: 5420
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ASYNCH_BAUD, value)

    @property
    @locked
    def asynch_rx_dionum(self) -> int:
        """
        The DIO line that will receive data. (RX)

        Register: ASYNCH_RX_DIONUM

        Address: 5405
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_RX_DIONUM)  # type: ignore

    @asynch_rx_dionum.setter
    @locked
    def asynch_rx_dionum(self, value: int) -> None:
        """
        The DIO line that will receive data. (RX)

        Register: ASYNCH_RX_DIONUM

        Address: 5405
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_RX_DIONUM, value)

    @property
    @locked
    def asynch_tx_dionum(self) -> int:
        """
        The DIO line that will transmit data. (TX)

        Register: ASYNCH_TX_DIONUM

        Address: 5410
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_TX_DIONUM)  # type: ignore

    @asynch_tx_dionum.setter
    @locked
    def asynch_tx_dionum(self, value: int) -> None:
        """
        The DIO line that will transmit data. (TX)

        Register: ASYNCH_TX_DIONUM

        Address: 5410
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_TX_DIONUM, value)

    @property
    @locked
    def asynch_num_data_bits(self) -> int:
        """
        The number of data bits per frame. 0-8, 0=8.

        Register: ASYNCH_NUM_DATA_BITS

        Address: 5415
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_DATA_BITS)  # type: ignore

    @asynch_num_data_bits.setter
    @locked
    def asynch_num_data_bits(self, value: int) -> None:
        """
        The number of data bits per frame. 0-8, 0=8.

        Register: ASYNCH_NUM_DATA_BITS

        Address: 5415
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_DATA_BITS, value)

    @property
    @locked
    def asynch_rx_buffer_size_bytes(self) -> int:
        """
        Number of bytes to use for the receiving buffer. Max is 2048. 0 = 200.

        Register: ASYNCH_RX_BUFFER_SIZE_BYTES

        Address: 5430
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_RX_BUFFER_SIZE_BYTES)  # type: ignore

    @asynch_rx_buffer_size_bytes.setter
    @locked
    def asynch_rx_buffer_size_bytes(self, value: int) -> None:
        """
        Number of bytes to use for the receiving buffer. Max is 2048. 0 = 200.

        Register: ASYNCH_RX_BUFFER_SIZE_BYTES

        Address: 5430
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_RX_BUFFER_SIZE_BYTES, value)

    @property
    @locked
    def asynch_num_bytes_rx(self) -> int:
        """
        The number of data bytes that have been received.

        Register: ASYNCH_NUM_BYTES_RX

        Address: 5435
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_BYTES_RX)  # type: ignore

    @property
    @locked
    def asynch_num_bytes_tx(self) -> int:
        """
        The number of bytes to be transmitted after writing to GO. Max is 256.

        Register: ASYNCH_NUM_BYTES_TX

        Address: 5440
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_BYTES_TX)  # type: ignore

    @asynch_num_bytes_tx.setter
    @locked
    def asynch_num_bytes_tx(self, value: int) -> None:
        """
        The number of bytes to be transmitted after writing to GO. Max is 256.

        Register: ASYNCH_NUM_BYTES_TX

        Address: 5440
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_BYTES_TX, value)

    @property
    @locked
    def asynch_tx_go(self) -> int:
        """
        Write a 1 to this register to initiate a transmission.

        Register: ASYNCH_TX_GO

        Address: 5450
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_TX_GO)  # type: ignore

    @asynch_tx_go.setter
    @locked
    def asynch_tx_go(self, value: int) -> None:
        """
        Write a 1 to this register to initiate a transmission.

        Register: ASYNCH_TX_GO

        Address: 5450
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_TX_GO, value)

    @property
    @locked
    def asynch_num_stop_bits(self) -> int:
        """
        The number of stop bits.
        Values:
        0 = zero stop bits,
        1 = one stop bit,
        2 = two stop bits.

        Register: ASYNCH_NUM_STOP_BITS

        Address: 5455
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_STOP_BITS)  # type: ignore

    @asynch_num_stop_bits.setter
    @locked
    def asynch_num_stop_bits(self, value: int) -> None:
        """
        The number of stop bits.
        Values:
        0 = zero stop bits,
        1 = one stop bit,
        2 = two stop bits.

        Register: ASYNCH_NUM_STOP_BITS

        Address: 5455
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_STOP_BITS, value)

    @property
    @locked
    def asynch_parity(self) -> int:
        """
        Parity setting:
        0=none,
        1=odd,
        2=even.

        Register: ASYNCH_PARITY

        Address: 5460
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_PARITY)  # type: ignore

    @asynch_parity.setter
    @locked
    def asynch_parity(self, value: int) -> None:
        """
        Parity setting:
        0=none,
        1=odd,
        2=even.

        Register: ASYNCH_PARITY

        Address: 5460
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_PARITY, value)

    @property
    @locked
    def asynch_num_parity_errors(self) -> int:
        """
        The number of parity errors that have been detected. Cleared when UART is enabled. Can also be cleared by
        writing 0.

        Register: ASYNCH_NUM_PARITY_ERRORS

        Address: 5465
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_PARITY_ERRORS)  # type: ignore

    @asynch_num_parity_errors.setter
    @locked
    def asynch_num_parity_errors(self, value: int) -> None:
        """
        The number of parity errors that have been detected. Cleared when UART is enabled. Can also be cleared by
        writing 0.

        Register: ASYNCH_NUM_PARITY_ERRORS

        Address: 5465
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_NUM_PARITY_ERRORS, value)

    @property
    @locked
    def asynch_data_tx(self) -> int:
        """
        Write data to be transmitted here. This register is a buffer.

        Register: ASYNCH_DATA_TX

        Address: 5490
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_DATA_TX)  # type: ignore

    @asynch_data_tx.setter
    @locked
    def asynch_data_tx(self, value: int) -> None:
        """
        Write data to be transmitted here. This register is a buffer.

        Register: ASYNCH_DATA_TX

        Address: 5490
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ASYNCH_DATA_TX, value)

    @property
    @locked
    def asynch_data_rx(self) -> int:
        """
        Read received data from here.

        This register is a buffer.
        Underrun behavior - fill with zeros.

        Register: ASYNCH_DATA_RX

        Address: 5495
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ASYNCH_DATA_RX)  # type: ignore

    @property
    @locked
    def lua_run(self) -> int:
        """
        Writing 1 compiles and runs the Lua script that is loaded in RAM. Writing zero stops the script and begins
        cleaning up memory. Users may poll the register after writing a value of 0 to verify that the VM is unloaded,
        and garbage collection is complete. 0 = VM fully unloaded. 1 = Running/in-progress.

        Register: LUA_RUN

        Address: 6000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_RUN)  # type: ignore

    @lua_run.setter
    @locked
    def lua_run(self, value: int) -> None:
        """
        Writing 1 compiles and runs the Lua script that is loaded in RAM. Writing zero stops the script and begins
        cleaning up memory. Users may poll the register after writing a value of 0 to verify that the VM is unloaded,
        and garbage collection is complete. 0 = VM fully unloaded. 1 = Running/in-progress.

        Register: LUA_RUN

        Address: 6000
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_RUN, value)

    @property
    @locked
    def lua_source_size(self) -> int:
        """
        Allocates RAM for the source code.

        Register: LUA_SOURCE_SIZE

        Address: 6012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_SOURCE_SIZE)  # type: ignore

    @lua_source_size.setter
    @locked
    def lua_source_size(self, value: int) -> None:
        """
        Allocates RAM for the source code.

        Register: LUA_SOURCE_SIZE

        Address: 6012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_SOURCE_SIZE, value)

    @property
    @locked
    def lua_source_write(self) -> int:
        """
        Write the source code here. Source will be saved to the RAM allocated by LUA_SOURCE_SIZE. This register is a
        buffer.

        Register: LUA_SOURCE_WRITE

        Address: 6014
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.LUA_SOURCE_WRITE)  # type: ignore

    @lua_source_write.setter
    @locked
    def lua_source_write(self, value: int) -> None:
        """
        Write the source code here. Source will be saved to the RAM allocated by LUA_SOURCE_SIZE. This register is a
        buffer.

        Register: LUA_SOURCE_WRITE

        Address: 6014
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.LUA_SOURCE_WRITE, value)

    @property
    @locked
    def lua_debug_enable(self) -> int:
        """
        Write 1 to this register to enable debugging.

        Register: LUA_DEBUG_ENABLE

        Address: 6020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_ENABLE)  # type: ignore

    @lua_debug_enable.setter
    @locked
    def lua_debug_enable(self, value: int) -> None:
        """
        Write 1 to this register to enable debugging.

        Register: LUA_DEBUG_ENABLE

        Address: 6020
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_ENABLE, value)

    @property
    @locked
    def lua_debug_num_bytes(self) -> int:
        """
        The number of data bytes in the debug buffer waiting to be read.

        Register: LUA_DEBUG_NUM_BYTES

        Address: 6022
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_NUM_BYTES)  # type: ignore

    @property
    @locked
    def lua_debug_data(self) -> int:
        """
        Read debug data from here. This register is a buffer.

        Register: LUA_DEBUG_DATA

        Address: 6024
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.LUA_DEBUG_DATA)  # type: ignore

    @property
    @locked
    def lua_save_to_flash(self) -> int:
        """
        Write 1 to save the loaded source code to flash.

        Register: LUA_SAVE_TO_FLASH

        Address: 6032
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_SAVE_TO_FLASH)  # type: ignore

    @lua_save_to_flash.setter
    @locked
    def lua_save_to_flash(self, value: int) -> None:
        """
        Write 1 to save the loaded source code to flash.

        Register: LUA_SAVE_TO_FLASH

        Address: 6032
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_SAVE_TO_FLASH, value)

    @property
    @locked
    def lua_load_saved(self) -> int:
        """
        Writing any value reads the script from Flash and loads it into RAM. The script can now be compiled and run.

        Register: LUA_LOAD_SAVED

        Address: 6034
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_LOAD_SAVED)  # type: ignore

    @lua_load_saved.setter
    @locked
    def lua_load_saved(self, value: int) -> None:
        """
        Writing any value reads the script from Flash and loads it into RAM. The script can now be compiled and run.

        Register: LUA_LOAD_SAVED

        Address: 6034
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_LOAD_SAVED, value)

    @property
    @locked
    def lua_saved_read_pointer(self) -> int:
        """
        Address within the saved Lua script in Flash to begin reading from.

        Register: LUA_SAVED_READ_POINTER

        Address: 6036
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_SAVED_READ_POINTER)  # type: ignore

    @lua_saved_read_pointer.setter
    @locked
    def lua_saved_read_pointer(self, value: int) -> None:
        """
        Address within the saved Lua script in Flash to begin reading from.

        Register: LUA_SAVED_READ_POINTER

        Address: 6036
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_SAVED_READ_POINTER, value)

    @property
    @locked
    def lua_saved_read(self) -> int:
        """
        Read script saved to flash from here.

        Register: LUA_SAVED_READ

        Address: 6038
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_SAVED_READ)  # type: ignore

    @property
    @locked
    def lua_no_warn_truncation(self) -> int:
        """
        Writing a 1 to this register will prevent truncation warnings from being displayed. This register will be
        cleared every time Lua is started.

        Register: LUA_NO_WARN_TRUNCATION

        Address: 6050
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LUA_NO_WARN_TRUNCATION)  # type: ignore

    @lua_no_warn_truncation.setter
    @locked
    def lua_no_warn_truncation(self, value: int) -> None:
        """
        Writing a 1 to this register will prevent truncation warnings from being displayed. This register will be
        cleared every time Lua is started.

        Register: LUA_NO_WARN_TRUNCATION

        Address: 6050
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.LUA_NO_WARN_TRUNCATION, value)

    @property
    @locked
    def lua_run_default(self) -> int:
        """
        If set to 1 the script saved in flash will be loaded and run when the LabJack boots up.

        Register: LUA_RUN_DEFAULT

        Address: 6100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_RUN_DEFAULT)  # type: ignore

    @lua_run_default.setter
    @locked
    def lua_run_default(self, value: int) -> None:
        """
        If set to 1 the script saved in flash will be loaded and run when the LabJack boots up.

        Register: LUA_RUN_DEFAULT

        Address: 6100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_RUN_DEFAULT, value)

    @property
    @locked
    def lua_debug_enable_default(self) -> int:
        """
        This is the value that will be loaded into LUA_DEBUG_ENABLE when the LabJack boots up.

        Register: LUA_DEBUG_ENABLE_DEFAULT

        Address: 6120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_ENABLE_DEFAULT)  # type: ignore

    @lua_debug_enable_default.setter
    @locked
    def lua_debug_enable_default(self, value: int) -> None:
        """
        This is the value that will be loaded into LUA_DEBUG_ENABLE when the LabJack boots up.

        Register: LUA_DEBUG_ENABLE_DEFAULT

        Address: 6120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_ENABLE_DEFAULT, value)

    @property
    @locked
    def lua_debug_num_bytes_default(self) -> int:
        """
        This is the number of bytes that will be used for the lua debug buffer.

        Register: LUA_DEBUG_NUM_BYTES_DEFAULT

        Address: 6122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_NUM_BYTES_DEFAULT)  # type: ignore

    @lua_debug_num_bytes_default.setter
    @locked
    def lua_debug_num_bytes_default(self, value: int) -> None:
        """
        This is the number of bytes that will be used for the lua debug buffer.

        Register: LUA_DEBUG_NUM_BYTES_DEFAULT

        Address: 6122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.LUA_DEBUG_NUM_BYTES_DEFAULT, value)

    @property
    @locked
    def ain0_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_A (AIN#(0:149)_EF_READ_A@0)

        Address: 7000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain1_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_A (AIN#(0:149)_EF_READ_A@1)

        Address: 7002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain2_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_A (AIN#(0:149)_EF_READ_A@2)

        Address: 7004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain3_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_A (AIN#(0:149)_EF_READ_A@3)

        Address: 7006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain4_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_A (AIN#(0:149)_EF_READ_A@4)

        Address: 7008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain5_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_A (AIN#(0:149)_EF_READ_A@5)

        Address: 7010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain6_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_A (AIN#(0:149)_EF_READ_A@6)

        Address: 7012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain7_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_A (AIN#(0:149)_EF_READ_A@7)

        Address: 7014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain8_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_A (AIN#(0:149)_EF_READ_A@8)

        Address: 7016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain9_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_A (AIN#(0:149)_EF_READ_A@9)

        Address: 7018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain10_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_A (AIN#(0:149)_EF_READ_A@10)

        Address: 7020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain11_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_A (AIN#(0:149)_EF_READ_A@11)

        Address: 7022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain12_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_A (AIN#(0:149)_EF_READ_A@12)

        Address: 7024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain13_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_A (AIN#(0:149)_EF_READ_A@13)

        Address: 7026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain14_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_A (AIN#(0:149)_EF_READ_A@14)

        Address: 7028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain15_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_A (AIN#(0:149)_EF_READ_A@15)

        Address: 7030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain16_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_A (AIN#(0:149)_EF_READ_A@16)

        Address: 7032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain17_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_A (AIN#(0:149)_EF_READ_A@17)

        Address: 7034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain18_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_A (AIN#(0:149)_EF_READ_A@18)

        Address: 7036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain19_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_A (AIN#(0:149)_EF_READ_A@19)

        Address: 7038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain20_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_A (AIN#(0:149)_EF_READ_A@20)

        Address: 7040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain21_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_A (AIN#(0:149)_EF_READ_A@21)

        Address: 7042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain22_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_A (AIN#(0:149)_EF_READ_A@22)

        Address: 7044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain23_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_A (AIN#(0:149)_EF_READ_A@23)

        Address: 7046
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain24_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_A (AIN#(0:149)_EF_READ_A@24)

        Address: 7048
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain25_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_A (AIN#(0:149)_EF_READ_A@25)

        Address: 7050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain26_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_A (AIN#(0:149)_EF_READ_A@26)

        Address: 7052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain27_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_A (AIN#(0:149)_EF_READ_A@27)

        Address: 7054
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain28_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_A (AIN#(0:149)_EF_READ_A@28)

        Address: 7056
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain29_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_A (AIN#(0:149)_EF_READ_A@29)

        Address: 7058
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain30_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_A (AIN#(0:149)_EF_READ_A@30)

        Address: 7060
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain31_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_A (AIN#(0:149)_EF_READ_A@31)

        Address: 7062
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain32_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_A (AIN#(0:149)_EF_READ_A@32)

        Address: 7064
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain33_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_A (AIN#(0:149)_EF_READ_A@33)

        Address: 7066
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain34_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_A (AIN#(0:149)_EF_READ_A@34)

        Address: 7068
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain35_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_A (AIN#(0:149)_EF_READ_A@35)

        Address: 7070
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain36_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_A (AIN#(0:149)_EF_READ_A@36)

        Address: 7072
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain37_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_A (AIN#(0:149)_EF_READ_A@37)

        Address: 7074
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain38_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_A (AIN#(0:149)_EF_READ_A@38)

        Address: 7076
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain39_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_A (AIN#(0:149)_EF_READ_A@39)

        Address: 7078
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain40_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_A (AIN#(0:149)_EF_READ_A@40)

        Address: 7080
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain41_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_A (AIN#(0:149)_EF_READ_A@41)

        Address: 7082
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain42_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_A (AIN#(0:149)_EF_READ_A@42)

        Address: 7084
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain43_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_A (AIN#(0:149)_EF_READ_A@43)

        Address: 7086
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain44_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_A (AIN#(0:149)_EF_READ_A@44)

        Address: 7088
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain45_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_A (AIN#(0:149)_EF_READ_A@45)

        Address: 7090
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain46_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_A (AIN#(0:149)_EF_READ_A@46)

        Address: 7092
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain47_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_A (AIN#(0:149)_EF_READ_A@47)

        Address: 7094
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain48_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_A (AIN#(0:149)_EF_READ_A@48)

        Address: 7096
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain49_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_A (AIN#(0:149)_EF_READ_A@49)

        Address: 7098
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain50_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_A (AIN#(0:149)_EF_READ_A@50)

        Address: 7100
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain51_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_A (AIN#(0:149)_EF_READ_A@51)

        Address: 7102
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain52_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_A (AIN#(0:149)_EF_READ_A@52)

        Address: 7104
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain53_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_A (AIN#(0:149)_EF_READ_A@53)

        Address: 7106
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain54_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_A (AIN#(0:149)_EF_READ_A@54)

        Address: 7108
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain55_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_A (AIN#(0:149)_EF_READ_A@55)

        Address: 7110
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain56_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_A (AIN#(0:149)_EF_READ_A@56)

        Address: 7112
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain57_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_A (AIN#(0:149)_EF_READ_A@57)

        Address: 7114
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain58_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_A (AIN#(0:149)_EF_READ_A@58)

        Address: 7116
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain59_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_A (AIN#(0:149)_EF_READ_A@59)

        Address: 7118
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain60_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_A (AIN#(0:149)_EF_READ_A@60)

        Address: 7120
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain61_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_A (AIN#(0:149)_EF_READ_A@61)

        Address: 7122
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain62_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_A (AIN#(0:149)_EF_READ_A@62)

        Address: 7124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain63_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_A (AIN#(0:149)_EF_READ_A@63)

        Address: 7126
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain64_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_A (AIN#(0:149)_EF_READ_A@64)

        Address: 7128
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain65_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_A (AIN#(0:149)_EF_READ_A@65)

        Address: 7130
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain66_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_A (AIN#(0:149)_EF_READ_A@66)

        Address: 7132
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain67_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_A (AIN#(0:149)_EF_READ_A@67)

        Address: 7134
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain68_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_A (AIN#(0:149)_EF_READ_A@68)

        Address: 7136
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain69_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_A (AIN#(0:149)_EF_READ_A@69)

        Address: 7138
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain70_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_A (AIN#(0:149)_EF_READ_A@70)

        Address: 7140
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain71_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_A (AIN#(0:149)_EF_READ_A@71)

        Address: 7142
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain72_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_A (AIN#(0:149)_EF_READ_A@72)

        Address: 7144
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain73_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_A (AIN#(0:149)_EF_READ_A@73)

        Address: 7146
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain74_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_A (AIN#(0:149)_EF_READ_A@74)

        Address: 7148
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain75_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_A (AIN#(0:149)_EF_READ_A@75)

        Address: 7150
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain76_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_A (AIN#(0:149)_EF_READ_A@76)

        Address: 7152
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain77_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_A (AIN#(0:149)_EF_READ_A@77)

        Address: 7154
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain78_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_A (AIN#(0:149)_EF_READ_A@78)

        Address: 7156
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain79_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_A (AIN#(0:149)_EF_READ_A@79)

        Address: 7158
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain80_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_A (AIN#(0:149)_EF_READ_A@80)

        Address: 7160
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain81_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_A (AIN#(0:149)_EF_READ_A@81)

        Address: 7162
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain82_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_A (AIN#(0:149)_EF_READ_A@82)

        Address: 7164
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain83_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_A (AIN#(0:149)_EF_READ_A@83)

        Address: 7166
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain84_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_A (AIN#(0:149)_EF_READ_A@84)

        Address: 7168
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain85_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_A (AIN#(0:149)_EF_READ_A@85)

        Address: 7170
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain86_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_A (AIN#(0:149)_EF_READ_A@86)

        Address: 7172
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain87_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_A (AIN#(0:149)_EF_READ_A@87)

        Address: 7174
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain88_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_A (AIN#(0:149)_EF_READ_A@88)

        Address: 7176
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain89_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_A (AIN#(0:149)_EF_READ_A@89)

        Address: 7178
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain90_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_A (AIN#(0:149)_EF_READ_A@90)

        Address: 7180
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain91_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_A (AIN#(0:149)_EF_READ_A@91)

        Address: 7182
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain92_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_A (AIN#(0:149)_EF_READ_A@92)

        Address: 7184
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain93_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_A (AIN#(0:149)_EF_READ_A@93)

        Address: 7186
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain94_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_A (AIN#(0:149)_EF_READ_A@94)

        Address: 7188
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain95_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_A (AIN#(0:149)_EF_READ_A@95)

        Address: 7190
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain96_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_A (AIN#(0:149)_EF_READ_A@96)

        Address: 7192
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain97_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_A (AIN#(0:149)_EF_READ_A@97)

        Address: 7194
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain98_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_A (AIN#(0:149)_EF_READ_A@98)

        Address: 7196
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain99_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_A (AIN#(0:149)_EF_READ_A@99)

        Address: 7198
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain100_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_A (AIN#(0:149)_EF_READ_A@100)

        Address: 7200
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain101_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_A (AIN#(0:149)_EF_READ_A@101)

        Address: 7202
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain102_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_A (AIN#(0:149)_EF_READ_A@102)

        Address: 7204
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain103_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_A (AIN#(0:149)_EF_READ_A@103)

        Address: 7206
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain104_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_A (AIN#(0:149)_EF_READ_A@104)

        Address: 7208
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain105_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_A (AIN#(0:149)_EF_READ_A@105)

        Address: 7210
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain106_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_A (AIN#(0:149)_EF_READ_A@106)

        Address: 7212
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain107_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_A (AIN#(0:149)_EF_READ_A@107)

        Address: 7214
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain108_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_A (AIN#(0:149)_EF_READ_A@108)

        Address: 7216
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain109_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_A (AIN#(0:149)_EF_READ_A@109)

        Address: 7218
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain110_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_A (AIN#(0:149)_EF_READ_A@110)

        Address: 7220
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain111_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_A (AIN#(0:149)_EF_READ_A@111)

        Address: 7222
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain112_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_A (AIN#(0:149)_EF_READ_A@112)

        Address: 7224
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain113_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_A (AIN#(0:149)_EF_READ_A@113)

        Address: 7226
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain114_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_A (AIN#(0:149)_EF_READ_A@114)

        Address: 7228
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain115_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_A (AIN#(0:149)_EF_READ_A@115)

        Address: 7230
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain116_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_A (AIN#(0:149)_EF_READ_A@116)

        Address: 7232
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain117_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_A (AIN#(0:149)_EF_READ_A@117)

        Address: 7234
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain118_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_A (AIN#(0:149)_EF_READ_A@118)

        Address: 7236
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain119_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_A (AIN#(0:149)_EF_READ_A@119)

        Address: 7238
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain120_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_A (AIN#(0:149)_EF_READ_A@120)

        Address: 7240
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain121_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_A (AIN#(0:149)_EF_READ_A@121)

        Address: 7242
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain122_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_A (AIN#(0:149)_EF_READ_A@122)

        Address: 7244
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain123_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_A (AIN#(0:149)_EF_READ_A@123)

        Address: 7246
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain124_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_A (AIN#(0:149)_EF_READ_A@124)

        Address: 7248
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain125_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_A (AIN#(0:149)_EF_READ_A@125)

        Address: 7250
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain126_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_A (AIN#(0:149)_EF_READ_A@126)

        Address: 7252
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain127_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_A (AIN#(0:149)_EF_READ_A@127)

        Address: 7254
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain128_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_A (AIN#(0:149)_EF_READ_A@128)

        Address: 7256
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain129_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_A (AIN#(0:149)_EF_READ_A@129)

        Address: 7258
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain130_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_A (AIN#(0:149)_EF_READ_A@130)

        Address: 7260
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain131_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_A (AIN#(0:149)_EF_READ_A@131)

        Address: 7262
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain132_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_A (AIN#(0:149)_EF_READ_A@132)

        Address: 7264
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain133_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_A (AIN#(0:149)_EF_READ_A@133)

        Address: 7266
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain134_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_A (AIN#(0:149)_EF_READ_A@134)

        Address: 7268
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain135_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_A (AIN#(0:149)_EF_READ_A@135)

        Address: 7270
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain136_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_A (AIN#(0:149)_EF_READ_A@136)

        Address: 7272
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain137_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_A (AIN#(0:149)_EF_READ_A@137)

        Address: 7274
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain138_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_A (AIN#(0:149)_EF_READ_A@138)

        Address: 7276
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain139_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_A (AIN#(0:149)_EF_READ_A@139)

        Address: 7278
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain140_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_A (AIN#(0:149)_EF_READ_A@140)

        Address: 7280
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain141_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_A (AIN#(0:149)_EF_READ_A@141)

        Address: 7282
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain142_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_A (AIN#(0:149)_EF_READ_A@142)

        Address: 7284
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain143_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_A (AIN#(0:149)_EF_READ_A@143)

        Address: 7286
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain144_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_A (AIN#(0:149)_EF_READ_A@144)

        Address: 7288
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain145_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_A (AIN#(0:149)_EF_READ_A@145)

        Address: 7290
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain146_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_A (AIN#(0:149)_EF_READ_A@146)

        Address: 7292
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain147_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_A (AIN#(0:149)_EF_READ_A@147)

        Address: 7294
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain148_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_A (AIN#(0:149)_EF_READ_A@148)

        Address: 7296
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain149_ef_read_a(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_A (AIN#(0:149)_EF_READ_A@149)

        Address: 7298
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_A)  # type: ignore

    @property
    @locked
    def ain0_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_B (AIN#(0:149)_EF_READ_B@0)

        Address: 7300
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_B)  # type: ignore

    @ain0_ef_read_b.setter
    @locked
    def ain0_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_B (AIN#(0:149)_EF_READ_B@0)

        Address: 7300
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_B, value)

    @property
    @locked
    def ain1_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_B (AIN#(0:149)_EF_READ_B@1)

        Address: 7302
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_B)  # type: ignore

    @ain1_ef_read_b.setter
    @locked
    def ain1_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_B (AIN#(0:149)_EF_READ_B@1)

        Address: 7302
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_B, value)

    @property
    @locked
    def ain2_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_B (AIN#(0:149)_EF_READ_B@2)

        Address: 7304
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_B)  # type: ignore

    @ain2_ef_read_b.setter
    @locked
    def ain2_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_B (AIN#(0:149)_EF_READ_B@2)

        Address: 7304
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_B, value)

    @property
    @locked
    def ain3_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_B (AIN#(0:149)_EF_READ_B@3)

        Address: 7306
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_B)  # type: ignore

    @ain3_ef_read_b.setter
    @locked
    def ain3_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_B (AIN#(0:149)_EF_READ_B@3)

        Address: 7306
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_B, value)

    @property
    @locked
    def ain4_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_B (AIN#(0:149)_EF_READ_B@4)

        Address: 7308
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_B)  # type: ignore

    @ain4_ef_read_b.setter
    @locked
    def ain4_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_B (AIN#(0:149)_EF_READ_B@4)

        Address: 7308
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_B, value)

    @property
    @locked
    def ain5_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_B (AIN#(0:149)_EF_READ_B@5)

        Address: 7310
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_B)  # type: ignore

    @ain5_ef_read_b.setter
    @locked
    def ain5_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_B (AIN#(0:149)_EF_READ_B@5)

        Address: 7310
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_B, value)

    @property
    @locked
    def ain6_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_B (AIN#(0:149)_EF_READ_B@6)

        Address: 7312
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_B)  # type: ignore

    @ain6_ef_read_b.setter
    @locked
    def ain6_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_B (AIN#(0:149)_EF_READ_B@6)

        Address: 7312
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_B, value)

    @property
    @locked
    def ain7_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_B (AIN#(0:149)_EF_READ_B@7)

        Address: 7314
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_B)  # type: ignore

    @ain7_ef_read_b.setter
    @locked
    def ain7_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_B (AIN#(0:149)_EF_READ_B@7)

        Address: 7314
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_B, value)

    @property
    @locked
    def ain8_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_B (AIN#(0:149)_EF_READ_B@8)

        Address: 7316
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_B)  # type: ignore

    @ain8_ef_read_b.setter
    @locked
    def ain8_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_B (AIN#(0:149)_EF_READ_B@8)

        Address: 7316
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_B, value)

    @property
    @locked
    def ain9_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_B (AIN#(0:149)_EF_READ_B@9)

        Address: 7318
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_B)  # type: ignore

    @ain9_ef_read_b.setter
    @locked
    def ain9_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_B (AIN#(0:149)_EF_READ_B@9)

        Address: 7318
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_B, value)

    @property
    @locked
    def ain10_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_B (AIN#(0:149)_EF_READ_B@10)

        Address: 7320
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_B)  # type: ignore

    @ain10_ef_read_b.setter
    @locked
    def ain10_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_B (AIN#(0:149)_EF_READ_B@10)

        Address: 7320
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_B, value)

    @property
    @locked
    def ain11_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_B (AIN#(0:149)_EF_READ_B@11)

        Address: 7322
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_B)  # type: ignore

    @ain11_ef_read_b.setter
    @locked
    def ain11_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_B (AIN#(0:149)_EF_READ_B@11)

        Address: 7322
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_B, value)

    @property
    @locked
    def ain12_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_B (AIN#(0:149)_EF_READ_B@12)

        Address: 7324
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_B)  # type: ignore

    @ain12_ef_read_b.setter
    @locked
    def ain12_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_B (AIN#(0:149)_EF_READ_B@12)

        Address: 7324
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_B, value)

    @property
    @locked
    def ain13_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_B (AIN#(0:149)_EF_READ_B@13)

        Address: 7326
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_B)  # type: ignore

    @ain13_ef_read_b.setter
    @locked
    def ain13_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_B (AIN#(0:149)_EF_READ_B@13)

        Address: 7326
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_B, value)

    @property
    @locked
    def ain14_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_B (AIN#(0:149)_EF_READ_B@14)

        Address: 7328
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_B)  # type: ignore

    @ain14_ef_read_b.setter
    @locked
    def ain14_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_B (AIN#(0:149)_EF_READ_B@14)

        Address: 7328
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_B, value)

    @property
    @locked
    def ain15_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_B (AIN#(0:149)_EF_READ_B@15)

        Address: 7330
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_B)  # type: ignore

    @ain15_ef_read_b.setter
    @locked
    def ain15_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_B (AIN#(0:149)_EF_READ_B@15)

        Address: 7330
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_B, value)

    @property
    @locked
    def ain16_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_B (AIN#(0:149)_EF_READ_B@16)

        Address: 7332
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_B)  # type: ignore

    @ain16_ef_read_b.setter
    @locked
    def ain16_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_B (AIN#(0:149)_EF_READ_B@16)

        Address: 7332
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_B, value)

    @property
    @locked
    def ain17_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_B (AIN#(0:149)_EF_READ_B@17)

        Address: 7334
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_B)  # type: ignore

    @ain17_ef_read_b.setter
    @locked
    def ain17_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_B (AIN#(0:149)_EF_READ_B@17)

        Address: 7334
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_B, value)

    @property
    @locked
    def ain18_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_B (AIN#(0:149)_EF_READ_B@18)

        Address: 7336
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_B)  # type: ignore

    @ain18_ef_read_b.setter
    @locked
    def ain18_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_B (AIN#(0:149)_EF_READ_B@18)

        Address: 7336
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_B, value)

    @property
    @locked
    def ain19_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_B (AIN#(0:149)_EF_READ_B@19)

        Address: 7338
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_B)  # type: ignore

    @ain19_ef_read_b.setter
    @locked
    def ain19_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_B (AIN#(0:149)_EF_READ_B@19)

        Address: 7338
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_B, value)

    @property
    @locked
    def ain20_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_B (AIN#(0:149)_EF_READ_B@20)

        Address: 7340
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_B)  # type: ignore

    @ain20_ef_read_b.setter
    @locked
    def ain20_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_B (AIN#(0:149)_EF_READ_B@20)

        Address: 7340
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_B, value)

    @property
    @locked
    def ain21_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_B (AIN#(0:149)_EF_READ_B@21)

        Address: 7342
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_B)  # type: ignore

    @ain21_ef_read_b.setter
    @locked
    def ain21_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_B (AIN#(0:149)_EF_READ_B@21)

        Address: 7342
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_B, value)

    @property
    @locked
    def ain22_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_B (AIN#(0:149)_EF_READ_B@22)

        Address: 7344
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_B)  # type: ignore

    @ain22_ef_read_b.setter
    @locked
    def ain22_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_B (AIN#(0:149)_EF_READ_B@22)

        Address: 7344
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_B, value)

    @property
    @locked
    def ain23_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_B (AIN#(0:149)_EF_READ_B@23)

        Address: 7346
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_B)  # type: ignore

    @ain23_ef_read_b.setter
    @locked
    def ain23_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_B (AIN#(0:149)_EF_READ_B@23)

        Address: 7346
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_B, value)

    @property
    @locked
    def ain24_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_B (AIN#(0:149)_EF_READ_B@24)

        Address: 7348
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_B)  # type: ignore

    @ain24_ef_read_b.setter
    @locked
    def ain24_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_B (AIN#(0:149)_EF_READ_B@24)

        Address: 7348
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_B, value)

    @property
    @locked
    def ain25_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_B (AIN#(0:149)_EF_READ_B@25)

        Address: 7350
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_B)  # type: ignore

    @ain25_ef_read_b.setter
    @locked
    def ain25_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_B (AIN#(0:149)_EF_READ_B@25)

        Address: 7350
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_B, value)

    @property
    @locked
    def ain26_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_B (AIN#(0:149)_EF_READ_B@26)

        Address: 7352
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_B)  # type: ignore

    @ain26_ef_read_b.setter
    @locked
    def ain26_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_B (AIN#(0:149)_EF_READ_B@26)

        Address: 7352
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_B, value)

    @property
    @locked
    def ain27_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_B (AIN#(0:149)_EF_READ_B@27)

        Address: 7354
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_B)  # type: ignore

    @ain27_ef_read_b.setter
    @locked
    def ain27_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_B (AIN#(0:149)_EF_READ_B@27)

        Address: 7354
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_B, value)

    @property
    @locked
    def ain28_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_B (AIN#(0:149)_EF_READ_B@28)

        Address: 7356
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_B)  # type: ignore

    @ain28_ef_read_b.setter
    @locked
    def ain28_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_B (AIN#(0:149)_EF_READ_B@28)

        Address: 7356
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_B, value)

    @property
    @locked
    def ain29_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_B (AIN#(0:149)_EF_READ_B@29)

        Address: 7358
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_B)  # type: ignore

    @ain29_ef_read_b.setter
    @locked
    def ain29_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_B (AIN#(0:149)_EF_READ_B@29)

        Address: 7358
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_B, value)

    @property
    @locked
    def ain30_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_B (AIN#(0:149)_EF_READ_B@30)

        Address: 7360
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_B)  # type: ignore

    @ain30_ef_read_b.setter
    @locked
    def ain30_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_B (AIN#(0:149)_EF_READ_B@30)

        Address: 7360
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_B, value)

    @property
    @locked
    def ain31_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_B (AIN#(0:149)_EF_READ_B@31)

        Address: 7362
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_B)  # type: ignore

    @ain31_ef_read_b.setter
    @locked
    def ain31_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_B (AIN#(0:149)_EF_READ_B@31)

        Address: 7362
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_B, value)

    @property
    @locked
    def ain32_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_B (AIN#(0:149)_EF_READ_B@32)

        Address: 7364
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_B)  # type: ignore

    @ain32_ef_read_b.setter
    @locked
    def ain32_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_B (AIN#(0:149)_EF_READ_B@32)

        Address: 7364
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_B, value)

    @property
    @locked
    def ain33_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_B (AIN#(0:149)_EF_READ_B@33)

        Address: 7366
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_B)  # type: ignore

    @ain33_ef_read_b.setter
    @locked
    def ain33_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_B (AIN#(0:149)_EF_READ_B@33)

        Address: 7366
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_B, value)

    @property
    @locked
    def ain34_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_B (AIN#(0:149)_EF_READ_B@34)

        Address: 7368
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_B)  # type: ignore

    @ain34_ef_read_b.setter
    @locked
    def ain34_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_B (AIN#(0:149)_EF_READ_B@34)

        Address: 7368
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_B, value)

    @property
    @locked
    def ain35_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_B (AIN#(0:149)_EF_READ_B@35)

        Address: 7370
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_B)  # type: ignore

    @ain35_ef_read_b.setter
    @locked
    def ain35_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_B (AIN#(0:149)_EF_READ_B@35)

        Address: 7370
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_B, value)

    @property
    @locked
    def ain36_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_B (AIN#(0:149)_EF_READ_B@36)

        Address: 7372
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_B)  # type: ignore

    @ain36_ef_read_b.setter
    @locked
    def ain36_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_B (AIN#(0:149)_EF_READ_B@36)

        Address: 7372
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_B, value)

    @property
    @locked
    def ain37_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_B (AIN#(0:149)_EF_READ_B@37)

        Address: 7374
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_B)  # type: ignore

    @ain37_ef_read_b.setter
    @locked
    def ain37_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_B (AIN#(0:149)_EF_READ_B@37)

        Address: 7374
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_B, value)

    @property
    @locked
    def ain38_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_B (AIN#(0:149)_EF_READ_B@38)

        Address: 7376
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_B)  # type: ignore

    @ain38_ef_read_b.setter
    @locked
    def ain38_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_B (AIN#(0:149)_EF_READ_B@38)

        Address: 7376
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_B, value)

    @property
    @locked
    def ain39_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_B (AIN#(0:149)_EF_READ_B@39)

        Address: 7378
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_B)  # type: ignore

    @ain39_ef_read_b.setter
    @locked
    def ain39_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_B (AIN#(0:149)_EF_READ_B@39)

        Address: 7378
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_B, value)

    @property
    @locked
    def ain40_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_B (AIN#(0:149)_EF_READ_B@40)

        Address: 7380
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_B)  # type: ignore

    @ain40_ef_read_b.setter
    @locked
    def ain40_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_B (AIN#(0:149)_EF_READ_B@40)

        Address: 7380
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_B, value)

    @property
    @locked
    def ain41_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_B (AIN#(0:149)_EF_READ_B@41)

        Address: 7382
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_B)  # type: ignore

    @ain41_ef_read_b.setter
    @locked
    def ain41_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_B (AIN#(0:149)_EF_READ_B@41)

        Address: 7382
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_B, value)

    @property
    @locked
    def ain42_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_B (AIN#(0:149)_EF_READ_B@42)

        Address: 7384
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_B)  # type: ignore

    @ain42_ef_read_b.setter
    @locked
    def ain42_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_B (AIN#(0:149)_EF_READ_B@42)

        Address: 7384
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_B, value)

    @property
    @locked
    def ain43_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_B (AIN#(0:149)_EF_READ_B@43)

        Address: 7386
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_B)  # type: ignore

    @ain43_ef_read_b.setter
    @locked
    def ain43_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_B (AIN#(0:149)_EF_READ_B@43)

        Address: 7386
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_B, value)

    @property
    @locked
    def ain44_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_B (AIN#(0:149)_EF_READ_B@44)

        Address: 7388
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_B)  # type: ignore

    @ain44_ef_read_b.setter
    @locked
    def ain44_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_B (AIN#(0:149)_EF_READ_B@44)

        Address: 7388
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_B, value)

    @property
    @locked
    def ain45_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_B (AIN#(0:149)_EF_READ_B@45)

        Address: 7390
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_B)  # type: ignore

    @ain45_ef_read_b.setter
    @locked
    def ain45_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_B (AIN#(0:149)_EF_READ_B@45)

        Address: 7390
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_B, value)

    @property
    @locked
    def ain46_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_B (AIN#(0:149)_EF_READ_B@46)

        Address: 7392
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_B)  # type: ignore

    @ain46_ef_read_b.setter
    @locked
    def ain46_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_B (AIN#(0:149)_EF_READ_B@46)

        Address: 7392
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_B, value)

    @property
    @locked
    def ain47_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_B (AIN#(0:149)_EF_READ_B@47)

        Address: 7394
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_B)  # type: ignore

    @ain47_ef_read_b.setter
    @locked
    def ain47_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_B (AIN#(0:149)_EF_READ_B@47)

        Address: 7394
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_B, value)

    @property
    @locked
    def ain48_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_B (AIN#(0:149)_EF_READ_B@48)

        Address: 7396
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_B)  # type: ignore

    @ain48_ef_read_b.setter
    @locked
    def ain48_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_B (AIN#(0:149)_EF_READ_B@48)

        Address: 7396
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_B, value)

    @property
    @locked
    def ain49_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_B (AIN#(0:149)_EF_READ_B@49)

        Address: 7398
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_B)  # type: ignore

    @ain49_ef_read_b.setter
    @locked
    def ain49_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_B (AIN#(0:149)_EF_READ_B@49)

        Address: 7398
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_B, value)

    @property
    @locked
    def ain50_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_B (AIN#(0:149)_EF_READ_B@50)

        Address: 7400
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_B)  # type: ignore

    @ain50_ef_read_b.setter
    @locked
    def ain50_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_B (AIN#(0:149)_EF_READ_B@50)

        Address: 7400
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_B, value)

    @property
    @locked
    def ain51_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_B (AIN#(0:149)_EF_READ_B@51)

        Address: 7402
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_B)  # type: ignore

    @ain51_ef_read_b.setter
    @locked
    def ain51_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_B (AIN#(0:149)_EF_READ_B@51)

        Address: 7402
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_B, value)

    @property
    @locked
    def ain52_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_B (AIN#(0:149)_EF_READ_B@52)

        Address: 7404
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_B)  # type: ignore

    @ain52_ef_read_b.setter
    @locked
    def ain52_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_B (AIN#(0:149)_EF_READ_B@52)

        Address: 7404
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_B, value)

    @property
    @locked
    def ain53_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_B (AIN#(0:149)_EF_READ_B@53)

        Address: 7406
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_B)  # type: ignore

    @ain53_ef_read_b.setter
    @locked
    def ain53_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_B (AIN#(0:149)_EF_READ_B@53)

        Address: 7406
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_B, value)

    @property
    @locked
    def ain54_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_B (AIN#(0:149)_EF_READ_B@54)

        Address: 7408
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_B)  # type: ignore

    @ain54_ef_read_b.setter
    @locked
    def ain54_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_B (AIN#(0:149)_EF_READ_B@54)

        Address: 7408
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_B, value)

    @property
    @locked
    def ain55_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_B (AIN#(0:149)_EF_READ_B@55)

        Address: 7410
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_B)  # type: ignore

    @ain55_ef_read_b.setter
    @locked
    def ain55_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_B (AIN#(0:149)_EF_READ_B@55)

        Address: 7410
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_B, value)

    @property
    @locked
    def ain56_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_B (AIN#(0:149)_EF_READ_B@56)

        Address: 7412
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_B)  # type: ignore

    @ain56_ef_read_b.setter
    @locked
    def ain56_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_B (AIN#(0:149)_EF_READ_B@56)

        Address: 7412
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_B, value)

    @property
    @locked
    def ain57_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_B (AIN#(0:149)_EF_READ_B@57)

        Address: 7414
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_B)  # type: ignore

    @ain57_ef_read_b.setter
    @locked
    def ain57_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_B (AIN#(0:149)_EF_READ_B@57)

        Address: 7414
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_B, value)

    @property
    @locked
    def ain58_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_B (AIN#(0:149)_EF_READ_B@58)

        Address: 7416
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_B)  # type: ignore

    @ain58_ef_read_b.setter
    @locked
    def ain58_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_B (AIN#(0:149)_EF_READ_B@58)

        Address: 7416
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_B, value)

    @property
    @locked
    def ain59_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_B (AIN#(0:149)_EF_READ_B@59)

        Address: 7418
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_B)  # type: ignore

    @ain59_ef_read_b.setter
    @locked
    def ain59_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_B (AIN#(0:149)_EF_READ_B@59)

        Address: 7418
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_B, value)

    @property
    @locked
    def ain60_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_B (AIN#(0:149)_EF_READ_B@60)

        Address: 7420
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_B)  # type: ignore

    @ain60_ef_read_b.setter
    @locked
    def ain60_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_B (AIN#(0:149)_EF_READ_B@60)

        Address: 7420
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_B, value)

    @property
    @locked
    def ain61_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_B (AIN#(0:149)_EF_READ_B@61)

        Address: 7422
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_B)  # type: ignore

    @ain61_ef_read_b.setter
    @locked
    def ain61_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_B (AIN#(0:149)_EF_READ_B@61)

        Address: 7422
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_B, value)

    @property
    @locked
    def ain62_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_B (AIN#(0:149)_EF_READ_B@62)

        Address: 7424
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_B)  # type: ignore

    @ain62_ef_read_b.setter
    @locked
    def ain62_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_B (AIN#(0:149)_EF_READ_B@62)

        Address: 7424
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_B, value)

    @property
    @locked
    def ain63_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_B (AIN#(0:149)_EF_READ_B@63)

        Address: 7426
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_B)  # type: ignore

    @ain63_ef_read_b.setter
    @locked
    def ain63_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_B (AIN#(0:149)_EF_READ_B@63)

        Address: 7426
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_B, value)

    @property
    @locked
    def ain64_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_B (AIN#(0:149)_EF_READ_B@64)

        Address: 7428
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_B)  # type: ignore

    @ain64_ef_read_b.setter
    @locked
    def ain64_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_B (AIN#(0:149)_EF_READ_B@64)

        Address: 7428
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_B, value)

    @property
    @locked
    def ain65_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_B (AIN#(0:149)_EF_READ_B@65)

        Address: 7430
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_B)  # type: ignore

    @ain65_ef_read_b.setter
    @locked
    def ain65_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_B (AIN#(0:149)_EF_READ_B@65)

        Address: 7430
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_B, value)

    @property
    @locked
    def ain66_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_B (AIN#(0:149)_EF_READ_B@66)

        Address: 7432
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_B)  # type: ignore

    @ain66_ef_read_b.setter
    @locked
    def ain66_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_B (AIN#(0:149)_EF_READ_B@66)

        Address: 7432
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_B, value)

    @property
    @locked
    def ain67_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_B (AIN#(0:149)_EF_READ_B@67)

        Address: 7434
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_B)  # type: ignore

    @ain67_ef_read_b.setter
    @locked
    def ain67_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_B (AIN#(0:149)_EF_READ_B@67)

        Address: 7434
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_B, value)

    @property
    @locked
    def ain68_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_B (AIN#(0:149)_EF_READ_B@68)

        Address: 7436
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_B)  # type: ignore

    @ain68_ef_read_b.setter
    @locked
    def ain68_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_B (AIN#(0:149)_EF_READ_B@68)

        Address: 7436
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_B, value)

    @property
    @locked
    def ain69_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_B (AIN#(0:149)_EF_READ_B@69)

        Address: 7438
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_B)  # type: ignore

    @ain69_ef_read_b.setter
    @locked
    def ain69_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_B (AIN#(0:149)_EF_READ_B@69)

        Address: 7438
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_B, value)

    @property
    @locked
    def ain70_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_B (AIN#(0:149)_EF_READ_B@70)

        Address: 7440
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_B)  # type: ignore

    @ain70_ef_read_b.setter
    @locked
    def ain70_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_B (AIN#(0:149)_EF_READ_B@70)

        Address: 7440
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_B, value)

    @property
    @locked
    def ain71_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_B (AIN#(0:149)_EF_READ_B@71)

        Address: 7442
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_B)  # type: ignore

    @ain71_ef_read_b.setter
    @locked
    def ain71_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_B (AIN#(0:149)_EF_READ_B@71)

        Address: 7442
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_B, value)

    @property
    @locked
    def ain72_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_B (AIN#(0:149)_EF_READ_B@72)

        Address: 7444
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_B)  # type: ignore

    @ain72_ef_read_b.setter
    @locked
    def ain72_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_B (AIN#(0:149)_EF_READ_B@72)

        Address: 7444
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_B, value)

    @property
    @locked
    def ain73_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_B (AIN#(0:149)_EF_READ_B@73)

        Address: 7446
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_B)  # type: ignore

    @ain73_ef_read_b.setter
    @locked
    def ain73_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_B (AIN#(0:149)_EF_READ_B@73)

        Address: 7446
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_B, value)

    @property
    @locked
    def ain74_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_B (AIN#(0:149)_EF_READ_B@74)

        Address: 7448
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_B)  # type: ignore

    @ain74_ef_read_b.setter
    @locked
    def ain74_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_B (AIN#(0:149)_EF_READ_B@74)

        Address: 7448
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_B, value)

    @property
    @locked
    def ain75_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_B (AIN#(0:149)_EF_READ_B@75)

        Address: 7450
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_B)  # type: ignore

    @ain75_ef_read_b.setter
    @locked
    def ain75_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_B (AIN#(0:149)_EF_READ_B@75)

        Address: 7450
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_B, value)

    @property
    @locked
    def ain76_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_B (AIN#(0:149)_EF_READ_B@76)

        Address: 7452
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_B)  # type: ignore

    @ain76_ef_read_b.setter
    @locked
    def ain76_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_B (AIN#(0:149)_EF_READ_B@76)

        Address: 7452
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_B, value)

    @property
    @locked
    def ain77_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_B (AIN#(0:149)_EF_READ_B@77)

        Address: 7454
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_B)  # type: ignore

    @ain77_ef_read_b.setter
    @locked
    def ain77_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_B (AIN#(0:149)_EF_READ_B@77)

        Address: 7454
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_B, value)

    @property
    @locked
    def ain78_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_B (AIN#(0:149)_EF_READ_B@78)

        Address: 7456
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_B)  # type: ignore

    @ain78_ef_read_b.setter
    @locked
    def ain78_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_B (AIN#(0:149)_EF_READ_B@78)

        Address: 7456
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_B, value)

    @property
    @locked
    def ain79_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_B (AIN#(0:149)_EF_READ_B@79)

        Address: 7458
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_B)  # type: ignore

    @ain79_ef_read_b.setter
    @locked
    def ain79_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_B (AIN#(0:149)_EF_READ_B@79)

        Address: 7458
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_B, value)

    @property
    @locked
    def ain80_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_B (AIN#(0:149)_EF_READ_B@80)

        Address: 7460
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_B)  # type: ignore

    @ain80_ef_read_b.setter
    @locked
    def ain80_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_B (AIN#(0:149)_EF_READ_B@80)

        Address: 7460
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_B, value)

    @property
    @locked
    def ain81_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_B (AIN#(0:149)_EF_READ_B@81)

        Address: 7462
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_B)  # type: ignore

    @ain81_ef_read_b.setter
    @locked
    def ain81_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_B (AIN#(0:149)_EF_READ_B@81)

        Address: 7462
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_B, value)

    @property
    @locked
    def ain82_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_B (AIN#(0:149)_EF_READ_B@82)

        Address: 7464
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_B)  # type: ignore

    @ain82_ef_read_b.setter
    @locked
    def ain82_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_B (AIN#(0:149)_EF_READ_B@82)

        Address: 7464
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_B, value)

    @property
    @locked
    def ain83_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_B (AIN#(0:149)_EF_READ_B@83)

        Address: 7466
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_B)  # type: ignore

    @ain83_ef_read_b.setter
    @locked
    def ain83_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_B (AIN#(0:149)_EF_READ_B@83)

        Address: 7466
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_B, value)

    @property
    @locked
    def ain84_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_B (AIN#(0:149)_EF_READ_B@84)

        Address: 7468
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_B)  # type: ignore

    @ain84_ef_read_b.setter
    @locked
    def ain84_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_B (AIN#(0:149)_EF_READ_B@84)

        Address: 7468
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_B, value)

    @property
    @locked
    def ain85_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_B (AIN#(0:149)_EF_READ_B@85)

        Address: 7470
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_B)  # type: ignore

    @ain85_ef_read_b.setter
    @locked
    def ain85_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_B (AIN#(0:149)_EF_READ_B@85)

        Address: 7470
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_B, value)

    @property
    @locked
    def ain86_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_B (AIN#(0:149)_EF_READ_B@86)

        Address: 7472
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_B)  # type: ignore

    @ain86_ef_read_b.setter
    @locked
    def ain86_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_B (AIN#(0:149)_EF_READ_B@86)

        Address: 7472
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_B, value)

    @property
    @locked
    def ain87_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_B (AIN#(0:149)_EF_READ_B@87)

        Address: 7474
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_B)  # type: ignore

    @ain87_ef_read_b.setter
    @locked
    def ain87_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_B (AIN#(0:149)_EF_READ_B@87)

        Address: 7474
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_B, value)

    @property
    @locked
    def ain88_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_B (AIN#(0:149)_EF_READ_B@88)

        Address: 7476
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_B)  # type: ignore

    @ain88_ef_read_b.setter
    @locked
    def ain88_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_B (AIN#(0:149)_EF_READ_B@88)

        Address: 7476
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_B, value)

    @property
    @locked
    def ain89_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_B (AIN#(0:149)_EF_READ_B@89)

        Address: 7478
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_B)  # type: ignore

    @ain89_ef_read_b.setter
    @locked
    def ain89_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_B (AIN#(0:149)_EF_READ_B@89)

        Address: 7478
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_B, value)

    @property
    @locked
    def ain90_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_B (AIN#(0:149)_EF_READ_B@90)

        Address: 7480
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_B)  # type: ignore

    @ain90_ef_read_b.setter
    @locked
    def ain90_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_B (AIN#(0:149)_EF_READ_B@90)

        Address: 7480
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_B, value)

    @property
    @locked
    def ain91_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_B (AIN#(0:149)_EF_READ_B@91)

        Address: 7482
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_B)  # type: ignore

    @ain91_ef_read_b.setter
    @locked
    def ain91_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_B (AIN#(0:149)_EF_READ_B@91)

        Address: 7482
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_B, value)

    @property
    @locked
    def ain92_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_B (AIN#(0:149)_EF_READ_B@92)

        Address: 7484
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_B)  # type: ignore

    @ain92_ef_read_b.setter
    @locked
    def ain92_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_B (AIN#(0:149)_EF_READ_B@92)

        Address: 7484
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_B, value)

    @property
    @locked
    def ain93_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_B (AIN#(0:149)_EF_READ_B@93)

        Address: 7486
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_B)  # type: ignore

    @ain93_ef_read_b.setter
    @locked
    def ain93_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_B (AIN#(0:149)_EF_READ_B@93)

        Address: 7486
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_B, value)

    @property
    @locked
    def ain94_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_B (AIN#(0:149)_EF_READ_B@94)

        Address: 7488
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_B)  # type: ignore

    @ain94_ef_read_b.setter
    @locked
    def ain94_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_B (AIN#(0:149)_EF_READ_B@94)

        Address: 7488
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_B, value)

    @property
    @locked
    def ain95_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_B (AIN#(0:149)_EF_READ_B@95)

        Address: 7490
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_B)  # type: ignore

    @ain95_ef_read_b.setter
    @locked
    def ain95_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_B (AIN#(0:149)_EF_READ_B@95)

        Address: 7490
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_B, value)

    @property
    @locked
    def ain96_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_B (AIN#(0:149)_EF_READ_B@96)

        Address: 7492
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_B)  # type: ignore

    @ain96_ef_read_b.setter
    @locked
    def ain96_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_B (AIN#(0:149)_EF_READ_B@96)

        Address: 7492
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_B, value)

    @property
    @locked
    def ain97_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_B (AIN#(0:149)_EF_READ_B@97)

        Address: 7494
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_B)  # type: ignore

    @ain97_ef_read_b.setter
    @locked
    def ain97_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_B (AIN#(0:149)_EF_READ_B@97)

        Address: 7494
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_B, value)

    @property
    @locked
    def ain98_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_B (AIN#(0:149)_EF_READ_B@98)

        Address: 7496
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_B)  # type: ignore

    @ain98_ef_read_b.setter
    @locked
    def ain98_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_B (AIN#(0:149)_EF_READ_B@98)

        Address: 7496
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_B, value)

    @property
    @locked
    def ain99_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_B (AIN#(0:149)_EF_READ_B@99)

        Address: 7498
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_B)  # type: ignore

    @ain99_ef_read_b.setter
    @locked
    def ain99_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_B (AIN#(0:149)_EF_READ_B@99)

        Address: 7498
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_B, value)

    @property
    @locked
    def ain100_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_B (AIN#(0:149)_EF_READ_B@100)

        Address: 7500
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_B)  # type: ignore

    @ain100_ef_read_b.setter
    @locked
    def ain100_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_B (AIN#(0:149)_EF_READ_B@100)

        Address: 7500
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_B, value)

    @property
    @locked
    def ain101_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_B (AIN#(0:149)_EF_READ_B@101)

        Address: 7502
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_B)  # type: ignore

    @ain101_ef_read_b.setter
    @locked
    def ain101_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_B (AIN#(0:149)_EF_READ_B@101)

        Address: 7502
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_B, value)

    @property
    @locked
    def ain102_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_B (AIN#(0:149)_EF_READ_B@102)

        Address: 7504
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_B)  # type: ignore

    @ain102_ef_read_b.setter
    @locked
    def ain102_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_B (AIN#(0:149)_EF_READ_B@102)

        Address: 7504
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_B, value)

    @property
    @locked
    def ain103_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_B (AIN#(0:149)_EF_READ_B@103)

        Address: 7506
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_B)  # type: ignore

    @ain103_ef_read_b.setter
    @locked
    def ain103_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_B (AIN#(0:149)_EF_READ_B@103)

        Address: 7506
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_B, value)

    @property
    @locked
    def ain104_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_B (AIN#(0:149)_EF_READ_B@104)

        Address: 7508
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_B)  # type: ignore

    @ain104_ef_read_b.setter
    @locked
    def ain104_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_B (AIN#(0:149)_EF_READ_B@104)

        Address: 7508
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_B, value)

    @property
    @locked
    def ain105_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_B (AIN#(0:149)_EF_READ_B@105)

        Address: 7510
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_B)  # type: ignore

    @ain105_ef_read_b.setter
    @locked
    def ain105_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_B (AIN#(0:149)_EF_READ_B@105)

        Address: 7510
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_B, value)

    @property
    @locked
    def ain106_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_B (AIN#(0:149)_EF_READ_B@106)

        Address: 7512
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_B)  # type: ignore

    @ain106_ef_read_b.setter
    @locked
    def ain106_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_B (AIN#(0:149)_EF_READ_B@106)

        Address: 7512
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_B, value)

    @property
    @locked
    def ain107_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_B (AIN#(0:149)_EF_READ_B@107)

        Address: 7514
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_B)  # type: ignore

    @ain107_ef_read_b.setter
    @locked
    def ain107_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_B (AIN#(0:149)_EF_READ_B@107)

        Address: 7514
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_B, value)

    @property
    @locked
    def ain108_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_B (AIN#(0:149)_EF_READ_B@108)

        Address: 7516
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_B)  # type: ignore

    @ain108_ef_read_b.setter
    @locked
    def ain108_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_B (AIN#(0:149)_EF_READ_B@108)

        Address: 7516
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_B, value)

    @property
    @locked
    def ain109_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_B (AIN#(0:149)_EF_READ_B@109)

        Address: 7518
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_B)  # type: ignore

    @ain109_ef_read_b.setter
    @locked
    def ain109_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_B (AIN#(0:149)_EF_READ_B@109)

        Address: 7518
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_B, value)

    @property
    @locked
    def ain110_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_B (AIN#(0:149)_EF_READ_B@110)

        Address: 7520
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_B)  # type: ignore

    @ain110_ef_read_b.setter
    @locked
    def ain110_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_B (AIN#(0:149)_EF_READ_B@110)

        Address: 7520
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_B, value)

    @property
    @locked
    def ain111_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_B (AIN#(0:149)_EF_READ_B@111)

        Address: 7522
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_B)  # type: ignore

    @ain111_ef_read_b.setter
    @locked
    def ain111_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_B (AIN#(0:149)_EF_READ_B@111)

        Address: 7522
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_B, value)

    @property
    @locked
    def ain112_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_B (AIN#(0:149)_EF_READ_B@112)

        Address: 7524
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_B)  # type: ignore

    @ain112_ef_read_b.setter
    @locked
    def ain112_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_B (AIN#(0:149)_EF_READ_B@112)

        Address: 7524
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_B, value)

    @property
    @locked
    def ain113_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_B (AIN#(0:149)_EF_READ_B@113)

        Address: 7526
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_B)  # type: ignore

    @ain113_ef_read_b.setter
    @locked
    def ain113_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_B (AIN#(0:149)_EF_READ_B@113)

        Address: 7526
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_B, value)

    @property
    @locked
    def ain114_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_B (AIN#(0:149)_EF_READ_B@114)

        Address: 7528
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_B)  # type: ignore

    @ain114_ef_read_b.setter
    @locked
    def ain114_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_B (AIN#(0:149)_EF_READ_B@114)

        Address: 7528
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_B, value)

    @property
    @locked
    def ain115_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_B (AIN#(0:149)_EF_READ_B@115)

        Address: 7530
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_B)  # type: ignore

    @ain115_ef_read_b.setter
    @locked
    def ain115_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_B (AIN#(0:149)_EF_READ_B@115)

        Address: 7530
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_B, value)

    @property
    @locked
    def ain116_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_B (AIN#(0:149)_EF_READ_B@116)

        Address: 7532
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_B)  # type: ignore

    @ain116_ef_read_b.setter
    @locked
    def ain116_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_B (AIN#(0:149)_EF_READ_B@116)

        Address: 7532
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_B, value)

    @property
    @locked
    def ain117_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_B (AIN#(0:149)_EF_READ_B@117)

        Address: 7534
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_B)  # type: ignore

    @ain117_ef_read_b.setter
    @locked
    def ain117_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_B (AIN#(0:149)_EF_READ_B@117)

        Address: 7534
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_B, value)

    @property
    @locked
    def ain118_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_B (AIN#(0:149)_EF_READ_B@118)

        Address: 7536
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_B)  # type: ignore

    @ain118_ef_read_b.setter
    @locked
    def ain118_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_B (AIN#(0:149)_EF_READ_B@118)

        Address: 7536
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_B, value)

    @property
    @locked
    def ain119_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_B (AIN#(0:149)_EF_READ_B@119)

        Address: 7538
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_B)  # type: ignore

    @ain119_ef_read_b.setter
    @locked
    def ain119_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_B (AIN#(0:149)_EF_READ_B@119)

        Address: 7538
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_B, value)

    @property
    @locked
    def ain120_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_B (AIN#(0:149)_EF_READ_B@120)

        Address: 7540
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_B)  # type: ignore

    @ain120_ef_read_b.setter
    @locked
    def ain120_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_B (AIN#(0:149)_EF_READ_B@120)

        Address: 7540
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_B, value)

    @property
    @locked
    def ain121_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_B (AIN#(0:149)_EF_READ_B@121)

        Address: 7542
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_B)  # type: ignore

    @ain121_ef_read_b.setter
    @locked
    def ain121_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_B (AIN#(0:149)_EF_READ_B@121)

        Address: 7542
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_B, value)

    @property
    @locked
    def ain122_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_B (AIN#(0:149)_EF_READ_B@122)

        Address: 7544
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_B)  # type: ignore

    @ain122_ef_read_b.setter
    @locked
    def ain122_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_B (AIN#(0:149)_EF_READ_B@122)

        Address: 7544
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_B, value)

    @property
    @locked
    def ain123_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_B (AIN#(0:149)_EF_READ_B@123)

        Address: 7546
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_B)  # type: ignore

    @ain123_ef_read_b.setter
    @locked
    def ain123_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_B (AIN#(0:149)_EF_READ_B@123)

        Address: 7546
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_B, value)

    @property
    @locked
    def ain124_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_B (AIN#(0:149)_EF_READ_B@124)

        Address: 7548
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_B)  # type: ignore

    @ain124_ef_read_b.setter
    @locked
    def ain124_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_B (AIN#(0:149)_EF_READ_B@124)

        Address: 7548
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_B, value)

    @property
    @locked
    def ain125_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_B (AIN#(0:149)_EF_READ_B@125)

        Address: 7550
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_B)  # type: ignore

    @ain125_ef_read_b.setter
    @locked
    def ain125_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_B (AIN#(0:149)_EF_READ_B@125)

        Address: 7550
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_B, value)

    @property
    @locked
    def ain126_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_B (AIN#(0:149)_EF_READ_B@126)

        Address: 7552
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_B)  # type: ignore

    @ain126_ef_read_b.setter
    @locked
    def ain126_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_B (AIN#(0:149)_EF_READ_B@126)

        Address: 7552
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_B, value)

    @property
    @locked
    def ain127_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_B (AIN#(0:149)_EF_READ_B@127)

        Address: 7554
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_B)  # type: ignore

    @ain127_ef_read_b.setter
    @locked
    def ain127_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_B (AIN#(0:149)_EF_READ_B@127)

        Address: 7554
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_B, value)

    @property
    @locked
    def ain128_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_B (AIN#(0:149)_EF_READ_B@128)

        Address: 7556
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_B)  # type: ignore

    @ain128_ef_read_b.setter
    @locked
    def ain128_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_B (AIN#(0:149)_EF_READ_B@128)

        Address: 7556
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_B, value)

    @property
    @locked
    def ain129_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_B (AIN#(0:149)_EF_READ_B@129)

        Address: 7558
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_B)  # type: ignore

    @ain129_ef_read_b.setter
    @locked
    def ain129_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_B (AIN#(0:149)_EF_READ_B@129)

        Address: 7558
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_B, value)

    @property
    @locked
    def ain130_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_B (AIN#(0:149)_EF_READ_B@130)

        Address: 7560
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_B)  # type: ignore

    @ain130_ef_read_b.setter
    @locked
    def ain130_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_B (AIN#(0:149)_EF_READ_B@130)

        Address: 7560
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_B, value)

    @property
    @locked
    def ain131_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_B (AIN#(0:149)_EF_READ_B@131)

        Address: 7562
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_B)  # type: ignore

    @ain131_ef_read_b.setter
    @locked
    def ain131_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_B (AIN#(0:149)_EF_READ_B@131)

        Address: 7562
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_B, value)

    @property
    @locked
    def ain132_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_B (AIN#(0:149)_EF_READ_B@132)

        Address: 7564
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_B)  # type: ignore

    @ain132_ef_read_b.setter
    @locked
    def ain132_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_B (AIN#(0:149)_EF_READ_B@132)

        Address: 7564
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_B, value)

    @property
    @locked
    def ain133_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_B (AIN#(0:149)_EF_READ_B@133)

        Address: 7566
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_B)  # type: ignore

    @ain133_ef_read_b.setter
    @locked
    def ain133_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_B (AIN#(0:149)_EF_READ_B@133)

        Address: 7566
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_B, value)

    @property
    @locked
    def ain134_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_B (AIN#(0:149)_EF_READ_B@134)

        Address: 7568
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_B)  # type: ignore

    @ain134_ef_read_b.setter
    @locked
    def ain134_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_B (AIN#(0:149)_EF_READ_B@134)

        Address: 7568
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_B, value)

    @property
    @locked
    def ain135_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_B (AIN#(0:149)_EF_READ_B@135)

        Address: 7570
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_B)  # type: ignore

    @ain135_ef_read_b.setter
    @locked
    def ain135_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_B (AIN#(0:149)_EF_READ_B@135)

        Address: 7570
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_B, value)

    @property
    @locked
    def ain136_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_B (AIN#(0:149)_EF_READ_B@136)

        Address: 7572
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_B)  # type: ignore

    @ain136_ef_read_b.setter
    @locked
    def ain136_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_B (AIN#(0:149)_EF_READ_B@136)

        Address: 7572
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_B, value)

    @property
    @locked
    def ain137_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_B (AIN#(0:149)_EF_READ_B@137)

        Address: 7574
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_B)  # type: ignore

    @ain137_ef_read_b.setter
    @locked
    def ain137_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_B (AIN#(0:149)_EF_READ_B@137)

        Address: 7574
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_B, value)

    @property
    @locked
    def ain138_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_B (AIN#(0:149)_EF_READ_B@138)

        Address: 7576
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_B)  # type: ignore

    @ain138_ef_read_b.setter
    @locked
    def ain138_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_B (AIN#(0:149)_EF_READ_B@138)

        Address: 7576
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_B, value)

    @property
    @locked
    def ain139_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_B (AIN#(0:149)_EF_READ_B@139)

        Address: 7578
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_B)  # type: ignore

    @ain139_ef_read_b.setter
    @locked
    def ain139_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_B (AIN#(0:149)_EF_READ_B@139)

        Address: 7578
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_B, value)

    @property
    @locked
    def ain140_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_B (AIN#(0:149)_EF_READ_B@140)

        Address: 7580
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_B)  # type: ignore

    @ain140_ef_read_b.setter
    @locked
    def ain140_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_B (AIN#(0:149)_EF_READ_B@140)

        Address: 7580
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_B, value)

    @property
    @locked
    def ain141_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_B (AIN#(0:149)_EF_READ_B@141)

        Address: 7582
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_B)  # type: ignore

    @ain141_ef_read_b.setter
    @locked
    def ain141_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_B (AIN#(0:149)_EF_READ_B@141)

        Address: 7582
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_B, value)

    @property
    @locked
    def ain142_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_B (AIN#(0:149)_EF_READ_B@142)

        Address: 7584
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_B)  # type: ignore

    @ain142_ef_read_b.setter
    @locked
    def ain142_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_B (AIN#(0:149)_EF_READ_B@142)

        Address: 7584
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_B, value)

    @property
    @locked
    def ain143_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_B (AIN#(0:149)_EF_READ_B@143)

        Address: 7586
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_B)  # type: ignore

    @ain143_ef_read_b.setter
    @locked
    def ain143_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_B (AIN#(0:149)_EF_READ_B@143)

        Address: 7586
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_B, value)

    @property
    @locked
    def ain144_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_B (AIN#(0:149)_EF_READ_B@144)

        Address: 7588
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_B)  # type: ignore

    @ain144_ef_read_b.setter
    @locked
    def ain144_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_B (AIN#(0:149)_EF_READ_B@144)

        Address: 7588
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_B, value)

    @property
    @locked
    def ain145_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_B (AIN#(0:149)_EF_READ_B@145)

        Address: 7590
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_B)  # type: ignore

    @ain145_ef_read_b.setter
    @locked
    def ain145_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_B (AIN#(0:149)_EF_READ_B@145)

        Address: 7590
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_B, value)

    @property
    @locked
    def ain146_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_B (AIN#(0:149)_EF_READ_B@146)

        Address: 7592
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_B)  # type: ignore

    @ain146_ef_read_b.setter
    @locked
    def ain146_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_B (AIN#(0:149)_EF_READ_B@146)

        Address: 7592
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_B, value)

    @property
    @locked
    def ain147_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_B (AIN#(0:149)_EF_READ_B@147)

        Address: 7594
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_B)  # type: ignore

    @ain147_ef_read_b.setter
    @locked
    def ain147_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_B (AIN#(0:149)_EF_READ_B@147)

        Address: 7594
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_B, value)

    @property
    @locked
    def ain148_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_B (AIN#(0:149)_EF_READ_B@148)

        Address: 7596
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_B)  # type: ignore

    @ain148_ef_read_b.setter
    @locked
    def ain148_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_B (AIN#(0:149)_EF_READ_B@148)

        Address: 7596
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_B, value)

    @property
    @locked
    def ain149_ef_read_b(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_B (AIN#(0:149)_EF_READ_B@149)

        Address: 7598
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_B)  # type: ignore

    @ain149_ef_read_b.setter
    @locked
    def ain149_ef_read_b(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_B (AIN#(0:149)_EF_READ_B@149)

        Address: 7598
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_B, value)

    @property
    @locked
    def ain0_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_C (AIN#(0:149)_EF_READ_C@0)

        Address: 7600
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_C)  # type: ignore

    @ain0_ef_read_c.setter
    @locked
    def ain0_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_C (AIN#(0:149)_EF_READ_C@0)

        Address: 7600
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_C, value)

    @property
    @locked
    def ain1_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_C (AIN#(0:149)_EF_READ_C@1)

        Address: 7602
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_C)  # type: ignore

    @ain1_ef_read_c.setter
    @locked
    def ain1_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_C (AIN#(0:149)_EF_READ_C@1)

        Address: 7602
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_C, value)

    @property
    @locked
    def ain2_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_C (AIN#(0:149)_EF_READ_C@2)

        Address: 7604
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_C)  # type: ignore

    @ain2_ef_read_c.setter
    @locked
    def ain2_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_C (AIN#(0:149)_EF_READ_C@2)

        Address: 7604
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_C, value)

    @property
    @locked
    def ain3_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_C (AIN#(0:149)_EF_READ_C@3)

        Address: 7606
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_C)  # type: ignore

    @ain3_ef_read_c.setter
    @locked
    def ain3_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_C (AIN#(0:149)_EF_READ_C@3)

        Address: 7606
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_C, value)

    @property
    @locked
    def ain4_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_C (AIN#(0:149)_EF_READ_C@4)

        Address: 7608
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_C)  # type: ignore

    @ain4_ef_read_c.setter
    @locked
    def ain4_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_C (AIN#(0:149)_EF_READ_C@4)

        Address: 7608
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_C, value)

    @property
    @locked
    def ain5_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_C (AIN#(0:149)_EF_READ_C@5)

        Address: 7610
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_C)  # type: ignore

    @ain5_ef_read_c.setter
    @locked
    def ain5_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_C (AIN#(0:149)_EF_READ_C@5)

        Address: 7610
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_C, value)

    @property
    @locked
    def ain6_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_C (AIN#(0:149)_EF_READ_C@6)

        Address: 7612
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_C)  # type: ignore

    @ain6_ef_read_c.setter
    @locked
    def ain6_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_C (AIN#(0:149)_EF_READ_C@6)

        Address: 7612
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_C, value)

    @property
    @locked
    def ain7_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_C (AIN#(0:149)_EF_READ_C@7)

        Address: 7614
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_C)  # type: ignore

    @ain7_ef_read_c.setter
    @locked
    def ain7_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_C (AIN#(0:149)_EF_READ_C@7)

        Address: 7614
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_C, value)

    @property
    @locked
    def ain8_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_C (AIN#(0:149)_EF_READ_C@8)

        Address: 7616
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_C)  # type: ignore

    @ain8_ef_read_c.setter
    @locked
    def ain8_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_C (AIN#(0:149)_EF_READ_C@8)

        Address: 7616
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_C, value)

    @property
    @locked
    def ain9_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_C (AIN#(0:149)_EF_READ_C@9)

        Address: 7618
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_C)  # type: ignore

    @ain9_ef_read_c.setter
    @locked
    def ain9_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_C (AIN#(0:149)_EF_READ_C@9)

        Address: 7618
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_C, value)

    @property
    @locked
    def ain10_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_C (AIN#(0:149)_EF_READ_C@10)

        Address: 7620
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_C)  # type: ignore

    @ain10_ef_read_c.setter
    @locked
    def ain10_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_C (AIN#(0:149)_EF_READ_C@10)

        Address: 7620
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_C, value)

    @property
    @locked
    def ain11_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_C (AIN#(0:149)_EF_READ_C@11)

        Address: 7622
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_C)  # type: ignore

    @ain11_ef_read_c.setter
    @locked
    def ain11_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_C (AIN#(0:149)_EF_READ_C@11)

        Address: 7622
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_C, value)

    @property
    @locked
    def ain12_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_C (AIN#(0:149)_EF_READ_C@12)

        Address: 7624
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_C)  # type: ignore

    @ain12_ef_read_c.setter
    @locked
    def ain12_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_C (AIN#(0:149)_EF_READ_C@12)

        Address: 7624
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_C, value)

    @property
    @locked
    def ain13_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_C (AIN#(0:149)_EF_READ_C@13)

        Address: 7626
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_C)  # type: ignore

    @ain13_ef_read_c.setter
    @locked
    def ain13_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_C (AIN#(0:149)_EF_READ_C@13)

        Address: 7626
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_C, value)

    @property
    @locked
    def ain14_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_C (AIN#(0:149)_EF_READ_C@14)

        Address: 7628
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_C)  # type: ignore

    @ain14_ef_read_c.setter
    @locked
    def ain14_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_C (AIN#(0:149)_EF_READ_C@14)

        Address: 7628
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_C, value)

    @property
    @locked
    def ain15_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_C (AIN#(0:149)_EF_READ_C@15)

        Address: 7630
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_C)  # type: ignore

    @ain15_ef_read_c.setter
    @locked
    def ain15_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_C (AIN#(0:149)_EF_READ_C@15)

        Address: 7630
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_C, value)

    @property
    @locked
    def ain16_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_C (AIN#(0:149)_EF_READ_C@16)

        Address: 7632
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_C)  # type: ignore

    @ain16_ef_read_c.setter
    @locked
    def ain16_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_C (AIN#(0:149)_EF_READ_C@16)

        Address: 7632
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_C, value)

    @property
    @locked
    def ain17_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_C (AIN#(0:149)_EF_READ_C@17)

        Address: 7634
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_C)  # type: ignore

    @ain17_ef_read_c.setter
    @locked
    def ain17_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_C (AIN#(0:149)_EF_READ_C@17)

        Address: 7634
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_C, value)

    @property
    @locked
    def ain18_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_C (AIN#(0:149)_EF_READ_C@18)

        Address: 7636
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_C)  # type: ignore

    @ain18_ef_read_c.setter
    @locked
    def ain18_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_C (AIN#(0:149)_EF_READ_C@18)

        Address: 7636
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_C, value)

    @property
    @locked
    def ain19_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_C (AIN#(0:149)_EF_READ_C@19)

        Address: 7638
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_C)  # type: ignore

    @ain19_ef_read_c.setter
    @locked
    def ain19_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_C (AIN#(0:149)_EF_READ_C@19)

        Address: 7638
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_C, value)

    @property
    @locked
    def ain20_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_C (AIN#(0:149)_EF_READ_C@20)

        Address: 7640
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_C)  # type: ignore

    @ain20_ef_read_c.setter
    @locked
    def ain20_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_C (AIN#(0:149)_EF_READ_C@20)

        Address: 7640
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_C, value)

    @property
    @locked
    def ain21_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_C (AIN#(0:149)_EF_READ_C@21)

        Address: 7642
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_C)  # type: ignore

    @ain21_ef_read_c.setter
    @locked
    def ain21_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_C (AIN#(0:149)_EF_READ_C@21)

        Address: 7642
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_C, value)

    @property
    @locked
    def ain22_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_C (AIN#(0:149)_EF_READ_C@22)

        Address: 7644
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_C)  # type: ignore

    @ain22_ef_read_c.setter
    @locked
    def ain22_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_C (AIN#(0:149)_EF_READ_C@22)

        Address: 7644
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_C, value)

    @property
    @locked
    def ain23_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_C (AIN#(0:149)_EF_READ_C@23)

        Address: 7646
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_C)  # type: ignore

    @ain23_ef_read_c.setter
    @locked
    def ain23_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_C (AIN#(0:149)_EF_READ_C@23)

        Address: 7646
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_C, value)

    @property
    @locked
    def ain24_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_C (AIN#(0:149)_EF_READ_C@24)

        Address: 7648
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_C)  # type: ignore

    @ain24_ef_read_c.setter
    @locked
    def ain24_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_C (AIN#(0:149)_EF_READ_C@24)

        Address: 7648
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_C, value)

    @property
    @locked
    def ain25_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_C (AIN#(0:149)_EF_READ_C@25)

        Address: 7650
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_C)  # type: ignore

    @ain25_ef_read_c.setter
    @locked
    def ain25_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_C (AIN#(0:149)_EF_READ_C@25)

        Address: 7650
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_C, value)

    @property
    @locked
    def ain26_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_C (AIN#(0:149)_EF_READ_C@26)

        Address: 7652
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_C)  # type: ignore

    @ain26_ef_read_c.setter
    @locked
    def ain26_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_C (AIN#(0:149)_EF_READ_C@26)

        Address: 7652
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_C, value)

    @property
    @locked
    def ain27_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_C (AIN#(0:149)_EF_READ_C@27)

        Address: 7654
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_C)  # type: ignore

    @ain27_ef_read_c.setter
    @locked
    def ain27_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_C (AIN#(0:149)_EF_READ_C@27)

        Address: 7654
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_C, value)

    @property
    @locked
    def ain28_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_C (AIN#(0:149)_EF_READ_C@28)

        Address: 7656
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_C)  # type: ignore

    @ain28_ef_read_c.setter
    @locked
    def ain28_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_C (AIN#(0:149)_EF_READ_C@28)

        Address: 7656
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_C, value)

    @property
    @locked
    def ain29_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_C (AIN#(0:149)_EF_READ_C@29)

        Address: 7658
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_C)  # type: ignore

    @ain29_ef_read_c.setter
    @locked
    def ain29_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_C (AIN#(0:149)_EF_READ_C@29)

        Address: 7658
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_C, value)

    @property
    @locked
    def ain30_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_C (AIN#(0:149)_EF_READ_C@30)

        Address: 7660
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_C)  # type: ignore

    @ain30_ef_read_c.setter
    @locked
    def ain30_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_C (AIN#(0:149)_EF_READ_C@30)

        Address: 7660
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_C, value)

    @property
    @locked
    def ain31_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_C (AIN#(0:149)_EF_READ_C@31)

        Address: 7662
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_C)  # type: ignore

    @ain31_ef_read_c.setter
    @locked
    def ain31_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_C (AIN#(0:149)_EF_READ_C@31)

        Address: 7662
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_C, value)

    @property
    @locked
    def ain32_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_C (AIN#(0:149)_EF_READ_C@32)

        Address: 7664
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_C)  # type: ignore

    @ain32_ef_read_c.setter
    @locked
    def ain32_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_C (AIN#(0:149)_EF_READ_C@32)

        Address: 7664
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_C, value)

    @property
    @locked
    def ain33_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_C (AIN#(0:149)_EF_READ_C@33)

        Address: 7666
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_C)  # type: ignore

    @ain33_ef_read_c.setter
    @locked
    def ain33_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_C (AIN#(0:149)_EF_READ_C@33)

        Address: 7666
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_C, value)

    @property
    @locked
    def ain34_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_C (AIN#(0:149)_EF_READ_C@34)

        Address: 7668
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_C)  # type: ignore

    @ain34_ef_read_c.setter
    @locked
    def ain34_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_C (AIN#(0:149)_EF_READ_C@34)

        Address: 7668
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_C, value)

    @property
    @locked
    def ain35_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_C (AIN#(0:149)_EF_READ_C@35)

        Address: 7670
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_C)  # type: ignore

    @ain35_ef_read_c.setter
    @locked
    def ain35_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_C (AIN#(0:149)_EF_READ_C@35)

        Address: 7670
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_C, value)

    @property
    @locked
    def ain36_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_C (AIN#(0:149)_EF_READ_C@36)

        Address: 7672
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_C)  # type: ignore

    @ain36_ef_read_c.setter
    @locked
    def ain36_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_C (AIN#(0:149)_EF_READ_C@36)

        Address: 7672
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_C, value)

    @property
    @locked
    def ain37_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_C (AIN#(0:149)_EF_READ_C@37)

        Address: 7674
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_C)  # type: ignore

    @ain37_ef_read_c.setter
    @locked
    def ain37_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_C (AIN#(0:149)_EF_READ_C@37)

        Address: 7674
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_C, value)

    @property
    @locked
    def ain38_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_C (AIN#(0:149)_EF_READ_C@38)

        Address: 7676
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_C)  # type: ignore

    @ain38_ef_read_c.setter
    @locked
    def ain38_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_C (AIN#(0:149)_EF_READ_C@38)

        Address: 7676
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_C, value)

    @property
    @locked
    def ain39_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_C (AIN#(0:149)_EF_READ_C@39)

        Address: 7678
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_C)  # type: ignore

    @ain39_ef_read_c.setter
    @locked
    def ain39_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_C (AIN#(0:149)_EF_READ_C@39)

        Address: 7678
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_C, value)

    @property
    @locked
    def ain40_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_C (AIN#(0:149)_EF_READ_C@40)

        Address: 7680
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_C)  # type: ignore

    @ain40_ef_read_c.setter
    @locked
    def ain40_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_C (AIN#(0:149)_EF_READ_C@40)

        Address: 7680
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_C, value)

    @property
    @locked
    def ain41_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_C (AIN#(0:149)_EF_READ_C@41)

        Address: 7682
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_C)  # type: ignore

    @ain41_ef_read_c.setter
    @locked
    def ain41_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_C (AIN#(0:149)_EF_READ_C@41)

        Address: 7682
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_C, value)

    @property
    @locked
    def ain42_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_C (AIN#(0:149)_EF_READ_C@42)

        Address: 7684
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_C)  # type: ignore

    @ain42_ef_read_c.setter
    @locked
    def ain42_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_C (AIN#(0:149)_EF_READ_C@42)

        Address: 7684
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_C, value)

    @property
    @locked
    def ain43_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_C (AIN#(0:149)_EF_READ_C@43)

        Address: 7686
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_C)  # type: ignore

    @ain43_ef_read_c.setter
    @locked
    def ain43_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_C (AIN#(0:149)_EF_READ_C@43)

        Address: 7686
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_C, value)

    @property
    @locked
    def ain44_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_C (AIN#(0:149)_EF_READ_C@44)

        Address: 7688
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_C)  # type: ignore

    @ain44_ef_read_c.setter
    @locked
    def ain44_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_C (AIN#(0:149)_EF_READ_C@44)

        Address: 7688
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_C, value)

    @property
    @locked
    def ain45_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_C (AIN#(0:149)_EF_READ_C@45)

        Address: 7690
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_C)  # type: ignore

    @ain45_ef_read_c.setter
    @locked
    def ain45_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_C (AIN#(0:149)_EF_READ_C@45)

        Address: 7690
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_C, value)

    @property
    @locked
    def ain46_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_C (AIN#(0:149)_EF_READ_C@46)

        Address: 7692
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_C)  # type: ignore

    @ain46_ef_read_c.setter
    @locked
    def ain46_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_C (AIN#(0:149)_EF_READ_C@46)

        Address: 7692
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_C, value)

    @property
    @locked
    def ain47_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_C (AIN#(0:149)_EF_READ_C@47)

        Address: 7694
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_C)  # type: ignore

    @ain47_ef_read_c.setter
    @locked
    def ain47_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_C (AIN#(0:149)_EF_READ_C@47)

        Address: 7694
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_C, value)

    @property
    @locked
    def ain48_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_C (AIN#(0:149)_EF_READ_C@48)

        Address: 7696
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_C)  # type: ignore

    @ain48_ef_read_c.setter
    @locked
    def ain48_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_C (AIN#(0:149)_EF_READ_C@48)

        Address: 7696
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_C, value)

    @property
    @locked
    def ain49_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_C (AIN#(0:149)_EF_READ_C@49)

        Address: 7698
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_C)  # type: ignore

    @ain49_ef_read_c.setter
    @locked
    def ain49_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_C (AIN#(0:149)_EF_READ_C@49)

        Address: 7698
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_C, value)

    @property
    @locked
    def ain50_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_C (AIN#(0:149)_EF_READ_C@50)

        Address: 7700
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_C)  # type: ignore

    @ain50_ef_read_c.setter
    @locked
    def ain50_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_C (AIN#(0:149)_EF_READ_C@50)

        Address: 7700
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_C, value)

    @property
    @locked
    def ain51_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_C (AIN#(0:149)_EF_READ_C@51)

        Address: 7702
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_C)  # type: ignore

    @ain51_ef_read_c.setter
    @locked
    def ain51_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_C (AIN#(0:149)_EF_READ_C@51)

        Address: 7702
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_C, value)

    @property
    @locked
    def ain52_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_C (AIN#(0:149)_EF_READ_C@52)

        Address: 7704
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_C)  # type: ignore

    @ain52_ef_read_c.setter
    @locked
    def ain52_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_C (AIN#(0:149)_EF_READ_C@52)

        Address: 7704
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_C, value)

    @property
    @locked
    def ain53_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_C (AIN#(0:149)_EF_READ_C@53)

        Address: 7706
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_C)  # type: ignore

    @ain53_ef_read_c.setter
    @locked
    def ain53_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_C (AIN#(0:149)_EF_READ_C@53)

        Address: 7706
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_C, value)

    @property
    @locked
    def ain54_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_C (AIN#(0:149)_EF_READ_C@54)

        Address: 7708
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_C)  # type: ignore

    @ain54_ef_read_c.setter
    @locked
    def ain54_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_C (AIN#(0:149)_EF_READ_C@54)

        Address: 7708
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_C, value)

    @property
    @locked
    def ain55_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_C (AIN#(0:149)_EF_READ_C@55)

        Address: 7710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_C)  # type: ignore

    @ain55_ef_read_c.setter
    @locked
    def ain55_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_C (AIN#(0:149)_EF_READ_C@55)

        Address: 7710
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_C, value)

    @property
    @locked
    def ain56_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_C (AIN#(0:149)_EF_READ_C@56)

        Address: 7712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_C)  # type: ignore

    @ain56_ef_read_c.setter
    @locked
    def ain56_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_C (AIN#(0:149)_EF_READ_C@56)

        Address: 7712
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_C, value)

    @property
    @locked
    def ain57_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_C (AIN#(0:149)_EF_READ_C@57)

        Address: 7714
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_C)  # type: ignore

    @ain57_ef_read_c.setter
    @locked
    def ain57_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_C (AIN#(0:149)_EF_READ_C@57)

        Address: 7714
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_C, value)

    @property
    @locked
    def ain58_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_C (AIN#(0:149)_EF_READ_C@58)

        Address: 7716
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_C)  # type: ignore

    @ain58_ef_read_c.setter
    @locked
    def ain58_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_C (AIN#(0:149)_EF_READ_C@58)

        Address: 7716
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_C, value)

    @property
    @locked
    def ain59_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_C (AIN#(0:149)_EF_READ_C@59)

        Address: 7718
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_C)  # type: ignore

    @ain59_ef_read_c.setter
    @locked
    def ain59_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_C (AIN#(0:149)_EF_READ_C@59)

        Address: 7718
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_C, value)

    @property
    @locked
    def ain60_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_C (AIN#(0:149)_EF_READ_C@60)

        Address: 7720
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_C)  # type: ignore

    @ain60_ef_read_c.setter
    @locked
    def ain60_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_C (AIN#(0:149)_EF_READ_C@60)

        Address: 7720
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_C, value)

    @property
    @locked
    def ain61_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_C (AIN#(0:149)_EF_READ_C@61)

        Address: 7722
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_C)  # type: ignore

    @ain61_ef_read_c.setter
    @locked
    def ain61_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_C (AIN#(0:149)_EF_READ_C@61)

        Address: 7722
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_C, value)

    @property
    @locked
    def ain62_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_C (AIN#(0:149)_EF_READ_C@62)

        Address: 7724
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_C)  # type: ignore

    @ain62_ef_read_c.setter
    @locked
    def ain62_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_C (AIN#(0:149)_EF_READ_C@62)

        Address: 7724
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_C, value)

    @property
    @locked
    def ain63_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_C (AIN#(0:149)_EF_READ_C@63)

        Address: 7726
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_C)  # type: ignore

    @ain63_ef_read_c.setter
    @locked
    def ain63_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_C (AIN#(0:149)_EF_READ_C@63)

        Address: 7726
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_C, value)

    @property
    @locked
    def ain64_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_C (AIN#(0:149)_EF_READ_C@64)

        Address: 7728
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_C)  # type: ignore

    @ain64_ef_read_c.setter
    @locked
    def ain64_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_C (AIN#(0:149)_EF_READ_C@64)

        Address: 7728
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_C, value)

    @property
    @locked
    def ain65_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_C (AIN#(0:149)_EF_READ_C@65)

        Address: 7730
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_C)  # type: ignore

    @ain65_ef_read_c.setter
    @locked
    def ain65_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_C (AIN#(0:149)_EF_READ_C@65)

        Address: 7730
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_C, value)

    @property
    @locked
    def ain66_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_C (AIN#(0:149)_EF_READ_C@66)

        Address: 7732
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_C)  # type: ignore

    @ain66_ef_read_c.setter
    @locked
    def ain66_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_C (AIN#(0:149)_EF_READ_C@66)

        Address: 7732
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_C, value)

    @property
    @locked
    def ain67_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_C (AIN#(0:149)_EF_READ_C@67)

        Address: 7734
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_C)  # type: ignore

    @ain67_ef_read_c.setter
    @locked
    def ain67_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_C (AIN#(0:149)_EF_READ_C@67)

        Address: 7734
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_C, value)

    @property
    @locked
    def ain68_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_C (AIN#(0:149)_EF_READ_C@68)

        Address: 7736
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_C)  # type: ignore

    @ain68_ef_read_c.setter
    @locked
    def ain68_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_C (AIN#(0:149)_EF_READ_C@68)

        Address: 7736
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_C, value)

    @property
    @locked
    def ain69_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_C (AIN#(0:149)_EF_READ_C@69)

        Address: 7738
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_C)  # type: ignore

    @ain69_ef_read_c.setter
    @locked
    def ain69_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_C (AIN#(0:149)_EF_READ_C@69)

        Address: 7738
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_C, value)

    @property
    @locked
    def ain70_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_C (AIN#(0:149)_EF_READ_C@70)

        Address: 7740
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_C)  # type: ignore

    @ain70_ef_read_c.setter
    @locked
    def ain70_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_C (AIN#(0:149)_EF_READ_C@70)

        Address: 7740
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_C, value)

    @property
    @locked
    def ain71_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_C (AIN#(0:149)_EF_READ_C@71)

        Address: 7742
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_C)  # type: ignore

    @ain71_ef_read_c.setter
    @locked
    def ain71_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_C (AIN#(0:149)_EF_READ_C@71)

        Address: 7742
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_C, value)

    @property
    @locked
    def ain72_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_C (AIN#(0:149)_EF_READ_C@72)

        Address: 7744
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_C)  # type: ignore

    @ain72_ef_read_c.setter
    @locked
    def ain72_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_C (AIN#(0:149)_EF_READ_C@72)

        Address: 7744
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_C, value)

    @property
    @locked
    def ain73_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_C (AIN#(0:149)_EF_READ_C@73)

        Address: 7746
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_C)  # type: ignore

    @ain73_ef_read_c.setter
    @locked
    def ain73_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_C (AIN#(0:149)_EF_READ_C@73)

        Address: 7746
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_C, value)

    @property
    @locked
    def ain74_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_C (AIN#(0:149)_EF_READ_C@74)

        Address: 7748
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_C)  # type: ignore

    @ain74_ef_read_c.setter
    @locked
    def ain74_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_C (AIN#(0:149)_EF_READ_C@74)

        Address: 7748
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_C, value)

    @property
    @locked
    def ain75_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_C (AIN#(0:149)_EF_READ_C@75)

        Address: 7750
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_C)  # type: ignore

    @ain75_ef_read_c.setter
    @locked
    def ain75_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_C (AIN#(0:149)_EF_READ_C@75)

        Address: 7750
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_C, value)

    @property
    @locked
    def ain76_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_C (AIN#(0:149)_EF_READ_C@76)

        Address: 7752
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_C)  # type: ignore

    @ain76_ef_read_c.setter
    @locked
    def ain76_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_C (AIN#(0:149)_EF_READ_C@76)

        Address: 7752
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_C, value)

    @property
    @locked
    def ain77_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_C (AIN#(0:149)_EF_READ_C@77)

        Address: 7754
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_C)  # type: ignore

    @ain77_ef_read_c.setter
    @locked
    def ain77_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_C (AIN#(0:149)_EF_READ_C@77)

        Address: 7754
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_C, value)

    @property
    @locked
    def ain78_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_C (AIN#(0:149)_EF_READ_C@78)

        Address: 7756
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_C)  # type: ignore

    @ain78_ef_read_c.setter
    @locked
    def ain78_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_C (AIN#(0:149)_EF_READ_C@78)

        Address: 7756
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_C, value)

    @property
    @locked
    def ain79_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_C (AIN#(0:149)_EF_READ_C@79)

        Address: 7758
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_C)  # type: ignore

    @ain79_ef_read_c.setter
    @locked
    def ain79_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_C (AIN#(0:149)_EF_READ_C@79)

        Address: 7758
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_C, value)

    @property
    @locked
    def ain80_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_C (AIN#(0:149)_EF_READ_C@80)

        Address: 7760
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_C)  # type: ignore

    @ain80_ef_read_c.setter
    @locked
    def ain80_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_C (AIN#(0:149)_EF_READ_C@80)

        Address: 7760
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_C, value)

    @property
    @locked
    def ain81_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_C (AIN#(0:149)_EF_READ_C@81)

        Address: 7762
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_C)  # type: ignore

    @ain81_ef_read_c.setter
    @locked
    def ain81_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_C (AIN#(0:149)_EF_READ_C@81)

        Address: 7762
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_C, value)

    @property
    @locked
    def ain82_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_C (AIN#(0:149)_EF_READ_C@82)

        Address: 7764
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_C)  # type: ignore

    @ain82_ef_read_c.setter
    @locked
    def ain82_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_C (AIN#(0:149)_EF_READ_C@82)

        Address: 7764
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_C, value)

    @property
    @locked
    def ain83_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_C (AIN#(0:149)_EF_READ_C@83)

        Address: 7766
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_C)  # type: ignore

    @ain83_ef_read_c.setter
    @locked
    def ain83_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_C (AIN#(0:149)_EF_READ_C@83)

        Address: 7766
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_C, value)

    @property
    @locked
    def ain84_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_C (AIN#(0:149)_EF_READ_C@84)

        Address: 7768
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_C)  # type: ignore

    @ain84_ef_read_c.setter
    @locked
    def ain84_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_C (AIN#(0:149)_EF_READ_C@84)

        Address: 7768
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_C, value)

    @property
    @locked
    def ain85_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_C (AIN#(0:149)_EF_READ_C@85)

        Address: 7770
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_C)  # type: ignore

    @ain85_ef_read_c.setter
    @locked
    def ain85_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_C (AIN#(0:149)_EF_READ_C@85)

        Address: 7770
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_C, value)

    @property
    @locked
    def ain86_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_C (AIN#(0:149)_EF_READ_C@86)

        Address: 7772
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_C)  # type: ignore

    @ain86_ef_read_c.setter
    @locked
    def ain86_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_C (AIN#(0:149)_EF_READ_C@86)

        Address: 7772
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_C, value)

    @property
    @locked
    def ain87_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_C (AIN#(0:149)_EF_READ_C@87)

        Address: 7774
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_C)  # type: ignore

    @ain87_ef_read_c.setter
    @locked
    def ain87_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_C (AIN#(0:149)_EF_READ_C@87)

        Address: 7774
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_C, value)

    @property
    @locked
    def ain88_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_C (AIN#(0:149)_EF_READ_C@88)

        Address: 7776
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_C)  # type: ignore

    @ain88_ef_read_c.setter
    @locked
    def ain88_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_C (AIN#(0:149)_EF_READ_C@88)

        Address: 7776
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_C, value)

    @property
    @locked
    def ain89_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_C (AIN#(0:149)_EF_READ_C@89)

        Address: 7778
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_C)  # type: ignore

    @ain89_ef_read_c.setter
    @locked
    def ain89_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_C (AIN#(0:149)_EF_READ_C@89)

        Address: 7778
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_C, value)

    @property
    @locked
    def ain90_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_C (AIN#(0:149)_EF_READ_C@90)

        Address: 7780
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_C)  # type: ignore

    @ain90_ef_read_c.setter
    @locked
    def ain90_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_C (AIN#(0:149)_EF_READ_C@90)

        Address: 7780
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_C, value)

    @property
    @locked
    def ain91_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_C (AIN#(0:149)_EF_READ_C@91)

        Address: 7782
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_C)  # type: ignore

    @ain91_ef_read_c.setter
    @locked
    def ain91_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_C (AIN#(0:149)_EF_READ_C@91)

        Address: 7782
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_C, value)

    @property
    @locked
    def ain92_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_C (AIN#(0:149)_EF_READ_C@92)

        Address: 7784
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_C)  # type: ignore

    @ain92_ef_read_c.setter
    @locked
    def ain92_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_C (AIN#(0:149)_EF_READ_C@92)

        Address: 7784
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_C, value)

    @property
    @locked
    def ain93_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_C (AIN#(0:149)_EF_READ_C@93)

        Address: 7786
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_C)  # type: ignore

    @ain93_ef_read_c.setter
    @locked
    def ain93_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_C (AIN#(0:149)_EF_READ_C@93)

        Address: 7786
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_C, value)

    @property
    @locked
    def ain94_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_C (AIN#(0:149)_EF_READ_C@94)

        Address: 7788
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_C)  # type: ignore

    @ain94_ef_read_c.setter
    @locked
    def ain94_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_C (AIN#(0:149)_EF_READ_C@94)

        Address: 7788
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_C, value)

    @property
    @locked
    def ain95_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_C (AIN#(0:149)_EF_READ_C@95)

        Address: 7790
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_C)  # type: ignore

    @ain95_ef_read_c.setter
    @locked
    def ain95_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_C (AIN#(0:149)_EF_READ_C@95)

        Address: 7790
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_C, value)

    @property
    @locked
    def ain96_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_C (AIN#(0:149)_EF_READ_C@96)

        Address: 7792
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_C)  # type: ignore

    @ain96_ef_read_c.setter
    @locked
    def ain96_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_C (AIN#(0:149)_EF_READ_C@96)

        Address: 7792
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_C, value)

    @property
    @locked
    def ain97_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_C (AIN#(0:149)_EF_READ_C@97)

        Address: 7794
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_C)  # type: ignore

    @ain97_ef_read_c.setter
    @locked
    def ain97_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_C (AIN#(0:149)_EF_READ_C@97)

        Address: 7794
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_C, value)

    @property
    @locked
    def ain98_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_C (AIN#(0:149)_EF_READ_C@98)

        Address: 7796
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_C)  # type: ignore

    @ain98_ef_read_c.setter
    @locked
    def ain98_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_C (AIN#(0:149)_EF_READ_C@98)

        Address: 7796
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_C, value)

    @property
    @locked
    def ain99_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_C (AIN#(0:149)_EF_READ_C@99)

        Address: 7798
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_C)  # type: ignore

    @ain99_ef_read_c.setter
    @locked
    def ain99_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_C (AIN#(0:149)_EF_READ_C@99)

        Address: 7798
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_C, value)

    @property
    @locked
    def ain100_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_C (AIN#(0:149)_EF_READ_C@100)

        Address: 7800
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_C)  # type: ignore

    @ain100_ef_read_c.setter
    @locked
    def ain100_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_C (AIN#(0:149)_EF_READ_C@100)

        Address: 7800
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_C, value)

    @property
    @locked
    def ain101_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_C (AIN#(0:149)_EF_READ_C@101)

        Address: 7802
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_C)  # type: ignore

    @ain101_ef_read_c.setter
    @locked
    def ain101_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_C (AIN#(0:149)_EF_READ_C@101)

        Address: 7802
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_C, value)

    @property
    @locked
    def ain102_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_C (AIN#(0:149)_EF_READ_C@102)

        Address: 7804
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_C)  # type: ignore

    @ain102_ef_read_c.setter
    @locked
    def ain102_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_C (AIN#(0:149)_EF_READ_C@102)

        Address: 7804
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_C, value)

    @property
    @locked
    def ain103_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_C (AIN#(0:149)_EF_READ_C@103)

        Address: 7806
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_C)  # type: ignore

    @ain103_ef_read_c.setter
    @locked
    def ain103_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_C (AIN#(0:149)_EF_READ_C@103)

        Address: 7806
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_C, value)

    @property
    @locked
    def ain104_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_C (AIN#(0:149)_EF_READ_C@104)

        Address: 7808
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_C)  # type: ignore

    @ain104_ef_read_c.setter
    @locked
    def ain104_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_C (AIN#(0:149)_EF_READ_C@104)

        Address: 7808
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_C, value)

    @property
    @locked
    def ain105_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_C (AIN#(0:149)_EF_READ_C@105)

        Address: 7810
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_C)  # type: ignore

    @ain105_ef_read_c.setter
    @locked
    def ain105_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_C (AIN#(0:149)_EF_READ_C@105)

        Address: 7810
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_C, value)

    @property
    @locked
    def ain106_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_C (AIN#(0:149)_EF_READ_C@106)

        Address: 7812
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_C)  # type: ignore

    @ain106_ef_read_c.setter
    @locked
    def ain106_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_C (AIN#(0:149)_EF_READ_C@106)

        Address: 7812
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_C, value)

    @property
    @locked
    def ain107_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_C (AIN#(0:149)_EF_READ_C@107)

        Address: 7814
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_C)  # type: ignore

    @ain107_ef_read_c.setter
    @locked
    def ain107_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_C (AIN#(0:149)_EF_READ_C@107)

        Address: 7814
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_C, value)

    @property
    @locked
    def ain108_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_C (AIN#(0:149)_EF_READ_C@108)

        Address: 7816
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_C)  # type: ignore

    @ain108_ef_read_c.setter
    @locked
    def ain108_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_C (AIN#(0:149)_EF_READ_C@108)

        Address: 7816
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_C, value)

    @property
    @locked
    def ain109_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_C (AIN#(0:149)_EF_READ_C@109)

        Address: 7818
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_C)  # type: ignore

    @ain109_ef_read_c.setter
    @locked
    def ain109_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_C (AIN#(0:149)_EF_READ_C@109)

        Address: 7818
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_C, value)

    @property
    @locked
    def ain110_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_C (AIN#(0:149)_EF_READ_C@110)

        Address: 7820
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_C)  # type: ignore

    @ain110_ef_read_c.setter
    @locked
    def ain110_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_C (AIN#(0:149)_EF_READ_C@110)

        Address: 7820
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_C, value)

    @property
    @locked
    def ain111_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_C (AIN#(0:149)_EF_READ_C@111)

        Address: 7822
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_C)  # type: ignore

    @ain111_ef_read_c.setter
    @locked
    def ain111_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_C (AIN#(0:149)_EF_READ_C@111)

        Address: 7822
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_C, value)

    @property
    @locked
    def ain112_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_C (AIN#(0:149)_EF_READ_C@112)

        Address: 7824
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_C)  # type: ignore

    @ain112_ef_read_c.setter
    @locked
    def ain112_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_C (AIN#(0:149)_EF_READ_C@112)

        Address: 7824
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_C, value)

    @property
    @locked
    def ain113_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_C (AIN#(0:149)_EF_READ_C@113)

        Address: 7826
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_C)  # type: ignore

    @ain113_ef_read_c.setter
    @locked
    def ain113_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_C (AIN#(0:149)_EF_READ_C@113)

        Address: 7826
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_C, value)

    @property
    @locked
    def ain114_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_C (AIN#(0:149)_EF_READ_C@114)

        Address: 7828
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_C)  # type: ignore

    @ain114_ef_read_c.setter
    @locked
    def ain114_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_C (AIN#(0:149)_EF_READ_C@114)

        Address: 7828
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_C, value)

    @property
    @locked
    def ain115_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_C (AIN#(0:149)_EF_READ_C@115)

        Address: 7830
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_C)  # type: ignore

    @ain115_ef_read_c.setter
    @locked
    def ain115_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_C (AIN#(0:149)_EF_READ_C@115)

        Address: 7830
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_C, value)

    @property
    @locked
    def ain116_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_C (AIN#(0:149)_EF_READ_C@116)

        Address: 7832
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_C)  # type: ignore

    @ain116_ef_read_c.setter
    @locked
    def ain116_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_C (AIN#(0:149)_EF_READ_C@116)

        Address: 7832
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_C, value)

    @property
    @locked
    def ain117_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_C (AIN#(0:149)_EF_READ_C@117)

        Address: 7834
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_C)  # type: ignore

    @ain117_ef_read_c.setter
    @locked
    def ain117_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_C (AIN#(0:149)_EF_READ_C@117)

        Address: 7834
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_C, value)

    @property
    @locked
    def ain118_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_C (AIN#(0:149)_EF_READ_C@118)

        Address: 7836
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_C)  # type: ignore

    @ain118_ef_read_c.setter
    @locked
    def ain118_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_C (AIN#(0:149)_EF_READ_C@118)

        Address: 7836
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_C, value)

    @property
    @locked
    def ain119_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_C (AIN#(0:149)_EF_READ_C@119)

        Address: 7838
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_C)  # type: ignore

    @ain119_ef_read_c.setter
    @locked
    def ain119_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_C (AIN#(0:149)_EF_READ_C@119)

        Address: 7838
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_C, value)

    @property
    @locked
    def ain120_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_C (AIN#(0:149)_EF_READ_C@120)

        Address: 7840
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_C)  # type: ignore

    @ain120_ef_read_c.setter
    @locked
    def ain120_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_C (AIN#(0:149)_EF_READ_C@120)

        Address: 7840
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_C, value)

    @property
    @locked
    def ain121_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_C (AIN#(0:149)_EF_READ_C@121)

        Address: 7842
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_C)  # type: ignore

    @ain121_ef_read_c.setter
    @locked
    def ain121_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_C (AIN#(0:149)_EF_READ_C@121)

        Address: 7842
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_C, value)

    @property
    @locked
    def ain122_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_C (AIN#(0:149)_EF_READ_C@122)

        Address: 7844
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_C)  # type: ignore

    @ain122_ef_read_c.setter
    @locked
    def ain122_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_C (AIN#(0:149)_EF_READ_C@122)

        Address: 7844
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_C, value)

    @property
    @locked
    def ain123_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_C (AIN#(0:149)_EF_READ_C@123)

        Address: 7846
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_C)  # type: ignore

    @ain123_ef_read_c.setter
    @locked
    def ain123_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_C (AIN#(0:149)_EF_READ_C@123)

        Address: 7846
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_C, value)

    @property
    @locked
    def ain124_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_C (AIN#(0:149)_EF_READ_C@124)

        Address: 7848
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_C)  # type: ignore

    @ain124_ef_read_c.setter
    @locked
    def ain124_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_C (AIN#(0:149)_EF_READ_C@124)

        Address: 7848
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_C, value)

    @property
    @locked
    def ain125_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_C (AIN#(0:149)_EF_READ_C@125)

        Address: 7850
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_C)  # type: ignore

    @ain125_ef_read_c.setter
    @locked
    def ain125_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_C (AIN#(0:149)_EF_READ_C@125)

        Address: 7850
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_C, value)

    @property
    @locked
    def ain126_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_C (AIN#(0:149)_EF_READ_C@126)

        Address: 7852
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_C)  # type: ignore

    @ain126_ef_read_c.setter
    @locked
    def ain126_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_C (AIN#(0:149)_EF_READ_C@126)

        Address: 7852
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_C, value)

    @property
    @locked
    def ain127_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_C (AIN#(0:149)_EF_READ_C@127)

        Address: 7854
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_C)  # type: ignore

    @ain127_ef_read_c.setter
    @locked
    def ain127_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_C (AIN#(0:149)_EF_READ_C@127)

        Address: 7854
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_C, value)

    @property
    @locked
    def ain128_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_C (AIN#(0:149)_EF_READ_C@128)

        Address: 7856
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_C)  # type: ignore

    @ain128_ef_read_c.setter
    @locked
    def ain128_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_C (AIN#(0:149)_EF_READ_C@128)

        Address: 7856
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_C, value)

    @property
    @locked
    def ain129_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_C (AIN#(0:149)_EF_READ_C@129)

        Address: 7858
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_C)  # type: ignore

    @ain129_ef_read_c.setter
    @locked
    def ain129_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_C (AIN#(0:149)_EF_READ_C@129)

        Address: 7858
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_C, value)

    @property
    @locked
    def ain130_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_C (AIN#(0:149)_EF_READ_C@130)

        Address: 7860
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_C)  # type: ignore

    @ain130_ef_read_c.setter
    @locked
    def ain130_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_C (AIN#(0:149)_EF_READ_C@130)

        Address: 7860
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_C, value)

    @property
    @locked
    def ain131_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_C (AIN#(0:149)_EF_READ_C@131)

        Address: 7862
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_C)  # type: ignore

    @ain131_ef_read_c.setter
    @locked
    def ain131_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_C (AIN#(0:149)_EF_READ_C@131)

        Address: 7862
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_C, value)

    @property
    @locked
    def ain132_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_C (AIN#(0:149)_EF_READ_C@132)

        Address: 7864
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_C)  # type: ignore

    @ain132_ef_read_c.setter
    @locked
    def ain132_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_C (AIN#(0:149)_EF_READ_C@132)

        Address: 7864
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_C, value)

    @property
    @locked
    def ain133_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_C (AIN#(0:149)_EF_READ_C@133)

        Address: 7866
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_C)  # type: ignore

    @ain133_ef_read_c.setter
    @locked
    def ain133_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_C (AIN#(0:149)_EF_READ_C@133)

        Address: 7866
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_C, value)

    @property
    @locked
    def ain134_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_C (AIN#(0:149)_EF_READ_C@134)

        Address: 7868
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_C)  # type: ignore

    @ain134_ef_read_c.setter
    @locked
    def ain134_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_C (AIN#(0:149)_EF_READ_C@134)

        Address: 7868
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_C, value)

    @property
    @locked
    def ain135_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_C (AIN#(0:149)_EF_READ_C@135)

        Address: 7870
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_C)  # type: ignore

    @ain135_ef_read_c.setter
    @locked
    def ain135_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_C (AIN#(0:149)_EF_READ_C@135)

        Address: 7870
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_C, value)

    @property
    @locked
    def ain136_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_C (AIN#(0:149)_EF_READ_C@136)

        Address: 7872
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_C)  # type: ignore

    @ain136_ef_read_c.setter
    @locked
    def ain136_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_C (AIN#(0:149)_EF_READ_C@136)

        Address: 7872
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_C, value)

    @property
    @locked
    def ain137_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_C (AIN#(0:149)_EF_READ_C@137)

        Address: 7874
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_C)  # type: ignore

    @ain137_ef_read_c.setter
    @locked
    def ain137_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_C (AIN#(0:149)_EF_READ_C@137)

        Address: 7874
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_C, value)

    @property
    @locked
    def ain138_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_C (AIN#(0:149)_EF_READ_C@138)

        Address: 7876
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_C)  # type: ignore

    @ain138_ef_read_c.setter
    @locked
    def ain138_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_C (AIN#(0:149)_EF_READ_C@138)

        Address: 7876
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_C, value)

    @property
    @locked
    def ain139_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_C (AIN#(0:149)_EF_READ_C@139)

        Address: 7878
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_C)  # type: ignore

    @ain139_ef_read_c.setter
    @locked
    def ain139_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_C (AIN#(0:149)_EF_READ_C@139)

        Address: 7878
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_C, value)

    @property
    @locked
    def ain140_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_C (AIN#(0:149)_EF_READ_C@140)

        Address: 7880
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_C)  # type: ignore

    @ain140_ef_read_c.setter
    @locked
    def ain140_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_C (AIN#(0:149)_EF_READ_C@140)

        Address: 7880
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_C, value)

    @property
    @locked
    def ain141_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_C (AIN#(0:149)_EF_READ_C@141)

        Address: 7882
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_C)  # type: ignore

    @ain141_ef_read_c.setter
    @locked
    def ain141_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_C (AIN#(0:149)_EF_READ_C@141)

        Address: 7882
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_C, value)

    @property
    @locked
    def ain142_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_C (AIN#(0:149)_EF_READ_C@142)

        Address: 7884
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_C)  # type: ignore

    @ain142_ef_read_c.setter
    @locked
    def ain142_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_C (AIN#(0:149)_EF_READ_C@142)

        Address: 7884
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_C, value)

    @property
    @locked
    def ain143_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_C (AIN#(0:149)_EF_READ_C@143)

        Address: 7886
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_C)  # type: ignore

    @ain143_ef_read_c.setter
    @locked
    def ain143_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_C (AIN#(0:149)_EF_READ_C@143)

        Address: 7886
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_C, value)

    @property
    @locked
    def ain144_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_C (AIN#(0:149)_EF_READ_C@144)

        Address: 7888
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_C)  # type: ignore

    @ain144_ef_read_c.setter
    @locked
    def ain144_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_C (AIN#(0:149)_EF_READ_C@144)

        Address: 7888
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_C, value)

    @property
    @locked
    def ain145_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_C (AIN#(0:149)_EF_READ_C@145)

        Address: 7890
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_C)  # type: ignore

    @ain145_ef_read_c.setter
    @locked
    def ain145_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_C (AIN#(0:149)_EF_READ_C@145)

        Address: 7890
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_C, value)

    @property
    @locked
    def ain146_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_C (AIN#(0:149)_EF_READ_C@146)

        Address: 7892
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_C)  # type: ignore

    @ain146_ef_read_c.setter
    @locked
    def ain146_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_C (AIN#(0:149)_EF_READ_C@146)

        Address: 7892
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_C, value)

    @property
    @locked
    def ain147_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_C (AIN#(0:149)_EF_READ_C@147)

        Address: 7894
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_C)  # type: ignore

    @ain147_ef_read_c.setter
    @locked
    def ain147_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_C (AIN#(0:149)_EF_READ_C@147)

        Address: 7894
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_C, value)

    @property
    @locked
    def ain148_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_C (AIN#(0:149)_EF_READ_C@148)

        Address: 7896
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_C)  # type: ignore

    @ain148_ef_read_c.setter
    @locked
    def ain148_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_C (AIN#(0:149)_EF_READ_C@148)

        Address: 7896
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_C, value)

    @property
    @locked
    def ain149_ef_read_c(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_C (AIN#(0:149)_EF_READ_C@149)

        Address: 7898
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_C)  # type: ignore

    @ain149_ef_read_c.setter
    @locked
    def ain149_ef_read_c(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_C (AIN#(0:149)_EF_READ_C@149)

        Address: 7898
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_C, value)

    @property
    @locked
    def ain0_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_READ_D (AIN#(0:149)_EF_READ_D@0)

        Address: 7900
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain1_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_READ_D (AIN#(0:149)_EF_READ_D@1)

        Address: 7902
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain2_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_READ_D (AIN#(0:149)_EF_READ_D@2)

        Address: 7904
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain3_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_READ_D (AIN#(0:149)_EF_READ_D@3)

        Address: 7906
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain4_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_READ_D (AIN#(0:149)_EF_READ_D@4)

        Address: 7908
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain5_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_READ_D (AIN#(0:149)_EF_READ_D@5)

        Address: 7910
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain6_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_READ_D (AIN#(0:149)_EF_READ_D@6)

        Address: 7912
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain7_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_READ_D (AIN#(0:149)_EF_READ_D@7)

        Address: 7914
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain8_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_READ_D (AIN#(0:149)_EF_READ_D@8)

        Address: 7916
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain9_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_READ_D (AIN#(0:149)_EF_READ_D@9)

        Address: 7918
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain10_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_READ_D (AIN#(0:149)_EF_READ_D@10)

        Address: 7920
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain11_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_READ_D (AIN#(0:149)_EF_READ_D@11)

        Address: 7922
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain12_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_READ_D (AIN#(0:149)_EF_READ_D@12)

        Address: 7924
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain13_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_READ_D (AIN#(0:149)_EF_READ_D@13)

        Address: 7926
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain14_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_READ_D (AIN#(0:149)_EF_READ_D@14)

        Address: 7928
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain15_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_READ_D (AIN#(0:149)_EF_READ_D@15)

        Address: 7930
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain16_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_READ_D (AIN#(0:149)_EF_READ_D@16)

        Address: 7932
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain17_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_READ_D (AIN#(0:149)_EF_READ_D@17)

        Address: 7934
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain18_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_READ_D (AIN#(0:149)_EF_READ_D@18)

        Address: 7936
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain19_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_READ_D (AIN#(0:149)_EF_READ_D@19)

        Address: 7938
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain20_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_READ_D (AIN#(0:149)_EF_READ_D@20)

        Address: 7940
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain21_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_READ_D (AIN#(0:149)_EF_READ_D@21)

        Address: 7942
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain22_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_READ_D (AIN#(0:149)_EF_READ_D@22)

        Address: 7944
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain23_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_READ_D (AIN#(0:149)_EF_READ_D@23)

        Address: 7946
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain24_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_READ_D (AIN#(0:149)_EF_READ_D@24)

        Address: 7948
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain25_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_READ_D (AIN#(0:149)_EF_READ_D@25)

        Address: 7950
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain26_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_READ_D (AIN#(0:149)_EF_READ_D@26)

        Address: 7952
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain27_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_READ_D (AIN#(0:149)_EF_READ_D@27)

        Address: 7954
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain28_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_READ_D (AIN#(0:149)_EF_READ_D@28)

        Address: 7956
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain29_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_READ_D (AIN#(0:149)_EF_READ_D@29)

        Address: 7958
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain30_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_READ_D (AIN#(0:149)_EF_READ_D@30)

        Address: 7960
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain31_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_READ_D (AIN#(0:149)_EF_READ_D@31)

        Address: 7962
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain32_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_READ_D (AIN#(0:149)_EF_READ_D@32)

        Address: 7964
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain33_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_READ_D (AIN#(0:149)_EF_READ_D@33)

        Address: 7966
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain34_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_READ_D (AIN#(0:149)_EF_READ_D@34)

        Address: 7968
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain35_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_READ_D (AIN#(0:149)_EF_READ_D@35)

        Address: 7970
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain36_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_READ_D (AIN#(0:149)_EF_READ_D@36)

        Address: 7972
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain37_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_READ_D (AIN#(0:149)_EF_READ_D@37)

        Address: 7974
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain38_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_READ_D (AIN#(0:149)_EF_READ_D@38)

        Address: 7976
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain39_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_READ_D (AIN#(0:149)_EF_READ_D@39)

        Address: 7978
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain40_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_READ_D (AIN#(0:149)_EF_READ_D@40)

        Address: 7980
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain41_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_READ_D (AIN#(0:149)_EF_READ_D@41)

        Address: 7982
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain42_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_READ_D (AIN#(0:149)_EF_READ_D@42)

        Address: 7984
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain43_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_READ_D (AIN#(0:149)_EF_READ_D@43)

        Address: 7986
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain44_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_READ_D (AIN#(0:149)_EF_READ_D@44)

        Address: 7988
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain45_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_READ_D (AIN#(0:149)_EF_READ_D@45)

        Address: 7990
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain46_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_READ_D (AIN#(0:149)_EF_READ_D@46)

        Address: 7992
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain47_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_READ_D (AIN#(0:149)_EF_READ_D@47)

        Address: 7994
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain48_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_READ_D (AIN#(0:149)_EF_READ_D@48)

        Address: 7996
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain49_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_READ_D (AIN#(0:149)_EF_READ_D@49)

        Address: 7998
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain50_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_READ_D (AIN#(0:149)_EF_READ_D@50)

        Address: 8000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain51_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_READ_D (AIN#(0:149)_EF_READ_D@51)

        Address: 8002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain52_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_READ_D (AIN#(0:149)_EF_READ_D@52)

        Address: 8004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain53_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_READ_D (AIN#(0:149)_EF_READ_D@53)

        Address: 8006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain54_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_READ_D (AIN#(0:149)_EF_READ_D@54)

        Address: 8008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain55_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_READ_D (AIN#(0:149)_EF_READ_D@55)

        Address: 8010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain56_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_READ_D (AIN#(0:149)_EF_READ_D@56)

        Address: 8012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain57_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_READ_D (AIN#(0:149)_EF_READ_D@57)

        Address: 8014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain58_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_READ_D (AIN#(0:149)_EF_READ_D@58)

        Address: 8016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain59_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_READ_D (AIN#(0:149)_EF_READ_D@59)

        Address: 8018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain60_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_READ_D (AIN#(0:149)_EF_READ_D@60)

        Address: 8020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain61_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_READ_D (AIN#(0:149)_EF_READ_D@61)

        Address: 8022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain62_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_READ_D (AIN#(0:149)_EF_READ_D@62)

        Address: 8024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain63_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_READ_D (AIN#(0:149)_EF_READ_D@63)

        Address: 8026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain64_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_READ_D (AIN#(0:149)_EF_READ_D@64)

        Address: 8028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain65_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_READ_D (AIN#(0:149)_EF_READ_D@65)

        Address: 8030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain66_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_READ_D (AIN#(0:149)_EF_READ_D@66)

        Address: 8032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain67_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_READ_D (AIN#(0:149)_EF_READ_D@67)

        Address: 8034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain68_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_READ_D (AIN#(0:149)_EF_READ_D@68)

        Address: 8036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain69_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_READ_D (AIN#(0:149)_EF_READ_D@69)

        Address: 8038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain70_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_READ_D (AIN#(0:149)_EF_READ_D@70)

        Address: 8040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain71_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_READ_D (AIN#(0:149)_EF_READ_D@71)

        Address: 8042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain72_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_READ_D (AIN#(0:149)_EF_READ_D@72)

        Address: 8044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain73_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_READ_D (AIN#(0:149)_EF_READ_D@73)

        Address: 8046
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain74_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_READ_D (AIN#(0:149)_EF_READ_D@74)

        Address: 8048
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain75_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_READ_D (AIN#(0:149)_EF_READ_D@75)

        Address: 8050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain76_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_READ_D (AIN#(0:149)_EF_READ_D@76)

        Address: 8052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain77_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_READ_D (AIN#(0:149)_EF_READ_D@77)

        Address: 8054
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain78_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_READ_D (AIN#(0:149)_EF_READ_D@78)

        Address: 8056
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain79_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_READ_D (AIN#(0:149)_EF_READ_D@79)

        Address: 8058
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain80_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_READ_D (AIN#(0:149)_EF_READ_D@80)

        Address: 8060
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain81_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_READ_D (AIN#(0:149)_EF_READ_D@81)

        Address: 8062
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain82_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_READ_D (AIN#(0:149)_EF_READ_D@82)

        Address: 8064
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain83_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_READ_D (AIN#(0:149)_EF_READ_D@83)

        Address: 8066
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain84_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_READ_D (AIN#(0:149)_EF_READ_D@84)

        Address: 8068
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain85_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_READ_D (AIN#(0:149)_EF_READ_D@85)

        Address: 8070
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain86_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_READ_D (AIN#(0:149)_EF_READ_D@86)

        Address: 8072
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain87_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_READ_D (AIN#(0:149)_EF_READ_D@87)

        Address: 8074
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain88_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_READ_D (AIN#(0:149)_EF_READ_D@88)

        Address: 8076
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain89_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_READ_D (AIN#(0:149)_EF_READ_D@89)

        Address: 8078
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain90_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_READ_D (AIN#(0:149)_EF_READ_D@90)

        Address: 8080
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain91_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_READ_D (AIN#(0:149)_EF_READ_D@91)

        Address: 8082
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain92_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_READ_D (AIN#(0:149)_EF_READ_D@92)

        Address: 8084
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain93_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_READ_D (AIN#(0:149)_EF_READ_D@93)

        Address: 8086
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain94_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_READ_D (AIN#(0:149)_EF_READ_D@94)

        Address: 8088
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain95_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_READ_D (AIN#(0:149)_EF_READ_D@95)

        Address: 8090
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain96_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_READ_D (AIN#(0:149)_EF_READ_D@96)

        Address: 8092
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain97_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_READ_D (AIN#(0:149)_EF_READ_D@97)

        Address: 8094
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain98_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_READ_D (AIN#(0:149)_EF_READ_D@98)

        Address: 8096
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain99_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_READ_D (AIN#(0:149)_EF_READ_D@99)

        Address: 8098
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain100_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_READ_D (AIN#(0:149)_EF_READ_D@100)

        Address: 8100
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain101_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_READ_D (AIN#(0:149)_EF_READ_D@101)

        Address: 8102
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain102_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_READ_D (AIN#(0:149)_EF_READ_D@102)

        Address: 8104
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain103_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_READ_D (AIN#(0:149)_EF_READ_D@103)

        Address: 8106
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain104_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_READ_D (AIN#(0:149)_EF_READ_D@104)

        Address: 8108
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain105_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_READ_D (AIN#(0:149)_EF_READ_D@105)

        Address: 8110
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain106_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_READ_D (AIN#(0:149)_EF_READ_D@106)

        Address: 8112
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain107_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_READ_D (AIN#(0:149)_EF_READ_D@107)

        Address: 8114
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain108_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_READ_D (AIN#(0:149)_EF_READ_D@108)

        Address: 8116
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain109_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_READ_D (AIN#(0:149)_EF_READ_D@109)

        Address: 8118
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain110_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_READ_D (AIN#(0:149)_EF_READ_D@110)

        Address: 8120
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain111_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_READ_D (AIN#(0:149)_EF_READ_D@111)

        Address: 8122
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain112_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_READ_D (AIN#(0:149)_EF_READ_D@112)

        Address: 8124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain113_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_READ_D (AIN#(0:149)_EF_READ_D@113)

        Address: 8126
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain114_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_READ_D (AIN#(0:149)_EF_READ_D@114)

        Address: 8128
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain115_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_READ_D (AIN#(0:149)_EF_READ_D@115)

        Address: 8130
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain116_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_READ_D (AIN#(0:149)_EF_READ_D@116)

        Address: 8132
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain117_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_READ_D (AIN#(0:149)_EF_READ_D@117)

        Address: 8134
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain118_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_READ_D (AIN#(0:149)_EF_READ_D@118)

        Address: 8136
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain119_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_READ_D (AIN#(0:149)_EF_READ_D@119)

        Address: 8138
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain120_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_READ_D (AIN#(0:149)_EF_READ_D@120)

        Address: 8140
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain121_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_READ_D (AIN#(0:149)_EF_READ_D@121)

        Address: 8142
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain122_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_READ_D (AIN#(0:149)_EF_READ_D@122)

        Address: 8144
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain123_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_READ_D (AIN#(0:149)_EF_READ_D@123)

        Address: 8146
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain124_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_READ_D (AIN#(0:149)_EF_READ_D@124)

        Address: 8148
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain125_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_READ_D (AIN#(0:149)_EF_READ_D@125)

        Address: 8150
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain126_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_READ_D (AIN#(0:149)_EF_READ_D@126)

        Address: 8152
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain127_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_READ_D (AIN#(0:149)_EF_READ_D@127)

        Address: 8154
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain128_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_READ_D (AIN#(0:149)_EF_READ_D@128)

        Address: 8156
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain129_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_READ_D (AIN#(0:149)_EF_READ_D@129)

        Address: 8158
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain130_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_READ_D (AIN#(0:149)_EF_READ_D@130)

        Address: 8160
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain131_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_READ_D (AIN#(0:149)_EF_READ_D@131)

        Address: 8162
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain132_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_READ_D (AIN#(0:149)_EF_READ_D@132)

        Address: 8164
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain133_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_READ_D (AIN#(0:149)_EF_READ_D@133)

        Address: 8166
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain134_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_READ_D (AIN#(0:149)_EF_READ_D@134)

        Address: 8168
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain135_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_READ_D (AIN#(0:149)_EF_READ_D@135)

        Address: 8170
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain136_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_READ_D (AIN#(0:149)_EF_READ_D@136)

        Address: 8172
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain137_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_READ_D (AIN#(0:149)_EF_READ_D@137)

        Address: 8174
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain138_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_READ_D (AIN#(0:149)_EF_READ_D@138)

        Address: 8176
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain139_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_READ_D (AIN#(0:149)_EF_READ_D@139)

        Address: 8178
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain140_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_READ_D (AIN#(0:149)_EF_READ_D@140)

        Address: 8180
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain141_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_READ_D (AIN#(0:149)_EF_READ_D@141)

        Address: 8182
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain142_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_READ_D (AIN#(0:149)_EF_READ_D@142)

        Address: 8184
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain143_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_READ_D (AIN#(0:149)_EF_READ_D@143)

        Address: 8186
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain144_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_READ_D (AIN#(0:149)_EF_READ_D@144)

        Address: 8188
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain145_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_READ_D (AIN#(0:149)_EF_READ_D@145)

        Address: 8190
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain146_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_READ_D (AIN#(0:149)_EF_READ_D@146)

        Address: 8192
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain147_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_READ_D (AIN#(0:149)_EF_READ_D@147)

        Address: 8194
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain148_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_READ_D (AIN#(0:149)_EF_READ_D@148)

        Address: 8196
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain149_ef_read_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_READ_D (AIN#(0:149)_EF_READ_D@149)

        Address: 8198
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_READ_D)  # type: ignore

    @property
    @locked
    def ain0_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN0_EF_INDEX (AIN#(0:149)_EF_INDEX@0)

        Address: 9000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_EF_INDEX)  # type: ignore

    @ain0_ef_index.setter
    @locked
    def ain0_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN0_EF_INDEX (AIN#(0:149)_EF_INDEX@0)

        Address: 9000
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN0_EF_INDEX, value)

    @property
    @locked
    def ain1_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN1_EF_INDEX (AIN#(0:149)_EF_INDEX@1)

        Address: 9002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_EF_INDEX)  # type: ignore

    @ain1_ef_index.setter
    @locked
    def ain1_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN1_EF_INDEX (AIN#(0:149)_EF_INDEX@1)

        Address: 9002
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN1_EF_INDEX, value)

    @property
    @locked
    def ain2_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN2_EF_INDEX (AIN#(0:149)_EF_INDEX@2)

        Address: 9004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_EF_INDEX)  # type: ignore

    @ain2_ef_index.setter
    @locked
    def ain2_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN2_EF_INDEX (AIN#(0:149)_EF_INDEX@2)

        Address: 9004
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN2_EF_INDEX, value)

    @property
    @locked
    def ain3_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN3_EF_INDEX (AIN#(0:149)_EF_INDEX@3)

        Address: 9006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_EF_INDEX)  # type: ignore

    @ain3_ef_index.setter
    @locked
    def ain3_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN3_EF_INDEX (AIN#(0:149)_EF_INDEX@3)

        Address: 9006
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN3_EF_INDEX, value)

    @property
    @locked
    def ain4_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN4_EF_INDEX (AIN#(0:149)_EF_INDEX@4)

        Address: 9008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_EF_INDEX)  # type: ignore

    @ain4_ef_index.setter
    @locked
    def ain4_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN4_EF_INDEX (AIN#(0:149)_EF_INDEX@4)

        Address: 9008
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN4_EF_INDEX, value)

    @property
    @locked
    def ain5_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN5_EF_INDEX (AIN#(0:149)_EF_INDEX@5)

        Address: 9010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_EF_INDEX)  # type: ignore

    @ain5_ef_index.setter
    @locked
    def ain5_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN5_EF_INDEX (AIN#(0:149)_EF_INDEX@5)

        Address: 9010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN5_EF_INDEX, value)

    @property
    @locked
    def ain6_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN6_EF_INDEX (AIN#(0:149)_EF_INDEX@6)

        Address: 9012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_EF_INDEX)  # type: ignore

    @ain6_ef_index.setter
    @locked
    def ain6_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN6_EF_INDEX (AIN#(0:149)_EF_INDEX@6)

        Address: 9012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN6_EF_INDEX, value)

    @property
    @locked
    def ain7_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN7_EF_INDEX (AIN#(0:149)_EF_INDEX@7)

        Address: 9014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_EF_INDEX)  # type: ignore

    @ain7_ef_index.setter
    @locked
    def ain7_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN7_EF_INDEX (AIN#(0:149)_EF_INDEX@7)

        Address: 9014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN7_EF_INDEX, value)

    @property
    @locked
    def ain8_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN8_EF_INDEX (AIN#(0:149)_EF_INDEX@8)

        Address: 9016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN8_EF_INDEX)  # type: ignore

    @ain8_ef_index.setter
    @locked
    def ain8_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN8_EF_INDEX (AIN#(0:149)_EF_INDEX@8)

        Address: 9016
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN8_EF_INDEX, value)

    @property
    @locked
    def ain9_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN9_EF_INDEX (AIN#(0:149)_EF_INDEX@9)

        Address: 9018
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN9_EF_INDEX)  # type: ignore

    @ain9_ef_index.setter
    @locked
    def ain9_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN9_EF_INDEX (AIN#(0:149)_EF_INDEX@9)

        Address: 9018
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN9_EF_INDEX, value)

    @property
    @locked
    def ain10_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN10_EF_INDEX (AIN#(0:149)_EF_INDEX@10)

        Address: 9020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN10_EF_INDEX)  # type: ignore

    @ain10_ef_index.setter
    @locked
    def ain10_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN10_EF_INDEX (AIN#(0:149)_EF_INDEX@10)

        Address: 9020
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN10_EF_INDEX, value)

    @property
    @locked
    def ain11_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN11_EF_INDEX (AIN#(0:149)_EF_INDEX@11)

        Address: 9022
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN11_EF_INDEX)  # type: ignore

    @ain11_ef_index.setter
    @locked
    def ain11_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN11_EF_INDEX (AIN#(0:149)_EF_INDEX@11)

        Address: 9022
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN11_EF_INDEX, value)

    @property
    @locked
    def ain12_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN12_EF_INDEX (AIN#(0:149)_EF_INDEX@12)

        Address: 9024
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN12_EF_INDEX)  # type: ignore

    @ain12_ef_index.setter
    @locked
    def ain12_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN12_EF_INDEX (AIN#(0:149)_EF_INDEX@12)

        Address: 9024
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN12_EF_INDEX, value)

    @property
    @locked
    def ain13_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN13_EF_INDEX (AIN#(0:149)_EF_INDEX@13)

        Address: 9026
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN13_EF_INDEX)  # type: ignore

    @ain13_ef_index.setter
    @locked
    def ain13_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN13_EF_INDEX (AIN#(0:149)_EF_INDEX@13)

        Address: 9026
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN13_EF_INDEX, value)

    @property
    @locked
    def ain14_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN14_EF_INDEX (AIN#(0:149)_EF_INDEX@14)

        Address: 9028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN14_EF_INDEX)  # type: ignore

    @ain14_ef_index.setter
    @locked
    def ain14_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN14_EF_INDEX (AIN#(0:149)_EF_INDEX@14)

        Address: 9028
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN14_EF_INDEX, value)

    @property
    @locked
    def ain15_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN15_EF_INDEX (AIN#(0:149)_EF_INDEX@15)

        Address: 9030
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN15_EF_INDEX)  # type: ignore

    @ain15_ef_index.setter
    @locked
    def ain15_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN15_EF_INDEX (AIN#(0:149)_EF_INDEX@15)

        Address: 9030
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN15_EF_INDEX, value)

    @property
    @locked
    def ain16_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN16_EF_INDEX (AIN#(0:149)_EF_INDEX@16)

        Address: 9032
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN16_EF_INDEX)  # type: ignore

    @ain16_ef_index.setter
    @locked
    def ain16_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN16_EF_INDEX (AIN#(0:149)_EF_INDEX@16)

        Address: 9032
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN16_EF_INDEX, value)

    @property
    @locked
    def ain17_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN17_EF_INDEX (AIN#(0:149)_EF_INDEX@17)

        Address: 9034
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN17_EF_INDEX)  # type: ignore

    @ain17_ef_index.setter
    @locked
    def ain17_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN17_EF_INDEX (AIN#(0:149)_EF_INDEX@17)

        Address: 9034
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN17_EF_INDEX, value)

    @property
    @locked
    def ain18_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN18_EF_INDEX (AIN#(0:149)_EF_INDEX@18)

        Address: 9036
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN18_EF_INDEX)  # type: ignore

    @ain18_ef_index.setter
    @locked
    def ain18_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN18_EF_INDEX (AIN#(0:149)_EF_INDEX@18)

        Address: 9036
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN18_EF_INDEX, value)

    @property
    @locked
    def ain19_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN19_EF_INDEX (AIN#(0:149)_EF_INDEX@19)

        Address: 9038
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN19_EF_INDEX)  # type: ignore

    @ain19_ef_index.setter
    @locked
    def ain19_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN19_EF_INDEX (AIN#(0:149)_EF_INDEX@19)

        Address: 9038
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN19_EF_INDEX, value)

    @property
    @locked
    def ain20_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN20_EF_INDEX (AIN#(0:149)_EF_INDEX@20)

        Address: 9040
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN20_EF_INDEX)  # type: ignore

    @ain20_ef_index.setter
    @locked
    def ain20_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN20_EF_INDEX (AIN#(0:149)_EF_INDEX@20)

        Address: 9040
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN20_EF_INDEX, value)

    @property
    @locked
    def ain21_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN21_EF_INDEX (AIN#(0:149)_EF_INDEX@21)

        Address: 9042
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN21_EF_INDEX)  # type: ignore

    @ain21_ef_index.setter
    @locked
    def ain21_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN21_EF_INDEX (AIN#(0:149)_EF_INDEX@21)

        Address: 9042
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN21_EF_INDEX, value)

    @property
    @locked
    def ain22_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN22_EF_INDEX (AIN#(0:149)_EF_INDEX@22)

        Address: 9044
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN22_EF_INDEX)  # type: ignore

    @ain22_ef_index.setter
    @locked
    def ain22_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN22_EF_INDEX (AIN#(0:149)_EF_INDEX@22)

        Address: 9044
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN22_EF_INDEX, value)

    @property
    @locked
    def ain23_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN23_EF_INDEX (AIN#(0:149)_EF_INDEX@23)

        Address: 9046
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN23_EF_INDEX)  # type: ignore

    @ain23_ef_index.setter
    @locked
    def ain23_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN23_EF_INDEX (AIN#(0:149)_EF_INDEX@23)

        Address: 9046
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN23_EF_INDEX, value)

    @property
    @locked
    def ain24_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN24_EF_INDEX (AIN#(0:149)_EF_INDEX@24)

        Address: 9048
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN24_EF_INDEX)  # type: ignore

    @ain24_ef_index.setter
    @locked
    def ain24_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN24_EF_INDEX (AIN#(0:149)_EF_INDEX@24)

        Address: 9048
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN24_EF_INDEX, value)

    @property
    @locked
    def ain25_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN25_EF_INDEX (AIN#(0:149)_EF_INDEX@25)

        Address: 9050
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN25_EF_INDEX)  # type: ignore

    @ain25_ef_index.setter
    @locked
    def ain25_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN25_EF_INDEX (AIN#(0:149)_EF_INDEX@25)

        Address: 9050
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN25_EF_INDEX, value)

    @property
    @locked
    def ain26_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN26_EF_INDEX (AIN#(0:149)_EF_INDEX@26)

        Address: 9052
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN26_EF_INDEX)  # type: ignore

    @ain26_ef_index.setter
    @locked
    def ain26_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN26_EF_INDEX (AIN#(0:149)_EF_INDEX@26)

        Address: 9052
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN26_EF_INDEX, value)

    @property
    @locked
    def ain27_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN27_EF_INDEX (AIN#(0:149)_EF_INDEX@27)

        Address: 9054
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN27_EF_INDEX)  # type: ignore

    @ain27_ef_index.setter
    @locked
    def ain27_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN27_EF_INDEX (AIN#(0:149)_EF_INDEX@27)

        Address: 9054
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN27_EF_INDEX, value)

    @property
    @locked
    def ain28_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN28_EF_INDEX (AIN#(0:149)_EF_INDEX@28)

        Address: 9056
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN28_EF_INDEX)  # type: ignore

    @ain28_ef_index.setter
    @locked
    def ain28_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN28_EF_INDEX (AIN#(0:149)_EF_INDEX@28)

        Address: 9056
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN28_EF_INDEX, value)

    @property
    @locked
    def ain29_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN29_EF_INDEX (AIN#(0:149)_EF_INDEX@29)

        Address: 9058
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN29_EF_INDEX)  # type: ignore

    @ain29_ef_index.setter
    @locked
    def ain29_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN29_EF_INDEX (AIN#(0:149)_EF_INDEX@29)

        Address: 9058
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN29_EF_INDEX, value)

    @property
    @locked
    def ain30_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN30_EF_INDEX (AIN#(0:149)_EF_INDEX@30)

        Address: 9060
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN30_EF_INDEX)  # type: ignore

    @ain30_ef_index.setter
    @locked
    def ain30_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN30_EF_INDEX (AIN#(0:149)_EF_INDEX@30)

        Address: 9060
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN30_EF_INDEX, value)

    @property
    @locked
    def ain31_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN31_EF_INDEX (AIN#(0:149)_EF_INDEX@31)

        Address: 9062
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN31_EF_INDEX)  # type: ignore

    @ain31_ef_index.setter
    @locked
    def ain31_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN31_EF_INDEX (AIN#(0:149)_EF_INDEX@31)

        Address: 9062
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN31_EF_INDEX, value)

    @property
    @locked
    def ain32_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN32_EF_INDEX (AIN#(0:149)_EF_INDEX@32)

        Address: 9064
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN32_EF_INDEX)  # type: ignore

    @ain32_ef_index.setter
    @locked
    def ain32_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN32_EF_INDEX (AIN#(0:149)_EF_INDEX@32)

        Address: 9064
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN32_EF_INDEX, value)

    @property
    @locked
    def ain33_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN33_EF_INDEX (AIN#(0:149)_EF_INDEX@33)

        Address: 9066
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN33_EF_INDEX)  # type: ignore

    @ain33_ef_index.setter
    @locked
    def ain33_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN33_EF_INDEX (AIN#(0:149)_EF_INDEX@33)

        Address: 9066
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN33_EF_INDEX, value)

    @property
    @locked
    def ain34_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN34_EF_INDEX (AIN#(0:149)_EF_INDEX@34)

        Address: 9068
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN34_EF_INDEX)  # type: ignore

    @ain34_ef_index.setter
    @locked
    def ain34_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN34_EF_INDEX (AIN#(0:149)_EF_INDEX@34)

        Address: 9068
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN34_EF_INDEX, value)

    @property
    @locked
    def ain35_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN35_EF_INDEX (AIN#(0:149)_EF_INDEX@35)

        Address: 9070
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN35_EF_INDEX)  # type: ignore

    @ain35_ef_index.setter
    @locked
    def ain35_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN35_EF_INDEX (AIN#(0:149)_EF_INDEX@35)

        Address: 9070
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN35_EF_INDEX, value)

    @property
    @locked
    def ain36_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN36_EF_INDEX (AIN#(0:149)_EF_INDEX@36)

        Address: 9072
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN36_EF_INDEX)  # type: ignore

    @ain36_ef_index.setter
    @locked
    def ain36_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN36_EF_INDEX (AIN#(0:149)_EF_INDEX@36)

        Address: 9072
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN36_EF_INDEX, value)

    @property
    @locked
    def ain37_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN37_EF_INDEX (AIN#(0:149)_EF_INDEX@37)

        Address: 9074
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN37_EF_INDEX)  # type: ignore

    @ain37_ef_index.setter
    @locked
    def ain37_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN37_EF_INDEX (AIN#(0:149)_EF_INDEX@37)

        Address: 9074
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN37_EF_INDEX, value)

    @property
    @locked
    def ain38_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN38_EF_INDEX (AIN#(0:149)_EF_INDEX@38)

        Address: 9076
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN38_EF_INDEX)  # type: ignore

    @ain38_ef_index.setter
    @locked
    def ain38_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN38_EF_INDEX (AIN#(0:149)_EF_INDEX@38)

        Address: 9076
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN38_EF_INDEX, value)

    @property
    @locked
    def ain39_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN39_EF_INDEX (AIN#(0:149)_EF_INDEX@39)

        Address: 9078
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN39_EF_INDEX)  # type: ignore

    @ain39_ef_index.setter
    @locked
    def ain39_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN39_EF_INDEX (AIN#(0:149)_EF_INDEX@39)

        Address: 9078
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN39_EF_INDEX, value)

    @property
    @locked
    def ain40_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN40_EF_INDEX (AIN#(0:149)_EF_INDEX@40)

        Address: 9080
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN40_EF_INDEX)  # type: ignore

    @ain40_ef_index.setter
    @locked
    def ain40_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN40_EF_INDEX (AIN#(0:149)_EF_INDEX@40)

        Address: 9080
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN40_EF_INDEX, value)

    @property
    @locked
    def ain41_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN41_EF_INDEX (AIN#(0:149)_EF_INDEX@41)

        Address: 9082
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN41_EF_INDEX)  # type: ignore

    @ain41_ef_index.setter
    @locked
    def ain41_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN41_EF_INDEX (AIN#(0:149)_EF_INDEX@41)

        Address: 9082
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN41_EF_INDEX, value)

    @property
    @locked
    def ain42_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN42_EF_INDEX (AIN#(0:149)_EF_INDEX@42)

        Address: 9084
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN42_EF_INDEX)  # type: ignore

    @ain42_ef_index.setter
    @locked
    def ain42_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN42_EF_INDEX (AIN#(0:149)_EF_INDEX@42)

        Address: 9084
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN42_EF_INDEX, value)

    @property
    @locked
    def ain43_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN43_EF_INDEX (AIN#(0:149)_EF_INDEX@43)

        Address: 9086
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN43_EF_INDEX)  # type: ignore

    @ain43_ef_index.setter
    @locked
    def ain43_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN43_EF_INDEX (AIN#(0:149)_EF_INDEX@43)

        Address: 9086
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN43_EF_INDEX, value)

    @property
    @locked
    def ain44_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN44_EF_INDEX (AIN#(0:149)_EF_INDEX@44)

        Address: 9088
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN44_EF_INDEX)  # type: ignore

    @ain44_ef_index.setter
    @locked
    def ain44_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN44_EF_INDEX (AIN#(0:149)_EF_INDEX@44)

        Address: 9088
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN44_EF_INDEX, value)

    @property
    @locked
    def ain45_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN45_EF_INDEX (AIN#(0:149)_EF_INDEX@45)

        Address: 9090
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN45_EF_INDEX)  # type: ignore

    @ain45_ef_index.setter
    @locked
    def ain45_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN45_EF_INDEX (AIN#(0:149)_EF_INDEX@45)

        Address: 9090
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN45_EF_INDEX, value)

    @property
    @locked
    def ain46_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN46_EF_INDEX (AIN#(0:149)_EF_INDEX@46)

        Address: 9092
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN46_EF_INDEX)  # type: ignore

    @ain46_ef_index.setter
    @locked
    def ain46_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN46_EF_INDEX (AIN#(0:149)_EF_INDEX@46)

        Address: 9092
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN46_EF_INDEX, value)

    @property
    @locked
    def ain47_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN47_EF_INDEX (AIN#(0:149)_EF_INDEX@47)

        Address: 9094
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN47_EF_INDEX)  # type: ignore

    @ain47_ef_index.setter
    @locked
    def ain47_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN47_EF_INDEX (AIN#(0:149)_EF_INDEX@47)

        Address: 9094
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN47_EF_INDEX, value)

    @property
    @locked
    def ain48_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN48_EF_INDEX (AIN#(0:149)_EF_INDEX@48)

        Address: 9096
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN48_EF_INDEX)  # type: ignore

    @ain48_ef_index.setter
    @locked
    def ain48_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN48_EF_INDEX (AIN#(0:149)_EF_INDEX@48)

        Address: 9096
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN48_EF_INDEX, value)

    @property
    @locked
    def ain49_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN49_EF_INDEX (AIN#(0:149)_EF_INDEX@49)

        Address: 9098
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN49_EF_INDEX)  # type: ignore

    @ain49_ef_index.setter
    @locked
    def ain49_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN49_EF_INDEX (AIN#(0:149)_EF_INDEX@49)

        Address: 9098
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN49_EF_INDEX, value)

    @property
    @locked
    def ain50_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN50_EF_INDEX (AIN#(0:149)_EF_INDEX@50)

        Address: 9100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN50_EF_INDEX)  # type: ignore

    @ain50_ef_index.setter
    @locked
    def ain50_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN50_EF_INDEX (AIN#(0:149)_EF_INDEX@50)

        Address: 9100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN50_EF_INDEX, value)

    @property
    @locked
    def ain51_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN51_EF_INDEX (AIN#(0:149)_EF_INDEX@51)

        Address: 9102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN51_EF_INDEX)  # type: ignore

    @ain51_ef_index.setter
    @locked
    def ain51_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN51_EF_INDEX (AIN#(0:149)_EF_INDEX@51)

        Address: 9102
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN51_EF_INDEX, value)

    @property
    @locked
    def ain52_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN52_EF_INDEX (AIN#(0:149)_EF_INDEX@52)

        Address: 9104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN52_EF_INDEX)  # type: ignore

    @ain52_ef_index.setter
    @locked
    def ain52_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN52_EF_INDEX (AIN#(0:149)_EF_INDEX@52)

        Address: 9104
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN52_EF_INDEX, value)

    @property
    @locked
    def ain53_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN53_EF_INDEX (AIN#(0:149)_EF_INDEX@53)

        Address: 9106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN53_EF_INDEX)  # type: ignore

    @ain53_ef_index.setter
    @locked
    def ain53_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN53_EF_INDEX (AIN#(0:149)_EF_INDEX@53)

        Address: 9106
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN53_EF_INDEX, value)

    @property
    @locked
    def ain54_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN54_EF_INDEX (AIN#(0:149)_EF_INDEX@54)

        Address: 9108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN54_EF_INDEX)  # type: ignore

    @ain54_ef_index.setter
    @locked
    def ain54_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN54_EF_INDEX (AIN#(0:149)_EF_INDEX@54)

        Address: 9108
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN54_EF_INDEX, value)

    @property
    @locked
    def ain55_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN55_EF_INDEX (AIN#(0:149)_EF_INDEX@55)

        Address: 9110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN55_EF_INDEX)  # type: ignore

    @ain55_ef_index.setter
    @locked
    def ain55_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN55_EF_INDEX (AIN#(0:149)_EF_INDEX@55)

        Address: 9110
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN55_EF_INDEX, value)

    @property
    @locked
    def ain56_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN56_EF_INDEX (AIN#(0:149)_EF_INDEX@56)

        Address: 9112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN56_EF_INDEX)  # type: ignore

    @ain56_ef_index.setter
    @locked
    def ain56_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN56_EF_INDEX (AIN#(0:149)_EF_INDEX@56)

        Address: 9112
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN56_EF_INDEX, value)

    @property
    @locked
    def ain57_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN57_EF_INDEX (AIN#(0:149)_EF_INDEX@57)

        Address: 9114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN57_EF_INDEX)  # type: ignore

    @ain57_ef_index.setter
    @locked
    def ain57_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN57_EF_INDEX (AIN#(0:149)_EF_INDEX@57)

        Address: 9114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN57_EF_INDEX, value)

    @property
    @locked
    def ain58_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN58_EF_INDEX (AIN#(0:149)_EF_INDEX@58)

        Address: 9116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN58_EF_INDEX)  # type: ignore

    @ain58_ef_index.setter
    @locked
    def ain58_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN58_EF_INDEX (AIN#(0:149)_EF_INDEX@58)

        Address: 9116
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN58_EF_INDEX, value)

    @property
    @locked
    def ain59_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN59_EF_INDEX (AIN#(0:149)_EF_INDEX@59)

        Address: 9118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN59_EF_INDEX)  # type: ignore

    @ain59_ef_index.setter
    @locked
    def ain59_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN59_EF_INDEX (AIN#(0:149)_EF_INDEX@59)

        Address: 9118
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN59_EF_INDEX, value)

    @property
    @locked
    def ain60_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN60_EF_INDEX (AIN#(0:149)_EF_INDEX@60)

        Address: 9120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN60_EF_INDEX)  # type: ignore

    @ain60_ef_index.setter
    @locked
    def ain60_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN60_EF_INDEX (AIN#(0:149)_EF_INDEX@60)

        Address: 9120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN60_EF_INDEX, value)

    @property
    @locked
    def ain61_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN61_EF_INDEX (AIN#(0:149)_EF_INDEX@61)

        Address: 9122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN61_EF_INDEX)  # type: ignore

    @ain61_ef_index.setter
    @locked
    def ain61_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN61_EF_INDEX (AIN#(0:149)_EF_INDEX@61)

        Address: 9122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN61_EF_INDEX, value)

    @property
    @locked
    def ain62_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN62_EF_INDEX (AIN#(0:149)_EF_INDEX@62)

        Address: 9124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN62_EF_INDEX)  # type: ignore

    @ain62_ef_index.setter
    @locked
    def ain62_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN62_EF_INDEX (AIN#(0:149)_EF_INDEX@62)

        Address: 9124
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN62_EF_INDEX, value)

    @property
    @locked
    def ain63_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN63_EF_INDEX (AIN#(0:149)_EF_INDEX@63)

        Address: 9126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN63_EF_INDEX)  # type: ignore

    @ain63_ef_index.setter
    @locked
    def ain63_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN63_EF_INDEX (AIN#(0:149)_EF_INDEX@63)

        Address: 9126
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN63_EF_INDEX, value)

    @property
    @locked
    def ain64_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN64_EF_INDEX (AIN#(0:149)_EF_INDEX@64)

        Address: 9128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN64_EF_INDEX)  # type: ignore

    @ain64_ef_index.setter
    @locked
    def ain64_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN64_EF_INDEX (AIN#(0:149)_EF_INDEX@64)

        Address: 9128
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN64_EF_INDEX, value)

    @property
    @locked
    def ain65_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN65_EF_INDEX (AIN#(0:149)_EF_INDEX@65)

        Address: 9130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN65_EF_INDEX)  # type: ignore

    @ain65_ef_index.setter
    @locked
    def ain65_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN65_EF_INDEX (AIN#(0:149)_EF_INDEX@65)

        Address: 9130
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN65_EF_INDEX, value)

    @property
    @locked
    def ain66_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN66_EF_INDEX (AIN#(0:149)_EF_INDEX@66)

        Address: 9132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN66_EF_INDEX)  # type: ignore

    @ain66_ef_index.setter
    @locked
    def ain66_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN66_EF_INDEX (AIN#(0:149)_EF_INDEX@66)

        Address: 9132
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN66_EF_INDEX, value)

    @property
    @locked
    def ain67_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN67_EF_INDEX (AIN#(0:149)_EF_INDEX@67)

        Address: 9134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN67_EF_INDEX)  # type: ignore

    @ain67_ef_index.setter
    @locked
    def ain67_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN67_EF_INDEX (AIN#(0:149)_EF_INDEX@67)

        Address: 9134
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN67_EF_INDEX, value)

    @property
    @locked
    def ain68_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN68_EF_INDEX (AIN#(0:149)_EF_INDEX@68)

        Address: 9136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN68_EF_INDEX)  # type: ignore

    @ain68_ef_index.setter
    @locked
    def ain68_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN68_EF_INDEX (AIN#(0:149)_EF_INDEX@68)

        Address: 9136
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN68_EF_INDEX, value)

    @property
    @locked
    def ain69_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN69_EF_INDEX (AIN#(0:149)_EF_INDEX@69)

        Address: 9138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN69_EF_INDEX)  # type: ignore

    @ain69_ef_index.setter
    @locked
    def ain69_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN69_EF_INDEX (AIN#(0:149)_EF_INDEX@69)

        Address: 9138
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN69_EF_INDEX, value)

    @property
    @locked
    def ain70_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN70_EF_INDEX (AIN#(0:149)_EF_INDEX@70)

        Address: 9140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN70_EF_INDEX)  # type: ignore

    @ain70_ef_index.setter
    @locked
    def ain70_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN70_EF_INDEX (AIN#(0:149)_EF_INDEX@70)

        Address: 9140
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN70_EF_INDEX, value)

    @property
    @locked
    def ain71_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN71_EF_INDEX (AIN#(0:149)_EF_INDEX@71)

        Address: 9142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN71_EF_INDEX)  # type: ignore

    @ain71_ef_index.setter
    @locked
    def ain71_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN71_EF_INDEX (AIN#(0:149)_EF_INDEX@71)

        Address: 9142
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN71_EF_INDEX, value)

    @property
    @locked
    def ain72_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN72_EF_INDEX (AIN#(0:149)_EF_INDEX@72)

        Address: 9144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN72_EF_INDEX)  # type: ignore

    @ain72_ef_index.setter
    @locked
    def ain72_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN72_EF_INDEX (AIN#(0:149)_EF_INDEX@72)

        Address: 9144
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN72_EF_INDEX, value)

    @property
    @locked
    def ain73_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN73_EF_INDEX (AIN#(0:149)_EF_INDEX@73)

        Address: 9146
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN73_EF_INDEX)  # type: ignore

    @ain73_ef_index.setter
    @locked
    def ain73_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN73_EF_INDEX (AIN#(0:149)_EF_INDEX@73)

        Address: 9146
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN73_EF_INDEX, value)

    @property
    @locked
    def ain74_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN74_EF_INDEX (AIN#(0:149)_EF_INDEX@74)

        Address: 9148
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN74_EF_INDEX)  # type: ignore

    @ain74_ef_index.setter
    @locked
    def ain74_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN74_EF_INDEX (AIN#(0:149)_EF_INDEX@74)

        Address: 9148
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN74_EF_INDEX, value)

    @property
    @locked
    def ain75_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN75_EF_INDEX (AIN#(0:149)_EF_INDEX@75)

        Address: 9150
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN75_EF_INDEX)  # type: ignore

    @ain75_ef_index.setter
    @locked
    def ain75_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN75_EF_INDEX (AIN#(0:149)_EF_INDEX@75)

        Address: 9150
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN75_EF_INDEX, value)

    @property
    @locked
    def ain76_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN76_EF_INDEX (AIN#(0:149)_EF_INDEX@76)

        Address: 9152
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN76_EF_INDEX)  # type: ignore

    @ain76_ef_index.setter
    @locked
    def ain76_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN76_EF_INDEX (AIN#(0:149)_EF_INDEX@76)

        Address: 9152
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN76_EF_INDEX, value)

    @property
    @locked
    def ain77_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN77_EF_INDEX (AIN#(0:149)_EF_INDEX@77)

        Address: 9154
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN77_EF_INDEX)  # type: ignore

    @ain77_ef_index.setter
    @locked
    def ain77_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN77_EF_INDEX (AIN#(0:149)_EF_INDEX@77)

        Address: 9154
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN77_EF_INDEX, value)

    @property
    @locked
    def ain78_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN78_EF_INDEX (AIN#(0:149)_EF_INDEX@78)

        Address: 9156
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN78_EF_INDEX)  # type: ignore

    @ain78_ef_index.setter
    @locked
    def ain78_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN78_EF_INDEX (AIN#(0:149)_EF_INDEX@78)

        Address: 9156
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN78_EF_INDEX, value)

    @property
    @locked
    def ain79_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN79_EF_INDEX (AIN#(0:149)_EF_INDEX@79)

        Address: 9158
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN79_EF_INDEX)  # type: ignore

    @ain79_ef_index.setter
    @locked
    def ain79_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN79_EF_INDEX (AIN#(0:149)_EF_INDEX@79)

        Address: 9158
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN79_EF_INDEX, value)

    @property
    @locked
    def ain80_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN80_EF_INDEX (AIN#(0:149)_EF_INDEX@80)

        Address: 9160
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN80_EF_INDEX)  # type: ignore

    @ain80_ef_index.setter
    @locked
    def ain80_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN80_EF_INDEX (AIN#(0:149)_EF_INDEX@80)

        Address: 9160
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN80_EF_INDEX, value)

    @property
    @locked
    def ain81_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN81_EF_INDEX (AIN#(0:149)_EF_INDEX@81)

        Address: 9162
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN81_EF_INDEX)  # type: ignore

    @ain81_ef_index.setter
    @locked
    def ain81_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN81_EF_INDEX (AIN#(0:149)_EF_INDEX@81)

        Address: 9162
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN81_EF_INDEX, value)

    @property
    @locked
    def ain82_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN82_EF_INDEX (AIN#(0:149)_EF_INDEX@82)

        Address: 9164
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN82_EF_INDEX)  # type: ignore

    @ain82_ef_index.setter
    @locked
    def ain82_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN82_EF_INDEX (AIN#(0:149)_EF_INDEX@82)

        Address: 9164
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN82_EF_INDEX, value)

    @property
    @locked
    def ain83_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN83_EF_INDEX (AIN#(0:149)_EF_INDEX@83)

        Address: 9166
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN83_EF_INDEX)  # type: ignore

    @ain83_ef_index.setter
    @locked
    def ain83_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN83_EF_INDEX (AIN#(0:149)_EF_INDEX@83)

        Address: 9166
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN83_EF_INDEX, value)

    @property
    @locked
    def ain84_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN84_EF_INDEX (AIN#(0:149)_EF_INDEX@84)

        Address: 9168
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN84_EF_INDEX)  # type: ignore

    @ain84_ef_index.setter
    @locked
    def ain84_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN84_EF_INDEX (AIN#(0:149)_EF_INDEX@84)

        Address: 9168
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN84_EF_INDEX, value)

    @property
    @locked
    def ain85_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN85_EF_INDEX (AIN#(0:149)_EF_INDEX@85)

        Address: 9170
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN85_EF_INDEX)  # type: ignore

    @ain85_ef_index.setter
    @locked
    def ain85_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN85_EF_INDEX (AIN#(0:149)_EF_INDEX@85)

        Address: 9170
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN85_EF_INDEX, value)

    @property
    @locked
    def ain86_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN86_EF_INDEX (AIN#(0:149)_EF_INDEX@86)

        Address: 9172
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN86_EF_INDEX)  # type: ignore

    @ain86_ef_index.setter
    @locked
    def ain86_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN86_EF_INDEX (AIN#(0:149)_EF_INDEX@86)

        Address: 9172
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN86_EF_INDEX, value)

    @property
    @locked
    def ain87_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN87_EF_INDEX (AIN#(0:149)_EF_INDEX@87)

        Address: 9174
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN87_EF_INDEX)  # type: ignore

    @ain87_ef_index.setter
    @locked
    def ain87_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN87_EF_INDEX (AIN#(0:149)_EF_INDEX@87)

        Address: 9174
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN87_EF_INDEX, value)

    @property
    @locked
    def ain88_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN88_EF_INDEX (AIN#(0:149)_EF_INDEX@88)

        Address: 9176
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN88_EF_INDEX)  # type: ignore

    @ain88_ef_index.setter
    @locked
    def ain88_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN88_EF_INDEX (AIN#(0:149)_EF_INDEX@88)

        Address: 9176
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN88_EF_INDEX, value)

    @property
    @locked
    def ain89_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN89_EF_INDEX (AIN#(0:149)_EF_INDEX@89)

        Address: 9178
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN89_EF_INDEX)  # type: ignore

    @ain89_ef_index.setter
    @locked
    def ain89_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN89_EF_INDEX (AIN#(0:149)_EF_INDEX@89)

        Address: 9178
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN89_EF_INDEX, value)

    @property
    @locked
    def ain90_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN90_EF_INDEX (AIN#(0:149)_EF_INDEX@90)

        Address: 9180
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN90_EF_INDEX)  # type: ignore

    @ain90_ef_index.setter
    @locked
    def ain90_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN90_EF_INDEX (AIN#(0:149)_EF_INDEX@90)

        Address: 9180
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN90_EF_INDEX, value)

    @property
    @locked
    def ain91_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN91_EF_INDEX (AIN#(0:149)_EF_INDEX@91)

        Address: 9182
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN91_EF_INDEX)  # type: ignore

    @ain91_ef_index.setter
    @locked
    def ain91_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN91_EF_INDEX (AIN#(0:149)_EF_INDEX@91)

        Address: 9182
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN91_EF_INDEX, value)

    @property
    @locked
    def ain92_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN92_EF_INDEX (AIN#(0:149)_EF_INDEX@92)

        Address: 9184
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN92_EF_INDEX)  # type: ignore

    @ain92_ef_index.setter
    @locked
    def ain92_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN92_EF_INDEX (AIN#(0:149)_EF_INDEX@92)

        Address: 9184
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN92_EF_INDEX, value)

    @property
    @locked
    def ain93_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN93_EF_INDEX (AIN#(0:149)_EF_INDEX@93)

        Address: 9186
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN93_EF_INDEX)  # type: ignore

    @ain93_ef_index.setter
    @locked
    def ain93_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN93_EF_INDEX (AIN#(0:149)_EF_INDEX@93)

        Address: 9186
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN93_EF_INDEX, value)

    @property
    @locked
    def ain94_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN94_EF_INDEX (AIN#(0:149)_EF_INDEX@94)

        Address: 9188
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN94_EF_INDEX)  # type: ignore

    @ain94_ef_index.setter
    @locked
    def ain94_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN94_EF_INDEX (AIN#(0:149)_EF_INDEX@94)

        Address: 9188
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN94_EF_INDEX, value)

    @property
    @locked
    def ain95_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN95_EF_INDEX (AIN#(0:149)_EF_INDEX@95)

        Address: 9190
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN95_EF_INDEX)  # type: ignore

    @ain95_ef_index.setter
    @locked
    def ain95_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN95_EF_INDEX (AIN#(0:149)_EF_INDEX@95)

        Address: 9190
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN95_EF_INDEX, value)

    @property
    @locked
    def ain96_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN96_EF_INDEX (AIN#(0:149)_EF_INDEX@96)

        Address: 9192
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN96_EF_INDEX)  # type: ignore

    @ain96_ef_index.setter
    @locked
    def ain96_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN96_EF_INDEX (AIN#(0:149)_EF_INDEX@96)

        Address: 9192
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN96_EF_INDEX, value)

    @property
    @locked
    def ain97_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN97_EF_INDEX (AIN#(0:149)_EF_INDEX@97)

        Address: 9194
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN97_EF_INDEX)  # type: ignore

    @ain97_ef_index.setter
    @locked
    def ain97_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN97_EF_INDEX (AIN#(0:149)_EF_INDEX@97)

        Address: 9194
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN97_EF_INDEX, value)

    @property
    @locked
    def ain98_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN98_EF_INDEX (AIN#(0:149)_EF_INDEX@98)

        Address: 9196
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN98_EF_INDEX)  # type: ignore

    @ain98_ef_index.setter
    @locked
    def ain98_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN98_EF_INDEX (AIN#(0:149)_EF_INDEX@98)

        Address: 9196
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN98_EF_INDEX, value)

    @property
    @locked
    def ain99_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN99_EF_INDEX (AIN#(0:149)_EF_INDEX@99)

        Address: 9198
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN99_EF_INDEX)  # type: ignore

    @ain99_ef_index.setter
    @locked
    def ain99_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN99_EF_INDEX (AIN#(0:149)_EF_INDEX@99)

        Address: 9198
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN99_EF_INDEX, value)

    @property
    @locked
    def ain100_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN100_EF_INDEX (AIN#(0:149)_EF_INDEX@100)

        Address: 9200
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN100_EF_INDEX)  # type: ignore

    @ain100_ef_index.setter
    @locked
    def ain100_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN100_EF_INDEX (AIN#(0:149)_EF_INDEX@100)

        Address: 9200
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN100_EF_INDEX, value)

    @property
    @locked
    def ain101_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN101_EF_INDEX (AIN#(0:149)_EF_INDEX@101)

        Address: 9202
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN101_EF_INDEX)  # type: ignore

    @ain101_ef_index.setter
    @locked
    def ain101_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN101_EF_INDEX (AIN#(0:149)_EF_INDEX@101)

        Address: 9202
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN101_EF_INDEX, value)

    @property
    @locked
    def ain102_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN102_EF_INDEX (AIN#(0:149)_EF_INDEX@102)

        Address: 9204
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN102_EF_INDEX)  # type: ignore

    @ain102_ef_index.setter
    @locked
    def ain102_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN102_EF_INDEX (AIN#(0:149)_EF_INDEX@102)

        Address: 9204
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN102_EF_INDEX, value)

    @property
    @locked
    def ain103_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN103_EF_INDEX (AIN#(0:149)_EF_INDEX@103)

        Address: 9206
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN103_EF_INDEX)  # type: ignore

    @ain103_ef_index.setter
    @locked
    def ain103_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN103_EF_INDEX (AIN#(0:149)_EF_INDEX@103)

        Address: 9206
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN103_EF_INDEX, value)

    @property
    @locked
    def ain104_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN104_EF_INDEX (AIN#(0:149)_EF_INDEX@104)

        Address: 9208
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN104_EF_INDEX)  # type: ignore

    @ain104_ef_index.setter
    @locked
    def ain104_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN104_EF_INDEX (AIN#(0:149)_EF_INDEX@104)

        Address: 9208
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN104_EF_INDEX, value)

    @property
    @locked
    def ain105_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN105_EF_INDEX (AIN#(0:149)_EF_INDEX@105)

        Address: 9210
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN105_EF_INDEX)  # type: ignore

    @ain105_ef_index.setter
    @locked
    def ain105_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN105_EF_INDEX (AIN#(0:149)_EF_INDEX@105)

        Address: 9210
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN105_EF_INDEX, value)

    @property
    @locked
    def ain106_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN106_EF_INDEX (AIN#(0:149)_EF_INDEX@106)

        Address: 9212
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN106_EF_INDEX)  # type: ignore

    @ain106_ef_index.setter
    @locked
    def ain106_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN106_EF_INDEX (AIN#(0:149)_EF_INDEX@106)

        Address: 9212
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN106_EF_INDEX, value)

    @property
    @locked
    def ain107_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN107_EF_INDEX (AIN#(0:149)_EF_INDEX@107)

        Address: 9214
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN107_EF_INDEX)  # type: ignore

    @ain107_ef_index.setter
    @locked
    def ain107_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN107_EF_INDEX (AIN#(0:149)_EF_INDEX@107)

        Address: 9214
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN107_EF_INDEX, value)

    @property
    @locked
    def ain108_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN108_EF_INDEX (AIN#(0:149)_EF_INDEX@108)

        Address: 9216
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN108_EF_INDEX)  # type: ignore

    @ain108_ef_index.setter
    @locked
    def ain108_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN108_EF_INDEX (AIN#(0:149)_EF_INDEX@108)

        Address: 9216
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN108_EF_INDEX, value)

    @property
    @locked
    def ain109_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN109_EF_INDEX (AIN#(0:149)_EF_INDEX@109)

        Address: 9218
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN109_EF_INDEX)  # type: ignore

    @ain109_ef_index.setter
    @locked
    def ain109_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN109_EF_INDEX (AIN#(0:149)_EF_INDEX@109)

        Address: 9218
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN109_EF_INDEX, value)

    @property
    @locked
    def ain110_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN110_EF_INDEX (AIN#(0:149)_EF_INDEX@110)

        Address: 9220
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN110_EF_INDEX)  # type: ignore

    @ain110_ef_index.setter
    @locked
    def ain110_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN110_EF_INDEX (AIN#(0:149)_EF_INDEX@110)

        Address: 9220
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN110_EF_INDEX, value)

    @property
    @locked
    def ain111_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN111_EF_INDEX (AIN#(0:149)_EF_INDEX@111)

        Address: 9222
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN111_EF_INDEX)  # type: ignore

    @ain111_ef_index.setter
    @locked
    def ain111_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN111_EF_INDEX (AIN#(0:149)_EF_INDEX@111)

        Address: 9222
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN111_EF_INDEX, value)

    @property
    @locked
    def ain112_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN112_EF_INDEX (AIN#(0:149)_EF_INDEX@112)

        Address: 9224
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN112_EF_INDEX)  # type: ignore

    @ain112_ef_index.setter
    @locked
    def ain112_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN112_EF_INDEX (AIN#(0:149)_EF_INDEX@112)

        Address: 9224
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN112_EF_INDEX, value)

    @property
    @locked
    def ain113_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN113_EF_INDEX (AIN#(0:149)_EF_INDEX@113)

        Address: 9226
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN113_EF_INDEX)  # type: ignore

    @ain113_ef_index.setter
    @locked
    def ain113_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN113_EF_INDEX (AIN#(0:149)_EF_INDEX@113)

        Address: 9226
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN113_EF_INDEX, value)

    @property
    @locked
    def ain114_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN114_EF_INDEX (AIN#(0:149)_EF_INDEX@114)

        Address: 9228
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN114_EF_INDEX)  # type: ignore

    @ain114_ef_index.setter
    @locked
    def ain114_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN114_EF_INDEX (AIN#(0:149)_EF_INDEX@114)

        Address: 9228
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN114_EF_INDEX, value)

    @property
    @locked
    def ain115_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN115_EF_INDEX (AIN#(0:149)_EF_INDEX@115)

        Address: 9230
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN115_EF_INDEX)  # type: ignore

    @ain115_ef_index.setter
    @locked
    def ain115_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN115_EF_INDEX (AIN#(0:149)_EF_INDEX@115)

        Address: 9230
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN115_EF_INDEX, value)

    @property
    @locked
    def ain116_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN116_EF_INDEX (AIN#(0:149)_EF_INDEX@116)

        Address: 9232
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN116_EF_INDEX)  # type: ignore

    @ain116_ef_index.setter
    @locked
    def ain116_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN116_EF_INDEX (AIN#(0:149)_EF_INDEX@116)

        Address: 9232
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN116_EF_INDEX, value)

    @property
    @locked
    def ain117_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN117_EF_INDEX (AIN#(0:149)_EF_INDEX@117)

        Address: 9234
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN117_EF_INDEX)  # type: ignore

    @ain117_ef_index.setter
    @locked
    def ain117_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN117_EF_INDEX (AIN#(0:149)_EF_INDEX@117)

        Address: 9234
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN117_EF_INDEX, value)

    @property
    @locked
    def ain118_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN118_EF_INDEX (AIN#(0:149)_EF_INDEX@118)

        Address: 9236
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN118_EF_INDEX)  # type: ignore

    @ain118_ef_index.setter
    @locked
    def ain118_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN118_EF_INDEX (AIN#(0:149)_EF_INDEX@118)

        Address: 9236
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN118_EF_INDEX, value)

    @property
    @locked
    def ain119_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN119_EF_INDEX (AIN#(0:149)_EF_INDEX@119)

        Address: 9238
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN119_EF_INDEX)  # type: ignore

    @ain119_ef_index.setter
    @locked
    def ain119_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN119_EF_INDEX (AIN#(0:149)_EF_INDEX@119)

        Address: 9238
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN119_EF_INDEX, value)

    @property
    @locked
    def ain120_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN120_EF_INDEX (AIN#(0:149)_EF_INDEX@120)

        Address: 9240
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN120_EF_INDEX)  # type: ignore

    @ain120_ef_index.setter
    @locked
    def ain120_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN120_EF_INDEX (AIN#(0:149)_EF_INDEX@120)

        Address: 9240
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN120_EF_INDEX, value)

    @property
    @locked
    def ain121_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN121_EF_INDEX (AIN#(0:149)_EF_INDEX@121)

        Address: 9242
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN121_EF_INDEX)  # type: ignore

    @ain121_ef_index.setter
    @locked
    def ain121_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN121_EF_INDEX (AIN#(0:149)_EF_INDEX@121)

        Address: 9242
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN121_EF_INDEX, value)

    @property
    @locked
    def ain122_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN122_EF_INDEX (AIN#(0:149)_EF_INDEX@122)

        Address: 9244
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN122_EF_INDEX)  # type: ignore

    @ain122_ef_index.setter
    @locked
    def ain122_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN122_EF_INDEX (AIN#(0:149)_EF_INDEX@122)

        Address: 9244
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN122_EF_INDEX, value)

    @property
    @locked
    def ain123_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN123_EF_INDEX (AIN#(0:149)_EF_INDEX@123)

        Address: 9246
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN123_EF_INDEX)  # type: ignore

    @ain123_ef_index.setter
    @locked
    def ain123_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN123_EF_INDEX (AIN#(0:149)_EF_INDEX@123)

        Address: 9246
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN123_EF_INDEX, value)

    @property
    @locked
    def ain124_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN124_EF_INDEX (AIN#(0:149)_EF_INDEX@124)

        Address: 9248
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN124_EF_INDEX)  # type: ignore

    @ain124_ef_index.setter
    @locked
    def ain124_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN124_EF_INDEX (AIN#(0:149)_EF_INDEX@124)

        Address: 9248
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN124_EF_INDEX, value)

    @property
    @locked
    def ain125_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN125_EF_INDEX (AIN#(0:149)_EF_INDEX@125)

        Address: 9250
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN125_EF_INDEX)  # type: ignore

    @ain125_ef_index.setter
    @locked
    def ain125_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN125_EF_INDEX (AIN#(0:149)_EF_INDEX@125)

        Address: 9250
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN125_EF_INDEX, value)

    @property
    @locked
    def ain126_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN126_EF_INDEX (AIN#(0:149)_EF_INDEX@126)

        Address: 9252
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN126_EF_INDEX)  # type: ignore

    @ain126_ef_index.setter
    @locked
    def ain126_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN126_EF_INDEX (AIN#(0:149)_EF_INDEX@126)

        Address: 9252
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN126_EF_INDEX, value)

    @property
    @locked
    def ain127_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN127_EF_INDEX (AIN#(0:149)_EF_INDEX@127)

        Address: 9254
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN127_EF_INDEX)  # type: ignore

    @ain127_ef_index.setter
    @locked
    def ain127_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN127_EF_INDEX (AIN#(0:149)_EF_INDEX@127)

        Address: 9254
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN127_EF_INDEX, value)

    @property
    @locked
    def ain128_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN128_EF_INDEX (AIN#(0:149)_EF_INDEX@128)

        Address: 9256
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN128_EF_INDEX)  # type: ignore

    @ain128_ef_index.setter
    @locked
    def ain128_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN128_EF_INDEX (AIN#(0:149)_EF_INDEX@128)

        Address: 9256
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN128_EF_INDEX, value)

    @property
    @locked
    def ain129_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN129_EF_INDEX (AIN#(0:149)_EF_INDEX@129)

        Address: 9258
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN129_EF_INDEX)  # type: ignore

    @ain129_ef_index.setter
    @locked
    def ain129_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN129_EF_INDEX (AIN#(0:149)_EF_INDEX@129)

        Address: 9258
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN129_EF_INDEX, value)

    @property
    @locked
    def ain130_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN130_EF_INDEX (AIN#(0:149)_EF_INDEX@130)

        Address: 9260
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN130_EF_INDEX)  # type: ignore

    @ain130_ef_index.setter
    @locked
    def ain130_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN130_EF_INDEX (AIN#(0:149)_EF_INDEX@130)

        Address: 9260
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN130_EF_INDEX, value)

    @property
    @locked
    def ain131_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN131_EF_INDEX (AIN#(0:149)_EF_INDEX@131)

        Address: 9262
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN131_EF_INDEX)  # type: ignore

    @ain131_ef_index.setter
    @locked
    def ain131_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN131_EF_INDEX (AIN#(0:149)_EF_INDEX@131)

        Address: 9262
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN131_EF_INDEX, value)

    @property
    @locked
    def ain132_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN132_EF_INDEX (AIN#(0:149)_EF_INDEX@132)

        Address: 9264
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN132_EF_INDEX)  # type: ignore

    @ain132_ef_index.setter
    @locked
    def ain132_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN132_EF_INDEX (AIN#(0:149)_EF_INDEX@132)

        Address: 9264
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN132_EF_INDEX, value)

    @property
    @locked
    def ain133_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN133_EF_INDEX (AIN#(0:149)_EF_INDEX@133)

        Address: 9266
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN133_EF_INDEX)  # type: ignore

    @ain133_ef_index.setter
    @locked
    def ain133_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN133_EF_INDEX (AIN#(0:149)_EF_INDEX@133)

        Address: 9266
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN133_EF_INDEX, value)

    @property
    @locked
    def ain134_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN134_EF_INDEX (AIN#(0:149)_EF_INDEX@134)

        Address: 9268
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN134_EF_INDEX)  # type: ignore

    @ain134_ef_index.setter
    @locked
    def ain134_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN134_EF_INDEX (AIN#(0:149)_EF_INDEX@134)

        Address: 9268
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN134_EF_INDEX, value)

    @property
    @locked
    def ain135_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN135_EF_INDEX (AIN#(0:149)_EF_INDEX@135)

        Address: 9270
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN135_EF_INDEX)  # type: ignore

    @ain135_ef_index.setter
    @locked
    def ain135_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN135_EF_INDEX (AIN#(0:149)_EF_INDEX@135)

        Address: 9270
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN135_EF_INDEX, value)

    @property
    @locked
    def ain136_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN136_EF_INDEX (AIN#(0:149)_EF_INDEX@136)

        Address: 9272
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN136_EF_INDEX)  # type: ignore

    @ain136_ef_index.setter
    @locked
    def ain136_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN136_EF_INDEX (AIN#(0:149)_EF_INDEX@136)

        Address: 9272
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN136_EF_INDEX, value)

    @property
    @locked
    def ain137_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN137_EF_INDEX (AIN#(0:149)_EF_INDEX@137)

        Address: 9274
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN137_EF_INDEX)  # type: ignore

    @ain137_ef_index.setter
    @locked
    def ain137_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN137_EF_INDEX (AIN#(0:149)_EF_INDEX@137)

        Address: 9274
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN137_EF_INDEX, value)

    @property
    @locked
    def ain138_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN138_EF_INDEX (AIN#(0:149)_EF_INDEX@138)

        Address: 9276
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN138_EF_INDEX)  # type: ignore

    @ain138_ef_index.setter
    @locked
    def ain138_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN138_EF_INDEX (AIN#(0:149)_EF_INDEX@138)

        Address: 9276
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN138_EF_INDEX, value)

    @property
    @locked
    def ain139_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN139_EF_INDEX (AIN#(0:149)_EF_INDEX@139)

        Address: 9278
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN139_EF_INDEX)  # type: ignore

    @ain139_ef_index.setter
    @locked
    def ain139_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN139_EF_INDEX (AIN#(0:149)_EF_INDEX@139)

        Address: 9278
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN139_EF_INDEX, value)

    @property
    @locked
    def ain140_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN140_EF_INDEX (AIN#(0:149)_EF_INDEX@140)

        Address: 9280
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN140_EF_INDEX)  # type: ignore

    @ain140_ef_index.setter
    @locked
    def ain140_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN140_EF_INDEX (AIN#(0:149)_EF_INDEX@140)

        Address: 9280
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN140_EF_INDEX, value)

    @property
    @locked
    def ain141_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN141_EF_INDEX (AIN#(0:149)_EF_INDEX@141)

        Address: 9282
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN141_EF_INDEX)  # type: ignore

    @ain141_ef_index.setter
    @locked
    def ain141_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN141_EF_INDEX (AIN#(0:149)_EF_INDEX@141)

        Address: 9282
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN141_EF_INDEX, value)

    @property
    @locked
    def ain142_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN142_EF_INDEX (AIN#(0:149)_EF_INDEX@142)

        Address: 9284
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN142_EF_INDEX)  # type: ignore

    @ain142_ef_index.setter
    @locked
    def ain142_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN142_EF_INDEX (AIN#(0:149)_EF_INDEX@142)

        Address: 9284
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN142_EF_INDEX, value)

    @property
    @locked
    def ain143_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN143_EF_INDEX (AIN#(0:149)_EF_INDEX@143)

        Address: 9286
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN143_EF_INDEX)  # type: ignore

    @ain143_ef_index.setter
    @locked
    def ain143_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN143_EF_INDEX (AIN#(0:149)_EF_INDEX@143)

        Address: 9286
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN143_EF_INDEX, value)

    @property
    @locked
    def ain144_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN144_EF_INDEX (AIN#(0:149)_EF_INDEX@144)

        Address: 9288
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN144_EF_INDEX)  # type: ignore

    @ain144_ef_index.setter
    @locked
    def ain144_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN144_EF_INDEX (AIN#(0:149)_EF_INDEX@144)

        Address: 9288
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN144_EF_INDEX, value)

    @property
    @locked
    def ain145_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN145_EF_INDEX (AIN#(0:149)_EF_INDEX@145)

        Address: 9290
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN145_EF_INDEX)  # type: ignore

    @ain145_ef_index.setter
    @locked
    def ain145_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN145_EF_INDEX (AIN#(0:149)_EF_INDEX@145)

        Address: 9290
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN145_EF_INDEX, value)

    @property
    @locked
    def ain146_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN146_EF_INDEX (AIN#(0:149)_EF_INDEX@146)

        Address: 9292
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN146_EF_INDEX)  # type: ignore

    @ain146_ef_index.setter
    @locked
    def ain146_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN146_EF_INDEX (AIN#(0:149)_EF_INDEX@146)

        Address: 9292
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN146_EF_INDEX, value)

    @property
    @locked
    def ain147_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN147_EF_INDEX (AIN#(0:149)_EF_INDEX@147)

        Address: 9294
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN147_EF_INDEX)  # type: ignore

    @ain147_ef_index.setter
    @locked
    def ain147_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN147_EF_INDEX (AIN#(0:149)_EF_INDEX@147)

        Address: 9294
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN147_EF_INDEX, value)

    @property
    @locked
    def ain148_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN148_EF_INDEX (AIN#(0:149)_EF_INDEX@148)

        Address: 9296
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN148_EF_INDEX)  # type: ignore

    @ain148_ef_index.setter
    @locked
    def ain148_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN148_EF_INDEX (AIN#(0:149)_EF_INDEX@148)

        Address: 9296
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN148_EF_INDEX, value)

    @property
    @locked
    def ain149_ef_index(self) -> int:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN149_EF_INDEX (AIN#(0:149)_EF_INDEX@149)

        Address: 9298
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN149_EF_INDEX)  # type: ignore

    @ain149_ef_index.setter
    @locked
    def ain149_ef_index(self, value: int) -> None:
        """
        Specify the desired extended feature for this analog input with the index value.
        List of index values:
        0=None(disabled);
        1=Offset and Slope;
        3=Max/Min/Avg;
        4=Resistance;
        5=Average and Threshold;
        10=RMS Flex;
        11=FlexRMS;
        20=Thermocouple type E;
        21=Thermocouple type J;
        22=Thermocouple type K;
        23=Thermocouple type R;
        24=Thermocouple type T;
        25=Thermocouple type S;
        27=Thermocouple type N;
        28=Thermocouple type B;
        30=Thermocouple type C;
        40=RTD model PT100;
        41=RTD model PT500;
        42=RTD model PT1000;
        50=Thermistor Steinhart-Hart;
        51=Thermistor Beta.

        Register: AIN149_EF_INDEX (AIN#(0:149)_EF_INDEX@149)

        Address: 9298
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN149_EF_INDEX, value)

    @property
    @locked
    def ain0_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@0)

        Address: 9300
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_A)  # type: ignore

    @ain0_ef_config_a.setter
    @locked
    def ain0_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@0)

        Address: 9300
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_A, value)

    @property
    @locked
    def ain1_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@1)

        Address: 9302
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_A)  # type: ignore

    @ain1_ef_config_a.setter
    @locked
    def ain1_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@1)

        Address: 9302
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_A, value)

    @property
    @locked
    def ain2_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@2)

        Address: 9304
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_A)  # type: ignore

    @ain2_ef_config_a.setter
    @locked
    def ain2_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@2)

        Address: 9304
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_A, value)

    @property
    @locked
    def ain3_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@3)

        Address: 9306
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_A)  # type: ignore

    @ain3_ef_config_a.setter
    @locked
    def ain3_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@3)

        Address: 9306
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_A, value)

    @property
    @locked
    def ain4_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@4)

        Address: 9308
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_A)  # type: ignore

    @ain4_ef_config_a.setter
    @locked
    def ain4_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@4)

        Address: 9308
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_A, value)

    @property
    @locked
    def ain5_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@5)

        Address: 9310
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_A)  # type: ignore

    @ain5_ef_config_a.setter
    @locked
    def ain5_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@5)

        Address: 9310
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_A, value)

    @property
    @locked
    def ain6_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@6)

        Address: 9312
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_A)  # type: ignore

    @ain6_ef_config_a.setter
    @locked
    def ain6_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@6)

        Address: 9312
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_A, value)

    @property
    @locked
    def ain7_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@7)

        Address: 9314
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_A)  # type: ignore

    @ain7_ef_config_a.setter
    @locked
    def ain7_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@7)

        Address: 9314
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_A, value)

    @property
    @locked
    def ain8_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@8)

        Address: 9316
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_A)  # type: ignore

    @ain8_ef_config_a.setter
    @locked
    def ain8_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@8)

        Address: 9316
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_A, value)

    @property
    @locked
    def ain9_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@9)

        Address: 9318
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_A)  # type: ignore

    @ain9_ef_config_a.setter
    @locked
    def ain9_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@9)

        Address: 9318
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_A, value)

    @property
    @locked
    def ain10_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@10)

        Address: 9320
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_A)  # type: ignore

    @ain10_ef_config_a.setter
    @locked
    def ain10_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@10)

        Address: 9320
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_A, value)

    @property
    @locked
    def ain11_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@11)

        Address: 9322
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_A)  # type: ignore

    @ain11_ef_config_a.setter
    @locked
    def ain11_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@11)

        Address: 9322
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_A, value)

    @property
    @locked
    def ain12_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@12)

        Address: 9324
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_A)  # type: ignore

    @ain12_ef_config_a.setter
    @locked
    def ain12_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@12)

        Address: 9324
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_A, value)

    @property
    @locked
    def ain13_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@13)

        Address: 9326
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_A)  # type: ignore

    @ain13_ef_config_a.setter
    @locked
    def ain13_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@13)

        Address: 9326
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_A, value)

    @property
    @locked
    def ain14_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@14)

        Address: 9328
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_A)  # type: ignore

    @ain14_ef_config_a.setter
    @locked
    def ain14_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@14)

        Address: 9328
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_A, value)

    @property
    @locked
    def ain15_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@15)

        Address: 9330
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_A)  # type: ignore

    @ain15_ef_config_a.setter
    @locked
    def ain15_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@15)

        Address: 9330
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_A, value)

    @property
    @locked
    def ain16_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@16)

        Address: 9332
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_A)  # type: ignore

    @ain16_ef_config_a.setter
    @locked
    def ain16_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@16)

        Address: 9332
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_A, value)

    @property
    @locked
    def ain17_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@17)

        Address: 9334
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_A)  # type: ignore

    @ain17_ef_config_a.setter
    @locked
    def ain17_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@17)

        Address: 9334
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_A, value)

    @property
    @locked
    def ain18_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@18)

        Address: 9336
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_A)  # type: ignore

    @ain18_ef_config_a.setter
    @locked
    def ain18_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@18)

        Address: 9336
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_A, value)

    @property
    @locked
    def ain19_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@19)

        Address: 9338
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_A)  # type: ignore

    @ain19_ef_config_a.setter
    @locked
    def ain19_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@19)

        Address: 9338
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_A, value)

    @property
    @locked
    def ain20_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@20)

        Address: 9340
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_A)  # type: ignore

    @ain20_ef_config_a.setter
    @locked
    def ain20_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@20)

        Address: 9340
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_A, value)

    @property
    @locked
    def ain21_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@21)

        Address: 9342
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_A)  # type: ignore

    @ain21_ef_config_a.setter
    @locked
    def ain21_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@21)

        Address: 9342
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_A, value)

    @property
    @locked
    def ain22_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@22)

        Address: 9344
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_A)  # type: ignore

    @ain22_ef_config_a.setter
    @locked
    def ain22_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@22)

        Address: 9344
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_A, value)

    @property
    @locked
    def ain23_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@23)

        Address: 9346
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_A)  # type: ignore

    @ain23_ef_config_a.setter
    @locked
    def ain23_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@23)

        Address: 9346
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_A, value)

    @property
    @locked
    def ain24_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@24)

        Address: 9348
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_A)  # type: ignore

    @ain24_ef_config_a.setter
    @locked
    def ain24_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@24)

        Address: 9348
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_A, value)

    @property
    @locked
    def ain25_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@25)

        Address: 9350
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_A)  # type: ignore

    @ain25_ef_config_a.setter
    @locked
    def ain25_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@25)

        Address: 9350
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_A, value)

    @property
    @locked
    def ain26_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@26)

        Address: 9352
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_A)  # type: ignore

    @ain26_ef_config_a.setter
    @locked
    def ain26_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@26)

        Address: 9352
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_A, value)

    @property
    @locked
    def ain27_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@27)

        Address: 9354
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_A)  # type: ignore

    @ain27_ef_config_a.setter
    @locked
    def ain27_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@27)

        Address: 9354
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_A, value)

    @property
    @locked
    def ain28_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@28)

        Address: 9356
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_A)  # type: ignore

    @ain28_ef_config_a.setter
    @locked
    def ain28_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@28)

        Address: 9356
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_A, value)

    @property
    @locked
    def ain29_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@29)

        Address: 9358
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_A)  # type: ignore

    @ain29_ef_config_a.setter
    @locked
    def ain29_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@29)

        Address: 9358
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_A, value)

    @property
    @locked
    def ain30_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@30)

        Address: 9360
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_A)  # type: ignore

    @ain30_ef_config_a.setter
    @locked
    def ain30_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@30)

        Address: 9360
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_A, value)

    @property
    @locked
    def ain31_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@31)

        Address: 9362
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_A)  # type: ignore

    @ain31_ef_config_a.setter
    @locked
    def ain31_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@31)

        Address: 9362
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_A, value)

    @property
    @locked
    def ain32_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@32)

        Address: 9364
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_A)  # type: ignore

    @ain32_ef_config_a.setter
    @locked
    def ain32_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@32)

        Address: 9364
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_A, value)

    @property
    @locked
    def ain33_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@33)

        Address: 9366
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_A)  # type: ignore

    @ain33_ef_config_a.setter
    @locked
    def ain33_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@33)

        Address: 9366
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_A, value)

    @property
    @locked
    def ain34_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@34)

        Address: 9368
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_A)  # type: ignore

    @ain34_ef_config_a.setter
    @locked
    def ain34_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@34)

        Address: 9368
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_A, value)

    @property
    @locked
    def ain35_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@35)

        Address: 9370
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_A)  # type: ignore

    @ain35_ef_config_a.setter
    @locked
    def ain35_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@35)

        Address: 9370
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_A, value)

    @property
    @locked
    def ain36_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@36)

        Address: 9372
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_A)  # type: ignore

    @ain36_ef_config_a.setter
    @locked
    def ain36_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@36)

        Address: 9372
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_A, value)

    @property
    @locked
    def ain37_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@37)

        Address: 9374
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_A)  # type: ignore

    @ain37_ef_config_a.setter
    @locked
    def ain37_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@37)

        Address: 9374
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_A, value)

    @property
    @locked
    def ain38_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@38)

        Address: 9376
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_A)  # type: ignore

    @ain38_ef_config_a.setter
    @locked
    def ain38_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@38)

        Address: 9376
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_A, value)

    @property
    @locked
    def ain39_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@39)

        Address: 9378
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_A)  # type: ignore

    @ain39_ef_config_a.setter
    @locked
    def ain39_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@39)

        Address: 9378
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_A, value)

    @property
    @locked
    def ain40_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@40)

        Address: 9380
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_A)  # type: ignore

    @ain40_ef_config_a.setter
    @locked
    def ain40_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@40)

        Address: 9380
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_A, value)

    @property
    @locked
    def ain41_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@41)

        Address: 9382
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_A)  # type: ignore

    @ain41_ef_config_a.setter
    @locked
    def ain41_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@41)

        Address: 9382
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_A, value)

    @property
    @locked
    def ain42_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@42)

        Address: 9384
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_A)  # type: ignore

    @ain42_ef_config_a.setter
    @locked
    def ain42_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@42)

        Address: 9384
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_A, value)

    @property
    @locked
    def ain43_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@43)

        Address: 9386
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_A)  # type: ignore

    @ain43_ef_config_a.setter
    @locked
    def ain43_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@43)

        Address: 9386
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_A, value)

    @property
    @locked
    def ain44_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@44)

        Address: 9388
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_A)  # type: ignore

    @ain44_ef_config_a.setter
    @locked
    def ain44_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@44)

        Address: 9388
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_A, value)

    @property
    @locked
    def ain45_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@45)

        Address: 9390
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_A)  # type: ignore

    @ain45_ef_config_a.setter
    @locked
    def ain45_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@45)

        Address: 9390
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_A, value)

    @property
    @locked
    def ain46_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@46)

        Address: 9392
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_A)  # type: ignore

    @ain46_ef_config_a.setter
    @locked
    def ain46_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@46)

        Address: 9392
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_A, value)

    @property
    @locked
    def ain47_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@47)

        Address: 9394
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_A)  # type: ignore

    @ain47_ef_config_a.setter
    @locked
    def ain47_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@47)

        Address: 9394
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_A, value)

    @property
    @locked
    def ain48_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@48)

        Address: 9396
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_A)  # type: ignore

    @ain48_ef_config_a.setter
    @locked
    def ain48_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@48)

        Address: 9396
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_A, value)

    @property
    @locked
    def ain49_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@49)

        Address: 9398
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_A)  # type: ignore

    @ain49_ef_config_a.setter
    @locked
    def ain49_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@49)

        Address: 9398
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_A, value)

    @property
    @locked
    def ain50_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@50)

        Address: 9400
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_A)  # type: ignore

    @ain50_ef_config_a.setter
    @locked
    def ain50_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@50)

        Address: 9400
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_A, value)

    @property
    @locked
    def ain51_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@51)

        Address: 9402
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_A)  # type: ignore

    @ain51_ef_config_a.setter
    @locked
    def ain51_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@51)

        Address: 9402
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_A, value)

    @property
    @locked
    def ain52_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@52)

        Address: 9404
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_A)  # type: ignore

    @ain52_ef_config_a.setter
    @locked
    def ain52_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@52)

        Address: 9404
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_A, value)

    @property
    @locked
    def ain53_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@53)

        Address: 9406
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_A)  # type: ignore

    @ain53_ef_config_a.setter
    @locked
    def ain53_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@53)

        Address: 9406
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_A, value)

    @property
    @locked
    def ain54_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@54)

        Address: 9408
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_A)  # type: ignore

    @ain54_ef_config_a.setter
    @locked
    def ain54_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@54)

        Address: 9408
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_A, value)

    @property
    @locked
    def ain55_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@55)

        Address: 9410
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_A)  # type: ignore

    @ain55_ef_config_a.setter
    @locked
    def ain55_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@55)

        Address: 9410
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_A, value)

    @property
    @locked
    def ain56_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@56)

        Address: 9412
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_A)  # type: ignore

    @ain56_ef_config_a.setter
    @locked
    def ain56_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@56)

        Address: 9412
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_A, value)

    @property
    @locked
    def ain57_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@57)

        Address: 9414
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_A)  # type: ignore

    @ain57_ef_config_a.setter
    @locked
    def ain57_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@57)

        Address: 9414
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_A, value)

    @property
    @locked
    def ain58_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@58)

        Address: 9416
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_A)  # type: ignore

    @ain58_ef_config_a.setter
    @locked
    def ain58_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@58)

        Address: 9416
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_A, value)

    @property
    @locked
    def ain59_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@59)

        Address: 9418
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_A)  # type: ignore

    @ain59_ef_config_a.setter
    @locked
    def ain59_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@59)

        Address: 9418
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_A, value)

    @property
    @locked
    def ain60_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@60)

        Address: 9420
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_A)  # type: ignore

    @ain60_ef_config_a.setter
    @locked
    def ain60_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@60)

        Address: 9420
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_A, value)

    @property
    @locked
    def ain61_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@61)

        Address: 9422
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_A)  # type: ignore

    @ain61_ef_config_a.setter
    @locked
    def ain61_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@61)

        Address: 9422
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_A, value)

    @property
    @locked
    def ain62_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@62)

        Address: 9424
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_A)  # type: ignore

    @ain62_ef_config_a.setter
    @locked
    def ain62_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@62)

        Address: 9424
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_A, value)

    @property
    @locked
    def ain63_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@63)

        Address: 9426
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_A)  # type: ignore

    @ain63_ef_config_a.setter
    @locked
    def ain63_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@63)

        Address: 9426
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_A, value)

    @property
    @locked
    def ain64_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@64)

        Address: 9428
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_A)  # type: ignore

    @ain64_ef_config_a.setter
    @locked
    def ain64_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@64)

        Address: 9428
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_A, value)

    @property
    @locked
    def ain65_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@65)

        Address: 9430
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_A)  # type: ignore

    @ain65_ef_config_a.setter
    @locked
    def ain65_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@65)

        Address: 9430
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_A, value)

    @property
    @locked
    def ain66_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@66)

        Address: 9432
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_A)  # type: ignore

    @ain66_ef_config_a.setter
    @locked
    def ain66_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@66)

        Address: 9432
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_A, value)

    @property
    @locked
    def ain67_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@67)

        Address: 9434
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_A)  # type: ignore

    @ain67_ef_config_a.setter
    @locked
    def ain67_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@67)

        Address: 9434
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_A, value)

    @property
    @locked
    def ain68_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@68)

        Address: 9436
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_A)  # type: ignore

    @ain68_ef_config_a.setter
    @locked
    def ain68_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@68)

        Address: 9436
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_A, value)

    @property
    @locked
    def ain69_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@69)

        Address: 9438
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_A)  # type: ignore

    @ain69_ef_config_a.setter
    @locked
    def ain69_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@69)

        Address: 9438
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_A, value)

    @property
    @locked
    def ain70_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@70)

        Address: 9440
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_A)  # type: ignore

    @ain70_ef_config_a.setter
    @locked
    def ain70_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@70)

        Address: 9440
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_A, value)

    @property
    @locked
    def ain71_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@71)

        Address: 9442
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_A)  # type: ignore

    @ain71_ef_config_a.setter
    @locked
    def ain71_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@71)

        Address: 9442
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_A, value)

    @property
    @locked
    def ain72_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@72)

        Address: 9444
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_A)  # type: ignore

    @ain72_ef_config_a.setter
    @locked
    def ain72_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@72)

        Address: 9444
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_A, value)

    @property
    @locked
    def ain73_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@73)

        Address: 9446
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_A)  # type: ignore

    @ain73_ef_config_a.setter
    @locked
    def ain73_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@73)

        Address: 9446
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_A, value)

    @property
    @locked
    def ain74_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@74)

        Address: 9448
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_A)  # type: ignore

    @ain74_ef_config_a.setter
    @locked
    def ain74_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@74)

        Address: 9448
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_A, value)

    @property
    @locked
    def ain75_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@75)

        Address: 9450
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_A)  # type: ignore

    @ain75_ef_config_a.setter
    @locked
    def ain75_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@75)

        Address: 9450
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_A, value)

    @property
    @locked
    def ain76_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@76)

        Address: 9452
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_A)  # type: ignore

    @ain76_ef_config_a.setter
    @locked
    def ain76_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@76)

        Address: 9452
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_A, value)

    @property
    @locked
    def ain77_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@77)

        Address: 9454
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_A)  # type: ignore

    @ain77_ef_config_a.setter
    @locked
    def ain77_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@77)

        Address: 9454
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_A, value)

    @property
    @locked
    def ain78_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@78)

        Address: 9456
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_A)  # type: ignore

    @ain78_ef_config_a.setter
    @locked
    def ain78_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@78)

        Address: 9456
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_A, value)

    @property
    @locked
    def ain79_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@79)

        Address: 9458
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_A)  # type: ignore

    @ain79_ef_config_a.setter
    @locked
    def ain79_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@79)

        Address: 9458
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_A, value)

    @property
    @locked
    def ain80_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@80)

        Address: 9460
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_A)  # type: ignore

    @ain80_ef_config_a.setter
    @locked
    def ain80_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@80)

        Address: 9460
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_A, value)

    @property
    @locked
    def ain81_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@81)

        Address: 9462
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_A)  # type: ignore

    @ain81_ef_config_a.setter
    @locked
    def ain81_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@81)

        Address: 9462
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_A, value)

    @property
    @locked
    def ain82_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@82)

        Address: 9464
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_A)  # type: ignore

    @ain82_ef_config_a.setter
    @locked
    def ain82_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@82)

        Address: 9464
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_A, value)

    @property
    @locked
    def ain83_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@83)

        Address: 9466
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_A)  # type: ignore

    @ain83_ef_config_a.setter
    @locked
    def ain83_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@83)

        Address: 9466
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_A, value)

    @property
    @locked
    def ain84_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@84)

        Address: 9468
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_A)  # type: ignore

    @ain84_ef_config_a.setter
    @locked
    def ain84_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@84)

        Address: 9468
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_A, value)

    @property
    @locked
    def ain85_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@85)

        Address: 9470
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_A)  # type: ignore

    @ain85_ef_config_a.setter
    @locked
    def ain85_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@85)

        Address: 9470
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_A, value)

    @property
    @locked
    def ain86_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@86)

        Address: 9472
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_A)  # type: ignore

    @ain86_ef_config_a.setter
    @locked
    def ain86_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@86)

        Address: 9472
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_A, value)

    @property
    @locked
    def ain87_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@87)

        Address: 9474
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_A)  # type: ignore

    @ain87_ef_config_a.setter
    @locked
    def ain87_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@87)

        Address: 9474
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_A, value)

    @property
    @locked
    def ain88_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@88)

        Address: 9476
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_A)  # type: ignore

    @ain88_ef_config_a.setter
    @locked
    def ain88_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@88)

        Address: 9476
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_A, value)

    @property
    @locked
    def ain89_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@89)

        Address: 9478
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_A)  # type: ignore

    @ain89_ef_config_a.setter
    @locked
    def ain89_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@89)

        Address: 9478
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_A, value)

    @property
    @locked
    def ain90_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@90)

        Address: 9480
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_A)  # type: ignore

    @ain90_ef_config_a.setter
    @locked
    def ain90_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@90)

        Address: 9480
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_A, value)

    @property
    @locked
    def ain91_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@91)

        Address: 9482
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_A)  # type: ignore

    @ain91_ef_config_a.setter
    @locked
    def ain91_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@91)

        Address: 9482
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_A, value)

    @property
    @locked
    def ain92_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@92)

        Address: 9484
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_A)  # type: ignore

    @ain92_ef_config_a.setter
    @locked
    def ain92_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@92)

        Address: 9484
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_A, value)

    @property
    @locked
    def ain93_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@93)

        Address: 9486
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_A)  # type: ignore

    @ain93_ef_config_a.setter
    @locked
    def ain93_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@93)

        Address: 9486
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_A, value)

    @property
    @locked
    def ain94_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@94)

        Address: 9488
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_A)  # type: ignore

    @ain94_ef_config_a.setter
    @locked
    def ain94_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@94)

        Address: 9488
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_A, value)

    @property
    @locked
    def ain95_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@95)

        Address: 9490
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_A)  # type: ignore

    @ain95_ef_config_a.setter
    @locked
    def ain95_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@95)

        Address: 9490
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_A, value)

    @property
    @locked
    def ain96_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@96)

        Address: 9492
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_A)  # type: ignore

    @ain96_ef_config_a.setter
    @locked
    def ain96_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@96)

        Address: 9492
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_A, value)

    @property
    @locked
    def ain97_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@97)

        Address: 9494
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_A)  # type: ignore

    @ain97_ef_config_a.setter
    @locked
    def ain97_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@97)

        Address: 9494
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_A, value)

    @property
    @locked
    def ain98_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@98)

        Address: 9496
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_A)  # type: ignore

    @ain98_ef_config_a.setter
    @locked
    def ain98_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@98)

        Address: 9496
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_A, value)

    @property
    @locked
    def ain99_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@99)

        Address: 9498
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_A)  # type: ignore

    @ain99_ef_config_a.setter
    @locked
    def ain99_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@99)

        Address: 9498
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_A, value)

    @property
    @locked
    def ain100_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@100)

        Address: 9500
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_A)  # type: ignore

    @ain100_ef_config_a.setter
    @locked
    def ain100_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@100)

        Address: 9500
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_A, value)

    @property
    @locked
    def ain101_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@101)

        Address: 9502
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_A)  # type: ignore

    @ain101_ef_config_a.setter
    @locked
    def ain101_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@101)

        Address: 9502
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_A, value)

    @property
    @locked
    def ain102_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@102)

        Address: 9504
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_A)  # type: ignore

    @ain102_ef_config_a.setter
    @locked
    def ain102_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@102)

        Address: 9504
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_A, value)

    @property
    @locked
    def ain103_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@103)

        Address: 9506
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_A)  # type: ignore

    @ain103_ef_config_a.setter
    @locked
    def ain103_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@103)

        Address: 9506
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_A, value)

    @property
    @locked
    def ain104_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@104)

        Address: 9508
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_A)  # type: ignore

    @ain104_ef_config_a.setter
    @locked
    def ain104_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@104)

        Address: 9508
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_A, value)

    @property
    @locked
    def ain105_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@105)

        Address: 9510
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_A)  # type: ignore

    @ain105_ef_config_a.setter
    @locked
    def ain105_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@105)

        Address: 9510
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_A, value)

    @property
    @locked
    def ain106_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@106)

        Address: 9512
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_A)  # type: ignore

    @ain106_ef_config_a.setter
    @locked
    def ain106_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@106)

        Address: 9512
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_A, value)

    @property
    @locked
    def ain107_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@107)

        Address: 9514
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_A)  # type: ignore

    @ain107_ef_config_a.setter
    @locked
    def ain107_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@107)

        Address: 9514
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_A, value)

    @property
    @locked
    def ain108_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@108)

        Address: 9516
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_A)  # type: ignore

    @ain108_ef_config_a.setter
    @locked
    def ain108_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@108)

        Address: 9516
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_A, value)

    @property
    @locked
    def ain109_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@109)

        Address: 9518
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_A)  # type: ignore

    @ain109_ef_config_a.setter
    @locked
    def ain109_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@109)

        Address: 9518
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_A, value)

    @property
    @locked
    def ain110_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@110)

        Address: 9520
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_A)  # type: ignore

    @ain110_ef_config_a.setter
    @locked
    def ain110_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@110)

        Address: 9520
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_A, value)

    @property
    @locked
    def ain111_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@111)

        Address: 9522
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_A)  # type: ignore

    @ain111_ef_config_a.setter
    @locked
    def ain111_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@111)

        Address: 9522
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_A, value)

    @property
    @locked
    def ain112_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@112)

        Address: 9524
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_A)  # type: ignore

    @ain112_ef_config_a.setter
    @locked
    def ain112_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@112)

        Address: 9524
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_A, value)

    @property
    @locked
    def ain113_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@113)

        Address: 9526
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_A)  # type: ignore

    @ain113_ef_config_a.setter
    @locked
    def ain113_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@113)

        Address: 9526
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_A, value)

    @property
    @locked
    def ain114_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@114)

        Address: 9528
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_A)  # type: ignore

    @ain114_ef_config_a.setter
    @locked
    def ain114_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@114)

        Address: 9528
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_A, value)

    @property
    @locked
    def ain115_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@115)

        Address: 9530
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_A)  # type: ignore

    @ain115_ef_config_a.setter
    @locked
    def ain115_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@115)

        Address: 9530
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_A, value)

    @property
    @locked
    def ain116_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@116)

        Address: 9532
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_A)  # type: ignore

    @ain116_ef_config_a.setter
    @locked
    def ain116_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@116)

        Address: 9532
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_A, value)

    @property
    @locked
    def ain117_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@117)

        Address: 9534
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_A)  # type: ignore

    @ain117_ef_config_a.setter
    @locked
    def ain117_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@117)

        Address: 9534
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_A, value)

    @property
    @locked
    def ain118_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@118)

        Address: 9536
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_A)  # type: ignore

    @ain118_ef_config_a.setter
    @locked
    def ain118_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@118)

        Address: 9536
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_A, value)

    @property
    @locked
    def ain119_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@119)

        Address: 9538
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_A)  # type: ignore

    @ain119_ef_config_a.setter
    @locked
    def ain119_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@119)

        Address: 9538
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_A, value)

    @property
    @locked
    def ain120_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@120)

        Address: 9540
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_A)  # type: ignore

    @ain120_ef_config_a.setter
    @locked
    def ain120_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@120)

        Address: 9540
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_A, value)

    @property
    @locked
    def ain121_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@121)

        Address: 9542
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_A)  # type: ignore

    @ain121_ef_config_a.setter
    @locked
    def ain121_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@121)

        Address: 9542
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_A, value)

    @property
    @locked
    def ain122_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@122)

        Address: 9544
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_A)  # type: ignore

    @ain122_ef_config_a.setter
    @locked
    def ain122_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@122)

        Address: 9544
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_A, value)

    @property
    @locked
    def ain123_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@123)

        Address: 9546
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_A)  # type: ignore

    @ain123_ef_config_a.setter
    @locked
    def ain123_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@123)

        Address: 9546
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_A, value)

    @property
    @locked
    def ain124_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@124)

        Address: 9548
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_A)  # type: ignore

    @ain124_ef_config_a.setter
    @locked
    def ain124_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@124)

        Address: 9548
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_A, value)

    @property
    @locked
    def ain125_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@125)

        Address: 9550
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_A)  # type: ignore

    @ain125_ef_config_a.setter
    @locked
    def ain125_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@125)

        Address: 9550
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_A, value)

    @property
    @locked
    def ain126_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@126)

        Address: 9552
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_A)  # type: ignore

    @ain126_ef_config_a.setter
    @locked
    def ain126_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@126)

        Address: 9552
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_A, value)

    @property
    @locked
    def ain127_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@127)

        Address: 9554
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_A)  # type: ignore

    @ain127_ef_config_a.setter
    @locked
    def ain127_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@127)

        Address: 9554
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_A, value)

    @property
    @locked
    def ain128_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@128)

        Address: 9556
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_A)  # type: ignore

    @ain128_ef_config_a.setter
    @locked
    def ain128_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@128)

        Address: 9556
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_A, value)

    @property
    @locked
    def ain129_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@129)

        Address: 9558
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_A)  # type: ignore

    @ain129_ef_config_a.setter
    @locked
    def ain129_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@129)

        Address: 9558
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_A, value)

    @property
    @locked
    def ain130_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@130)

        Address: 9560
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_A)  # type: ignore

    @ain130_ef_config_a.setter
    @locked
    def ain130_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@130)

        Address: 9560
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_A, value)

    @property
    @locked
    def ain131_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@131)

        Address: 9562
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_A)  # type: ignore

    @ain131_ef_config_a.setter
    @locked
    def ain131_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@131)

        Address: 9562
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_A, value)

    @property
    @locked
    def ain132_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@132)

        Address: 9564
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_A)  # type: ignore

    @ain132_ef_config_a.setter
    @locked
    def ain132_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@132)

        Address: 9564
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_A, value)

    @property
    @locked
    def ain133_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@133)

        Address: 9566
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_A)  # type: ignore

    @ain133_ef_config_a.setter
    @locked
    def ain133_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@133)

        Address: 9566
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_A, value)

    @property
    @locked
    def ain134_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@134)

        Address: 9568
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_A)  # type: ignore

    @ain134_ef_config_a.setter
    @locked
    def ain134_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@134)

        Address: 9568
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_A, value)

    @property
    @locked
    def ain135_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@135)

        Address: 9570
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_A)  # type: ignore

    @ain135_ef_config_a.setter
    @locked
    def ain135_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@135)

        Address: 9570
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_A, value)

    @property
    @locked
    def ain136_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@136)

        Address: 9572
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_A)  # type: ignore

    @ain136_ef_config_a.setter
    @locked
    def ain136_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@136)

        Address: 9572
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_A, value)

    @property
    @locked
    def ain137_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@137)

        Address: 9574
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_A)  # type: ignore

    @ain137_ef_config_a.setter
    @locked
    def ain137_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@137)

        Address: 9574
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_A, value)

    @property
    @locked
    def ain138_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@138)

        Address: 9576
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_A)  # type: ignore

    @ain138_ef_config_a.setter
    @locked
    def ain138_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@138)

        Address: 9576
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_A, value)

    @property
    @locked
    def ain139_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@139)

        Address: 9578
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_A)  # type: ignore

    @ain139_ef_config_a.setter
    @locked
    def ain139_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@139)

        Address: 9578
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_A, value)

    @property
    @locked
    def ain140_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@140)

        Address: 9580
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_A)  # type: ignore

    @ain140_ef_config_a.setter
    @locked
    def ain140_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@140)

        Address: 9580
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_A, value)

    @property
    @locked
    def ain141_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@141)

        Address: 9582
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_A)  # type: ignore

    @ain141_ef_config_a.setter
    @locked
    def ain141_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@141)

        Address: 9582
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_A, value)

    @property
    @locked
    def ain142_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@142)

        Address: 9584
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_A)  # type: ignore

    @ain142_ef_config_a.setter
    @locked
    def ain142_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@142)

        Address: 9584
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_A, value)

    @property
    @locked
    def ain143_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@143)

        Address: 9586
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_A)  # type: ignore

    @ain143_ef_config_a.setter
    @locked
    def ain143_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@143)

        Address: 9586
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_A, value)

    @property
    @locked
    def ain144_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@144)

        Address: 9588
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_A)  # type: ignore

    @ain144_ef_config_a.setter
    @locked
    def ain144_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@144)

        Address: 9588
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_A, value)

    @property
    @locked
    def ain145_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@145)

        Address: 9590
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_A)  # type: ignore

    @ain145_ef_config_a.setter
    @locked
    def ain145_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@145)

        Address: 9590
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_A, value)

    @property
    @locked
    def ain146_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@146)

        Address: 9592
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_A)  # type: ignore

    @ain146_ef_config_a.setter
    @locked
    def ain146_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@146)

        Address: 9592
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_A, value)

    @property
    @locked
    def ain147_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@147)

        Address: 9594
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_A)  # type: ignore

    @ain147_ef_config_a.setter
    @locked
    def ain147_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@147)

        Address: 9594
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_A, value)

    @property
    @locked
    def ain148_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@148)

        Address: 9596
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_A)  # type: ignore

    @ain148_ef_config_a.setter
    @locked
    def ain148_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@148)

        Address: 9596
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_A, value)

    @property
    @locked
    def ain149_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@149)

        Address: 9598
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_A)  # type: ignore

    @ain149_ef_config_a.setter
    @locked
    def ain149_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_A (AIN#(0:149)_EF_CONFIG_A@149)

        Address: 9598
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_A, value)

    @property
    @locked
    def ain0_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@0)

        Address: 9600
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_B)  # type: ignore

    @ain0_ef_config_b.setter
    @locked
    def ain0_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@0)

        Address: 9600
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_B, value)

    @property
    @locked
    def ain1_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@1)

        Address: 9602
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_B)  # type: ignore

    @ain1_ef_config_b.setter
    @locked
    def ain1_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@1)

        Address: 9602
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_B, value)

    @property
    @locked
    def ain2_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@2)

        Address: 9604
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_B)  # type: ignore

    @ain2_ef_config_b.setter
    @locked
    def ain2_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@2)

        Address: 9604
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_B, value)

    @property
    @locked
    def ain3_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@3)

        Address: 9606
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_B)  # type: ignore

    @ain3_ef_config_b.setter
    @locked
    def ain3_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@3)

        Address: 9606
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_B, value)

    @property
    @locked
    def ain4_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@4)

        Address: 9608
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_B)  # type: ignore

    @ain4_ef_config_b.setter
    @locked
    def ain4_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@4)

        Address: 9608
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_B, value)

    @property
    @locked
    def ain5_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@5)

        Address: 9610
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_B)  # type: ignore

    @ain5_ef_config_b.setter
    @locked
    def ain5_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@5)

        Address: 9610
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_B, value)

    @property
    @locked
    def ain6_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@6)

        Address: 9612
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_B)  # type: ignore

    @ain6_ef_config_b.setter
    @locked
    def ain6_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@6)

        Address: 9612
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_B, value)

    @property
    @locked
    def ain7_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@7)

        Address: 9614
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_B)  # type: ignore

    @ain7_ef_config_b.setter
    @locked
    def ain7_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@7)

        Address: 9614
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_B, value)

    @property
    @locked
    def ain8_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@8)

        Address: 9616
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_B)  # type: ignore

    @ain8_ef_config_b.setter
    @locked
    def ain8_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@8)

        Address: 9616
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_B, value)

    @property
    @locked
    def ain9_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@9)

        Address: 9618
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_B)  # type: ignore

    @ain9_ef_config_b.setter
    @locked
    def ain9_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@9)

        Address: 9618
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_B, value)

    @property
    @locked
    def ain10_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@10)

        Address: 9620
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_B)  # type: ignore

    @ain10_ef_config_b.setter
    @locked
    def ain10_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@10)

        Address: 9620
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_B, value)

    @property
    @locked
    def ain11_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@11)

        Address: 9622
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_B)  # type: ignore

    @ain11_ef_config_b.setter
    @locked
    def ain11_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@11)

        Address: 9622
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_B, value)

    @property
    @locked
    def ain12_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@12)

        Address: 9624
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_B)  # type: ignore

    @ain12_ef_config_b.setter
    @locked
    def ain12_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@12)

        Address: 9624
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_B, value)

    @property
    @locked
    def ain13_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@13)

        Address: 9626
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_B)  # type: ignore

    @ain13_ef_config_b.setter
    @locked
    def ain13_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@13)

        Address: 9626
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_B, value)

    @property
    @locked
    def ain14_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@14)

        Address: 9628
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_B)  # type: ignore

    @ain14_ef_config_b.setter
    @locked
    def ain14_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@14)

        Address: 9628
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_B, value)

    @property
    @locked
    def ain15_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@15)

        Address: 9630
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_B)  # type: ignore

    @ain15_ef_config_b.setter
    @locked
    def ain15_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@15)

        Address: 9630
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_B, value)

    @property
    @locked
    def ain16_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@16)

        Address: 9632
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_B)  # type: ignore

    @ain16_ef_config_b.setter
    @locked
    def ain16_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@16)

        Address: 9632
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_B, value)

    @property
    @locked
    def ain17_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@17)

        Address: 9634
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_B)  # type: ignore

    @ain17_ef_config_b.setter
    @locked
    def ain17_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@17)

        Address: 9634
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_B, value)

    @property
    @locked
    def ain18_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@18)

        Address: 9636
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_B)  # type: ignore

    @ain18_ef_config_b.setter
    @locked
    def ain18_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@18)

        Address: 9636
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_B, value)

    @property
    @locked
    def ain19_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@19)

        Address: 9638
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_B)  # type: ignore

    @ain19_ef_config_b.setter
    @locked
    def ain19_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@19)

        Address: 9638
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_B, value)

    @property
    @locked
    def ain20_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@20)

        Address: 9640
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_B)  # type: ignore

    @ain20_ef_config_b.setter
    @locked
    def ain20_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@20)

        Address: 9640
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_B, value)

    @property
    @locked
    def ain21_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@21)

        Address: 9642
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_B)  # type: ignore

    @ain21_ef_config_b.setter
    @locked
    def ain21_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@21)

        Address: 9642
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_B, value)

    @property
    @locked
    def ain22_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@22)

        Address: 9644
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_B)  # type: ignore

    @ain22_ef_config_b.setter
    @locked
    def ain22_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@22)

        Address: 9644
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_B, value)

    @property
    @locked
    def ain23_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@23)

        Address: 9646
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_B)  # type: ignore

    @ain23_ef_config_b.setter
    @locked
    def ain23_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@23)

        Address: 9646
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_B, value)

    @property
    @locked
    def ain24_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@24)

        Address: 9648
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_B)  # type: ignore

    @ain24_ef_config_b.setter
    @locked
    def ain24_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@24)

        Address: 9648
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_B, value)

    @property
    @locked
    def ain25_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@25)

        Address: 9650
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_B)  # type: ignore

    @ain25_ef_config_b.setter
    @locked
    def ain25_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@25)

        Address: 9650
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_B, value)

    @property
    @locked
    def ain26_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@26)

        Address: 9652
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_B)  # type: ignore

    @ain26_ef_config_b.setter
    @locked
    def ain26_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@26)

        Address: 9652
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_B, value)

    @property
    @locked
    def ain27_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@27)

        Address: 9654
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_B)  # type: ignore

    @ain27_ef_config_b.setter
    @locked
    def ain27_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@27)

        Address: 9654
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_B, value)

    @property
    @locked
    def ain28_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@28)

        Address: 9656
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_B)  # type: ignore

    @ain28_ef_config_b.setter
    @locked
    def ain28_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@28)

        Address: 9656
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_B, value)

    @property
    @locked
    def ain29_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@29)

        Address: 9658
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_B)  # type: ignore

    @ain29_ef_config_b.setter
    @locked
    def ain29_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@29)

        Address: 9658
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_B, value)

    @property
    @locked
    def ain30_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@30)

        Address: 9660
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_B)  # type: ignore

    @ain30_ef_config_b.setter
    @locked
    def ain30_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@30)

        Address: 9660
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_B, value)

    @property
    @locked
    def ain31_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@31)

        Address: 9662
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_B)  # type: ignore

    @ain31_ef_config_b.setter
    @locked
    def ain31_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@31)

        Address: 9662
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_B, value)

    @property
    @locked
    def ain32_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@32)

        Address: 9664
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_B)  # type: ignore

    @ain32_ef_config_b.setter
    @locked
    def ain32_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@32)

        Address: 9664
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_B, value)

    @property
    @locked
    def ain33_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@33)

        Address: 9666
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_B)  # type: ignore

    @ain33_ef_config_b.setter
    @locked
    def ain33_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@33)

        Address: 9666
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_B, value)

    @property
    @locked
    def ain34_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@34)

        Address: 9668
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_B)  # type: ignore

    @ain34_ef_config_b.setter
    @locked
    def ain34_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@34)

        Address: 9668
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_B, value)

    @property
    @locked
    def ain35_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@35)

        Address: 9670
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_B)  # type: ignore

    @ain35_ef_config_b.setter
    @locked
    def ain35_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@35)

        Address: 9670
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_B, value)

    @property
    @locked
    def ain36_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@36)

        Address: 9672
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_B)  # type: ignore

    @ain36_ef_config_b.setter
    @locked
    def ain36_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@36)

        Address: 9672
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_B, value)

    @property
    @locked
    def ain37_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@37)

        Address: 9674
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_B)  # type: ignore

    @ain37_ef_config_b.setter
    @locked
    def ain37_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@37)

        Address: 9674
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_B, value)

    @property
    @locked
    def ain38_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@38)

        Address: 9676
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_B)  # type: ignore

    @ain38_ef_config_b.setter
    @locked
    def ain38_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@38)

        Address: 9676
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_B, value)

    @property
    @locked
    def ain39_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@39)

        Address: 9678
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_B)  # type: ignore

    @ain39_ef_config_b.setter
    @locked
    def ain39_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@39)

        Address: 9678
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_B, value)

    @property
    @locked
    def ain40_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@40)

        Address: 9680
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_B)  # type: ignore

    @ain40_ef_config_b.setter
    @locked
    def ain40_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@40)

        Address: 9680
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_B, value)

    @property
    @locked
    def ain41_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@41)

        Address: 9682
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_B)  # type: ignore

    @ain41_ef_config_b.setter
    @locked
    def ain41_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@41)

        Address: 9682
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_B, value)

    @property
    @locked
    def ain42_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@42)

        Address: 9684
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_B)  # type: ignore

    @ain42_ef_config_b.setter
    @locked
    def ain42_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@42)

        Address: 9684
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_B, value)

    @property
    @locked
    def ain43_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@43)

        Address: 9686
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_B)  # type: ignore

    @ain43_ef_config_b.setter
    @locked
    def ain43_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@43)

        Address: 9686
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_B, value)

    @property
    @locked
    def ain44_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@44)

        Address: 9688
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_B)  # type: ignore

    @ain44_ef_config_b.setter
    @locked
    def ain44_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@44)

        Address: 9688
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_B, value)

    @property
    @locked
    def ain45_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@45)

        Address: 9690
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_B)  # type: ignore

    @ain45_ef_config_b.setter
    @locked
    def ain45_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@45)

        Address: 9690
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_B, value)

    @property
    @locked
    def ain46_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@46)

        Address: 9692
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_B)  # type: ignore

    @ain46_ef_config_b.setter
    @locked
    def ain46_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@46)

        Address: 9692
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_B, value)

    @property
    @locked
    def ain47_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@47)

        Address: 9694
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_B)  # type: ignore

    @ain47_ef_config_b.setter
    @locked
    def ain47_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@47)

        Address: 9694
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_B, value)

    @property
    @locked
    def ain48_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@48)

        Address: 9696
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_B)  # type: ignore

    @ain48_ef_config_b.setter
    @locked
    def ain48_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@48)

        Address: 9696
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_B, value)

    @property
    @locked
    def ain49_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@49)

        Address: 9698
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_B)  # type: ignore

    @ain49_ef_config_b.setter
    @locked
    def ain49_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@49)

        Address: 9698
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_B, value)

    @property
    @locked
    def ain50_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@50)

        Address: 9700
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_B)  # type: ignore

    @ain50_ef_config_b.setter
    @locked
    def ain50_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@50)

        Address: 9700
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_B, value)

    @property
    @locked
    def ain51_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@51)

        Address: 9702
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_B)  # type: ignore

    @ain51_ef_config_b.setter
    @locked
    def ain51_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@51)

        Address: 9702
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_B, value)

    @property
    @locked
    def ain52_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@52)

        Address: 9704
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_B)  # type: ignore

    @ain52_ef_config_b.setter
    @locked
    def ain52_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@52)

        Address: 9704
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_B, value)

    @property
    @locked
    def ain53_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@53)

        Address: 9706
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_B)  # type: ignore

    @ain53_ef_config_b.setter
    @locked
    def ain53_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@53)

        Address: 9706
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_B, value)

    @property
    @locked
    def ain54_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@54)

        Address: 9708
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_B)  # type: ignore

    @ain54_ef_config_b.setter
    @locked
    def ain54_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@54)

        Address: 9708
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_B, value)

    @property
    @locked
    def ain55_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@55)

        Address: 9710
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_B)  # type: ignore

    @ain55_ef_config_b.setter
    @locked
    def ain55_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@55)

        Address: 9710
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_B, value)

    @property
    @locked
    def ain56_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@56)

        Address: 9712
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_B)  # type: ignore

    @ain56_ef_config_b.setter
    @locked
    def ain56_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@56)

        Address: 9712
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_B, value)

    @property
    @locked
    def ain57_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@57)

        Address: 9714
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_B)  # type: ignore

    @ain57_ef_config_b.setter
    @locked
    def ain57_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@57)

        Address: 9714
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_B, value)

    @property
    @locked
    def ain58_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@58)

        Address: 9716
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_B)  # type: ignore

    @ain58_ef_config_b.setter
    @locked
    def ain58_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@58)

        Address: 9716
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_B, value)

    @property
    @locked
    def ain59_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@59)

        Address: 9718
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_B)  # type: ignore

    @ain59_ef_config_b.setter
    @locked
    def ain59_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@59)

        Address: 9718
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_B, value)

    @property
    @locked
    def ain60_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@60)

        Address: 9720
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_B)  # type: ignore

    @ain60_ef_config_b.setter
    @locked
    def ain60_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@60)

        Address: 9720
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_B, value)

    @property
    @locked
    def ain61_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@61)

        Address: 9722
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_B)  # type: ignore

    @ain61_ef_config_b.setter
    @locked
    def ain61_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@61)

        Address: 9722
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_B, value)

    @property
    @locked
    def ain62_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@62)

        Address: 9724
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_B)  # type: ignore

    @ain62_ef_config_b.setter
    @locked
    def ain62_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@62)

        Address: 9724
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_B, value)

    @property
    @locked
    def ain63_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@63)

        Address: 9726
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_B)  # type: ignore

    @ain63_ef_config_b.setter
    @locked
    def ain63_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@63)

        Address: 9726
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_B, value)

    @property
    @locked
    def ain64_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@64)

        Address: 9728
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_B)  # type: ignore

    @ain64_ef_config_b.setter
    @locked
    def ain64_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@64)

        Address: 9728
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_B, value)

    @property
    @locked
    def ain65_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@65)

        Address: 9730
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_B)  # type: ignore

    @ain65_ef_config_b.setter
    @locked
    def ain65_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@65)

        Address: 9730
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_B, value)

    @property
    @locked
    def ain66_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@66)

        Address: 9732
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_B)  # type: ignore

    @ain66_ef_config_b.setter
    @locked
    def ain66_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@66)

        Address: 9732
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_B, value)

    @property
    @locked
    def ain67_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@67)

        Address: 9734
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_B)  # type: ignore

    @ain67_ef_config_b.setter
    @locked
    def ain67_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@67)

        Address: 9734
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_B, value)

    @property
    @locked
    def ain68_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@68)

        Address: 9736
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_B)  # type: ignore

    @ain68_ef_config_b.setter
    @locked
    def ain68_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@68)

        Address: 9736
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_B, value)

    @property
    @locked
    def ain69_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@69)

        Address: 9738
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_B)  # type: ignore

    @ain69_ef_config_b.setter
    @locked
    def ain69_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@69)

        Address: 9738
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_B, value)

    @property
    @locked
    def ain70_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@70)

        Address: 9740
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_B)  # type: ignore

    @ain70_ef_config_b.setter
    @locked
    def ain70_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@70)

        Address: 9740
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_B, value)

    @property
    @locked
    def ain71_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@71)

        Address: 9742
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_B)  # type: ignore

    @ain71_ef_config_b.setter
    @locked
    def ain71_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@71)

        Address: 9742
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_B, value)

    @property
    @locked
    def ain72_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@72)

        Address: 9744
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_B)  # type: ignore

    @ain72_ef_config_b.setter
    @locked
    def ain72_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@72)

        Address: 9744
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_B, value)

    @property
    @locked
    def ain73_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@73)

        Address: 9746
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_B)  # type: ignore

    @ain73_ef_config_b.setter
    @locked
    def ain73_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@73)

        Address: 9746
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_B, value)

    @property
    @locked
    def ain74_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@74)

        Address: 9748
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_B)  # type: ignore

    @ain74_ef_config_b.setter
    @locked
    def ain74_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@74)

        Address: 9748
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_B, value)

    @property
    @locked
    def ain75_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@75)

        Address: 9750
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_B)  # type: ignore

    @ain75_ef_config_b.setter
    @locked
    def ain75_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@75)

        Address: 9750
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_B, value)

    @property
    @locked
    def ain76_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@76)

        Address: 9752
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_B)  # type: ignore

    @ain76_ef_config_b.setter
    @locked
    def ain76_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@76)

        Address: 9752
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_B, value)

    @property
    @locked
    def ain77_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@77)

        Address: 9754
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_B)  # type: ignore

    @ain77_ef_config_b.setter
    @locked
    def ain77_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@77)

        Address: 9754
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_B, value)

    @property
    @locked
    def ain78_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@78)

        Address: 9756
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_B)  # type: ignore

    @ain78_ef_config_b.setter
    @locked
    def ain78_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@78)

        Address: 9756
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_B, value)

    @property
    @locked
    def ain79_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@79)

        Address: 9758
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_B)  # type: ignore

    @ain79_ef_config_b.setter
    @locked
    def ain79_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@79)

        Address: 9758
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_B, value)

    @property
    @locked
    def ain80_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@80)

        Address: 9760
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_B)  # type: ignore

    @ain80_ef_config_b.setter
    @locked
    def ain80_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@80)

        Address: 9760
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_B, value)

    @property
    @locked
    def ain81_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@81)

        Address: 9762
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_B)  # type: ignore

    @ain81_ef_config_b.setter
    @locked
    def ain81_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@81)

        Address: 9762
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_B, value)

    @property
    @locked
    def ain82_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@82)

        Address: 9764
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_B)  # type: ignore

    @ain82_ef_config_b.setter
    @locked
    def ain82_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@82)

        Address: 9764
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_B, value)

    @property
    @locked
    def ain83_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@83)

        Address: 9766
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_B)  # type: ignore

    @ain83_ef_config_b.setter
    @locked
    def ain83_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@83)

        Address: 9766
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_B, value)

    @property
    @locked
    def ain84_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@84)

        Address: 9768
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_B)  # type: ignore

    @ain84_ef_config_b.setter
    @locked
    def ain84_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@84)

        Address: 9768
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_B, value)

    @property
    @locked
    def ain85_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@85)

        Address: 9770
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_B)  # type: ignore

    @ain85_ef_config_b.setter
    @locked
    def ain85_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@85)

        Address: 9770
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_B, value)

    @property
    @locked
    def ain86_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@86)

        Address: 9772
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_B)  # type: ignore

    @ain86_ef_config_b.setter
    @locked
    def ain86_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@86)

        Address: 9772
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_B, value)

    @property
    @locked
    def ain87_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@87)

        Address: 9774
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_B)  # type: ignore

    @ain87_ef_config_b.setter
    @locked
    def ain87_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@87)

        Address: 9774
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_B, value)

    @property
    @locked
    def ain88_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@88)

        Address: 9776
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_B)  # type: ignore

    @ain88_ef_config_b.setter
    @locked
    def ain88_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@88)

        Address: 9776
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_B, value)

    @property
    @locked
    def ain89_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@89)

        Address: 9778
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_B)  # type: ignore

    @ain89_ef_config_b.setter
    @locked
    def ain89_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@89)

        Address: 9778
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_B, value)

    @property
    @locked
    def ain90_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@90)

        Address: 9780
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_B)  # type: ignore

    @ain90_ef_config_b.setter
    @locked
    def ain90_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@90)

        Address: 9780
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_B, value)

    @property
    @locked
    def ain91_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@91)

        Address: 9782
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_B)  # type: ignore

    @ain91_ef_config_b.setter
    @locked
    def ain91_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@91)

        Address: 9782
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_B, value)

    @property
    @locked
    def ain92_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@92)

        Address: 9784
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_B)  # type: ignore

    @ain92_ef_config_b.setter
    @locked
    def ain92_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@92)

        Address: 9784
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_B, value)

    @property
    @locked
    def ain93_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@93)

        Address: 9786
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_B)  # type: ignore

    @ain93_ef_config_b.setter
    @locked
    def ain93_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@93)

        Address: 9786
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_B, value)

    @property
    @locked
    def ain94_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@94)

        Address: 9788
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_B)  # type: ignore

    @ain94_ef_config_b.setter
    @locked
    def ain94_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@94)

        Address: 9788
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_B, value)

    @property
    @locked
    def ain95_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@95)

        Address: 9790
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_B)  # type: ignore

    @ain95_ef_config_b.setter
    @locked
    def ain95_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@95)

        Address: 9790
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_B, value)

    @property
    @locked
    def ain96_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@96)

        Address: 9792
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_B)  # type: ignore

    @ain96_ef_config_b.setter
    @locked
    def ain96_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@96)

        Address: 9792
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_B, value)

    @property
    @locked
    def ain97_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@97)

        Address: 9794
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_B)  # type: ignore

    @ain97_ef_config_b.setter
    @locked
    def ain97_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@97)

        Address: 9794
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_B, value)

    @property
    @locked
    def ain98_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@98)

        Address: 9796
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_B)  # type: ignore

    @ain98_ef_config_b.setter
    @locked
    def ain98_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@98)

        Address: 9796
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_B, value)

    @property
    @locked
    def ain99_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@99)

        Address: 9798
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_B)  # type: ignore

    @ain99_ef_config_b.setter
    @locked
    def ain99_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@99)

        Address: 9798
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_B, value)

    @property
    @locked
    def ain100_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@100)

        Address: 9800
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_B)  # type: ignore

    @ain100_ef_config_b.setter
    @locked
    def ain100_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@100)

        Address: 9800
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_B, value)

    @property
    @locked
    def ain101_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@101)

        Address: 9802
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_B)  # type: ignore

    @ain101_ef_config_b.setter
    @locked
    def ain101_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@101)

        Address: 9802
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_B, value)

    @property
    @locked
    def ain102_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@102)

        Address: 9804
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_B)  # type: ignore

    @ain102_ef_config_b.setter
    @locked
    def ain102_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@102)

        Address: 9804
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_B, value)

    @property
    @locked
    def ain103_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@103)

        Address: 9806
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_B)  # type: ignore

    @ain103_ef_config_b.setter
    @locked
    def ain103_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@103)

        Address: 9806
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_B, value)

    @property
    @locked
    def ain104_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@104)

        Address: 9808
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_B)  # type: ignore

    @ain104_ef_config_b.setter
    @locked
    def ain104_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@104)

        Address: 9808
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_B, value)

    @property
    @locked
    def ain105_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@105)

        Address: 9810
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_B)  # type: ignore

    @ain105_ef_config_b.setter
    @locked
    def ain105_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@105)

        Address: 9810
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_B, value)

    @property
    @locked
    def ain106_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@106)

        Address: 9812
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_B)  # type: ignore

    @ain106_ef_config_b.setter
    @locked
    def ain106_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@106)

        Address: 9812
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_B, value)

    @property
    @locked
    def ain107_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@107)

        Address: 9814
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_B)  # type: ignore

    @ain107_ef_config_b.setter
    @locked
    def ain107_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@107)

        Address: 9814
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_B, value)

    @property
    @locked
    def ain108_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@108)

        Address: 9816
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_B)  # type: ignore

    @ain108_ef_config_b.setter
    @locked
    def ain108_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@108)

        Address: 9816
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_B, value)

    @property
    @locked
    def ain109_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@109)

        Address: 9818
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_B)  # type: ignore

    @ain109_ef_config_b.setter
    @locked
    def ain109_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@109)

        Address: 9818
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_B, value)

    @property
    @locked
    def ain110_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@110)

        Address: 9820
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_B)  # type: ignore

    @ain110_ef_config_b.setter
    @locked
    def ain110_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@110)

        Address: 9820
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_B, value)

    @property
    @locked
    def ain111_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@111)

        Address: 9822
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_B)  # type: ignore

    @ain111_ef_config_b.setter
    @locked
    def ain111_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@111)

        Address: 9822
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_B, value)

    @property
    @locked
    def ain112_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@112)

        Address: 9824
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_B)  # type: ignore

    @ain112_ef_config_b.setter
    @locked
    def ain112_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@112)

        Address: 9824
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_B, value)

    @property
    @locked
    def ain113_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@113)

        Address: 9826
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_B)  # type: ignore

    @ain113_ef_config_b.setter
    @locked
    def ain113_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@113)

        Address: 9826
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_B, value)

    @property
    @locked
    def ain114_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@114)

        Address: 9828
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_B)  # type: ignore

    @ain114_ef_config_b.setter
    @locked
    def ain114_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@114)

        Address: 9828
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_B, value)

    @property
    @locked
    def ain115_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@115)

        Address: 9830
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_B)  # type: ignore

    @ain115_ef_config_b.setter
    @locked
    def ain115_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@115)

        Address: 9830
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_B, value)

    @property
    @locked
    def ain116_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@116)

        Address: 9832
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_B)  # type: ignore

    @ain116_ef_config_b.setter
    @locked
    def ain116_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@116)

        Address: 9832
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_B, value)

    @property
    @locked
    def ain117_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@117)

        Address: 9834
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_B)  # type: ignore

    @ain117_ef_config_b.setter
    @locked
    def ain117_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@117)

        Address: 9834
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_B, value)

    @property
    @locked
    def ain118_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@118)

        Address: 9836
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_B)  # type: ignore

    @ain118_ef_config_b.setter
    @locked
    def ain118_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@118)

        Address: 9836
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_B, value)

    @property
    @locked
    def ain119_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@119)

        Address: 9838
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_B)  # type: ignore

    @ain119_ef_config_b.setter
    @locked
    def ain119_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@119)

        Address: 9838
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_B, value)

    @property
    @locked
    def ain120_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@120)

        Address: 9840
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_B)  # type: ignore

    @ain120_ef_config_b.setter
    @locked
    def ain120_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@120)

        Address: 9840
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_B, value)

    @property
    @locked
    def ain121_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@121)

        Address: 9842
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_B)  # type: ignore

    @ain121_ef_config_b.setter
    @locked
    def ain121_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@121)

        Address: 9842
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_B, value)

    @property
    @locked
    def ain122_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@122)

        Address: 9844
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_B)  # type: ignore

    @ain122_ef_config_b.setter
    @locked
    def ain122_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@122)

        Address: 9844
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_B, value)

    @property
    @locked
    def ain123_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@123)

        Address: 9846
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_B)  # type: ignore

    @ain123_ef_config_b.setter
    @locked
    def ain123_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@123)

        Address: 9846
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_B, value)

    @property
    @locked
    def ain124_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@124)

        Address: 9848
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_B)  # type: ignore

    @ain124_ef_config_b.setter
    @locked
    def ain124_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@124)

        Address: 9848
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_B, value)

    @property
    @locked
    def ain125_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@125)

        Address: 9850
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_B)  # type: ignore

    @ain125_ef_config_b.setter
    @locked
    def ain125_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@125)

        Address: 9850
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_B, value)

    @property
    @locked
    def ain126_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@126)

        Address: 9852
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_B)  # type: ignore

    @ain126_ef_config_b.setter
    @locked
    def ain126_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@126)

        Address: 9852
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_B, value)

    @property
    @locked
    def ain127_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@127)

        Address: 9854
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_B)  # type: ignore

    @ain127_ef_config_b.setter
    @locked
    def ain127_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@127)

        Address: 9854
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_B, value)

    @property
    @locked
    def ain128_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@128)

        Address: 9856
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_B)  # type: ignore

    @ain128_ef_config_b.setter
    @locked
    def ain128_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@128)

        Address: 9856
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_B, value)

    @property
    @locked
    def ain129_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@129)

        Address: 9858
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_B)  # type: ignore

    @ain129_ef_config_b.setter
    @locked
    def ain129_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@129)

        Address: 9858
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_B, value)

    @property
    @locked
    def ain130_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@130)

        Address: 9860
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_B)  # type: ignore

    @ain130_ef_config_b.setter
    @locked
    def ain130_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@130)

        Address: 9860
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_B, value)

    @property
    @locked
    def ain131_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@131)

        Address: 9862
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_B)  # type: ignore

    @ain131_ef_config_b.setter
    @locked
    def ain131_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@131)

        Address: 9862
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_B, value)

    @property
    @locked
    def ain132_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@132)

        Address: 9864
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_B)  # type: ignore

    @ain132_ef_config_b.setter
    @locked
    def ain132_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@132)

        Address: 9864
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_B, value)

    @property
    @locked
    def ain133_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@133)

        Address: 9866
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_B)  # type: ignore

    @ain133_ef_config_b.setter
    @locked
    def ain133_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@133)

        Address: 9866
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_B, value)

    @property
    @locked
    def ain134_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@134)

        Address: 9868
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_B)  # type: ignore

    @ain134_ef_config_b.setter
    @locked
    def ain134_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@134)

        Address: 9868
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_B, value)

    @property
    @locked
    def ain135_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@135)

        Address: 9870
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_B)  # type: ignore

    @ain135_ef_config_b.setter
    @locked
    def ain135_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@135)

        Address: 9870
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_B, value)

    @property
    @locked
    def ain136_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@136)

        Address: 9872
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_B)  # type: ignore

    @ain136_ef_config_b.setter
    @locked
    def ain136_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@136)

        Address: 9872
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_B, value)

    @property
    @locked
    def ain137_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@137)

        Address: 9874
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_B)  # type: ignore

    @ain137_ef_config_b.setter
    @locked
    def ain137_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@137)

        Address: 9874
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_B, value)

    @property
    @locked
    def ain138_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@138)

        Address: 9876
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_B)  # type: ignore

    @ain138_ef_config_b.setter
    @locked
    def ain138_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@138)

        Address: 9876
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_B, value)

    @property
    @locked
    def ain139_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@139)

        Address: 9878
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_B)  # type: ignore

    @ain139_ef_config_b.setter
    @locked
    def ain139_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@139)

        Address: 9878
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_B, value)

    @property
    @locked
    def ain140_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@140)

        Address: 9880
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_B)  # type: ignore

    @ain140_ef_config_b.setter
    @locked
    def ain140_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@140)

        Address: 9880
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_B, value)

    @property
    @locked
    def ain141_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@141)

        Address: 9882
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_B)  # type: ignore

    @ain141_ef_config_b.setter
    @locked
    def ain141_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@141)

        Address: 9882
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_B, value)

    @property
    @locked
    def ain142_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@142)

        Address: 9884
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_B)  # type: ignore

    @ain142_ef_config_b.setter
    @locked
    def ain142_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@142)

        Address: 9884
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_B, value)

    @property
    @locked
    def ain143_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@143)

        Address: 9886
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_B)  # type: ignore

    @ain143_ef_config_b.setter
    @locked
    def ain143_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@143)

        Address: 9886
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_B, value)

    @property
    @locked
    def ain144_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@144)

        Address: 9888
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_B)  # type: ignore

    @ain144_ef_config_b.setter
    @locked
    def ain144_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@144)

        Address: 9888
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_B, value)

    @property
    @locked
    def ain145_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@145)

        Address: 9890
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_B)  # type: ignore

    @ain145_ef_config_b.setter
    @locked
    def ain145_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@145)

        Address: 9890
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_B, value)

    @property
    @locked
    def ain146_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@146)

        Address: 9892
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_B)  # type: ignore

    @ain146_ef_config_b.setter
    @locked
    def ain146_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@146)

        Address: 9892
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_B, value)

    @property
    @locked
    def ain147_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@147)

        Address: 9894
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_B)  # type: ignore

    @ain147_ef_config_b.setter
    @locked
    def ain147_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@147)

        Address: 9894
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_B, value)

    @property
    @locked
    def ain148_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@148)

        Address: 9896
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_B)  # type: ignore

    @ain148_ef_config_b.setter
    @locked
    def ain148_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@148)

        Address: 9896
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_B, value)

    @property
    @locked
    def ain149_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@149)

        Address: 9898
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_B)  # type: ignore

    @ain149_ef_config_b.setter
    @locked
    def ain149_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_B (AIN#(0:149)_EF_CONFIG_B@149)

        Address: 9898
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_B, value)

    @property
    @locked
    def ain0_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@0)

        Address: 9900
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_C)  # type: ignore

    @ain0_ef_config_c.setter
    @locked
    def ain0_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@0)

        Address: 9900
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN0_EF_CONFIG_C, value)

    @property
    @locked
    def ain1_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@1)

        Address: 9902
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_C)  # type: ignore

    @ain1_ef_config_c.setter
    @locked
    def ain1_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@1)

        Address: 9902
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN1_EF_CONFIG_C, value)

    @property
    @locked
    def ain2_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@2)

        Address: 9904
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_C)  # type: ignore

    @ain2_ef_config_c.setter
    @locked
    def ain2_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@2)

        Address: 9904
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN2_EF_CONFIG_C, value)

    @property
    @locked
    def ain3_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@3)

        Address: 9906
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_C)  # type: ignore

    @ain3_ef_config_c.setter
    @locked
    def ain3_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@3)

        Address: 9906
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN3_EF_CONFIG_C, value)

    @property
    @locked
    def ain4_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@4)

        Address: 9908
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_C)  # type: ignore

    @ain4_ef_config_c.setter
    @locked
    def ain4_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@4)

        Address: 9908
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN4_EF_CONFIG_C, value)

    @property
    @locked
    def ain5_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@5)

        Address: 9910
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_C)  # type: ignore

    @ain5_ef_config_c.setter
    @locked
    def ain5_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@5)

        Address: 9910
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN5_EF_CONFIG_C, value)

    @property
    @locked
    def ain6_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@6)

        Address: 9912
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_C)  # type: ignore

    @ain6_ef_config_c.setter
    @locked
    def ain6_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@6)

        Address: 9912
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN6_EF_CONFIG_C, value)

    @property
    @locked
    def ain7_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@7)

        Address: 9914
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_C)  # type: ignore

    @ain7_ef_config_c.setter
    @locked
    def ain7_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@7)

        Address: 9914
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN7_EF_CONFIG_C, value)

    @property
    @locked
    def ain8_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@8)

        Address: 9916
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_C)  # type: ignore

    @ain8_ef_config_c.setter
    @locked
    def ain8_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@8)

        Address: 9916
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN8_EF_CONFIG_C, value)

    @property
    @locked
    def ain9_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@9)

        Address: 9918
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_C)  # type: ignore

    @ain9_ef_config_c.setter
    @locked
    def ain9_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@9)

        Address: 9918
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN9_EF_CONFIG_C, value)

    @property
    @locked
    def ain10_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@10)

        Address: 9920
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_C)  # type: ignore

    @ain10_ef_config_c.setter
    @locked
    def ain10_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@10)

        Address: 9920
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN10_EF_CONFIG_C, value)

    @property
    @locked
    def ain11_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@11)

        Address: 9922
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_C)  # type: ignore

    @ain11_ef_config_c.setter
    @locked
    def ain11_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@11)

        Address: 9922
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN11_EF_CONFIG_C, value)

    @property
    @locked
    def ain12_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@12)

        Address: 9924
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_C)  # type: ignore

    @ain12_ef_config_c.setter
    @locked
    def ain12_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@12)

        Address: 9924
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN12_EF_CONFIG_C, value)

    @property
    @locked
    def ain13_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@13)

        Address: 9926
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_C)  # type: ignore

    @ain13_ef_config_c.setter
    @locked
    def ain13_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@13)

        Address: 9926
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN13_EF_CONFIG_C, value)

    @property
    @locked
    def ain14_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@14)

        Address: 9928
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_C)  # type: ignore

    @ain14_ef_config_c.setter
    @locked
    def ain14_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@14)

        Address: 9928
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN14_EF_CONFIG_C, value)

    @property
    @locked
    def ain15_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@15)

        Address: 9930
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_C)  # type: ignore

    @ain15_ef_config_c.setter
    @locked
    def ain15_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@15)

        Address: 9930
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN15_EF_CONFIG_C, value)

    @property
    @locked
    def ain16_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@16)

        Address: 9932
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_C)  # type: ignore

    @ain16_ef_config_c.setter
    @locked
    def ain16_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@16)

        Address: 9932
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN16_EF_CONFIG_C, value)

    @property
    @locked
    def ain17_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@17)

        Address: 9934
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_C)  # type: ignore

    @ain17_ef_config_c.setter
    @locked
    def ain17_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@17)

        Address: 9934
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN17_EF_CONFIG_C, value)

    @property
    @locked
    def ain18_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@18)

        Address: 9936
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_C)  # type: ignore

    @ain18_ef_config_c.setter
    @locked
    def ain18_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@18)

        Address: 9936
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN18_EF_CONFIG_C, value)

    @property
    @locked
    def ain19_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@19)

        Address: 9938
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_C)  # type: ignore

    @ain19_ef_config_c.setter
    @locked
    def ain19_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@19)

        Address: 9938
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN19_EF_CONFIG_C, value)

    @property
    @locked
    def ain20_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@20)

        Address: 9940
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_C)  # type: ignore

    @ain20_ef_config_c.setter
    @locked
    def ain20_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@20)

        Address: 9940
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN20_EF_CONFIG_C, value)

    @property
    @locked
    def ain21_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@21)

        Address: 9942
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_C)  # type: ignore

    @ain21_ef_config_c.setter
    @locked
    def ain21_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@21)

        Address: 9942
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN21_EF_CONFIG_C, value)

    @property
    @locked
    def ain22_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@22)

        Address: 9944
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_C)  # type: ignore

    @ain22_ef_config_c.setter
    @locked
    def ain22_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@22)

        Address: 9944
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN22_EF_CONFIG_C, value)

    @property
    @locked
    def ain23_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@23)

        Address: 9946
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_C)  # type: ignore

    @ain23_ef_config_c.setter
    @locked
    def ain23_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@23)

        Address: 9946
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN23_EF_CONFIG_C, value)

    @property
    @locked
    def ain24_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@24)

        Address: 9948
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_C)  # type: ignore

    @ain24_ef_config_c.setter
    @locked
    def ain24_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@24)

        Address: 9948
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN24_EF_CONFIG_C, value)

    @property
    @locked
    def ain25_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@25)

        Address: 9950
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_C)  # type: ignore

    @ain25_ef_config_c.setter
    @locked
    def ain25_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@25)

        Address: 9950
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN25_EF_CONFIG_C, value)

    @property
    @locked
    def ain26_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@26)

        Address: 9952
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_C)  # type: ignore

    @ain26_ef_config_c.setter
    @locked
    def ain26_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@26)

        Address: 9952
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN26_EF_CONFIG_C, value)

    @property
    @locked
    def ain27_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@27)

        Address: 9954
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_C)  # type: ignore

    @ain27_ef_config_c.setter
    @locked
    def ain27_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@27)

        Address: 9954
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN27_EF_CONFIG_C, value)

    @property
    @locked
    def ain28_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@28)

        Address: 9956
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_C)  # type: ignore

    @ain28_ef_config_c.setter
    @locked
    def ain28_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@28)

        Address: 9956
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN28_EF_CONFIG_C, value)

    @property
    @locked
    def ain29_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@29)

        Address: 9958
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_C)  # type: ignore

    @ain29_ef_config_c.setter
    @locked
    def ain29_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@29)

        Address: 9958
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN29_EF_CONFIG_C, value)

    @property
    @locked
    def ain30_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@30)

        Address: 9960
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_C)  # type: ignore

    @ain30_ef_config_c.setter
    @locked
    def ain30_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@30)

        Address: 9960
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN30_EF_CONFIG_C, value)

    @property
    @locked
    def ain31_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@31)

        Address: 9962
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_C)  # type: ignore

    @ain31_ef_config_c.setter
    @locked
    def ain31_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@31)

        Address: 9962
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN31_EF_CONFIG_C, value)

    @property
    @locked
    def ain32_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@32)

        Address: 9964
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_C)  # type: ignore

    @ain32_ef_config_c.setter
    @locked
    def ain32_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@32)

        Address: 9964
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN32_EF_CONFIG_C, value)

    @property
    @locked
    def ain33_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@33)

        Address: 9966
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_C)  # type: ignore

    @ain33_ef_config_c.setter
    @locked
    def ain33_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@33)

        Address: 9966
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN33_EF_CONFIG_C, value)

    @property
    @locked
    def ain34_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@34)

        Address: 9968
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_C)  # type: ignore

    @ain34_ef_config_c.setter
    @locked
    def ain34_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@34)

        Address: 9968
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN34_EF_CONFIG_C, value)

    @property
    @locked
    def ain35_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@35)

        Address: 9970
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_C)  # type: ignore

    @ain35_ef_config_c.setter
    @locked
    def ain35_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@35)

        Address: 9970
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN35_EF_CONFIG_C, value)

    @property
    @locked
    def ain36_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@36)

        Address: 9972
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_C)  # type: ignore

    @ain36_ef_config_c.setter
    @locked
    def ain36_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@36)

        Address: 9972
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN36_EF_CONFIG_C, value)

    @property
    @locked
    def ain37_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@37)

        Address: 9974
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_C)  # type: ignore

    @ain37_ef_config_c.setter
    @locked
    def ain37_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@37)

        Address: 9974
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN37_EF_CONFIG_C, value)

    @property
    @locked
    def ain38_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@38)

        Address: 9976
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_C)  # type: ignore

    @ain38_ef_config_c.setter
    @locked
    def ain38_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@38)

        Address: 9976
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN38_EF_CONFIG_C, value)

    @property
    @locked
    def ain39_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@39)

        Address: 9978
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_C)  # type: ignore

    @ain39_ef_config_c.setter
    @locked
    def ain39_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@39)

        Address: 9978
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN39_EF_CONFIG_C, value)

    @property
    @locked
    def ain40_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@40)

        Address: 9980
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_C)  # type: ignore

    @ain40_ef_config_c.setter
    @locked
    def ain40_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@40)

        Address: 9980
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN40_EF_CONFIG_C, value)

    @property
    @locked
    def ain41_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@41)

        Address: 9982
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_C)  # type: ignore

    @ain41_ef_config_c.setter
    @locked
    def ain41_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@41)

        Address: 9982
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN41_EF_CONFIG_C, value)

    @property
    @locked
    def ain42_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@42)

        Address: 9984
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_C)  # type: ignore

    @ain42_ef_config_c.setter
    @locked
    def ain42_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@42)

        Address: 9984
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN42_EF_CONFIG_C, value)

    @property
    @locked
    def ain43_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@43)

        Address: 9986
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_C)  # type: ignore

    @ain43_ef_config_c.setter
    @locked
    def ain43_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@43)

        Address: 9986
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN43_EF_CONFIG_C, value)

    @property
    @locked
    def ain44_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@44)

        Address: 9988
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_C)  # type: ignore

    @ain44_ef_config_c.setter
    @locked
    def ain44_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@44)

        Address: 9988
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN44_EF_CONFIG_C, value)

    @property
    @locked
    def ain45_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@45)

        Address: 9990
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_C)  # type: ignore

    @ain45_ef_config_c.setter
    @locked
    def ain45_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@45)

        Address: 9990
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN45_EF_CONFIG_C, value)

    @property
    @locked
    def ain46_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@46)

        Address: 9992
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_C)  # type: ignore

    @ain46_ef_config_c.setter
    @locked
    def ain46_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@46)

        Address: 9992
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN46_EF_CONFIG_C, value)

    @property
    @locked
    def ain47_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@47)

        Address: 9994
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_C)  # type: ignore

    @ain47_ef_config_c.setter
    @locked
    def ain47_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@47)

        Address: 9994
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN47_EF_CONFIG_C, value)

    @property
    @locked
    def ain48_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@48)

        Address: 9996
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_C)  # type: ignore

    @ain48_ef_config_c.setter
    @locked
    def ain48_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@48)

        Address: 9996
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN48_EF_CONFIG_C, value)

    @property
    @locked
    def ain49_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@49)

        Address: 9998
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_C)  # type: ignore

    @ain49_ef_config_c.setter
    @locked
    def ain49_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@49)

        Address: 9998
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN49_EF_CONFIG_C, value)

    @property
    @locked
    def ain50_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@50)

        Address: 10000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_C)  # type: ignore

    @ain50_ef_config_c.setter
    @locked
    def ain50_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@50)

        Address: 10000
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN50_EF_CONFIG_C, value)

    @property
    @locked
    def ain51_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@51)

        Address: 10002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_C)  # type: ignore

    @ain51_ef_config_c.setter
    @locked
    def ain51_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@51)

        Address: 10002
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN51_EF_CONFIG_C, value)

    @property
    @locked
    def ain52_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@52)

        Address: 10004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_C)  # type: ignore

    @ain52_ef_config_c.setter
    @locked
    def ain52_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@52)

        Address: 10004
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN52_EF_CONFIG_C, value)

    @property
    @locked
    def ain53_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@53)

        Address: 10006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_C)  # type: ignore

    @ain53_ef_config_c.setter
    @locked
    def ain53_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@53)

        Address: 10006
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN53_EF_CONFIG_C, value)

    @property
    @locked
    def ain54_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@54)

        Address: 10008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_C)  # type: ignore

    @ain54_ef_config_c.setter
    @locked
    def ain54_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@54)

        Address: 10008
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN54_EF_CONFIG_C, value)

    @property
    @locked
    def ain55_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@55)

        Address: 10010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_C)  # type: ignore

    @ain55_ef_config_c.setter
    @locked
    def ain55_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@55)

        Address: 10010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN55_EF_CONFIG_C, value)

    @property
    @locked
    def ain56_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@56)

        Address: 10012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_C)  # type: ignore

    @ain56_ef_config_c.setter
    @locked
    def ain56_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@56)

        Address: 10012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN56_EF_CONFIG_C, value)

    @property
    @locked
    def ain57_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@57)

        Address: 10014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_C)  # type: ignore

    @ain57_ef_config_c.setter
    @locked
    def ain57_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@57)

        Address: 10014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN57_EF_CONFIG_C, value)

    @property
    @locked
    def ain58_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@58)

        Address: 10016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_C)  # type: ignore

    @ain58_ef_config_c.setter
    @locked
    def ain58_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@58)

        Address: 10016
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN58_EF_CONFIG_C, value)

    @property
    @locked
    def ain59_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@59)

        Address: 10018
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_C)  # type: ignore

    @ain59_ef_config_c.setter
    @locked
    def ain59_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@59)

        Address: 10018
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN59_EF_CONFIG_C, value)

    @property
    @locked
    def ain60_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@60)

        Address: 10020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_C)  # type: ignore

    @ain60_ef_config_c.setter
    @locked
    def ain60_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@60)

        Address: 10020
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN60_EF_CONFIG_C, value)

    @property
    @locked
    def ain61_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@61)

        Address: 10022
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_C)  # type: ignore

    @ain61_ef_config_c.setter
    @locked
    def ain61_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@61)

        Address: 10022
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN61_EF_CONFIG_C, value)

    @property
    @locked
    def ain62_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@62)

        Address: 10024
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_C)  # type: ignore

    @ain62_ef_config_c.setter
    @locked
    def ain62_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@62)

        Address: 10024
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN62_EF_CONFIG_C, value)

    @property
    @locked
    def ain63_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@63)

        Address: 10026
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_C)  # type: ignore

    @ain63_ef_config_c.setter
    @locked
    def ain63_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@63)

        Address: 10026
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN63_EF_CONFIG_C, value)

    @property
    @locked
    def ain64_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@64)

        Address: 10028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_C)  # type: ignore

    @ain64_ef_config_c.setter
    @locked
    def ain64_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@64)

        Address: 10028
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN64_EF_CONFIG_C, value)

    @property
    @locked
    def ain65_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@65)

        Address: 10030
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_C)  # type: ignore

    @ain65_ef_config_c.setter
    @locked
    def ain65_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@65)

        Address: 10030
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN65_EF_CONFIG_C, value)

    @property
    @locked
    def ain66_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@66)

        Address: 10032
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_C)  # type: ignore

    @ain66_ef_config_c.setter
    @locked
    def ain66_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@66)

        Address: 10032
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN66_EF_CONFIG_C, value)

    @property
    @locked
    def ain67_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@67)

        Address: 10034
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_C)  # type: ignore

    @ain67_ef_config_c.setter
    @locked
    def ain67_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@67)

        Address: 10034
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN67_EF_CONFIG_C, value)

    @property
    @locked
    def ain68_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@68)

        Address: 10036
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_C)  # type: ignore

    @ain68_ef_config_c.setter
    @locked
    def ain68_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@68)

        Address: 10036
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN68_EF_CONFIG_C, value)

    @property
    @locked
    def ain69_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@69)

        Address: 10038
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_C)  # type: ignore

    @ain69_ef_config_c.setter
    @locked
    def ain69_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@69)

        Address: 10038
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN69_EF_CONFIG_C, value)

    @property
    @locked
    def ain70_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@70)

        Address: 10040
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_C)  # type: ignore

    @ain70_ef_config_c.setter
    @locked
    def ain70_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@70)

        Address: 10040
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN70_EF_CONFIG_C, value)

    @property
    @locked
    def ain71_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@71)

        Address: 10042
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_C)  # type: ignore

    @ain71_ef_config_c.setter
    @locked
    def ain71_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@71)

        Address: 10042
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN71_EF_CONFIG_C, value)

    @property
    @locked
    def ain72_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@72)

        Address: 10044
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_C)  # type: ignore

    @ain72_ef_config_c.setter
    @locked
    def ain72_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@72)

        Address: 10044
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN72_EF_CONFIG_C, value)

    @property
    @locked
    def ain73_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@73)

        Address: 10046
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_C)  # type: ignore

    @ain73_ef_config_c.setter
    @locked
    def ain73_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@73)

        Address: 10046
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN73_EF_CONFIG_C, value)

    @property
    @locked
    def ain74_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@74)

        Address: 10048
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_C)  # type: ignore

    @ain74_ef_config_c.setter
    @locked
    def ain74_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@74)

        Address: 10048
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN74_EF_CONFIG_C, value)

    @property
    @locked
    def ain75_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@75)

        Address: 10050
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_C)  # type: ignore

    @ain75_ef_config_c.setter
    @locked
    def ain75_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@75)

        Address: 10050
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN75_EF_CONFIG_C, value)

    @property
    @locked
    def ain76_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@76)

        Address: 10052
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_C)  # type: ignore

    @ain76_ef_config_c.setter
    @locked
    def ain76_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@76)

        Address: 10052
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN76_EF_CONFIG_C, value)

    @property
    @locked
    def ain77_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@77)

        Address: 10054
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_C)  # type: ignore

    @ain77_ef_config_c.setter
    @locked
    def ain77_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@77)

        Address: 10054
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN77_EF_CONFIG_C, value)

    @property
    @locked
    def ain78_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@78)

        Address: 10056
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_C)  # type: ignore

    @ain78_ef_config_c.setter
    @locked
    def ain78_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@78)

        Address: 10056
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN78_EF_CONFIG_C, value)

    @property
    @locked
    def ain79_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@79)

        Address: 10058
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_C)  # type: ignore

    @ain79_ef_config_c.setter
    @locked
    def ain79_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@79)

        Address: 10058
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN79_EF_CONFIG_C, value)

    @property
    @locked
    def ain80_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@80)

        Address: 10060
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_C)  # type: ignore

    @ain80_ef_config_c.setter
    @locked
    def ain80_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@80)

        Address: 10060
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN80_EF_CONFIG_C, value)

    @property
    @locked
    def ain81_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@81)

        Address: 10062
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_C)  # type: ignore

    @ain81_ef_config_c.setter
    @locked
    def ain81_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@81)

        Address: 10062
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN81_EF_CONFIG_C, value)

    @property
    @locked
    def ain82_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@82)

        Address: 10064
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_C)  # type: ignore

    @ain82_ef_config_c.setter
    @locked
    def ain82_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@82)

        Address: 10064
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN82_EF_CONFIG_C, value)

    @property
    @locked
    def ain83_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@83)

        Address: 10066
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_C)  # type: ignore

    @ain83_ef_config_c.setter
    @locked
    def ain83_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@83)

        Address: 10066
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN83_EF_CONFIG_C, value)

    @property
    @locked
    def ain84_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@84)

        Address: 10068
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_C)  # type: ignore

    @ain84_ef_config_c.setter
    @locked
    def ain84_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@84)

        Address: 10068
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN84_EF_CONFIG_C, value)

    @property
    @locked
    def ain85_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@85)

        Address: 10070
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_C)  # type: ignore

    @ain85_ef_config_c.setter
    @locked
    def ain85_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@85)

        Address: 10070
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN85_EF_CONFIG_C, value)

    @property
    @locked
    def ain86_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@86)

        Address: 10072
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_C)  # type: ignore

    @ain86_ef_config_c.setter
    @locked
    def ain86_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@86)

        Address: 10072
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN86_EF_CONFIG_C, value)

    @property
    @locked
    def ain87_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@87)

        Address: 10074
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_C)  # type: ignore

    @ain87_ef_config_c.setter
    @locked
    def ain87_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@87)

        Address: 10074
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN87_EF_CONFIG_C, value)

    @property
    @locked
    def ain88_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@88)

        Address: 10076
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_C)  # type: ignore

    @ain88_ef_config_c.setter
    @locked
    def ain88_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@88)

        Address: 10076
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN88_EF_CONFIG_C, value)

    @property
    @locked
    def ain89_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@89)

        Address: 10078
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_C)  # type: ignore

    @ain89_ef_config_c.setter
    @locked
    def ain89_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@89)

        Address: 10078
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN89_EF_CONFIG_C, value)

    @property
    @locked
    def ain90_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@90)

        Address: 10080
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_C)  # type: ignore

    @ain90_ef_config_c.setter
    @locked
    def ain90_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@90)

        Address: 10080
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN90_EF_CONFIG_C, value)

    @property
    @locked
    def ain91_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@91)

        Address: 10082
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_C)  # type: ignore

    @ain91_ef_config_c.setter
    @locked
    def ain91_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@91)

        Address: 10082
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN91_EF_CONFIG_C, value)

    @property
    @locked
    def ain92_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@92)

        Address: 10084
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_C)  # type: ignore

    @ain92_ef_config_c.setter
    @locked
    def ain92_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@92)

        Address: 10084
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN92_EF_CONFIG_C, value)

    @property
    @locked
    def ain93_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@93)

        Address: 10086
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_C)  # type: ignore

    @ain93_ef_config_c.setter
    @locked
    def ain93_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@93)

        Address: 10086
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN93_EF_CONFIG_C, value)

    @property
    @locked
    def ain94_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@94)

        Address: 10088
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_C)  # type: ignore

    @ain94_ef_config_c.setter
    @locked
    def ain94_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@94)

        Address: 10088
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN94_EF_CONFIG_C, value)

    @property
    @locked
    def ain95_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@95)

        Address: 10090
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_C)  # type: ignore

    @ain95_ef_config_c.setter
    @locked
    def ain95_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@95)

        Address: 10090
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN95_EF_CONFIG_C, value)

    @property
    @locked
    def ain96_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@96)

        Address: 10092
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_C)  # type: ignore

    @ain96_ef_config_c.setter
    @locked
    def ain96_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@96)

        Address: 10092
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN96_EF_CONFIG_C, value)

    @property
    @locked
    def ain97_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@97)

        Address: 10094
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_C)  # type: ignore

    @ain97_ef_config_c.setter
    @locked
    def ain97_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@97)

        Address: 10094
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN97_EF_CONFIG_C, value)

    @property
    @locked
    def ain98_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@98)

        Address: 10096
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_C)  # type: ignore

    @ain98_ef_config_c.setter
    @locked
    def ain98_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@98)

        Address: 10096
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN98_EF_CONFIG_C, value)

    @property
    @locked
    def ain99_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@99)

        Address: 10098
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_C)  # type: ignore

    @ain99_ef_config_c.setter
    @locked
    def ain99_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@99)

        Address: 10098
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN99_EF_CONFIG_C, value)

    @property
    @locked
    def ain100_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@100)

        Address: 10100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_C)  # type: ignore

    @ain100_ef_config_c.setter
    @locked
    def ain100_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@100)

        Address: 10100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN100_EF_CONFIG_C, value)

    @property
    @locked
    def ain101_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@101)

        Address: 10102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_C)  # type: ignore

    @ain101_ef_config_c.setter
    @locked
    def ain101_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@101)

        Address: 10102
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN101_EF_CONFIG_C, value)

    @property
    @locked
    def ain102_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@102)

        Address: 10104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_C)  # type: ignore

    @ain102_ef_config_c.setter
    @locked
    def ain102_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@102)

        Address: 10104
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN102_EF_CONFIG_C, value)

    @property
    @locked
    def ain103_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@103)

        Address: 10106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_C)  # type: ignore

    @ain103_ef_config_c.setter
    @locked
    def ain103_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@103)

        Address: 10106
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN103_EF_CONFIG_C, value)

    @property
    @locked
    def ain104_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@104)

        Address: 10108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_C)  # type: ignore

    @ain104_ef_config_c.setter
    @locked
    def ain104_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@104)

        Address: 10108
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN104_EF_CONFIG_C, value)

    @property
    @locked
    def ain105_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@105)

        Address: 10110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_C)  # type: ignore

    @ain105_ef_config_c.setter
    @locked
    def ain105_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@105)

        Address: 10110
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN105_EF_CONFIG_C, value)

    @property
    @locked
    def ain106_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@106)

        Address: 10112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_C)  # type: ignore

    @ain106_ef_config_c.setter
    @locked
    def ain106_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@106)

        Address: 10112
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN106_EF_CONFIG_C, value)

    @property
    @locked
    def ain107_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@107)

        Address: 10114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_C)  # type: ignore

    @ain107_ef_config_c.setter
    @locked
    def ain107_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@107)

        Address: 10114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN107_EF_CONFIG_C, value)

    @property
    @locked
    def ain108_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@108)

        Address: 10116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_C)  # type: ignore

    @ain108_ef_config_c.setter
    @locked
    def ain108_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@108)

        Address: 10116
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN108_EF_CONFIG_C, value)

    @property
    @locked
    def ain109_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@109)

        Address: 10118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_C)  # type: ignore

    @ain109_ef_config_c.setter
    @locked
    def ain109_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@109)

        Address: 10118
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN109_EF_CONFIG_C, value)

    @property
    @locked
    def ain110_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@110)

        Address: 10120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_C)  # type: ignore

    @ain110_ef_config_c.setter
    @locked
    def ain110_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@110)

        Address: 10120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN110_EF_CONFIG_C, value)

    @property
    @locked
    def ain111_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@111)

        Address: 10122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_C)  # type: ignore

    @ain111_ef_config_c.setter
    @locked
    def ain111_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@111)

        Address: 10122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN111_EF_CONFIG_C, value)

    @property
    @locked
    def ain112_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@112)

        Address: 10124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_C)  # type: ignore

    @ain112_ef_config_c.setter
    @locked
    def ain112_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@112)

        Address: 10124
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN112_EF_CONFIG_C, value)

    @property
    @locked
    def ain113_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@113)

        Address: 10126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_C)  # type: ignore

    @ain113_ef_config_c.setter
    @locked
    def ain113_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@113)

        Address: 10126
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN113_EF_CONFIG_C, value)

    @property
    @locked
    def ain114_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@114)

        Address: 10128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_C)  # type: ignore

    @ain114_ef_config_c.setter
    @locked
    def ain114_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@114)

        Address: 10128
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN114_EF_CONFIG_C, value)

    @property
    @locked
    def ain115_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@115)

        Address: 10130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_C)  # type: ignore

    @ain115_ef_config_c.setter
    @locked
    def ain115_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@115)

        Address: 10130
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN115_EF_CONFIG_C, value)

    @property
    @locked
    def ain116_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@116)

        Address: 10132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_C)  # type: ignore

    @ain116_ef_config_c.setter
    @locked
    def ain116_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@116)

        Address: 10132
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN116_EF_CONFIG_C, value)

    @property
    @locked
    def ain117_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@117)

        Address: 10134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_C)  # type: ignore

    @ain117_ef_config_c.setter
    @locked
    def ain117_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@117)

        Address: 10134
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN117_EF_CONFIG_C, value)

    @property
    @locked
    def ain118_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@118)

        Address: 10136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_C)  # type: ignore

    @ain118_ef_config_c.setter
    @locked
    def ain118_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@118)

        Address: 10136
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN118_EF_CONFIG_C, value)

    @property
    @locked
    def ain119_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@119)

        Address: 10138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_C)  # type: ignore

    @ain119_ef_config_c.setter
    @locked
    def ain119_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@119)

        Address: 10138
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN119_EF_CONFIG_C, value)

    @property
    @locked
    def ain120_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@120)

        Address: 10140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_C)  # type: ignore

    @ain120_ef_config_c.setter
    @locked
    def ain120_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@120)

        Address: 10140
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN120_EF_CONFIG_C, value)

    @property
    @locked
    def ain121_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@121)

        Address: 10142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_C)  # type: ignore

    @ain121_ef_config_c.setter
    @locked
    def ain121_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@121)

        Address: 10142
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN121_EF_CONFIG_C, value)

    @property
    @locked
    def ain122_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@122)

        Address: 10144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_C)  # type: ignore

    @ain122_ef_config_c.setter
    @locked
    def ain122_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@122)

        Address: 10144
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN122_EF_CONFIG_C, value)

    @property
    @locked
    def ain123_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@123)

        Address: 10146
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_C)  # type: ignore

    @ain123_ef_config_c.setter
    @locked
    def ain123_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@123)

        Address: 10146
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN123_EF_CONFIG_C, value)

    @property
    @locked
    def ain124_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@124)

        Address: 10148
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_C)  # type: ignore

    @ain124_ef_config_c.setter
    @locked
    def ain124_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@124)

        Address: 10148
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN124_EF_CONFIG_C, value)

    @property
    @locked
    def ain125_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@125)

        Address: 10150
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_C)  # type: ignore

    @ain125_ef_config_c.setter
    @locked
    def ain125_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@125)

        Address: 10150
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN125_EF_CONFIG_C, value)

    @property
    @locked
    def ain126_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@126)

        Address: 10152
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_C)  # type: ignore

    @ain126_ef_config_c.setter
    @locked
    def ain126_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@126)

        Address: 10152
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN126_EF_CONFIG_C, value)

    @property
    @locked
    def ain127_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@127)

        Address: 10154
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_C)  # type: ignore

    @ain127_ef_config_c.setter
    @locked
    def ain127_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@127)

        Address: 10154
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN127_EF_CONFIG_C, value)

    @property
    @locked
    def ain128_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@128)

        Address: 10156
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_C)  # type: ignore

    @ain128_ef_config_c.setter
    @locked
    def ain128_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@128)

        Address: 10156
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN128_EF_CONFIG_C, value)

    @property
    @locked
    def ain129_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@129)

        Address: 10158
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_C)  # type: ignore

    @ain129_ef_config_c.setter
    @locked
    def ain129_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@129)

        Address: 10158
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN129_EF_CONFIG_C, value)

    @property
    @locked
    def ain130_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@130)

        Address: 10160
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_C)  # type: ignore

    @ain130_ef_config_c.setter
    @locked
    def ain130_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@130)

        Address: 10160
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN130_EF_CONFIG_C, value)

    @property
    @locked
    def ain131_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@131)

        Address: 10162
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_C)  # type: ignore

    @ain131_ef_config_c.setter
    @locked
    def ain131_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@131)

        Address: 10162
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN131_EF_CONFIG_C, value)

    @property
    @locked
    def ain132_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@132)

        Address: 10164
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_C)  # type: ignore

    @ain132_ef_config_c.setter
    @locked
    def ain132_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@132)

        Address: 10164
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN132_EF_CONFIG_C, value)

    @property
    @locked
    def ain133_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@133)

        Address: 10166
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_C)  # type: ignore

    @ain133_ef_config_c.setter
    @locked
    def ain133_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@133)

        Address: 10166
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN133_EF_CONFIG_C, value)

    @property
    @locked
    def ain134_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@134)

        Address: 10168
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_C)  # type: ignore

    @ain134_ef_config_c.setter
    @locked
    def ain134_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@134)

        Address: 10168
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN134_EF_CONFIG_C, value)

    @property
    @locked
    def ain135_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@135)

        Address: 10170
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_C)  # type: ignore

    @ain135_ef_config_c.setter
    @locked
    def ain135_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@135)

        Address: 10170
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN135_EF_CONFIG_C, value)

    @property
    @locked
    def ain136_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@136)

        Address: 10172
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_C)  # type: ignore

    @ain136_ef_config_c.setter
    @locked
    def ain136_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@136)

        Address: 10172
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN136_EF_CONFIG_C, value)

    @property
    @locked
    def ain137_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@137)

        Address: 10174
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_C)  # type: ignore

    @ain137_ef_config_c.setter
    @locked
    def ain137_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@137)

        Address: 10174
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN137_EF_CONFIG_C, value)

    @property
    @locked
    def ain138_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@138)

        Address: 10176
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_C)  # type: ignore

    @ain138_ef_config_c.setter
    @locked
    def ain138_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@138)

        Address: 10176
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN138_EF_CONFIG_C, value)

    @property
    @locked
    def ain139_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@139)

        Address: 10178
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_C)  # type: ignore

    @ain139_ef_config_c.setter
    @locked
    def ain139_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@139)

        Address: 10178
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN139_EF_CONFIG_C, value)

    @property
    @locked
    def ain140_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@140)

        Address: 10180
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_C)  # type: ignore

    @ain140_ef_config_c.setter
    @locked
    def ain140_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@140)

        Address: 10180
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN140_EF_CONFIG_C, value)

    @property
    @locked
    def ain141_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@141)

        Address: 10182
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_C)  # type: ignore

    @ain141_ef_config_c.setter
    @locked
    def ain141_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@141)

        Address: 10182
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN141_EF_CONFIG_C, value)

    @property
    @locked
    def ain142_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@142)

        Address: 10184
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_C)  # type: ignore

    @ain142_ef_config_c.setter
    @locked
    def ain142_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@142)

        Address: 10184
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN142_EF_CONFIG_C, value)

    @property
    @locked
    def ain143_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@143)

        Address: 10186
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_C)  # type: ignore

    @ain143_ef_config_c.setter
    @locked
    def ain143_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@143)

        Address: 10186
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN143_EF_CONFIG_C, value)

    @property
    @locked
    def ain144_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@144)

        Address: 10188
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_C)  # type: ignore

    @ain144_ef_config_c.setter
    @locked
    def ain144_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@144)

        Address: 10188
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN144_EF_CONFIG_C, value)

    @property
    @locked
    def ain145_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@145)

        Address: 10190
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_C)  # type: ignore

    @ain145_ef_config_c.setter
    @locked
    def ain145_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@145)

        Address: 10190
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN145_EF_CONFIG_C, value)

    @property
    @locked
    def ain146_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@146)

        Address: 10192
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_C)  # type: ignore

    @ain146_ef_config_c.setter
    @locked
    def ain146_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@146)

        Address: 10192
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN146_EF_CONFIG_C, value)

    @property
    @locked
    def ain147_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@147)

        Address: 10194
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_C)  # type: ignore

    @ain147_ef_config_c.setter
    @locked
    def ain147_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@147)

        Address: 10194
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN147_EF_CONFIG_C, value)

    @property
    @locked
    def ain148_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@148)

        Address: 10196
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_C)  # type: ignore

    @ain148_ef_config_c.setter
    @locked
    def ain148_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@148)

        Address: 10196
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN148_EF_CONFIG_C, value)

    @property
    @locked
    def ain149_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@149)

        Address: 10198
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_C)  # type: ignore

    @ain149_ef_config_c.setter
    @locked
    def ain149_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_C (AIN#(0:149)_EF_CONFIG_C@149)

        Address: 10198
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN149_EF_CONFIG_C, value)

    @property
    @locked
    def ain0_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@0)

        Address: 10200
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_D)  # type: ignore

    @ain0_ef_config_d.setter
    @locked
    def ain0_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@0)

        Address: 10200
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_D, value)

    @property
    @locked
    def ain1_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@1)

        Address: 10202
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_D)  # type: ignore

    @ain1_ef_config_d.setter
    @locked
    def ain1_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@1)

        Address: 10202
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_D, value)

    @property
    @locked
    def ain2_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@2)

        Address: 10204
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_D)  # type: ignore

    @ain2_ef_config_d.setter
    @locked
    def ain2_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@2)

        Address: 10204
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_D, value)

    @property
    @locked
    def ain3_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@3)

        Address: 10206
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_D)  # type: ignore

    @ain3_ef_config_d.setter
    @locked
    def ain3_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@3)

        Address: 10206
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_D, value)

    @property
    @locked
    def ain4_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@4)

        Address: 10208
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_D)  # type: ignore

    @ain4_ef_config_d.setter
    @locked
    def ain4_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@4)

        Address: 10208
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_D, value)

    @property
    @locked
    def ain5_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@5)

        Address: 10210
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_D)  # type: ignore

    @ain5_ef_config_d.setter
    @locked
    def ain5_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@5)

        Address: 10210
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_D, value)

    @property
    @locked
    def ain6_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@6)

        Address: 10212
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_D)  # type: ignore

    @ain6_ef_config_d.setter
    @locked
    def ain6_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@6)

        Address: 10212
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_D, value)

    @property
    @locked
    def ain7_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@7)

        Address: 10214
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_D)  # type: ignore

    @ain7_ef_config_d.setter
    @locked
    def ain7_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@7)

        Address: 10214
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_D, value)

    @property
    @locked
    def ain8_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@8)

        Address: 10216
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_D)  # type: ignore

    @ain8_ef_config_d.setter
    @locked
    def ain8_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@8)

        Address: 10216
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_D, value)

    @property
    @locked
    def ain9_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@9)

        Address: 10218
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_D)  # type: ignore

    @ain9_ef_config_d.setter
    @locked
    def ain9_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@9)

        Address: 10218
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_D, value)

    @property
    @locked
    def ain10_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@10)

        Address: 10220
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_D)  # type: ignore

    @ain10_ef_config_d.setter
    @locked
    def ain10_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@10)

        Address: 10220
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_D, value)

    @property
    @locked
    def ain11_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@11)

        Address: 10222
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_D)  # type: ignore

    @ain11_ef_config_d.setter
    @locked
    def ain11_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@11)

        Address: 10222
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_D, value)

    @property
    @locked
    def ain12_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@12)

        Address: 10224
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_D)  # type: ignore

    @ain12_ef_config_d.setter
    @locked
    def ain12_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@12)

        Address: 10224
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_D, value)

    @property
    @locked
    def ain13_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@13)

        Address: 10226
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_D)  # type: ignore

    @ain13_ef_config_d.setter
    @locked
    def ain13_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@13)

        Address: 10226
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_D, value)

    @property
    @locked
    def ain14_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@14)

        Address: 10228
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_D)  # type: ignore

    @ain14_ef_config_d.setter
    @locked
    def ain14_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@14)

        Address: 10228
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_D, value)

    @property
    @locked
    def ain15_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@15)

        Address: 10230
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_D)  # type: ignore

    @ain15_ef_config_d.setter
    @locked
    def ain15_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@15)

        Address: 10230
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_D, value)

    @property
    @locked
    def ain16_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@16)

        Address: 10232
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_D)  # type: ignore

    @ain16_ef_config_d.setter
    @locked
    def ain16_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@16)

        Address: 10232
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_D, value)

    @property
    @locked
    def ain17_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@17)

        Address: 10234
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_D)  # type: ignore

    @ain17_ef_config_d.setter
    @locked
    def ain17_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@17)

        Address: 10234
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_D, value)

    @property
    @locked
    def ain18_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@18)

        Address: 10236
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_D)  # type: ignore

    @ain18_ef_config_d.setter
    @locked
    def ain18_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@18)

        Address: 10236
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_D, value)

    @property
    @locked
    def ain19_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@19)

        Address: 10238
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_D)  # type: ignore

    @ain19_ef_config_d.setter
    @locked
    def ain19_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@19)

        Address: 10238
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_D, value)

    @property
    @locked
    def ain20_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@20)

        Address: 10240
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_D)  # type: ignore

    @ain20_ef_config_d.setter
    @locked
    def ain20_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@20)

        Address: 10240
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_D, value)

    @property
    @locked
    def ain21_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@21)

        Address: 10242
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_D)  # type: ignore

    @ain21_ef_config_d.setter
    @locked
    def ain21_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@21)

        Address: 10242
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_D, value)

    @property
    @locked
    def ain22_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@22)

        Address: 10244
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_D)  # type: ignore

    @ain22_ef_config_d.setter
    @locked
    def ain22_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@22)

        Address: 10244
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_D, value)

    @property
    @locked
    def ain23_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@23)

        Address: 10246
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_D)  # type: ignore

    @ain23_ef_config_d.setter
    @locked
    def ain23_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@23)

        Address: 10246
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_D, value)

    @property
    @locked
    def ain24_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@24)

        Address: 10248
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_D)  # type: ignore

    @ain24_ef_config_d.setter
    @locked
    def ain24_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@24)

        Address: 10248
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_D, value)

    @property
    @locked
    def ain25_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@25)

        Address: 10250
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_D)  # type: ignore

    @ain25_ef_config_d.setter
    @locked
    def ain25_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@25)

        Address: 10250
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_D, value)

    @property
    @locked
    def ain26_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@26)

        Address: 10252
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_D)  # type: ignore

    @ain26_ef_config_d.setter
    @locked
    def ain26_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@26)

        Address: 10252
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_D, value)

    @property
    @locked
    def ain27_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@27)

        Address: 10254
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_D)  # type: ignore

    @ain27_ef_config_d.setter
    @locked
    def ain27_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@27)

        Address: 10254
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_D, value)

    @property
    @locked
    def ain28_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@28)

        Address: 10256
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_D)  # type: ignore

    @ain28_ef_config_d.setter
    @locked
    def ain28_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@28)

        Address: 10256
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_D, value)

    @property
    @locked
    def ain29_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@29)

        Address: 10258
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_D)  # type: ignore

    @ain29_ef_config_d.setter
    @locked
    def ain29_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@29)

        Address: 10258
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_D, value)

    @property
    @locked
    def ain30_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@30)

        Address: 10260
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_D)  # type: ignore

    @ain30_ef_config_d.setter
    @locked
    def ain30_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@30)

        Address: 10260
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_D, value)

    @property
    @locked
    def ain31_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@31)

        Address: 10262
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_D)  # type: ignore

    @ain31_ef_config_d.setter
    @locked
    def ain31_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@31)

        Address: 10262
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_D, value)

    @property
    @locked
    def ain32_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@32)

        Address: 10264
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_D)  # type: ignore

    @ain32_ef_config_d.setter
    @locked
    def ain32_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@32)

        Address: 10264
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_D, value)

    @property
    @locked
    def ain33_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@33)

        Address: 10266
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_D)  # type: ignore

    @ain33_ef_config_d.setter
    @locked
    def ain33_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@33)

        Address: 10266
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_D, value)

    @property
    @locked
    def ain34_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@34)

        Address: 10268
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_D)  # type: ignore

    @ain34_ef_config_d.setter
    @locked
    def ain34_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@34)

        Address: 10268
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_D, value)

    @property
    @locked
    def ain35_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@35)

        Address: 10270
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_D)  # type: ignore

    @ain35_ef_config_d.setter
    @locked
    def ain35_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@35)

        Address: 10270
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_D, value)

    @property
    @locked
    def ain36_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@36)

        Address: 10272
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_D)  # type: ignore

    @ain36_ef_config_d.setter
    @locked
    def ain36_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@36)

        Address: 10272
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_D, value)

    @property
    @locked
    def ain37_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@37)

        Address: 10274
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_D)  # type: ignore

    @ain37_ef_config_d.setter
    @locked
    def ain37_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@37)

        Address: 10274
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_D, value)

    @property
    @locked
    def ain38_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@38)

        Address: 10276
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_D)  # type: ignore

    @ain38_ef_config_d.setter
    @locked
    def ain38_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@38)

        Address: 10276
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_D, value)

    @property
    @locked
    def ain39_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@39)

        Address: 10278
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_D)  # type: ignore

    @ain39_ef_config_d.setter
    @locked
    def ain39_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@39)

        Address: 10278
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_D, value)

    @property
    @locked
    def ain40_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@40)

        Address: 10280
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_D)  # type: ignore

    @ain40_ef_config_d.setter
    @locked
    def ain40_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@40)

        Address: 10280
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_D, value)

    @property
    @locked
    def ain41_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@41)

        Address: 10282
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_D)  # type: ignore

    @ain41_ef_config_d.setter
    @locked
    def ain41_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@41)

        Address: 10282
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_D, value)

    @property
    @locked
    def ain42_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@42)

        Address: 10284
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_D)  # type: ignore

    @ain42_ef_config_d.setter
    @locked
    def ain42_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@42)

        Address: 10284
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_D, value)

    @property
    @locked
    def ain43_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@43)

        Address: 10286
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_D)  # type: ignore

    @ain43_ef_config_d.setter
    @locked
    def ain43_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@43)

        Address: 10286
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_D, value)

    @property
    @locked
    def ain44_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@44)

        Address: 10288
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_D)  # type: ignore

    @ain44_ef_config_d.setter
    @locked
    def ain44_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@44)

        Address: 10288
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_D, value)

    @property
    @locked
    def ain45_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@45)

        Address: 10290
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_D)  # type: ignore

    @ain45_ef_config_d.setter
    @locked
    def ain45_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@45)

        Address: 10290
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_D, value)

    @property
    @locked
    def ain46_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@46)

        Address: 10292
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_D)  # type: ignore

    @ain46_ef_config_d.setter
    @locked
    def ain46_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@46)

        Address: 10292
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_D, value)

    @property
    @locked
    def ain47_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@47)

        Address: 10294
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_D)  # type: ignore

    @ain47_ef_config_d.setter
    @locked
    def ain47_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@47)

        Address: 10294
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_D, value)

    @property
    @locked
    def ain48_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@48)

        Address: 10296
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_D)  # type: ignore

    @ain48_ef_config_d.setter
    @locked
    def ain48_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@48)

        Address: 10296
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_D, value)

    @property
    @locked
    def ain49_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@49)

        Address: 10298
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_D)  # type: ignore

    @ain49_ef_config_d.setter
    @locked
    def ain49_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@49)

        Address: 10298
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_D, value)

    @property
    @locked
    def ain50_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@50)

        Address: 10300
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_D)  # type: ignore

    @ain50_ef_config_d.setter
    @locked
    def ain50_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@50)

        Address: 10300
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_D, value)

    @property
    @locked
    def ain51_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@51)

        Address: 10302
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_D)  # type: ignore

    @ain51_ef_config_d.setter
    @locked
    def ain51_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@51)

        Address: 10302
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_D, value)

    @property
    @locked
    def ain52_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@52)

        Address: 10304
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_D)  # type: ignore

    @ain52_ef_config_d.setter
    @locked
    def ain52_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@52)

        Address: 10304
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_D, value)

    @property
    @locked
    def ain53_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@53)

        Address: 10306
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_D)  # type: ignore

    @ain53_ef_config_d.setter
    @locked
    def ain53_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@53)

        Address: 10306
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_D, value)

    @property
    @locked
    def ain54_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@54)

        Address: 10308
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_D)  # type: ignore

    @ain54_ef_config_d.setter
    @locked
    def ain54_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@54)

        Address: 10308
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_D, value)

    @property
    @locked
    def ain55_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@55)

        Address: 10310
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_D)  # type: ignore

    @ain55_ef_config_d.setter
    @locked
    def ain55_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@55)

        Address: 10310
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_D, value)

    @property
    @locked
    def ain56_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@56)

        Address: 10312
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_D)  # type: ignore

    @ain56_ef_config_d.setter
    @locked
    def ain56_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@56)

        Address: 10312
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_D, value)

    @property
    @locked
    def ain57_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@57)

        Address: 10314
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_D)  # type: ignore

    @ain57_ef_config_d.setter
    @locked
    def ain57_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@57)

        Address: 10314
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_D, value)

    @property
    @locked
    def ain58_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@58)

        Address: 10316
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_D)  # type: ignore

    @ain58_ef_config_d.setter
    @locked
    def ain58_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@58)

        Address: 10316
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_D, value)

    @property
    @locked
    def ain59_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@59)

        Address: 10318
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_D)  # type: ignore

    @ain59_ef_config_d.setter
    @locked
    def ain59_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@59)

        Address: 10318
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_D, value)

    @property
    @locked
    def ain60_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@60)

        Address: 10320
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_D)  # type: ignore

    @ain60_ef_config_d.setter
    @locked
    def ain60_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@60)

        Address: 10320
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_D, value)

    @property
    @locked
    def ain61_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@61)

        Address: 10322
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_D)  # type: ignore

    @ain61_ef_config_d.setter
    @locked
    def ain61_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@61)

        Address: 10322
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_D, value)

    @property
    @locked
    def ain62_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@62)

        Address: 10324
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_D)  # type: ignore

    @ain62_ef_config_d.setter
    @locked
    def ain62_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@62)

        Address: 10324
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_D, value)

    @property
    @locked
    def ain63_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@63)

        Address: 10326
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_D)  # type: ignore

    @ain63_ef_config_d.setter
    @locked
    def ain63_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@63)

        Address: 10326
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_D, value)

    @property
    @locked
    def ain64_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@64)

        Address: 10328
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_D)  # type: ignore

    @ain64_ef_config_d.setter
    @locked
    def ain64_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@64)

        Address: 10328
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_D, value)

    @property
    @locked
    def ain65_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@65)

        Address: 10330
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_D)  # type: ignore

    @ain65_ef_config_d.setter
    @locked
    def ain65_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@65)

        Address: 10330
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_D, value)

    @property
    @locked
    def ain66_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@66)

        Address: 10332
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_D)  # type: ignore

    @ain66_ef_config_d.setter
    @locked
    def ain66_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@66)

        Address: 10332
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_D, value)

    @property
    @locked
    def ain67_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@67)

        Address: 10334
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_D)  # type: ignore

    @ain67_ef_config_d.setter
    @locked
    def ain67_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@67)

        Address: 10334
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_D, value)

    @property
    @locked
    def ain68_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@68)

        Address: 10336
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_D)  # type: ignore

    @ain68_ef_config_d.setter
    @locked
    def ain68_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@68)

        Address: 10336
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_D, value)

    @property
    @locked
    def ain69_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@69)

        Address: 10338
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_D)  # type: ignore

    @ain69_ef_config_d.setter
    @locked
    def ain69_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@69)

        Address: 10338
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_D, value)

    @property
    @locked
    def ain70_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@70)

        Address: 10340
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_D)  # type: ignore

    @ain70_ef_config_d.setter
    @locked
    def ain70_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@70)

        Address: 10340
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_D, value)

    @property
    @locked
    def ain71_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@71)

        Address: 10342
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_D)  # type: ignore

    @ain71_ef_config_d.setter
    @locked
    def ain71_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@71)

        Address: 10342
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_D, value)

    @property
    @locked
    def ain72_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@72)

        Address: 10344
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_D)  # type: ignore

    @ain72_ef_config_d.setter
    @locked
    def ain72_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@72)

        Address: 10344
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_D, value)

    @property
    @locked
    def ain73_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@73)

        Address: 10346
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_D)  # type: ignore

    @ain73_ef_config_d.setter
    @locked
    def ain73_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@73)

        Address: 10346
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_D, value)

    @property
    @locked
    def ain74_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@74)

        Address: 10348
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_D)  # type: ignore

    @ain74_ef_config_d.setter
    @locked
    def ain74_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@74)

        Address: 10348
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_D, value)

    @property
    @locked
    def ain75_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@75)

        Address: 10350
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_D)  # type: ignore

    @ain75_ef_config_d.setter
    @locked
    def ain75_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@75)

        Address: 10350
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_D, value)

    @property
    @locked
    def ain76_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@76)

        Address: 10352
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_D)  # type: ignore

    @ain76_ef_config_d.setter
    @locked
    def ain76_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@76)

        Address: 10352
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_D, value)

    @property
    @locked
    def ain77_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@77)

        Address: 10354
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_D)  # type: ignore

    @ain77_ef_config_d.setter
    @locked
    def ain77_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@77)

        Address: 10354
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_D, value)

    @property
    @locked
    def ain78_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@78)

        Address: 10356
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_D)  # type: ignore

    @ain78_ef_config_d.setter
    @locked
    def ain78_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@78)

        Address: 10356
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_D, value)

    @property
    @locked
    def ain79_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@79)

        Address: 10358
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_D)  # type: ignore

    @ain79_ef_config_d.setter
    @locked
    def ain79_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@79)

        Address: 10358
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_D, value)

    @property
    @locked
    def ain80_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@80)

        Address: 10360
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_D)  # type: ignore

    @ain80_ef_config_d.setter
    @locked
    def ain80_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@80)

        Address: 10360
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_D, value)

    @property
    @locked
    def ain81_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@81)

        Address: 10362
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_D)  # type: ignore

    @ain81_ef_config_d.setter
    @locked
    def ain81_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@81)

        Address: 10362
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_D, value)

    @property
    @locked
    def ain82_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@82)

        Address: 10364
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_D)  # type: ignore

    @ain82_ef_config_d.setter
    @locked
    def ain82_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@82)

        Address: 10364
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_D, value)

    @property
    @locked
    def ain83_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@83)

        Address: 10366
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_D)  # type: ignore

    @ain83_ef_config_d.setter
    @locked
    def ain83_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@83)

        Address: 10366
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_D, value)

    @property
    @locked
    def ain84_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@84)

        Address: 10368
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_D)  # type: ignore

    @ain84_ef_config_d.setter
    @locked
    def ain84_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@84)

        Address: 10368
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_D, value)

    @property
    @locked
    def ain85_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@85)

        Address: 10370
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_D)  # type: ignore

    @ain85_ef_config_d.setter
    @locked
    def ain85_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@85)

        Address: 10370
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_D, value)

    @property
    @locked
    def ain86_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@86)

        Address: 10372
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_D)  # type: ignore

    @ain86_ef_config_d.setter
    @locked
    def ain86_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@86)

        Address: 10372
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_D, value)

    @property
    @locked
    def ain87_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@87)

        Address: 10374
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_D)  # type: ignore

    @ain87_ef_config_d.setter
    @locked
    def ain87_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@87)

        Address: 10374
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_D, value)

    @property
    @locked
    def ain88_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@88)

        Address: 10376
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_D)  # type: ignore

    @ain88_ef_config_d.setter
    @locked
    def ain88_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@88)

        Address: 10376
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_D, value)

    @property
    @locked
    def ain89_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@89)

        Address: 10378
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_D)  # type: ignore

    @ain89_ef_config_d.setter
    @locked
    def ain89_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@89)

        Address: 10378
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_D, value)

    @property
    @locked
    def ain90_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@90)

        Address: 10380
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_D)  # type: ignore

    @ain90_ef_config_d.setter
    @locked
    def ain90_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@90)

        Address: 10380
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_D, value)

    @property
    @locked
    def ain91_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@91)

        Address: 10382
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_D)  # type: ignore

    @ain91_ef_config_d.setter
    @locked
    def ain91_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@91)

        Address: 10382
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_D, value)

    @property
    @locked
    def ain92_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@92)

        Address: 10384
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_D)  # type: ignore

    @ain92_ef_config_d.setter
    @locked
    def ain92_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@92)

        Address: 10384
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_D, value)

    @property
    @locked
    def ain93_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@93)

        Address: 10386
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_D)  # type: ignore

    @ain93_ef_config_d.setter
    @locked
    def ain93_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@93)

        Address: 10386
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_D, value)

    @property
    @locked
    def ain94_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@94)

        Address: 10388
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_D)  # type: ignore

    @ain94_ef_config_d.setter
    @locked
    def ain94_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@94)

        Address: 10388
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_D, value)

    @property
    @locked
    def ain95_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@95)

        Address: 10390
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_D)  # type: ignore

    @ain95_ef_config_d.setter
    @locked
    def ain95_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@95)

        Address: 10390
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_D, value)

    @property
    @locked
    def ain96_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@96)

        Address: 10392
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_D)  # type: ignore

    @ain96_ef_config_d.setter
    @locked
    def ain96_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@96)

        Address: 10392
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_D, value)

    @property
    @locked
    def ain97_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@97)

        Address: 10394
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_D)  # type: ignore

    @ain97_ef_config_d.setter
    @locked
    def ain97_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@97)

        Address: 10394
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_D, value)

    @property
    @locked
    def ain98_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@98)

        Address: 10396
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_D)  # type: ignore

    @ain98_ef_config_d.setter
    @locked
    def ain98_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@98)

        Address: 10396
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_D, value)

    @property
    @locked
    def ain99_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@99)

        Address: 10398
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_D)  # type: ignore

    @ain99_ef_config_d.setter
    @locked
    def ain99_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@99)

        Address: 10398
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_D, value)

    @property
    @locked
    def ain100_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@100)

        Address: 10400
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_D)  # type: ignore

    @ain100_ef_config_d.setter
    @locked
    def ain100_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@100)

        Address: 10400
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_D, value)

    @property
    @locked
    def ain101_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@101)

        Address: 10402
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_D)  # type: ignore

    @ain101_ef_config_d.setter
    @locked
    def ain101_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@101)

        Address: 10402
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_D, value)

    @property
    @locked
    def ain102_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@102)

        Address: 10404
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_D)  # type: ignore

    @ain102_ef_config_d.setter
    @locked
    def ain102_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@102)

        Address: 10404
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_D, value)

    @property
    @locked
    def ain103_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@103)

        Address: 10406
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_D)  # type: ignore

    @ain103_ef_config_d.setter
    @locked
    def ain103_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@103)

        Address: 10406
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_D, value)

    @property
    @locked
    def ain104_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@104)

        Address: 10408
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_D)  # type: ignore

    @ain104_ef_config_d.setter
    @locked
    def ain104_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@104)

        Address: 10408
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_D, value)

    @property
    @locked
    def ain105_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@105)

        Address: 10410
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_D)  # type: ignore

    @ain105_ef_config_d.setter
    @locked
    def ain105_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@105)

        Address: 10410
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_D, value)

    @property
    @locked
    def ain106_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@106)

        Address: 10412
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_D)  # type: ignore

    @ain106_ef_config_d.setter
    @locked
    def ain106_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@106)

        Address: 10412
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_D, value)

    @property
    @locked
    def ain107_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@107)

        Address: 10414
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_D)  # type: ignore

    @ain107_ef_config_d.setter
    @locked
    def ain107_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@107)

        Address: 10414
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_D, value)

    @property
    @locked
    def ain108_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@108)

        Address: 10416
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_D)  # type: ignore

    @ain108_ef_config_d.setter
    @locked
    def ain108_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@108)

        Address: 10416
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_D, value)

    @property
    @locked
    def ain109_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@109)

        Address: 10418
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_D)  # type: ignore

    @ain109_ef_config_d.setter
    @locked
    def ain109_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@109)

        Address: 10418
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_D, value)

    @property
    @locked
    def ain110_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@110)

        Address: 10420
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_D)  # type: ignore

    @ain110_ef_config_d.setter
    @locked
    def ain110_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@110)

        Address: 10420
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_D, value)

    @property
    @locked
    def ain111_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@111)

        Address: 10422
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_D)  # type: ignore

    @ain111_ef_config_d.setter
    @locked
    def ain111_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@111)

        Address: 10422
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_D, value)

    @property
    @locked
    def ain112_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@112)

        Address: 10424
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_D)  # type: ignore

    @ain112_ef_config_d.setter
    @locked
    def ain112_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@112)

        Address: 10424
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_D, value)

    @property
    @locked
    def ain113_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@113)

        Address: 10426
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_D)  # type: ignore

    @ain113_ef_config_d.setter
    @locked
    def ain113_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@113)

        Address: 10426
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_D, value)

    @property
    @locked
    def ain114_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@114)

        Address: 10428
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_D)  # type: ignore

    @ain114_ef_config_d.setter
    @locked
    def ain114_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@114)

        Address: 10428
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_D, value)

    @property
    @locked
    def ain115_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@115)

        Address: 10430
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_D)  # type: ignore

    @ain115_ef_config_d.setter
    @locked
    def ain115_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@115)

        Address: 10430
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_D, value)

    @property
    @locked
    def ain116_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@116)

        Address: 10432
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_D)  # type: ignore

    @ain116_ef_config_d.setter
    @locked
    def ain116_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@116)

        Address: 10432
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_D, value)

    @property
    @locked
    def ain117_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@117)

        Address: 10434
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_D)  # type: ignore

    @ain117_ef_config_d.setter
    @locked
    def ain117_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@117)

        Address: 10434
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_D, value)

    @property
    @locked
    def ain118_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@118)

        Address: 10436
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_D)  # type: ignore

    @ain118_ef_config_d.setter
    @locked
    def ain118_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@118)

        Address: 10436
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_D, value)

    @property
    @locked
    def ain119_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@119)

        Address: 10438
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_D)  # type: ignore

    @ain119_ef_config_d.setter
    @locked
    def ain119_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@119)

        Address: 10438
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_D, value)

    @property
    @locked
    def ain120_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@120)

        Address: 10440
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_D)  # type: ignore

    @ain120_ef_config_d.setter
    @locked
    def ain120_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@120)

        Address: 10440
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_D, value)

    @property
    @locked
    def ain121_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@121)

        Address: 10442
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_D)  # type: ignore

    @ain121_ef_config_d.setter
    @locked
    def ain121_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@121)

        Address: 10442
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_D, value)

    @property
    @locked
    def ain122_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@122)

        Address: 10444
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_D)  # type: ignore

    @ain122_ef_config_d.setter
    @locked
    def ain122_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@122)

        Address: 10444
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_D, value)

    @property
    @locked
    def ain123_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@123)

        Address: 10446
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_D)  # type: ignore

    @ain123_ef_config_d.setter
    @locked
    def ain123_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@123)

        Address: 10446
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_D, value)

    @property
    @locked
    def ain124_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@124)

        Address: 10448
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_D)  # type: ignore

    @ain124_ef_config_d.setter
    @locked
    def ain124_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@124)

        Address: 10448
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_D, value)

    @property
    @locked
    def ain125_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@125)

        Address: 10450
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_D)  # type: ignore

    @ain125_ef_config_d.setter
    @locked
    def ain125_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@125)

        Address: 10450
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_D, value)

    @property
    @locked
    def ain126_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@126)

        Address: 10452
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_D)  # type: ignore

    @ain126_ef_config_d.setter
    @locked
    def ain126_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@126)

        Address: 10452
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_D, value)

    @property
    @locked
    def ain127_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@127)

        Address: 10454
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_D)  # type: ignore

    @ain127_ef_config_d.setter
    @locked
    def ain127_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@127)

        Address: 10454
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_D, value)

    @property
    @locked
    def ain128_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@128)

        Address: 10456
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_D)  # type: ignore

    @ain128_ef_config_d.setter
    @locked
    def ain128_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@128)

        Address: 10456
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_D, value)

    @property
    @locked
    def ain129_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@129)

        Address: 10458
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_D)  # type: ignore

    @ain129_ef_config_d.setter
    @locked
    def ain129_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@129)

        Address: 10458
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_D, value)

    @property
    @locked
    def ain130_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@130)

        Address: 10460
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_D)  # type: ignore

    @ain130_ef_config_d.setter
    @locked
    def ain130_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@130)

        Address: 10460
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_D, value)

    @property
    @locked
    def ain131_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@131)

        Address: 10462
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_D)  # type: ignore

    @ain131_ef_config_d.setter
    @locked
    def ain131_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@131)

        Address: 10462
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_D, value)

    @property
    @locked
    def ain132_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@132)

        Address: 10464
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_D)  # type: ignore

    @ain132_ef_config_d.setter
    @locked
    def ain132_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@132)

        Address: 10464
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_D, value)

    @property
    @locked
    def ain133_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@133)

        Address: 10466
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_D)  # type: ignore

    @ain133_ef_config_d.setter
    @locked
    def ain133_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@133)

        Address: 10466
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_D, value)

    @property
    @locked
    def ain134_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@134)

        Address: 10468
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_D)  # type: ignore

    @ain134_ef_config_d.setter
    @locked
    def ain134_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@134)

        Address: 10468
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_D, value)

    @property
    @locked
    def ain135_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@135)

        Address: 10470
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_D)  # type: ignore

    @ain135_ef_config_d.setter
    @locked
    def ain135_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@135)

        Address: 10470
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_D, value)

    @property
    @locked
    def ain136_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@136)

        Address: 10472
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_D)  # type: ignore

    @ain136_ef_config_d.setter
    @locked
    def ain136_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@136)

        Address: 10472
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_D, value)

    @property
    @locked
    def ain137_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@137)

        Address: 10474
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_D)  # type: ignore

    @ain137_ef_config_d.setter
    @locked
    def ain137_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@137)

        Address: 10474
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_D, value)

    @property
    @locked
    def ain138_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@138)

        Address: 10476
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_D)  # type: ignore

    @ain138_ef_config_d.setter
    @locked
    def ain138_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@138)

        Address: 10476
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_D, value)

    @property
    @locked
    def ain139_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@139)

        Address: 10478
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_D)  # type: ignore

    @ain139_ef_config_d.setter
    @locked
    def ain139_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@139)

        Address: 10478
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_D, value)

    @property
    @locked
    def ain140_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@140)

        Address: 10480
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_D)  # type: ignore

    @ain140_ef_config_d.setter
    @locked
    def ain140_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@140)

        Address: 10480
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_D, value)

    @property
    @locked
    def ain141_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@141)

        Address: 10482
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_D)  # type: ignore

    @ain141_ef_config_d.setter
    @locked
    def ain141_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@141)

        Address: 10482
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_D, value)

    @property
    @locked
    def ain142_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@142)

        Address: 10484
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_D)  # type: ignore

    @ain142_ef_config_d.setter
    @locked
    def ain142_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@142)

        Address: 10484
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_D, value)

    @property
    @locked
    def ain143_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@143)

        Address: 10486
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_D)  # type: ignore

    @ain143_ef_config_d.setter
    @locked
    def ain143_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@143)

        Address: 10486
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_D, value)

    @property
    @locked
    def ain144_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@144)

        Address: 10488
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_D)  # type: ignore

    @ain144_ef_config_d.setter
    @locked
    def ain144_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@144)

        Address: 10488
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_D, value)

    @property
    @locked
    def ain145_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@145)

        Address: 10490
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_D)  # type: ignore

    @ain145_ef_config_d.setter
    @locked
    def ain145_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@145)

        Address: 10490
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_D, value)

    @property
    @locked
    def ain146_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@146)

        Address: 10492
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_D)  # type: ignore

    @ain146_ef_config_d.setter
    @locked
    def ain146_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@146)

        Address: 10492
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_D, value)

    @property
    @locked
    def ain147_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@147)

        Address: 10494
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_D)  # type: ignore

    @ain147_ef_config_d.setter
    @locked
    def ain147_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@147)

        Address: 10494
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_D, value)

    @property
    @locked
    def ain148_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@148)

        Address: 10496
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_D)  # type: ignore

    @ain148_ef_config_d.setter
    @locked
    def ain148_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@148)

        Address: 10496
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_D, value)

    @property
    @locked
    def ain149_ef_config_d(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@149)

        Address: 10498
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_D)  # type: ignore

    @ain149_ef_config_d.setter
    @locked
    def ain149_ef_config_d(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_D (AIN#(0:149)_EF_CONFIG_D@149)

        Address: 10498
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_D, value)

    @property
    @locked
    def ain0_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@0)

        Address: 10500
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_E)  # type: ignore

    @ain0_ef_config_e.setter
    @locked
    def ain0_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@0)

        Address: 10500
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_E, value)

    @property
    @locked
    def ain1_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@1)

        Address: 10502
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_E)  # type: ignore

    @ain1_ef_config_e.setter
    @locked
    def ain1_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@1)

        Address: 10502
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_E, value)

    @property
    @locked
    def ain2_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@2)

        Address: 10504
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_E)  # type: ignore

    @ain2_ef_config_e.setter
    @locked
    def ain2_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@2)

        Address: 10504
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_E, value)

    @property
    @locked
    def ain3_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@3)

        Address: 10506
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_E)  # type: ignore

    @ain3_ef_config_e.setter
    @locked
    def ain3_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@3)

        Address: 10506
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_E, value)

    @property
    @locked
    def ain4_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@4)

        Address: 10508
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_E)  # type: ignore

    @ain4_ef_config_e.setter
    @locked
    def ain4_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@4)

        Address: 10508
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_E, value)

    @property
    @locked
    def ain5_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@5)

        Address: 10510
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_E)  # type: ignore

    @ain5_ef_config_e.setter
    @locked
    def ain5_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@5)

        Address: 10510
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_E, value)

    @property
    @locked
    def ain6_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@6)

        Address: 10512
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_E)  # type: ignore

    @ain6_ef_config_e.setter
    @locked
    def ain6_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@6)

        Address: 10512
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_E, value)

    @property
    @locked
    def ain7_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@7)

        Address: 10514
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_E)  # type: ignore

    @ain7_ef_config_e.setter
    @locked
    def ain7_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@7)

        Address: 10514
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_E, value)

    @property
    @locked
    def ain8_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@8)

        Address: 10516
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_E)  # type: ignore

    @ain8_ef_config_e.setter
    @locked
    def ain8_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@8)

        Address: 10516
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_E, value)

    @property
    @locked
    def ain9_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@9)

        Address: 10518
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_E)  # type: ignore

    @ain9_ef_config_e.setter
    @locked
    def ain9_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@9)

        Address: 10518
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_E, value)

    @property
    @locked
    def ain10_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@10)

        Address: 10520
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_E)  # type: ignore

    @ain10_ef_config_e.setter
    @locked
    def ain10_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@10)

        Address: 10520
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_E, value)

    @property
    @locked
    def ain11_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@11)

        Address: 10522
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_E)  # type: ignore

    @ain11_ef_config_e.setter
    @locked
    def ain11_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@11)

        Address: 10522
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_E, value)

    @property
    @locked
    def ain12_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@12)

        Address: 10524
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_E)  # type: ignore

    @ain12_ef_config_e.setter
    @locked
    def ain12_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@12)

        Address: 10524
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_E, value)

    @property
    @locked
    def ain13_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@13)

        Address: 10526
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_E)  # type: ignore

    @ain13_ef_config_e.setter
    @locked
    def ain13_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@13)

        Address: 10526
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_E, value)

    @property
    @locked
    def ain14_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@14)

        Address: 10528
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_E)  # type: ignore

    @ain14_ef_config_e.setter
    @locked
    def ain14_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@14)

        Address: 10528
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_E, value)

    @property
    @locked
    def ain15_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@15)

        Address: 10530
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_E)  # type: ignore

    @ain15_ef_config_e.setter
    @locked
    def ain15_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@15)

        Address: 10530
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_E, value)

    @property
    @locked
    def ain16_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@16)

        Address: 10532
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_E)  # type: ignore

    @ain16_ef_config_e.setter
    @locked
    def ain16_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@16)

        Address: 10532
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_E, value)

    @property
    @locked
    def ain17_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@17)

        Address: 10534
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_E)  # type: ignore

    @ain17_ef_config_e.setter
    @locked
    def ain17_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@17)

        Address: 10534
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_E, value)

    @property
    @locked
    def ain18_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@18)

        Address: 10536
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_E)  # type: ignore

    @ain18_ef_config_e.setter
    @locked
    def ain18_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@18)

        Address: 10536
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_E, value)

    @property
    @locked
    def ain19_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@19)

        Address: 10538
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_E)  # type: ignore

    @ain19_ef_config_e.setter
    @locked
    def ain19_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@19)

        Address: 10538
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_E, value)

    @property
    @locked
    def ain20_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@20)

        Address: 10540
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_E)  # type: ignore

    @ain20_ef_config_e.setter
    @locked
    def ain20_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@20)

        Address: 10540
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_E, value)

    @property
    @locked
    def ain21_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@21)

        Address: 10542
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_E)  # type: ignore

    @ain21_ef_config_e.setter
    @locked
    def ain21_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@21)

        Address: 10542
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_E, value)

    @property
    @locked
    def ain22_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@22)

        Address: 10544
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_E)  # type: ignore

    @ain22_ef_config_e.setter
    @locked
    def ain22_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@22)

        Address: 10544
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_E, value)

    @property
    @locked
    def ain23_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@23)

        Address: 10546
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_E)  # type: ignore

    @ain23_ef_config_e.setter
    @locked
    def ain23_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@23)

        Address: 10546
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_E, value)

    @property
    @locked
    def ain24_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@24)

        Address: 10548
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_E)  # type: ignore

    @ain24_ef_config_e.setter
    @locked
    def ain24_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@24)

        Address: 10548
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_E, value)

    @property
    @locked
    def ain25_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@25)

        Address: 10550
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_E)  # type: ignore

    @ain25_ef_config_e.setter
    @locked
    def ain25_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@25)

        Address: 10550
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_E, value)

    @property
    @locked
    def ain26_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@26)

        Address: 10552
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_E)  # type: ignore

    @ain26_ef_config_e.setter
    @locked
    def ain26_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@26)

        Address: 10552
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_E, value)

    @property
    @locked
    def ain27_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@27)

        Address: 10554
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_E)  # type: ignore

    @ain27_ef_config_e.setter
    @locked
    def ain27_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@27)

        Address: 10554
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_E, value)

    @property
    @locked
    def ain28_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@28)

        Address: 10556
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_E)  # type: ignore

    @ain28_ef_config_e.setter
    @locked
    def ain28_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@28)

        Address: 10556
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_E, value)

    @property
    @locked
    def ain29_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@29)

        Address: 10558
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_E)  # type: ignore

    @ain29_ef_config_e.setter
    @locked
    def ain29_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@29)

        Address: 10558
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_E, value)

    @property
    @locked
    def ain30_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@30)

        Address: 10560
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_E)  # type: ignore

    @ain30_ef_config_e.setter
    @locked
    def ain30_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@30)

        Address: 10560
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_E, value)

    @property
    @locked
    def ain31_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@31)

        Address: 10562
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_E)  # type: ignore

    @ain31_ef_config_e.setter
    @locked
    def ain31_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@31)

        Address: 10562
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_E, value)

    @property
    @locked
    def ain32_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@32)

        Address: 10564
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_E)  # type: ignore

    @ain32_ef_config_e.setter
    @locked
    def ain32_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@32)

        Address: 10564
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_E, value)

    @property
    @locked
    def ain33_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@33)

        Address: 10566
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_E)  # type: ignore

    @ain33_ef_config_e.setter
    @locked
    def ain33_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@33)

        Address: 10566
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_E, value)

    @property
    @locked
    def ain34_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@34)

        Address: 10568
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_E)  # type: ignore

    @ain34_ef_config_e.setter
    @locked
    def ain34_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@34)

        Address: 10568
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_E, value)

    @property
    @locked
    def ain35_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@35)

        Address: 10570
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_E)  # type: ignore

    @ain35_ef_config_e.setter
    @locked
    def ain35_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@35)

        Address: 10570
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_E, value)

    @property
    @locked
    def ain36_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@36)

        Address: 10572
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_E)  # type: ignore

    @ain36_ef_config_e.setter
    @locked
    def ain36_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@36)

        Address: 10572
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_E, value)

    @property
    @locked
    def ain37_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@37)

        Address: 10574
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_E)  # type: ignore

    @ain37_ef_config_e.setter
    @locked
    def ain37_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@37)

        Address: 10574
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_E, value)

    @property
    @locked
    def ain38_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@38)

        Address: 10576
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_E)  # type: ignore

    @ain38_ef_config_e.setter
    @locked
    def ain38_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@38)

        Address: 10576
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_E, value)

    @property
    @locked
    def ain39_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@39)

        Address: 10578
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_E)  # type: ignore

    @ain39_ef_config_e.setter
    @locked
    def ain39_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@39)

        Address: 10578
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_E, value)

    @property
    @locked
    def ain40_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@40)

        Address: 10580
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_E)  # type: ignore

    @ain40_ef_config_e.setter
    @locked
    def ain40_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@40)

        Address: 10580
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_E, value)

    @property
    @locked
    def ain41_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@41)

        Address: 10582
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_E)  # type: ignore

    @ain41_ef_config_e.setter
    @locked
    def ain41_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@41)

        Address: 10582
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_E, value)

    @property
    @locked
    def ain42_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@42)

        Address: 10584
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_E)  # type: ignore

    @ain42_ef_config_e.setter
    @locked
    def ain42_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@42)

        Address: 10584
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_E, value)

    @property
    @locked
    def ain43_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@43)

        Address: 10586
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_E)  # type: ignore

    @ain43_ef_config_e.setter
    @locked
    def ain43_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@43)

        Address: 10586
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_E, value)

    @property
    @locked
    def ain44_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@44)

        Address: 10588
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_E)  # type: ignore

    @ain44_ef_config_e.setter
    @locked
    def ain44_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@44)

        Address: 10588
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_E, value)

    @property
    @locked
    def ain45_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@45)

        Address: 10590
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_E)  # type: ignore

    @ain45_ef_config_e.setter
    @locked
    def ain45_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@45)

        Address: 10590
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_E, value)

    @property
    @locked
    def ain46_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@46)

        Address: 10592
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_E)  # type: ignore

    @ain46_ef_config_e.setter
    @locked
    def ain46_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@46)

        Address: 10592
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_E, value)

    @property
    @locked
    def ain47_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@47)

        Address: 10594
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_E)  # type: ignore

    @ain47_ef_config_e.setter
    @locked
    def ain47_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@47)

        Address: 10594
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_E, value)

    @property
    @locked
    def ain48_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@48)

        Address: 10596
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_E)  # type: ignore

    @ain48_ef_config_e.setter
    @locked
    def ain48_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@48)

        Address: 10596
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_E, value)

    @property
    @locked
    def ain49_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@49)

        Address: 10598
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_E)  # type: ignore

    @ain49_ef_config_e.setter
    @locked
    def ain49_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@49)

        Address: 10598
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_E, value)

    @property
    @locked
    def ain50_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@50)

        Address: 10600
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_E)  # type: ignore

    @ain50_ef_config_e.setter
    @locked
    def ain50_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@50)

        Address: 10600
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_E, value)

    @property
    @locked
    def ain51_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@51)

        Address: 10602
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_E)  # type: ignore

    @ain51_ef_config_e.setter
    @locked
    def ain51_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@51)

        Address: 10602
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_E, value)

    @property
    @locked
    def ain52_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@52)

        Address: 10604
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_E)  # type: ignore

    @ain52_ef_config_e.setter
    @locked
    def ain52_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@52)

        Address: 10604
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_E, value)

    @property
    @locked
    def ain53_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@53)

        Address: 10606
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_E)  # type: ignore

    @ain53_ef_config_e.setter
    @locked
    def ain53_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@53)

        Address: 10606
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_E, value)

    @property
    @locked
    def ain54_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@54)

        Address: 10608
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_E)  # type: ignore

    @ain54_ef_config_e.setter
    @locked
    def ain54_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@54)

        Address: 10608
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_E, value)

    @property
    @locked
    def ain55_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@55)

        Address: 10610
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_E)  # type: ignore

    @ain55_ef_config_e.setter
    @locked
    def ain55_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@55)

        Address: 10610
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_E, value)

    @property
    @locked
    def ain56_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@56)

        Address: 10612
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_E)  # type: ignore

    @ain56_ef_config_e.setter
    @locked
    def ain56_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@56)

        Address: 10612
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_E, value)

    @property
    @locked
    def ain57_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@57)

        Address: 10614
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_E)  # type: ignore

    @ain57_ef_config_e.setter
    @locked
    def ain57_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@57)

        Address: 10614
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_E, value)

    @property
    @locked
    def ain58_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@58)

        Address: 10616
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_E)  # type: ignore

    @ain58_ef_config_e.setter
    @locked
    def ain58_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@58)

        Address: 10616
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_E, value)

    @property
    @locked
    def ain59_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@59)

        Address: 10618
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_E)  # type: ignore

    @ain59_ef_config_e.setter
    @locked
    def ain59_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@59)

        Address: 10618
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_E, value)

    @property
    @locked
    def ain60_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@60)

        Address: 10620
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_E)  # type: ignore

    @ain60_ef_config_e.setter
    @locked
    def ain60_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@60)

        Address: 10620
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_E, value)

    @property
    @locked
    def ain61_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@61)

        Address: 10622
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_E)  # type: ignore

    @ain61_ef_config_e.setter
    @locked
    def ain61_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@61)

        Address: 10622
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_E, value)

    @property
    @locked
    def ain62_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@62)

        Address: 10624
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_E)  # type: ignore

    @ain62_ef_config_e.setter
    @locked
    def ain62_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@62)

        Address: 10624
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_E, value)

    @property
    @locked
    def ain63_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@63)

        Address: 10626
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_E)  # type: ignore

    @ain63_ef_config_e.setter
    @locked
    def ain63_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@63)

        Address: 10626
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_E, value)

    @property
    @locked
    def ain64_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@64)

        Address: 10628
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_E)  # type: ignore

    @ain64_ef_config_e.setter
    @locked
    def ain64_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@64)

        Address: 10628
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_E, value)

    @property
    @locked
    def ain65_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@65)

        Address: 10630
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_E)  # type: ignore

    @ain65_ef_config_e.setter
    @locked
    def ain65_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@65)

        Address: 10630
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_E, value)

    @property
    @locked
    def ain66_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@66)

        Address: 10632
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_E)  # type: ignore

    @ain66_ef_config_e.setter
    @locked
    def ain66_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@66)

        Address: 10632
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_E, value)

    @property
    @locked
    def ain67_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@67)

        Address: 10634
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_E)  # type: ignore

    @ain67_ef_config_e.setter
    @locked
    def ain67_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@67)

        Address: 10634
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_E, value)

    @property
    @locked
    def ain68_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@68)

        Address: 10636
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_E)  # type: ignore

    @ain68_ef_config_e.setter
    @locked
    def ain68_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@68)

        Address: 10636
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_E, value)

    @property
    @locked
    def ain69_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@69)

        Address: 10638
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_E)  # type: ignore

    @ain69_ef_config_e.setter
    @locked
    def ain69_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@69)

        Address: 10638
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_E, value)

    @property
    @locked
    def ain70_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@70)

        Address: 10640
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_E)  # type: ignore

    @ain70_ef_config_e.setter
    @locked
    def ain70_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@70)

        Address: 10640
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_E, value)

    @property
    @locked
    def ain71_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@71)

        Address: 10642
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_E)  # type: ignore

    @ain71_ef_config_e.setter
    @locked
    def ain71_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@71)

        Address: 10642
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_E, value)

    @property
    @locked
    def ain72_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@72)

        Address: 10644
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_E)  # type: ignore

    @ain72_ef_config_e.setter
    @locked
    def ain72_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@72)

        Address: 10644
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_E, value)

    @property
    @locked
    def ain73_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@73)

        Address: 10646
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_E)  # type: ignore

    @ain73_ef_config_e.setter
    @locked
    def ain73_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@73)

        Address: 10646
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_E, value)

    @property
    @locked
    def ain74_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@74)

        Address: 10648
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_E)  # type: ignore

    @ain74_ef_config_e.setter
    @locked
    def ain74_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@74)

        Address: 10648
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_E, value)

    @property
    @locked
    def ain75_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@75)

        Address: 10650
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_E)  # type: ignore

    @ain75_ef_config_e.setter
    @locked
    def ain75_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@75)

        Address: 10650
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_E, value)

    @property
    @locked
    def ain76_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@76)

        Address: 10652
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_E)  # type: ignore

    @ain76_ef_config_e.setter
    @locked
    def ain76_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@76)

        Address: 10652
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_E, value)

    @property
    @locked
    def ain77_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@77)

        Address: 10654
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_E)  # type: ignore

    @ain77_ef_config_e.setter
    @locked
    def ain77_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@77)

        Address: 10654
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_E, value)

    @property
    @locked
    def ain78_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@78)

        Address: 10656
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_E)  # type: ignore

    @ain78_ef_config_e.setter
    @locked
    def ain78_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@78)

        Address: 10656
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_E, value)

    @property
    @locked
    def ain79_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@79)

        Address: 10658
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_E)  # type: ignore

    @ain79_ef_config_e.setter
    @locked
    def ain79_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@79)

        Address: 10658
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_E, value)

    @property
    @locked
    def ain80_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@80)

        Address: 10660
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_E)  # type: ignore

    @ain80_ef_config_e.setter
    @locked
    def ain80_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@80)

        Address: 10660
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_E, value)

    @property
    @locked
    def ain81_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@81)

        Address: 10662
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_E)  # type: ignore

    @ain81_ef_config_e.setter
    @locked
    def ain81_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@81)

        Address: 10662
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_E, value)

    @property
    @locked
    def ain82_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@82)

        Address: 10664
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_E)  # type: ignore

    @ain82_ef_config_e.setter
    @locked
    def ain82_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@82)

        Address: 10664
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_E, value)

    @property
    @locked
    def ain83_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@83)

        Address: 10666
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_E)  # type: ignore

    @ain83_ef_config_e.setter
    @locked
    def ain83_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@83)

        Address: 10666
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_E, value)

    @property
    @locked
    def ain84_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@84)

        Address: 10668
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_E)  # type: ignore

    @ain84_ef_config_e.setter
    @locked
    def ain84_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@84)

        Address: 10668
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_E, value)

    @property
    @locked
    def ain85_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@85)

        Address: 10670
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_E)  # type: ignore

    @ain85_ef_config_e.setter
    @locked
    def ain85_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@85)

        Address: 10670
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_E, value)

    @property
    @locked
    def ain86_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@86)

        Address: 10672
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_E)  # type: ignore

    @ain86_ef_config_e.setter
    @locked
    def ain86_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@86)

        Address: 10672
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_E, value)

    @property
    @locked
    def ain87_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@87)

        Address: 10674
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_E)  # type: ignore

    @ain87_ef_config_e.setter
    @locked
    def ain87_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@87)

        Address: 10674
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_E, value)

    @property
    @locked
    def ain88_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@88)

        Address: 10676
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_E)  # type: ignore

    @ain88_ef_config_e.setter
    @locked
    def ain88_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@88)

        Address: 10676
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_E, value)

    @property
    @locked
    def ain89_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@89)

        Address: 10678
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_E)  # type: ignore

    @ain89_ef_config_e.setter
    @locked
    def ain89_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@89)

        Address: 10678
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_E, value)

    @property
    @locked
    def ain90_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@90)

        Address: 10680
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_E)  # type: ignore

    @ain90_ef_config_e.setter
    @locked
    def ain90_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@90)

        Address: 10680
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_E, value)

    @property
    @locked
    def ain91_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@91)

        Address: 10682
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_E)  # type: ignore

    @ain91_ef_config_e.setter
    @locked
    def ain91_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@91)

        Address: 10682
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_E, value)

    @property
    @locked
    def ain92_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@92)

        Address: 10684
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_E)  # type: ignore

    @ain92_ef_config_e.setter
    @locked
    def ain92_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@92)

        Address: 10684
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_E, value)

    @property
    @locked
    def ain93_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@93)

        Address: 10686
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_E)  # type: ignore

    @ain93_ef_config_e.setter
    @locked
    def ain93_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@93)

        Address: 10686
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_E, value)

    @property
    @locked
    def ain94_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@94)

        Address: 10688
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_E)  # type: ignore

    @ain94_ef_config_e.setter
    @locked
    def ain94_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@94)

        Address: 10688
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_E, value)

    @property
    @locked
    def ain95_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@95)

        Address: 10690
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_E)  # type: ignore

    @ain95_ef_config_e.setter
    @locked
    def ain95_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@95)

        Address: 10690
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_E, value)

    @property
    @locked
    def ain96_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@96)

        Address: 10692
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_E)  # type: ignore

    @ain96_ef_config_e.setter
    @locked
    def ain96_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@96)

        Address: 10692
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_E, value)

    @property
    @locked
    def ain97_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@97)

        Address: 10694
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_E)  # type: ignore

    @ain97_ef_config_e.setter
    @locked
    def ain97_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@97)

        Address: 10694
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_E, value)

    @property
    @locked
    def ain98_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@98)

        Address: 10696
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_E)  # type: ignore

    @ain98_ef_config_e.setter
    @locked
    def ain98_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@98)

        Address: 10696
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_E, value)

    @property
    @locked
    def ain99_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@99)

        Address: 10698
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_E)  # type: ignore

    @ain99_ef_config_e.setter
    @locked
    def ain99_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@99)

        Address: 10698
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_E, value)

    @property
    @locked
    def ain100_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@100)

        Address: 10700
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_E)  # type: ignore

    @ain100_ef_config_e.setter
    @locked
    def ain100_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@100)

        Address: 10700
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_E, value)

    @property
    @locked
    def ain101_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@101)

        Address: 10702
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_E)  # type: ignore

    @ain101_ef_config_e.setter
    @locked
    def ain101_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@101)

        Address: 10702
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_E, value)

    @property
    @locked
    def ain102_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@102)

        Address: 10704
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_E)  # type: ignore

    @ain102_ef_config_e.setter
    @locked
    def ain102_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@102)

        Address: 10704
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_E, value)

    @property
    @locked
    def ain103_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@103)

        Address: 10706
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_E)  # type: ignore

    @ain103_ef_config_e.setter
    @locked
    def ain103_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@103)

        Address: 10706
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_E, value)

    @property
    @locked
    def ain104_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@104)

        Address: 10708
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_E)  # type: ignore

    @ain104_ef_config_e.setter
    @locked
    def ain104_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@104)

        Address: 10708
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_E, value)

    @property
    @locked
    def ain105_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@105)

        Address: 10710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_E)  # type: ignore

    @ain105_ef_config_e.setter
    @locked
    def ain105_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@105)

        Address: 10710
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_E, value)

    @property
    @locked
    def ain106_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@106)

        Address: 10712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_E)  # type: ignore

    @ain106_ef_config_e.setter
    @locked
    def ain106_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@106)

        Address: 10712
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_E, value)

    @property
    @locked
    def ain107_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@107)

        Address: 10714
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_E)  # type: ignore

    @ain107_ef_config_e.setter
    @locked
    def ain107_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@107)

        Address: 10714
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_E, value)

    @property
    @locked
    def ain108_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@108)

        Address: 10716
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_E)  # type: ignore

    @ain108_ef_config_e.setter
    @locked
    def ain108_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@108)

        Address: 10716
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_E, value)

    @property
    @locked
    def ain109_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@109)

        Address: 10718
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_E)  # type: ignore

    @ain109_ef_config_e.setter
    @locked
    def ain109_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@109)

        Address: 10718
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_E, value)

    @property
    @locked
    def ain110_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@110)

        Address: 10720
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_E)  # type: ignore

    @ain110_ef_config_e.setter
    @locked
    def ain110_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@110)

        Address: 10720
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_E, value)

    @property
    @locked
    def ain111_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@111)

        Address: 10722
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_E)  # type: ignore

    @ain111_ef_config_e.setter
    @locked
    def ain111_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@111)

        Address: 10722
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_E, value)

    @property
    @locked
    def ain112_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@112)

        Address: 10724
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_E)  # type: ignore

    @ain112_ef_config_e.setter
    @locked
    def ain112_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@112)

        Address: 10724
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_E, value)

    @property
    @locked
    def ain113_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@113)

        Address: 10726
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_E)  # type: ignore

    @ain113_ef_config_e.setter
    @locked
    def ain113_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@113)

        Address: 10726
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_E, value)

    @property
    @locked
    def ain114_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@114)

        Address: 10728
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_E)  # type: ignore

    @ain114_ef_config_e.setter
    @locked
    def ain114_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@114)

        Address: 10728
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_E, value)

    @property
    @locked
    def ain115_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@115)

        Address: 10730
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_E)  # type: ignore

    @ain115_ef_config_e.setter
    @locked
    def ain115_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@115)

        Address: 10730
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_E, value)

    @property
    @locked
    def ain116_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@116)

        Address: 10732
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_E)  # type: ignore

    @ain116_ef_config_e.setter
    @locked
    def ain116_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@116)

        Address: 10732
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_E, value)

    @property
    @locked
    def ain117_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@117)

        Address: 10734
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_E)  # type: ignore

    @ain117_ef_config_e.setter
    @locked
    def ain117_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@117)

        Address: 10734
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_E, value)

    @property
    @locked
    def ain118_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@118)

        Address: 10736
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_E)  # type: ignore

    @ain118_ef_config_e.setter
    @locked
    def ain118_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@118)

        Address: 10736
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_E, value)

    @property
    @locked
    def ain119_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@119)

        Address: 10738
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_E)  # type: ignore

    @ain119_ef_config_e.setter
    @locked
    def ain119_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@119)

        Address: 10738
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_E, value)

    @property
    @locked
    def ain120_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@120)

        Address: 10740
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_E)  # type: ignore

    @ain120_ef_config_e.setter
    @locked
    def ain120_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@120)

        Address: 10740
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_E, value)

    @property
    @locked
    def ain121_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@121)

        Address: 10742
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_E)  # type: ignore

    @ain121_ef_config_e.setter
    @locked
    def ain121_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@121)

        Address: 10742
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_E, value)

    @property
    @locked
    def ain122_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@122)

        Address: 10744
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_E)  # type: ignore

    @ain122_ef_config_e.setter
    @locked
    def ain122_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@122)

        Address: 10744
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_E, value)

    @property
    @locked
    def ain123_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@123)

        Address: 10746
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_E)  # type: ignore

    @ain123_ef_config_e.setter
    @locked
    def ain123_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@123)

        Address: 10746
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_E, value)

    @property
    @locked
    def ain124_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@124)

        Address: 10748
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_E)  # type: ignore

    @ain124_ef_config_e.setter
    @locked
    def ain124_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@124)

        Address: 10748
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_E, value)

    @property
    @locked
    def ain125_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@125)

        Address: 10750
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_E)  # type: ignore

    @ain125_ef_config_e.setter
    @locked
    def ain125_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@125)

        Address: 10750
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_E, value)

    @property
    @locked
    def ain126_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@126)

        Address: 10752
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_E)  # type: ignore

    @ain126_ef_config_e.setter
    @locked
    def ain126_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@126)

        Address: 10752
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_E, value)

    @property
    @locked
    def ain127_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@127)

        Address: 10754
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_E)  # type: ignore

    @ain127_ef_config_e.setter
    @locked
    def ain127_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@127)

        Address: 10754
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_E, value)

    @property
    @locked
    def ain128_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@128)

        Address: 10756
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_E)  # type: ignore

    @ain128_ef_config_e.setter
    @locked
    def ain128_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@128)

        Address: 10756
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_E, value)

    @property
    @locked
    def ain129_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@129)

        Address: 10758
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_E)  # type: ignore

    @ain129_ef_config_e.setter
    @locked
    def ain129_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@129)

        Address: 10758
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_E, value)

    @property
    @locked
    def ain130_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@130)

        Address: 10760
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_E)  # type: ignore

    @ain130_ef_config_e.setter
    @locked
    def ain130_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@130)

        Address: 10760
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_E, value)

    @property
    @locked
    def ain131_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@131)

        Address: 10762
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_E)  # type: ignore

    @ain131_ef_config_e.setter
    @locked
    def ain131_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@131)

        Address: 10762
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_E, value)

    @property
    @locked
    def ain132_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@132)

        Address: 10764
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_E)  # type: ignore

    @ain132_ef_config_e.setter
    @locked
    def ain132_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@132)

        Address: 10764
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_E, value)

    @property
    @locked
    def ain133_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@133)

        Address: 10766
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_E)  # type: ignore

    @ain133_ef_config_e.setter
    @locked
    def ain133_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@133)

        Address: 10766
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_E, value)

    @property
    @locked
    def ain134_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@134)

        Address: 10768
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_E)  # type: ignore

    @ain134_ef_config_e.setter
    @locked
    def ain134_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@134)

        Address: 10768
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_E, value)

    @property
    @locked
    def ain135_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@135)

        Address: 10770
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_E)  # type: ignore

    @ain135_ef_config_e.setter
    @locked
    def ain135_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@135)

        Address: 10770
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_E, value)

    @property
    @locked
    def ain136_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@136)

        Address: 10772
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_E)  # type: ignore

    @ain136_ef_config_e.setter
    @locked
    def ain136_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@136)

        Address: 10772
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_E, value)

    @property
    @locked
    def ain137_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@137)

        Address: 10774
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_E)  # type: ignore

    @ain137_ef_config_e.setter
    @locked
    def ain137_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@137)

        Address: 10774
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_E, value)

    @property
    @locked
    def ain138_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@138)

        Address: 10776
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_E)  # type: ignore

    @ain138_ef_config_e.setter
    @locked
    def ain138_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@138)

        Address: 10776
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_E, value)

    @property
    @locked
    def ain139_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@139)

        Address: 10778
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_E)  # type: ignore

    @ain139_ef_config_e.setter
    @locked
    def ain139_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@139)

        Address: 10778
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_E, value)

    @property
    @locked
    def ain140_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@140)

        Address: 10780
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_E)  # type: ignore

    @ain140_ef_config_e.setter
    @locked
    def ain140_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@140)

        Address: 10780
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_E, value)

    @property
    @locked
    def ain141_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@141)

        Address: 10782
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_E)  # type: ignore

    @ain141_ef_config_e.setter
    @locked
    def ain141_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@141)

        Address: 10782
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_E, value)

    @property
    @locked
    def ain142_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@142)

        Address: 10784
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_E)  # type: ignore

    @ain142_ef_config_e.setter
    @locked
    def ain142_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@142)

        Address: 10784
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_E, value)

    @property
    @locked
    def ain143_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@143)

        Address: 10786
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_E)  # type: ignore

    @ain143_ef_config_e.setter
    @locked
    def ain143_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@143)

        Address: 10786
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_E, value)

    @property
    @locked
    def ain144_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@144)

        Address: 10788
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_E)  # type: ignore

    @ain144_ef_config_e.setter
    @locked
    def ain144_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@144)

        Address: 10788
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_E, value)

    @property
    @locked
    def ain145_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@145)

        Address: 10790
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_E)  # type: ignore

    @ain145_ef_config_e.setter
    @locked
    def ain145_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@145)

        Address: 10790
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_E, value)

    @property
    @locked
    def ain146_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@146)

        Address: 10792
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_E)  # type: ignore

    @ain146_ef_config_e.setter
    @locked
    def ain146_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@146)

        Address: 10792
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_E, value)

    @property
    @locked
    def ain147_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@147)

        Address: 10794
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_E)  # type: ignore

    @ain147_ef_config_e.setter
    @locked
    def ain147_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@147)

        Address: 10794
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_E, value)

    @property
    @locked
    def ain148_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@148)

        Address: 10796
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_E)  # type: ignore

    @ain148_ef_config_e.setter
    @locked
    def ain148_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@148)

        Address: 10796
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_E, value)

    @property
    @locked
    def ain149_ef_config_e(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@149)

        Address: 10798
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_E)  # type: ignore

    @ain149_ef_config_e.setter
    @locked
    def ain149_ef_config_e(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_E (AIN#(0:149)_EF_CONFIG_E@149)

        Address: 10798
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_E, value)

    @property
    @locked
    def ain0_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@0)

        Address: 10800
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_F)  # type: ignore

    @ain0_ef_config_f.setter
    @locked
    def ain0_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@0)

        Address: 10800
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_F, value)

    @property
    @locked
    def ain1_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@1)

        Address: 10802
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_F)  # type: ignore

    @ain1_ef_config_f.setter
    @locked
    def ain1_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@1)

        Address: 10802
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_F, value)

    @property
    @locked
    def ain2_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@2)

        Address: 10804
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_F)  # type: ignore

    @ain2_ef_config_f.setter
    @locked
    def ain2_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@2)

        Address: 10804
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_F, value)

    @property
    @locked
    def ain3_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@3)

        Address: 10806
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_F)  # type: ignore

    @ain3_ef_config_f.setter
    @locked
    def ain3_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@3)

        Address: 10806
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_F, value)

    @property
    @locked
    def ain4_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@4)

        Address: 10808
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_F)  # type: ignore

    @ain4_ef_config_f.setter
    @locked
    def ain4_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@4)

        Address: 10808
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_F, value)

    @property
    @locked
    def ain5_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@5)

        Address: 10810
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_F)  # type: ignore

    @ain5_ef_config_f.setter
    @locked
    def ain5_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@5)

        Address: 10810
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_F, value)

    @property
    @locked
    def ain6_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@6)

        Address: 10812
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_F)  # type: ignore

    @ain6_ef_config_f.setter
    @locked
    def ain6_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@6)

        Address: 10812
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_F, value)

    @property
    @locked
    def ain7_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@7)

        Address: 10814
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_F)  # type: ignore

    @ain7_ef_config_f.setter
    @locked
    def ain7_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@7)

        Address: 10814
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_F, value)

    @property
    @locked
    def ain8_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@8)

        Address: 10816
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_F)  # type: ignore

    @ain8_ef_config_f.setter
    @locked
    def ain8_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@8)

        Address: 10816
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_F, value)

    @property
    @locked
    def ain9_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@9)

        Address: 10818
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_F)  # type: ignore

    @ain9_ef_config_f.setter
    @locked
    def ain9_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@9)

        Address: 10818
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_F, value)

    @property
    @locked
    def ain10_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@10)

        Address: 10820
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_F)  # type: ignore

    @ain10_ef_config_f.setter
    @locked
    def ain10_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@10)

        Address: 10820
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_F, value)

    @property
    @locked
    def ain11_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@11)

        Address: 10822
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_F)  # type: ignore

    @ain11_ef_config_f.setter
    @locked
    def ain11_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@11)

        Address: 10822
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_F, value)

    @property
    @locked
    def ain12_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@12)

        Address: 10824
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_F)  # type: ignore

    @ain12_ef_config_f.setter
    @locked
    def ain12_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@12)

        Address: 10824
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_F, value)

    @property
    @locked
    def ain13_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@13)

        Address: 10826
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_F)  # type: ignore

    @ain13_ef_config_f.setter
    @locked
    def ain13_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@13)

        Address: 10826
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_F, value)

    @property
    @locked
    def ain14_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@14)

        Address: 10828
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_F)  # type: ignore

    @ain14_ef_config_f.setter
    @locked
    def ain14_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@14)

        Address: 10828
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_F, value)

    @property
    @locked
    def ain15_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@15)

        Address: 10830
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_F)  # type: ignore

    @ain15_ef_config_f.setter
    @locked
    def ain15_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@15)

        Address: 10830
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_F, value)

    @property
    @locked
    def ain16_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@16)

        Address: 10832
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_F)  # type: ignore

    @ain16_ef_config_f.setter
    @locked
    def ain16_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@16)

        Address: 10832
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_F, value)

    @property
    @locked
    def ain17_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@17)

        Address: 10834
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_F)  # type: ignore

    @ain17_ef_config_f.setter
    @locked
    def ain17_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@17)

        Address: 10834
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_F, value)

    @property
    @locked
    def ain18_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@18)

        Address: 10836
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_F)  # type: ignore

    @ain18_ef_config_f.setter
    @locked
    def ain18_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@18)

        Address: 10836
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_F, value)

    @property
    @locked
    def ain19_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@19)

        Address: 10838
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_F)  # type: ignore

    @ain19_ef_config_f.setter
    @locked
    def ain19_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@19)

        Address: 10838
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_F, value)

    @property
    @locked
    def ain20_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@20)

        Address: 10840
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_F)  # type: ignore

    @ain20_ef_config_f.setter
    @locked
    def ain20_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@20)

        Address: 10840
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_F, value)

    @property
    @locked
    def ain21_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@21)

        Address: 10842
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_F)  # type: ignore

    @ain21_ef_config_f.setter
    @locked
    def ain21_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@21)

        Address: 10842
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_F, value)

    @property
    @locked
    def ain22_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@22)

        Address: 10844
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_F)  # type: ignore

    @ain22_ef_config_f.setter
    @locked
    def ain22_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@22)

        Address: 10844
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_F, value)

    @property
    @locked
    def ain23_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@23)

        Address: 10846
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_F)  # type: ignore

    @ain23_ef_config_f.setter
    @locked
    def ain23_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@23)

        Address: 10846
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_F, value)

    @property
    @locked
    def ain24_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@24)

        Address: 10848
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_F)  # type: ignore

    @ain24_ef_config_f.setter
    @locked
    def ain24_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@24)

        Address: 10848
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_F, value)

    @property
    @locked
    def ain25_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@25)

        Address: 10850
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_F)  # type: ignore

    @ain25_ef_config_f.setter
    @locked
    def ain25_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@25)

        Address: 10850
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_F, value)

    @property
    @locked
    def ain26_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@26)

        Address: 10852
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_F)  # type: ignore

    @ain26_ef_config_f.setter
    @locked
    def ain26_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@26)

        Address: 10852
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_F, value)

    @property
    @locked
    def ain27_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@27)

        Address: 10854
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_F)  # type: ignore

    @ain27_ef_config_f.setter
    @locked
    def ain27_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@27)

        Address: 10854
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_F, value)

    @property
    @locked
    def ain28_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@28)

        Address: 10856
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_F)  # type: ignore

    @ain28_ef_config_f.setter
    @locked
    def ain28_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@28)

        Address: 10856
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_F, value)

    @property
    @locked
    def ain29_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@29)

        Address: 10858
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_F)  # type: ignore

    @ain29_ef_config_f.setter
    @locked
    def ain29_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@29)

        Address: 10858
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_F, value)

    @property
    @locked
    def ain30_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@30)

        Address: 10860
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_F)  # type: ignore

    @ain30_ef_config_f.setter
    @locked
    def ain30_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@30)

        Address: 10860
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_F, value)

    @property
    @locked
    def ain31_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@31)

        Address: 10862
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_F)  # type: ignore

    @ain31_ef_config_f.setter
    @locked
    def ain31_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@31)

        Address: 10862
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_F, value)

    @property
    @locked
    def ain32_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@32)

        Address: 10864
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_F)  # type: ignore

    @ain32_ef_config_f.setter
    @locked
    def ain32_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@32)

        Address: 10864
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_F, value)

    @property
    @locked
    def ain33_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@33)

        Address: 10866
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_F)  # type: ignore

    @ain33_ef_config_f.setter
    @locked
    def ain33_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@33)

        Address: 10866
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_F, value)

    @property
    @locked
    def ain34_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@34)

        Address: 10868
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_F)  # type: ignore

    @ain34_ef_config_f.setter
    @locked
    def ain34_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@34)

        Address: 10868
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_F, value)

    @property
    @locked
    def ain35_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@35)

        Address: 10870
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_F)  # type: ignore

    @ain35_ef_config_f.setter
    @locked
    def ain35_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@35)

        Address: 10870
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_F, value)

    @property
    @locked
    def ain36_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@36)

        Address: 10872
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_F)  # type: ignore

    @ain36_ef_config_f.setter
    @locked
    def ain36_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@36)

        Address: 10872
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_F, value)

    @property
    @locked
    def ain37_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@37)

        Address: 10874
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_F)  # type: ignore

    @ain37_ef_config_f.setter
    @locked
    def ain37_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@37)

        Address: 10874
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_F, value)

    @property
    @locked
    def ain38_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@38)

        Address: 10876
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_F)  # type: ignore

    @ain38_ef_config_f.setter
    @locked
    def ain38_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@38)

        Address: 10876
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_F, value)

    @property
    @locked
    def ain39_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@39)

        Address: 10878
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_F)  # type: ignore

    @ain39_ef_config_f.setter
    @locked
    def ain39_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@39)

        Address: 10878
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_F, value)

    @property
    @locked
    def ain40_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@40)

        Address: 10880
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_F)  # type: ignore

    @ain40_ef_config_f.setter
    @locked
    def ain40_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@40)

        Address: 10880
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_F, value)

    @property
    @locked
    def ain41_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@41)

        Address: 10882
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_F)  # type: ignore

    @ain41_ef_config_f.setter
    @locked
    def ain41_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@41)

        Address: 10882
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_F, value)

    @property
    @locked
    def ain42_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@42)

        Address: 10884
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_F)  # type: ignore

    @ain42_ef_config_f.setter
    @locked
    def ain42_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@42)

        Address: 10884
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_F, value)

    @property
    @locked
    def ain43_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@43)

        Address: 10886
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_F)  # type: ignore

    @ain43_ef_config_f.setter
    @locked
    def ain43_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@43)

        Address: 10886
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_F, value)

    @property
    @locked
    def ain44_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@44)

        Address: 10888
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_F)  # type: ignore

    @ain44_ef_config_f.setter
    @locked
    def ain44_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@44)

        Address: 10888
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_F, value)

    @property
    @locked
    def ain45_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@45)

        Address: 10890
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_F)  # type: ignore

    @ain45_ef_config_f.setter
    @locked
    def ain45_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@45)

        Address: 10890
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_F, value)

    @property
    @locked
    def ain46_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@46)

        Address: 10892
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_F)  # type: ignore

    @ain46_ef_config_f.setter
    @locked
    def ain46_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@46)

        Address: 10892
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_F, value)

    @property
    @locked
    def ain47_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@47)

        Address: 10894
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_F)  # type: ignore

    @ain47_ef_config_f.setter
    @locked
    def ain47_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@47)

        Address: 10894
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_F, value)

    @property
    @locked
    def ain48_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@48)

        Address: 10896
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_F)  # type: ignore

    @ain48_ef_config_f.setter
    @locked
    def ain48_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@48)

        Address: 10896
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_F, value)

    @property
    @locked
    def ain49_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@49)

        Address: 10898
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_F)  # type: ignore

    @ain49_ef_config_f.setter
    @locked
    def ain49_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@49)

        Address: 10898
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_F, value)

    @property
    @locked
    def ain50_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@50)

        Address: 10900
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_F)  # type: ignore

    @ain50_ef_config_f.setter
    @locked
    def ain50_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@50)

        Address: 10900
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_F, value)

    @property
    @locked
    def ain51_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@51)

        Address: 10902
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_F)  # type: ignore

    @ain51_ef_config_f.setter
    @locked
    def ain51_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@51)

        Address: 10902
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_F, value)

    @property
    @locked
    def ain52_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@52)

        Address: 10904
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_F)  # type: ignore

    @ain52_ef_config_f.setter
    @locked
    def ain52_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@52)

        Address: 10904
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_F, value)

    @property
    @locked
    def ain53_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@53)

        Address: 10906
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_F)  # type: ignore

    @ain53_ef_config_f.setter
    @locked
    def ain53_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@53)

        Address: 10906
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_F, value)

    @property
    @locked
    def ain54_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@54)

        Address: 10908
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_F)  # type: ignore

    @ain54_ef_config_f.setter
    @locked
    def ain54_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@54)

        Address: 10908
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_F, value)

    @property
    @locked
    def ain55_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@55)

        Address: 10910
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_F)  # type: ignore

    @ain55_ef_config_f.setter
    @locked
    def ain55_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@55)

        Address: 10910
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_F, value)

    @property
    @locked
    def ain56_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@56)

        Address: 10912
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_F)  # type: ignore

    @ain56_ef_config_f.setter
    @locked
    def ain56_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@56)

        Address: 10912
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_F, value)

    @property
    @locked
    def ain57_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@57)

        Address: 10914
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_F)  # type: ignore

    @ain57_ef_config_f.setter
    @locked
    def ain57_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@57)

        Address: 10914
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_F, value)

    @property
    @locked
    def ain58_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@58)

        Address: 10916
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_F)  # type: ignore

    @ain58_ef_config_f.setter
    @locked
    def ain58_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@58)

        Address: 10916
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_F, value)

    @property
    @locked
    def ain59_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@59)

        Address: 10918
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_F)  # type: ignore

    @ain59_ef_config_f.setter
    @locked
    def ain59_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@59)

        Address: 10918
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_F, value)

    @property
    @locked
    def ain60_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@60)

        Address: 10920
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_F)  # type: ignore

    @ain60_ef_config_f.setter
    @locked
    def ain60_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@60)

        Address: 10920
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_F, value)

    @property
    @locked
    def ain61_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@61)

        Address: 10922
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_F)  # type: ignore

    @ain61_ef_config_f.setter
    @locked
    def ain61_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@61)

        Address: 10922
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_F, value)

    @property
    @locked
    def ain62_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@62)

        Address: 10924
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_F)  # type: ignore

    @ain62_ef_config_f.setter
    @locked
    def ain62_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@62)

        Address: 10924
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_F, value)

    @property
    @locked
    def ain63_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@63)

        Address: 10926
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_F)  # type: ignore

    @ain63_ef_config_f.setter
    @locked
    def ain63_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@63)

        Address: 10926
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_F, value)

    @property
    @locked
    def ain64_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@64)

        Address: 10928
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_F)  # type: ignore

    @ain64_ef_config_f.setter
    @locked
    def ain64_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@64)

        Address: 10928
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_F, value)

    @property
    @locked
    def ain65_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@65)

        Address: 10930
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_F)  # type: ignore

    @ain65_ef_config_f.setter
    @locked
    def ain65_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@65)

        Address: 10930
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_F, value)

    @property
    @locked
    def ain66_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@66)

        Address: 10932
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_F)  # type: ignore

    @ain66_ef_config_f.setter
    @locked
    def ain66_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@66)

        Address: 10932
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_F, value)

    @property
    @locked
    def ain67_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@67)

        Address: 10934
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_F)  # type: ignore

    @ain67_ef_config_f.setter
    @locked
    def ain67_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@67)

        Address: 10934
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_F, value)

    @property
    @locked
    def ain68_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@68)

        Address: 10936
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_F)  # type: ignore

    @ain68_ef_config_f.setter
    @locked
    def ain68_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@68)

        Address: 10936
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_F, value)

    @property
    @locked
    def ain69_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@69)

        Address: 10938
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_F)  # type: ignore

    @ain69_ef_config_f.setter
    @locked
    def ain69_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@69)

        Address: 10938
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_F, value)

    @property
    @locked
    def ain70_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@70)

        Address: 10940
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_F)  # type: ignore

    @ain70_ef_config_f.setter
    @locked
    def ain70_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@70)

        Address: 10940
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_F, value)

    @property
    @locked
    def ain71_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@71)

        Address: 10942
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_F)  # type: ignore

    @ain71_ef_config_f.setter
    @locked
    def ain71_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@71)

        Address: 10942
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_F, value)

    @property
    @locked
    def ain72_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@72)

        Address: 10944
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_F)  # type: ignore

    @ain72_ef_config_f.setter
    @locked
    def ain72_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@72)

        Address: 10944
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_F, value)

    @property
    @locked
    def ain73_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@73)

        Address: 10946
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_F)  # type: ignore

    @ain73_ef_config_f.setter
    @locked
    def ain73_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@73)

        Address: 10946
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_F, value)

    @property
    @locked
    def ain74_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@74)

        Address: 10948
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_F)  # type: ignore

    @ain74_ef_config_f.setter
    @locked
    def ain74_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@74)

        Address: 10948
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_F, value)

    @property
    @locked
    def ain75_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@75)

        Address: 10950
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_F)  # type: ignore

    @ain75_ef_config_f.setter
    @locked
    def ain75_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@75)

        Address: 10950
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_F, value)

    @property
    @locked
    def ain76_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@76)

        Address: 10952
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_F)  # type: ignore

    @ain76_ef_config_f.setter
    @locked
    def ain76_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@76)

        Address: 10952
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_F, value)

    @property
    @locked
    def ain77_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@77)

        Address: 10954
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_F)  # type: ignore

    @ain77_ef_config_f.setter
    @locked
    def ain77_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@77)

        Address: 10954
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_F, value)

    @property
    @locked
    def ain78_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@78)

        Address: 10956
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_F)  # type: ignore

    @ain78_ef_config_f.setter
    @locked
    def ain78_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@78)

        Address: 10956
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_F, value)

    @property
    @locked
    def ain79_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@79)

        Address: 10958
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_F)  # type: ignore

    @ain79_ef_config_f.setter
    @locked
    def ain79_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@79)

        Address: 10958
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_F, value)

    @property
    @locked
    def ain80_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@80)

        Address: 10960
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_F)  # type: ignore

    @ain80_ef_config_f.setter
    @locked
    def ain80_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@80)

        Address: 10960
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_F, value)

    @property
    @locked
    def ain81_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@81)

        Address: 10962
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_F)  # type: ignore

    @ain81_ef_config_f.setter
    @locked
    def ain81_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@81)

        Address: 10962
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_F, value)

    @property
    @locked
    def ain82_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@82)

        Address: 10964
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_F)  # type: ignore

    @ain82_ef_config_f.setter
    @locked
    def ain82_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@82)

        Address: 10964
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_F, value)

    @property
    @locked
    def ain83_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@83)

        Address: 10966
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_F)  # type: ignore

    @ain83_ef_config_f.setter
    @locked
    def ain83_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@83)

        Address: 10966
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_F, value)

    @property
    @locked
    def ain84_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@84)

        Address: 10968
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_F)  # type: ignore

    @ain84_ef_config_f.setter
    @locked
    def ain84_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@84)

        Address: 10968
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_F, value)

    @property
    @locked
    def ain85_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@85)

        Address: 10970
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_F)  # type: ignore

    @ain85_ef_config_f.setter
    @locked
    def ain85_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@85)

        Address: 10970
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_F, value)

    @property
    @locked
    def ain86_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@86)

        Address: 10972
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_F)  # type: ignore

    @ain86_ef_config_f.setter
    @locked
    def ain86_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@86)

        Address: 10972
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_F, value)

    @property
    @locked
    def ain87_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@87)

        Address: 10974
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_F)  # type: ignore

    @ain87_ef_config_f.setter
    @locked
    def ain87_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@87)

        Address: 10974
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_F, value)

    @property
    @locked
    def ain88_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@88)

        Address: 10976
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_F)  # type: ignore

    @ain88_ef_config_f.setter
    @locked
    def ain88_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@88)

        Address: 10976
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_F, value)

    @property
    @locked
    def ain89_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@89)

        Address: 10978
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_F)  # type: ignore

    @ain89_ef_config_f.setter
    @locked
    def ain89_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@89)

        Address: 10978
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_F, value)

    @property
    @locked
    def ain90_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@90)

        Address: 10980
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_F)  # type: ignore

    @ain90_ef_config_f.setter
    @locked
    def ain90_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@90)

        Address: 10980
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_F, value)

    @property
    @locked
    def ain91_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@91)

        Address: 10982
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_F)  # type: ignore

    @ain91_ef_config_f.setter
    @locked
    def ain91_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@91)

        Address: 10982
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_F, value)

    @property
    @locked
    def ain92_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@92)

        Address: 10984
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_F)  # type: ignore

    @ain92_ef_config_f.setter
    @locked
    def ain92_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@92)

        Address: 10984
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_F, value)

    @property
    @locked
    def ain93_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@93)

        Address: 10986
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_F)  # type: ignore

    @ain93_ef_config_f.setter
    @locked
    def ain93_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@93)

        Address: 10986
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_F, value)

    @property
    @locked
    def ain94_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@94)

        Address: 10988
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_F)  # type: ignore

    @ain94_ef_config_f.setter
    @locked
    def ain94_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@94)

        Address: 10988
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_F, value)

    @property
    @locked
    def ain95_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@95)

        Address: 10990
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_F)  # type: ignore

    @ain95_ef_config_f.setter
    @locked
    def ain95_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@95)

        Address: 10990
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_F, value)

    @property
    @locked
    def ain96_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@96)

        Address: 10992
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_F)  # type: ignore

    @ain96_ef_config_f.setter
    @locked
    def ain96_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@96)

        Address: 10992
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_F, value)

    @property
    @locked
    def ain97_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@97)

        Address: 10994
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_F)  # type: ignore

    @ain97_ef_config_f.setter
    @locked
    def ain97_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@97)

        Address: 10994
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_F, value)

    @property
    @locked
    def ain98_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@98)

        Address: 10996
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_F)  # type: ignore

    @ain98_ef_config_f.setter
    @locked
    def ain98_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@98)

        Address: 10996
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_F, value)

    @property
    @locked
    def ain99_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@99)

        Address: 10998
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_F)  # type: ignore

    @ain99_ef_config_f.setter
    @locked
    def ain99_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@99)

        Address: 10998
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_F, value)

    @property
    @locked
    def ain100_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@100)

        Address: 11000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_F)  # type: ignore

    @ain100_ef_config_f.setter
    @locked
    def ain100_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@100)

        Address: 11000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_F, value)

    @property
    @locked
    def ain101_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@101)

        Address: 11002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_F)  # type: ignore

    @ain101_ef_config_f.setter
    @locked
    def ain101_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@101)

        Address: 11002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_F, value)

    @property
    @locked
    def ain102_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@102)

        Address: 11004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_F)  # type: ignore

    @ain102_ef_config_f.setter
    @locked
    def ain102_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@102)

        Address: 11004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_F, value)

    @property
    @locked
    def ain103_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@103)

        Address: 11006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_F)  # type: ignore

    @ain103_ef_config_f.setter
    @locked
    def ain103_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@103)

        Address: 11006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_F, value)

    @property
    @locked
    def ain104_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@104)

        Address: 11008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_F)  # type: ignore

    @ain104_ef_config_f.setter
    @locked
    def ain104_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@104)

        Address: 11008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_F, value)

    @property
    @locked
    def ain105_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@105)

        Address: 11010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_F)  # type: ignore

    @ain105_ef_config_f.setter
    @locked
    def ain105_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@105)

        Address: 11010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_F, value)

    @property
    @locked
    def ain106_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@106)

        Address: 11012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_F)  # type: ignore

    @ain106_ef_config_f.setter
    @locked
    def ain106_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@106)

        Address: 11012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_F, value)

    @property
    @locked
    def ain107_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@107)

        Address: 11014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_F)  # type: ignore

    @ain107_ef_config_f.setter
    @locked
    def ain107_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@107)

        Address: 11014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_F, value)

    @property
    @locked
    def ain108_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@108)

        Address: 11016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_F)  # type: ignore

    @ain108_ef_config_f.setter
    @locked
    def ain108_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@108)

        Address: 11016
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_F, value)

    @property
    @locked
    def ain109_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@109)

        Address: 11018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_F)  # type: ignore

    @ain109_ef_config_f.setter
    @locked
    def ain109_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@109)

        Address: 11018
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_F, value)

    @property
    @locked
    def ain110_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@110)

        Address: 11020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_F)  # type: ignore

    @ain110_ef_config_f.setter
    @locked
    def ain110_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@110)

        Address: 11020
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_F, value)

    @property
    @locked
    def ain111_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@111)

        Address: 11022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_F)  # type: ignore

    @ain111_ef_config_f.setter
    @locked
    def ain111_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@111)

        Address: 11022
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_F, value)

    @property
    @locked
    def ain112_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@112)

        Address: 11024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_F)  # type: ignore

    @ain112_ef_config_f.setter
    @locked
    def ain112_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@112)

        Address: 11024
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_F, value)

    @property
    @locked
    def ain113_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@113)

        Address: 11026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_F)  # type: ignore

    @ain113_ef_config_f.setter
    @locked
    def ain113_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@113)

        Address: 11026
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_F, value)

    @property
    @locked
    def ain114_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@114)

        Address: 11028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_F)  # type: ignore

    @ain114_ef_config_f.setter
    @locked
    def ain114_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@114)

        Address: 11028
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_F, value)

    @property
    @locked
    def ain115_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@115)

        Address: 11030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_F)  # type: ignore

    @ain115_ef_config_f.setter
    @locked
    def ain115_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@115)

        Address: 11030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_F, value)

    @property
    @locked
    def ain116_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@116)

        Address: 11032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_F)  # type: ignore

    @ain116_ef_config_f.setter
    @locked
    def ain116_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@116)

        Address: 11032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_F, value)

    @property
    @locked
    def ain117_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@117)

        Address: 11034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_F)  # type: ignore

    @ain117_ef_config_f.setter
    @locked
    def ain117_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@117)

        Address: 11034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_F, value)

    @property
    @locked
    def ain118_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@118)

        Address: 11036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_F)  # type: ignore

    @ain118_ef_config_f.setter
    @locked
    def ain118_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@118)

        Address: 11036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_F, value)

    @property
    @locked
    def ain119_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@119)

        Address: 11038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_F)  # type: ignore

    @ain119_ef_config_f.setter
    @locked
    def ain119_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@119)

        Address: 11038
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_F, value)

    @property
    @locked
    def ain120_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@120)

        Address: 11040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_F)  # type: ignore

    @ain120_ef_config_f.setter
    @locked
    def ain120_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@120)

        Address: 11040
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_F, value)

    @property
    @locked
    def ain121_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@121)

        Address: 11042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_F)  # type: ignore

    @ain121_ef_config_f.setter
    @locked
    def ain121_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@121)

        Address: 11042
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_F, value)

    @property
    @locked
    def ain122_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@122)

        Address: 11044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_F)  # type: ignore

    @ain122_ef_config_f.setter
    @locked
    def ain122_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@122)

        Address: 11044
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_F, value)

    @property
    @locked
    def ain123_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@123)

        Address: 11046
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_F)  # type: ignore

    @ain123_ef_config_f.setter
    @locked
    def ain123_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@123)

        Address: 11046
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_F, value)

    @property
    @locked
    def ain124_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@124)

        Address: 11048
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_F)  # type: ignore

    @ain124_ef_config_f.setter
    @locked
    def ain124_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@124)

        Address: 11048
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_F, value)

    @property
    @locked
    def ain125_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@125)

        Address: 11050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_F)  # type: ignore

    @ain125_ef_config_f.setter
    @locked
    def ain125_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@125)

        Address: 11050
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_F, value)

    @property
    @locked
    def ain126_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@126)

        Address: 11052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_F)  # type: ignore

    @ain126_ef_config_f.setter
    @locked
    def ain126_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@126)

        Address: 11052
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_F, value)

    @property
    @locked
    def ain127_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@127)

        Address: 11054
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_F)  # type: ignore

    @ain127_ef_config_f.setter
    @locked
    def ain127_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@127)

        Address: 11054
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_F, value)

    @property
    @locked
    def ain128_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@128)

        Address: 11056
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_F)  # type: ignore

    @ain128_ef_config_f.setter
    @locked
    def ain128_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@128)

        Address: 11056
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_F, value)

    @property
    @locked
    def ain129_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@129)

        Address: 11058
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_F)  # type: ignore

    @ain129_ef_config_f.setter
    @locked
    def ain129_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@129)

        Address: 11058
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_F, value)

    @property
    @locked
    def ain130_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@130)

        Address: 11060
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_F)  # type: ignore

    @ain130_ef_config_f.setter
    @locked
    def ain130_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@130)

        Address: 11060
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_F, value)

    @property
    @locked
    def ain131_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@131)

        Address: 11062
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_F)  # type: ignore

    @ain131_ef_config_f.setter
    @locked
    def ain131_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@131)

        Address: 11062
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_F, value)

    @property
    @locked
    def ain132_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@132)

        Address: 11064
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_F)  # type: ignore

    @ain132_ef_config_f.setter
    @locked
    def ain132_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@132)

        Address: 11064
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_F, value)

    @property
    @locked
    def ain133_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@133)

        Address: 11066
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_F)  # type: ignore

    @ain133_ef_config_f.setter
    @locked
    def ain133_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@133)

        Address: 11066
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_F, value)

    @property
    @locked
    def ain134_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@134)

        Address: 11068
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_F)  # type: ignore

    @ain134_ef_config_f.setter
    @locked
    def ain134_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@134)

        Address: 11068
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_F, value)

    @property
    @locked
    def ain135_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@135)

        Address: 11070
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_F)  # type: ignore

    @ain135_ef_config_f.setter
    @locked
    def ain135_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@135)

        Address: 11070
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_F, value)

    @property
    @locked
    def ain136_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@136)

        Address: 11072
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_F)  # type: ignore

    @ain136_ef_config_f.setter
    @locked
    def ain136_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@136)

        Address: 11072
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_F, value)

    @property
    @locked
    def ain137_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@137)

        Address: 11074
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_F)  # type: ignore

    @ain137_ef_config_f.setter
    @locked
    def ain137_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@137)

        Address: 11074
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_F, value)

    @property
    @locked
    def ain138_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@138)

        Address: 11076
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_F)  # type: ignore

    @ain138_ef_config_f.setter
    @locked
    def ain138_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@138)

        Address: 11076
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_F, value)

    @property
    @locked
    def ain139_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@139)

        Address: 11078
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_F)  # type: ignore

    @ain139_ef_config_f.setter
    @locked
    def ain139_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@139)

        Address: 11078
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_F, value)

    @property
    @locked
    def ain140_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@140)

        Address: 11080
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_F)  # type: ignore

    @ain140_ef_config_f.setter
    @locked
    def ain140_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@140)

        Address: 11080
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_F, value)

    @property
    @locked
    def ain141_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@141)

        Address: 11082
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_F)  # type: ignore

    @ain141_ef_config_f.setter
    @locked
    def ain141_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@141)

        Address: 11082
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_F, value)

    @property
    @locked
    def ain142_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@142)

        Address: 11084
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_F)  # type: ignore

    @ain142_ef_config_f.setter
    @locked
    def ain142_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@142)

        Address: 11084
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_F, value)

    @property
    @locked
    def ain143_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@143)

        Address: 11086
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_F)  # type: ignore

    @ain143_ef_config_f.setter
    @locked
    def ain143_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@143)

        Address: 11086
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_F, value)

    @property
    @locked
    def ain144_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@144)

        Address: 11088
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_F)  # type: ignore

    @ain144_ef_config_f.setter
    @locked
    def ain144_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@144)

        Address: 11088
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_F, value)

    @property
    @locked
    def ain145_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@145)

        Address: 11090
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_F)  # type: ignore

    @ain145_ef_config_f.setter
    @locked
    def ain145_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@145)

        Address: 11090
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_F, value)

    @property
    @locked
    def ain146_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@146)

        Address: 11092
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_F)  # type: ignore

    @ain146_ef_config_f.setter
    @locked
    def ain146_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@146)

        Address: 11092
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_F, value)

    @property
    @locked
    def ain147_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@147)

        Address: 11094
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_F)  # type: ignore

    @ain147_ef_config_f.setter
    @locked
    def ain147_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@147)

        Address: 11094
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_F, value)

    @property
    @locked
    def ain148_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@148)

        Address: 11096
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_F)  # type: ignore

    @ain148_ef_config_f.setter
    @locked
    def ain148_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@148)

        Address: 11096
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_F, value)

    @property
    @locked
    def ain149_ef_config_f(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@149)

        Address: 11098
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_F)  # type: ignore

    @ain149_ef_config_f.setter
    @locked
    def ain149_ef_config_f(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_F (AIN#(0:149)_EF_CONFIG_F@149)

        Address: 11098
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_F, value)

    @property
    @locked
    def ain0_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@0)

        Address: 11100
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_G)  # type: ignore

    @ain0_ef_config_g.setter
    @locked
    def ain0_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@0)

        Address: 11100
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_G, value)

    @property
    @locked
    def ain1_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@1)

        Address: 11102
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_G)  # type: ignore

    @ain1_ef_config_g.setter
    @locked
    def ain1_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@1)

        Address: 11102
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_G, value)

    @property
    @locked
    def ain2_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@2)

        Address: 11104
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_G)  # type: ignore

    @ain2_ef_config_g.setter
    @locked
    def ain2_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@2)

        Address: 11104
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_G, value)

    @property
    @locked
    def ain3_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@3)

        Address: 11106
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_G)  # type: ignore

    @ain3_ef_config_g.setter
    @locked
    def ain3_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@3)

        Address: 11106
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_G, value)

    @property
    @locked
    def ain4_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@4)

        Address: 11108
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_G)  # type: ignore

    @ain4_ef_config_g.setter
    @locked
    def ain4_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@4)

        Address: 11108
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_G, value)

    @property
    @locked
    def ain5_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@5)

        Address: 11110
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_G)  # type: ignore

    @ain5_ef_config_g.setter
    @locked
    def ain5_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@5)

        Address: 11110
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_G, value)

    @property
    @locked
    def ain6_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@6)

        Address: 11112
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_G)  # type: ignore

    @ain6_ef_config_g.setter
    @locked
    def ain6_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@6)

        Address: 11112
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_G, value)

    @property
    @locked
    def ain7_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@7)

        Address: 11114
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_G)  # type: ignore

    @ain7_ef_config_g.setter
    @locked
    def ain7_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@7)

        Address: 11114
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_G, value)

    @property
    @locked
    def ain8_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@8)

        Address: 11116
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_G)  # type: ignore

    @ain8_ef_config_g.setter
    @locked
    def ain8_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@8)

        Address: 11116
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_G, value)

    @property
    @locked
    def ain9_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@9)

        Address: 11118
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_G)  # type: ignore

    @ain9_ef_config_g.setter
    @locked
    def ain9_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@9)

        Address: 11118
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_G, value)

    @property
    @locked
    def ain10_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@10)

        Address: 11120
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_G)  # type: ignore

    @ain10_ef_config_g.setter
    @locked
    def ain10_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@10)

        Address: 11120
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_G, value)

    @property
    @locked
    def ain11_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@11)

        Address: 11122
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_G)  # type: ignore

    @ain11_ef_config_g.setter
    @locked
    def ain11_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@11)

        Address: 11122
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_G, value)

    @property
    @locked
    def ain12_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@12)

        Address: 11124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_G)  # type: ignore

    @ain12_ef_config_g.setter
    @locked
    def ain12_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@12)

        Address: 11124
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_G, value)

    @property
    @locked
    def ain13_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@13)

        Address: 11126
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_G)  # type: ignore

    @ain13_ef_config_g.setter
    @locked
    def ain13_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@13)

        Address: 11126
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_G, value)

    @property
    @locked
    def ain14_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@14)

        Address: 11128
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_G)  # type: ignore

    @ain14_ef_config_g.setter
    @locked
    def ain14_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@14)

        Address: 11128
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_G, value)

    @property
    @locked
    def ain15_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@15)

        Address: 11130
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_G)  # type: ignore

    @ain15_ef_config_g.setter
    @locked
    def ain15_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@15)

        Address: 11130
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_G, value)

    @property
    @locked
    def ain16_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@16)

        Address: 11132
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_G)  # type: ignore

    @ain16_ef_config_g.setter
    @locked
    def ain16_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@16)

        Address: 11132
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_G, value)

    @property
    @locked
    def ain17_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@17)

        Address: 11134
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_G)  # type: ignore

    @ain17_ef_config_g.setter
    @locked
    def ain17_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@17)

        Address: 11134
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_G, value)

    @property
    @locked
    def ain18_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@18)

        Address: 11136
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_G)  # type: ignore

    @ain18_ef_config_g.setter
    @locked
    def ain18_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@18)

        Address: 11136
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_G, value)

    @property
    @locked
    def ain19_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@19)

        Address: 11138
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_G)  # type: ignore

    @ain19_ef_config_g.setter
    @locked
    def ain19_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@19)

        Address: 11138
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_G, value)

    @property
    @locked
    def ain20_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@20)

        Address: 11140
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_G)  # type: ignore

    @ain20_ef_config_g.setter
    @locked
    def ain20_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@20)

        Address: 11140
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_G, value)

    @property
    @locked
    def ain21_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@21)

        Address: 11142
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_G)  # type: ignore

    @ain21_ef_config_g.setter
    @locked
    def ain21_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@21)

        Address: 11142
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_G, value)

    @property
    @locked
    def ain22_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@22)

        Address: 11144
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_G)  # type: ignore

    @ain22_ef_config_g.setter
    @locked
    def ain22_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@22)

        Address: 11144
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_G, value)

    @property
    @locked
    def ain23_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@23)

        Address: 11146
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_G)  # type: ignore

    @ain23_ef_config_g.setter
    @locked
    def ain23_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@23)

        Address: 11146
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_G, value)

    @property
    @locked
    def ain24_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@24)

        Address: 11148
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_G)  # type: ignore

    @ain24_ef_config_g.setter
    @locked
    def ain24_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@24)

        Address: 11148
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_G, value)

    @property
    @locked
    def ain25_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@25)

        Address: 11150
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_G)  # type: ignore

    @ain25_ef_config_g.setter
    @locked
    def ain25_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@25)

        Address: 11150
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_G, value)

    @property
    @locked
    def ain26_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@26)

        Address: 11152
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_G)  # type: ignore

    @ain26_ef_config_g.setter
    @locked
    def ain26_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@26)

        Address: 11152
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_G, value)

    @property
    @locked
    def ain27_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@27)

        Address: 11154
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_G)  # type: ignore

    @ain27_ef_config_g.setter
    @locked
    def ain27_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@27)

        Address: 11154
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_G, value)

    @property
    @locked
    def ain28_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@28)

        Address: 11156
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_G)  # type: ignore

    @ain28_ef_config_g.setter
    @locked
    def ain28_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@28)

        Address: 11156
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_G, value)

    @property
    @locked
    def ain29_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@29)

        Address: 11158
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_G)  # type: ignore

    @ain29_ef_config_g.setter
    @locked
    def ain29_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@29)

        Address: 11158
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_G, value)

    @property
    @locked
    def ain30_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@30)

        Address: 11160
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_G)  # type: ignore

    @ain30_ef_config_g.setter
    @locked
    def ain30_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@30)

        Address: 11160
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_G, value)

    @property
    @locked
    def ain31_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@31)

        Address: 11162
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_G)  # type: ignore

    @ain31_ef_config_g.setter
    @locked
    def ain31_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@31)

        Address: 11162
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_G, value)

    @property
    @locked
    def ain32_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@32)

        Address: 11164
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_G)  # type: ignore

    @ain32_ef_config_g.setter
    @locked
    def ain32_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@32)

        Address: 11164
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_G, value)

    @property
    @locked
    def ain33_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@33)

        Address: 11166
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_G)  # type: ignore

    @ain33_ef_config_g.setter
    @locked
    def ain33_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@33)

        Address: 11166
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_G, value)

    @property
    @locked
    def ain34_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@34)

        Address: 11168
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_G)  # type: ignore

    @ain34_ef_config_g.setter
    @locked
    def ain34_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@34)

        Address: 11168
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_G, value)

    @property
    @locked
    def ain35_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@35)

        Address: 11170
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_G)  # type: ignore

    @ain35_ef_config_g.setter
    @locked
    def ain35_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@35)

        Address: 11170
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_G, value)

    @property
    @locked
    def ain36_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@36)

        Address: 11172
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_G)  # type: ignore

    @ain36_ef_config_g.setter
    @locked
    def ain36_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@36)

        Address: 11172
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_G, value)

    @property
    @locked
    def ain37_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@37)

        Address: 11174
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_G)  # type: ignore

    @ain37_ef_config_g.setter
    @locked
    def ain37_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@37)

        Address: 11174
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_G, value)

    @property
    @locked
    def ain38_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@38)

        Address: 11176
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_G)  # type: ignore

    @ain38_ef_config_g.setter
    @locked
    def ain38_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@38)

        Address: 11176
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_G, value)

    @property
    @locked
    def ain39_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@39)

        Address: 11178
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_G)  # type: ignore

    @ain39_ef_config_g.setter
    @locked
    def ain39_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@39)

        Address: 11178
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_G, value)

    @property
    @locked
    def ain40_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@40)

        Address: 11180
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_G)  # type: ignore

    @ain40_ef_config_g.setter
    @locked
    def ain40_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@40)

        Address: 11180
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_G, value)

    @property
    @locked
    def ain41_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@41)

        Address: 11182
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_G)  # type: ignore

    @ain41_ef_config_g.setter
    @locked
    def ain41_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@41)

        Address: 11182
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_G, value)

    @property
    @locked
    def ain42_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@42)

        Address: 11184
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_G)  # type: ignore

    @ain42_ef_config_g.setter
    @locked
    def ain42_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@42)

        Address: 11184
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_G, value)

    @property
    @locked
    def ain43_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@43)

        Address: 11186
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_G)  # type: ignore

    @ain43_ef_config_g.setter
    @locked
    def ain43_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@43)

        Address: 11186
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_G, value)

    @property
    @locked
    def ain44_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@44)

        Address: 11188
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_G)  # type: ignore

    @ain44_ef_config_g.setter
    @locked
    def ain44_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@44)

        Address: 11188
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_G, value)

    @property
    @locked
    def ain45_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@45)

        Address: 11190
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_G)  # type: ignore

    @ain45_ef_config_g.setter
    @locked
    def ain45_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@45)

        Address: 11190
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_G, value)

    @property
    @locked
    def ain46_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@46)

        Address: 11192
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_G)  # type: ignore

    @ain46_ef_config_g.setter
    @locked
    def ain46_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@46)

        Address: 11192
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_G, value)

    @property
    @locked
    def ain47_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@47)

        Address: 11194
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_G)  # type: ignore

    @ain47_ef_config_g.setter
    @locked
    def ain47_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@47)

        Address: 11194
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_G, value)

    @property
    @locked
    def ain48_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@48)

        Address: 11196
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_G)  # type: ignore

    @ain48_ef_config_g.setter
    @locked
    def ain48_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@48)

        Address: 11196
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_G, value)

    @property
    @locked
    def ain49_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@49)

        Address: 11198
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_G)  # type: ignore

    @ain49_ef_config_g.setter
    @locked
    def ain49_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@49)

        Address: 11198
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_G, value)

    @property
    @locked
    def ain50_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@50)

        Address: 11200
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_G)  # type: ignore

    @ain50_ef_config_g.setter
    @locked
    def ain50_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@50)

        Address: 11200
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_G, value)

    @property
    @locked
    def ain51_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@51)

        Address: 11202
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_G)  # type: ignore

    @ain51_ef_config_g.setter
    @locked
    def ain51_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@51)

        Address: 11202
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_G, value)

    @property
    @locked
    def ain52_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@52)

        Address: 11204
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_G)  # type: ignore

    @ain52_ef_config_g.setter
    @locked
    def ain52_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@52)

        Address: 11204
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_G, value)

    @property
    @locked
    def ain53_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@53)

        Address: 11206
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_G)  # type: ignore

    @ain53_ef_config_g.setter
    @locked
    def ain53_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@53)

        Address: 11206
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_G, value)

    @property
    @locked
    def ain54_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@54)

        Address: 11208
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_G)  # type: ignore

    @ain54_ef_config_g.setter
    @locked
    def ain54_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@54)

        Address: 11208
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_G, value)

    @property
    @locked
    def ain55_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@55)

        Address: 11210
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_G)  # type: ignore

    @ain55_ef_config_g.setter
    @locked
    def ain55_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@55)

        Address: 11210
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_G, value)

    @property
    @locked
    def ain56_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@56)

        Address: 11212
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_G)  # type: ignore

    @ain56_ef_config_g.setter
    @locked
    def ain56_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@56)

        Address: 11212
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_G, value)

    @property
    @locked
    def ain57_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@57)

        Address: 11214
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_G)  # type: ignore

    @ain57_ef_config_g.setter
    @locked
    def ain57_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@57)

        Address: 11214
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_G, value)

    @property
    @locked
    def ain58_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@58)

        Address: 11216
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_G)  # type: ignore

    @ain58_ef_config_g.setter
    @locked
    def ain58_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@58)

        Address: 11216
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_G, value)

    @property
    @locked
    def ain59_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@59)

        Address: 11218
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_G)  # type: ignore

    @ain59_ef_config_g.setter
    @locked
    def ain59_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@59)

        Address: 11218
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_G, value)

    @property
    @locked
    def ain60_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@60)

        Address: 11220
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_G)  # type: ignore

    @ain60_ef_config_g.setter
    @locked
    def ain60_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@60)

        Address: 11220
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_G, value)

    @property
    @locked
    def ain61_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@61)

        Address: 11222
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_G)  # type: ignore

    @ain61_ef_config_g.setter
    @locked
    def ain61_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@61)

        Address: 11222
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_G, value)

    @property
    @locked
    def ain62_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@62)

        Address: 11224
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_G)  # type: ignore

    @ain62_ef_config_g.setter
    @locked
    def ain62_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@62)

        Address: 11224
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_G, value)

    @property
    @locked
    def ain63_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@63)

        Address: 11226
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_G)  # type: ignore

    @ain63_ef_config_g.setter
    @locked
    def ain63_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@63)

        Address: 11226
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_G, value)

    @property
    @locked
    def ain64_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@64)

        Address: 11228
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_G)  # type: ignore

    @ain64_ef_config_g.setter
    @locked
    def ain64_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@64)

        Address: 11228
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_G, value)

    @property
    @locked
    def ain65_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@65)

        Address: 11230
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_G)  # type: ignore

    @ain65_ef_config_g.setter
    @locked
    def ain65_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@65)

        Address: 11230
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_G, value)

    @property
    @locked
    def ain66_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@66)

        Address: 11232
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_G)  # type: ignore

    @ain66_ef_config_g.setter
    @locked
    def ain66_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@66)

        Address: 11232
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_G, value)

    @property
    @locked
    def ain67_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@67)

        Address: 11234
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_G)  # type: ignore

    @ain67_ef_config_g.setter
    @locked
    def ain67_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@67)

        Address: 11234
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_G, value)

    @property
    @locked
    def ain68_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@68)

        Address: 11236
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_G)  # type: ignore

    @ain68_ef_config_g.setter
    @locked
    def ain68_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@68)

        Address: 11236
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_G, value)

    @property
    @locked
    def ain69_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@69)

        Address: 11238
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_G)  # type: ignore

    @ain69_ef_config_g.setter
    @locked
    def ain69_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@69)

        Address: 11238
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_G, value)

    @property
    @locked
    def ain70_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@70)

        Address: 11240
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_G)  # type: ignore

    @ain70_ef_config_g.setter
    @locked
    def ain70_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@70)

        Address: 11240
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_G, value)

    @property
    @locked
    def ain71_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@71)

        Address: 11242
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_G)  # type: ignore

    @ain71_ef_config_g.setter
    @locked
    def ain71_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@71)

        Address: 11242
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_G, value)

    @property
    @locked
    def ain72_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@72)

        Address: 11244
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_G)  # type: ignore

    @ain72_ef_config_g.setter
    @locked
    def ain72_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@72)

        Address: 11244
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_G, value)

    @property
    @locked
    def ain73_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@73)

        Address: 11246
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_G)  # type: ignore

    @ain73_ef_config_g.setter
    @locked
    def ain73_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@73)

        Address: 11246
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_G, value)

    @property
    @locked
    def ain74_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@74)

        Address: 11248
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_G)  # type: ignore

    @ain74_ef_config_g.setter
    @locked
    def ain74_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@74)

        Address: 11248
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_G, value)

    @property
    @locked
    def ain75_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@75)

        Address: 11250
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_G)  # type: ignore

    @ain75_ef_config_g.setter
    @locked
    def ain75_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@75)

        Address: 11250
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_G, value)

    @property
    @locked
    def ain76_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@76)

        Address: 11252
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_G)  # type: ignore

    @ain76_ef_config_g.setter
    @locked
    def ain76_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@76)

        Address: 11252
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_G, value)

    @property
    @locked
    def ain77_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@77)

        Address: 11254
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_G)  # type: ignore

    @ain77_ef_config_g.setter
    @locked
    def ain77_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@77)

        Address: 11254
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_G, value)

    @property
    @locked
    def ain78_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@78)

        Address: 11256
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_G)  # type: ignore

    @ain78_ef_config_g.setter
    @locked
    def ain78_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@78)

        Address: 11256
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_G, value)

    @property
    @locked
    def ain79_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@79)

        Address: 11258
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_G)  # type: ignore

    @ain79_ef_config_g.setter
    @locked
    def ain79_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@79)

        Address: 11258
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_G, value)

    @property
    @locked
    def ain80_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@80)

        Address: 11260
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_G)  # type: ignore

    @ain80_ef_config_g.setter
    @locked
    def ain80_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@80)

        Address: 11260
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_G, value)

    @property
    @locked
    def ain81_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@81)

        Address: 11262
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_G)  # type: ignore

    @ain81_ef_config_g.setter
    @locked
    def ain81_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@81)

        Address: 11262
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_G, value)

    @property
    @locked
    def ain82_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@82)

        Address: 11264
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_G)  # type: ignore

    @ain82_ef_config_g.setter
    @locked
    def ain82_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@82)

        Address: 11264
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_G, value)

    @property
    @locked
    def ain83_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@83)

        Address: 11266
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_G)  # type: ignore

    @ain83_ef_config_g.setter
    @locked
    def ain83_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@83)

        Address: 11266
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_G, value)

    @property
    @locked
    def ain84_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@84)

        Address: 11268
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_G)  # type: ignore

    @ain84_ef_config_g.setter
    @locked
    def ain84_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@84)

        Address: 11268
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_G, value)

    @property
    @locked
    def ain85_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@85)

        Address: 11270
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_G)  # type: ignore

    @ain85_ef_config_g.setter
    @locked
    def ain85_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@85)

        Address: 11270
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_G, value)

    @property
    @locked
    def ain86_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@86)

        Address: 11272
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_G)  # type: ignore

    @ain86_ef_config_g.setter
    @locked
    def ain86_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@86)

        Address: 11272
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_G, value)

    @property
    @locked
    def ain87_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@87)

        Address: 11274
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_G)  # type: ignore

    @ain87_ef_config_g.setter
    @locked
    def ain87_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@87)

        Address: 11274
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_G, value)

    @property
    @locked
    def ain88_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@88)

        Address: 11276
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_G)  # type: ignore

    @ain88_ef_config_g.setter
    @locked
    def ain88_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@88)

        Address: 11276
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_G, value)

    @property
    @locked
    def ain89_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@89)

        Address: 11278
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_G)  # type: ignore

    @ain89_ef_config_g.setter
    @locked
    def ain89_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@89)

        Address: 11278
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_G, value)

    @property
    @locked
    def ain90_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@90)

        Address: 11280
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_G)  # type: ignore

    @ain90_ef_config_g.setter
    @locked
    def ain90_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@90)

        Address: 11280
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_G, value)

    @property
    @locked
    def ain91_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@91)

        Address: 11282
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_G)  # type: ignore

    @ain91_ef_config_g.setter
    @locked
    def ain91_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@91)

        Address: 11282
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_G, value)

    @property
    @locked
    def ain92_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@92)

        Address: 11284
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_G)  # type: ignore

    @ain92_ef_config_g.setter
    @locked
    def ain92_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@92)

        Address: 11284
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_G, value)

    @property
    @locked
    def ain93_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@93)

        Address: 11286
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_G)  # type: ignore

    @ain93_ef_config_g.setter
    @locked
    def ain93_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@93)

        Address: 11286
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_G, value)

    @property
    @locked
    def ain94_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@94)

        Address: 11288
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_G)  # type: ignore

    @ain94_ef_config_g.setter
    @locked
    def ain94_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@94)

        Address: 11288
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_G, value)

    @property
    @locked
    def ain95_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@95)

        Address: 11290
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_G)  # type: ignore

    @ain95_ef_config_g.setter
    @locked
    def ain95_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@95)

        Address: 11290
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_G, value)

    @property
    @locked
    def ain96_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@96)

        Address: 11292
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_G)  # type: ignore

    @ain96_ef_config_g.setter
    @locked
    def ain96_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@96)

        Address: 11292
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_G, value)

    @property
    @locked
    def ain97_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@97)

        Address: 11294
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_G)  # type: ignore

    @ain97_ef_config_g.setter
    @locked
    def ain97_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@97)

        Address: 11294
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_G, value)

    @property
    @locked
    def ain98_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@98)

        Address: 11296
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_G)  # type: ignore

    @ain98_ef_config_g.setter
    @locked
    def ain98_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@98)

        Address: 11296
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_G, value)

    @property
    @locked
    def ain99_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@99)

        Address: 11298
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_G)  # type: ignore

    @ain99_ef_config_g.setter
    @locked
    def ain99_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@99)

        Address: 11298
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_G, value)

    @property
    @locked
    def ain100_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@100)

        Address: 11300
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_G)  # type: ignore

    @ain100_ef_config_g.setter
    @locked
    def ain100_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@100)

        Address: 11300
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_G, value)

    @property
    @locked
    def ain101_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@101)

        Address: 11302
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_G)  # type: ignore

    @ain101_ef_config_g.setter
    @locked
    def ain101_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@101)

        Address: 11302
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_G, value)

    @property
    @locked
    def ain102_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@102)

        Address: 11304
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_G)  # type: ignore

    @ain102_ef_config_g.setter
    @locked
    def ain102_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@102)

        Address: 11304
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_G, value)

    @property
    @locked
    def ain103_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@103)

        Address: 11306
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_G)  # type: ignore

    @ain103_ef_config_g.setter
    @locked
    def ain103_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@103)

        Address: 11306
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_G, value)

    @property
    @locked
    def ain104_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@104)

        Address: 11308
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_G)  # type: ignore

    @ain104_ef_config_g.setter
    @locked
    def ain104_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@104)

        Address: 11308
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_G, value)

    @property
    @locked
    def ain105_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@105)

        Address: 11310
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_G)  # type: ignore

    @ain105_ef_config_g.setter
    @locked
    def ain105_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@105)

        Address: 11310
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_G, value)

    @property
    @locked
    def ain106_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@106)

        Address: 11312
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_G)  # type: ignore

    @ain106_ef_config_g.setter
    @locked
    def ain106_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@106)

        Address: 11312
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_G, value)

    @property
    @locked
    def ain107_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@107)

        Address: 11314
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_G)  # type: ignore

    @ain107_ef_config_g.setter
    @locked
    def ain107_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@107)

        Address: 11314
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_G, value)

    @property
    @locked
    def ain108_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@108)

        Address: 11316
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_G)  # type: ignore

    @ain108_ef_config_g.setter
    @locked
    def ain108_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@108)

        Address: 11316
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_G, value)

    @property
    @locked
    def ain109_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@109)

        Address: 11318
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_G)  # type: ignore

    @ain109_ef_config_g.setter
    @locked
    def ain109_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@109)

        Address: 11318
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_G, value)

    @property
    @locked
    def ain110_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@110)

        Address: 11320
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_G)  # type: ignore

    @ain110_ef_config_g.setter
    @locked
    def ain110_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@110)

        Address: 11320
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_G, value)

    @property
    @locked
    def ain111_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@111)

        Address: 11322
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_G)  # type: ignore

    @ain111_ef_config_g.setter
    @locked
    def ain111_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@111)

        Address: 11322
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_G, value)

    @property
    @locked
    def ain112_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@112)

        Address: 11324
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_G)  # type: ignore

    @ain112_ef_config_g.setter
    @locked
    def ain112_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@112)

        Address: 11324
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_G, value)

    @property
    @locked
    def ain113_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@113)

        Address: 11326
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_G)  # type: ignore

    @ain113_ef_config_g.setter
    @locked
    def ain113_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@113)

        Address: 11326
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_G, value)

    @property
    @locked
    def ain114_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@114)

        Address: 11328
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_G)  # type: ignore

    @ain114_ef_config_g.setter
    @locked
    def ain114_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@114)

        Address: 11328
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_G, value)

    @property
    @locked
    def ain115_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@115)

        Address: 11330
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_G)  # type: ignore

    @ain115_ef_config_g.setter
    @locked
    def ain115_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@115)

        Address: 11330
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_G, value)

    @property
    @locked
    def ain116_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@116)

        Address: 11332
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_G)  # type: ignore

    @ain116_ef_config_g.setter
    @locked
    def ain116_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@116)

        Address: 11332
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_G, value)

    @property
    @locked
    def ain117_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@117)

        Address: 11334
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_G)  # type: ignore

    @ain117_ef_config_g.setter
    @locked
    def ain117_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@117)

        Address: 11334
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_G, value)

    @property
    @locked
    def ain118_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@118)

        Address: 11336
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_G)  # type: ignore

    @ain118_ef_config_g.setter
    @locked
    def ain118_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@118)

        Address: 11336
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_G, value)

    @property
    @locked
    def ain119_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@119)

        Address: 11338
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_G)  # type: ignore

    @ain119_ef_config_g.setter
    @locked
    def ain119_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@119)

        Address: 11338
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_G, value)

    @property
    @locked
    def ain120_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@120)

        Address: 11340
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_G)  # type: ignore

    @ain120_ef_config_g.setter
    @locked
    def ain120_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@120)

        Address: 11340
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_G, value)

    @property
    @locked
    def ain121_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@121)

        Address: 11342
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_G)  # type: ignore

    @ain121_ef_config_g.setter
    @locked
    def ain121_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@121)

        Address: 11342
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_G, value)

    @property
    @locked
    def ain122_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@122)

        Address: 11344
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_G)  # type: ignore

    @ain122_ef_config_g.setter
    @locked
    def ain122_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@122)

        Address: 11344
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_G, value)

    @property
    @locked
    def ain123_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@123)

        Address: 11346
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_G)  # type: ignore

    @ain123_ef_config_g.setter
    @locked
    def ain123_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@123)

        Address: 11346
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_G, value)

    @property
    @locked
    def ain124_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@124)

        Address: 11348
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_G)  # type: ignore

    @ain124_ef_config_g.setter
    @locked
    def ain124_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@124)

        Address: 11348
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_G, value)

    @property
    @locked
    def ain125_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@125)

        Address: 11350
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_G)  # type: ignore

    @ain125_ef_config_g.setter
    @locked
    def ain125_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@125)

        Address: 11350
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_G, value)

    @property
    @locked
    def ain126_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@126)

        Address: 11352
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_G)  # type: ignore

    @ain126_ef_config_g.setter
    @locked
    def ain126_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@126)

        Address: 11352
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_G, value)

    @property
    @locked
    def ain127_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@127)

        Address: 11354
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_G)  # type: ignore

    @ain127_ef_config_g.setter
    @locked
    def ain127_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@127)

        Address: 11354
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_G, value)

    @property
    @locked
    def ain128_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@128)

        Address: 11356
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_G)  # type: ignore

    @ain128_ef_config_g.setter
    @locked
    def ain128_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@128)

        Address: 11356
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_G, value)

    @property
    @locked
    def ain129_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@129)

        Address: 11358
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_G)  # type: ignore

    @ain129_ef_config_g.setter
    @locked
    def ain129_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@129)

        Address: 11358
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_G, value)

    @property
    @locked
    def ain130_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@130)

        Address: 11360
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_G)  # type: ignore

    @ain130_ef_config_g.setter
    @locked
    def ain130_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@130)

        Address: 11360
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_G, value)

    @property
    @locked
    def ain131_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@131)

        Address: 11362
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_G)  # type: ignore

    @ain131_ef_config_g.setter
    @locked
    def ain131_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@131)

        Address: 11362
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_G, value)

    @property
    @locked
    def ain132_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@132)

        Address: 11364
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_G)  # type: ignore

    @ain132_ef_config_g.setter
    @locked
    def ain132_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@132)

        Address: 11364
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_G, value)

    @property
    @locked
    def ain133_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@133)

        Address: 11366
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_G)  # type: ignore

    @ain133_ef_config_g.setter
    @locked
    def ain133_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@133)

        Address: 11366
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_G, value)

    @property
    @locked
    def ain134_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@134)

        Address: 11368
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_G)  # type: ignore

    @ain134_ef_config_g.setter
    @locked
    def ain134_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@134)

        Address: 11368
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_G, value)

    @property
    @locked
    def ain135_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@135)

        Address: 11370
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_G)  # type: ignore

    @ain135_ef_config_g.setter
    @locked
    def ain135_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@135)

        Address: 11370
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_G, value)

    @property
    @locked
    def ain136_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@136)

        Address: 11372
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_G)  # type: ignore

    @ain136_ef_config_g.setter
    @locked
    def ain136_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@136)

        Address: 11372
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_G, value)

    @property
    @locked
    def ain137_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@137)

        Address: 11374
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_G)  # type: ignore

    @ain137_ef_config_g.setter
    @locked
    def ain137_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@137)

        Address: 11374
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_G, value)

    @property
    @locked
    def ain138_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@138)

        Address: 11376
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_G)  # type: ignore

    @ain138_ef_config_g.setter
    @locked
    def ain138_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@138)

        Address: 11376
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_G, value)

    @property
    @locked
    def ain139_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@139)

        Address: 11378
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_G)  # type: ignore

    @ain139_ef_config_g.setter
    @locked
    def ain139_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@139)

        Address: 11378
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_G, value)

    @property
    @locked
    def ain140_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@140)

        Address: 11380
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_G)  # type: ignore

    @ain140_ef_config_g.setter
    @locked
    def ain140_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@140)

        Address: 11380
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_G, value)

    @property
    @locked
    def ain141_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@141)

        Address: 11382
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_G)  # type: ignore

    @ain141_ef_config_g.setter
    @locked
    def ain141_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@141)

        Address: 11382
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_G, value)

    @property
    @locked
    def ain142_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@142)

        Address: 11384
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_G)  # type: ignore

    @ain142_ef_config_g.setter
    @locked
    def ain142_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@142)

        Address: 11384
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_G, value)

    @property
    @locked
    def ain143_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@143)

        Address: 11386
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_G)  # type: ignore

    @ain143_ef_config_g.setter
    @locked
    def ain143_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@143)

        Address: 11386
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_G, value)

    @property
    @locked
    def ain144_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@144)

        Address: 11388
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_G)  # type: ignore

    @ain144_ef_config_g.setter
    @locked
    def ain144_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@144)

        Address: 11388
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_G, value)

    @property
    @locked
    def ain145_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@145)

        Address: 11390
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_G)  # type: ignore

    @ain145_ef_config_g.setter
    @locked
    def ain145_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@145)

        Address: 11390
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_G, value)

    @property
    @locked
    def ain146_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@146)

        Address: 11392
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_G)  # type: ignore

    @ain146_ef_config_g.setter
    @locked
    def ain146_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@146)

        Address: 11392
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_G, value)

    @property
    @locked
    def ain147_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@147)

        Address: 11394
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_G)  # type: ignore

    @ain147_ef_config_g.setter
    @locked
    def ain147_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@147)

        Address: 11394
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_G, value)

    @property
    @locked
    def ain148_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@148)

        Address: 11396
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_G)  # type: ignore

    @ain148_ef_config_g.setter
    @locked
    def ain148_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@148)

        Address: 11396
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_G, value)

    @property
    @locked
    def ain149_ef_config_g(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@149)

        Address: 11398
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_G)  # type: ignore

    @ain149_ef_config_g.setter
    @locked
    def ain149_ef_config_g(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_G (AIN#(0:149)_EF_CONFIG_G@149)

        Address: 11398
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_G, value)

    @property
    @locked
    def ain0_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@0)

        Address: 11400
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_H)  # type: ignore

    @ain0_ef_config_h.setter
    @locked
    def ain0_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@0)

        Address: 11400
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_H, value)

    @property
    @locked
    def ain1_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@1)

        Address: 11402
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_H)  # type: ignore

    @ain1_ef_config_h.setter
    @locked
    def ain1_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@1)

        Address: 11402
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_H, value)

    @property
    @locked
    def ain2_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@2)

        Address: 11404
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_H)  # type: ignore

    @ain2_ef_config_h.setter
    @locked
    def ain2_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@2)

        Address: 11404
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_H, value)

    @property
    @locked
    def ain3_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@3)

        Address: 11406
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_H)  # type: ignore

    @ain3_ef_config_h.setter
    @locked
    def ain3_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@3)

        Address: 11406
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_H, value)

    @property
    @locked
    def ain4_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@4)

        Address: 11408
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_H)  # type: ignore

    @ain4_ef_config_h.setter
    @locked
    def ain4_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@4)

        Address: 11408
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_H, value)

    @property
    @locked
    def ain5_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@5)

        Address: 11410
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_H)  # type: ignore

    @ain5_ef_config_h.setter
    @locked
    def ain5_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@5)

        Address: 11410
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_H, value)

    @property
    @locked
    def ain6_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@6)

        Address: 11412
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_H)  # type: ignore

    @ain6_ef_config_h.setter
    @locked
    def ain6_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@6)

        Address: 11412
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_H, value)

    @property
    @locked
    def ain7_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@7)

        Address: 11414
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_H)  # type: ignore

    @ain7_ef_config_h.setter
    @locked
    def ain7_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@7)

        Address: 11414
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_H, value)

    @property
    @locked
    def ain8_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@8)

        Address: 11416
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_H)  # type: ignore

    @ain8_ef_config_h.setter
    @locked
    def ain8_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@8)

        Address: 11416
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_H, value)

    @property
    @locked
    def ain9_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@9)

        Address: 11418
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_H)  # type: ignore

    @ain9_ef_config_h.setter
    @locked
    def ain9_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@9)

        Address: 11418
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_H, value)

    @property
    @locked
    def ain10_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@10)

        Address: 11420
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_H)  # type: ignore

    @ain10_ef_config_h.setter
    @locked
    def ain10_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@10)

        Address: 11420
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_H, value)

    @property
    @locked
    def ain11_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@11)

        Address: 11422
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_H)  # type: ignore

    @ain11_ef_config_h.setter
    @locked
    def ain11_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@11)

        Address: 11422
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_H, value)

    @property
    @locked
    def ain12_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@12)

        Address: 11424
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_H)  # type: ignore

    @ain12_ef_config_h.setter
    @locked
    def ain12_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@12)

        Address: 11424
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_H, value)

    @property
    @locked
    def ain13_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@13)

        Address: 11426
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_H)  # type: ignore

    @ain13_ef_config_h.setter
    @locked
    def ain13_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@13)

        Address: 11426
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_H, value)

    @property
    @locked
    def ain14_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@14)

        Address: 11428
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_H)  # type: ignore

    @ain14_ef_config_h.setter
    @locked
    def ain14_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@14)

        Address: 11428
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_H, value)

    @property
    @locked
    def ain15_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@15)

        Address: 11430
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_H)  # type: ignore

    @ain15_ef_config_h.setter
    @locked
    def ain15_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@15)

        Address: 11430
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_H, value)

    @property
    @locked
    def ain16_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@16)

        Address: 11432
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_H)  # type: ignore

    @ain16_ef_config_h.setter
    @locked
    def ain16_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@16)

        Address: 11432
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_H, value)

    @property
    @locked
    def ain17_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@17)

        Address: 11434
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_H)  # type: ignore

    @ain17_ef_config_h.setter
    @locked
    def ain17_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@17)

        Address: 11434
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_H, value)

    @property
    @locked
    def ain18_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@18)

        Address: 11436
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_H)  # type: ignore

    @ain18_ef_config_h.setter
    @locked
    def ain18_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@18)

        Address: 11436
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_H, value)

    @property
    @locked
    def ain19_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@19)

        Address: 11438
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_H)  # type: ignore

    @ain19_ef_config_h.setter
    @locked
    def ain19_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@19)

        Address: 11438
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_H, value)

    @property
    @locked
    def ain20_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@20)

        Address: 11440
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_H)  # type: ignore

    @ain20_ef_config_h.setter
    @locked
    def ain20_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@20)

        Address: 11440
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_H, value)

    @property
    @locked
    def ain21_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@21)

        Address: 11442
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_H)  # type: ignore

    @ain21_ef_config_h.setter
    @locked
    def ain21_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@21)

        Address: 11442
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_H, value)

    @property
    @locked
    def ain22_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@22)

        Address: 11444
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_H)  # type: ignore

    @ain22_ef_config_h.setter
    @locked
    def ain22_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@22)

        Address: 11444
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_H, value)

    @property
    @locked
    def ain23_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@23)

        Address: 11446
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_H)  # type: ignore

    @ain23_ef_config_h.setter
    @locked
    def ain23_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@23)

        Address: 11446
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_H, value)

    @property
    @locked
    def ain24_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@24)

        Address: 11448
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_H)  # type: ignore

    @ain24_ef_config_h.setter
    @locked
    def ain24_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@24)

        Address: 11448
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_H, value)

    @property
    @locked
    def ain25_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@25)

        Address: 11450
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_H)  # type: ignore

    @ain25_ef_config_h.setter
    @locked
    def ain25_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@25)

        Address: 11450
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_H, value)

    @property
    @locked
    def ain26_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@26)

        Address: 11452
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_H)  # type: ignore

    @ain26_ef_config_h.setter
    @locked
    def ain26_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@26)

        Address: 11452
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_H, value)

    @property
    @locked
    def ain27_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@27)

        Address: 11454
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_H)  # type: ignore

    @ain27_ef_config_h.setter
    @locked
    def ain27_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@27)

        Address: 11454
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_H, value)

    @property
    @locked
    def ain28_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@28)

        Address: 11456
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_H)  # type: ignore

    @ain28_ef_config_h.setter
    @locked
    def ain28_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@28)

        Address: 11456
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_H, value)

    @property
    @locked
    def ain29_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@29)

        Address: 11458
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_H)  # type: ignore

    @ain29_ef_config_h.setter
    @locked
    def ain29_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@29)

        Address: 11458
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_H, value)

    @property
    @locked
    def ain30_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@30)

        Address: 11460
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_H)  # type: ignore

    @ain30_ef_config_h.setter
    @locked
    def ain30_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@30)

        Address: 11460
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_H, value)

    @property
    @locked
    def ain31_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@31)

        Address: 11462
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_H)  # type: ignore

    @ain31_ef_config_h.setter
    @locked
    def ain31_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@31)

        Address: 11462
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_H, value)

    @property
    @locked
    def ain32_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@32)

        Address: 11464
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_H)  # type: ignore

    @ain32_ef_config_h.setter
    @locked
    def ain32_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@32)

        Address: 11464
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_H, value)

    @property
    @locked
    def ain33_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@33)

        Address: 11466
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_H)  # type: ignore

    @ain33_ef_config_h.setter
    @locked
    def ain33_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@33)

        Address: 11466
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_H, value)

    @property
    @locked
    def ain34_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@34)

        Address: 11468
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_H)  # type: ignore

    @ain34_ef_config_h.setter
    @locked
    def ain34_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@34)

        Address: 11468
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_H, value)

    @property
    @locked
    def ain35_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@35)

        Address: 11470
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_H)  # type: ignore

    @ain35_ef_config_h.setter
    @locked
    def ain35_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@35)

        Address: 11470
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_H, value)

    @property
    @locked
    def ain36_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@36)

        Address: 11472
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_H)  # type: ignore

    @ain36_ef_config_h.setter
    @locked
    def ain36_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@36)

        Address: 11472
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_H, value)

    @property
    @locked
    def ain37_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@37)

        Address: 11474
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_H)  # type: ignore

    @ain37_ef_config_h.setter
    @locked
    def ain37_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@37)

        Address: 11474
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_H, value)

    @property
    @locked
    def ain38_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@38)

        Address: 11476
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_H)  # type: ignore

    @ain38_ef_config_h.setter
    @locked
    def ain38_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@38)

        Address: 11476
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_H, value)

    @property
    @locked
    def ain39_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@39)

        Address: 11478
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_H)  # type: ignore

    @ain39_ef_config_h.setter
    @locked
    def ain39_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@39)

        Address: 11478
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_H, value)

    @property
    @locked
    def ain40_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@40)

        Address: 11480
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_H)  # type: ignore

    @ain40_ef_config_h.setter
    @locked
    def ain40_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@40)

        Address: 11480
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_H, value)

    @property
    @locked
    def ain41_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@41)

        Address: 11482
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_H)  # type: ignore

    @ain41_ef_config_h.setter
    @locked
    def ain41_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@41)

        Address: 11482
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_H, value)

    @property
    @locked
    def ain42_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@42)

        Address: 11484
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_H)  # type: ignore

    @ain42_ef_config_h.setter
    @locked
    def ain42_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@42)

        Address: 11484
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_H, value)

    @property
    @locked
    def ain43_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@43)

        Address: 11486
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_H)  # type: ignore

    @ain43_ef_config_h.setter
    @locked
    def ain43_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@43)

        Address: 11486
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_H, value)

    @property
    @locked
    def ain44_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@44)

        Address: 11488
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_H)  # type: ignore

    @ain44_ef_config_h.setter
    @locked
    def ain44_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@44)

        Address: 11488
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_H, value)

    @property
    @locked
    def ain45_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@45)

        Address: 11490
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_H)  # type: ignore

    @ain45_ef_config_h.setter
    @locked
    def ain45_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@45)

        Address: 11490
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_H, value)

    @property
    @locked
    def ain46_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@46)

        Address: 11492
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_H)  # type: ignore

    @ain46_ef_config_h.setter
    @locked
    def ain46_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@46)

        Address: 11492
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_H, value)

    @property
    @locked
    def ain47_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@47)

        Address: 11494
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_H)  # type: ignore

    @ain47_ef_config_h.setter
    @locked
    def ain47_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@47)

        Address: 11494
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_H, value)

    @property
    @locked
    def ain48_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@48)

        Address: 11496
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_H)  # type: ignore

    @ain48_ef_config_h.setter
    @locked
    def ain48_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@48)

        Address: 11496
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_H, value)

    @property
    @locked
    def ain49_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@49)

        Address: 11498
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_H)  # type: ignore

    @ain49_ef_config_h.setter
    @locked
    def ain49_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@49)

        Address: 11498
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_H, value)

    @property
    @locked
    def ain50_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@50)

        Address: 11500
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_H)  # type: ignore

    @ain50_ef_config_h.setter
    @locked
    def ain50_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@50)

        Address: 11500
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_H, value)

    @property
    @locked
    def ain51_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@51)

        Address: 11502
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_H)  # type: ignore

    @ain51_ef_config_h.setter
    @locked
    def ain51_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@51)

        Address: 11502
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_H, value)

    @property
    @locked
    def ain52_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@52)

        Address: 11504
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_H)  # type: ignore

    @ain52_ef_config_h.setter
    @locked
    def ain52_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@52)

        Address: 11504
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_H, value)

    @property
    @locked
    def ain53_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@53)

        Address: 11506
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_H)  # type: ignore

    @ain53_ef_config_h.setter
    @locked
    def ain53_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@53)

        Address: 11506
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_H, value)

    @property
    @locked
    def ain54_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@54)

        Address: 11508
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_H)  # type: ignore

    @ain54_ef_config_h.setter
    @locked
    def ain54_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@54)

        Address: 11508
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_H, value)

    @property
    @locked
    def ain55_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@55)

        Address: 11510
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_H)  # type: ignore

    @ain55_ef_config_h.setter
    @locked
    def ain55_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@55)

        Address: 11510
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_H, value)

    @property
    @locked
    def ain56_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@56)

        Address: 11512
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_H)  # type: ignore

    @ain56_ef_config_h.setter
    @locked
    def ain56_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@56)

        Address: 11512
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_H, value)

    @property
    @locked
    def ain57_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@57)

        Address: 11514
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_H)  # type: ignore

    @ain57_ef_config_h.setter
    @locked
    def ain57_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@57)

        Address: 11514
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_H, value)

    @property
    @locked
    def ain58_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@58)

        Address: 11516
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_H)  # type: ignore

    @ain58_ef_config_h.setter
    @locked
    def ain58_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@58)

        Address: 11516
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_H, value)

    @property
    @locked
    def ain59_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@59)

        Address: 11518
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_H)  # type: ignore

    @ain59_ef_config_h.setter
    @locked
    def ain59_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@59)

        Address: 11518
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_H, value)

    @property
    @locked
    def ain60_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@60)

        Address: 11520
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_H)  # type: ignore

    @ain60_ef_config_h.setter
    @locked
    def ain60_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@60)

        Address: 11520
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_H, value)

    @property
    @locked
    def ain61_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@61)

        Address: 11522
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_H)  # type: ignore

    @ain61_ef_config_h.setter
    @locked
    def ain61_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@61)

        Address: 11522
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_H, value)

    @property
    @locked
    def ain62_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@62)

        Address: 11524
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_H)  # type: ignore

    @ain62_ef_config_h.setter
    @locked
    def ain62_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@62)

        Address: 11524
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_H, value)

    @property
    @locked
    def ain63_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@63)

        Address: 11526
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_H)  # type: ignore

    @ain63_ef_config_h.setter
    @locked
    def ain63_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@63)

        Address: 11526
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_H, value)

    @property
    @locked
    def ain64_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@64)

        Address: 11528
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_H)  # type: ignore

    @ain64_ef_config_h.setter
    @locked
    def ain64_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@64)

        Address: 11528
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_H, value)

    @property
    @locked
    def ain65_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@65)

        Address: 11530
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_H)  # type: ignore

    @ain65_ef_config_h.setter
    @locked
    def ain65_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@65)

        Address: 11530
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_H, value)

    @property
    @locked
    def ain66_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@66)

        Address: 11532
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_H)  # type: ignore

    @ain66_ef_config_h.setter
    @locked
    def ain66_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@66)

        Address: 11532
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_H, value)

    @property
    @locked
    def ain67_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@67)

        Address: 11534
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_H)  # type: ignore

    @ain67_ef_config_h.setter
    @locked
    def ain67_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@67)

        Address: 11534
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_H, value)

    @property
    @locked
    def ain68_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@68)

        Address: 11536
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_H)  # type: ignore

    @ain68_ef_config_h.setter
    @locked
    def ain68_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@68)

        Address: 11536
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_H, value)

    @property
    @locked
    def ain69_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@69)

        Address: 11538
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_H)  # type: ignore

    @ain69_ef_config_h.setter
    @locked
    def ain69_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@69)

        Address: 11538
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_H, value)

    @property
    @locked
    def ain70_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@70)

        Address: 11540
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_H)  # type: ignore

    @ain70_ef_config_h.setter
    @locked
    def ain70_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@70)

        Address: 11540
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_H, value)

    @property
    @locked
    def ain71_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@71)

        Address: 11542
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_H)  # type: ignore

    @ain71_ef_config_h.setter
    @locked
    def ain71_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@71)

        Address: 11542
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_H, value)

    @property
    @locked
    def ain72_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@72)

        Address: 11544
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_H)  # type: ignore

    @ain72_ef_config_h.setter
    @locked
    def ain72_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@72)

        Address: 11544
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_H, value)

    @property
    @locked
    def ain73_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@73)

        Address: 11546
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_H)  # type: ignore

    @ain73_ef_config_h.setter
    @locked
    def ain73_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@73)

        Address: 11546
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_H, value)

    @property
    @locked
    def ain74_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@74)

        Address: 11548
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_H)  # type: ignore

    @ain74_ef_config_h.setter
    @locked
    def ain74_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@74)

        Address: 11548
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_H, value)

    @property
    @locked
    def ain75_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@75)

        Address: 11550
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_H)  # type: ignore

    @ain75_ef_config_h.setter
    @locked
    def ain75_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@75)

        Address: 11550
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_H, value)

    @property
    @locked
    def ain76_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@76)

        Address: 11552
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_H)  # type: ignore

    @ain76_ef_config_h.setter
    @locked
    def ain76_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@76)

        Address: 11552
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_H, value)

    @property
    @locked
    def ain77_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@77)

        Address: 11554
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_H)  # type: ignore

    @ain77_ef_config_h.setter
    @locked
    def ain77_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@77)

        Address: 11554
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_H, value)

    @property
    @locked
    def ain78_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@78)

        Address: 11556
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_H)  # type: ignore

    @ain78_ef_config_h.setter
    @locked
    def ain78_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@78)

        Address: 11556
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_H, value)

    @property
    @locked
    def ain79_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@79)

        Address: 11558
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_H)  # type: ignore

    @ain79_ef_config_h.setter
    @locked
    def ain79_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@79)

        Address: 11558
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_H, value)

    @property
    @locked
    def ain80_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@80)

        Address: 11560
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_H)  # type: ignore

    @ain80_ef_config_h.setter
    @locked
    def ain80_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@80)

        Address: 11560
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_H, value)

    @property
    @locked
    def ain81_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@81)

        Address: 11562
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_H)  # type: ignore

    @ain81_ef_config_h.setter
    @locked
    def ain81_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@81)

        Address: 11562
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_H, value)

    @property
    @locked
    def ain82_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@82)

        Address: 11564
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_H)  # type: ignore

    @ain82_ef_config_h.setter
    @locked
    def ain82_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@82)

        Address: 11564
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_H, value)

    @property
    @locked
    def ain83_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@83)

        Address: 11566
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_H)  # type: ignore

    @ain83_ef_config_h.setter
    @locked
    def ain83_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@83)

        Address: 11566
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_H, value)

    @property
    @locked
    def ain84_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@84)

        Address: 11568
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_H)  # type: ignore

    @ain84_ef_config_h.setter
    @locked
    def ain84_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@84)

        Address: 11568
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_H, value)

    @property
    @locked
    def ain85_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@85)

        Address: 11570
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_H)  # type: ignore

    @ain85_ef_config_h.setter
    @locked
    def ain85_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@85)

        Address: 11570
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_H, value)

    @property
    @locked
    def ain86_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@86)

        Address: 11572
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_H)  # type: ignore

    @ain86_ef_config_h.setter
    @locked
    def ain86_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@86)

        Address: 11572
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_H, value)

    @property
    @locked
    def ain87_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@87)

        Address: 11574
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_H)  # type: ignore

    @ain87_ef_config_h.setter
    @locked
    def ain87_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@87)

        Address: 11574
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_H, value)

    @property
    @locked
    def ain88_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@88)

        Address: 11576
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_H)  # type: ignore

    @ain88_ef_config_h.setter
    @locked
    def ain88_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@88)

        Address: 11576
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_H, value)

    @property
    @locked
    def ain89_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@89)

        Address: 11578
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_H)  # type: ignore

    @ain89_ef_config_h.setter
    @locked
    def ain89_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@89)

        Address: 11578
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_H, value)

    @property
    @locked
    def ain90_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@90)

        Address: 11580
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_H)  # type: ignore

    @ain90_ef_config_h.setter
    @locked
    def ain90_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@90)

        Address: 11580
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_H, value)

    @property
    @locked
    def ain91_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@91)

        Address: 11582
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_H)  # type: ignore

    @ain91_ef_config_h.setter
    @locked
    def ain91_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@91)

        Address: 11582
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_H, value)

    @property
    @locked
    def ain92_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@92)

        Address: 11584
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_H)  # type: ignore

    @ain92_ef_config_h.setter
    @locked
    def ain92_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@92)

        Address: 11584
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_H, value)

    @property
    @locked
    def ain93_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@93)

        Address: 11586
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_H)  # type: ignore

    @ain93_ef_config_h.setter
    @locked
    def ain93_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@93)

        Address: 11586
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_H, value)

    @property
    @locked
    def ain94_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@94)

        Address: 11588
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_H)  # type: ignore

    @ain94_ef_config_h.setter
    @locked
    def ain94_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@94)

        Address: 11588
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_H, value)

    @property
    @locked
    def ain95_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@95)

        Address: 11590
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_H)  # type: ignore

    @ain95_ef_config_h.setter
    @locked
    def ain95_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@95)

        Address: 11590
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_H, value)

    @property
    @locked
    def ain96_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@96)

        Address: 11592
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_H)  # type: ignore

    @ain96_ef_config_h.setter
    @locked
    def ain96_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@96)

        Address: 11592
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_H, value)

    @property
    @locked
    def ain97_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@97)

        Address: 11594
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_H)  # type: ignore

    @ain97_ef_config_h.setter
    @locked
    def ain97_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@97)

        Address: 11594
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_H, value)

    @property
    @locked
    def ain98_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@98)

        Address: 11596
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_H)  # type: ignore

    @ain98_ef_config_h.setter
    @locked
    def ain98_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@98)

        Address: 11596
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_H, value)

    @property
    @locked
    def ain99_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@99)

        Address: 11598
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_H)  # type: ignore

    @ain99_ef_config_h.setter
    @locked
    def ain99_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@99)

        Address: 11598
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_H, value)

    @property
    @locked
    def ain100_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@100)

        Address: 11600
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_H)  # type: ignore

    @ain100_ef_config_h.setter
    @locked
    def ain100_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@100)

        Address: 11600
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_H, value)

    @property
    @locked
    def ain101_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@101)

        Address: 11602
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_H)  # type: ignore

    @ain101_ef_config_h.setter
    @locked
    def ain101_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@101)

        Address: 11602
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_H, value)

    @property
    @locked
    def ain102_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@102)

        Address: 11604
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_H)  # type: ignore

    @ain102_ef_config_h.setter
    @locked
    def ain102_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@102)

        Address: 11604
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_H, value)

    @property
    @locked
    def ain103_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@103)

        Address: 11606
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_H)  # type: ignore

    @ain103_ef_config_h.setter
    @locked
    def ain103_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@103)

        Address: 11606
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_H, value)

    @property
    @locked
    def ain104_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@104)

        Address: 11608
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_H)  # type: ignore

    @ain104_ef_config_h.setter
    @locked
    def ain104_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@104)

        Address: 11608
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_H, value)

    @property
    @locked
    def ain105_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@105)

        Address: 11610
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_H)  # type: ignore

    @ain105_ef_config_h.setter
    @locked
    def ain105_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@105)

        Address: 11610
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_H, value)

    @property
    @locked
    def ain106_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@106)

        Address: 11612
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_H)  # type: ignore

    @ain106_ef_config_h.setter
    @locked
    def ain106_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@106)

        Address: 11612
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_H, value)

    @property
    @locked
    def ain107_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@107)

        Address: 11614
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_H)  # type: ignore

    @ain107_ef_config_h.setter
    @locked
    def ain107_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@107)

        Address: 11614
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_H, value)

    @property
    @locked
    def ain108_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@108)

        Address: 11616
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_H)  # type: ignore

    @ain108_ef_config_h.setter
    @locked
    def ain108_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@108)

        Address: 11616
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_H, value)

    @property
    @locked
    def ain109_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@109)

        Address: 11618
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_H)  # type: ignore

    @ain109_ef_config_h.setter
    @locked
    def ain109_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@109)

        Address: 11618
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_H, value)

    @property
    @locked
    def ain110_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@110)

        Address: 11620
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_H)  # type: ignore

    @ain110_ef_config_h.setter
    @locked
    def ain110_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@110)

        Address: 11620
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_H, value)

    @property
    @locked
    def ain111_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@111)

        Address: 11622
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_H)  # type: ignore

    @ain111_ef_config_h.setter
    @locked
    def ain111_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@111)

        Address: 11622
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_H, value)

    @property
    @locked
    def ain112_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@112)

        Address: 11624
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_H)  # type: ignore

    @ain112_ef_config_h.setter
    @locked
    def ain112_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@112)

        Address: 11624
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_H, value)

    @property
    @locked
    def ain113_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@113)

        Address: 11626
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_H)  # type: ignore

    @ain113_ef_config_h.setter
    @locked
    def ain113_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@113)

        Address: 11626
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_H, value)

    @property
    @locked
    def ain114_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@114)

        Address: 11628
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_H)  # type: ignore

    @ain114_ef_config_h.setter
    @locked
    def ain114_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@114)

        Address: 11628
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_H, value)

    @property
    @locked
    def ain115_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@115)

        Address: 11630
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_H)  # type: ignore

    @ain115_ef_config_h.setter
    @locked
    def ain115_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@115)

        Address: 11630
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_H, value)

    @property
    @locked
    def ain116_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@116)

        Address: 11632
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_H)  # type: ignore

    @ain116_ef_config_h.setter
    @locked
    def ain116_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@116)

        Address: 11632
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_H, value)

    @property
    @locked
    def ain117_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@117)

        Address: 11634
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_H)  # type: ignore

    @ain117_ef_config_h.setter
    @locked
    def ain117_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@117)

        Address: 11634
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_H, value)

    @property
    @locked
    def ain118_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@118)

        Address: 11636
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_H)  # type: ignore

    @ain118_ef_config_h.setter
    @locked
    def ain118_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@118)

        Address: 11636
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_H, value)

    @property
    @locked
    def ain119_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@119)

        Address: 11638
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_H)  # type: ignore

    @ain119_ef_config_h.setter
    @locked
    def ain119_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@119)

        Address: 11638
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_H, value)

    @property
    @locked
    def ain120_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@120)

        Address: 11640
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_H)  # type: ignore

    @ain120_ef_config_h.setter
    @locked
    def ain120_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@120)

        Address: 11640
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_H, value)

    @property
    @locked
    def ain121_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@121)

        Address: 11642
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_H)  # type: ignore

    @ain121_ef_config_h.setter
    @locked
    def ain121_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@121)

        Address: 11642
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_H, value)

    @property
    @locked
    def ain122_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@122)

        Address: 11644
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_H)  # type: ignore

    @ain122_ef_config_h.setter
    @locked
    def ain122_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@122)

        Address: 11644
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_H, value)

    @property
    @locked
    def ain123_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@123)

        Address: 11646
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_H)  # type: ignore

    @ain123_ef_config_h.setter
    @locked
    def ain123_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@123)

        Address: 11646
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_H, value)

    @property
    @locked
    def ain124_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@124)

        Address: 11648
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_H)  # type: ignore

    @ain124_ef_config_h.setter
    @locked
    def ain124_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@124)

        Address: 11648
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_H, value)

    @property
    @locked
    def ain125_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@125)

        Address: 11650
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_H)  # type: ignore

    @ain125_ef_config_h.setter
    @locked
    def ain125_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@125)

        Address: 11650
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_H, value)

    @property
    @locked
    def ain126_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@126)

        Address: 11652
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_H)  # type: ignore

    @ain126_ef_config_h.setter
    @locked
    def ain126_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@126)

        Address: 11652
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_H, value)

    @property
    @locked
    def ain127_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@127)

        Address: 11654
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_H)  # type: ignore

    @ain127_ef_config_h.setter
    @locked
    def ain127_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@127)

        Address: 11654
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_H, value)

    @property
    @locked
    def ain128_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@128)

        Address: 11656
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_H)  # type: ignore

    @ain128_ef_config_h.setter
    @locked
    def ain128_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@128)

        Address: 11656
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_H, value)

    @property
    @locked
    def ain129_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@129)

        Address: 11658
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_H)  # type: ignore

    @ain129_ef_config_h.setter
    @locked
    def ain129_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@129)

        Address: 11658
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_H, value)

    @property
    @locked
    def ain130_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@130)

        Address: 11660
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_H)  # type: ignore

    @ain130_ef_config_h.setter
    @locked
    def ain130_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@130)

        Address: 11660
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_H, value)

    @property
    @locked
    def ain131_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@131)

        Address: 11662
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_H)  # type: ignore

    @ain131_ef_config_h.setter
    @locked
    def ain131_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@131)

        Address: 11662
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_H, value)

    @property
    @locked
    def ain132_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@132)

        Address: 11664
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_H)  # type: ignore

    @ain132_ef_config_h.setter
    @locked
    def ain132_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@132)

        Address: 11664
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_H, value)

    @property
    @locked
    def ain133_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@133)

        Address: 11666
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_H)  # type: ignore

    @ain133_ef_config_h.setter
    @locked
    def ain133_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@133)

        Address: 11666
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_H, value)

    @property
    @locked
    def ain134_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@134)

        Address: 11668
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_H)  # type: ignore

    @ain134_ef_config_h.setter
    @locked
    def ain134_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@134)

        Address: 11668
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_H, value)

    @property
    @locked
    def ain135_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@135)

        Address: 11670
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_H)  # type: ignore

    @ain135_ef_config_h.setter
    @locked
    def ain135_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@135)

        Address: 11670
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_H, value)

    @property
    @locked
    def ain136_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@136)

        Address: 11672
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_H)  # type: ignore

    @ain136_ef_config_h.setter
    @locked
    def ain136_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@136)

        Address: 11672
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_H, value)

    @property
    @locked
    def ain137_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@137)

        Address: 11674
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_H)  # type: ignore

    @ain137_ef_config_h.setter
    @locked
    def ain137_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@137)

        Address: 11674
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_H, value)

    @property
    @locked
    def ain138_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@138)

        Address: 11676
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_H)  # type: ignore

    @ain138_ef_config_h.setter
    @locked
    def ain138_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@138)

        Address: 11676
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_H, value)

    @property
    @locked
    def ain139_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@139)

        Address: 11678
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_H)  # type: ignore

    @ain139_ef_config_h.setter
    @locked
    def ain139_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@139)

        Address: 11678
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_H, value)

    @property
    @locked
    def ain140_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@140)

        Address: 11680
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_H)  # type: ignore

    @ain140_ef_config_h.setter
    @locked
    def ain140_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@140)

        Address: 11680
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_H, value)

    @property
    @locked
    def ain141_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@141)

        Address: 11682
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_H)  # type: ignore

    @ain141_ef_config_h.setter
    @locked
    def ain141_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@141)

        Address: 11682
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_H, value)

    @property
    @locked
    def ain142_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@142)

        Address: 11684
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_H)  # type: ignore

    @ain142_ef_config_h.setter
    @locked
    def ain142_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@142)

        Address: 11684
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_H, value)

    @property
    @locked
    def ain143_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@143)

        Address: 11686
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_H)  # type: ignore

    @ain143_ef_config_h.setter
    @locked
    def ain143_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@143)

        Address: 11686
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_H, value)

    @property
    @locked
    def ain144_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@144)

        Address: 11688
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_H)  # type: ignore

    @ain144_ef_config_h.setter
    @locked
    def ain144_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@144)

        Address: 11688
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_H, value)

    @property
    @locked
    def ain145_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@145)

        Address: 11690
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_H)  # type: ignore

    @ain145_ef_config_h.setter
    @locked
    def ain145_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@145)

        Address: 11690
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_H, value)

    @property
    @locked
    def ain146_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@146)

        Address: 11692
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_H)  # type: ignore

    @ain146_ef_config_h.setter
    @locked
    def ain146_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@146)

        Address: 11692
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_H, value)

    @property
    @locked
    def ain147_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@147)

        Address: 11694
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_H)  # type: ignore

    @ain147_ef_config_h.setter
    @locked
    def ain147_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@147)

        Address: 11694
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_H, value)

    @property
    @locked
    def ain148_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@148)

        Address: 11696
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_H)  # type: ignore

    @ain148_ef_config_h.setter
    @locked
    def ain148_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@148)

        Address: 11696
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_H, value)

    @property
    @locked
    def ain149_ef_config_h(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@149)

        Address: 11698
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_H)  # type: ignore

    @ain149_ef_config_h.setter
    @locked
    def ain149_ef_config_h(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_H (AIN#(0:149)_EF_CONFIG_H@149)

        Address: 11698
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_H, value)

    @property
    @locked
    def ain0_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@0)

        Address: 11700
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_I)  # type: ignore

    @ain0_ef_config_i.setter
    @locked
    def ain0_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@0)

        Address: 11700
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_I, value)

    @property
    @locked
    def ain1_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@1)

        Address: 11702
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_I)  # type: ignore

    @ain1_ef_config_i.setter
    @locked
    def ain1_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@1)

        Address: 11702
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_I, value)

    @property
    @locked
    def ain2_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@2)

        Address: 11704
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_I)  # type: ignore

    @ain2_ef_config_i.setter
    @locked
    def ain2_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@2)

        Address: 11704
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_I, value)

    @property
    @locked
    def ain3_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@3)

        Address: 11706
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_I)  # type: ignore

    @ain3_ef_config_i.setter
    @locked
    def ain3_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@3)

        Address: 11706
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_I, value)

    @property
    @locked
    def ain4_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@4)

        Address: 11708
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_I)  # type: ignore

    @ain4_ef_config_i.setter
    @locked
    def ain4_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@4)

        Address: 11708
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_I, value)

    @property
    @locked
    def ain5_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@5)

        Address: 11710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_I)  # type: ignore

    @ain5_ef_config_i.setter
    @locked
    def ain5_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@5)

        Address: 11710
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_I, value)

    @property
    @locked
    def ain6_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@6)

        Address: 11712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_I)  # type: ignore

    @ain6_ef_config_i.setter
    @locked
    def ain6_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@6)

        Address: 11712
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_I, value)

    @property
    @locked
    def ain7_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@7)

        Address: 11714
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_I)  # type: ignore

    @ain7_ef_config_i.setter
    @locked
    def ain7_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@7)

        Address: 11714
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_I, value)

    @property
    @locked
    def ain8_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@8)

        Address: 11716
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_I)  # type: ignore

    @ain8_ef_config_i.setter
    @locked
    def ain8_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@8)

        Address: 11716
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_I, value)

    @property
    @locked
    def ain9_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@9)

        Address: 11718
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_I)  # type: ignore

    @ain9_ef_config_i.setter
    @locked
    def ain9_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@9)

        Address: 11718
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_I, value)

    @property
    @locked
    def ain10_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@10)

        Address: 11720
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_I)  # type: ignore

    @ain10_ef_config_i.setter
    @locked
    def ain10_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@10)

        Address: 11720
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_I, value)

    @property
    @locked
    def ain11_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@11)

        Address: 11722
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_I)  # type: ignore

    @ain11_ef_config_i.setter
    @locked
    def ain11_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@11)

        Address: 11722
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_I, value)

    @property
    @locked
    def ain12_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@12)

        Address: 11724
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_I)  # type: ignore

    @ain12_ef_config_i.setter
    @locked
    def ain12_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@12)

        Address: 11724
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_I, value)

    @property
    @locked
    def ain13_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@13)

        Address: 11726
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_I)  # type: ignore

    @ain13_ef_config_i.setter
    @locked
    def ain13_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@13)

        Address: 11726
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_I, value)

    @property
    @locked
    def ain14_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@14)

        Address: 11728
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_I)  # type: ignore

    @ain14_ef_config_i.setter
    @locked
    def ain14_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@14)

        Address: 11728
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_I, value)

    @property
    @locked
    def ain15_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@15)

        Address: 11730
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_I)  # type: ignore

    @ain15_ef_config_i.setter
    @locked
    def ain15_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@15)

        Address: 11730
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_I, value)

    @property
    @locked
    def ain16_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@16)

        Address: 11732
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_I)  # type: ignore

    @ain16_ef_config_i.setter
    @locked
    def ain16_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@16)

        Address: 11732
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_I, value)

    @property
    @locked
    def ain17_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@17)

        Address: 11734
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_I)  # type: ignore

    @ain17_ef_config_i.setter
    @locked
    def ain17_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@17)

        Address: 11734
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_I, value)

    @property
    @locked
    def ain18_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@18)

        Address: 11736
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_I)  # type: ignore

    @ain18_ef_config_i.setter
    @locked
    def ain18_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@18)

        Address: 11736
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_I, value)

    @property
    @locked
    def ain19_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@19)

        Address: 11738
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_I)  # type: ignore

    @ain19_ef_config_i.setter
    @locked
    def ain19_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@19)

        Address: 11738
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_I, value)

    @property
    @locked
    def ain20_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@20)

        Address: 11740
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_I)  # type: ignore

    @ain20_ef_config_i.setter
    @locked
    def ain20_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@20)

        Address: 11740
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_I, value)

    @property
    @locked
    def ain21_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@21)

        Address: 11742
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_I)  # type: ignore

    @ain21_ef_config_i.setter
    @locked
    def ain21_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@21)

        Address: 11742
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_I, value)

    @property
    @locked
    def ain22_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@22)

        Address: 11744
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_I)  # type: ignore

    @ain22_ef_config_i.setter
    @locked
    def ain22_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@22)

        Address: 11744
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_I, value)

    @property
    @locked
    def ain23_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@23)

        Address: 11746
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_I)  # type: ignore

    @ain23_ef_config_i.setter
    @locked
    def ain23_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@23)

        Address: 11746
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_I, value)

    @property
    @locked
    def ain24_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@24)

        Address: 11748
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_I)  # type: ignore

    @ain24_ef_config_i.setter
    @locked
    def ain24_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@24)

        Address: 11748
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_I, value)

    @property
    @locked
    def ain25_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@25)

        Address: 11750
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_I)  # type: ignore

    @ain25_ef_config_i.setter
    @locked
    def ain25_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@25)

        Address: 11750
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_I, value)

    @property
    @locked
    def ain26_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@26)

        Address: 11752
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_I)  # type: ignore

    @ain26_ef_config_i.setter
    @locked
    def ain26_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@26)

        Address: 11752
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_I, value)

    @property
    @locked
    def ain27_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@27)

        Address: 11754
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_I)  # type: ignore

    @ain27_ef_config_i.setter
    @locked
    def ain27_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@27)

        Address: 11754
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_I, value)

    @property
    @locked
    def ain28_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@28)

        Address: 11756
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_I)  # type: ignore

    @ain28_ef_config_i.setter
    @locked
    def ain28_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@28)

        Address: 11756
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_I, value)

    @property
    @locked
    def ain29_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@29)

        Address: 11758
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_I)  # type: ignore

    @ain29_ef_config_i.setter
    @locked
    def ain29_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@29)

        Address: 11758
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_I, value)

    @property
    @locked
    def ain30_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@30)

        Address: 11760
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_I)  # type: ignore

    @ain30_ef_config_i.setter
    @locked
    def ain30_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@30)

        Address: 11760
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_I, value)

    @property
    @locked
    def ain31_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@31)

        Address: 11762
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_I)  # type: ignore

    @ain31_ef_config_i.setter
    @locked
    def ain31_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@31)

        Address: 11762
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_I, value)

    @property
    @locked
    def ain32_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@32)

        Address: 11764
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_I)  # type: ignore

    @ain32_ef_config_i.setter
    @locked
    def ain32_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@32)

        Address: 11764
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_I, value)

    @property
    @locked
    def ain33_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@33)

        Address: 11766
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_I)  # type: ignore

    @ain33_ef_config_i.setter
    @locked
    def ain33_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@33)

        Address: 11766
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_I, value)

    @property
    @locked
    def ain34_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@34)

        Address: 11768
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_I)  # type: ignore

    @ain34_ef_config_i.setter
    @locked
    def ain34_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@34)

        Address: 11768
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_I, value)

    @property
    @locked
    def ain35_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@35)

        Address: 11770
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_I)  # type: ignore

    @ain35_ef_config_i.setter
    @locked
    def ain35_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@35)

        Address: 11770
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_I, value)

    @property
    @locked
    def ain36_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@36)

        Address: 11772
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_I)  # type: ignore

    @ain36_ef_config_i.setter
    @locked
    def ain36_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@36)

        Address: 11772
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_I, value)

    @property
    @locked
    def ain37_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@37)

        Address: 11774
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_I)  # type: ignore

    @ain37_ef_config_i.setter
    @locked
    def ain37_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@37)

        Address: 11774
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_I, value)

    @property
    @locked
    def ain38_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@38)

        Address: 11776
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_I)  # type: ignore

    @ain38_ef_config_i.setter
    @locked
    def ain38_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@38)

        Address: 11776
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_I, value)

    @property
    @locked
    def ain39_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@39)

        Address: 11778
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_I)  # type: ignore

    @ain39_ef_config_i.setter
    @locked
    def ain39_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@39)

        Address: 11778
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_I, value)

    @property
    @locked
    def ain40_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@40)

        Address: 11780
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_I)  # type: ignore

    @ain40_ef_config_i.setter
    @locked
    def ain40_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@40)

        Address: 11780
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_I, value)

    @property
    @locked
    def ain41_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@41)

        Address: 11782
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_I)  # type: ignore

    @ain41_ef_config_i.setter
    @locked
    def ain41_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@41)

        Address: 11782
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_I, value)

    @property
    @locked
    def ain42_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@42)

        Address: 11784
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_I)  # type: ignore

    @ain42_ef_config_i.setter
    @locked
    def ain42_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@42)

        Address: 11784
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_I, value)

    @property
    @locked
    def ain43_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@43)

        Address: 11786
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_I)  # type: ignore

    @ain43_ef_config_i.setter
    @locked
    def ain43_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@43)

        Address: 11786
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_I, value)

    @property
    @locked
    def ain44_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@44)

        Address: 11788
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_I)  # type: ignore

    @ain44_ef_config_i.setter
    @locked
    def ain44_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@44)

        Address: 11788
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_I, value)

    @property
    @locked
    def ain45_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@45)

        Address: 11790
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_I)  # type: ignore

    @ain45_ef_config_i.setter
    @locked
    def ain45_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@45)

        Address: 11790
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_I, value)

    @property
    @locked
    def ain46_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@46)

        Address: 11792
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_I)  # type: ignore

    @ain46_ef_config_i.setter
    @locked
    def ain46_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@46)

        Address: 11792
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_I, value)

    @property
    @locked
    def ain47_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@47)

        Address: 11794
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_I)  # type: ignore

    @ain47_ef_config_i.setter
    @locked
    def ain47_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@47)

        Address: 11794
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_I, value)

    @property
    @locked
    def ain48_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@48)

        Address: 11796
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_I)  # type: ignore

    @ain48_ef_config_i.setter
    @locked
    def ain48_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@48)

        Address: 11796
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_I, value)

    @property
    @locked
    def ain49_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@49)

        Address: 11798
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_I)  # type: ignore

    @ain49_ef_config_i.setter
    @locked
    def ain49_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@49)

        Address: 11798
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_I, value)

    @property
    @locked
    def ain50_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@50)

        Address: 11800
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_I)  # type: ignore

    @ain50_ef_config_i.setter
    @locked
    def ain50_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@50)

        Address: 11800
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_I, value)

    @property
    @locked
    def ain51_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@51)

        Address: 11802
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_I)  # type: ignore

    @ain51_ef_config_i.setter
    @locked
    def ain51_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@51)

        Address: 11802
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_I, value)

    @property
    @locked
    def ain52_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@52)

        Address: 11804
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_I)  # type: ignore

    @ain52_ef_config_i.setter
    @locked
    def ain52_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@52)

        Address: 11804
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_I, value)

    @property
    @locked
    def ain53_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@53)

        Address: 11806
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_I)  # type: ignore

    @ain53_ef_config_i.setter
    @locked
    def ain53_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@53)

        Address: 11806
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_I, value)

    @property
    @locked
    def ain54_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@54)

        Address: 11808
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_I)  # type: ignore

    @ain54_ef_config_i.setter
    @locked
    def ain54_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@54)

        Address: 11808
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_I, value)

    @property
    @locked
    def ain55_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@55)

        Address: 11810
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_I)  # type: ignore

    @ain55_ef_config_i.setter
    @locked
    def ain55_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@55)

        Address: 11810
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_I, value)

    @property
    @locked
    def ain56_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@56)

        Address: 11812
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_I)  # type: ignore

    @ain56_ef_config_i.setter
    @locked
    def ain56_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@56)

        Address: 11812
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_I, value)

    @property
    @locked
    def ain57_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@57)

        Address: 11814
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_I)  # type: ignore

    @ain57_ef_config_i.setter
    @locked
    def ain57_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@57)

        Address: 11814
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_I, value)

    @property
    @locked
    def ain58_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@58)

        Address: 11816
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_I)  # type: ignore

    @ain58_ef_config_i.setter
    @locked
    def ain58_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@58)

        Address: 11816
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_I, value)

    @property
    @locked
    def ain59_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@59)

        Address: 11818
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_I)  # type: ignore

    @ain59_ef_config_i.setter
    @locked
    def ain59_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@59)

        Address: 11818
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_I, value)

    @property
    @locked
    def ain60_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@60)

        Address: 11820
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_I)  # type: ignore

    @ain60_ef_config_i.setter
    @locked
    def ain60_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@60)

        Address: 11820
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_I, value)

    @property
    @locked
    def ain61_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@61)

        Address: 11822
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_I)  # type: ignore

    @ain61_ef_config_i.setter
    @locked
    def ain61_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@61)

        Address: 11822
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_I, value)

    @property
    @locked
    def ain62_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@62)

        Address: 11824
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_I)  # type: ignore

    @ain62_ef_config_i.setter
    @locked
    def ain62_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@62)

        Address: 11824
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_I, value)

    @property
    @locked
    def ain63_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@63)

        Address: 11826
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_I)  # type: ignore

    @ain63_ef_config_i.setter
    @locked
    def ain63_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@63)

        Address: 11826
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_I, value)

    @property
    @locked
    def ain64_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@64)

        Address: 11828
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_I)  # type: ignore

    @ain64_ef_config_i.setter
    @locked
    def ain64_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@64)

        Address: 11828
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_I, value)

    @property
    @locked
    def ain65_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@65)

        Address: 11830
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_I)  # type: ignore

    @ain65_ef_config_i.setter
    @locked
    def ain65_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@65)

        Address: 11830
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_I, value)

    @property
    @locked
    def ain66_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@66)

        Address: 11832
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_I)  # type: ignore

    @ain66_ef_config_i.setter
    @locked
    def ain66_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@66)

        Address: 11832
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_I, value)

    @property
    @locked
    def ain67_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@67)

        Address: 11834
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_I)  # type: ignore

    @ain67_ef_config_i.setter
    @locked
    def ain67_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@67)

        Address: 11834
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_I, value)

    @property
    @locked
    def ain68_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@68)

        Address: 11836
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_I)  # type: ignore

    @ain68_ef_config_i.setter
    @locked
    def ain68_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@68)

        Address: 11836
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_I, value)

    @property
    @locked
    def ain69_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@69)

        Address: 11838
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_I)  # type: ignore

    @ain69_ef_config_i.setter
    @locked
    def ain69_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@69)

        Address: 11838
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_I, value)

    @property
    @locked
    def ain70_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@70)

        Address: 11840
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_I)  # type: ignore

    @ain70_ef_config_i.setter
    @locked
    def ain70_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@70)

        Address: 11840
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_I, value)

    @property
    @locked
    def ain71_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@71)

        Address: 11842
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_I)  # type: ignore

    @ain71_ef_config_i.setter
    @locked
    def ain71_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@71)

        Address: 11842
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_I, value)

    @property
    @locked
    def ain72_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@72)

        Address: 11844
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_I)  # type: ignore

    @ain72_ef_config_i.setter
    @locked
    def ain72_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@72)

        Address: 11844
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_I, value)

    @property
    @locked
    def ain73_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@73)

        Address: 11846
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_I)  # type: ignore

    @ain73_ef_config_i.setter
    @locked
    def ain73_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@73)

        Address: 11846
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_I, value)

    @property
    @locked
    def ain74_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@74)

        Address: 11848
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_I)  # type: ignore

    @ain74_ef_config_i.setter
    @locked
    def ain74_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@74)

        Address: 11848
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_I, value)

    @property
    @locked
    def ain75_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@75)

        Address: 11850
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_I)  # type: ignore

    @ain75_ef_config_i.setter
    @locked
    def ain75_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@75)

        Address: 11850
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_I, value)

    @property
    @locked
    def ain76_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@76)

        Address: 11852
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_I)  # type: ignore

    @ain76_ef_config_i.setter
    @locked
    def ain76_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@76)

        Address: 11852
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_I, value)

    @property
    @locked
    def ain77_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@77)

        Address: 11854
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_I)  # type: ignore

    @ain77_ef_config_i.setter
    @locked
    def ain77_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@77)

        Address: 11854
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_I, value)

    @property
    @locked
    def ain78_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@78)

        Address: 11856
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_I)  # type: ignore

    @ain78_ef_config_i.setter
    @locked
    def ain78_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@78)

        Address: 11856
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_I, value)

    @property
    @locked
    def ain79_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@79)

        Address: 11858
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_I)  # type: ignore

    @ain79_ef_config_i.setter
    @locked
    def ain79_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@79)

        Address: 11858
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_I, value)

    @property
    @locked
    def ain80_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@80)

        Address: 11860
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_I)  # type: ignore

    @ain80_ef_config_i.setter
    @locked
    def ain80_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@80)

        Address: 11860
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_I, value)

    @property
    @locked
    def ain81_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@81)

        Address: 11862
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_I)  # type: ignore

    @ain81_ef_config_i.setter
    @locked
    def ain81_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@81)

        Address: 11862
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_I, value)

    @property
    @locked
    def ain82_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@82)

        Address: 11864
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_I)  # type: ignore

    @ain82_ef_config_i.setter
    @locked
    def ain82_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@82)

        Address: 11864
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_I, value)

    @property
    @locked
    def ain83_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@83)

        Address: 11866
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_I)  # type: ignore

    @ain83_ef_config_i.setter
    @locked
    def ain83_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@83)

        Address: 11866
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_I, value)

    @property
    @locked
    def ain84_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@84)

        Address: 11868
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_I)  # type: ignore

    @ain84_ef_config_i.setter
    @locked
    def ain84_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@84)

        Address: 11868
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_I, value)

    @property
    @locked
    def ain85_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@85)

        Address: 11870
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_I)  # type: ignore

    @ain85_ef_config_i.setter
    @locked
    def ain85_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@85)

        Address: 11870
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_I, value)

    @property
    @locked
    def ain86_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@86)

        Address: 11872
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_I)  # type: ignore

    @ain86_ef_config_i.setter
    @locked
    def ain86_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@86)

        Address: 11872
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_I, value)

    @property
    @locked
    def ain87_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@87)

        Address: 11874
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_I)  # type: ignore

    @ain87_ef_config_i.setter
    @locked
    def ain87_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@87)

        Address: 11874
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_I, value)

    @property
    @locked
    def ain88_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@88)

        Address: 11876
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_I)  # type: ignore

    @ain88_ef_config_i.setter
    @locked
    def ain88_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@88)

        Address: 11876
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_I, value)

    @property
    @locked
    def ain89_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@89)

        Address: 11878
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_I)  # type: ignore

    @ain89_ef_config_i.setter
    @locked
    def ain89_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@89)

        Address: 11878
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_I, value)

    @property
    @locked
    def ain90_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@90)

        Address: 11880
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_I)  # type: ignore

    @ain90_ef_config_i.setter
    @locked
    def ain90_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@90)

        Address: 11880
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_I, value)

    @property
    @locked
    def ain91_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@91)

        Address: 11882
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_I)  # type: ignore

    @ain91_ef_config_i.setter
    @locked
    def ain91_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@91)

        Address: 11882
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_I, value)

    @property
    @locked
    def ain92_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@92)

        Address: 11884
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_I)  # type: ignore

    @ain92_ef_config_i.setter
    @locked
    def ain92_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@92)

        Address: 11884
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_I, value)

    @property
    @locked
    def ain93_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@93)

        Address: 11886
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_I)  # type: ignore

    @ain93_ef_config_i.setter
    @locked
    def ain93_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@93)

        Address: 11886
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_I, value)

    @property
    @locked
    def ain94_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@94)

        Address: 11888
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_I)  # type: ignore

    @ain94_ef_config_i.setter
    @locked
    def ain94_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@94)

        Address: 11888
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_I, value)

    @property
    @locked
    def ain95_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@95)

        Address: 11890
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_I)  # type: ignore

    @ain95_ef_config_i.setter
    @locked
    def ain95_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@95)

        Address: 11890
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_I, value)

    @property
    @locked
    def ain96_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@96)

        Address: 11892
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_I)  # type: ignore

    @ain96_ef_config_i.setter
    @locked
    def ain96_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@96)

        Address: 11892
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_I, value)

    @property
    @locked
    def ain97_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@97)

        Address: 11894
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_I)  # type: ignore

    @ain97_ef_config_i.setter
    @locked
    def ain97_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@97)

        Address: 11894
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_I, value)

    @property
    @locked
    def ain98_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@98)

        Address: 11896
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_I)  # type: ignore

    @ain98_ef_config_i.setter
    @locked
    def ain98_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@98)

        Address: 11896
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_I, value)

    @property
    @locked
    def ain99_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@99)

        Address: 11898
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_I)  # type: ignore

    @ain99_ef_config_i.setter
    @locked
    def ain99_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@99)

        Address: 11898
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_I, value)

    @property
    @locked
    def ain100_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@100)

        Address: 11900
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_I)  # type: ignore

    @ain100_ef_config_i.setter
    @locked
    def ain100_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@100)

        Address: 11900
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_I, value)

    @property
    @locked
    def ain101_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@101)

        Address: 11902
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_I)  # type: ignore

    @ain101_ef_config_i.setter
    @locked
    def ain101_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@101)

        Address: 11902
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_I, value)

    @property
    @locked
    def ain102_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@102)

        Address: 11904
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_I)  # type: ignore

    @ain102_ef_config_i.setter
    @locked
    def ain102_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@102)

        Address: 11904
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_I, value)

    @property
    @locked
    def ain103_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@103)

        Address: 11906
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_I)  # type: ignore

    @ain103_ef_config_i.setter
    @locked
    def ain103_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@103)

        Address: 11906
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_I, value)

    @property
    @locked
    def ain104_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@104)

        Address: 11908
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_I)  # type: ignore

    @ain104_ef_config_i.setter
    @locked
    def ain104_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@104)

        Address: 11908
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_I, value)

    @property
    @locked
    def ain105_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@105)

        Address: 11910
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_I)  # type: ignore

    @ain105_ef_config_i.setter
    @locked
    def ain105_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@105)

        Address: 11910
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_I, value)

    @property
    @locked
    def ain106_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@106)

        Address: 11912
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_I)  # type: ignore

    @ain106_ef_config_i.setter
    @locked
    def ain106_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@106)

        Address: 11912
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_I, value)

    @property
    @locked
    def ain107_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@107)

        Address: 11914
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_I)  # type: ignore

    @ain107_ef_config_i.setter
    @locked
    def ain107_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@107)

        Address: 11914
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_I, value)

    @property
    @locked
    def ain108_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@108)

        Address: 11916
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_I)  # type: ignore

    @ain108_ef_config_i.setter
    @locked
    def ain108_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@108)

        Address: 11916
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_I, value)

    @property
    @locked
    def ain109_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@109)

        Address: 11918
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_I)  # type: ignore

    @ain109_ef_config_i.setter
    @locked
    def ain109_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@109)

        Address: 11918
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_I, value)

    @property
    @locked
    def ain110_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@110)

        Address: 11920
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_I)  # type: ignore

    @ain110_ef_config_i.setter
    @locked
    def ain110_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@110)

        Address: 11920
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_I, value)

    @property
    @locked
    def ain111_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@111)

        Address: 11922
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_I)  # type: ignore

    @ain111_ef_config_i.setter
    @locked
    def ain111_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@111)

        Address: 11922
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_I, value)

    @property
    @locked
    def ain112_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@112)

        Address: 11924
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_I)  # type: ignore

    @ain112_ef_config_i.setter
    @locked
    def ain112_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@112)

        Address: 11924
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_I, value)

    @property
    @locked
    def ain113_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@113)

        Address: 11926
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_I)  # type: ignore

    @ain113_ef_config_i.setter
    @locked
    def ain113_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@113)

        Address: 11926
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_I, value)

    @property
    @locked
    def ain114_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@114)

        Address: 11928
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_I)  # type: ignore

    @ain114_ef_config_i.setter
    @locked
    def ain114_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@114)

        Address: 11928
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_I, value)

    @property
    @locked
    def ain115_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@115)

        Address: 11930
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_I)  # type: ignore

    @ain115_ef_config_i.setter
    @locked
    def ain115_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@115)

        Address: 11930
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_I, value)

    @property
    @locked
    def ain116_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@116)

        Address: 11932
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_I)  # type: ignore

    @ain116_ef_config_i.setter
    @locked
    def ain116_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@116)

        Address: 11932
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_I, value)

    @property
    @locked
    def ain117_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@117)

        Address: 11934
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_I)  # type: ignore

    @ain117_ef_config_i.setter
    @locked
    def ain117_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@117)

        Address: 11934
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_I, value)

    @property
    @locked
    def ain118_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@118)

        Address: 11936
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_I)  # type: ignore

    @ain118_ef_config_i.setter
    @locked
    def ain118_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@118)

        Address: 11936
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_I, value)

    @property
    @locked
    def ain119_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@119)

        Address: 11938
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_I)  # type: ignore

    @ain119_ef_config_i.setter
    @locked
    def ain119_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@119)

        Address: 11938
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_I, value)

    @property
    @locked
    def ain120_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@120)

        Address: 11940
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_I)  # type: ignore

    @ain120_ef_config_i.setter
    @locked
    def ain120_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@120)

        Address: 11940
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_I, value)

    @property
    @locked
    def ain121_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@121)

        Address: 11942
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_I)  # type: ignore

    @ain121_ef_config_i.setter
    @locked
    def ain121_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@121)

        Address: 11942
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_I, value)

    @property
    @locked
    def ain122_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@122)

        Address: 11944
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_I)  # type: ignore

    @ain122_ef_config_i.setter
    @locked
    def ain122_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@122)

        Address: 11944
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_I, value)

    @property
    @locked
    def ain123_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@123)

        Address: 11946
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_I)  # type: ignore

    @ain123_ef_config_i.setter
    @locked
    def ain123_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@123)

        Address: 11946
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_I, value)

    @property
    @locked
    def ain124_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@124)

        Address: 11948
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_I)  # type: ignore

    @ain124_ef_config_i.setter
    @locked
    def ain124_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@124)

        Address: 11948
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_I, value)

    @property
    @locked
    def ain125_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@125)

        Address: 11950
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_I)  # type: ignore

    @ain125_ef_config_i.setter
    @locked
    def ain125_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@125)

        Address: 11950
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_I, value)

    @property
    @locked
    def ain126_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@126)

        Address: 11952
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_I)  # type: ignore

    @ain126_ef_config_i.setter
    @locked
    def ain126_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@126)

        Address: 11952
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_I, value)

    @property
    @locked
    def ain127_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@127)

        Address: 11954
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_I)  # type: ignore

    @ain127_ef_config_i.setter
    @locked
    def ain127_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@127)

        Address: 11954
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_I, value)

    @property
    @locked
    def ain128_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@128)

        Address: 11956
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_I)  # type: ignore

    @ain128_ef_config_i.setter
    @locked
    def ain128_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@128)

        Address: 11956
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_I, value)

    @property
    @locked
    def ain129_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@129)

        Address: 11958
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_I)  # type: ignore

    @ain129_ef_config_i.setter
    @locked
    def ain129_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@129)

        Address: 11958
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_I, value)

    @property
    @locked
    def ain130_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@130)

        Address: 11960
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_I)  # type: ignore

    @ain130_ef_config_i.setter
    @locked
    def ain130_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@130)

        Address: 11960
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_I, value)

    @property
    @locked
    def ain131_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@131)

        Address: 11962
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_I)  # type: ignore

    @ain131_ef_config_i.setter
    @locked
    def ain131_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@131)

        Address: 11962
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_I, value)

    @property
    @locked
    def ain132_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@132)

        Address: 11964
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_I)  # type: ignore

    @ain132_ef_config_i.setter
    @locked
    def ain132_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@132)

        Address: 11964
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_I, value)

    @property
    @locked
    def ain133_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@133)

        Address: 11966
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_I)  # type: ignore

    @ain133_ef_config_i.setter
    @locked
    def ain133_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@133)

        Address: 11966
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_I, value)

    @property
    @locked
    def ain134_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@134)

        Address: 11968
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_I)  # type: ignore

    @ain134_ef_config_i.setter
    @locked
    def ain134_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@134)

        Address: 11968
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_I, value)

    @property
    @locked
    def ain135_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@135)

        Address: 11970
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_I)  # type: ignore

    @ain135_ef_config_i.setter
    @locked
    def ain135_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@135)

        Address: 11970
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_I, value)

    @property
    @locked
    def ain136_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@136)

        Address: 11972
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_I)  # type: ignore

    @ain136_ef_config_i.setter
    @locked
    def ain136_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@136)

        Address: 11972
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_I, value)

    @property
    @locked
    def ain137_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@137)

        Address: 11974
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_I)  # type: ignore

    @ain137_ef_config_i.setter
    @locked
    def ain137_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@137)

        Address: 11974
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_I, value)

    @property
    @locked
    def ain138_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@138)

        Address: 11976
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_I)  # type: ignore

    @ain138_ef_config_i.setter
    @locked
    def ain138_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@138)

        Address: 11976
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_I, value)

    @property
    @locked
    def ain139_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@139)

        Address: 11978
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_I)  # type: ignore

    @ain139_ef_config_i.setter
    @locked
    def ain139_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@139)

        Address: 11978
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_I, value)

    @property
    @locked
    def ain140_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@140)

        Address: 11980
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_I)  # type: ignore

    @ain140_ef_config_i.setter
    @locked
    def ain140_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@140)

        Address: 11980
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_I, value)

    @property
    @locked
    def ain141_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@141)

        Address: 11982
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_I)  # type: ignore

    @ain141_ef_config_i.setter
    @locked
    def ain141_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@141)

        Address: 11982
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_I, value)

    @property
    @locked
    def ain142_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@142)

        Address: 11984
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_I)  # type: ignore

    @ain142_ef_config_i.setter
    @locked
    def ain142_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@142)

        Address: 11984
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_I, value)

    @property
    @locked
    def ain143_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@143)

        Address: 11986
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_I)  # type: ignore

    @ain143_ef_config_i.setter
    @locked
    def ain143_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@143)

        Address: 11986
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_I, value)

    @property
    @locked
    def ain144_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@144)

        Address: 11988
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_I)  # type: ignore

    @ain144_ef_config_i.setter
    @locked
    def ain144_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@144)

        Address: 11988
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_I, value)

    @property
    @locked
    def ain145_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@145)

        Address: 11990
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_I)  # type: ignore

    @ain145_ef_config_i.setter
    @locked
    def ain145_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@145)

        Address: 11990
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_I, value)

    @property
    @locked
    def ain146_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@146)

        Address: 11992
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_I)  # type: ignore

    @ain146_ef_config_i.setter
    @locked
    def ain146_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@146)

        Address: 11992
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_I, value)

    @property
    @locked
    def ain147_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@147)

        Address: 11994
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_I)  # type: ignore

    @ain147_ef_config_i.setter
    @locked
    def ain147_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@147)

        Address: 11994
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_I, value)

    @property
    @locked
    def ain148_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@148)

        Address: 11996
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_I)  # type: ignore

    @ain148_ef_config_i.setter
    @locked
    def ain148_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@148)

        Address: 11996
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_I, value)

    @property
    @locked
    def ain149_ef_config_i(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@149)

        Address: 11998
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_I)  # type: ignore

    @ain149_ef_config_i.setter
    @locked
    def ain149_ef_config_i(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_I (AIN#(0:149)_EF_CONFIG_I@149)

        Address: 11998
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_I, value)

    @property
    @locked
    def ain0_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@0)

        Address: 12000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_J)  # type: ignore

    @ain0_ef_config_j.setter
    @locked
    def ain0_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN0_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@0)

        Address: 12000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_EF_CONFIG_J, value)

    @property
    @locked
    def ain1_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@1)

        Address: 12002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_J)  # type: ignore

    @ain1_ef_config_j.setter
    @locked
    def ain1_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN1_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@1)

        Address: 12002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_EF_CONFIG_J, value)

    @property
    @locked
    def ain2_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@2)

        Address: 12004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_J)  # type: ignore

    @ain2_ef_config_j.setter
    @locked
    def ain2_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN2_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@2)

        Address: 12004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_EF_CONFIG_J, value)

    @property
    @locked
    def ain3_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@3)

        Address: 12006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_J)  # type: ignore

    @ain3_ef_config_j.setter
    @locked
    def ain3_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN3_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@3)

        Address: 12006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_EF_CONFIG_J, value)

    @property
    @locked
    def ain4_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@4)

        Address: 12008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_J)  # type: ignore

    @ain4_ef_config_j.setter
    @locked
    def ain4_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN4_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@4)

        Address: 12008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_EF_CONFIG_J, value)

    @property
    @locked
    def ain5_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@5)

        Address: 12010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_J)  # type: ignore

    @ain5_ef_config_j.setter
    @locked
    def ain5_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN5_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@5)

        Address: 12010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_EF_CONFIG_J, value)

    @property
    @locked
    def ain6_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@6)

        Address: 12012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_J)  # type: ignore

    @ain6_ef_config_j.setter
    @locked
    def ain6_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN6_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@6)

        Address: 12012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_EF_CONFIG_J, value)

    @property
    @locked
    def ain7_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@7)

        Address: 12014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_J)  # type: ignore

    @ain7_ef_config_j.setter
    @locked
    def ain7_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN7_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@7)

        Address: 12014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_EF_CONFIG_J, value)

    @property
    @locked
    def ain8_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@8)

        Address: 12016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_J)  # type: ignore

    @ain8_ef_config_j.setter
    @locked
    def ain8_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN8_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@8)

        Address: 12016
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN8_EF_CONFIG_J, value)

    @property
    @locked
    def ain9_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@9)

        Address: 12018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_J)  # type: ignore

    @ain9_ef_config_j.setter
    @locked
    def ain9_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN9_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@9)

        Address: 12018
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN9_EF_CONFIG_J, value)

    @property
    @locked
    def ain10_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@10)

        Address: 12020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_J)  # type: ignore

    @ain10_ef_config_j.setter
    @locked
    def ain10_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN10_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@10)

        Address: 12020
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN10_EF_CONFIG_J, value)

    @property
    @locked
    def ain11_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@11)

        Address: 12022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_J)  # type: ignore

    @ain11_ef_config_j.setter
    @locked
    def ain11_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN11_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@11)

        Address: 12022
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN11_EF_CONFIG_J, value)

    @property
    @locked
    def ain12_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@12)

        Address: 12024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_J)  # type: ignore

    @ain12_ef_config_j.setter
    @locked
    def ain12_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN12_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@12)

        Address: 12024
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN12_EF_CONFIG_J, value)

    @property
    @locked
    def ain13_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@13)

        Address: 12026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_J)  # type: ignore

    @ain13_ef_config_j.setter
    @locked
    def ain13_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN13_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@13)

        Address: 12026
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN13_EF_CONFIG_J, value)

    @property
    @locked
    def ain14_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@14)

        Address: 12028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_J)  # type: ignore

    @ain14_ef_config_j.setter
    @locked
    def ain14_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN14_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@14)

        Address: 12028
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN14_EF_CONFIG_J, value)

    @property
    @locked
    def ain15_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@15)

        Address: 12030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_J)  # type: ignore

    @ain15_ef_config_j.setter
    @locked
    def ain15_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN15_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@15)

        Address: 12030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN15_EF_CONFIG_J, value)

    @property
    @locked
    def ain16_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@16)

        Address: 12032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_J)  # type: ignore

    @ain16_ef_config_j.setter
    @locked
    def ain16_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN16_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@16)

        Address: 12032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN16_EF_CONFIG_J, value)

    @property
    @locked
    def ain17_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@17)

        Address: 12034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_J)  # type: ignore

    @ain17_ef_config_j.setter
    @locked
    def ain17_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN17_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@17)

        Address: 12034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN17_EF_CONFIG_J, value)

    @property
    @locked
    def ain18_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@18)

        Address: 12036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_J)  # type: ignore

    @ain18_ef_config_j.setter
    @locked
    def ain18_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN18_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@18)

        Address: 12036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN18_EF_CONFIG_J, value)

    @property
    @locked
    def ain19_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@19)

        Address: 12038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_J)  # type: ignore

    @ain19_ef_config_j.setter
    @locked
    def ain19_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN19_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@19)

        Address: 12038
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN19_EF_CONFIG_J, value)

    @property
    @locked
    def ain20_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@20)

        Address: 12040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_J)  # type: ignore

    @ain20_ef_config_j.setter
    @locked
    def ain20_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN20_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@20)

        Address: 12040
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN20_EF_CONFIG_J, value)

    @property
    @locked
    def ain21_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@21)

        Address: 12042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_J)  # type: ignore

    @ain21_ef_config_j.setter
    @locked
    def ain21_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN21_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@21)

        Address: 12042
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN21_EF_CONFIG_J, value)

    @property
    @locked
    def ain22_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@22)

        Address: 12044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_J)  # type: ignore

    @ain22_ef_config_j.setter
    @locked
    def ain22_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN22_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@22)

        Address: 12044
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN22_EF_CONFIG_J, value)

    @property
    @locked
    def ain23_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@23)

        Address: 12046
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_J)  # type: ignore

    @ain23_ef_config_j.setter
    @locked
    def ain23_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN23_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@23)

        Address: 12046
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN23_EF_CONFIG_J, value)

    @property
    @locked
    def ain24_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@24)

        Address: 12048
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_J)  # type: ignore

    @ain24_ef_config_j.setter
    @locked
    def ain24_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN24_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@24)

        Address: 12048
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN24_EF_CONFIG_J, value)

    @property
    @locked
    def ain25_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@25)

        Address: 12050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_J)  # type: ignore

    @ain25_ef_config_j.setter
    @locked
    def ain25_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN25_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@25)

        Address: 12050
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN25_EF_CONFIG_J, value)

    @property
    @locked
    def ain26_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@26)

        Address: 12052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_J)  # type: ignore

    @ain26_ef_config_j.setter
    @locked
    def ain26_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN26_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@26)

        Address: 12052
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN26_EF_CONFIG_J, value)

    @property
    @locked
    def ain27_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@27)

        Address: 12054
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_J)  # type: ignore

    @ain27_ef_config_j.setter
    @locked
    def ain27_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN27_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@27)

        Address: 12054
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN27_EF_CONFIG_J, value)

    @property
    @locked
    def ain28_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@28)

        Address: 12056
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_J)  # type: ignore

    @ain28_ef_config_j.setter
    @locked
    def ain28_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN28_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@28)

        Address: 12056
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN28_EF_CONFIG_J, value)

    @property
    @locked
    def ain29_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@29)

        Address: 12058
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_J)  # type: ignore

    @ain29_ef_config_j.setter
    @locked
    def ain29_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN29_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@29)

        Address: 12058
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN29_EF_CONFIG_J, value)

    @property
    @locked
    def ain30_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@30)

        Address: 12060
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_J)  # type: ignore

    @ain30_ef_config_j.setter
    @locked
    def ain30_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN30_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@30)

        Address: 12060
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN30_EF_CONFIG_J, value)

    @property
    @locked
    def ain31_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@31)

        Address: 12062
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_J)  # type: ignore

    @ain31_ef_config_j.setter
    @locked
    def ain31_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN31_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@31)

        Address: 12062
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN31_EF_CONFIG_J, value)

    @property
    @locked
    def ain32_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@32)

        Address: 12064
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_J)  # type: ignore

    @ain32_ef_config_j.setter
    @locked
    def ain32_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN32_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@32)

        Address: 12064
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN32_EF_CONFIG_J, value)

    @property
    @locked
    def ain33_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@33)

        Address: 12066
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_J)  # type: ignore

    @ain33_ef_config_j.setter
    @locked
    def ain33_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN33_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@33)

        Address: 12066
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN33_EF_CONFIG_J, value)

    @property
    @locked
    def ain34_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@34)

        Address: 12068
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_J)  # type: ignore

    @ain34_ef_config_j.setter
    @locked
    def ain34_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN34_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@34)

        Address: 12068
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN34_EF_CONFIG_J, value)

    @property
    @locked
    def ain35_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@35)

        Address: 12070
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_J)  # type: ignore

    @ain35_ef_config_j.setter
    @locked
    def ain35_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN35_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@35)

        Address: 12070
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN35_EF_CONFIG_J, value)

    @property
    @locked
    def ain36_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@36)

        Address: 12072
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_J)  # type: ignore

    @ain36_ef_config_j.setter
    @locked
    def ain36_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN36_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@36)

        Address: 12072
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN36_EF_CONFIG_J, value)

    @property
    @locked
    def ain37_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@37)

        Address: 12074
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_J)  # type: ignore

    @ain37_ef_config_j.setter
    @locked
    def ain37_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN37_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@37)

        Address: 12074
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN37_EF_CONFIG_J, value)

    @property
    @locked
    def ain38_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@38)

        Address: 12076
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_J)  # type: ignore

    @ain38_ef_config_j.setter
    @locked
    def ain38_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN38_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@38)

        Address: 12076
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN38_EF_CONFIG_J, value)

    @property
    @locked
    def ain39_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@39)

        Address: 12078
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_J)  # type: ignore

    @ain39_ef_config_j.setter
    @locked
    def ain39_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN39_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@39)

        Address: 12078
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN39_EF_CONFIG_J, value)

    @property
    @locked
    def ain40_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@40)

        Address: 12080
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_J)  # type: ignore

    @ain40_ef_config_j.setter
    @locked
    def ain40_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN40_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@40)

        Address: 12080
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN40_EF_CONFIG_J, value)

    @property
    @locked
    def ain41_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@41)

        Address: 12082
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_J)  # type: ignore

    @ain41_ef_config_j.setter
    @locked
    def ain41_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN41_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@41)

        Address: 12082
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN41_EF_CONFIG_J, value)

    @property
    @locked
    def ain42_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@42)

        Address: 12084
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_J)  # type: ignore

    @ain42_ef_config_j.setter
    @locked
    def ain42_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN42_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@42)

        Address: 12084
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN42_EF_CONFIG_J, value)

    @property
    @locked
    def ain43_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@43)

        Address: 12086
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_J)  # type: ignore

    @ain43_ef_config_j.setter
    @locked
    def ain43_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN43_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@43)

        Address: 12086
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN43_EF_CONFIG_J, value)

    @property
    @locked
    def ain44_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@44)

        Address: 12088
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_J)  # type: ignore

    @ain44_ef_config_j.setter
    @locked
    def ain44_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN44_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@44)

        Address: 12088
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN44_EF_CONFIG_J, value)

    @property
    @locked
    def ain45_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@45)

        Address: 12090
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_J)  # type: ignore

    @ain45_ef_config_j.setter
    @locked
    def ain45_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN45_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@45)

        Address: 12090
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN45_EF_CONFIG_J, value)

    @property
    @locked
    def ain46_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@46)

        Address: 12092
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_J)  # type: ignore

    @ain46_ef_config_j.setter
    @locked
    def ain46_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN46_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@46)

        Address: 12092
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN46_EF_CONFIG_J, value)

    @property
    @locked
    def ain47_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@47)

        Address: 12094
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_J)  # type: ignore

    @ain47_ef_config_j.setter
    @locked
    def ain47_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN47_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@47)

        Address: 12094
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN47_EF_CONFIG_J, value)

    @property
    @locked
    def ain48_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@48)

        Address: 12096
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_J)  # type: ignore

    @ain48_ef_config_j.setter
    @locked
    def ain48_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN48_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@48)

        Address: 12096
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN48_EF_CONFIG_J, value)

    @property
    @locked
    def ain49_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@49)

        Address: 12098
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_J)  # type: ignore

    @ain49_ef_config_j.setter
    @locked
    def ain49_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN49_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@49)

        Address: 12098
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN49_EF_CONFIG_J, value)

    @property
    @locked
    def ain50_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@50)

        Address: 12100
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_J)  # type: ignore

    @ain50_ef_config_j.setter
    @locked
    def ain50_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN50_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@50)

        Address: 12100
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN50_EF_CONFIG_J, value)

    @property
    @locked
    def ain51_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@51)

        Address: 12102
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_J)  # type: ignore

    @ain51_ef_config_j.setter
    @locked
    def ain51_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN51_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@51)

        Address: 12102
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN51_EF_CONFIG_J, value)

    @property
    @locked
    def ain52_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@52)

        Address: 12104
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_J)  # type: ignore

    @ain52_ef_config_j.setter
    @locked
    def ain52_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN52_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@52)

        Address: 12104
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN52_EF_CONFIG_J, value)

    @property
    @locked
    def ain53_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@53)

        Address: 12106
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_J)  # type: ignore

    @ain53_ef_config_j.setter
    @locked
    def ain53_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN53_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@53)

        Address: 12106
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN53_EF_CONFIG_J, value)

    @property
    @locked
    def ain54_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@54)

        Address: 12108
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_J)  # type: ignore

    @ain54_ef_config_j.setter
    @locked
    def ain54_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN54_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@54)

        Address: 12108
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN54_EF_CONFIG_J, value)

    @property
    @locked
    def ain55_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@55)

        Address: 12110
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_J)  # type: ignore

    @ain55_ef_config_j.setter
    @locked
    def ain55_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN55_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@55)

        Address: 12110
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN55_EF_CONFIG_J, value)

    @property
    @locked
    def ain56_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@56)

        Address: 12112
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_J)  # type: ignore

    @ain56_ef_config_j.setter
    @locked
    def ain56_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN56_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@56)

        Address: 12112
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN56_EF_CONFIG_J, value)

    @property
    @locked
    def ain57_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@57)

        Address: 12114
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_J)  # type: ignore

    @ain57_ef_config_j.setter
    @locked
    def ain57_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN57_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@57)

        Address: 12114
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN57_EF_CONFIG_J, value)

    @property
    @locked
    def ain58_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@58)

        Address: 12116
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_J)  # type: ignore

    @ain58_ef_config_j.setter
    @locked
    def ain58_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN58_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@58)

        Address: 12116
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN58_EF_CONFIG_J, value)

    @property
    @locked
    def ain59_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@59)

        Address: 12118
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_J)  # type: ignore

    @ain59_ef_config_j.setter
    @locked
    def ain59_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN59_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@59)

        Address: 12118
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN59_EF_CONFIG_J, value)

    @property
    @locked
    def ain60_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@60)

        Address: 12120
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_J)  # type: ignore

    @ain60_ef_config_j.setter
    @locked
    def ain60_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN60_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@60)

        Address: 12120
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN60_EF_CONFIG_J, value)

    @property
    @locked
    def ain61_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@61)

        Address: 12122
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_J)  # type: ignore

    @ain61_ef_config_j.setter
    @locked
    def ain61_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN61_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@61)

        Address: 12122
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN61_EF_CONFIG_J, value)

    @property
    @locked
    def ain62_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@62)

        Address: 12124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_J)  # type: ignore

    @ain62_ef_config_j.setter
    @locked
    def ain62_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN62_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@62)

        Address: 12124
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN62_EF_CONFIG_J, value)

    @property
    @locked
    def ain63_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@63)

        Address: 12126
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_J)  # type: ignore

    @ain63_ef_config_j.setter
    @locked
    def ain63_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN63_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@63)

        Address: 12126
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN63_EF_CONFIG_J, value)

    @property
    @locked
    def ain64_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@64)

        Address: 12128
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_J)  # type: ignore

    @ain64_ef_config_j.setter
    @locked
    def ain64_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN64_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@64)

        Address: 12128
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN64_EF_CONFIG_J, value)

    @property
    @locked
    def ain65_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@65)

        Address: 12130
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_J)  # type: ignore

    @ain65_ef_config_j.setter
    @locked
    def ain65_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN65_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@65)

        Address: 12130
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN65_EF_CONFIG_J, value)

    @property
    @locked
    def ain66_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@66)

        Address: 12132
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_J)  # type: ignore

    @ain66_ef_config_j.setter
    @locked
    def ain66_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN66_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@66)

        Address: 12132
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN66_EF_CONFIG_J, value)

    @property
    @locked
    def ain67_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@67)

        Address: 12134
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_J)  # type: ignore

    @ain67_ef_config_j.setter
    @locked
    def ain67_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN67_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@67)

        Address: 12134
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN67_EF_CONFIG_J, value)

    @property
    @locked
    def ain68_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@68)

        Address: 12136
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_J)  # type: ignore

    @ain68_ef_config_j.setter
    @locked
    def ain68_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN68_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@68)

        Address: 12136
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN68_EF_CONFIG_J, value)

    @property
    @locked
    def ain69_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@69)

        Address: 12138
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_J)  # type: ignore

    @ain69_ef_config_j.setter
    @locked
    def ain69_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN69_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@69)

        Address: 12138
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN69_EF_CONFIG_J, value)

    @property
    @locked
    def ain70_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@70)

        Address: 12140
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_J)  # type: ignore

    @ain70_ef_config_j.setter
    @locked
    def ain70_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN70_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@70)

        Address: 12140
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN70_EF_CONFIG_J, value)

    @property
    @locked
    def ain71_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@71)

        Address: 12142
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_J)  # type: ignore

    @ain71_ef_config_j.setter
    @locked
    def ain71_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN71_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@71)

        Address: 12142
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN71_EF_CONFIG_J, value)

    @property
    @locked
    def ain72_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@72)

        Address: 12144
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_J)  # type: ignore

    @ain72_ef_config_j.setter
    @locked
    def ain72_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN72_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@72)

        Address: 12144
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN72_EF_CONFIG_J, value)

    @property
    @locked
    def ain73_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@73)

        Address: 12146
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_J)  # type: ignore

    @ain73_ef_config_j.setter
    @locked
    def ain73_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN73_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@73)

        Address: 12146
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN73_EF_CONFIG_J, value)

    @property
    @locked
    def ain74_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@74)

        Address: 12148
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_J)  # type: ignore

    @ain74_ef_config_j.setter
    @locked
    def ain74_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN74_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@74)

        Address: 12148
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN74_EF_CONFIG_J, value)

    @property
    @locked
    def ain75_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@75)

        Address: 12150
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_J)  # type: ignore

    @ain75_ef_config_j.setter
    @locked
    def ain75_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN75_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@75)

        Address: 12150
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN75_EF_CONFIG_J, value)

    @property
    @locked
    def ain76_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@76)

        Address: 12152
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_J)  # type: ignore

    @ain76_ef_config_j.setter
    @locked
    def ain76_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN76_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@76)

        Address: 12152
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN76_EF_CONFIG_J, value)

    @property
    @locked
    def ain77_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@77)

        Address: 12154
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_J)  # type: ignore

    @ain77_ef_config_j.setter
    @locked
    def ain77_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN77_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@77)

        Address: 12154
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN77_EF_CONFIG_J, value)

    @property
    @locked
    def ain78_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@78)

        Address: 12156
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_J)  # type: ignore

    @ain78_ef_config_j.setter
    @locked
    def ain78_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN78_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@78)

        Address: 12156
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN78_EF_CONFIG_J, value)

    @property
    @locked
    def ain79_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@79)

        Address: 12158
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_J)  # type: ignore

    @ain79_ef_config_j.setter
    @locked
    def ain79_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN79_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@79)

        Address: 12158
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN79_EF_CONFIG_J, value)

    @property
    @locked
    def ain80_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@80)

        Address: 12160
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_J)  # type: ignore

    @ain80_ef_config_j.setter
    @locked
    def ain80_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN80_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@80)

        Address: 12160
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN80_EF_CONFIG_J, value)

    @property
    @locked
    def ain81_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@81)

        Address: 12162
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_J)  # type: ignore

    @ain81_ef_config_j.setter
    @locked
    def ain81_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN81_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@81)

        Address: 12162
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN81_EF_CONFIG_J, value)

    @property
    @locked
    def ain82_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@82)

        Address: 12164
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_J)  # type: ignore

    @ain82_ef_config_j.setter
    @locked
    def ain82_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN82_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@82)

        Address: 12164
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN82_EF_CONFIG_J, value)

    @property
    @locked
    def ain83_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@83)

        Address: 12166
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_J)  # type: ignore

    @ain83_ef_config_j.setter
    @locked
    def ain83_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN83_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@83)

        Address: 12166
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN83_EF_CONFIG_J, value)

    @property
    @locked
    def ain84_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@84)

        Address: 12168
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_J)  # type: ignore

    @ain84_ef_config_j.setter
    @locked
    def ain84_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN84_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@84)

        Address: 12168
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN84_EF_CONFIG_J, value)

    @property
    @locked
    def ain85_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@85)

        Address: 12170
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_J)  # type: ignore

    @ain85_ef_config_j.setter
    @locked
    def ain85_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN85_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@85)

        Address: 12170
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN85_EF_CONFIG_J, value)

    @property
    @locked
    def ain86_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@86)

        Address: 12172
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_J)  # type: ignore

    @ain86_ef_config_j.setter
    @locked
    def ain86_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN86_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@86)

        Address: 12172
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN86_EF_CONFIG_J, value)

    @property
    @locked
    def ain87_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@87)

        Address: 12174
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_J)  # type: ignore

    @ain87_ef_config_j.setter
    @locked
    def ain87_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN87_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@87)

        Address: 12174
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN87_EF_CONFIG_J, value)

    @property
    @locked
    def ain88_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@88)

        Address: 12176
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_J)  # type: ignore

    @ain88_ef_config_j.setter
    @locked
    def ain88_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN88_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@88)

        Address: 12176
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN88_EF_CONFIG_J, value)

    @property
    @locked
    def ain89_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@89)

        Address: 12178
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_J)  # type: ignore

    @ain89_ef_config_j.setter
    @locked
    def ain89_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN89_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@89)

        Address: 12178
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN89_EF_CONFIG_J, value)

    @property
    @locked
    def ain90_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@90)

        Address: 12180
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_J)  # type: ignore

    @ain90_ef_config_j.setter
    @locked
    def ain90_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN90_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@90)

        Address: 12180
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN90_EF_CONFIG_J, value)

    @property
    @locked
    def ain91_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@91)

        Address: 12182
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_J)  # type: ignore

    @ain91_ef_config_j.setter
    @locked
    def ain91_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN91_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@91)

        Address: 12182
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN91_EF_CONFIG_J, value)

    @property
    @locked
    def ain92_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@92)

        Address: 12184
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_J)  # type: ignore

    @ain92_ef_config_j.setter
    @locked
    def ain92_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN92_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@92)

        Address: 12184
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN92_EF_CONFIG_J, value)

    @property
    @locked
    def ain93_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@93)

        Address: 12186
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_J)  # type: ignore

    @ain93_ef_config_j.setter
    @locked
    def ain93_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN93_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@93)

        Address: 12186
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN93_EF_CONFIG_J, value)

    @property
    @locked
    def ain94_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@94)

        Address: 12188
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_J)  # type: ignore

    @ain94_ef_config_j.setter
    @locked
    def ain94_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN94_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@94)

        Address: 12188
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN94_EF_CONFIG_J, value)

    @property
    @locked
    def ain95_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@95)

        Address: 12190
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_J)  # type: ignore

    @ain95_ef_config_j.setter
    @locked
    def ain95_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN95_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@95)

        Address: 12190
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN95_EF_CONFIG_J, value)

    @property
    @locked
    def ain96_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@96)

        Address: 12192
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_J)  # type: ignore

    @ain96_ef_config_j.setter
    @locked
    def ain96_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN96_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@96)

        Address: 12192
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN96_EF_CONFIG_J, value)

    @property
    @locked
    def ain97_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@97)

        Address: 12194
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_J)  # type: ignore

    @ain97_ef_config_j.setter
    @locked
    def ain97_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN97_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@97)

        Address: 12194
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN97_EF_CONFIG_J, value)

    @property
    @locked
    def ain98_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@98)

        Address: 12196
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_J)  # type: ignore

    @ain98_ef_config_j.setter
    @locked
    def ain98_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN98_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@98)

        Address: 12196
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN98_EF_CONFIG_J, value)

    @property
    @locked
    def ain99_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@99)

        Address: 12198
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_J)  # type: ignore

    @ain99_ef_config_j.setter
    @locked
    def ain99_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN99_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@99)

        Address: 12198
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN99_EF_CONFIG_J, value)

    @property
    @locked
    def ain100_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@100)

        Address: 12200
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_J)  # type: ignore

    @ain100_ef_config_j.setter
    @locked
    def ain100_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN100_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@100)

        Address: 12200
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN100_EF_CONFIG_J, value)

    @property
    @locked
    def ain101_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@101)

        Address: 12202
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_J)  # type: ignore

    @ain101_ef_config_j.setter
    @locked
    def ain101_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN101_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@101)

        Address: 12202
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN101_EF_CONFIG_J, value)

    @property
    @locked
    def ain102_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@102)

        Address: 12204
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_J)  # type: ignore

    @ain102_ef_config_j.setter
    @locked
    def ain102_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN102_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@102)

        Address: 12204
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN102_EF_CONFIG_J, value)

    @property
    @locked
    def ain103_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@103)

        Address: 12206
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_J)  # type: ignore

    @ain103_ef_config_j.setter
    @locked
    def ain103_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN103_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@103)

        Address: 12206
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN103_EF_CONFIG_J, value)

    @property
    @locked
    def ain104_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@104)

        Address: 12208
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_J)  # type: ignore

    @ain104_ef_config_j.setter
    @locked
    def ain104_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN104_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@104)

        Address: 12208
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN104_EF_CONFIG_J, value)

    @property
    @locked
    def ain105_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@105)

        Address: 12210
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_J)  # type: ignore

    @ain105_ef_config_j.setter
    @locked
    def ain105_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN105_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@105)

        Address: 12210
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN105_EF_CONFIG_J, value)

    @property
    @locked
    def ain106_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@106)

        Address: 12212
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_J)  # type: ignore

    @ain106_ef_config_j.setter
    @locked
    def ain106_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN106_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@106)

        Address: 12212
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN106_EF_CONFIG_J, value)

    @property
    @locked
    def ain107_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@107)

        Address: 12214
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_J)  # type: ignore

    @ain107_ef_config_j.setter
    @locked
    def ain107_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN107_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@107)

        Address: 12214
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN107_EF_CONFIG_J, value)

    @property
    @locked
    def ain108_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@108)

        Address: 12216
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_J)  # type: ignore

    @ain108_ef_config_j.setter
    @locked
    def ain108_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN108_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@108)

        Address: 12216
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN108_EF_CONFIG_J, value)

    @property
    @locked
    def ain109_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@109)

        Address: 12218
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_J)  # type: ignore

    @ain109_ef_config_j.setter
    @locked
    def ain109_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN109_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@109)

        Address: 12218
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN109_EF_CONFIG_J, value)

    @property
    @locked
    def ain110_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@110)

        Address: 12220
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_J)  # type: ignore

    @ain110_ef_config_j.setter
    @locked
    def ain110_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN110_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@110)

        Address: 12220
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN110_EF_CONFIG_J, value)

    @property
    @locked
    def ain111_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@111)

        Address: 12222
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_J)  # type: ignore

    @ain111_ef_config_j.setter
    @locked
    def ain111_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN111_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@111)

        Address: 12222
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN111_EF_CONFIG_J, value)

    @property
    @locked
    def ain112_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@112)

        Address: 12224
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_J)  # type: ignore

    @ain112_ef_config_j.setter
    @locked
    def ain112_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN112_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@112)

        Address: 12224
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN112_EF_CONFIG_J, value)

    @property
    @locked
    def ain113_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@113)

        Address: 12226
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_J)  # type: ignore

    @ain113_ef_config_j.setter
    @locked
    def ain113_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN113_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@113)

        Address: 12226
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN113_EF_CONFIG_J, value)

    @property
    @locked
    def ain114_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@114)

        Address: 12228
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_J)  # type: ignore

    @ain114_ef_config_j.setter
    @locked
    def ain114_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN114_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@114)

        Address: 12228
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN114_EF_CONFIG_J, value)

    @property
    @locked
    def ain115_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@115)

        Address: 12230
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_J)  # type: ignore

    @ain115_ef_config_j.setter
    @locked
    def ain115_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN115_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@115)

        Address: 12230
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN115_EF_CONFIG_J, value)

    @property
    @locked
    def ain116_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@116)

        Address: 12232
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_J)  # type: ignore

    @ain116_ef_config_j.setter
    @locked
    def ain116_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN116_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@116)

        Address: 12232
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN116_EF_CONFIG_J, value)

    @property
    @locked
    def ain117_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@117)

        Address: 12234
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_J)  # type: ignore

    @ain117_ef_config_j.setter
    @locked
    def ain117_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN117_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@117)

        Address: 12234
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN117_EF_CONFIG_J, value)

    @property
    @locked
    def ain118_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@118)

        Address: 12236
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_J)  # type: ignore

    @ain118_ef_config_j.setter
    @locked
    def ain118_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN118_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@118)

        Address: 12236
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN118_EF_CONFIG_J, value)

    @property
    @locked
    def ain119_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@119)

        Address: 12238
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_J)  # type: ignore

    @ain119_ef_config_j.setter
    @locked
    def ain119_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN119_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@119)

        Address: 12238
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN119_EF_CONFIG_J, value)

    @property
    @locked
    def ain120_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@120)

        Address: 12240
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_J)  # type: ignore

    @ain120_ef_config_j.setter
    @locked
    def ain120_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN120_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@120)

        Address: 12240
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN120_EF_CONFIG_J, value)

    @property
    @locked
    def ain121_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@121)

        Address: 12242
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_J)  # type: ignore

    @ain121_ef_config_j.setter
    @locked
    def ain121_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN121_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@121)

        Address: 12242
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN121_EF_CONFIG_J, value)

    @property
    @locked
    def ain122_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@122)

        Address: 12244
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_J)  # type: ignore

    @ain122_ef_config_j.setter
    @locked
    def ain122_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN122_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@122)

        Address: 12244
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN122_EF_CONFIG_J, value)

    @property
    @locked
    def ain123_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@123)

        Address: 12246
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_J)  # type: ignore

    @ain123_ef_config_j.setter
    @locked
    def ain123_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN123_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@123)

        Address: 12246
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN123_EF_CONFIG_J, value)

    @property
    @locked
    def ain124_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@124)

        Address: 12248
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_J)  # type: ignore

    @ain124_ef_config_j.setter
    @locked
    def ain124_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN124_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@124)

        Address: 12248
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN124_EF_CONFIG_J, value)

    @property
    @locked
    def ain125_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@125)

        Address: 12250
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_J)  # type: ignore

    @ain125_ef_config_j.setter
    @locked
    def ain125_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN125_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@125)

        Address: 12250
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN125_EF_CONFIG_J, value)

    @property
    @locked
    def ain126_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@126)

        Address: 12252
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_J)  # type: ignore

    @ain126_ef_config_j.setter
    @locked
    def ain126_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN126_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@126)

        Address: 12252
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN126_EF_CONFIG_J, value)

    @property
    @locked
    def ain127_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@127)

        Address: 12254
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_J)  # type: ignore

    @ain127_ef_config_j.setter
    @locked
    def ain127_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN127_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@127)

        Address: 12254
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN127_EF_CONFIG_J, value)

    @property
    @locked
    def ain128_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@128)

        Address: 12256
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_J)  # type: ignore

    @ain128_ef_config_j.setter
    @locked
    def ain128_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN128_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@128)

        Address: 12256
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN128_EF_CONFIG_J, value)

    @property
    @locked
    def ain129_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@129)

        Address: 12258
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_J)  # type: ignore

    @ain129_ef_config_j.setter
    @locked
    def ain129_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN129_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@129)

        Address: 12258
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN129_EF_CONFIG_J, value)

    @property
    @locked
    def ain130_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@130)

        Address: 12260
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_J)  # type: ignore

    @ain130_ef_config_j.setter
    @locked
    def ain130_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN130_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@130)

        Address: 12260
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN130_EF_CONFIG_J, value)

    @property
    @locked
    def ain131_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@131)

        Address: 12262
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_J)  # type: ignore

    @ain131_ef_config_j.setter
    @locked
    def ain131_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN131_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@131)

        Address: 12262
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN131_EF_CONFIG_J, value)

    @property
    @locked
    def ain132_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@132)

        Address: 12264
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_J)  # type: ignore

    @ain132_ef_config_j.setter
    @locked
    def ain132_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN132_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@132)

        Address: 12264
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN132_EF_CONFIG_J, value)

    @property
    @locked
    def ain133_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@133)

        Address: 12266
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_J)  # type: ignore

    @ain133_ef_config_j.setter
    @locked
    def ain133_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN133_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@133)

        Address: 12266
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN133_EF_CONFIG_J, value)

    @property
    @locked
    def ain134_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@134)

        Address: 12268
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_J)  # type: ignore

    @ain134_ef_config_j.setter
    @locked
    def ain134_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN134_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@134)

        Address: 12268
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN134_EF_CONFIG_J, value)

    @property
    @locked
    def ain135_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@135)

        Address: 12270
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_J)  # type: ignore

    @ain135_ef_config_j.setter
    @locked
    def ain135_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN135_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@135)

        Address: 12270
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN135_EF_CONFIG_J, value)

    @property
    @locked
    def ain136_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@136)

        Address: 12272
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_J)  # type: ignore

    @ain136_ef_config_j.setter
    @locked
    def ain136_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN136_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@136)

        Address: 12272
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN136_EF_CONFIG_J, value)

    @property
    @locked
    def ain137_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@137)

        Address: 12274
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_J)  # type: ignore

    @ain137_ef_config_j.setter
    @locked
    def ain137_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN137_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@137)

        Address: 12274
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN137_EF_CONFIG_J, value)

    @property
    @locked
    def ain138_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@138)

        Address: 12276
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_J)  # type: ignore

    @ain138_ef_config_j.setter
    @locked
    def ain138_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN138_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@138)

        Address: 12276
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN138_EF_CONFIG_J, value)

    @property
    @locked
    def ain139_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@139)

        Address: 12278
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_J)  # type: ignore

    @ain139_ef_config_j.setter
    @locked
    def ain139_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN139_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@139)

        Address: 12278
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN139_EF_CONFIG_J, value)

    @property
    @locked
    def ain140_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@140)

        Address: 12280
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_J)  # type: ignore

    @ain140_ef_config_j.setter
    @locked
    def ain140_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN140_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@140)

        Address: 12280
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN140_EF_CONFIG_J, value)

    @property
    @locked
    def ain141_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@141)

        Address: 12282
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_J)  # type: ignore

    @ain141_ef_config_j.setter
    @locked
    def ain141_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN141_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@141)

        Address: 12282
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN141_EF_CONFIG_J, value)

    @property
    @locked
    def ain142_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@142)

        Address: 12284
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_J)  # type: ignore

    @ain142_ef_config_j.setter
    @locked
    def ain142_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN142_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@142)

        Address: 12284
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN142_EF_CONFIG_J, value)

    @property
    @locked
    def ain143_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@143)

        Address: 12286
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_J)  # type: ignore

    @ain143_ef_config_j.setter
    @locked
    def ain143_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN143_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@143)

        Address: 12286
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN143_EF_CONFIG_J, value)

    @property
    @locked
    def ain144_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@144)

        Address: 12288
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_J)  # type: ignore

    @ain144_ef_config_j.setter
    @locked
    def ain144_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN144_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@144)

        Address: 12288
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN144_EF_CONFIG_J, value)

    @property
    @locked
    def ain145_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@145)

        Address: 12290
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_J)  # type: ignore

    @ain145_ef_config_j.setter
    @locked
    def ain145_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN145_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@145)

        Address: 12290
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN145_EF_CONFIG_J, value)

    @property
    @locked
    def ain146_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@146)

        Address: 12292
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_J)  # type: ignore

    @ain146_ef_config_j.setter
    @locked
    def ain146_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN146_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@146)

        Address: 12292
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN146_EF_CONFIG_J, value)

    @property
    @locked
    def ain147_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@147)

        Address: 12294
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_J)  # type: ignore

    @ain147_ef_config_j.setter
    @locked
    def ain147_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN147_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@147)

        Address: 12294
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN147_EF_CONFIG_J, value)

    @property
    @locked
    def ain148_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@148)

        Address: 12296
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_J)  # type: ignore

    @ain148_ef_config_j.setter
    @locked
    def ain148_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN148_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@148)

        Address: 12296
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN148_EF_CONFIG_J, value)

    @property
    @locked
    def ain149_ef_config_j(self) -> float:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@149)

        Address: 12298
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_J)  # type: ignore

    @ain149_ef_config_j.setter
    @locked
    def ain149_ef_config_j(self, value: float) -> None:
        """
        Function dependent on selected feature index.

        Register: AIN149_EF_CONFIG_J (AIN#(0:149)_EF_CONFIG_J@149)

        Address: 12298
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN149_EF_CONFIG_J, value)

    @property
    @locked
    def tdac0(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC0 (TDAC#(0:21)@0)

        Address: 30000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC0)  # type: ignore

    @tdac0.setter
    @locked
    def tdac0(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC0 (TDAC#(0:21)@0)

        Address: 30000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC0, value)

    @property
    @locked
    def tdac1(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC1 (TDAC#(0:21)@1)

        Address: 30002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC1)  # type: ignore

    @tdac1.setter
    @locked
    def tdac1(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC1 (TDAC#(0:21)@1)

        Address: 30002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC1, value)

    @property
    @locked
    def tdac2(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC2 (TDAC#(0:21)@2)

        Address: 30004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC2)  # type: ignore

    @tdac2.setter
    @locked
    def tdac2(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC2 (TDAC#(0:21)@2)

        Address: 30004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC2, value)

    @property
    @locked
    def tdac3(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC3 (TDAC#(0:21)@3)

        Address: 30006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC3)  # type: ignore

    @tdac3.setter
    @locked
    def tdac3(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC3 (TDAC#(0:21)@3)

        Address: 30006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC3, value)

    @property
    @locked
    def tdac4(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC4 (TDAC#(0:21)@4)

        Address: 30008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC4)  # type: ignore

    @tdac4.setter
    @locked
    def tdac4(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC4 (TDAC#(0:21)@4)

        Address: 30008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC4, value)

    @property
    @locked
    def tdac5(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC5 (TDAC#(0:21)@5)

        Address: 30010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC5)  # type: ignore

    @tdac5.setter
    @locked
    def tdac5(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC5 (TDAC#(0:21)@5)

        Address: 30010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC5, value)

    @property
    @locked
    def tdac6(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC6 (TDAC#(0:21)@6)

        Address: 30012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC6)  # type: ignore

    @tdac6.setter
    @locked
    def tdac6(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC6 (TDAC#(0:21)@6)

        Address: 30012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC6, value)

    @property
    @locked
    def tdac7(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC7 (TDAC#(0:21)@7)

        Address: 30014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC7)  # type: ignore

    @tdac7.setter
    @locked
    def tdac7(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC7 (TDAC#(0:21)@7)

        Address: 30014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC7, value)

    @property
    @locked
    def tdac8(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC8 (TDAC#(0:21)@8)

        Address: 30016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC8)  # type: ignore

    @tdac8.setter
    @locked
    def tdac8(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC8 (TDAC#(0:21)@8)

        Address: 30016
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC8, value)

    @property
    @locked
    def tdac9(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC9 (TDAC#(0:21)@9)

        Address: 30018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC9)  # type: ignore

    @tdac9.setter
    @locked
    def tdac9(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC9 (TDAC#(0:21)@9)

        Address: 30018
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC9, value)

    @property
    @locked
    def tdac10(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC10 (TDAC#(0:21)@10)

        Address: 30020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC10)  # type: ignore

    @tdac10.setter
    @locked
    def tdac10(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC10 (TDAC#(0:21)@10)

        Address: 30020
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC10, value)

    @property
    @locked
    def tdac11(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC11 (TDAC#(0:21)@11)

        Address: 30022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC11)  # type: ignore

    @tdac11.setter
    @locked
    def tdac11(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC11 (TDAC#(0:21)@11)

        Address: 30022
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC11, value)

    @property
    @locked
    def tdac12(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC12 (TDAC#(0:21)@12)

        Address: 30024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC12)  # type: ignore

    @tdac12.setter
    @locked
    def tdac12(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC12 (TDAC#(0:21)@12)

        Address: 30024
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC12, value)

    @property
    @locked
    def tdac13(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC13 (TDAC#(0:21)@13)

        Address: 30026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC13)  # type: ignore

    @tdac13.setter
    @locked
    def tdac13(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC13 (TDAC#(0:21)@13)

        Address: 30026
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC13, value)

    @property
    @locked
    def tdac14(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC14 (TDAC#(0:21)@14)

        Address: 30028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC14)  # type: ignore

    @tdac14.setter
    @locked
    def tdac14(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC14 (TDAC#(0:21)@14)

        Address: 30028
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC14, value)

    @property
    @locked
    def tdac15(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC15 (TDAC#(0:21)@15)

        Address: 30030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC15)  # type: ignore

    @tdac15.setter
    @locked
    def tdac15(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC15 (TDAC#(0:21)@15)

        Address: 30030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC15, value)

    @property
    @locked
    def tdac16(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC16 (TDAC#(0:21)@16)

        Address: 30032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC16)  # type: ignore

    @tdac16.setter
    @locked
    def tdac16(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC16 (TDAC#(0:21)@16)

        Address: 30032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC16, value)

    @property
    @locked
    def tdac17(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC17 (TDAC#(0:21)@17)

        Address: 30034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC17)  # type: ignore

    @tdac17.setter
    @locked
    def tdac17(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC17 (TDAC#(0:21)@17)

        Address: 30034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC17, value)

    @property
    @locked
    def tdac18(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC18 (TDAC#(0:21)@18)

        Address: 30036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC18)  # type: ignore

    @tdac18.setter
    @locked
    def tdac18(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC18 (TDAC#(0:21)@18)

        Address: 30036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC18, value)

    @property
    @locked
    def tdac19(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC19 (TDAC#(0:21)@19)

        Address: 30038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC19)  # type: ignore

    @tdac19.setter
    @locked
    def tdac19(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC19 (TDAC#(0:21)@19)

        Address: 30038
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC19, value)

    @property
    @locked
    def tdac20(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC20 (TDAC#(0:21)@20)

        Address: 30040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC20)  # type: ignore

    @tdac20.setter
    @locked
    def tdac20(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC20 (TDAC#(0:21)@20)

        Address: 30040
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC20, value)

    @property
    @locked
    def tdac21(self) -> float:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC21 (TDAC#(0:21)@21)

        Address: 30042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TDAC21)  # type: ignore

    @tdac21.setter
    @locked
    def tdac21(self, value: float) -> None:
        """
        Update a voltage output on a connected LJTick-DAC accessory. Even TDAC# = DACA, Odd TDAC# = DACB. For
        instance, if LJTick-DAC accessory is connected to FIO2/FIO3 block on main device, TDAC2 corresponds with DACA,
        and TDAC3 corresponds with DACB.

        Register: TDAC21 (TDAC#(0:21)@21)

        Address: 30042
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TDAC21, value)

    @property
    @locked
    def tdac_serial_number(self) -> int:
        """
        Returns the serial number of an LJTick-DAC, and forces a re-read of the calibration constants. Which LJTDAC
        is determined by the last write to TDAC# ... whether it was successful or not.

        Register: TDAC_SERIAL_NUMBER

        Address: 55200
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TDAC_SERIAL_NUMBER)  # type: ignore

    @property
    @locked
    def tdac_speed_throttle(self) -> int:
        """
        Sets the I2C clock speed that will be used when communicating with the TDAC. Default value is 65516. See
        I2C_SPEED_THROTTLE for more detail.

        Register: TDAC_SPEED_THROTTLE

        Address: 55202
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TDAC_SPEED_THROTTLE)  # type: ignore

    @tdac_speed_throttle.setter
    @locked
    def tdac_speed_throttle(self, value: int) -> None:
        """
        Sets the I2C clock speed that will be used when communicating with the TDAC. Default value is 65516. See
        I2C_SPEED_THROTTLE for more detail.

        Register: TDAC_SPEED_THROTTLE

        Address: 55202
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.TDAC_SPEED_THROTTLE, value)

    @property
    @locked
    def sbus0_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS0_TEMP (SBUS#(0:22)_TEMP@0)

        Address: 30100
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS0_TEMP)  # type: ignore

    @property
    @locked
    def sbus1_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS1_TEMP (SBUS#(0:22)_TEMP@1)

        Address: 30102
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS1_TEMP)  # type: ignore

    @property
    @locked
    def sbus2_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS2_TEMP (SBUS#(0:22)_TEMP@2)

        Address: 30104
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS2_TEMP)  # type: ignore

    @property
    @locked
    def sbus3_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS3_TEMP (SBUS#(0:22)_TEMP@3)

        Address: 30106
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS3_TEMP)  # type: ignore

    @property
    @locked
    def sbus4_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS4_TEMP (SBUS#(0:22)_TEMP@4)

        Address: 30108
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS4_TEMP)  # type: ignore

    @property
    @locked
    def sbus5_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS5_TEMP (SBUS#(0:22)_TEMP@5)

        Address: 30110
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS5_TEMP)  # type: ignore

    @property
    @locked
    def sbus6_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS6_TEMP (SBUS#(0:22)_TEMP@6)

        Address: 30112
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS6_TEMP)  # type: ignore

    @property
    @locked
    def sbus7_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS7_TEMP (SBUS#(0:22)_TEMP@7)

        Address: 30114
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS7_TEMP)  # type: ignore

    @property
    @locked
    def sbus8_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS8_TEMP (SBUS#(0:22)_TEMP@8)

        Address: 30116
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS8_TEMP)  # type: ignore

    @property
    @locked
    def sbus9_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS9_TEMP (SBUS#(0:22)_TEMP@9)

        Address: 30118
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS9_TEMP)  # type: ignore

    @property
    @locked
    def sbus10_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS10_TEMP (SBUS#(0:22)_TEMP@10)

        Address: 30120
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS10_TEMP)  # type: ignore

    @property
    @locked
    def sbus11_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS11_TEMP (SBUS#(0:22)_TEMP@11)

        Address: 30122
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS11_TEMP)  # type: ignore

    @property
    @locked
    def sbus12_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS12_TEMP (SBUS#(0:22)_TEMP@12)

        Address: 30124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS12_TEMP)  # type: ignore

    @property
    @locked
    def sbus13_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS13_TEMP (SBUS#(0:22)_TEMP@13)

        Address: 30126
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS13_TEMP)  # type: ignore

    @property
    @locked
    def sbus14_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS14_TEMP (SBUS#(0:22)_TEMP@14)

        Address: 30128
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS14_TEMP)  # type: ignore

    @property
    @locked
    def sbus15_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS15_TEMP (SBUS#(0:22)_TEMP@15)

        Address: 30130
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS15_TEMP)  # type: ignore

    @property
    @locked
    def sbus16_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS16_TEMP (SBUS#(0:22)_TEMP@16)

        Address: 30132
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS16_TEMP)  # type: ignore

    @property
    @locked
    def sbus17_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS17_TEMP (SBUS#(0:22)_TEMP@17)

        Address: 30134
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS17_TEMP)  # type: ignore

    @property
    @locked
    def sbus18_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS18_TEMP (SBUS#(0:22)_TEMP@18)

        Address: 30136
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS18_TEMP)  # type: ignore

    @property
    @locked
    def sbus19_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS19_TEMP (SBUS#(0:22)_TEMP@19)

        Address: 30138
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS19_TEMP)  # type: ignore

    @property
    @locked
    def sbus20_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS20_TEMP (SBUS#(0:22)_TEMP@20)

        Address: 30140
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS20_TEMP)  # type: ignore

    @property
    @locked
    def sbus21_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS21_TEMP (SBUS#(0:22)_TEMP@21)

        Address: 30142
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS21_TEMP)  # type: ignore

    @property
    @locked
    def sbus22_temp(self) -> float:
        """
        Reads temperature in Kelvin from an SBUS sensor (EI-1050/SHT1x/SHT7x). SBUS# is the DIO line for the EI-1050
        enable. If SBUS# is the same as the value specified for data or clock line, there will be no control of an
        enable line.

        Register: SBUS22_TEMP (SBUS#(0:22)_TEMP@22)

        Address: 30144
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS22_TEMP)  # type: ignore

    @property
    @locked
    def sbus0_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS0_RH (SBUS#(0:22)_RH@0)

        Address: 30150
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS0_RH)  # type: ignore

    @property
    @locked
    def sbus1_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS1_RH (SBUS#(0:22)_RH@1)

        Address: 30152
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS1_RH)  # type: ignore

    @property
    @locked
    def sbus2_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS2_RH (SBUS#(0:22)_RH@2)

        Address: 30154
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS2_RH)  # type: ignore

    @property
    @locked
    def sbus3_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS3_RH (SBUS#(0:22)_RH@3)

        Address: 30156
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS3_RH)  # type: ignore

    @property
    @locked
    def sbus4_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS4_RH (SBUS#(0:22)_RH@4)

        Address: 30158
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS4_RH)  # type: ignore

    @property
    @locked
    def sbus5_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS5_RH (SBUS#(0:22)_RH@5)

        Address: 30160
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS5_RH)  # type: ignore

    @property
    @locked
    def sbus6_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS6_RH (SBUS#(0:22)_RH@6)

        Address: 30162
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS6_RH)  # type: ignore

    @property
    @locked
    def sbus7_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS7_RH (SBUS#(0:22)_RH@7)

        Address: 30164
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS7_RH)  # type: ignore

    @property
    @locked
    def sbus8_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS8_RH (SBUS#(0:22)_RH@8)

        Address: 30166
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS8_RH)  # type: ignore

    @property
    @locked
    def sbus9_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS9_RH (SBUS#(0:22)_RH@9)

        Address: 30168
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS9_RH)  # type: ignore

    @property
    @locked
    def sbus10_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS10_RH (SBUS#(0:22)_RH@10)

        Address: 30170
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS10_RH)  # type: ignore

    @property
    @locked
    def sbus11_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS11_RH (SBUS#(0:22)_RH@11)

        Address: 30172
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS11_RH)  # type: ignore

    @property
    @locked
    def sbus12_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS12_RH (SBUS#(0:22)_RH@12)

        Address: 30174
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS12_RH)  # type: ignore

    @property
    @locked
    def sbus13_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS13_RH (SBUS#(0:22)_RH@13)

        Address: 30176
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS13_RH)  # type: ignore

    @property
    @locked
    def sbus14_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS14_RH (SBUS#(0:22)_RH@14)

        Address: 30178
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS14_RH)  # type: ignore

    @property
    @locked
    def sbus15_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS15_RH (SBUS#(0:22)_RH@15)

        Address: 30180
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS15_RH)  # type: ignore

    @property
    @locked
    def sbus16_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS16_RH (SBUS#(0:22)_RH@16)

        Address: 30182
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS16_RH)  # type: ignore

    @property
    @locked
    def sbus17_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS17_RH (SBUS#(0:22)_RH@17)

        Address: 30184
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS17_RH)  # type: ignore

    @property
    @locked
    def sbus18_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS18_RH (SBUS#(0:22)_RH@18)

        Address: 30186
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS18_RH)  # type: ignore

    @property
    @locked
    def sbus19_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS19_RH (SBUS#(0:22)_RH@19)

        Address: 30188
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS19_RH)  # type: ignore

    @property
    @locked
    def sbus20_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS20_RH (SBUS#(0:22)_RH@20)

        Address: 30190
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS20_RH)  # type: ignore

    @property
    @locked
    def sbus21_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS21_RH (SBUS#(0:22)_RH@21)

        Address: 30192
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS21_RH)  # type: ignore

    @property
    @locked
    def sbus22_rh(self) -> float:
        """
        Reads humidity in % from an external SBUS sensor (EI-1050/SHT1x/SHT7x). # is the DIO line for the EI-1050
        enable. If # is the same as the value specified for data or clock line, there will be no control of an enable
        line.

        Register: SBUS22_RH (SBUS#(0:22)_RH@22)

        Address: 30194
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.SBUS22_RH)  # type: ignore

    @property
    @locked
    def sbus0_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS0_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@0)

        Address: 30200
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS0_DATA_DIONUM)  # type: ignore

    @sbus0_data_dionum.setter
    @locked
    def sbus0_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS0_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@0)

        Address: 30200
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS0_DATA_DIONUM, value)

    @property
    @locked
    def sbus1_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS1_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@1)

        Address: 30201
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS1_DATA_DIONUM)  # type: ignore

    @sbus1_data_dionum.setter
    @locked
    def sbus1_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS1_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@1)

        Address: 30201
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS1_DATA_DIONUM, value)

    @property
    @locked
    def sbus2_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS2_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@2)

        Address: 30202
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS2_DATA_DIONUM)  # type: ignore

    @sbus2_data_dionum.setter
    @locked
    def sbus2_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS2_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@2)

        Address: 30202
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS2_DATA_DIONUM, value)

    @property
    @locked
    def sbus3_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS3_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@3)

        Address: 30203
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS3_DATA_DIONUM)  # type: ignore

    @sbus3_data_dionum.setter
    @locked
    def sbus3_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS3_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@3)

        Address: 30203
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS3_DATA_DIONUM, value)

    @property
    @locked
    def sbus4_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS4_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@4)

        Address: 30204
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS4_DATA_DIONUM)  # type: ignore

    @sbus4_data_dionum.setter
    @locked
    def sbus4_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS4_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@4)

        Address: 30204
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS4_DATA_DIONUM, value)

    @property
    @locked
    def sbus5_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS5_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@5)

        Address: 30205
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS5_DATA_DIONUM)  # type: ignore

    @sbus5_data_dionum.setter
    @locked
    def sbus5_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS5_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@5)

        Address: 30205
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS5_DATA_DIONUM, value)

    @property
    @locked
    def sbus6_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS6_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@6)

        Address: 30206
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS6_DATA_DIONUM)  # type: ignore

    @sbus6_data_dionum.setter
    @locked
    def sbus6_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS6_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@6)

        Address: 30206
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS6_DATA_DIONUM, value)

    @property
    @locked
    def sbus7_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS7_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@7)

        Address: 30207
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS7_DATA_DIONUM)  # type: ignore

    @sbus7_data_dionum.setter
    @locked
    def sbus7_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS7_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@7)

        Address: 30207
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS7_DATA_DIONUM, value)

    @property
    @locked
    def sbus8_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS8_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@8)

        Address: 30208
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS8_DATA_DIONUM)  # type: ignore

    @sbus8_data_dionum.setter
    @locked
    def sbus8_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS8_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@8)

        Address: 30208
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS8_DATA_DIONUM, value)

    @property
    @locked
    def sbus9_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS9_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@9)

        Address: 30209
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS9_DATA_DIONUM)  # type: ignore

    @sbus9_data_dionum.setter
    @locked
    def sbus9_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS9_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@9)

        Address: 30209
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS9_DATA_DIONUM, value)

    @property
    @locked
    def sbus10_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS10_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@10)

        Address: 30210
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS10_DATA_DIONUM)  # type: ignore

    @sbus10_data_dionum.setter
    @locked
    def sbus10_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS10_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@10)

        Address: 30210
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS10_DATA_DIONUM, value)

    @property
    @locked
    def sbus11_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS11_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@11)

        Address: 30211
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS11_DATA_DIONUM)  # type: ignore

    @sbus11_data_dionum.setter
    @locked
    def sbus11_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS11_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@11)

        Address: 30211
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS11_DATA_DIONUM, value)

    @property
    @locked
    def sbus12_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS12_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@12)

        Address: 30212
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS12_DATA_DIONUM)  # type: ignore

    @sbus12_data_dionum.setter
    @locked
    def sbus12_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS12_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@12)

        Address: 30212
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS12_DATA_DIONUM, value)

    @property
    @locked
    def sbus13_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS13_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@13)

        Address: 30213
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS13_DATA_DIONUM)  # type: ignore

    @sbus13_data_dionum.setter
    @locked
    def sbus13_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS13_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@13)

        Address: 30213
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS13_DATA_DIONUM, value)

    @property
    @locked
    def sbus14_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS14_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@14)

        Address: 30214
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS14_DATA_DIONUM)  # type: ignore

    @sbus14_data_dionum.setter
    @locked
    def sbus14_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS14_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@14)

        Address: 30214
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS14_DATA_DIONUM, value)

    @property
    @locked
    def sbus15_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS15_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@15)

        Address: 30215
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS15_DATA_DIONUM)  # type: ignore

    @sbus15_data_dionum.setter
    @locked
    def sbus15_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS15_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@15)

        Address: 30215
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS15_DATA_DIONUM, value)

    @property
    @locked
    def sbus16_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS16_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@16)

        Address: 30216
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS16_DATA_DIONUM)  # type: ignore

    @sbus16_data_dionum.setter
    @locked
    def sbus16_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS16_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@16)

        Address: 30216
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS16_DATA_DIONUM, value)

    @property
    @locked
    def sbus17_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS17_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@17)

        Address: 30217
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS17_DATA_DIONUM)  # type: ignore

    @sbus17_data_dionum.setter
    @locked
    def sbus17_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS17_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@17)

        Address: 30217
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS17_DATA_DIONUM, value)

    @property
    @locked
    def sbus18_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS18_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@18)

        Address: 30218
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS18_DATA_DIONUM)  # type: ignore

    @sbus18_data_dionum.setter
    @locked
    def sbus18_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS18_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@18)

        Address: 30218
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS18_DATA_DIONUM, value)

    @property
    @locked
    def sbus19_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS19_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@19)

        Address: 30219
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS19_DATA_DIONUM)  # type: ignore

    @sbus19_data_dionum.setter
    @locked
    def sbus19_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS19_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@19)

        Address: 30219
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS19_DATA_DIONUM, value)

    @property
    @locked
    def sbus20_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS20_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@20)

        Address: 30220
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS20_DATA_DIONUM)  # type: ignore

    @sbus20_data_dionum.setter
    @locked
    def sbus20_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS20_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@20)

        Address: 30220
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS20_DATA_DIONUM, value)

    @property
    @locked
    def sbus21_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS21_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@21)

        Address: 30221
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS21_DATA_DIONUM)  # type: ignore

    @sbus21_data_dionum.setter
    @locked
    def sbus21_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS21_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@21)

        Address: 30221
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS21_DATA_DIONUM, value)

    @property
    @locked
    def sbus22_data_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS22_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@22)

        Address: 30222
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS22_DATA_DIONUM)  # type: ignore

    @sbus22_data_dionum.setter
    @locked
    def sbus22_data_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's data line is connected to.

        Register: SBUS22_DATA_DIONUM (SBUS#(0:22)_DATA_DIONUM@22)

        Address: 30222
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS22_DATA_DIONUM, value)

    @property
    @locked
    def sbus0_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS0_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@0)

        Address: 30225
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS0_CLOCK_DIONUM)  # type: ignore

    @sbus0_clock_dionum.setter
    @locked
    def sbus0_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS0_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@0)

        Address: 30225
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS0_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus1_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS1_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@1)

        Address: 30226
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS1_CLOCK_DIONUM)  # type: ignore

    @sbus1_clock_dionum.setter
    @locked
    def sbus1_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS1_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@1)

        Address: 30226
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS1_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus2_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS2_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@2)

        Address: 30227
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS2_CLOCK_DIONUM)  # type: ignore

    @sbus2_clock_dionum.setter
    @locked
    def sbus2_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS2_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@2)

        Address: 30227
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS2_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus3_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS3_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@3)

        Address: 30228
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS3_CLOCK_DIONUM)  # type: ignore

    @sbus3_clock_dionum.setter
    @locked
    def sbus3_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS3_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@3)

        Address: 30228
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS3_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus4_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS4_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@4)

        Address: 30229
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS4_CLOCK_DIONUM)  # type: ignore

    @sbus4_clock_dionum.setter
    @locked
    def sbus4_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS4_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@4)

        Address: 30229
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS4_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus5_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS5_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@5)

        Address: 30230
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS5_CLOCK_DIONUM)  # type: ignore

    @sbus5_clock_dionum.setter
    @locked
    def sbus5_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS5_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@5)

        Address: 30230
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS5_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus6_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS6_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@6)

        Address: 30231
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS6_CLOCK_DIONUM)  # type: ignore

    @sbus6_clock_dionum.setter
    @locked
    def sbus6_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS6_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@6)

        Address: 30231
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS6_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus7_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS7_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@7)

        Address: 30232
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS7_CLOCK_DIONUM)  # type: ignore

    @sbus7_clock_dionum.setter
    @locked
    def sbus7_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS7_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@7)

        Address: 30232
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS7_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus8_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS8_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@8)

        Address: 30233
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS8_CLOCK_DIONUM)  # type: ignore

    @sbus8_clock_dionum.setter
    @locked
    def sbus8_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS8_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@8)

        Address: 30233
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS8_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus9_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS9_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@9)

        Address: 30234
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS9_CLOCK_DIONUM)  # type: ignore

    @sbus9_clock_dionum.setter
    @locked
    def sbus9_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS9_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@9)

        Address: 30234
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS9_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus10_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS10_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@10)

        Address: 30235
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS10_CLOCK_DIONUM)  # type: ignore

    @sbus10_clock_dionum.setter
    @locked
    def sbus10_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS10_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@10)

        Address: 30235
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS10_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus11_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS11_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@11)

        Address: 30236
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS11_CLOCK_DIONUM)  # type: ignore

    @sbus11_clock_dionum.setter
    @locked
    def sbus11_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS11_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@11)

        Address: 30236
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS11_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus12_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS12_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@12)

        Address: 30237
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS12_CLOCK_DIONUM)  # type: ignore

    @sbus12_clock_dionum.setter
    @locked
    def sbus12_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS12_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@12)

        Address: 30237
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS12_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus13_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS13_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@13)

        Address: 30238
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS13_CLOCK_DIONUM)  # type: ignore

    @sbus13_clock_dionum.setter
    @locked
    def sbus13_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS13_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@13)

        Address: 30238
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS13_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus14_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS14_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@14)

        Address: 30239
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS14_CLOCK_DIONUM)  # type: ignore

    @sbus14_clock_dionum.setter
    @locked
    def sbus14_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS14_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@14)

        Address: 30239
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS14_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus15_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS15_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@15)

        Address: 30240
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS15_CLOCK_DIONUM)  # type: ignore

    @sbus15_clock_dionum.setter
    @locked
    def sbus15_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS15_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@15)

        Address: 30240
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS15_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus16_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS16_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@16)

        Address: 30241
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS16_CLOCK_DIONUM)  # type: ignore

    @sbus16_clock_dionum.setter
    @locked
    def sbus16_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS16_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@16)

        Address: 30241
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS16_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus17_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS17_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@17)

        Address: 30242
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS17_CLOCK_DIONUM)  # type: ignore

    @sbus17_clock_dionum.setter
    @locked
    def sbus17_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS17_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@17)

        Address: 30242
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS17_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus18_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS18_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@18)

        Address: 30243
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS18_CLOCK_DIONUM)  # type: ignore

    @sbus18_clock_dionum.setter
    @locked
    def sbus18_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS18_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@18)

        Address: 30243
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS18_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus19_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS19_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@19)

        Address: 30244
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS19_CLOCK_DIONUM)  # type: ignore

    @sbus19_clock_dionum.setter
    @locked
    def sbus19_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS19_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@19)

        Address: 30244
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS19_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus20_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS20_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@20)

        Address: 30245
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS20_CLOCK_DIONUM)  # type: ignore

    @sbus20_clock_dionum.setter
    @locked
    def sbus20_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS20_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@20)

        Address: 30245
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS20_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus21_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS21_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@21)

        Address: 30246
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS21_CLOCK_DIONUM)  # type: ignore

    @sbus21_clock_dionum.setter
    @locked
    def sbus21_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS21_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@21)

        Address: 30246
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS21_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus22_clock_dionum(self) -> int:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS22_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@22)

        Address: 30247
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS22_CLOCK_DIONUM)  # type: ignore

    @sbus22_clock_dionum.setter
    @locked
    def sbus22_clock_dionum(self, value: int) -> None:
        """
        This is the DIO# that the external sensor's clock line is connected to.

        Register: SBUS22_CLOCK_DIONUM (SBUS#(0:22)_CLOCK_DIONUM@22)

        Address: 30247
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS22_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus0_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS0_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@0)

        Address: 30250
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS0_BACKGROUND_ENABLE)  # type: ignore

    @sbus0_background_enable.setter
    @locked
    def sbus0_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS0_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@0)

        Address: 30250
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS0_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus1_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS1_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@1)

        Address: 30251
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS1_BACKGROUND_ENABLE)  # type: ignore

    @sbus1_background_enable.setter
    @locked
    def sbus1_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS1_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@1)

        Address: 30251
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS1_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus2_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS2_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@2)

        Address: 30252
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS2_BACKGROUND_ENABLE)  # type: ignore

    @sbus2_background_enable.setter
    @locked
    def sbus2_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS2_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@2)

        Address: 30252
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS2_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus3_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS3_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@3)

        Address: 30253
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS3_BACKGROUND_ENABLE)  # type: ignore

    @sbus3_background_enable.setter
    @locked
    def sbus3_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS3_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@3)

        Address: 30253
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS3_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus4_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS4_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@4)

        Address: 30254
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS4_BACKGROUND_ENABLE)  # type: ignore

    @sbus4_background_enable.setter
    @locked
    def sbus4_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS4_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@4)

        Address: 30254
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS4_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus5_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS5_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@5)

        Address: 30255
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS5_BACKGROUND_ENABLE)  # type: ignore

    @sbus5_background_enable.setter
    @locked
    def sbus5_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS5_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@5)

        Address: 30255
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS5_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus6_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS6_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@6)

        Address: 30256
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS6_BACKGROUND_ENABLE)  # type: ignore

    @sbus6_background_enable.setter
    @locked
    def sbus6_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS6_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@6)

        Address: 30256
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS6_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus7_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS7_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@7)

        Address: 30257
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS7_BACKGROUND_ENABLE)  # type: ignore

    @sbus7_background_enable.setter
    @locked
    def sbus7_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS7_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@7)

        Address: 30257
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS7_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus8_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS8_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@8)

        Address: 30258
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS8_BACKGROUND_ENABLE)  # type: ignore

    @sbus8_background_enable.setter
    @locked
    def sbus8_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS8_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@8)

        Address: 30258
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS8_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus9_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS9_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@9)

        Address: 30259
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS9_BACKGROUND_ENABLE)  # type: ignore

    @sbus9_background_enable.setter
    @locked
    def sbus9_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS9_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@9)

        Address: 30259
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS9_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus10_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS10_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@10)

        Address: 30260
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS10_BACKGROUND_ENABLE)  # type: ignore

    @sbus10_background_enable.setter
    @locked
    def sbus10_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS10_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@10)

        Address: 30260
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS10_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus11_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS11_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@11)

        Address: 30261
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS11_BACKGROUND_ENABLE)  # type: ignore

    @sbus11_background_enable.setter
    @locked
    def sbus11_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS11_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@11)

        Address: 30261
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS11_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus12_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS12_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@12)

        Address: 30262
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS12_BACKGROUND_ENABLE)  # type: ignore

    @sbus12_background_enable.setter
    @locked
    def sbus12_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS12_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@12)

        Address: 30262
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS12_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus13_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS13_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@13)

        Address: 30263
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS13_BACKGROUND_ENABLE)  # type: ignore

    @sbus13_background_enable.setter
    @locked
    def sbus13_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS13_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@13)

        Address: 30263
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS13_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus14_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS14_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@14)

        Address: 30264
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS14_BACKGROUND_ENABLE)  # type: ignore

    @sbus14_background_enable.setter
    @locked
    def sbus14_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS14_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@14)

        Address: 30264
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS14_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus15_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS15_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@15)

        Address: 30265
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS15_BACKGROUND_ENABLE)  # type: ignore

    @sbus15_background_enable.setter
    @locked
    def sbus15_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS15_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@15)

        Address: 30265
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS15_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus16_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS16_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@16)

        Address: 30266
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS16_BACKGROUND_ENABLE)  # type: ignore

    @sbus16_background_enable.setter
    @locked
    def sbus16_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS16_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@16)

        Address: 30266
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS16_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus17_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS17_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@17)

        Address: 30267
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS17_BACKGROUND_ENABLE)  # type: ignore

    @sbus17_background_enable.setter
    @locked
    def sbus17_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS17_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@17)

        Address: 30267
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS17_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus18_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS18_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@18)

        Address: 30268
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS18_BACKGROUND_ENABLE)  # type: ignore

    @sbus18_background_enable.setter
    @locked
    def sbus18_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS18_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@18)

        Address: 30268
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS18_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus19_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS19_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@19)

        Address: 30269
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS19_BACKGROUND_ENABLE)  # type: ignore

    @sbus19_background_enable.setter
    @locked
    def sbus19_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS19_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@19)

        Address: 30269
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS19_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus20_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS20_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@20)

        Address: 30270
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS20_BACKGROUND_ENABLE)  # type: ignore

    @sbus20_background_enable.setter
    @locked
    def sbus20_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS20_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@20)

        Address: 30270
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS20_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus21_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS21_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@21)

        Address: 30271
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS21_BACKGROUND_ENABLE)  # type: ignore

    @sbus21_background_enable.setter
    @locked
    def sbus21_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS21_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@21)

        Address: 30271
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS21_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus22_background_enable(self) -> int:
        """
        Currently unsupported.

        Register: SBUS22_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@22)

        Address: 30272
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS22_BACKGROUND_ENABLE)  # type: ignore

    @sbus22_background_enable.setter
    @locked
    def sbus22_background_enable(self, value: int) -> None:
        """
        Currently unsupported.

        Register: SBUS22_BACKGROUND_ENABLE (SBUS#(0:22)_BACKGROUND_ENABLE@22)

        Address: 30272
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS22_BACKGROUND_ENABLE, value)

    @property
    @locked
    def sbus_all_data_dionum(self) -> int:
        """
        A write to this global parameter sets all SBUS data line registers to the same value. A read will return the
        correct setting if all channels are set the same, but otherwise will return 0xFF.

        Register: SBUS_ALL_DATA_DIONUM

        Address: 30275
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS_ALL_DATA_DIONUM)  # type: ignore

    @sbus_all_data_dionum.setter
    @locked
    def sbus_all_data_dionum(self, value: int) -> None:
        """
        A write to this global parameter sets all SBUS data line registers to the same value. A read will return the
        correct setting if all channels are set the same, but otherwise will return 0xFF.

        Register: SBUS_ALL_DATA_DIONUM

        Address: 30275
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS_ALL_DATA_DIONUM, value)

    @property
    @locked
    def sbus_all_clock_dionum(self) -> int:
        """
        A write to this global parameter sets all SBUS clock line registers to the same value. A read will return
        the correct setting if all channels are set the same, but otherwise will return 0xFF.

        Register: SBUS_ALL_CLOCK_DIONUM

        Address: 30276
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS_ALL_CLOCK_DIONUM)  # type: ignore

    @sbus_all_clock_dionum.setter
    @locked
    def sbus_all_clock_dionum(self, value: int) -> None:
        """
        A write to this global parameter sets all SBUS clock line registers to the same value. A read will return
        the correct setting if all channels are set the same, but otherwise will return 0xFF.

        Register: SBUS_ALL_CLOCK_DIONUM

        Address: 30276
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS_ALL_CLOCK_DIONUM, value)

    @property
    @locked
    def sbus_all_power_dionum(self) -> int:
        """
        Sets the power line. This DIO is set to output-high upon any read of SBUS#_TEMP or SBUS#_RH. Default is FIO6
        for the T4 and FIO2 for the T7. An FIO line can power up to 4 sensors while an EIO/CIO/MIO line or DAC line can
        power up to 20 sensors. Set to 9999 to disable. To use multiple power lines, use a DAC line for power, or
        otherwise control power yourself, set this to 9999 and then control power using writes to normal registers such
        as FIO5, EIO0, or DAC0.

        Register: SBUS_ALL_POWER_DIONUM

        Address: 30277
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS_ALL_POWER_DIONUM)  # type: ignore

    @sbus_all_power_dionum.setter
    @locked
    def sbus_all_power_dionum(self, value: int) -> None:
        """
        Sets the power line. This DIO is set to output-high upon any read of SBUS#_TEMP or SBUS#_RH. Default is FIO6
        for the T4 and FIO2 for the T7. An FIO line can power up to 4 sensors while an EIO/CIO/MIO line or DAC line can
        power up to 20 sensors. Set to 9999 to disable. To use multiple power lines, use a DAC line for power, or
        otherwise control power yourself, set this to 9999 and then control power using writes to normal registers such
        as FIO5, EIO0, or DAC0.

        Register: SBUS_ALL_POWER_DIONUM

        Address: 30277
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS_ALL_POWER_DIONUM, value)

    @property
    @locked
    def sbus_all_clock_speed(self) -> int:
        """
        Sets the clock speed. The clock is software generated so the resulting frequency is not exact. Valid range
        is 0-65535. Larger values are faster. 0 is the fastest option and is equivalent to 65536. A value of 0 is ~200
        kHz. A value of 65000 is ~9.1 kHz.

        Register: SBUS_ALL_CLOCK_SPEED

        Address: 30278
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.SBUS_ALL_CLOCK_SPEED)  # type: ignore

    @sbus_all_clock_speed.setter
    @locked
    def sbus_all_clock_speed(self, value: int) -> None:
        """
        Sets the clock speed. The clock is software generated so the resulting frequency is not exact. Valid range
        is 0-65535. Larger values are faster. 0 is the fastest option and is equivalent to 65536. A value of 0 is ~200
        kHz. A value of 65000 is ~9.1 kHz.

        Register: SBUS_ALL_CLOCK_SPEED

        Address: 30278
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.SBUS_ALL_CLOCK_SPEED, value)

    @property
    @locked
    def ain0_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN0_RANGE (AIN#(0:7)_RANGE@0)

        Address: 40000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_RANGE)  # type: ignore

    @ain0_range.setter
    @locked
    def ain0_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN0_RANGE (AIN#(0:7)_RANGE@0)

        Address: 40000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_RANGE, value)

    @property
    @locked
    def ain1_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN1_RANGE (AIN#(0:7)_RANGE@1)

        Address: 40002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_RANGE)  # type: ignore

    @ain1_range.setter
    @locked
    def ain1_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN1_RANGE (AIN#(0:7)_RANGE@1)

        Address: 40002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_RANGE, value)

    @property
    @locked
    def ain2_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN2_RANGE (AIN#(0:7)_RANGE@2)

        Address: 40004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_RANGE)  # type: ignore

    @ain2_range.setter
    @locked
    def ain2_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN2_RANGE (AIN#(0:7)_RANGE@2)

        Address: 40004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_RANGE, value)

    @property
    @locked
    def ain3_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN3_RANGE (AIN#(0:7)_RANGE@3)

        Address: 40006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_RANGE)  # type: ignore

    @ain3_range.setter
    @locked
    def ain3_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN3_RANGE (AIN#(0:7)_RANGE@3)

        Address: 40006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_RANGE, value)

    @property
    @locked
    def ain4_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN4_RANGE (AIN#(0:7)_RANGE@4)

        Address: 40008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_RANGE)  # type: ignore

    @ain4_range.setter
    @locked
    def ain4_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN4_RANGE (AIN#(0:7)_RANGE@4)

        Address: 40008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_RANGE, value)

    @property
    @locked
    def ain5_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN5_RANGE (AIN#(0:7)_RANGE@5)

        Address: 40010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_RANGE)  # type: ignore

    @ain5_range.setter
    @locked
    def ain5_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN5_RANGE (AIN#(0:7)_RANGE@5)

        Address: 40010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_RANGE, value)

    @property
    @locked
    def ain6_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN6_RANGE (AIN#(0:7)_RANGE@6)

        Address: 40012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_RANGE)  # type: ignore

    @ain6_range.setter
    @locked
    def ain6_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN6_RANGE (AIN#(0:7)_RANGE@6)

        Address: 40012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_RANGE, value)

    @property
    @locked
    def ain7_range(self) -> float:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN7_RANGE (AIN#(0:7)_RANGE@7)

        Address: 40014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_RANGE)  # type: ignore

    @ain7_range.setter
    @locked
    def ain7_range(self, value: float) -> None:
        """
        The range/span of each analog input. Write the highest expected input voltage.

        Register: AIN7_RANGE (AIN#(0:7)_RANGE@7)

        Address: 40014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_RANGE, value)

    @property
    @locked
    def ain0_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN0_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@0)

        Address: 41000
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN0_NEGATIVE_CH)  # type: ignore

    @ain0_negative_ch.setter
    @locked
    def ain0_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN0_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@0)

        Address: 41000
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN0_NEGATIVE_CH, value)

    @property
    @locked
    def ain1_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN1_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@1)

        Address: 41001
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN1_NEGATIVE_CH)  # type: ignore

    @ain1_negative_ch.setter
    @locked
    def ain1_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN1_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@1)

        Address: 41001
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN1_NEGATIVE_CH, value)

    @property
    @locked
    def ain2_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN2_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@2)

        Address: 41002
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN2_NEGATIVE_CH)  # type: ignore

    @ain2_negative_ch.setter
    @locked
    def ain2_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN2_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@2)

        Address: 41002
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN2_NEGATIVE_CH, value)

    @property
    @locked
    def ain3_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN3_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@3)

        Address: 41003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN3_NEGATIVE_CH)  # type: ignore

    @ain3_negative_ch.setter
    @locked
    def ain3_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN3_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@3)

        Address: 41003
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN3_NEGATIVE_CH, value)

    @property
    @locked
    def ain4_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN4_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@4)

        Address: 41004
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN4_NEGATIVE_CH)  # type: ignore

    @ain4_negative_ch.setter
    @locked
    def ain4_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN4_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@4)

        Address: 41004
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN4_NEGATIVE_CH, value)

    @property
    @locked
    def ain5_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN5_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@5)

        Address: 41005
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN5_NEGATIVE_CH)  # type: ignore

    @ain5_negative_ch.setter
    @locked
    def ain5_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN5_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@5)

        Address: 41005
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN5_NEGATIVE_CH, value)

    @property
    @locked
    def ain6_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN6_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@6)

        Address: 41006
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN6_NEGATIVE_CH)  # type: ignore

    @ain6_negative_ch.setter
    @locked
    def ain6_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN6_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@6)

        Address: 41006
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN6_NEGATIVE_CH, value)

    @property
    @locked
    def ain7_negative_ch(self) -> int:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN7_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@7)

        Address: 41007
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN7_NEGATIVE_CH)  # type: ignore

    @ain7_negative_ch.setter
    @locked
    def ain7_negative_ch(self, value: int) -> None:
        """
        Specifies the negative channel to be used for each positive channel. 199=Default=> Single-Ended.

        Register: AIN7_NEGATIVE_CH (AIN#(0:7)_NEGATIVE_CH@7)

        Address: 41007
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN7_NEGATIVE_CH, value)

    @property
    @locked
    def ain0_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN0_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@0)

        Address: 41500
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN0_RESOLUTION_INDEX)  # type: ignore

    @ain0_resolution_index.setter
    @locked
    def ain0_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN0_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@0)

        Address: 41500
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN0_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain1_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN1_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@1)

        Address: 41501
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN1_RESOLUTION_INDEX)  # type: ignore

    @ain1_resolution_index.setter
    @locked
    def ain1_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN1_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@1)

        Address: 41501
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN1_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain2_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN2_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@2)

        Address: 41502
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN2_RESOLUTION_INDEX)  # type: ignore

    @ain2_resolution_index.setter
    @locked
    def ain2_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN2_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@2)

        Address: 41502
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN2_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain3_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN3_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@3)

        Address: 41503
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN3_RESOLUTION_INDEX)  # type: ignore

    @ain3_resolution_index.setter
    @locked
    def ain3_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN3_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@3)

        Address: 41503
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN3_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain4_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN4_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@4)

        Address: 41504
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN4_RESOLUTION_INDEX)  # type: ignore

    @ain4_resolution_index.setter
    @locked
    def ain4_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN4_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@4)

        Address: 41504
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN4_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain5_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN5_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@5)

        Address: 41505
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN5_RESOLUTION_INDEX)  # type: ignore

    @ain5_resolution_index.setter
    @locked
    def ain5_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN5_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@5)

        Address: 41505
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN5_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain6_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN6_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@6)

        Address: 41506
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN6_RESOLUTION_INDEX)  # type: ignore

    @ain6_resolution_index.setter
    @locked
    def ain6_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN6_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@6)

        Address: 41506
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN6_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain7_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN7_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@7)

        Address: 41507
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN7_RESOLUTION_INDEX)  # type: ignore

    @ain7_resolution_index.setter
    @locked
    def ain7_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times.

        Register: AIN7_RESOLUTION_INDEX (AIN#(0:7)_RESOLUTION_INDEX@7)

        Address: 41507
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN7_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain0_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN0_SETTLING_US (AIN#(0:7)_SETTLING_US@0)

        Address: 42000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN0_SETTLING_US)  # type: ignore

    @ain0_settling_us.setter
    @locked
    def ain0_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN0_SETTLING_US (AIN#(0:7)_SETTLING_US@0)

        Address: 42000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN0_SETTLING_US, value)

    @property
    @locked
    def ain1_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN1_SETTLING_US (AIN#(0:7)_SETTLING_US@1)

        Address: 42002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN1_SETTLING_US)  # type: ignore

    @ain1_settling_us.setter
    @locked
    def ain1_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN1_SETTLING_US (AIN#(0:7)_SETTLING_US@1)

        Address: 42002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN1_SETTLING_US, value)

    @property
    @locked
    def ain2_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN2_SETTLING_US (AIN#(0:7)_SETTLING_US@2)

        Address: 42004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN2_SETTLING_US)  # type: ignore

    @ain2_settling_us.setter
    @locked
    def ain2_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN2_SETTLING_US (AIN#(0:7)_SETTLING_US@2)

        Address: 42004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN2_SETTLING_US, value)

    @property
    @locked
    def ain3_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN3_SETTLING_US (AIN#(0:7)_SETTLING_US@3)

        Address: 42006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN3_SETTLING_US)  # type: ignore

    @ain3_settling_us.setter
    @locked
    def ain3_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN3_SETTLING_US (AIN#(0:7)_SETTLING_US@3)

        Address: 42006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN3_SETTLING_US, value)

    @property
    @locked
    def ain4_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN4_SETTLING_US (AIN#(0:7)_SETTLING_US@4)

        Address: 42008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN4_SETTLING_US)  # type: ignore

    @ain4_settling_us.setter
    @locked
    def ain4_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN4_SETTLING_US (AIN#(0:7)_SETTLING_US@4)

        Address: 42008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN4_SETTLING_US, value)

    @property
    @locked
    def ain5_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN5_SETTLING_US (AIN#(0:7)_SETTLING_US@5)

        Address: 42010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN5_SETTLING_US)  # type: ignore

    @ain5_settling_us.setter
    @locked
    def ain5_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN5_SETTLING_US (AIN#(0:7)_SETTLING_US@5)

        Address: 42010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN5_SETTLING_US, value)

    @property
    @locked
    def ain6_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN6_SETTLING_US (AIN#(0:7)_SETTLING_US@6)

        Address: 42012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN6_SETTLING_US)  # type: ignore

    @ain6_settling_us.setter
    @locked
    def ain6_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN6_SETTLING_US (AIN#(0:7)_SETTLING_US@6)

        Address: 42012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN6_SETTLING_US, value)

    @property
    @locked
    def ain7_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN7_SETTLING_US (AIN#(0:7)_SETTLING_US@7)

        Address: 42014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN7_SETTLING_US)  # type: ignore

    @ain7_settling_us.setter
    @locked
    def ain7_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings.

        Register: AIN7_SETTLING_US (AIN#(0:7)_SETTLING_US@7)

        Address: 42014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN7_SETTLING_US, value)

    @property
    @locked
    def ain_channel_enable(self) -> int:
        """
        T8 Only. This register is a bitmask representing the enable states of the analog inputs.

        Register: AIN_CHANNEL_ENABLE

        Address: 43700
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN_CHANNEL_ENABLE)  # type: ignore

    @ain_channel_enable.setter
    @locked
    def ain_channel_enable(self, value: int) -> None:
        """
        T8 Only. This register is a bitmask representing the enable states of the analog inputs.

        Register: AIN_CHANNEL_ENABLE

        Address: 43700
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN_CHANNEL_ENABLE, value)

    @property
    @locked
    def ain_sampling_rate_hz(self) -> float:
        """
        T8 Only. Writing to this register sets the analog sampling rate. Not all rates are possible. Read
        AIN_SAMPLING_RATE_ACTUAL_HZ to get the actual rate.

        Register: AIN_SAMPLING_RATE_HZ

        Address: 43710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN_SAMPLING_RATE_HZ)  # type: ignore

    @ain_sampling_rate_hz.setter
    @locked
    def ain_sampling_rate_hz(self, value: float) -> None:
        """
        T8 Only. Writing to this register sets the analog sampling rate. Not all rates are possible. Read
        AIN_SAMPLING_RATE_ACTUAL_HZ to get the actual rate.

        Register: AIN_SAMPLING_RATE_HZ

        Address: 43710
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN_SAMPLING_RATE_HZ, value)

    @property
    @locked
    def ain_sampling_rate_actual_hz(self) -> float:
        """
        T8 Only. Returns the AIN system's actual sampling rate.

        Register: AIN_SAMPLING_RATE_ACTUAL_HZ

        Address: 43712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN_SAMPLING_RATE_ACTUAL_HZ)  # type: ignore

    @property
    @locked
    def ain_all_range(self) -> float:
        """
        A write to this global parameter affects all AIN. A read will return the correct setting if all channels are
        set the same, but otherwise will return -9999.

        Register: AIN_ALL_RANGE

        Address: 43900
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN_ALL_RANGE)  # type: ignore

    @ain_all_range.setter
    @locked
    def ain_all_range(self, value: float) -> None:
        """
        A write to this global parameter affects all AIN. A read will return the correct setting if all channels are
        set the same, but otherwise will return -9999.

        Register: AIN_ALL_RANGE

        Address: 43900
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN_ALL_RANGE, value)

    @property
    @locked
    def ain_all_negative_ch(self) -> int:
        """
        A write to this global parameter affects all AIN. Writing 1 will set all AINs to differential. Writing 199
        will set all AINs to single-ended. A read will return 1 if all AINs are set to differential and 199 if all AINs
        are set to single-ended. If AIN configurations are not consistent 0xFFFF will be returned.

        Register: AIN_ALL_NEGATIVE_CH

        Address: 43902
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN_ALL_NEGATIVE_CH)  # type: ignore

    @ain_all_negative_ch.setter
    @locked
    def ain_all_negative_ch(self, value: int) -> None:
        """
        A write to this global parameter affects all AIN. Writing 1 will set all AINs to differential. Writing 199
        will set all AINs to single-ended. A read will return 1 if all AINs are set to differential and 199 if all AINs
        are set to single-ended. If AIN configurations are not consistent 0xFFFF will be returned.

        Register: AIN_ALL_NEGATIVE_CH

        Address: 43902
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN_ALL_NEGATIVE_CH, value)

    @property
    @locked
    def ain_all_resolution_index(self) -> int:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times. A write to this global parameter affects all AIN. A read will return
        the correct setting if all channels are set the same, but otherwise will return 0xFFFF.

        Register: AIN_ALL_RESOLUTION_INDEX

        Address: 43903
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.AIN_ALL_RESOLUTION_INDEX)  # type: ignore

    @ain_all_resolution_index.setter
    @locked
    def ain_all_resolution_index(self, value: int) -> None:
        """
        The resolution index for command-response and AIN-EF readings. A larger resolution index generally results
        in lower noise and longer sample times. A write to this global parameter affects all AIN. A read will return
        the correct setting if all channels are set the same, but otherwise will return 0xFFFF.

        Register: AIN_ALL_RESOLUTION_INDEX

        Address: 43903
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.AIN_ALL_RESOLUTION_INDEX, value)

    @property
    @locked
    def ain_all_settling_us(self) -> float:
        """
        Settling time for command-response and AIN-EF readings. A write to this global parameter affects all AIN. A
        read will return the correct setting if all channels are set the same, but otherwise will return -9999. Max is
        50,000 us.

        Register: AIN_ALL_SETTLING_US

        Address: 43904
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.AIN_ALL_SETTLING_US)  # type: ignore

    @ain_all_settling_us.setter
    @locked
    def ain_all_settling_us(self, value: float) -> None:
        """
        Settling time for command-response and AIN-EF readings. A write to this global parameter affects all AIN. A
        read will return the correct setting if all channels are set the same, but otherwise will return -9999. Max is
        50,000 us.

        Register: AIN_ALL_SETTLING_US

        Address: 43904
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.AIN_ALL_SETTLING_US, value)

    @transactional
    def setup_ain_all(self, range_: float = 0.0, settling_time_us: int = 0, resolution_index: int = 0) -> None:
        """
        Setup all AIN channels.
        """
        self.ain_all_range = range_
        self.ain_all_settling_us = settling_time_us
        self.ain_all_resolution_index = resolution_index

    @property
    @locked
    def ain_all_ef_index(self) -> int:
        """
        Write 0 to deactivate AIN_EF on all AINs. No other values may be written to this register. Reads will return
        the AIN_EF index if all 128 AINs are set to the same value. If values are not the same returns 0xFFFF (65535).

        Register: AIN_ALL_EF_INDEX

        Address: 43906
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN_ALL_EF_INDEX)  # type: ignore

    @ain_all_ef_index.setter
    @locked
    def ain_all_ef_index(self, value: int) -> None:
        """
        Write 0 to deactivate AIN_EF on all AINs. No other values may be written to this register. Reads will return
        the AIN_EF index if all 128 AINs are set to the same value. If values are not the same returns 0xFFFF (65535).

        Register: AIN_ALL_EF_INDEX

        Address: 43906
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.AIN_ALL_EF_INDEX, value)

    @property
    @locked
    def dio_ef_clock0_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK0_ENABLE

        Address: 44900
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK0_ENABLE)  # type: ignore

    @dio_ef_clock0_enable.setter
    @locked
    def dio_ef_clock0_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK0_ENABLE

        Address: 44900
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK0_ENABLE, value)

    @property
    @locked
    def dio_ef_clock0_divisor(self) -> int:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK0_DIVISOR

        Address: 44901
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK0_DIVISOR)  # type: ignore

    @dio_ef_clock0_divisor.setter
    @locked
    def dio_ef_clock0_divisor(self, value: int) -> None:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK0_DIVISOR

        Address: 44901
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK0_DIVISOR, value)

    @property
    @locked
    def dio_ef_clock0_options(self) -> int:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK0_OPTIONS

        Address: 44902
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK0_OPTIONS)  # type: ignore

    @dio_ef_clock0_options.setter
    @locked
    def dio_ef_clock0_options(self, value: int) -> None:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK0_OPTIONS

        Address: 44902
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK0_OPTIONS, value)

    @property
    @locked
    def dio_ef_clock0_roll_value(self) -> int:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK0_ROLL_VALUE

        Address: 44904
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK0_ROLL_VALUE)  # type: ignore

    @dio_ef_clock0_roll_value.setter
    @locked
    def dio_ef_clock0_roll_value(self, value: int) -> None:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK0_ROLL_VALUE

        Address: 44904
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK0_ROLL_VALUE, value)

    @property
    @locked
    def dio_ef_clock1_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK1_ENABLE

        Address: 44910
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK1_ENABLE)  # type: ignore

    @dio_ef_clock1_enable.setter
    @locked
    def dio_ef_clock1_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK1_ENABLE

        Address: 44910
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK1_ENABLE, value)

    @property
    @locked
    def dio_ef_clock1_divisor(self) -> int:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK1_DIVISOR

        Address: 44911
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK1_DIVISOR)  # type: ignore

    @dio_ef_clock1_divisor.setter
    @locked
    def dio_ef_clock1_divisor(self, value: int) -> None:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK1_DIVISOR

        Address: 44911
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK1_DIVISOR, value)

    @property
    @locked
    def dio_ef_clock1_options(self) -> int:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK1_OPTIONS

        Address: 44912
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK1_OPTIONS)  # type: ignore

    @dio_ef_clock1_options.setter
    @locked
    def dio_ef_clock1_options(self, value: int) -> None:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK1_OPTIONS

        Address: 44912
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK1_OPTIONS, value)

    @property
    @locked
    def dio_ef_clock1_roll_value(self) -> int:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK1_ROLL_VALUE

        Address: 44914
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK1_ROLL_VALUE)  # type: ignore

    @dio_ef_clock1_roll_value.setter
    @locked
    def dio_ef_clock1_roll_value(self, value: int) -> None:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK1_ROLL_VALUE

        Address: 44914
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK1_ROLL_VALUE, value)

    @property
    @locked
    def dio_ef_clock2_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK2_ENABLE

        Address: 44920
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK2_ENABLE)  # type: ignore

    @dio_ef_clock2_enable.setter
    @locked
    def dio_ef_clock2_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration.

        Register: DIO_EF_CLOCK2_ENABLE

        Address: 44920
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK2_ENABLE, value)

    @property
    @locked
    def dio_ef_clock2_divisor(self) -> int:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK2_DIVISOR

        Address: 44921
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK2_DIVISOR)  # type: ignore

    @dio_ef_clock2_divisor.setter
    @locked
    def dio_ef_clock2_divisor(self, value: int) -> None:
        """
        Divides the core clock.
        Valid options:
        1,2,4,8,16,32,64,256.

        Register: DIO_EF_CLOCK2_DIVISOR

        Address: 44921
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.DIO_EF_CLOCK2_DIVISOR, value)

    @property
    @locked
    def dio_ef_clock2_options(self) -> int:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK2_OPTIONS

        Address: 44922
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK2_OPTIONS)  # type: ignore

    @dio_ef_clock2_options.setter
    @locked
    def dio_ef_clock2_options(self, value: int) -> None:
        """
        Bitmask:
        bit0:
        1 = use external clock.
        All other bits reserved.

        Register: DIO_EF_CLOCK2_OPTIONS

        Address: 44922
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK2_OPTIONS, value)

    @property
    @locked
    def dio_ef_clock2_roll_value(self) -> int:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK2_ROLL_VALUE

        Address: 44924
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK2_ROLL_VALUE)  # type: ignore

    @dio_ef_clock2_roll_value.setter
    @locked
    def dio_ef_clock2_roll_value(self, value: int) -> None:
        """
        The clock will count to this value and then start over at zero. The clock pulses counted are those after the
        divisor. 0 results in the max roll value possible. This is a 32-bit value (0-4294967295) if using a 32-bit
        clock, and a 16-bit value (0-65535) if using a 16-bit clock.

        Register: DIO_EF_CLOCK2_ROLL_VALUE

        Address: 44924
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK2_ROLL_VALUE, value)

    @property
    @locked
    def dio_ef_clock0_count(self) -> int:
        """
        Current tick count of this clock. Will read between 0 and ROLL_VALUE-1.

        Register: DIO_EF_CLOCK0_COUNT

        Address: 44908
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK0_COUNT)  # type: ignore

    @property
    @locked
    def dio_ef_clock1_count(self) -> int:
        """
        Current tick count of this clock. Will read between 0 and ROLL_VALUE-1.

        Register: DIO_EF_CLOCK1_COUNT

        Address: 44918
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK1_COUNT)  # type: ignore

    @property
    @locked
    def dio_ef_clock2_count(self) -> int:
        """
        Current tick count of this clock. Will read between 0 and ROLL_VALUE-1.

        Register: DIO_EF_CLOCK2_COUNT

        Address: 44928
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO_EF_CLOCK2_COUNT)  # type: ignore

    @property
    @locked
    def dio0_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO0_EF_ENABLE (DIO#(0:22)_EF_ENABLE@0)

        Address: 44000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_ENABLE)  # type: ignore

    @dio0_ef_enable.setter
    @locked
    def dio0_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO0_EF_ENABLE (DIO#(0:22)_EF_ENABLE@0)

        Address: 44000
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_ENABLE, value)

    @property
    @locked
    def dio1_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO1_EF_ENABLE (DIO#(0:22)_EF_ENABLE@1)

        Address: 44002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_ENABLE)  # type: ignore

    @dio1_ef_enable.setter
    @locked
    def dio1_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO1_EF_ENABLE (DIO#(0:22)_EF_ENABLE@1)

        Address: 44002
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_ENABLE, value)

    @property
    @locked
    def dio2_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO2_EF_ENABLE (DIO#(0:22)_EF_ENABLE@2)

        Address: 44004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_ENABLE)  # type: ignore

    @dio2_ef_enable.setter
    @locked
    def dio2_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO2_EF_ENABLE (DIO#(0:22)_EF_ENABLE@2)

        Address: 44004
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_ENABLE, value)

    @property
    @locked
    def dio3_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO3_EF_ENABLE (DIO#(0:22)_EF_ENABLE@3)

        Address: 44006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_ENABLE)  # type: ignore

    @dio3_ef_enable.setter
    @locked
    def dio3_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO3_EF_ENABLE (DIO#(0:22)_EF_ENABLE@3)

        Address: 44006
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_ENABLE, value)

    @property
    @locked
    def dio4_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO4_EF_ENABLE (DIO#(0:22)_EF_ENABLE@4)

        Address: 44008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_ENABLE)  # type: ignore

    @dio4_ef_enable.setter
    @locked
    def dio4_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO4_EF_ENABLE (DIO#(0:22)_EF_ENABLE@4)

        Address: 44008
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_ENABLE, value)

    @property
    @locked
    def dio5_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO5_EF_ENABLE (DIO#(0:22)_EF_ENABLE@5)

        Address: 44010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_ENABLE)  # type: ignore

    @dio5_ef_enable.setter
    @locked
    def dio5_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO5_EF_ENABLE (DIO#(0:22)_EF_ENABLE@5)

        Address: 44010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_ENABLE, value)

    @property
    @locked
    def dio6_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO6_EF_ENABLE (DIO#(0:22)_EF_ENABLE@6)

        Address: 44012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_ENABLE)  # type: ignore

    @dio6_ef_enable.setter
    @locked
    def dio6_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO6_EF_ENABLE (DIO#(0:22)_EF_ENABLE@6)

        Address: 44012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_ENABLE, value)

    @property
    @locked
    def dio7_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO7_EF_ENABLE (DIO#(0:22)_EF_ENABLE@7)

        Address: 44014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_ENABLE)  # type: ignore

    @dio7_ef_enable.setter
    @locked
    def dio7_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO7_EF_ENABLE (DIO#(0:22)_EF_ENABLE@7)

        Address: 44014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_ENABLE, value)

    @property
    @locked
    def dio8_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO8_EF_ENABLE (DIO#(0:22)_EF_ENABLE@8)

        Address: 44016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_ENABLE)  # type: ignore

    @dio8_ef_enable.setter
    @locked
    def dio8_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO8_EF_ENABLE (DIO#(0:22)_EF_ENABLE@8)

        Address: 44016
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_ENABLE, value)

    @property
    @locked
    def dio9_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO9_EF_ENABLE (DIO#(0:22)_EF_ENABLE@9)

        Address: 44018
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_ENABLE)  # type: ignore

    @dio9_ef_enable.setter
    @locked
    def dio9_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO9_EF_ENABLE (DIO#(0:22)_EF_ENABLE@9)

        Address: 44018
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_ENABLE, value)

    @property
    @locked
    def dio10_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO10_EF_ENABLE (DIO#(0:22)_EF_ENABLE@10)

        Address: 44020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_ENABLE)  # type: ignore

    @dio10_ef_enable.setter
    @locked
    def dio10_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO10_EF_ENABLE (DIO#(0:22)_EF_ENABLE@10)

        Address: 44020
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_ENABLE, value)

    @property
    @locked
    def dio11_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO11_EF_ENABLE (DIO#(0:22)_EF_ENABLE@11)

        Address: 44022
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_ENABLE)  # type: ignore

    @dio11_ef_enable.setter
    @locked
    def dio11_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO11_EF_ENABLE (DIO#(0:22)_EF_ENABLE@11)

        Address: 44022
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_ENABLE, value)

    @property
    @locked
    def dio12_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO12_EF_ENABLE (DIO#(0:22)_EF_ENABLE@12)

        Address: 44024
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_ENABLE)  # type: ignore

    @dio12_ef_enable.setter
    @locked
    def dio12_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO12_EF_ENABLE (DIO#(0:22)_EF_ENABLE@12)

        Address: 44024
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_ENABLE, value)

    @property
    @locked
    def dio13_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO13_EF_ENABLE (DIO#(0:22)_EF_ENABLE@13)

        Address: 44026
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_ENABLE)  # type: ignore

    @dio13_ef_enable.setter
    @locked
    def dio13_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO13_EF_ENABLE (DIO#(0:22)_EF_ENABLE@13)

        Address: 44026
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_ENABLE, value)

    @property
    @locked
    def dio14_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO14_EF_ENABLE (DIO#(0:22)_EF_ENABLE@14)

        Address: 44028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_ENABLE)  # type: ignore

    @dio14_ef_enable.setter
    @locked
    def dio14_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO14_EF_ENABLE (DIO#(0:22)_EF_ENABLE@14)

        Address: 44028
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_ENABLE, value)

    @property
    @locked
    def dio15_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO15_EF_ENABLE (DIO#(0:22)_EF_ENABLE@15)

        Address: 44030
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_ENABLE)  # type: ignore

    @dio15_ef_enable.setter
    @locked
    def dio15_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO15_EF_ENABLE (DIO#(0:22)_EF_ENABLE@15)

        Address: 44030
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_ENABLE, value)

    @property
    @locked
    def dio16_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO16_EF_ENABLE (DIO#(0:22)_EF_ENABLE@16)

        Address: 44032
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_ENABLE)  # type: ignore

    @dio16_ef_enable.setter
    @locked
    def dio16_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO16_EF_ENABLE (DIO#(0:22)_EF_ENABLE@16)

        Address: 44032
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_ENABLE, value)

    @property
    @locked
    def dio17_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO17_EF_ENABLE (DIO#(0:22)_EF_ENABLE@17)

        Address: 44034
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_ENABLE)  # type: ignore

    @dio17_ef_enable.setter
    @locked
    def dio17_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO17_EF_ENABLE (DIO#(0:22)_EF_ENABLE@17)

        Address: 44034
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_ENABLE, value)

    @property
    @locked
    def dio18_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO18_EF_ENABLE (DIO#(0:22)_EF_ENABLE@18)

        Address: 44036
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_ENABLE)  # type: ignore

    @dio18_ef_enable.setter
    @locked
    def dio18_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO18_EF_ENABLE (DIO#(0:22)_EF_ENABLE@18)

        Address: 44036
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_ENABLE, value)

    @property
    @locked
    def dio19_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO19_EF_ENABLE (DIO#(0:22)_EF_ENABLE@19)

        Address: 44038
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_ENABLE)  # type: ignore

    @dio19_ef_enable.setter
    @locked
    def dio19_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO19_EF_ENABLE (DIO#(0:22)_EF_ENABLE@19)

        Address: 44038
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_ENABLE, value)

    @property
    @locked
    def dio20_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO20_EF_ENABLE (DIO#(0:22)_EF_ENABLE@20)

        Address: 44040
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_ENABLE)  # type: ignore

    @dio20_ef_enable.setter
    @locked
    def dio20_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO20_EF_ENABLE (DIO#(0:22)_EF_ENABLE@20)

        Address: 44040
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_ENABLE, value)

    @property
    @locked
    def dio21_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO21_EF_ENABLE (DIO#(0:22)_EF_ENABLE@21)

        Address: 44042
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_ENABLE)  # type: ignore

    @dio21_ef_enable.setter
    @locked
    def dio21_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO21_EF_ENABLE (DIO#(0:22)_EF_ENABLE@21)

        Address: 44042
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_ENABLE, value)

    @property
    @locked
    def dio22_ef_enable(self) -> int:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO22_EF_ENABLE (DIO#(0:22)_EF_ENABLE@22)

        Address: 44044
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_ENABLE)  # type: ignore

    @dio22_ef_enable.setter
    @locked
    def dio22_ef_enable(self, value: int) -> None:
        """
        1 = enabled. 0 = disabled. Must be disabled during configuration. Note that DIO-EF reads work when disabled
        and do not return an error.

        Register: DIO22_EF_ENABLE (DIO#(0:22)_EF_ENABLE@22)

        Address: 44044
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_ENABLE, value)

    @property
    @locked
    def dio0_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO0_EF_INDEX (DIO#(0:22)_EF_INDEX@0)

        Address: 44100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_INDEX)  # type: ignore

    @dio0_ef_index.setter
    @locked
    def dio0_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO0_EF_INDEX (DIO#(0:22)_EF_INDEX@0)

        Address: 44100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_INDEX, value)

    @property
    @locked
    def dio1_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO1_EF_INDEX (DIO#(0:22)_EF_INDEX@1)

        Address: 44102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_INDEX)  # type: ignore

    @dio1_ef_index.setter
    @locked
    def dio1_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO1_EF_INDEX (DIO#(0:22)_EF_INDEX@1)

        Address: 44102
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_INDEX, value)

    @property
    @locked
    def dio2_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO2_EF_INDEX (DIO#(0:22)_EF_INDEX@2)

        Address: 44104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_INDEX)  # type: ignore

    @dio2_ef_index.setter
    @locked
    def dio2_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO2_EF_INDEX (DIO#(0:22)_EF_INDEX@2)

        Address: 44104
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_INDEX, value)

    @property
    @locked
    def dio3_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO3_EF_INDEX (DIO#(0:22)_EF_INDEX@3)

        Address: 44106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_INDEX)  # type: ignore

    @dio3_ef_index.setter
    @locked
    def dio3_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO3_EF_INDEX (DIO#(0:22)_EF_INDEX@3)

        Address: 44106
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_INDEX, value)

    @property
    @locked
    def dio4_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO4_EF_INDEX (DIO#(0:22)_EF_INDEX@4)

        Address: 44108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_INDEX)  # type: ignore

    @dio4_ef_index.setter
    @locked
    def dio4_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO4_EF_INDEX (DIO#(0:22)_EF_INDEX@4)

        Address: 44108
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_INDEX, value)

    @property
    @locked
    def dio5_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO5_EF_INDEX (DIO#(0:22)_EF_INDEX@5)

        Address: 44110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_INDEX)  # type: ignore

    @dio5_ef_index.setter
    @locked
    def dio5_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO5_EF_INDEX (DIO#(0:22)_EF_INDEX@5)

        Address: 44110
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_INDEX, value)

    @property
    @locked
    def dio6_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO6_EF_INDEX (DIO#(0:22)_EF_INDEX@6)

        Address: 44112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_INDEX)  # type: ignore

    @dio6_ef_index.setter
    @locked
    def dio6_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO6_EF_INDEX (DIO#(0:22)_EF_INDEX@6)

        Address: 44112
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_INDEX, value)

    @property
    @locked
    def dio7_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO7_EF_INDEX (DIO#(0:22)_EF_INDEX@7)

        Address: 44114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_INDEX)  # type: ignore

    @dio7_ef_index.setter
    @locked
    def dio7_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO7_EF_INDEX (DIO#(0:22)_EF_INDEX@7)

        Address: 44114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_INDEX, value)

    @property
    @locked
    def dio8_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO8_EF_INDEX (DIO#(0:22)_EF_INDEX@8)

        Address: 44116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_INDEX)  # type: ignore

    @dio8_ef_index.setter
    @locked
    def dio8_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO8_EF_INDEX (DIO#(0:22)_EF_INDEX@8)

        Address: 44116
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_INDEX, value)

    @property
    @locked
    def dio9_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO9_EF_INDEX (DIO#(0:22)_EF_INDEX@9)

        Address: 44118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_INDEX)  # type: ignore

    @dio9_ef_index.setter
    @locked
    def dio9_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO9_EF_INDEX (DIO#(0:22)_EF_INDEX@9)

        Address: 44118
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_INDEX, value)

    @property
    @locked
    def dio10_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO10_EF_INDEX (DIO#(0:22)_EF_INDEX@10)

        Address: 44120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_INDEX)  # type: ignore

    @dio10_ef_index.setter
    @locked
    def dio10_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO10_EF_INDEX (DIO#(0:22)_EF_INDEX@10)

        Address: 44120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_INDEX, value)

    @property
    @locked
    def dio11_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO11_EF_INDEX (DIO#(0:22)_EF_INDEX@11)

        Address: 44122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_INDEX)  # type: ignore

    @dio11_ef_index.setter
    @locked
    def dio11_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO11_EF_INDEX (DIO#(0:22)_EF_INDEX@11)

        Address: 44122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_INDEX, value)

    @property
    @locked
    def dio12_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO12_EF_INDEX (DIO#(0:22)_EF_INDEX@12)

        Address: 44124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_INDEX)  # type: ignore

    @dio12_ef_index.setter
    @locked
    def dio12_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO12_EF_INDEX (DIO#(0:22)_EF_INDEX@12)

        Address: 44124
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_INDEX, value)

    @property
    @locked
    def dio13_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO13_EF_INDEX (DIO#(0:22)_EF_INDEX@13)

        Address: 44126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_INDEX)  # type: ignore

    @dio13_ef_index.setter
    @locked
    def dio13_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO13_EF_INDEX (DIO#(0:22)_EF_INDEX@13)

        Address: 44126
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_INDEX, value)

    @property
    @locked
    def dio14_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO14_EF_INDEX (DIO#(0:22)_EF_INDEX@14)

        Address: 44128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_INDEX)  # type: ignore

    @dio14_ef_index.setter
    @locked
    def dio14_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO14_EF_INDEX (DIO#(0:22)_EF_INDEX@14)

        Address: 44128
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_INDEX, value)

    @property
    @locked
    def dio15_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO15_EF_INDEX (DIO#(0:22)_EF_INDEX@15)

        Address: 44130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_INDEX)  # type: ignore

    @dio15_ef_index.setter
    @locked
    def dio15_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO15_EF_INDEX (DIO#(0:22)_EF_INDEX@15)

        Address: 44130
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_INDEX, value)

    @property
    @locked
    def dio16_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO16_EF_INDEX (DIO#(0:22)_EF_INDEX@16)

        Address: 44132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_INDEX)  # type: ignore

    @dio16_ef_index.setter
    @locked
    def dio16_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO16_EF_INDEX (DIO#(0:22)_EF_INDEX@16)

        Address: 44132
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_INDEX, value)

    @property
    @locked
    def dio17_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO17_EF_INDEX (DIO#(0:22)_EF_INDEX@17)

        Address: 44134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_INDEX)  # type: ignore

    @dio17_ef_index.setter
    @locked
    def dio17_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO17_EF_INDEX (DIO#(0:22)_EF_INDEX@17)

        Address: 44134
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_INDEX, value)

    @property
    @locked
    def dio18_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO18_EF_INDEX (DIO#(0:22)_EF_INDEX@18)

        Address: 44136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_INDEX)  # type: ignore

    @dio18_ef_index.setter
    @locked
    def dio18_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO18_EF_INDEX (DIO#(0:22)_EF_INDEX@18)

        Address: 44136
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_INDEX, value)

    @property
    @locked
    def dio19_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO19_EF_INDEX (DIO#(0:22)_EF_INDEX@19)

        Address: 44138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_INDEX)  # type: ignore

    @dio19_ef_index.setter
    @locked
    def dio19_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO19_EF_INDEX (DIO#(0:22)_EF_INDEX@19)

        Address: 44138
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_INDEX, value)

    @property
    @locked
    def dio20_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO20_EF_INDEX (DIO#(0:22)_EF_INDEX@20)

        Address: 44140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_INDEX)  # type: ignore

    @dio20_ef_index.setter
    @locked
    def dio20_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO20_EF_INDEX (DIO#(0:22)_EF_INDEX@20)

        Address: 44140
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_INDEX, value)

    @property
    @locked
    def dio21_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO21_EF_INDEX (DIO#(0:22)_EF_INDEX@21)

        Address: 44142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_INDEX)  # type: ignore

    @dio21_ef_index.setter
    @locked
    def dio21_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO21_EF_INDEX (DIO#(0:22)_EF_INDEX@21)

        Address: 44142
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_INDEX, value)

    @property
    @locked
    def dio22_ef_index(self) -> int:
        """
        An index to specify the feature you want.

        Register: DIO22_EF_INDEX (DIO#(0:22)_EF_INDEX@22)

        Address: 44144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_INDEX)  # type: ignore

    @dio22_ef_index.setter
    @locked
    def dio22_ef_index(self, value: int) -> None:
        """
        An index to specify the feature you want.

        Register: DIO22_EF_INDEX (DIO#(0:22)_EF_INDEX@22)

        Address: 44144
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_INDEX, value)

    @property
    @locked
    def dio0_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@0)

        Address: 44200
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_OPTIONS)  # type: ignore

    @dio0_ef_options.setter
    @locked
    def dio0_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@0)

        Address: 44200
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_OPTIONS, value)

    @property
    @locked
    def dio1_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@1)

        Address: 44202
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_OPTIONS)  # type: ignore

    @dio1_ef_options.setter
    @locked
    def dio1_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@1)

        Address: 44202
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_OPTIONS, value)

    @property
    @locked
    def dio2_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@2)

        Address: 44204
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_OPTIONS)  # type: ignore

    @dio2_ef_options.setter
    @locked
    def dio2_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@2)

        Address: 44204
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_OPTIONS, value)

    @property
    @locked
    def dio3_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@3)

        Address: 44206
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_OPTIONS)  # type: ignore

    @dio3_ef_options.setter
    @locked
    def dio3_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@3)

        Address: 44206
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_OPTIONS, value)

    @property
    @locked
    def dio4_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@4)

        Address: 44208
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_OPTIONS)  # type: ignore

    @dio4_ef_options.setter
    @locked
    def dio4_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@4)

        Address: 44208
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_OPTIONS, value)

    @property
    @locked
    def dio5_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@5)

        Address: 44210
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_OPTIONS)  # type: ignore

    @dio5_ef_options.setter
    @locked
    def dio5_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@5)

        Address: 44210
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_OPTIONS, value)

    @property
    @locked
    def dio6_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@6)

        Address: 44212
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_OPTIONS)  # type: ignore

    @dio6_ef_options.setter
    @locked
    def dio6_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@6)

        Address: 44212
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_OPTIONS, value)

    @property
    @locked
    def dio7_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@7)

        Address: 44214
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_OPTIONS)  # type: ignore

    @dio7_ef_options.setter
    @locked
    def dio7_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@7)

        Address: 44214
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_OPTIONS, value)

    @property
    @locked
    def dio8_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@8)

        Address: 44216
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_OPTIONS)  # type: ignore

    @dio8_ef_options.setter
    @locked
    def dio8_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@8)

        Address: 44216
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_OPTIONS, value)

    @property
    @locked
    def dio9_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@9)

        Address: 44218
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_OPTIONS)  # type: ignore

    @dio9_ef_options.setter
    @locked
    def dio9_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@9)

        Address: 44218
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_OPTIONS, value)

    @property
    @locked
    def dio10_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@10)

        Address: 44220
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_OPTIONS)  # type: ignore

    @dio10_ef_options.setter
    @locked
    def dio10_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@10)

        Address: 44220
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_OPTIONS, value)

    @property
    @locked
    def dio11_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@11)

        Address: 44222
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_OPTIONS)  # type: ignore

    @dio11_ef_options.setter
    @locked
    def dio11_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@11)

        Address: 44222
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_OPTIONS, value)

    @property
    @locked
    def dio12_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@12)

        Address: 44224
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_OPTIONS)  # type: ignore

    @dio12_ef_options.setter
    @locked
    def dio12_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@12)

        Address: 44224
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_OPTIONS, value)

    @property
    @locked
    def dio13_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@13)

        Address: 44226
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_OPTIONS)  # type: ignore

    @dio13_ef_options.setter
    @locked
    def dio13_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@13)

        Address: 44226
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_OPTIONS, value)

    @property
    @locked
    def dio14_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@14)

        Address: 44228
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_OPTIONS)  # type: ignore

    @dio14_ef_options.setter
    @locked
    def dio14_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@14)

        Address: 44228
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_OPTIONS, value)

    @property
    @locked
    def dio15_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@15)

        Address: 44230
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_OPTIONS)  # type: ignore

    @dio15_ef_options.setter
    @locked
    def dio15_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@15)

        Address: 44230
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_OPTIONS, value)

    @property
    @locked
    def dio16_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@16)

        Address: 44232
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_OPTIONS)  # type: ignore

    @dio16_ef_options.setter
    @locked
    def dio16_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@16)

        Address: 44232
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_OPTIONS, value)

    @property
    @locked
    def dio17_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@17)

        Address: 44234
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_OPTIONS)  # type: ignore

    @dio17_ef_options.setter
    @locked
    def dio17_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@17)

        Address: 44234
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_OPTIONS, value)

    @property
    @locked
    def dio18_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@18)

        Address: 44236
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_OPTIONS)  # type: ignore

    @dio18_ef_options.setter
    @locked
    def dio18_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@18)

        Address: 44236
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_OPTIONS, value)

    @property
    @locked
    def dio19_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@19)

        Address: 44238
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_OPTIONS)  # type: ignore

    @dio19_ef_options.setter
    @locked
    def dio19_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@19)

        Address: 44238
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_OPTIONS, value)

    @property
    @locked
    def dio20_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@20)

        Address: 44240
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_OPTIONS)  # type: ignore

    @dio20_ef_options.setter
    @locked
    def dio20_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@20)

        Address: 44240
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_OPTIONS, value)

    @property
    @locked
    def dio21_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@21)

        Address: 44242
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_OPTIONS)  # type: ignore

    @dio21_ef_options.setter
    @locked
    def dio21_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@21)

        Address: 44242
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_OPTIONS, value)

    @property
    @locked
    def dio22_ef_options(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@22)

        Address: 44244
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_OPTIONS)  # type: ignore

    @dio22_ef_options.setter
    @locked
    def dio22_ef_options(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_OPTIONS (DIO#(0:22)_EF_OPTIONS@22)

        Address: 44244
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_OPTIONS, value)

    @property
    @locked
    def dio0_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@0)

        Address: 44300
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_A)  # type: ignore

    @dio0_ef_config_a.setter
    @locked
    def dio0_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@0)

        Address: 44300
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_A, value)

    @property
    @locked
    def dio1_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@1)

        Address: 44302
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_A)  # type: ignore

    @dio1_ef_config_a.setter
    @locked
    def dio1_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@1)

        Address: 44302
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_A, value)

    @property
    @locked
    def dio2_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@2)

        Address: 44304
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_A)  # type: ignore

    @dio2_ef_config_a.setter
    @locked
    def dio2_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@2)

        Address: 44304
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_A, value)

    @property
    @locked
    def dio3_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@3)

        Address: 44306
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_A)  # type: ignore

    @dio3_ef_config_a.setter
    @locked
    def dio3_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@3)

        Address: 44306
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_A, value)

    @property
    @locked
    def dio4_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@4)

        Address: 44308
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_A)  # type: ignore

    @dio4_ef_config_a.setter
    @locked
    def dio4_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@4)

        Address: 44308
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_A, value)

    @property
    @locked
    def dio5_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@5)

        Address: 44310
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_A)  # type: ignore

    @dio5_ef_config_a.setter
    @locked
    def dio5_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@5)

        Address: 44310
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_A, value)

    @property
    @locked
    def dio6_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@6)

        Address: 44312
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_A)  # type: ignore

    @dio6_ef_config_a.setter
    @locked
    def dio6_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@6)

        Address: 44312
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_A, value)

    @property
    @locked
    def dio7_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@7)

        Address: 44314
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_A)  # type: ignore

    @dio7_ef_config_a.setter
    @locked
    def dio7_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@7)

        Address: 44314
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_A, value)

    @property
    @locked
    def dio8_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@8)

        Address: 44316
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_A)  # type: ignore

    @dio8_ef_config_a.setter
    @locked
    def dio8_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@8)

        Address: 44316
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_A, value)

    @property
    @locked
    def dio9_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@9)

        Address: 44318
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_A)  # type: ignore

    @dio9_ef_config_a.setter
    @locked
    def dio9_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@9)

        Address: 44318
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_A, value)

    @property
    @locked
    def dio10_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@10)

        Address: 44320
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_A)  # type: ignore

    @dio10_ef_config_a.setter
    @locked
    def dio10_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@10)

        Address: 44320
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_A, value)

    @property
    @locked
    def dio11_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@11)

        Address: 44322
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_A)  # type: ignore

    @dio11_ef_config_a.setter
    @locked
    def dio11_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@11)

        Address: 44322
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_A, value)

    @property
    @locked
    def dio12_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@12)

        Address: 44324
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_A)  # type: ignore

    @dio12_ef_config_a.setter
    @locked
    def dio12_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@12)

        Address: 44324
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_A, value)

    @property
    @locked
    def dio13_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@13)

        Address: 44326
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_A)  # type: ignore

    @dio13_ef_config_a.setter
    @locked
    def dio13_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@13)

        Address: 44326
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_A, value)

    @property
    @locked
    def dio14_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@14)

        Address: 44328
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_A)  # type: ignore

    @dio14_ef_config_a.setter
    @locked
    def dio14_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@14)

        Address: 44328
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_A, value)

    @property
    @locked
    def dio15_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@15)

        Address: 44330
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_A)  # type: ignore

    @dio15_ef_config_a.setter
    @locked
    def dio15_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@15)

        Address: 44330
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_A, value)

    @property
    @locked
    def dio16_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@16)

        Address: 44332
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_A)  # type: ignore

    @dio16_ef_config_a.setter
    @locked
    def dio16_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@16)

        Address: 44332
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_A, value)

    @property
    @locked
    def dio17_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@17)

        Address: 44334
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_A)  # type: ignore

    @dio17_ef_config_a.setter
    @locked
    def dio17_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@17)

        Address: 44334
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_A, value)

    @property
    @locked
    def dio18_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@18)

        Address: 44336
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_A)  # type: ignore

    @dio18_ef_config_a.setter
    @locked
    def dio18_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@18)

        Address: 44336
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_A, value)

    @property
    @locked
    def dio19_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@19)

        Address: 44338
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_A)  # type: ignore

    @dio19_ef_config_a.setter
    @locked
    def dio19_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@19)

        Address: 44338
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_A, value)

    @property
    @locked
    def dio20_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@20)

        Address: 44340
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_A)  # type: ignore

    @dio20_ef_config_a.setter
    @locked
    def dio20_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@20)

        Address: 44340
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_A, value)

    @property
    @locked
    def dio21_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@21)

        Address: 44342
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_A)  # type: ignore

    @dio21_ef_config_a.setter
    @locked
    def dio21_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@21)

        Address: 44342
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_A, value)

    @property
    @locked
    def dio22_ef_config_a(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@22)

        Address: 44344
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_A)  # type: ignore

    @dio22_ef_config_a.setter
    @locked
    def dio22_ef_config_a(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_A (DIO#(0:22)_EF_CONFIG_A@22)

        Address: 44344
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_A, value)

    @property
    @locked
    def dio0_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@0)

        Address: 44400
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_B)  # type: ignore

    @dio0_ef_config_b.setter
    @locked
    def dio0_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@0)

        Address: 44400
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_B, value)

    @property
    @locked
    def dio1_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@1)

        Address: 44402
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_B)  # type: ignore

    @dio1_ef_config_b.setter
    @locked
    def dio1_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@1)

        Address: 44402
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_B, value)

    @property
    @locked
    def dio2_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@2)

        Address: 44404
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_B)  # type: ignore

    @dio2_ef_config_b.setter
    @locked
    def dio2_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@2)

        Address: 44404
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_B, value)

    @property
    @locked
    def dio3_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@3)

        Address: 44406
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_B)  # type: ignore

    @dio3_ef_config_b.setter
    @locked
    def dio3_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@3)

        Address: 44406
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_B, value)

    @property
    @locked
    def dio4_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@4)

        Address: 44408
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_B)  # type: ignore

    @dio4_ef_config_b.setter
    @locked
    def dio4_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@4)

        Address: 44408
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_B, value)

    @property
    @locked
    def dio5_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@5)

        Address: 44410
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_B)  # type: ignore

    @dio5_ef_config_b.setter
    @locked
    def dio5_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@5)

        Address: 44410
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_B, value)

    @property
    @locked
    def dio6_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@6)

        Address: 44412
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_B)  # type: ignore

    @dio6_ef_config_b.setter
    @locked
    def dio6_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@6)

        Address: 44412
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_B, value)

    @property
    @locked
    def dio7_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@7)

        Address: 44414
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_B)  # type: ignore

    @dio7_ef_config_b.setter
    @locked
    def dio7_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@7)

        Address: 44414
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_B, value)

    @property
    @locked
    def dio8_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@8)

        Address: 44416
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_B)  # type: ignore

    @dio8_ef_config_b.setter
    @locked
    def dio8_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@8)

        Address: 44416
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_B, value)

    @property
    @locked
    def dio9_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@9)

        Address: 44418
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_B)  # type: ignore

    @dio9_ef_config_b.setter
    @locked
    def dio9_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@9)

        Address: 44418
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_B, value)

    @property
    @locked
    def dio10_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@10)

        Address: 44420
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_B)  # type: ignore

    @dio10_ef_config_b.setter
    @locked
    def dio10_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@10)

        Address: 44420
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_B, value)

    @property
    @locked
    def dio11_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@11)

        Address: 44422
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_B)  # type: ignore

    @dio11_ef_config_b.setter
    @locked
    def dio11_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@11)

        Address: 44422
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_B, value)

    @property
    @locked
    def dio12_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@12)

        Address: 44424
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_B)  # type: ignore

    @dio12_ef_config_b.setter
    @locked
    def dio12_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@12)

        Address: 44424
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_B, value)

    @property
    @locked
    def dio13_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@13)

        Address: 44426
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_B)  # type: ignore

    @dio13_ef_config_b.setter
    @locked
    def dio13_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@13)

        Address: 44426
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_B, value)

    @property
    @locked
    def dio14_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@14)

        Address: 44428
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_B)  # type: ignore

    @dio14_ef_config_b.setter
    @locked
    def dio14_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@14)

        Address: 44428
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_B, value)

    @property
    @locked
    def dio15_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@15)

        Address: 44430
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_B)  # type: ignore

    @dio15_ef_config_b.setter
    @locked
    def dio15_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@15)

        Address: 44430
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_B, value)

    @property
    @locked
    def dio16_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@16)

        Address: 44432
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_B)  # type: ignore

    @dio16_ef_config_b.setter
    @locked
    def dio16_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@16)

        Address: 44432
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_B, value)

    @property
    @locked
    def dio17_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@17)

        Address: 44434
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_B)  # type: ignore

    @dio17_ef_config_b.setter
    @locked
    def dio17_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@17)

        Address: 44434
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_B, value)

    @property
    @locked
    def dio18_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@18)

        Address: 44436
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_B)  # type: ignore

    @dio18_ef_config_b.setter
    @locked
    def dio18_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@18)

        Address: 44436
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_B, value)

    @property
    @locked
    def dio19_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@19)

        Address: 44438
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_B)  # type: ignore

    @dio19_ef_config_b.setter
    @locked
    def dio19_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@19)

        Address: 44438
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_B, value)

    @property
    @locked
    def dio20_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@20)

        Address: 44440
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_B)  # type: ignore

    @dio20_ef_config_b.setter
    @locked
    def dio20_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@20)

        Address: 44440
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_B, value)

    @property
    @locked
    def dio21_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@21)

        Address: 44442
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_B)  # type: ignore

    @dio21_ef_config_b.setter
    @locked
    def dio21_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@21)

        Address: 44442
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_B, value)

    @property
    @locked
    def dio22_ef_config_b(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@22)

        Address: 44444
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_B)  # type: ignore

    @dio22_ef_config_b.setter
    @locked
    def dio22_ef_config_b(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_B (DIO#(0:22)_EF_CONFIG_B@22)

        Address: 44444
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_B, value)

    @property
    @locked
    def dio0_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@0)

        Address: 44500
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_C)  # type: ignore

    @dio0_ef_config_c.setter
    @locked
    def dio0_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@0)

        Address: 44500
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_C, value)

    @property
    @locked
    def dio1_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@1)

        Address: 44502
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_C)  # type: ignore

    @dio1_ef_config_c.setter
    @locked
    def dio1_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@1)

        Address: 44502
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_C, value)

    @property
    @locked
    def dio2_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@2)

        Address: 44504
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_C)  # type: ignore

    @dio2_ef_config_c.setter
    @locked
    def dio2_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@2)

        Address: 44504
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_C, value)

    @property
    @locked
    def dio3_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@3)

        Address: 44506
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_C)  # type: ignore

    @dio3_ef_config_c.setter
    @locked
    def dio3_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@3)

        Address: 44506
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_C, value)

    @property
    @locked
    def dio4_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@4)

        Address: 44508
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_C)  # type: ignore

    @dio4_ef_config_c.setter
    @locked
    def dio4_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@4)

        Address: 44508
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_C, value)

    @property
    @locked
    def dio5_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@5)

        Address: 44510
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_C)  # type: ignore

    @dio5_ef_config_c.setter
    @locked
    def dio5_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@5)

        Address: 44510
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_C, value)

    @property
    @locked
    def dio6_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@6)

        Address: 44512
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_C)  # type: ignore

    @dio6_ef_config_c.setter
    @locked
    def dio6_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@6)

        Address: 44512
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_C, value)

    @property
    @locked
    def dio7_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@7)

        Address: 44514
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_C)  # type: ignore

    @dio7_ef_config_c.setter
    @locked
    def dio7_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@7)

        Address: 44514
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_C, value)

    @property
    @locked
    def dio8_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@8)

        Address: 44516
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_C)  # type: ignore

    @dio8_ef_config_c.setter
    @locked
    def dio8_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@8)

        Address: 44516
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_C, value)

    @property
    @locked
    def dio9_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@9)

        Address: 44518
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_C)  # type: ignore

    @dio9_ef_config_c.setter
    @locked
    def dio9_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@9)

        Address: 44518
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_C, value)

    @property
    @locked
    def dio10_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@10)

        Address: 44520
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_C)  # type: ignore

    @dio10_ef_config_c.setter
    @locked
    def dio10_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@10)

        Address: 44520
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_C, value)

    @property
    @locked
    def dio11_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@11)

        Address: 44522
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_C)  # type: ignore

    @dio11_ef_config_c.setter
    @locked
    def dio11_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@11)

        Address: 44522
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_C, value)

    @property
    @locked
    def dio12_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@12)

        Address: 44524
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_C)  # type: ignore

    @dio12_ef_config_c.setter
    @locked
    def dio12_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@12)

        Address: 44524
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_C, value)

    @property
    @locked
    def dio13_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@13)

        Address: 44526
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_C)  # type: ignore

    @dio13_ef_config_c.setter
    @locked
    def dio13_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@13)

        Address: 44526
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_C, value)

    @property
    @locked
    def dio14_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@14)

        Address: 44528
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_C)  # type: ignore

    @dio14_ef_config_c.setter
    @locked
    def dio14_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@14)

        Address: 44528
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_C, value)

    @property
    @locked
    def dio15_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@15)

        Address: 44530
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_C)  # type: ignore

    @dio15_ef_config_c.setter
    @locked
    def dio15_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@15)

        Address: 44530
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_C, value)

    @property
    @locked
    def dio16_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@16)

        Address: 44532
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_C)  # type: ignore

    @dio16_ef_config_c.setter
    @locked
    def dio16_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@16)

        Address: 44532
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_C, value)

    @property
    @locked
    def dio17_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@17)

        Address: 44534
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_C)  # type: ignore

    @dio17_ef_config_c.setter
    @locked
    def dio17_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@17)

        Address: 44534
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_C, value)

    @property
    @locked
    def dio18_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@18)

        Address: 44536
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_C)  # type: ignore

    @dio18_ef_config_c.setter
    @locked
    def dio18_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@18)

        Address: 44536
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_C, value)

    @property
    @locked
    def dio19_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@19)

        Address: 44538
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_C)  # type: ignore

    @dio19_ef_config_c.setter
    @locked
    def dio19_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@19)

        Address: 44538
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_C, value)

    @property
    @locked
    def dio20_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@20)

        Address: 44540
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_C)  # type: ignore

    @dio20_ef_config_c.setter
    @locked
    def dio20_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@20)

        Address: 44540
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_C, value)

    @property
    @locked
    def dio21_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@21)

        Address: 44542
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_C)  # type: ignore

    @dio21_ef_config_c.setter
    @locked
    def dio21_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@21)

        Address: 44542
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_C, value)

    @property
    @locked
    def dio22_ef_config_c(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@22)

        Address: 44544
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_C)  # type: ignore

    @dio22_ef_config_c.setter
    @locked
    def dio22_ef_config_c(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_C (DIO#(0:22)_EF_CONFIG_C@22)

        Address: 44544
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_C, value)

    @property
    @locked
    def dio0_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@0)

        Address: 44600
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_D)  # type: ignore

    @dio0_ef_config_d.setter
    @locked
    def dio0_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO0_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@0)

        Address: 44600
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO0_EF_CONFIG_D, value)

    @property
    @locked
    def dio1_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@1)

        Address: 44602
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_D)  # type: ignore

    @dio1_ef_config_d.setter
    @locked
    def dio1_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO1_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@1)

        Address: 44602
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO1_EF_CONFIG_D, value)

    @property
    @locked
    def dio2_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@2)

        Address: 44604
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_D)  # type: ignore

    @dio2_ef_config_d.setter
    @locked
    def dio2_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO2_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@2)

        Address: 44604
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO2_EF_CONFIG_D, value)

    @property
    @locked
    def dio3_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@3)

        Address: 44606
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_D)  # type: ignore

    @dio3_ef_config_d.setter
    @locked
    def dio3_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO3_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@3)

        Address: 44606
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO3_EF_CONFIG_D, value)

    @property
    @locked
    def dio4_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@4)

        Address: 44608
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_D)  # type: ignore

    @dio4_ef_config_d.setter
    @locked
    def dio4_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO4_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@4)

        Address: 44608
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO4_EF_CONFIG_D, value)

    @property
    @locked
    def dio5_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@5)

        Address: 44610
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_D)  # type: ignore

    @dio5_ef_config_d.setter
    @locked
    def dio5_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO5_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@5)

        Address: 44610
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO5_EF_CONFIG_D, value)

    @property
    @locked
    def dio6_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@6)

        Address: 44612
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_D)  # type: ignore

    @dio6_ef_config_d.setter
    @locked
    def dio6_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO6_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@6)

        Address: 44612
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO6_EF_CONFIG_D, value)

    @property
    @locked
    def dio7_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@7)

        Address: 44614
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_D)  # type: ignore

    @dio7_ef_config_d.setter
    @locked
    def dio7_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO7_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@7)

        Address: 44614
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO7_EF_CONFIG_D, value)

    @property
    @locked
    def dio8_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@8)

        Address: 44616
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_D)  # type: ignore

    @dio8_ef_config_d.setter
    @locked
    def dio8_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO8_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@8)

        Address: 44616
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO8_EF_CONFIG_D, value)

    @property
    @locked
    def dio9_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@9)

        Address: 44618
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_D)  # type: ignore

    @dio9_ef_config_d.setter
    @locked
    def dio9_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO9_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@9)

        Address: 44618
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO9_EF_CONFIG_D, value)

    @property
    @locked
    def dio10_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@10)

        Address: 44620
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_D)  # type: ignore

    @dio10_ef_config_d.setter
    @locked
    def dio10_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO10_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@10)

        Address: 44620
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO10_EF_CONFIG_D, value)

    @property
    @locked
    def dio11_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@11)

        Address: 44622
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_D)  # type: ignore

    @dio11_ef_config_d.setter
    @locked
    def dio11_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO11_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@11)

        Address: 44622
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO11_EF_CONFIG_D, value)

    @property
    @locked
    def dio12_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@12)

        Address: 44624
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_D)  # type: ignore

    @dio12_ef_config_d.setter
    @locked
    def dio12_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO12_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@12)

        Address: 44624
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO12_EF_CONFIG_D, value)

    @property
    @locked
    def dio13_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@13)

        Address: 44626
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_D)  # type: ignore

    @dio13_ef_config_d.setter
    @locked
    def dio13_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO13_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@13)

        Address: 44626
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO13_EF_CONFIG_D, value)

    @property
    @locked
    def dio14_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@14)

        Address: 44628
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_D)  # type: ignore

    @dio14_ef_config_d.setter
    @locked
    def dio14_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO14_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@14)

        Address: 44628
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO14_EF_CONFIG_D, value)

    @property
    @locked
    def dio15_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@15)

        Address: 44630
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_D)  # type: ignore

    @dio15_ef_config_d.setter
    @locked
    def dio15_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO15_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@15)

        Address: 44630
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO15_EF_CONFIG_D, value)

    @property
    @locked
    def dio16_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@16)

        Address: 44632
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_D)  # type: ignore

    @dio16_ef_config_d.setter
    @locked
    def dio16_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO16_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@16)

        Address: 44632
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO16_EF_CONFIG_D, value)

    @property
    @locked
    def dio17_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@17)

        Address: 44634
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_D)  # type: ignore

    @dio17_ef_config_d.setter
    @locked
    def dio17_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO17_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@17)

        Address: 44634
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO17_EF_CONFIG_D, value)

    @property
    @locked
    def dio18_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@18)

        Address: 44636
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_D)  # type: ignore

    @dio18_ef_config_d.setter
    @locked
    def dio18_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO18_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@18)

        Address: 44636
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO18_EF_CONFIG_D, value)

    @property
    @locked
    def dio19_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@19)

        Address: 44638
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_D)  # type: ignore

    @dio19_ef_config_d.setter
    @locked
    def dio19_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO19_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@19)

        Address: 44638
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO19_EF_CONFIG_D, value)

    @property
    @locked
    def dio20_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@20)

        Address: 44640
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_D)  # type: ignore

    @dio20_ef_config_d.setter
    @locked
    def dio20_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO20_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@20)

        Address: 44640
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO20_EF_CONFIG_D, value)

    @property
    @locked
    def dio21_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@21)

        Address: 44642
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_D)  # type: ignore

    @dio21_ef_config_d.setter
    @locked
    def dio21_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO21_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@21)

        Address: 44642
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO21_EF_CONFIG_D, value)

    @property
    @locked
    def dio22_ef_config_d(self) -> int:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@22)

        Address: 44644
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_D)  # type: ignore

    @dio22_ef_config_d.setter
    @locked
    def dio22_ef_config_d(self, value: int) -> None:
        """
        Function dependent on selected feature index.

        Register: DIO22_EF_CONFIG_D (DIO#(0:22)_EF_CONFIG_D@22)

        Address: 44644
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DIO22_EF_CONFIG_D, value)

    @property
    @locked
    def dio0_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO0_EF_READ_A (DIO#(0:22)_EF_READ_A@0)

        Address: 3000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio1_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO1_EF_READ_A (DIO#(0:22)_EF_READ_A@1)

        Address: 3002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio2_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO2_EF_READ_A (DIO#(0:22)_EF_READ_A@2)

        Address: 3004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio3_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO3_EF_READ_A (DIO#(0:22)_EF_READ_A@3)

        Address: 3006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio4_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO4_EF_READ_A (DIO#(0:22)_EF_READ_A@4)

        Address: 3008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio5_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO5_EF_READ_A (DIO#(0:22)_EF_READ_A@5)

        Address: 3010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio6_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO6_EF_READ_A (DIO#(0:22)_EF_READ_A@6)

        Address: 3012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio7_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO7_EF_READ_A (DIO#(0:22)_EF_READ_A@7)

        Address: 3014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio8_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO8_EF_READ_A (DIO#(0:22)_EF_READ_A@8)

        Address: 3016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio9_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO9_EF_READ_A (DIO#(0:22)_EF_READ_A@9)

        Address: 3018
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio10_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO10_EF_READ_A (DIO#(0:22)_EF_READ_A@10)

        Address: 3020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio11_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO11_EF_READ_A (DIO#(0:22)_EF_READ_A@11)

        Address: 3022
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio12_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO12_EF_READ_A (DIO#(0:22)_EF_READ_A@12)

        Address: 3024
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio13_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO13_EF_READ_A (DIO#(0:22)_EF_READ_A@13)

        Address: 3026
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio14_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO14_EF_READ_A (DIO#(0:22)_EF_READ_A@14)

        Address: 3028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio15_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO15_EF_READ_A (DIO#(0:22)_EF_READ_A@15)

        Address: 3030
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio16_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO16_EF_READ_A (DIO#(0:22)_EF_READ_A@16)

        Address: 3032
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio17_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO17_EF_READ_A (DIO#(0:22)_EF_READ_A@17)

        Address: 3034
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio18_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO18_EF_READ_A (DIO#(0:22)_EF_READ_A@18)

        Address: 3036
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio19_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO19_EF_READ_A (DIO#(0:22)_EF_READ_A@19)

        Address: 3038
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio20_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO20_EF_READ_A (DIO#(0:22)_EF_READ_A@20)

        Address: 3040
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio21_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO21_EF_READ_A (DIO#(0:22)_EF_READ_A@21)

        Address: 3042
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio22_ef_read_a(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO22_EF_READ_A (DIO#(0:22)_EF_READ_A@22)

        Address: 3044
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_READ_A)  # type: ignore

    @property
    @locked
    def dio0_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO0_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@0)

        Address: 3100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio1_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO1_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@1)

        Address: 3102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio2_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO2_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@2)

        Address: 3104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio3_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO3_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@3)

        Address: 3106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio4_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO4_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@4)

        Address: 3108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio5_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO5_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@5)

        Address: 3110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio6_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO6_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@6)

        Address: 3112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio7_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO7_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@7)

        Address: 3114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio8_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO8_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@8)

        Address: 3116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio9_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO9_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@9)

        Address: 3118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio10_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO10_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@10)

        Address: 3120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio11_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO11_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@11)

        Address: 3122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio12_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO12_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@12)

        Address: 3124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio13_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO13_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@13)

        Address: 3126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio14_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO14_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@14)

        Address: 3128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio15_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO15_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@15)

        Address: 3130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio16_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO16_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@16)

        Address: 3132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio17_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO17_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@17)

        Address: 3134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio18_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO18_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@18)

        Address: 3136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio19_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO19_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@19)

        Address: 3138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio20_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO20_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@20)

        Address: 3140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio21_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO21_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@21)

        Address: 3142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio22_ef_read_a_and_reset(self) -> int:
        """
        Reads the same value as DIO#(0:22)_EF_READ_A and forces a reset.

        Register: DIO22_EF_READ_A_AND_RESET (DIO#(0:22)_EF_READ_A_AND_RESET@22)

        Address: 3144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_READ_A_AND_RESET)  # type: ignore

    @property
    @locked
    def dio0_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO0_EF_READ_B (DIO#(0:22)_EF_READ_B@0)

        Address: 3200
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO0_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio1_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO1_EF_READ_B (DIO#(0:22)_EF_READ_B@1)

        Address: 3202
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO1_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio2_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO2_EF_READ_B (DIO#(0:22)_EF_READ_B@2)

        Address: 3204
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO2_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio3_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO3_EF_READ_B (DIO#(0:22)_EF_READ_B@3)

        Address: 3206
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO3_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio4_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO4_EF_READ_B (DIO#(0:22)_EF_READ_B@4)

        Address: 3208
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO4_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio5_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO5_EF_READ_B (DIO#(0:22)_EF_READ_B@5)

        Address: 3210
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO5_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio6_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO6_EF_READ_B (DIO#(0:22)_EF_READ_B@6)

        Address: 3212
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO6_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio7_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO7_EF_READ_B (DIO#(0:22)_EF_READ_B@7)

        Address: 3214
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO7_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio8_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO8_EF_READ_B (DIO#(0:22)_EF_READ_B@8)

        Address: 3216
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO8_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio9_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO9_EF_READ_B (DIO#(0:22)_EF_READ_B@9)

        Address: 3218
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO9_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio10_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO10_EF_READ_B (DIO#(0:22)_EF_READ_B@10)

        Address: 3220
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO10_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio11_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO11_EF_READ_B (DIO#(0:22)_EF_READ_B@11)

        Address: 3222
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO11_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio12_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO12_EF_READ_B (DIO#(0:22)_EF_READ_B@12)

        Address: 3224
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO12_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio13_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO13_EF_READ_B (DIO#(0:22)_EF_READ_B@13)

        Address: 3226
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO13_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio14_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO14_EF_READ_B (DIO#(0:22)_EF_READ_B@14)

        Address: 3228
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO14_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio15_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO15_EF_READ_B (DIO#(0:22)_EF_READ_B@15)

        Address: 3230
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO15_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio16_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO16_EF_READ_B (DIO#(0:22)_EF_READ_B@16)

        Address: 3232
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO16_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio17_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO17_EF_READ_B (DIO#(0:22)_EF_READ_B@17)

        Address: 3234
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO17_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio18_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO18_EF_READ_B (DIO#(0:22)_EF_READ_B@18)

        Address: 3236
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO18_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio19_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO19_EF_READ_B (DIO#(0:22)_EF_READ_B@19)

        Address: 3238
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO19_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio20_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO20_EF_READ_B (DIO#(0:22)_EF_READ_B@20)

        Address: 3240
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO20_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio21_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO21_EF_READ_B (DIO#(0:22)_EF_READ_B@21)

        Address: 3242
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO21_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio22_ef_read_b(self) -> int:
        """
        Reads an unsigned integer value. The meaning of the integer is dependent on selected feature index.

        Register: DIO22_EF_READ_B (DIO#(0:22)_EF_READ_B@22)

        Address: 3244
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DIO22_EF_READ_B)  # type: ignore

    @property
    @locked
    def dio0_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO0_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@0)

        Address: 3500
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO0_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio1_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO1_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@1)

        Address: 3502
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO1_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio2_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO2_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@2)

        Address: 3504
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO2_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio3_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO3_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@3)

        Address: 3506
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO3_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio4_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO4_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@4)

        Address: 3508
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO4_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio5_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO5_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@5)

        Address: 3510
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO5_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio6_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO6_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@6)

        Address: 3512
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO6_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio7_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO7_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@7)

        Address: 3514
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO7_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio8_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO8_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@8)

        Address: 3516
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO8_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio9_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO9_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@9)

        Address: 3518
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO9_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio10_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO10_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@10)

        Address: 3520
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO10_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio11_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO11_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@11)

        Address: 3522
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO11_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio12_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO12_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@12)

        Address: 3524
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO12_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio13_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO13_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@13)

        Address: 3526
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO13_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio14_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO14_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@14)

        Address: 3528
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO14_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio15_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO15_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@15)

        Address: 3530
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO15_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio16_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO16_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@16)

        Address: 3532
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO16_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio17_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO17_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@17)

        Address: 3534
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO17_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio18_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO18_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@18)

        Address: 3536
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO18_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio19_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO19_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@19)

        Address: 3538
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO19_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio20_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO20_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@20)

        Address: 3540
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO20_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio21_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO21_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@21)

        Address: 3542
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO21_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio22_ef_read_a_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO22_EF_READ_A_F (DIO#(0:22)_EF_READ_A_F@22)

        Address: 3544
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO22_EF_READ_A_F)  # type: ignore

    @property
    @locked
    def dio0_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO0_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@0)

        Address: 3600
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO0_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio1_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO1_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@1)

        Address: 3602
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO1_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio2_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO2_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@2)

        Address: 3604
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO2_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio3_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO3_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@3)

        Address: 3606
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO3_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio4_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO4_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@4)

        Address: 3608
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO4_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio5_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO5_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@5)

        Address: 3610
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO5_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio6_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO6_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@6)

        Address: 3612
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO6_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio7_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO7_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@7)

        Address: 3614
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO7_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio8_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO8_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@8)

        Address: 3616
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO8_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio9_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO9_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@9)

        Address: 3618
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO9_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio10_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO10_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@10)

        Address: 3620
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO10_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio11_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO11_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@11)

        Address: 3622
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO11_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio12_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO12_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@12)

        Address: 3624
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO12_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio13_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO13_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@13)

        Address: 3626
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO13_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio14_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO14_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@14)

        Address: 3628
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO14_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio15_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO15_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@15)

        Address: 3630
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO15_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio16_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO16_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@16)

        Address: 3632
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO16_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio17_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO17_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@17)

        Address: 3634
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO17_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio18_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO18_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@18)

        Address: 3636
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO18_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio19_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO19_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@19)

        Address: 3638
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO19_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio20_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO20_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@20)

        Address: 3640
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO20_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio21_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO21_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@21)

        Address: 3642
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO21_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio22_ef_read_a_f_and_reset(self) -> float:
        """
        Reads a floating point value and forces a reset. The meaning of value is dependent on selected feature
        index.

        Register: DIO22_EF_READ_A_F_AND_RESET (DIO#(0:22)_EF_READ_A_F_AND_RESET@22)

        Address: 3644
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO22_EF_READ_A_F_AND_RESET)  # type: ignore

    @property
    @locked
    def dio0_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO0_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@0)

        Address: 3700
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO0_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio1_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO1_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@1)

        Address: 3702
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO1_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio2_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO2_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@2)

        Address: 3704
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO2_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio3_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO3_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@3)

        Address: 3706
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO3_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio4_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO4_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@4)

        Address: 3708
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO4_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio5_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO5_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@5)

        Address: 3710
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO5_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio6_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO6_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@6)

        Address: 3712
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO6_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio7_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO7_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@7)

        Address: 3714
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO7_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio8_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO8_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@8)

        Address: 3716
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO8_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio9_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO9_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@9)

        Address: 3718
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO9_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio10_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO10_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@10)

        Address: 3720
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO10_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio11_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO11_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@11)

        Address: 3722
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO11_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio12_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO12_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@12)

        Address: 3724
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO12_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio13_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO13_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@13)

        Address: 3726
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO13_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio14_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO14_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@14)

        Address: 3728
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO14_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio15_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO15_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@15)

        Address: 3730
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO15_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio16_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO16_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@16)

        Address: 3732
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO16_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio17_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO17_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@17)

        Address: 3734
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO17_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio18_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO18_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@18)

        Address: 3736
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO18_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio19_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO19_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@19)

        Address: 3738
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO19_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio20_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO20_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@20)

        Address: 3740
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO20_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio21_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO21_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@21)

        Address: 3742
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO21_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio22_ef_read_b_f(self) -> float:
        """
        Reads a floating point value. The meaning of value is dependent on selected feature index.

        Register: DIO22_EF_READ_B_F (DIO#(0:22)_EF_READ_B_F@22)

        Address: 3744
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO22_EF_READ_B_F)  # type: ignore

    @property
    @locked
    def dio0_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO0_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@0)

        Address: 45000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO0_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio0_ef_easy_frequency_in.setter
    @locked
    def dio0_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO0_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@0)

        Address: 45000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO0_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio1_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO1_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@1)

        Address: 45002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO1_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio1_ef_easy_frequency_in.setter
    @locked
    def dio1_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO1_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@1)

        Address: 45002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO1_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio2_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO2_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@2)

        Address: 45004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO2_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio2_ef_easy_frequency_in.setter
    @locked
    def dio2_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO2_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@2)

        Address: 45004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO2_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio3_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO3_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@3)

        Address: 45006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO3_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio3_ef_easy_frequency_in.setter
    @locked
    def dio3_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO3_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@3)

        Address: 45006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO3_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio4_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO4_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@4)

        Address: 45008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO4_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio4_ef_easy_frequency_in.setter
    @locked
    def dio4_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO4_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@4)

        Address: 45008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO4_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio5_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO5_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@5)

        Address: 45010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO5_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio5_ef_easy_frequency_in.setter
    @locked
    def dio5_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO5_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@5)

        Address: 45010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO5_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio6_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO6_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@6)

        Address: 45012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO6_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio6_ef_easy_frequency_in.setter
    @locked
    def dio6_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO6_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@6)

        Address: 45012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO6_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio7_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO7_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@7)

        Address: 45014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO7_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio7_ef_easy_frequency_in.setter
    @locked
    def dio7_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO7_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@7)

        Address: 45014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO7_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio8_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO8_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@8)

        Address: 45016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO8_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio8_ef_easy_frequency_in.setter
    @locked
    def dio8_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO8_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@8)

        Address: 45016
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO8_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio9_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO9_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@9)

        Address: 45018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO9_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio9_ef_easy_frequency_in.setter
    @locked
    def dio9_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO9_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@9)

        Address: 45018
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO9_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio10_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO10_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@10)

        Address: 45020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO10_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio10_ef_easy_frequency_in.setter
    @locked
    def dio10_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO10_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@10)

        Address: 45020
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO10_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio11_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO11_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@11)

        Address: 45022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO11_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio11_ef_easy_frequency_in.setter
    @locked
    def dio11_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO11_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@11)

        Address: 45022
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO11_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio12_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO12_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@12)

        Address: 45024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO12_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio12_ef_easy_frequency_in.setter
    @locked
    def dio12_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO12_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@12)

        Address: 45024
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO12_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio13_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO13_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@13)

        Address: 45026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO13_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio13_ef_easy_frequency_in.setter
    @locked
    def dio13_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO13_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@13)

        Address: 45026
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO13_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio14_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO14_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@14)

        Address: 45028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO14_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio14_ef_easy_frequency_in.setter
    @locked
    def dio14_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO14_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@14)

        Address: 45028
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO14_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio15_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO15_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@15)

        Address: 45030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO15_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio15_ef_easy_frequency_in.setter
    @locked
    def dio15_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO15_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@15)

        Address: 45030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO15_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio16_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO16_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@16)

        Address: 45032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO16_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio16_ef_easy_frequency_in.setter
    @locked
    def dio16_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO16_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@16)

        Address: 45032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO16_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio17_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO17_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@17)

        Address: 45034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO17_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio17_ef_easy_frequency_in.setter
    @locked
    def dio17_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO17_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@17)

        Address: 45034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO17_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio18_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO18_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@18)

        Address: 45036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO18_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio18_ef_easy_frequency_in.setter
    @locked
    def dio18_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO18_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@18)

        Address: 45036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO18_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio19_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO19_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@19)

        Address: 45038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO19_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio19_ef_easy_frequency_in.setter
    @locked
    def dio19_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO19_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@19)

        Address: 45038
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO19_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio20_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO20_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@20)

        Address: 45040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO20_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio20_ef_easy_frequency_in.setter
    @locked
    def dio20_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO20_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@20)

        Address: 45040
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO20_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio21_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO21_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@21)

        Address: 45042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO21_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio21_ef_easy_frequency_in.setter
    @locked
    def dio21_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO21_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@21)

        Address: 45042
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO21_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def dio22_ef_easy_frequency_in(self) -> float:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO22_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@22)

        Address: 45044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.DIO22_EF_EASY_FREQUENCY_IN)  # type: ignore

    @dio22_ef_easy_frequency_in.setter
    @locked
    def dio22_ef_easy_frequency_in(self, value: float) -> None:
        """
        Deprecated. Automatically configures the associated DIO_EF for feature index 11. If already configured for
        index 11, reads the result and resets the DIO_EF.

        Register: DIO22_EF_EASY_FREQUENCY_IN (DIO#(0:22)_EF_EASY_FREQUENCY_IN@22)

        Address: 45044
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.DIO22_EF_EASY_FREQUENCY_IN, value)

    @property
    @locked
    def power_ethernet(self) -> int:
        """
        The current ON/OFF state of the Ethernet module.

        Register: POWER_ETHERNET

        Address: 48003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_ETHERNET)  # type: ignore

    @power_ethernet.setter
    @locked
    def power_ethernet(self, value: int) -> None:
        """
        The current ON/OFF state of the Ethernet module.

        Register: POWER_ETHERNET

        Address: 48003
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_ETHERNET, value)

    @property
    @locked
    def power_wifi(self) -> int:
        """
        The current ON/OFF state of the WiFi module.

        Register: POWER_WIFI

        Address: 48004
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_WIFI)  # type: ignore

    @power_wifi.setter
    @locked
    def power_wifi(self, value: int) -> None:
        """
        The current ON/OFF state of the WiFi module.

        Register: POWER_WIFI

        Address: 48004
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_WIFI, value)

    @property
    @locked
    def power_ain(self) -> int:
        """
        The current ON/OFF state of the analog input module.

        Register: POWER_AIN

        Address: 48005
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_AIN)  # type: ignore

    @power_ain.setter
    @locked
    def power_ain(self, value: int) -> None:
        """
        The current ON/OFF state of the analog input module.

        Register: POWER_AIN

        Address: 48005
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_AIN, value)

    @property
    @locked
    def power_led(self) -> int:
        """
        Sets the LED operation:
        0 = Off.
        Useful for lower power applications.
        1 = normal.
        2 = Lower power, LEDs will still blink but will normally be off.
        3 = Reserved.
        4 = Manual, in this mode the LEDs can be user controlled.

        Register: POWER_LED

        Address: 48006
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_LED)  # type: ignore

    @power_led.setter
    @locked
    def power_led(self, value: int) -> None:
        """
        Sets the LED operation:
        0 = Off.
        Useful for lower power applications.
        1 = normal.
        2 = Lower power, LEDs will still blink but will normally be off.
        3 = Reserved.
        4 = Manual, in this mode the LEDs can be user controlled.

        Register: POWER_LED

        Address: 48006
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_LED, value)

    @property
    @locked
    def power_ethernet_default(self) -> int:
        """
        The ON/OFF state of the Ethernet module after a power-cycle to the device. Provided to optionally reduce
        power consumption.

        Register: POWER_ETHERNET_DEFAULT

        Address: 48053
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_ETHERNET_DEFAULT)  # type: ignore

    @power_ethernet_default.setter
    @locked
    def power_ethernet_default(self, value: int) -> None:
        """
        The ON/OFF state of the Ethernet module after a power-cycle to the device. Provided to optionally reduce
        power consumption.

        Register: POWER_ETHERNET_DEFAULT

        Address: 48053
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_ETHERNET_DEFAULT, value)

    @property
    @locked
    def power_wifi_default(self) -> int:
        """
        The ON/OFF state of the WiFi module after a power-cycle to the device. Provided to optionally reduce power
        consumption.

        Register: POWER_WIFI_DEFAULT

        Address: 48054
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_WIFI_DEFAULT)  # type: ignore

    @power_wifi_default.setter
    @locked
    def power_wifi_default(self, value: int) -> None:
        """
        The ON/OFF state of the WiFi module after a power-cycle to the device. Provided to optionally reduce power
        consumption.

        Register: POWER_WIFI_DEFAULT

        Address: 48054
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_WIFI_DEFAULT, value)

    @property
    @locked
    def power_ain_default(self) -> int:
        """
        The ON/OFF state of the analog input module after a power-cycle to the device. Provided to optionally reduce
        power consumption.

        Register: POWER_AIN_DEFAULT

        Address: 48055
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_AIN_DEFAULT)  # type: ignore

    @power_ain_default.setter
    @locked
    def power_ain_default(self, value: int) -> None:
        """
        The ON/OFF state of the analog input module after a power-cycle to the device. Provided to optionally reduce
        power consumption.

        Register: POWER_AIN_DEFAULT

        Address: 48055
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_AIN_DEFAULT, value)

    @property
    @locked
    def power_led_default(self) -> int:
        """
        The ON/OFF state of the LEDs after a power-cycle to the device.

        Register: POWER_LED_DEFAULT

        Address: 48056
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.POWER_LED_DEFAULT)  # type: ignore

    @power_led_default.setter
    @locked
    def power_led_default(self, value: int) -> None:
        """
        The ON/OFF state of the LEDs after a power-cycle to the device.

        Register: POWER_LED_DEFAULT

        Address: 48056
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.POWER_LED_DEFAULT, value)

    @property
    @locked
    def io_config_check_for_factory(self) -> int:
        """
        Returns 1 if default settings are the same as factory settings.

        Register: IO_CONFIG_CHECK_FOR_FACTORY

        Address: 49000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CHECK_FOR_FACTORY)  # type: ignore

    @property
    @locked
    def io_config_set_default_to_current(self) -> int:
        """
        Write a 1 to cause new default (reboot/power-up) values to be saved to flash.
        Current values are retrieved and saved as the new defaults.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_DEFAULT_TO_CURRENT

        Address: 49002
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.IO_CONFIG_SET_DEFAULT_TO_CURRENT
        )

    @io_config_set_default_to_current.setter
    @locked
    def io_config_set_default_to_current(self, value: int) -> None:
        """
        Write a 1 to cause new default (reboot/power-up) values to be saved to flash.
        Current values are retrieved and saved as the new defaults.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_DEFAULT_TO_CURRENT

        Address: 49002
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.IO_CONFIG_SET_DEFAULT_TO_CURRENT, value)

    @property
    @locked
    def io_config_set_default_to_factory(self) -> int:
        """
        Write a 1 to cause new default (reboot/power-up) values to be saved to flash.
        Factory values are retrieved and saved as the new defaults.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_DEFAULT_TO_FACTORY

        Address: 49004
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.IO_CONFIG_SET_DEFAULT_TO_FACTORY
        )

    @io_config_set_default_to_factory.setter
    @locked
    def io_config_set_default_to_factory(self, value: int) -> None:
        """
        Write a 1 to cause new default (reboot/power-up) values to be saved to flash.
        Factory values are retrieved and saved as the new defaults.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_DEFAULT_TO_FACTORY

        Address: 49004
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.IO_CONFIG_SET_DEFAULT_TO_FACTORY, value)

    @property
    @locked
    def io_config_factory_pread(self) -> int:
        """
        The address in the factory configuration data which reads will start from.

        Register: IO_CONFIG_FACTORY_pREAD

        Address: 49006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_FACTORY_PREAD)  # type: ignore

    @io_config_factory_pread.setter
    @locked
    def io_config_factory_pread(self, value: int) -> None:
        """
        The address in the factory configuration data which reads will start from.

        Register: IO_CONFIG_FACTORY_pREAD

        Address: 49006
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.IO_CONFIG_FACTORY_PREAD, value)

    @property
    @locked
    def io_config_factory_read(self) -> int:
        """
        Data read from factory configuration.

        Register: IO_CONFIG_FACTORY_READ

        Address: 49008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_FACTORY_READ)  # type: ignore

    @property
    @locked
    def io_config_default_pread(self) -> int:
        """
        The address in the configuration data which reads will start from.

        Register: IO_CONFIG_DEFAULT_pREAD

        Address: 49010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_DEFAULT_PREAD)  # type: ignore

    @io_config_default_pread.setter
    @locked
    def io_config_default_pread(self, value: int) -> None:
        """
        The address in the configuration data which reads will start from.

        Register: IO_CONFIG_DEFAULT_pREAD

        Address: 49010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.IO_CONFIG_DEFAULT_PREAD, value)

    @property
    @locked
    def io_config_default_read(self) -> int:
        """
        Data read from the startup configuration.

        Register: IO_CONFIG_DEFAULT_READ

        Address: 49012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_DEFAULT_READ)  # type: ignore

    @property
    @locked
    def io_config_current_pread(self) -> int:
        """
        The address in the current configuration data which reads will start from.

        Register: IO_CONFIG_CURRENT_pREAD

        Address: 49014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CURRENT_PREAD)  # type: ignore

    @io_config_current_pread.setter
    @locked
    def io_config_current_pread(self, value: int) -> None:
        """
        The address in the current configuration data which reads will start from.

        Register: IO_CONFIG_CURRENT_pREAD

        Address: 49014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CURRENT_PREAD, value)

    @property
    @locked
    def io_config_current_read(self) -> int:
        """
        Data read from the current configuration.

        Register: IO_CONFIG_CURRENT_READ

        Address: 49016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CURRENT_READ)  # type: ignore

    @property
    @locked
    def io_config_check_for_default(self) -> int:
        """
        Returns 1 if the current configuration matches the default configuration.

        Register: IO_CONFIG_CHECK_FOR_DEFAULT

        Address: 49018
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CHECK_FOR_DEFAULT)  # type: ignore

    @property
    @locked
    def io_config_current_crc32(self) -> int:
        """
        Collects the current IO configuration and calculates of a CRC32.

        Register: IO_CONFIG_CURRENT_CRC32

        Address: 49020
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.IO_CONFIG_CURRENT_CRC32)  # type: ignore

    @property
    @locked
    def cleanse(self) -> int:
        """
        Writing 0x5317052E to this register will trigger a system cleanse.

        Register: CLEANSE

        Address: 49090
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.CLEANSE)  # type: ignore

    @cleanse.setter
    @locked
    def cleanse(self, value: int) -> None:
        """
        Writing 0x5317052E to this register will trigger a system cleanse.

        Register: CLEANSE

        Address: 49090
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.CLEANSE, value)

    @property
    @locked
    def core_timer(self) -> int:
        """
        Internal 32-bit system timer running at 1/2 core speed, thus normally 80M/2 => 40 MHz.

        Register: CORE_TIMER

        Address: 61520
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.CORE_TIMER)  # type: ignore

    @property
    @locked
    def system_timer_20hz(self) -> int:
        """
        Internal 32-bit system timer running at 20Hz.

        Register: SYSTEM_TIMER_20HZ

        Address: 61522
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.SYSTEM_TIMER_20HZ)  # type: ignore

    @property
    @locked
    def spc_frequency_out_enable_deprecated(self) -> int:
        """
        This has been deprecated in favor of DAC1_FREQUENCY_OUT_ENABLE. 0 = off, 1 = output 10 Hz signal on SPC.
        Note that stream uses SPC for a timing output, so if you use this while streaming you will get a lot more
        counts.

        Register: SPC_FREQUENCY_OUT_ENABLE_DEPRECATED

        Address: 61530
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.SPC_FREQUENCY_OUT_ENABLE_DEPRECATED
        )

    @spc_frequency_out_enable_deprecated.setter
    @locked
    def spc_frequency_out_enable_deprecated(self, value: int) -> None:
        """
        This has been deprecated in favor of DAC1_FREQUENCY_OUT_ENABLE. 0 = off, 1 = output 10 Hz signal on SPC.
        Note that stream uses SPC for a timing output, so if you use this while streaming you will get a lot more
        counts.

        Register: SPC_FREQUENCY_OUT_ENABLE_DEPRECATED

        Address: 61530
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.SPC_FREQUENCY_OUT_ENABLE_DEPRECATED, value)

    @property
    @locked
    def dac1_frequency_out_enable(self) -> int:
        """
        0 = off, 1 = output 10 Hz signal on DAC1. The signal will be a square wave with peaks of 0 and 3.3V. Note
        that writing to DAC1 or enabling a stream out which is targeting DAC1 will disable this feature.

        Register: DAC1_FREQUENCY_OUT_ENABLE

        Address: 61532
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DAC1_FREQUENCY_OUT_ENABLE)  # type: ignore

    @dac1_frequency_out_enable.setter
    @locked
    def dac1_frequency_out_enable(self, value: int) -> None:
        """
        0 = off, 1 = output 10 Hz signal on DAC1. The signal will be a square wave with peaks of 0 and 3.3V. Note
        that writing to DAC1 or enabling a stream out which is targeting DAC1 will disable this feature.

        Register: DAC1_FREQUENCY_OUT_ENABLE

        Address: 61532
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DAC1_FREQUENCY_OUT_ENABLE, value)

    @property
    @locked
    def wait_us_blocking(self) -> int:
        """
        Delays for x microseconds. Range is 0-100000. This operation is Blocking. While a blocking function is
        running no other registers can be read / written.

        Register: WAIT_US_BLOCKING

        Address: 61590
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WAIT_US_BLOCKING)  # type: ignore

    @wait_us_blocking.setter
    @locked
    def wait_us_blocking(self, value: int) -> None:
        """
        Delays for x microseconds. Range is 0-100000. This operation is Blocking. While a blocking function is
        running no other registers can be read / written.

        Register: WAIT_US_BLOCKING

        Address: 61590
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WAIT_US_BLOCKING, value)

    @property
    @locked
    def io_config_set_current_to_factory(self) -> int:
        """
        Write a 1 to set current values to factory configuration.
        The factory values are retrieved from flash and written to the current configuration registers.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_CURRENT_TO_FACTORY

        Address: 61990
        """
        return self.client.read_holding_register_ubint16(  # type: ignore
            RegisterAddresses.IO_CONFIG_SET_CURRENT_TO_FACTORY
        )

    @io_config_set_current_to_factory.setter
    @locked
    def io_config_set_current_to_factory(self, value: int) -> None:
        """
        Write a 1 to set current values to factory configuration.
        The factory values are retrieved from flash and written to the current configuration registers.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_CURRENT_TO_FACTORY

        Address: 61990
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.IO_CONFIG_SET_CURRENT_TO_FACTORY, value)

    @property
    @locked
    def io_config_set_current_to_default(self) -> int:
        """
        Write a 1 to set current values to default configuration.
        The default values are retrieved from flash and written to the current configuration registers, thus this
        behaves similar to reboot/power-up.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_CURRENT_TO_DEFAULT

        Address: 61991
        """
        return self.client.read_holding_register_ubint16(  # type: ignore
            RegisterAddresses.IO_CONFIG_SET_CURRENT_TO_DEFAULT
        )

    @io_config_set_current_to_default.setter
    @locked
    def io_config_set_current_to_default(self, value: int) -> None:
        """
        Write a 1 to set current values to default configuration.
        The default values are retrieved from flash and written to the current configuration registers, thus this
        behaves similar to reboot/power-up.
        Systems affected:
        AIN, DIO, DAC, AIN_EF, DIO_EF.

        Register: IO_CONFIG_SET_CURRENT_TO_DEFAULT

        Address: 61991
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.IO_CONFIG_SET_CURRENT_TO_DEFAULT, value)

    @property
    @locked
    def system_reboot(self) -> int:
        """
        Issues a device reboot. Must write 0x4C4Axxxx, where xxxx is number of 50ms ticks to wait before reboot. To
        reboot immediately write 0x4C4A0000 (d1279918080).

        Register: SYSTEM_REBOOT

        Address: 61998
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.SYSTEM_REBOOT)  # type: ignore

    @system_reboot.setter
    @locked
    def system_reboot(self, value: int) -> None:
        """
        Issues a device reboot. Must write 0x4C4Axxxx, where xxxx is number of 50ms ticks to wait before reboot. To
        reboot immediately write 0x4C4A0000 (d1279918080).

        Register: SYSTEM_REBOOT

        Address: 61998
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.SYSTEM_REBOOT, value)

    @property
    @locked
    def temperature_air_k(self) -> float:
        """
        Returns the estimated ambient air temperature just outside of the device in its red plastic enclosure.
        This register is equal to TEMPERATURE_DEVICE_K - 4.3.
        If Ethernet and/or WiFi is enabled, subtract an extra 0.6 for each.

        Register: TEMPERATURE_AIR_K

        Address: 60050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE_AIR_K)  # type: ignore

    @property
    @locked
    def temperature_device_k(self) -> float:
        """
        Takes a reading from the internal temperature sensor using range=+/-10V and resolution=8, and applies the
        formula Volts*-92.6+467.6 to return kelvins.

        Register: TEMPERATURE_DEVICE_K

        Address: 60052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEMPERATURE_DEVICE_K)  # type: ignore

    @property
    @locked
    def calibration_constants_status(self) -> float:
        """
        T8 Only. Returns the status of the calibration data saved in flash. 1 = calibration looks good. 0 =
        calibration invalid.

        Register: CALIBRATION_CONSTANTS_STATUS

        Address: 60091
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.CALIBRATION_CONSTANTS_STATUS)  # type: ignore

    @property
    @locked
    def calibration_source(self) -> int:
        """
        T8 Only. Sets and reads the source of the current set of calibration constants. 0 = no calibration. 1 =
        binary calibration, 2 = Calibration set from Flash, 3 = Nominal Calibration. When setting to 2, the constants
        will be checked and if invalid 1 will be used instead.

        Register: CALIBRATION_SOURCE

        Address: 60092
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.CALIBRATION_SOURCE)  # type: ignore

    @calibration_source.setter
    @locked
    def calibration_source(self, value: int) -> None:
        """
        T8 Only. Sets and reads the source of the current set of calibration constants. 0 = no calibration. 1 =
        binary calibration, 2 = Calibration set from Flash, 3 = Nominal Calibration. When setting to 2, the constants
        will be checked and if invalid 1 will be used instead.

        Register: CALIBRATION_SOURCE

        Address: 60092
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.CALIBRATION_SOURCE, value)

    @property
    @locked
    def sntp_update_interval(self) -> int:
        """
        Sets the SNTP retry time in seconds. A value of zero will disable SNTP(0=default). Values must be 10 or
        greater.

        Register: SNTP_UPDATE_INTERVAL

        Address: 49702
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.SNTP_UPDATE_INTERVAL)  # type: ignore

    @sntp_update_interval.setter
    @locked
    def sntp_update_interval(self, value: int) -> None:
        """
        Sets the SNTP retry time in seconds. A value of zero will disable SNTP(0=default). Values must be 10 or
        greater.

        Register: SNTP_UPDATE_INTERVAL

        Address: 49702
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.SNTP_UPDATE_INTERVAL, value)

    @property
    @locked
    def rtc_time_s(self) -> int:
        """
        Read the current time in seconds since Jan, 1970, aka Epoch or Unix time. This value is calculated from the
        80 MHz crystal, not the RTC 32 kHz crystal. Non-pro devices do not have a real time clock, so the reported time
        is either seconds since device startup or the time reported by the network time protocol over Ethernet. Pro
        devices have a real time clock that will be used to initialize this register at startup, the time can then be
        updated by NTP.

        Register: RTC_TIME_S

        Address: 61500
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.RTC_TIME_S)  # type: ignore

    @property
    @locked
    def system_counter_10khz(self) -> int:
        """
        A 10 kHz counter synchronized to RTC_TIME_S. This register can be appended to RTC_TIME_S as the decimal
        portion to get 0.1 ms resolution.

        Register: SYSTEM_COUNTER_10KHZ

        Address: 61502
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.SYSTEM_COUNTER_10KHZ)  # type: ignore

    @property
    @locked
    def rtc_set_time_s(self) -> int:
        """
        Write a new timestamp to the RTC in seconds since Jan, 1970, aka Epoch or Unix timestamp.

        Register: RTC_SET_TIME_S

        Address: 61504
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.RTC_SET_TIME_S)  # type: ignore

    @rtc_set_time_s.setter
    @locked
    def rtc_set_time_s(self, value: int) -> None:
        """
        Write a new timestamp to the RTC in seconds since Jan, 1970, aka Epoch or Unix timestamp.

        Register: RTC_SET_TIME_S

        Address: 61504
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.RTC_SET_TIME_S, value)

    @property
    @locked
    def rtc_set_time_sntp(self) -> int:
        """
        Write any value to instruct the T7 to update its clock from a SNTP server. Requires that
        SNTP_UPDATE_INTERVAL is non-zero.

        Register: RTC_SET_TIME_SNTP

        Address: 61506
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.RTC_SET_TIME_SNTP)  # type: ignore

    @rtc_set_time_sntp.setter
    @locked
    def rtc_set_time_sntp(self, value: int) -> None:
        """
        Write any value to instruct the T7 to update its clock from a SNTP server. Requires that
        SNTP_UPDATE_INTERVAL is non-zero.

        Register: RTC_SET_TIME_SNTP

        Address: 61506
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.RTC_SET_TIME_SNTP, value)

    @property
    @locked
    def rtc_time_calendar(self) -> int:
        """
        Read six consecutive addresses of type UINT16, starting with this address. The result will be in year,
        month, day, hour, minute, second calendar format. i.e. [2014, 10, 21, 18, 55, 35]

        Register: RTC_TIME_CALENDAR

        Address: 61510
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.RTC_TIME_CALENDAR)  # type: ignore

    @property
    @locked
    def ethernet_ip(self) -> str:
        """
        Read the current IP address of wired Ethernet.

        Register: ETHERNET_IP

        Address: 49100
        """
        return self.client.read_holding_register_data(RegisterAddresses.ETHERNET_IP, IPv4Address).get()  # type: ignore

    @property
    @locked
    def ethernet_subnet(self) -> str:
        """
        Read the current subnet of wired Ethernet.

        Register: ETHERNET_SUBNET

        Address: 49102
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_SUBNET, IPv4Address
        ).get()

    @property
    @locked
    def ethernet_gateway(self) -> str:
        """
        Read the current gateway of wired Ethernet.

        Register: ETHERNET_GATEWAY

        Address: 49104
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_GATEWAY, IPv4Address
        ).get()

    @property
    @locked
    def ethernet_dns(self) -> str:
        """
        Read the current DNS of wired Ethernet.

        Register: ETHERNET_DNS

        Address: 49106
        """
        return self.client.read_holding_register_data(RegisterAddresses.ETHERNET_DNS, IPv4Address).get()  # type: ignore

    @property
    @locked
    def ethernet_altdns(self) -> str:
        """
        Read the current Alt DNS of wired Ethernet.

        Register: ETHERNET_ALTDNS

        Address: 49108
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_ALTDNS, IPv4Address
        ).get()

    @property
    @locked
    def ethernet_dhcp_enable(self) -> int:
        """
        Read the current Enabled/Disabled state of Ethernet DHCP.

        Register: ETHERNET_DHCP_ENABLE

        Address: 49110
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ETHERNET_DHCP_ENABLE)  # type: ignore

    @property
    @locked
    def ethernet_udp_discovery_only(self) -> int:
        """
        Restricts which Modbus operations are allowed over UDP. When set to 1, only the registers needed for
        discovering units can be read and any other operation will throw an error.

        Register: ETHERNET_UDP_DISCOVERY_ONLY

        Address: 49115
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ETHERNET_UDP_DISCOVERY_ONLY)  # type: ignore

    @property
    @locked
    def ethernet_ip_default(self) -> str:
        """
        The IP address of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_IP_DEFAULT

        Address: 49150
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_IP_DEFAULT, IPv4Address
        ).get()

    @ethernet_ip_default.setter
    @locked
    def ethernet_ip_default(self, value: str) -> None:
        """
        The IP address of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_IP_DEFAULT

        Address: 49150
        """
        self.client.write_holding_register_data(RegisterAddresses.ETHERNET_IP_DEFAULT, IPv4Address(value))

    @property
    @locked
    def ethernet_subnet_default(self) -> str:
        """
        The subnet of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_SUBNET_DEFAULT

        Address: 49152
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_SUBNET_DEFAULT, IPv4Address
        ).get()

    @ethernet_subnet_default.setter
    @locked
    def ethernet_subnet_default(self, value: str) -> None:
        """
        The subnet of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_SUBNET_DEFAULT

        Address: 49152
        """
        self.client.write_holding_register_data(RegisterAddresses.ETHERNET_SUBNET_DEFAULT, IPv4Address(value))

    @property
    @locked
    def ethernet_gateway_default(self) -> str:
        """
        The gateway of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_GATEWAY_DEFAULT

        Address: 49154
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_GATEWAY_DEFAULT, IPv4Address
        ).get()

    @ethernet_gateway_default.setter
    @locked
    def ethernet_gateway_default(self, value: str) -> None:
        """
        The gateway of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_GATEWAY_DEFAULT

        Address: 49154
        """
        self.client.write_holding_register_data(RegisterAddresses.ETHERNET_GATEWAY_DEFAULT, IPv4Address(value))

    @property
    @locked
    def ethernet_dns_default(self) -> str:
        """
        The DNS of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_DNS_DEFAULT

        Address: 49156
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_DNS_DEFAULT, IPv4Address
        ).get()

    @ethernet_dns_default.setter
    @locked
    def ethernet_dns_default(self, value: str) -> None:
        """
        The DNS of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_DNS_DEFAULT

        Address: 49156
        """
        self.client.write_holding_register_data(RegisterAddresses.ETHERNET_DNS_DEFAULT, IPv4Address(value))

    @property
    @locked
    def ethernet_altdns_default(self) -> str:
        """
        The Alt DNS of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_ALTDNS_DEFAULT

        Address: 49158
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.ETHERNET_ALTDNS_DEFAULT, IPv4Address
        ).get()

    @ethernet_altdns_default.setter
    @locked
    def ethernet_altdns_default(self, value: str) -> None:
        """
        The Alt DNS of wired Ethernet after a power-cycle to the device.

        Register: ETHERNET_ALTDNS_DEFAULT

        Address: 49158
        """
        self.client.write_holding_register_data(RegisterAddresses.ETHERNET_ALTDNS_DEFAULT, IPv4Address(value))

    @property
    @locked
    def ethernet_dhcp_enable_default(self) -> int:
        """
        The Enabled/Disabled state of Ethernet DHCP after a power-cycle to the device.

        Register: ETHERNET_DHCP_ENABLE_DEFAULT

        Address: 49160
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.ETHERNET_DHCP_ENABLE_DEFAULT)  # type: ignore

    @ethernet_dhcp_enable_default.setter
    @locked
    def ethernet_dhcp_enable_default(self, value: int) -> None:
        """
        The Enabled/Disabled state of Ethernet DHCP after a power-cycle to the device.

        Register: ETHERNET_DHCP_ENABLE_DEFAULT

        Address: 49160
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.ETHERNET_DHCP_ENABLE_DEFAULT, value)

    @property
    @locked
    def ethernet_udp_discovery_only_default(self) -> int:
        """
        The Enabled/Disabled state of ETHERNET_UDP_DISCOVERY_ONLY after a power-cycle to the device.

        Register: ETHERNET_UDP_DISCOVERY_ONLY_DEFAULT

        Address: 49165
        """
        return self.client.read_holding_register_ubint16(  # type: ignore
            RegisterAddresses.ETHERNET_UDP_DISCOVERY_ONLY_DEFAULT
        )

    @property
    @locked
    def ethernet_apply_settings(self) -> int:
        """
        Writing 1 to this register power-cycles Ethernet. It tells the device to waits 1s before turning off
        Ethernet and then 500ms before turning it back on.

        Register: ETHERNET_APPLY_SETTINGS

        Address: 49190
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.ETHERNET_APPLY_SETTINGS)  # type: ignore

    @ethernet_apply_settings.setter
    @locked
    def ethernet_apply_settings(self, value: int) -> None:
        """
        Writing 1 to this register power-cycles Ethernet. It tells the device to waits 1s before turning off
        Ethernet and then 500ms before turning it back on.

        Register: ETHERNET_APPLY_SETTINGS

        Address: 49190
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.ETHERNET_APPLY_SETTINGS, value)

    @property
    @locked
    def wifi_ip(self) -> str:
        """
        Read the current IP address of WiFi.

        Register: WIFI_IP

        Address: 49200
        """
        return self.client.read_holding_register_data(RegisterAddresses.WIFI_IP, IPv4Address).get()  # type: ignore

    @property
    @locked
    def wifi_subnet(self) -> str:
        """
        Read the current subnet of WiFi.

        Register: WIFI_SUBNET

        Address: 49202
        """
        return self.client.read_holding_register_data(RegisterAddresses.WIFI_SUBNET, IPv4Address).get()  # type: ignore

    @property
    @locked
    def wifi_gateway(self) -> str:
        """
        Read the current gateway of WiFi.

        Register: WIFI_GATEWAY

        Address: 49204
        """
        return self.client.read_holding_register_data(RegisterAddresses.WIFI_GATEWAY, IPv4Address).get()  # type: ignore

    @property
    @locked
    def wifi_dhcp_enable(self) -> int:
        """
        Read the current Enabled/Disabled state of WiFi DHCP.

        Register: WIFI_DHCP_ENABLE

        Address: 49210
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.WIFI_DHCP_ENABLE)  # type: ignore

    @property
    @locked
    def wifi_udp_discovery_only(self) -> int:
        """
        Restricts which Modbus operations are allowed over UDP. When set to 1, only the registers needed for
        discovering units can be read and any other operation will throw an error.

        Register: WIFI_UDP_DISCOVERY_ONLY

        Address: 49215
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.WIFI_UDP_DISCOVERY_ONLY)  # type: ignore

    @property
    @locked
    def wifi_ssid(self) -> str:
        """
        Read the current SSID (network name) of WiFi.

        Register: WIFI_SSID

        Address: 49300
        """
        return self.client.read_holding_register_string(RegisterAddresses.WIFI_SSID, length=50).decode(  # type: ignore
            errors="ignore"
        )

    @property
    @locked
    def wifi_ip_default(self) -> str:
        """
        The new IP address of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_IP_DEFAULT

        Address: 49250
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.WIFI_IP_DEFAULT, IPv4Address
        ).get()

    @wifi_ip_default.setter
    @locked
    def wifi_ip_default(self, value: str) -> None:
        """
        The new IP address of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_IP_DEFAULT

        Address: 49250
        """
        self.client.write_holding_register_data(RegisterAddresses.WIFI_IP_DEFAULT, IPv4Address(value))

    @property
    @locked
    def wifi_subnet_default(self) -> str:
        """
        The new subnet of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_SUBNET_DEFAULT

        Address: 49252
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.WIFI_SUBNET_DEFAULT, IPv4Address
        ).get()

    @wifi_subnet_default.setter
    @locked
    def wifi_subnet_default(self, value: str) -> None:
        """
        The new subnet of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_SUBNET_DEFAULT

        Address: 49252
        """
        self.client.write_holding_register_data(RegisterAddresses.WIFI_SUBNET_DEFAULT, IPv4Address(value))

    @property
    @locked
    def wifi_gateway_default(self) -> str:
        """
        The new gateway of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_GATEWAY_DEFAULT

        Address: 49254
        """
        return self.client.read_holding_register_data(  # type: ignore
            RegisterAddresses.WIFI_GATEWAY_DEFAULT, IPv4Address
        ).get()

    @wifi_gateway_default.setter
    @locked
    def wifi_gateway_default(self, value: str) -> None:
        """
        The new gateway of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_GATEWAY_DEFAULT

        Address: 49254
        """
        self.client.write_holding_register_data(RegisterAddresses.WIFI_GATEWAY_DEFAULT, IPv4Address(value))

    @property
    @locked
    def wifi_dhcp_enable_default(self) -> int:
        """
        The new Enabled/Disabled state of WiFi DHCP. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_DHCP_ENABLE_DEFAULT

        Address: 49260
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.WIFI_DHCP_ENABLE_DEFAULT)  # type: ignore

    @wifi_dhcp_enable_default.setter
    @locked
    def wifi_dhcp_enable_default(self, value: int) -> None:
        """
        The new Enabled/Disabled state of WiFi DHCP. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_DHCP_ENABLE_DEFAULT

        Address: 49260
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.WIFI_DHCP_ENABLE_DEFAULT, value)

    @property
    @locked
    def wifi_udp_discovery_only_default(self) -> int:
        """
        The Enabled/Disabled state of WIFI_UDP_DISCOVERY_ONLY after a power-cycle to the device.

        Register: WIFI_UDP_DISCOVERY_ONLY_DEFAULT

        Address: 49265
        """
        return self.client.read_holding_register_ubint16(  # type: ignore
            RegisterAddresses.WIFI_UDP_DISCOVERY_ONLY_DEFAULT
        )

    @property
    @locked
    def wifi_ssid_default(self) -> str:
        """
        The new SSID (network name) of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_SSID_DEFAULT

        Address: 49325
        """
        return self.client.read_holding_register_string(  # type: ignore
            RegisterAddresses.WIFI_SSID_DEFAULT, length=50
        ).decode(errors="ignore")

    @wifi_ssid_default.setter
    @locked
    def wifi_ssid_default(self, value: str) -> None:
        """
        The new SSID (network name) of WiFi. Use WIFI_APPLY_SETTINGS.

        Register: WIFI_SSID_DEFAULT

        Address: 49325
        """
        self.client.write_holding_register_string(
            RegisterAddresses.WIFI_SSID_DEFAULT, value.encode(errors="ignore"), length=50
        )

    @property
    @locked
    def wifi_password_default(self) -> str:
        """
        Write the password for the WiFi network, then use WIFI_APPLY_SETTINGS.

        Register: WIFI_PASSWORD_DEFAULT

        Address: 49350
        """
        return self.client.read_holding_register_string(  # type: ignore
            RegisterAddresses.WIFI_PASSWORD_DEFAULT, length=50
        ).decode(errors="ignore")

    @wifi_password_default.setter
    @locked
    def wifi_password_default(self, value: str) -> None:
        """
        Write the password for the WiFi network, then use WIFI_APPLY_SETTINGS.

        Register: WIFI_PASSWORD_DEFAULT

        Address: 49350
        """
        self.client.write_holding_register_string(
            RegisterAddresses.WIFI_PASSWORD_DEFAULT, value.encode(errors="ignore"), length=50
        )

    @property
    @locked
    def wifi_apply_settings(self) -> int:
        """
        Apply all new WiFi settings:
        IP, Subnet, Gateway, DHCP, SSID, Password.
        1=Apply

        Register: WIFI_APPLY_SETTINGS

        Address: 49400
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WIFI_APPLY_SETTINGS)  # type: ignore

    @wifi_apply_settings.setter
    @locked
    def wifi_apply_settings(self, value: int) -> None:
        """
        Apply all new WiFi settings:
        IP, Subnet, Gateway, DHCP, SSID, Password.
        1=Apply

        Register: WIFI_APPLY_SETTINGS

        Address: 49400
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WIFI_APPLY_SETTINGS, value)

    @property
    @locked
    def wifi_firmware_update_to_versionx(self) -> float:
        """
        Start an update by using USB or Ethernet to write the desired version to update to.

        Register: WIFI_FIRMWARE_UPDATE_TO_VERSIONX

        Address: 49402
        """
        return self.client.read_holding_register_bfloat(  # type: ignore
            RegisterAddresses.WIFI_FIRMWARE_UPDATE_TO_VERSIONX
        )

    @wifi_firmware_update_to_versionx.setter
    @locked
    def wifi_firmware_update_to_versionx(self, value: float) -> None:
        """
        Start an update by using USB or Ethernet to write the desired version to update to.

        Register: WIFI_FIRMWARE_UPDATE_TO_VERSIONX

        Address: 49402
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.WIFI_FIRMWARE_UPDATE_TO_VERSIONX, value)

    @property
    @locked
    def wifi_status(self) -> int:
        """
        Status Codes:
        ASSOCIATED = 2900, ASSOCIATING = 2901, ASSOCIATION_FAILED = 2902, UNPOWERED = 2903, BOOTING = 2904,
        START_FAILED = 2905, APPLYING_SETTINGS = 2906, DHCP_STARTED = 2907, OTHER = 2909

        Register: WIFI_STATUS

        Address: 49450
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WIFI_STATUS)  # type: ignore

    @property
    @locked
    def wifi_rssi(self) -> float:
        """
        WiFi RSSI (signal strength). Typical values are -40 for very good, and -75 for very weak. The T7
        microcontroller only gets a new RSSI value from the WiFi module when WiFi communication occurs.

        Register: WIFI_RSSI

        Address: 49452
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.WIFI_RSSI)  # type: ignore

    @property
    @locked
    def wifi_firmware_update_status(self) -> int:
        """
        CONFIGURING = 2920, IN_PROGRESS = 2921, REBOOTING = 2923, UPDATE_SUCCESS = 2924, UPDATE_FAILED = 2925.

        Register: WIFI_FIRMWARE_UPDATE_STATUS

        Address: 49454
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WIFI_FIRMWARE_UPDATE_STATUS)  # type: ignore

    @property
    @locked
    def ain0_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN0_BINARY (AIN#(0:7)_BINARY@0)

        Address: 50000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_BINARY)  # type: ignore

    @property
    @locked
    def ain1_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN1_BINARY (AIN#(0:7)_BINARY@1)

        Address: 50002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_BINARY)  # type: ignore

    @property
    @locked
    def ain2_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN2_BINARY (AIN#(0:7)_BINARY@2)

        Address: 50004
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_BINARY)  # type: ignore

    @property
    @locked
    def ain3_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN3_BINARY (AIN#(0:7)_BINARY@3)

        Address: 50006
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_BINARY)  # type: ignore

    @property
    @locked
    def ain4_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN4_BINARY (AIN#(0:7)_BINARY@4)

        Address: 50008
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_BINARY)  # type: ignore

    @property
    @locked
    def ain5_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN5_BINARY (AIN#(0:7)_BINARY@5)

        Address: 50010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_BINARY)  # type: ignore

    @property
    @locked
    def ain6_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN6_BINARY (AIN#(0:7)_BINARY@6)

        Address: 50012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_BINARY)  # type: ignore

    @property
    @locked
    def ain7_binary(self) -> int:
        """
        Returns the 24-bit binary representation of the specified analog input. If you prefer 16-bit representation,
        simply divide this by 256. This is for command-response only. Stream always returns binary and LJM applies cal
        constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN7_BINARY (AIN#(0:7)_BINARY@7)

        Address: 50014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_BINARY)  # type: ignore

    @property
    @locked
    def temperature0_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE0_BINARY (TEMPERATURE#(0:7)_BINARY@0)

        Address: 50600
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE0_BINARY)  # type: ignore

    @property
    @locked
    def temperature1_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE1_BINARY (TEMPERATURE#(0:7)_BINARY@1)

        Address: 50602
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE1_BINARY)  # type: ignore

    @property
    @locked
    def temperature2_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE2_BINARY (TEMPERATURE#(0:7)_BINARY@2)

        Address: 50604
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE2_BINARY)  # type: ignore

    @property
    @locked
    def temperature3_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE3_BINARY (TEMPERATURE#(0:7)_BINARY@3)

        Address: 50606
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE3_BINARY)  # type: ignore

    @property
    @locked
    def temperature4_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE4_BINARY (TEMPERATURE#(0:7)_BINARY@4)

        Address: 50608
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE4_BINARY)  # type: ignore

    @property
    @locked
    def temperature5_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE5_BINARY (TEMPERATURE#(0:7)_BINARY@5)

        Address: 50610
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE5_BINARY)  # type: ignore

    @property
    @locked
    def temperature6_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE6_BINARY (TEMPERATURE#(0:7)_BINARY@6)

        Address: 50612
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE6_BINARY)  # type: ignore

    @property
    @locked
    def temperature7_binary(self) -> int:
        """
        T8 Only. Returns the 24-bit binary representation of the voltage output of the temperature sensor for the
        specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for command-
        response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE7_BINARY (TEMPERATURE#(0:7)_BINARY@7)

        Address: 50614
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE7_BINARY)  # type: ignore

    @property
    @locked
    def ain0_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN0_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@0)

        Address: 50650
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN0_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain1_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN1_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@1)

        Address: 50652
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN1_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain2_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN2_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@2)

        Address: 50654
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN2_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain3_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN3_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@3)

        Address: 50656
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN3_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain4_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN4_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@4)

        Address: 50658
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN4_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain5_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN5_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@5)

        Address: 50660
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN5_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain6_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN6_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@6)

        Address: 50662
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN6_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def ain7_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the specified analog input. If you prefer 16-bit
        representation, simply divide this by 256. This is for command-response only. Stream always returns binary and
        LJM applies cal constants, so the LJM config flag LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: AIN7_BINARY_CAPTURE (AIN#(0:7)_BINARY_CAPTURE@7)

        Address: 50664
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.AIN7_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature0_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE0_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@0)

        Address: 50700
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE0_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature1_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE1_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@1)

        Address: 50702
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE1_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature2_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE2_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@2)

        Address: 50704
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE2_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature3_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE3_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@3)

        Address: 50706
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE3_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature4_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE4_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@4)

        Address: 50708
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE4_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature5_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE5_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@5)

        Address: 50710
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE5_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature6_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE6_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@6)

        Address: 50712
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE6_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def temperature7_binary_capture(self) -> int:
        """
        T8 Only. Returns the saved 24-bit binary representation of the voltage output of the temperature sensor for
        the specified analog input. If you prefer 16-bit representation, simply divide this by 256. This is for
        command-response only. Stream always returns binary and LJM applies cal constants, so the LJM config flag
        LJM_STREAM_AIN_BINARY is used to get binary values.

        Register: TEMPERATURE7_BINARY_CAPTURE (TEMPERATURE#(0:7)_BINARY_CAPTURE@7)

        Address: 50714
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEMPERATURE7_BINARY_CAPTURE)  # type: ignore

    @property
    @locked
    def dac0_binary(self) -> int:
        """
        Writes binary values to the DACs. Binary values are 16-bit. If the DAC's resolution is less than 16 then the
        lower bits are ignored. 0 = lowest output, 65535 = highest output.

        Register: DAC0_BINARY (DAC#(0:1)_BINARY@0)

        Address: 51000
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DAC0_BINARY)  # type: ignore

    @dac0_binary.setter
    @locked
    def dac0_binary(self, value: int) -> None:
        """
        Writes binary values to the DACs. Binary values are 16-bit. If the DAC's resolution is less than 16 then the
        lower bits are ignored. 0 = lowest output, 65535 = highest output.

        Register: DAC0_BINARY (DAC#(0:1)_BINARY@0)

        Address: 51000
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DAC0_BINARY, value)

    @property
    @locked
    def dac1_binary(self) -> int:
        """
        Writes binary values to the DACs. Binary values are 16-bit. If the DAC's resolution is less than 16 then the
        lower bits are ignored. 0 = lowest output, 65535 = highest output.

        Register: DAC1_BINARY (DAC#(0:1)_BINARY@1)

        Address: 51002
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DAC1_BINARY)  # type: ignore

    @dac1_binary.setter
    @locked
    def dac1_binary(self, value: int) -> None:
        """
        Writes binary values to the DACs. Binary values are 16-bit. If the DAC's resolution is less than 16 then the
        lower bits are ignored. 0 = lowest output, 65535 = highest output.

        Register: DAC1_BINARY (DAC#(0:1)_BINARY@1)

        Address: 51002
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.DAC1_BINARY, value)

    @property
    @locked
    def user_ram0_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_F32 (USER_RAM#(0:39)_F32@0)

        Address: 46000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM0_F32)  # type: ignore

    @user_ram0_f32.setter
    @locked
    def user_ram0_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_F32 (USER_RAM#(0:39)_F32@0)

        Address: 46000
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM0_F32, value)

    @property
    @locked
    def user_ram1_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_F32 (USER_RAM#(0:39)_F32@1)

        Address: 46002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM1_F32)  # type: ignore

    @user_ram1_f32.setter
    @locked
    def user_ram1_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_F32 (USER_RAM#(0:39)_F32@1)

        Address: 46002
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM1_F32, value)

    @property
    @locked
    def user_ram2_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_F32 (USER_RAM#(0:39)_F32@2)

        Address: 46004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM2_F32)  # type: ignore

    @user_ram2_f32.setter
    @locked
    def user_ram2_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_F32 (USER_RAM#(0:39)_F32@2)

        Address: 46004
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM2_F32, value)

    @property
    @locked
    def user_ram3_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_F32 (USER_RAM#(0:39)_F32@3)

        Address: 46006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM3_F32)  # type: ignore

    @user_ram3_f32.setter
    @locked
    def user_ram3_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_F32 (USER_RAM#(0:39)_F32@3)

        Address: 46006
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM3_F32, value)

    @property
    @locked
    def user_ram4_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_F32 (USER_RAM#(0:39)_F32@4)

        Address: 46008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM4_F32)  # type: ignore

    @user_ram4_f32.setter
    @locked
    def user_ram4_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_F32 (USER_RAM#(0:39)_F32@4)

        Address: 46008
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM4_F32, value)

    @property
    @locked
    def user_ram5_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_F32 (USER_RAM#(0:39)_F32@5)

        Address: 46010
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM5_F32)  # type: ignore

    @user_ram5_f32.setter
    @locked
    def user_ram5_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_F32 (USER_RAM#(0:39)_F32@5)

        Address: 46010
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM5_F32, value)

    @property
    @locked
    def user_ram6_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_F32 (USER_RAM#(0:39)_F32@6)

        Address: 46012
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM6_F32)  # type: ignore

    @user_ram6_f32.setter
    @locked
    def user_ram6_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_F32 (USER_RAM#(0:39)_F32@6)

        Address: 46012
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM6_F32, value)

    @property
    @locked
    def user_ram7_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_F32 (USER_RAM#(0:39)_F32@7)

        Address: 46014
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM7_F32)  # type: ignore

    @user_ram7_f32.setter
    @locked
    def user_ram7_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_F32 (USER_RAM#(0:39)_F32@7)

        Address: 46014
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM7_F32, value)

    @property
    @locked
    def user_ram8_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_F32 (USER_RAM#(0:39)_F32@8)

        Address: 46016
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM8_F32)  # type: ignore

    @user_ram8_f32.setter
    @locked
    def user_ram8_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_F32 (USER_RAM#(0:39)_F32@8)

        Address: 46016
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM8_F32, value)

    @property
    @locked
    def user_ram9_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_F32 (USER_RAM#(0:39)_F32@9)

        Address: 46018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM9_F32)  # type: ignore

    @user_ram9_f32.setter
    @locked
    def user_ram9_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_F32 (USER_RAM#(0:39)_F32@9)

        Address: 46018
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM9_F32, value)

    @property
    @locked
    def user_ram10_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_F32 (USER_RAM#(0:39)_F32@10)

        Address: 46020
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM10_F32)  # type: ignore

    @user_ram10_f32.setter
    @locked
    def user_ram10_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_F32 (USER_RAM#(0:39)_F32@10)

        Address: 46020
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM10_F32, value)

    @property
    @locked
    def user_ram11_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_F32 (USER_RAM#(0:39)_F32@11)

        Address: 46022
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM11_F32)  # type: ignore

    @user_ram11_f32.setter
    @locked
    def user_ram11_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_F32 (USER_RAM#(0:39)_F32@11)

        Address: 46022
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM11_F32, value)

    @property
    @locked
    def user_ram12_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_F32 (USER_RAM#(0:39)_F32@12)

        Address: 46024
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM12_F32)  # type: ignore

    @user_ram12_f32.setter
    @locked
    def user_ram12_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_F32 (USER_RAM#(0:39)_F32@12)

        Address: 46024
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM12_F32, value)

    @property
    @locked
    def user_ram13_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_F32 (USER_RAM#(0:39)_F32@13)

        Address: 46026
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM13_F32)  # type: ignore

    @user_ram13_f32.setter
    @locked
    def user_ram13_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_F32 (USER_RAM#(0:39)_F32@13)

        Address: 46026
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM13_F32, value)

    @property
    @locked
    def user_ram14_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_F32 (USER_RAM#(0:39)_F32@14)

        Address: 46028
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM14_F32)  # type: ignore

    @user_ram14_f32.setter
    @locked
    def user_ram14_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_F32 (USER_RAM#(0:39)_F32@14)

        Address: 46028
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM14_F32, value)

    @property
    @locked
    def user_ram15_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_F32 (USER_RAM#(0:39)_F32@15)

        Address: 46030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM15_F32)  # type: ignore

    @user_ram15_f32.setter
    @locked
    def user_ram15_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_F32 (USER_RAM#(0:39)_F32@15)

        Address: 46030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM15_F32, value)

    @property
    @locked
    def user_ram16_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_F32 (USER_RAM#(0:39)_F32@16)

        Address: 46032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM16_F32)  # type: ignore

    @user_ram16_f32.setter
    @locked
    def user_ram16_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_F32 (USER_RAM#(0:39)_F32@16)

        Address: 46032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM16_F32, value)

    @property
    @locked
    def user_ram17_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_F32 (USER_RAM#(0:39)_F32@17)

        Address: 46034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM17_F32)  # type: ignore

    @user_ram17_f32.setter
    @locked
    def user_ram17_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_F32 (USER_RAM#(0:39)_F32@17)

        Address: 46034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM17_F32, value)

    @property
    @locked
    def user_ram18_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_F32 (USER_RAM#(0:39)_F32@18)

        Address: 46036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM18_F32)  # type: ignore

    @user_ram18_f32.setter
    @locked
    def user_ram18_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_F32 (USER_RAM#(0:39)_F32@18)

        Address: 46036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM18_F32, value)

    @property
    @locked
    def user_ram19_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_F32 (USER_RAM#(0:39)_F32@19)

        Address: 46038
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM19_F32)  # type: ignore

    @user_ram19_f32.setter
    @locked
    def user_ram19_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_F32 (USER_RAM#(0:39)_F32@19)

        Address: 46038
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM19_F32, value)

    @property
    @locked
    def user_ram20_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM20_F32 (USER_RAM#(0:39)_F32@20)

        Address: 46040
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM20_F32)  # type: ignore

    @user_ram20_f32.setter
    @locked
    def user_ram20_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM20_F32 (USER_RAM#(0:39)_F32@20)

        Address: 46040
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM20_F32, value)

    @property
    @locked
    def user_ram21_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM21_F32 (USER_RAM#(0:39)_F32@21)

        Address: 46042
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM21_F32)  # type: ignore

    @user_ram21_f32.setter
    @locked
    def user_ram21_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM21_F32 (USER_RAM#(0:39)_F32@21)

        Address: 46042
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM21_F32, value)

    @property
    @locked
    def user_ram22_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM22_F32 (USER_RAM#(0:39)_F32@22)

        Address: 46044
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM22_F32)  # type: ignore

    @user_ram22_f32.setter
    @locked
    def user_ram22_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM22_F32 (USER_RAM#(0:39)_F32@22)

        Address: 46044
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM22_F32, value)

    @property
    @locked
    def user_ram23_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM23_F32 (USER_RAM#(0:39)_F32@23)

        Address: 46046
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM23_F32)  # type: ignore

    @user_ram23_f32.setter
    @locked
    def user_ram23_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM23_F32 (USER_RAM#(0:39)_F32@23)

        Address: 46046
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM23_F32, value)

    @property
    @locked
    def user_ram24_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM24_F32 (USER_RAM#(0:39)_F32@24)

        Address: 46048
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM24_F32)  # type: ignore

    @user_ram24_f32.setter
    @locked
    def user_ram24_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM24_F32 (USER_RAM#(0:39)_F32@24)

        Address: 46048
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM24_F32, value)

    @property
    @locked
    def user_ram25_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM25_F32 (USER_RAM#(0:39)_F32@25)

        Address: 46050
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM25_F32)  # type: ignore

    @user_ram25_f32.setter
    @locked
    def user_ram25_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM25_F32 (USER_RAM#(0:39)_F32@25)

        Address: 46050
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM25_F32, value)

    @property
    @locked
    def user_ram26_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM26_F32 (USER_RAM#(0:39)_F32@26)

        Address: 46052
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM26_F32)  # type: ignore

    @user_ram26_f32.setter
    @locked
    def user_ram26_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM26_F32 (USER_RAM#(0:39)_F32@26)

        Address: 46052
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM26_F32, value)

    @property
    @locked
    def user_ram27_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM27_F32 (USER_RAM#(0:39)_F32@27)

        Address: 46054
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM27_F32)  # type: ignore

    @user_ram27_f32.setter
    @locked
    def user_ram27_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM27_F32 (USER_RAM#(0:39)_F32@27)

        Address: 46054
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM27_F32, value)

    @property
    @locked
    def user_ram28_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM28_F32 (USER_RAM#(0:39)_F32@28)

        Address: 46056
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM28_F32)  # type: ignore

    @user_ram28_f32.setter
    @locked
    def user_ram28_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM28_F32 (USER_RAM#(0:39)_F32@28)

        Address: 46056
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM28_F32, value)

    @property
    @locked
    def user_ram29_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM29_F32 (USER_RAM#(0:39)_F32@29)

        Address: 46058
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM29_F32)  # type: ignore

    @user_ram29_f32.setter
    @locked
    def user_ram29_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM29_F32 (USER_RAM#(0:39)_F32@29)

        Address: 46058
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM29_F32, value)

    @property
    @locked
    def user_ram30_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM30_F32 (USER_RAM#(0:39)_F32@30)

        Address: 46060
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM30_F32)  # type: ignore

    @user_ram30_f32.setter
    @locked
    def user_ram30_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM30_F32 (USER_RAM#(0:39)_F32@30)

        Address: 46060
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM30_F32, value)

    @property
    @locked
    def user_ram31_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM31_F32 (USER_RAM#(0:39)_F32@31)

        Address: 46062
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM31_F32)  # type: ignore

    @user_ram31_f32.setter
    @locked
    def user_ram31_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM31_F32 (USER_RAM#(0:39)_F32@31)

        Address: 46062
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM31_F32, value)

    @property
    @locked
    def user_ram32_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM32_F32 (USER_RAM#(0:39)_F32@32)

        Address: 46064
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM32_F32)  # type: ignore

    @user_ram32_f32.setter
    @locked
    def user_ram32_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM32_F32 (USER_RAM#(0:39)_F32@32)

        Address: 46064
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM32_F32, value)

    @property
    @locked
    def user_ram33_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM33_F32 (USER_RAM#(0:39)_F32@33)

        Address: 46066
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM33_F32)  # type: ignore

    @user_ram33_f32.setter
    @locked
    def user_ram33_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM33_F32 (USER_RAM#(0:39)_F32@33)

        Address: 46066
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM33_F32, value)

    @property
    @locked
    def user_ram34_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM34_F32 (USER_RAM#(0:39)_F32@34)

        Address: 46068
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM34_F32)  # type: ignore

    @user_ram34_f32.setter
    @locked
    def user_ram34_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM34_F32 (USER_RAM#(0:39)_F32@34)

        Address: 46068
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM34_F32, value)

    @property
    @locked
    def user_ram35_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM35_F32 (USER_RAM#(0:39)_F32@35)

        Address: 46070
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM35_F32)  # type: ignore

    @user_ram35_f32.setter
    @locked
    def user_ram35_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM35_F32 (USER_RAM#(0:39)_F32@35)

        Address: 46070
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM35_F32, value)

    @property
    @locked
    def user_ram36_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM36_F32 (USER_RAM#(0:39)_F32@36)

        Address: 46072
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM36_F32)  # type: ignore

    @user_ram36_f32.setter
    @locked
    def user_ram36_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM36_F32 (USER_RAM#(0:39)_F32@36)

        Address: 46072
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM36_F32, value)

    @property
    @locked
    def user_ram37_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM37_F32 (USER_RAM#(0:39)_F32@37)

        Address: 46074
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM37_F32)  # type: ignore

    @user_ram37_f32.setter
    @locked
    def user_ram37_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM37_F32 (USER_RAM#(0:39)_F32@37)

        Address: 46074
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM37_F32, value)

    @property
    @locked
    def user_ram38_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM38_F32 (USER_RAM#(0:39)_F32@38)

        Address: 46076
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM38_F32)  # type: ignore

    @user_ram38_f32.setter
    @locked
    def user_ram38_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM38_F32 (USER_RAM#(0:39)_F32@38)

        Address: 46076
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM38_F32, value)

    @property
    @locked
    def user_ram39_f32(self) -> float:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM39_F32 (USER_RAM#(0:39)_F32@39)

        Address: 46078
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM39_F32)  # type: ignore

    @user_ram39_f32.setter
    @locked
    def user_ram39_f32(self, value: float) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM39_F32 (USER_RAM#(0:39)_F32@39)

        Address: 46078
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM39_F32, value)

    @property
    @locked
    def user_ram0_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_I32 (USER_RAM#(0:9)_I32@0)

        Address: 46080
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM0_I32)  # type: ignore

    @user_ram0_i32.setter
    @locked
    def user_ram0_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_I32 (USER_RAM#(0:9)_I32@0)

        Address: 46080
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM0_I32, value)

    @property
    @locked
    def user_ram1_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_I32 (USER_RAM#(0:9)_I32@1)

        Address: 46082
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM1_I32)  # type: ignore

    @user_ram1_i32.setter
    @locked
    def user_ram1_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_I32 (USER_RAM#(0:9)_I32@1)

        Address: 46082
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM1_I32, value)

    @property
    @locked
    def user_ram2_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_I32 (USER_RAM#(0:9)_I32@2)

        Address: 46084
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM2_I32)  # type: ignore

    @user_ram2_i32.setter
    @locked
    def user_ram2_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_I32 (USER_RAM#(0:9)_I32@2)

        Address: 46084
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM2_I32, value)

    @property
    @locked
    def user_ram3_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_I32 (USER_RAM#(0:9)_I32@3)

        Address: 46086
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM3_I32)  # type: ignore

    @user_ram3_i32.setter
    @locked
    def user_ram3_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_I32 (USER_RAM#(0:9)_I32@3)

        Address: 46086
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM3_I32, value)

    @property
    @locked
    def user_ram4_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_I32 (USER_RAM#(0:9)_I32@4)

        Address: 46088
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM4_I32)  # type: ignore

    @user_ram4_i32.setter
    @locked
    def user_ram4_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_I32 (USER_RAM#(0:9)_I32@4)

        Address: 46088
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM4_I32, value)

    @property
    @locked
    def user_ram5_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_I32 (USER_RAM#(0:9)_I32@5)

        Address: 46090
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM5_I32)  # type: ignore

    @user_ram5_i32.setter
    @locked
    def user_ram5_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_I32 (USER_RAM#(0:9)_I32@5)

        Address: 46090
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM5_I32, value)

    @property
    @locked
    def user_ram6_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_I32 (USER_RAM#(0:9)_I32@6)

        Address: 46092
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM6_I32)  # type: ignore

    @user_ram6_i32.setter
    @locked
    def user_ram6_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_I32 (USER_RAM#(0:9)_I32@6)

        Address: 46092
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM6_I32, value)

    @property
    @locked
    def user_ram7_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_I32 (USER_RAM#(0:9)_I32@7)

        Address: 46094
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM7_I32)  # type: ignore

    @user_ram7_i32.setter
    @locked
    def user_ram7_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_I32 (USER_RAM#(0:9)_I32@7)

        Address: 46094
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM7_I32, value)

    @property
    @locked
    def user_ram8_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_I32 (USER_RAM#(0:9)_I32@8)

        Address: 46096
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM8_I32)  # type: ignore

    @user_ram8_i32.setter
    @locked
    def user_ram8_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_I32 (USER_RAM#(0:9)_I32@8)

        Address: 46096
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM8_I32, value)

    @property
    @locked
    def user_ram9_i32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_I32 (USER_RAM#(0:9)_I32@9)

        Address: 46098
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM9_I32)  # type: ignore

    @user_ram9_i32.setter
    @locked
    def user_ram9_i32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_I32 (USER_RAM#(0:9)_I32@9)

        Address: 46098
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM9_I32, value)

    @property
    @locked
    def user_ram0_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_U32 (USER_RAM#(0:39)_U32@0)

        Address: 46100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM0_U32)  # type: ignore

    @user_ram0_u32.setter
    @locked
    def user_ram0_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_U32 (USER_RAM#(0:39)_U32@0)

        Address: 46100
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM0_U32, value)

    @property
    @locked
    def user_ram1_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_U32 (USER_RAM#(0:39)_U32@1)

        Address: 46102
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM1_U32)  # type: ignore

    @user_ram1_u32.setter
    @locked
    def user_ram1_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_U32 (USER_RAM#(0:39)_U32@1)

        Address: 46102
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM1_U32, value)

    @property
    @locked
    def user_ram2_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_U32 (USER_RAM#(0:39)_U32@2)

        Address: 46104
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM2_U32)  # type: ignore

    @user_ram2_u32.setter
    @locked
    def user_ram2_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_U32 (USER_RAM#(0:39)_U32@2)

        Address: 46104
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM2_U32, value)

    @property
    @locked
    def user_ram3_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_U32 (USER_RAM#(0:39)_U32@3)

        Address: 46106
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM3_U32)  # type: ignore

    @user_ram3_u32.setter
    @locked
    def user_ram3_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_U32 (USER_RAM#(0:39)_U32@3)

        Address: 46106
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM3_U32, value)

    @property
    @locked
    def user_ram4_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_U32 (USER_RAM#(0:39)_U32@4)

        Address: 46108
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM4_U32)  # type: ignore

    @user_ram4_u32.setter
    @locked
    def user_ram4_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_U32 (USER_RAM#(0:39)_U32@4)

        Address: 46108
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM4_U32, value)

    @property
    @locked
    def user_ram5_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_U32 (USER_RAM#(0:39)_U32@5)

        Address: 46110
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM5_U32)  # type: ignore

    @user_ram5_u32.setter
    @locked
    def user_ram5_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_U32 (USER_RAM#(0:39)_U32@5)

        Address: 46110
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM5_U32, value)

    @property
    @locked
    def user_ram6_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_U32 (USER_RAM#(0:39)_U32@6)

        Address: 46112
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM6_U32)  # type: ignore

    @user_ram6_u32.setter
    @locked
    def user_ram6_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_U32 (USER_RAM#(0:39)_U32@6)

        Address: 46112
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM6_U32, value)

    @property
    @locked
    def user_ram7_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_U32 (USER_RAM#(0:39)_U32@7)

        Address: 46114
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM7_U32)  # type: ignore

    @user_ram7_u32.setter
    @locked
    def user_ram7_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_U32 (USER_RAM#(0:39)_U32@7)

        Address: 46114
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM7_U32, value)

    @property
    @locked
    def user_ram8_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_U32 (USER_RAM#(0:39)_U32@8)

        Address: 46116
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM8_U32)  # type: ignore

    @user_ram8_u32.setter
    @locked
    def user_ram8_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_U32 (USER_RAM#(0:39)_U32@8)

        Address: 46116
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM8_U32, value)

    @property
    @locked
    def user_ram9_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_U32 (USER_RAM#(0:39)_U32@9)

        Address: 46118
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM9_U32)  # type: ignore

    @user_ram9_u32.setter
    @locked
    def user_ram9_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_U32 (USER_RAM#(0:39)_U32@9)

        Address: 46118
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM9_U32, value)

    @property
    @locked
    def user_ram10_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_U32 (USER_RAM#(0:39)_U32@10)

        Address: 46120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM10_U32)  # type: ignore

    @user_ram10_u32.setter
    @locked
    def user_ram10_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_U32 (USER_RAM#(0:39)_U32@10)

        Address: 46120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM10_U32, value)

    @property
    @locked
    def user_ram11_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_U32 (USER_RAM#(0:39)_U32@11)

        Address: 46122
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM11_U32)  # type: ignore

    @user_ram11_u32.setter
    @locked
    def user_ram11_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_U32 (USER_RAM#(0:39)_U32@11)

        Address: 46122
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM11_U32, value)

    @property
    @locked
    def user_ram12_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_U32 (USER_RAM#(0:39)_U32@12)

        Address: 46124
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM12_U32)  # type: ignore

    @user_ram12_u32.setter
    @locked
    def user_ram12_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_U32 (USER_RAM#(0:39)_U32@12)

        Address: 46124
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM12_U32, value)

    @property
    @locked
    def user_ram13_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_U32 (USER_RAM#(0:39)_U32@13)

        Address: 46126
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM13_U32)  # type: ignore

    @user_ram13_u32.setter
    @locked
    def user_ram13_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_U32 (USER_RAM#(0:39)_U32@13)

        Address: 46126
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM13_U32, value)

    @property
    @locked
    def user_ram14_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_U32 (USER_RAM#(0:39)_U32@14)

        Address: 46128
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM14_U32)  # type: ignore

    @user_ram14_u32.setter
    @locked
    def user_ram14_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_U32 (USER_RAM#(0:39)_U32@14)

        Address: 46128
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM14_U32, value)

    @property
    @locked
    def user_ram15_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_U32 (USER_RAM#(0:39)_U32@15)

        Address: 46130
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM15_U32)  # type: ignore

    @user_ram15_u32.setter
    @locked
    def user_ram15_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_U32 (USER_RAM#(0:39)_U32@15)

        Address: 46130
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM15_U32, value)

    @property
    @locked
    def user_ram16_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_U32 (USER_RAM#(0:39)_U32@16)

        Address: 46132
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM16_U32)  # type: ignore

    @user_ram16_u32.setter
    @locked
    def user_ram16_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_U32 (USER_RAM#(0:39)_U32@16)

        Address: 46132
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM16_U32, value)

    @property
    @locked
    def user_ram17_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_U32 (USER_RAM#(0:39)_U32@17)

        Address: 46134
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM17_U32)  # type: ignore

    @user_ram17_u32.setter
    @locked
    def user_ram17_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_U32 (USER_RAM#(0:39)_U32@17)

        Address: 46134
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM17_U32, value)

    @property
    @locked
    def user_ram18_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_U32 (USER_RAM#(0:39)_U32@18)

        Address: 46136
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM18_U32)  # type: ignore

    @user_ram18_u32.setter
    @locked
    def user_ram18_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_U32 (USER_RAM#(0:39)_U32@18)

        Address: 46136
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM18_U32, value)

    @property
    @locked
    def user_ram19_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_U32 (USER_RAM#(0:39)_U32@19)

        Address: 46138
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM19_U32)  # type: ignore

    @user_ram19_u32.setter
    @locked
    def user_ram19_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_U32 (USER_RAM#(0:39)_U32@19)

        Address: 46138
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM19_U32, value)

    @property
    @locked
    def user_ram20_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM20_U32 (USER_RAM#(0:39)_U32@20)

        Address: 46140
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM20_U32)  # type: ignore

    @user_ram20_u32.setter
    @locked
    def user_ram20_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM20_U32 (USER_RAM#(0:39)_U32@20)

        Address: 46140
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM20_U32, value)

    @property
    @locked
    def user_ram21_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM21_U32 (USER_RAM#(0:39)_U32@21)

        Address: 46142
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM21_U32)  # type: ignore

    @user_ram21_u32.setter
    @locked
    def user_ram21_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM21_U32 (USER_RAM#(0:39)_U32@21)

        Address: 46142
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM21_U32, value)

    @property
    @locked
    def user_ram22_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM22_U32 (USER_RAM#(0:39)_U32@22)

        Address: 46144
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM22_U32)  # type: ignore

    @user_ram22_u32.setter
    @locked
    def user_ram22_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM22_U32 (USER_RAM#(0:39)_U32@22)

        Address: 46144
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM22_U32, value)

    @property
    @locked
    def user_ram23_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM23_U32 (USER_RAM#(0:39)_U32@23)

        Address: 46146
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM23_U32)  # type: ignore

    @user_ram23_u32.setter
    @locked
    def user_ram23_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM23_U32 (USER_RAM#(0:39)_U32@23)

        Address: 46146
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM23_U32, value)

    @property
    @locked
    def user_ram24_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM24_U32 (USER_RAM#(0:39)_U32@24)

        Address: 46148
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM24_U32)  # type: ignore

    @user_ram24_u32.setter
    @locked
    def user_ram24_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM24_U32 (USER_RAM#(0:39)_U32@24)

        Address: 46148
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM24_U32, value)

    @property
    @locked
    def user_ram25_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM25_U32 (USER_RAM#(0:39)_U32@25)

        Address: 46150
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM25_U32)  # type: ignore

    @user_ram25_u32.setter
    @locked
    def user_ram25_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM25_U32 (USER_RAM#(0:39)_U32@25)

        Address: 46150
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM25_U32, value)

    @property
    @locked
    def user_ram26_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM26_U32 (USER_RAM#(0:39)_U32@26)

        Address: 46152
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM26_U32)  # type: ignore

    @user_ram26_u32.setter
    @locked
    def user_ram26_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM26_U32 (USER_RAM#(0:39)_U32@26)

        Address: 46152
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM26_U32, value)

    @property
    @locked
    def user_ram27_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM27_U32 (USER_RAM#(0:39)_U32@27)

        Address: 46154
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM27_U32)  # type: ignore

    @user_ram27_u32.setter
    @locked
    def user_ram27_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM27_U32 (USER_RAM#(0:39)_U32@27)

        Address: 46154
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM27_U32, value)

    @property
    @locked
    def user_ram28_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM28_U32 (USER_RAM#(0:39)_U32@28)

        Address: 46156
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM28_U32)  # type: ignore

    @user_ram28_u32.setter
    @locked
    def user_ram28_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM28_U32 (USER_RAM#(0:39)_U32@28)

        Address: 46156
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM28_U32, value)

    @property
    @locked
    def user_ram29_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM29_U32 (USER_RAM#(0:39)_U32@29)

        Address: 46158
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM29_U32)  # type: ignore

    @user_ram29_u32.setter
    @locked
    def user_ram29_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM29_U32 (USER_RAM#(0:39)_U32@29)

        Address: 46158
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM29_U32, value)

    @property
    @locked
    def user_ram30_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM30_U32 (USER_RAM#(0:39)_U32@30)

        Address: 46160
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM30_U32)  # type: ignore

    @user_ram30_u32.setter
    @locked
    def user_ram30_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM30_U32 (USER_RAM#(0:39)_U32@30)

        Address: 46160
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM30_U32, value)

    @property
    @locked
    def user_ram31_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM31_U32 (USER_RAM#(0:39)_U32@31)

        Address: 46162
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM31_U32)  # type: ignore

    @user_ram31_u32.setter
    @locked
    def user_ram31_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM31_U32 (USER_RAM#(0:39)_U32@31)

        Address: 46162
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM31_U32, value)

    @property
    @locked
    def user_ram32_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM32_U32 (USER_RAM#(0:39)_U32@32)

        Address: 46164
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM32_U32)  # type: ignore

    @user_ram32_u32.setter
    @locked
    def user_ram32_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM32_U32 (USER_RAM#(0:39)_U32@32)

        Address: 46164
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM32_U32, value)

    @property
    @locked
    def user_ram33_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM33_U32 (USER_RAM#(0:39)_U32@33)

        Address: 46166
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM33_U32)  # type: ignore

    @user_ram33_u32.setter
    @locked
    def user_ram33_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM33_U32 (USER_RAM#(0:39)_U32@33)

        Address: 46166
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM33_U32, value)

    @property
    @locked
    def user_ram34_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM34_U32 (USER_RAM#(0:39)_U32@34)

        Address: 46168
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM34_U32)  # type: ignore

    @user_ram34_u32.setter
    @locked
    def user_ram34_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM34_U32 (USER_RAM#(0:39)_U32@34)

        Address: 46168
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM34_U32, value)

    @property
    @locked
    def user_ram35_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM35_U32 (USER_RAM#(0:39)_U32@35)

        Address: 46170
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM35_U32)  # type: ignore

    @user_ram35_u32.setter
    @locked
    def user_ram35_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM35_U32 (USER_RAM#(0:39)_U32@35)

        Address: 46170
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM35_U32, value)

    @property
    @locked
    def user_ram36_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM36_U32 (USER_RAM#(0:39)_U32@36)

        Address: 46172
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM36_U32)  # type: ignore

    @user_ram36_u32.setter
    @locked
    def user_ram36_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM36_U32 (USER_RAM#(0:39)_U32@36)

        Address: 46172
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM36_U32, value)

    @property
    @locked
    def user_ram37_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM37_U32 (USER_RAM#(0:39)_U32@37)

        Address: 46174
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM37_U32)  # type: ignore

    @user_ram37_u32.setter
    @locked
    def user_ram37_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM37_U32 (USER_RAM#(0:39)_U32@37)

        Address: 46174
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM37_U32, value)

    @property
    @locked
    def user_ram38_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM38_U32 (USER_RAM#(0:39)_U32@38)

        Address: 46176
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM38_U32)  # type: ignore

    @user_ram38_u32.setter
    @locked
    def user_ram38_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM38_U32 (USER_RAM#(0:39)_U32@38)

        Address: 46176
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM38_U32, value)

    @property
    @locked
    def user_ram39_u32(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM39_U32 (USER_RAM#(0:39)_U32@39)

        Address: 46178
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM39_U32)  # type: ignore

    @user_ram39_u32.setter
    @locked
    def user_ram39_u32(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM39_U32 (USER_RAM#(0:39)_U32@39)

        Address: 46178
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM39_U32, value)

    @property
    @locked
    def user_ram0_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_U16 (USER_RAM#(0:19)_U16@0)

        Address: 46180
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM0_U16)  # type: ignore

    @user_ram0_u16.setter
    @locked
    def user_ram0_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM0_U16 (USER_RAM#(0:19)_U16@0)

        Address: 46180
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM0_U16, value)

    @property
    @locked
    def user_ram1_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_U16 (USER_RAM#(0:19)_U16@1)

        Address: 46181
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM1_U16)  # type: ignore

    @user_ram1_u16.setter
    @locked
    def user_ram1_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM1_U16 (USER_RAM#(0:19)_U16@1)

        Address: 46181
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM1_U16, value)

    @property
    @locked
    def user_ram2_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_U16 (USER_RAM#(0:19)_U16@2)

        Address: 46182
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM2_U16)  # type: ignore

    @user_ram2_u16.setter
    @locked
    def user_ram2_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM2_U16 (USER_RAM#(0:19)_U16@2)

        Address: 46182
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM2_U16, value)

    @property
    @locked
    def user_ram3_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_U16 (USER_RAM#(0:19)_U16@3)

        Address: 46183
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM3_U16)  # type: ignore

    @user_ram3_u16.setter
    @locked
    def user_ram3_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM3_U16 (USER_RAM#(0:19)_U16@3)

        Address: 46183
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM3_U16, value)

    @property
    @locked
    def user_ram4_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_U16 (USER_RAM#(0:19)_U16@4)

        Address: 46184
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM4_U16)  # type: ignore

    @user_ram4_u16.setter
    @locked
    def user_ram4_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM4_U16 (USER_RAM#(0:19)_U16@4)

        Address: 46184
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM4_U16, value)

    @property
    @locked
    def user_ram5_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_U16 (USER_RAM#(0:19)_U16@5)

        Address: 46185
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM5_U16)  # type: ignore

    @user_ram5_u16.setter
    @locked
    def user_ram5_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM5_U16 (USER_RAM#(0:19)_U16@5)

        Address: 46185
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM5_U16, value)

    @property
    @locked
    def user_ram6_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_U16 (USER_RAM#(0:19)_U16@6)

        Address: 46186
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM6_U16)  # type: ignore

    @user_ram6_u16.setter
    @locked
    def user_ram6_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM6_U16 (USER_RAM#(0:19)_U16@6)

        Address: 46186
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM6_U16, value)

    @property
    @locked
    def user_ram7_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_U16 (USER_RAM#(0:19)_U16@7)

        Address: 46187
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM7_U16)  # type: ignore

    @user_ram7_u16.setter
    @locked
    def user_ram7_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM7_U16 (USER_RAM#(0:19)_U16@7)

        Address: 46187
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM7_U16, value)

    @property
    @locked
    def user_ram8_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_U16 (USER_RAM#(0:19)_U16@8)

        Address: 46188
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM8_U16)  # type: ignore

    @user_ram8_u16.setter
    @locked
    def user_ram8_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM8_U16 (USER_RAM#(0:19)_U16@8)

        Address: 46188
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM8_U16, value)

    @property
    @locked
    def user_ram9_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_U16 (USER_RAM#(0:19)_U16@9)

        Address: 46189
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM9_U16)  # type: ignore

    @user_ram9_u16.setter
    @locked
    def user_ram9_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM9_U16 (USER_RAM#(0:19)_U16@9)

        Address: 46189
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM9_U16, value)

    @property
    @locked
    def user_ram10_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_U16 (USER_RAM#(0:19)_U16@10)

        Address: 46190
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM10_U16)  # type: ignore

    @user_ram10_u16.setter
    @locked
    def user_ram10_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM10_U16 (USER_RAM#(0:19)_U16@10)

        Address: 46190
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM10_U16, value)

    @property
    @locked
    def user_ram11_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_U16 (USER_RAM#(0:19)_U16@11)

        Address: 46191
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM11_U16)  # type: ignore

    @user_ram11_u16.setter
    @locked
    def user_ram11_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM11_U16 (USER_RAM#(0:19)_U16@11)

        Address: 46191
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM11_U16, value)

    @property
    @locked
    def user_ram12_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_U16 (USER_RAM#(0:19)_U16@12)

        Address: 46192
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM12_U16)  # type: ignore

    @user_ram12_u16.setter
    @locked
    def user_ram12_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM12_U16 (USER_RAM#(0:19)_U16@12)

        Address: 46192
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM12_U16, value)

    @property
    @locked
    def user_ram13_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_U16 (USER_RAM#(0:19)_U16@13)

        Address: 46193
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM13_U16)  # type: ignore

    @user_ram13_u16.setter
    @locked
    def user_ram13_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM13_U16 (USER_RAM#(0:19)_U16@13)

        Address: 46193
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM13_U16, value)

    @property
    @locked
    def user_ram14_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_U16 (USER_RAM#(0:19)_U16@14)

        Address: 46194
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM14_U16)  # type: ignore

    @user_ram14_u16.setter
    @locked
    def user_ram14_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM14_U16 (USER_RAM#(0:19)_U16@14)

        Address: 46194
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM14_U16, value)

    @property
    @locked
    def user_ram15_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_U16 (USER_RAM#(0:19)_U16@15)

        Address: 46195
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM15_U16)  # type: ignore

    @user_ram15_u16.setter
    @locked
    def user_ram15_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM15_U16 (USER_RAM#(0:19)_U16@15)

        Address: 46195
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM15_U16, value)

    @property
    @locked
    def user_ram16_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_U16 (USER_RAM#(0:19)_U16@16)

        Address: 46196
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM16_U16)  # type: ignore

    @user_ram16_u16.setter
    @locked
    def user_ram16_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM16_U16 (USER_RAM#(0:19)_U16@16)

        Address: 46196
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM16_U16, value)

    @property
    @locked
    def user_ram17_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_U16 (USER_RAM#(0:19)_U16@17)

        Address: 46197
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM17_U16)  # type: ignore

    @user_ram17_u16.setter
    @locked
    def user_ram17_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM17_U16 (USER_RAM#(0:19)_U16@17)

        Address: 46197
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM17_U16, value)

    @property
    @locked
    def user_ram18_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_U16 (USER_RAM#(0:19)_U16@18)

        Address: 46198
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM18_U16)  # type: ignore

    @user_ram18_u16.setter
    @locked
    def user_ram18_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM18_U16 (USER_RAM#(0:19)_U16@18)

        Address: 46198
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM18_U16, value)

    @property
    @locked
    def user_ram19_u16(self) -> int:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_U16 (USER_RAM#(0:19)_U16@19)

        Address: 46199
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM19_U16)  # type: ignore

    @user_ram19_u16.setter
    @locked
    def user_ram19_u16(self, value: int) -> None:
        """
        Generic RAM registers. Useful for passing data between a host computer and a Lua script. Will not return an
        error if alternate data types are used.

        Register: USER_RAM19_U16 (USER_RAM#(0:19)_U16@19)

        Address: 46199
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM19_U16, value)

    @property
    @locked
    def user_ram_fifo0_data_u16(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@0)

        Address: 47000
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO0_DATA_U16)  # type: ignore

    @user_ram_fifo0_data_u16.setter
    @locked
    def user_ram_fifo0_data_u16(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@0)

        Address: 47000
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO0_DATA_U16, value)

    @property
    @locked
    def user_ram_fifo1_data_u16(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@1)

        Address: 47001
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO1_DATA_U16)  # type: ignore

    @user_ram_fifo1_data_u16.setter
    @locked
    def user_ram_fifo1_data_u16(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@1)

        Address: 47001
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO1_DATA_U16, value)

    @property
    @locked
    def user_ram_fifo2_data_u16(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@2)

        Address: 47002
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO2_DATA_U16)  # type: ignore

    @user_ram_fifo2_data_u16.setter
    @locked
    def user_ram_fifo2_data_u16(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@2)

        Address: 47002
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO2_DATA_U16, value)

    @property
    @locked
    def user_ram_fifo3_data_u16(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@3)

        Address: 47003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO3_DATA_U16)  # type: ignore

    @user_ram_fifo3_data_u16.setter
    @locked
    def user_ram_fifo3_data_u16(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_U16 (USER_RAM_FIFO#(0:3)_DATA_U16@3)

        Address: 47003
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.USER_RAM_FIFO3_DATA_U16, value)

    @property
    @locked
    def user_ram_fifo0_data_u32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@0)

        Address: 47010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO0_DATA_U32)  # type: ignore

    @user_ram_fifo0_data_u32.setter
    @locked
    def user_ram_fifo0_data_u32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@0)

        Address: 47010
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO0_DATA_U32, value)

    @property
    @locked
    def user_ram_fifo1_data_u32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@1)

        Address: 47012
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO1_DATA_U32)  # type: ignore

    @user_ram_fifo1_data_u32.setter
    @locked
    def user_ram_fifo1_data_u32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@1)

        Address: 47012
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO1_DATA_U32, value)

    @property
    @locked
    def user_ram_fifo2_data_u32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@2)

        Address: 47014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO2_DATA_U32)  # type: ignore

    @user_ram_fifo2_data_u32.setter
    @locked
    def user_ram_fifo2_data_u32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@2)

        Address: 47014
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO2_DATA_U32, value)

    @property
    @locked
    def user_ram_fifo3_data_u32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@3)

        Address: 47016
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO3_DATA_U32)  # type: ignore

    @user_ram_fifo3_data_u32.setter
    @locked
    def user_ram_fifo3_data_u32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_U32 (USER_RAM_FIFO#(0:3)_DATA_U32@3)

        Address: 47016
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO3_DATA_U32, value)

    @property
    @locked
    def user_ram_fifo0_data_i32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@0)

        Address: 47020
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO0_DATA_I32)  # type: ignore

    @user_ram_fifo0_data_i32.setter
    @locked
    def user_ram_fifo0_data_i32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@0)

        Address: 47020
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO0_DATA_I32, value)

    @property
    @locked
    def user_ram_fifo1_data_i32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@1)

        Address: 47022
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO1_DATA_I32)  # type: ignore

    @user_ram_fifo1_data_i32.setter
    @locked
    def user_ram_fifo1_data_i32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@1)

        Address: 47022
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO1_DATA_I32, value)

    @property
    @locked
    def user_ram_fifo2_data_i32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@2)

        Address: 47024
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO2_DATA_I32)  # type: ignore

    @user_ram_fifo2_data_i32.setter
    @locked
    def user_ram_fifo2_data_i32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@2)

        Address: 47024
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO2_DATA_I32, value)

    @property
    @locked
    def user_ram_fifo3_data_i32(self) -> int:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@3)

        Address: 47026
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO3_DATA_I32)  # type: ignore

    @user_ram_fifo3_data_i32.setter
    @locked
    def user_ram_fifo3_data_i32(self, value: int) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_I32 (USER_RAM_FIFO#(0:3)_DATA_I32@3)

        Address: 47026
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.USER_RAM_FIFO3_DATA_I32, value)

    @property
    @locked
    def user_ram_fifo0_data_f32(self) -> float:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@0)

        Address: 47030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO0_DATA_F32)  # type: ignore

    @user_ram_fifo0_data_f32.setter
    @locked
    def user_ram_fifo0_data_f32(self, value: float) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO0_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@0)

        Address: 47030
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO0_DATA_F32, value)

    @property
    @locked
    def user_ram_fifo1_data_f32(self) -> float:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@1)

        Address: 47032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO1_DATA_F32)  # type: ignore

    @user_ram_fifo1_data_f32.setter
    @locked
    def user_ram_fifo1_data_f32(self, value: float) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO1_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@1)

        Address: 47032
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO1_DATA_F32, value)

    @property
    @locked
    def user_ram_fifo2_data_f32(self) -> float:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@2)

        Address: 47034
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO2_DATA_F32)  # type: ignore

    @user_ram_fifo2_data_f32.setter
    @locked
    def user_ram_fifo2_data_f32(self, value: float) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO2_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@2)

        Address: 47034
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO2_DATA_F32, value)

    @property
    @locked
    def user_ram_fifo3_data_f32(self) -> float:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@3)

        Address: 47036
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO3_DATA_F32)  # type: ignore

    @user_ram_fifo3_data_f32.setter
    @locked
    def user_ram_fifo3_data_f32(self, value: float) -> None:
        """
        Generic FIFO buffer. Useful for passing ORDERED or SEQUENTIAL data between various endpoints, such as
        between a host and a Lua script. Use up to 4 FIFO buffers simultaneously->1 of each data type, all 4 different
        data types, or a mixture. e.g. FIFO0_DATA_U16 points to the same memory as other FIFO0 registers, such that
        there are a total of 4 memory blocks: FIFO0, FIFO1, FIFO2 and FIFO3. It is possible to write into a FIFO buffer
        using a different datatype than is being used to read out of it.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: USER_RAM_FIFO3_DATA_F32 (USER_RAM_FIFO#(0:3)_DATA_F32@3)

        Address: 47036
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.USER_RAM_FIFO3_DATA_F32, value)

    @property
    @locked
    def user_ram_fifo0_allocate_num_bytes(self) -> int:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO0_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@0)

        Address: 47900
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO0_ALLOCATE_NUM_BYTES
        )

    @user_ram_fifo0_allocate_num_bytes.setter
    @locked
    def user_ram_fifo0_allocate_num_bytes(self, value: int) -> None:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO0_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@0)

        Address: 47900
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO0_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def user_ram_fifo1_allocate_num_bytes(self) -> int:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO1_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@1)

        Address: 47902
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO1_ALLOCATE_NUM_BYTES
        )

    @user_ram_fifo1_allocate_num_bytes.setter
    @locked
    def user_ram_fifo1_allocate_num_bytes(self, value: int) -> None:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO1_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@1)

        Address: 47902
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO1_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def user_ram_fifo2_allocate_num_bytes(self) -> int:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO2_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@2)

        Address: 47904
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO2_ALLOCATE_NUM_BYTES
        )

    @user_ram_fifo2_allocate_num_bytes.setter
    @locked
    def user_ram_fifo2_allocate_num_bytes(self, value: int) -> None:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO2_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@2)

        Address: 47904
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO2_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def user_ram_fifo3_allocate_num_bytes(self) -> int:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO3_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@3)

        Address: 47906
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO3_ALLOCATE_NUM_BYTES
        )

    @user_ram_fifo3_allocate_num_bytes.setter
    @locked
    def user_ram_fifo3_allocate_num_bytes(self, value: int) -> None:
        """
        Allocate memory for a FIFO buffer. Number of bytes should be sufficient to store users max transfer array
        size. Note that FLOAT32, INT32, and UINT32 require 4 bytes per value, and UINT16 require 2 bytes per value.
        Maximum size is limited by available memory. Care should be taken to conserve enough memory for other
        operations such as AIN_EF, Lua, Stream etc.

        Register: USER_RAM_FIFO3_ALLOCATE_NUM_BYTES (USER_RAM_FIFO#(0:3)_ALLOCATE_NUM_BYTES@3)

        Address: 47906
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO3_ALLOCATE_NUM_BYTES, value)

    @property
    @locked
    def user_ram_fifo0_num_bytes_in_fifo(self) -> int:
        """
        Poll this register to see when new data is available/ready. Each read of the FIFO buffer decreases this
        value, and each write to the FIFO buffer increases this value.

        At any point in time, the following equation holds:
        Nbytes = Nwritten - Nread.

        Register: USER_RAM_FIFO0_NUM_BYTES_IN_FIFO (USER_RAM_FIFO#(0:3)_NUM_BYTES_IN_FIFO@0)

        Address: 47910
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO0_NUM_BYTES_IN_FIFO
        )

    @property
    @locked
    def user_ram_fifo1_num_bytes_in_fifo(self) -> int:
        """
        Poll this register to see when new data is available/ready. Each read of the FIFO buffer decreases this
        value, and each write to the FIFO buffer increases this value.

        At any point in time, the following equation holds:
        Nbytes = Nwritten - Nread.

        Register: USER_RAM_FIFO1_NUM_BYTES_IN_FIFO (USER_RAM_FIFO#(0:3)_NUM_BYTES_IN_FIFO@1)

        Address: 47912
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO1_NUM_BYTES_IN_FIFO
        )

    @property
    @locked
    def user_ram_fifo2_num_bytes_in_fifo(self) -> int:
        """
        Poll this register to see when new data is available/ready. Each read of the FIFO buffer decreases this
        value, and each write to the FIFO buffer increases this value.

        At any point in time, the following equation holds:
        Nbytes = Nwritten - Nread.

        Register: USER_RAM_FIFO2_NUM_BYTES_IN_FIFO (USER_RAM_FIFO#(0:3)_NUM_BYTES_IN_FIFO@2)

        Address: 47914
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO2_NUM_BYTES_IN_FIFO
        )

    @property
    @locked
    def user_ram_fifo3_num_bytes_in_fifo(self) -> int:
        """
        Poll this register to see when new data is available/ready. Each read of the FIFO buffer decreases this
        value, and each write to the FIFO buffer increases this value.

        At any point in time, the following equation holds:
        Nbytes = Nwritten - Nread.

        Register: USER_RAM_FIFO3_NUM_BYTES_IN_FIFO (USER_RAM_FIFO#(0:3)_NUM_BYTES_IN_FIFO@3)

        Address: 47916
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.USER_RAM_FIFO3_NUM_BYTES_IN_FIFO
        )

    @property
    @locked
    def user_ram_fifo0_empty(self) -> int:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO0_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@0)

        Address: 47930
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO0_EMPTY)  # type: ignore

    @user_ram_fifo0_empty.setter
    @locked
    def user_ram_fifo0_empty(self, value: int) -> None:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO0_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@0)

        Address: 47930
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO0_EMPTY, value)

    @property
    @locked
    def user_ram_fifo1_empty(self) -> int:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO1_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@1)

        Address: 47932
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO1_EMPTY)  # type: ignore

    @user_ram_fifo1_empty.setter
    @locked
    def user_ram_fifo1_empty(self, value: int) -> None:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO1_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@1)

        Address: 47932
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO1_EMPTY, value)

    @property
    @locked
    def user_ram_fifo2_empty(self) -> int:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO2_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@2)

        Address: 47934
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO2_EMPTY)  # type: ignore

    @user_ram_fifo2_empty.setter
    @locked
    def user_ram_fifo2_empty(self, value: int) -> None:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO2_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@2)

        Address: 47934
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO2_EMPTY, value)

    @property
    @locked
    def user_ram_fifo3_empty(self) -> int:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO3_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@3)

        Address: 47936
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO3_EMPTY)  # type: ignore

    @user_ram_fifo3_empty.setter
    @locked
    def user_ram_fifo3_empty(self, value: int) -> None:
        """
        Write any value to this register to efficiently empty, flush, or otherwise clear data from the FIFO.

        Register: USER_RAM_FIFO3_EMPTY (USER_RAM_FIFO#(0:3)_EMPTY@3)

        Address: 47936
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.USER_RAM_FIFO3_EMPTY, value)

    @property
    @locked
    def last_err_detail(self) -> int:
        """
        Returns the last LabJack error code seen on the device.

        Register: LAST_ERR_DETAIL

        Address: 55000
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LAST_ERR_DETAIL)  # type: ignore

    @property
    @locked
    def last_mb_err(self) -> int:
        """
        Returns the last Modbus TCP error code seen on the device.

        Register: LAST_MB_ERR

        Address: 55001
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LAST_MB_ERR)  # type: ignore

    @property
    @locked
    def last_err_frame(self) -> int:
        """
        Returns the frame number that caused the last error.

        Register: LAST_ERR_FRAME

        Address: 55002
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LAST_ERR_FRAME)  # type: ignore

    @property
    @locked
    def last_err_transaction_id(self) -> int:
        """
        Returns the transaction ID of the Modbus TCP packet that caused the last error.

        Register: LAST_ERR_TRANSACTION_ID

        Address: 55003
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.LAST_ERR_TRANSACTION_ID)  # type: ignore

    @property
    @locked
    def test(self) -> int:
        """
        A read of this test register should always return 0x00112233 or d1122867. If your software has the word swap
        quirk, you will incorrectly read 0x22330011 or 573767697. If your software has the address-1 quirk, a UINT16
        (1-register) read from 55101 will incorrectly return 0x0011 (should read 0x2233). This register is unlike
        others, in that it allows you can read a single word from 55100 or 55101, or of course 2 words from 55100.

        Register: TEST

        Address: 55100
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEST)  # type: ignore

    @property
    @locked
    def test_uint16(self) -> int:
        """
        Write a value and read back to test UINT16 operation. Default is 0x0011 or d17.

        Register: TEST_UINT16

        Address: 55110
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.TEST_UINT16)  # type: ignore

    @test_uint16.setter
    @locked
    def test_uint16(self, value: int) -> None:
        """
        Write a value and read back to test UINT16 operation. Default is 0x0011 or d17.

        Register: TEST_UINT16

        Address: 55110
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.TEST_UINT16, value)

    @property
    @locked
    def test_uint32(self) -> int:
        """
        Write a value and read back to test UINT32 operation. Default is 0x00112233 or d1122867. If your software
        has the word swap quirk, the default will incorrectly read 0x22330011 or 573767697.

        Register: TEST_UINT32

        Address: 55120
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.TEST_UINT32)  # type: ignore

    @test_uint32.setter
    @locked
    def test_uint32(self, value: int) -> None:
        """
        Write a value and read back to test UINT32 operation. Default is 0x00112233 or d1122867. If your software
        has the word swap quirk, the default will incorrectly read 0x22330011 or 573767697.

        Register: TEST_UINT32

        Address: 55120
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.TEST_UINT32, value)

    @property
    @locked
    def test_int32(self) -> int:
        """
        Write a value and read back to test INT32 operation. Default is 0x8899AABB or d-2003195205. If your software
        has the word swap quirk, the default will incorrectly read 0xAABB8899 or -1430550375.

        Register: TEST_INT32

        Address: 55122
        """
        return self.client.read_holding_register_sbint32(RegisterAddresses.TEST_INT32)  # type: ignore

    @test_int32.setter
    @locked
    def test_int32(self, value: int) -> None:
        """
        Write a value and read back to test INT32 operation. Default is 0x8899AABB or d-2003195205. If your software
        has the word swap quirk, the default will incorrectly read 0xAABB8899 or -1430550375.

        Register: TEST_INT32

        Address: 55122
        """
        self.client.write_holding_register_sbint32(RegisterAddresses.TEST_INT32, value)

    @property
    @locked
    def test_float32(self) -> float:
        """
        Write a value and read back to test FLOAT32 operation. Default is 0xC61C3C00 or -9999.0. If your software
        has the word swap quirk, the default will incorrectly read 0x3C00C61C or 0.00786.

        Register: TEST_FLOAT32

        Address: 55124
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.TEST_FLOAT32)  # type: ignore

    @test_float32.setter
    @locked
    def test_float32(self, value: float) -> None:
        """
        Write a value and read back to test FLOAT32 operation. Default is 0xC61C3C00 or -9999.0. If your software
        has the word swap quirk, the default will incorrectly read 0x3C00C61C or 0.00786.

        Register: TEST_FLOAT32

        Address: 55124
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.TEST_FLOAT32, value)

    @property
    @locked
    def product_id(self) -> float:
        """
        The numeric identifier of the device. Such as 7 for a T7 / T7-Pro.

        Register: PRODUCT_ID

        Address: 60000
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.PRODUCT_ID)  # type: ignore

    @property
    @locked
    def hardware_version(self) -> float:
        """
        The hardware version of the device.

        Register: HARDWARE_VERSION

        Address: 60002
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.HARDWARE_VERSION)  # type: ignore

    @property
    @locked
    def firmware_version(self) -> float:
        """
        The current firmware version installed on the main processor.

        Register: FIRMWARE_VERSION

        Address: 60004
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.FIRMWARE_VERSION)  # type: ignore

    @property
    @locked
    def bootloader_version(self) -> float:
        """
        The bootloader version installed on the main processor.

        Register: BOOTLOADER_VERSION

        Address: 60006
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.BOOTLOADER_VERSION)  # type: ignore

    @property
    @locked
    def wifi_version(self) -> float:
        """
        The current firmware version of the WiFi module, if available.

        Register: WIFI_VERSION

        Address: 60008
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.WIFI_VERSION)  # type: ignore

    @property
    @locked
    def hardware_installed(self) -> int:
        """
        Bitmask indicating installed hardware options.
        bit0:
        High Resolution ADC,
        bit1:
        WiFi,
        bit2:
        RTC,
        bit3:
        microSD.

        Register: HARDWARE_INSTALLED

        Address: 60010
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.HARDWARE_INSTALLED)  # type: ignore

    @property
    @locked
    def ljmcommon_version(self) -> float:
        """
        The MAC address of the wired Ethernet module.

        Register: LJMCOMMON_VERSION

        Address: 60018
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.LJMCOMMON_VERSION)  # type: ignore

    @property
    @locked
    def ethernet_mac(self) -> str:
        """
        The MAC address of the wired Ethernet module.

        Register: ETHERNET_MAC

        Address: 60020
        """
        return self.client.read_holding_register_data(RegisterAddresses.ETHERNET_MAC, MacAddress).get()  # type: ignore

    @property
    @locked
    def wifi_mac(self) -> str:
        """
        The MAC address of the WiFi module.

        Register: WIFI_MAC

        Address: 60024
        """
        return self.client.read_holding_register_data(RegisterAddresses.WIFI_MAC, MacAddress).get()  # type: ignore

    @property
    @locked
    def serial_number(self) -> int:
        """
        The serial number of the device.

        Register: SERIAL_NUMBER

        Address: 60028
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.SERIAL_NUMBER)  # type: ignore

    @property
    @locked
    def internal_vs_volts(self) -> float:
        """
        T8 Only. Returns the power supply voltage.

        Register: INTERNAL_VS_VOLTS

        Address: 60030
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.INTERNAL_VS_VOLTS)  # type: ignore

    @property
    @locked
    def internal_is_amps(self) -> float:
        """
        T8 Only. Returns the power supply current draw.

        Register: INTERNAL_IS_AMPS

        Address: 60032
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.INTERNAL_IS_AMPS)  # type: ignore

    @property
    @locked
    def device_name_default(self) -> str:
        """
        Reads return the current device name. Writes update the default and current device name. A reboot is
        necessary to update the name reported by NBNS. Up to 49 characters, cannot contain periods.

        Register: DEVICE_NAME_DEFAULT

        Address: 60500
        """
        return self.client.read_holding_register_string(  # type: ignore
            RegisterAddresses.DEVICE_NAME_DEFAULT, length=50
        ).decode(errors="ignore")

    @device_name_default.setter
    @locked
    def device_name_default(self, value: str) -> None:
        """
        Reads return the current device name. Writes update the default and current device name. A reboot is
        necessary to update the name reported by NBNS. Up to 49 characters, cannot contain periods.

        Register: DEVICE_NAME_DEFAULT

        Address: 60500
        """
        self.client.write_holding_register_string(
            RegisterAddresses.DEVICE_NAME_DEFAULT, value.encode(errors="ignore"), length=50
        )

    @property
    @locked
    def current_source_10ua_cal_value(self) -> float:
        """
        Fixed current source value in Amps for the 10UA terminal. This value is stored during factory calibration,
        it is not a current reading. Using the equation V=IR, with a known current and voltage, it is possible to
        calculate resistance of RTDs.

        Register: CURRENT_SOURCE_10UA_CAL_VALUE

        Address: 1900
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.CURRENT_SOURCE_10UA_CAL_VALUE)  # type: ignore

    @property
    @locked
    def current_source_200ua_cal_value(self) -> float:
        """
        Fixed current source value in Amps for the 200UA terminal. This value is stored during factory calibration,
        it is not a current reading. Using the equation V=IR, with a known current and voltage, it is possible to
        calculate resistance of RTDs.

        Register: CURRENT_SOURCE_200UA_CAL_VALUE

        Address: 1902
        """
        return self.client.read_holding_register_bfloat(  # type: ignore
            RegisterAddresses.CURRENT_SOURCE_200UA_CAL_VALUE
        )

    @property
    @locked
    def file_io_dir_change(self) -> int:
        """
        Write any value to this register to change the current working directory (CWD). Must first designate which
        directory to open by writing to FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_DIR_CHANGE

        Address: 60600
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_CHANGE)  # type: ignore

    @file_io_dir_change.setter
    @locked
    def file_io_dir_change(self, value: int) -> None:
        """
        Write any value to this register to change the current working directory (CWD). Must first designate which
        directory to open by writing to FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_DIR_CHANGE

        Address: 60600
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_CHANGE, value)

    @property
    @locked
    def file_io_dir_current(self) -> int:
        """
        Write any value to this register to load the current working directory into FILE_IO_PATH_READ, and its
        length into FILE_IO_PATH_READ_LEN_BYTES. Can be used to identify current position in a file tree.

        Register: FILE_IO_DIR_CURRENT

        Address: 60601
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_CURRENT)  # type: ignore

    @file_io_dir_current.setter
    @locked
    def file_io_dir_current(self, value: int) -> None:
        """
        Write any value to this register to load the current working directory into FILE_IO_PATH_READ, and its
        length into FILE_IO_PATH_READ_LEN_BYTES. Can be used to identify current position in a file tree.

        Register: FILE_IO_DIR_CURRENT

        Address: 60601
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_CURRENT, value)

    @property
    @locked
    def file_io_dir_make(self) -> int:
        """
        Unimplemented.

        Register: FILE_IO_DIR_MAKE

        Address: 60602
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_MAKE)  # type: ignore

    @file_io_dir_make.setter
    @locked
    def file_io_dir_make(self, value: int) -> None:
        """
        Unimplemented.

        Register: FILE_IO_DIR_MAKE

        Address: 60602
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_MAKE, value)

    @property
    @locked
    def file_io_dir_remove(self) -> int:
        """
        Unimplemented.

        Register: FILE_IO_DIR_REMOVE

        Address: 60603
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_REMOVE)  # type: ignore

    @file_io_dir_remove.setter
    @locked
    def file_io_dir_remove(self, value: int) -> None:
        """
        Unimplemented.

        Register: FILE_IO_DIR_REMOVE

        Address: 60603
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_REMOVE, value)

    @property
    @locked
    def file_io_dir_first(self) -> int:
        """
        Write any value to this register to initiate iteration through files and directories in the CWD.
        Typical sequence:
        FILE_IO_DIR_FIRST(W), then loop through:
        [FILE_IO_NAME_READ_LEN(R), FILE_IO_NAME_READ(R), FILE_IO_ATTRIBUTES(R),
        FILE_IO_SIZE_BYTES(R), FILE_IO_DIR_NEXT(W)]
        ..until any of the following errors:
        FILE_IO_END_OF_CWD (2966), FILE_IO_INVALID_OBJECT (2809), or FILE_IO_NOT_FOUND (2960).

        Register: FILE_IO_DIR_FIRST

        Address: 60610
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_FIRST)  # type: ignore

    @file_io_dir_first.setter
    @locked
    def file_io_dir_first(self, value: int) -> None:
        """
        Write any value to this register to initiate iteration through files and directories in the CWD.
        Typical sequence:
        FILE_IO_DIR_FIRST(W), then loop through:
        [FILE_IO_NAME_READ_LEN(R), FILE_IO_NAME_READ(R), FILE_IO_ATTRIBUTES(R),
        FILE_IO_SIZE_BYTES(R), FILE_IO_DIR_NEXT(W)]
        ..until any of the following errors:
        FILE_IO_END_OF_CWD (2966), FILE_IO_INVALID_OBJECT (2809), or FILE_IO_NOT_FOUND (2960).

        Register: FILE_IO_DIR_FIRST

        Address: 60610
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_FIRST, value)

    @property
    @locked
    def file_io_dir_next(self) -> int:
        """
        Write any value to this register to continue iteration through files and directories in the CWD.

        Register: FILE_IO_DIR_NEXT

        Address: 60611
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_NEXT)  # type: ignore

    @file_io_dir_next.setter
    @locked
    def file_io_dir_next(self, value: int) -> None:
        """
        Write any value to this register to continue iteration through files and directories in the CWD.

        Register: FILE_IO_DIR_NEXT

        Address: 60611
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DIR_NEXT, value)

    @property
    @locked
    def file_io_open(self) -> int:
        """
        Write any value to this register to open a file. Must first designate which file to open by writing to
        FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_OPEN

        Address: 60620
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_OPEN)  # type: ignore

    @file_io_open.setter
    @locked
    def file_io_open(self, value: int) -> None:
        """
        Write any value to this register to open a file. Must first designate which file to open by writing to
        FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_OPEN

        Address: 60620
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_OPEN, value)

    @property
    @locked
    def file_io_close(self) -> int:
        """
        Write any value to this register to close the open file.

        Register: FILE_IO_CLOSE

        Address: 60621
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_CLOSE)  # type: ignore

    @file_io_close.setter
    @locked
    def file_io_close(self, value: int) -> None:
        """
        Write any value to this register to close the open file.

        Register: FILE_IO_CLOSE

        Address: 60621
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_CLOSE, value)

    @property
    @locked
    def file_io_delete(self) -> int:
        """
        Write any value to this register to delete the active file. Must first designate which file to delete by
        writing to FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_DELETE

        Address: 60622
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_DELETE)  # type: ignore

    @file_io_delete.setter
    @locked
    def file_io_delete(self, value: int) -> None:
        """
        Write any value to this register to delete the active file. Must first designate which file to delete by
        writing to FILE_IO_PATH_WRITE_LEN_BYTES then FILE_IO_PATH_WRITE.

        Register: FILE_IO_DELETE

        Address: 60622
        """
        self.client.write_holding_register_ubint16(RegisterAddresses.FILE_IO_DELETE, value)

    @property
    @locked
    def file_io_attributes(self) -> int:
        """
        Used to differentiate files from directories/folders.
        Bitmask:
        Bit0:
        Reserved,
        Bit1:
        Reserved,
        Bit2:
        Reserved,
        Bit3:
        Reserved,
        Bit4:
        1=Directory,
        Bit5:
        1=File.

        Register: FILE_IO_ATTRIBUTES

        Address: 60623
        """
        return self.client.read_holding_register_ubint16(RegisterAddresses.FILE_IO_ATTRIBUTES)  # type: ignore

    @property
    @locked
    def file_io_size_bytes(self) -> int:
        """
        The size of the file in bytes. Directories have 0 size.

        Register: FILE_IO_SIZE_BYTES

        Address: 60628
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_SIZE_BYTES)  # type: ignore

    @property
    @locked
    def file_io_disk_sector_size_bytes(self) -> int:
        """
        The size of each sector in the SD card in bytes. In Windows this is called the Allocation Size.

        Register: FILE_IO_DISK_SECTOR_SIZE_BYTES

        Address: 60630
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.FILE_IO_DISK_SECTOR_SIZE_BYTES
        )

    @property
    @locked
    def file_io_disk_sectors_per_cluster(self) -> int:
        """
        The number of sectors in each cluster. Captured on read of FILE_IO_DISK_SECTOR_SIZE_BYTES.

        Register: FILE_IO_DISK_SECTORS_PER_CLUSTER

        Address: 60632
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.FILE_IO_DISK_SECTORS_PER_CLUSTER
        )

    @property
    @locked
    def file_io_disk_total_clusters(self) -> int:
        """
        The total number of clusters in the SD card. Captured on read of FILE_IO_DISK_SECTOR_SIZE_BYTES.

        Register: FILE_IO_DISK_TOTAL_CLUSTERS

        Address: 60634
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_DISK_TOTAL_CLUSTERS)  # type: ignore

    @property
    @locked
    def file_io_disk_free_clusters(self) -> int:
        """
        Free (available) clusters in the SD card. Used to determine free space. Captured on read of
        FILE_IO_DISK_SECTOR_SIZE_BYTES.

        Register: FILE_IO_DISK_FREE_CLUSTERS

        Address: 60636
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_DISK_FREE_CLUSTERS)  # type: ignore

    @property
    @locked
    def file_io_disk_format_index(self) -> int:
        """
        Used to determine the format of the SD card. 0=None or Unknown, 1=FAT12, 2=FAT16(Windows FAT), 3=FAT32.

        Register: FILE_IO_DISK_FORMAT_INDEX

        Address: 60638
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_DISK_FORMAT_INDEX)  # type: ignore

    @property
    @locked
    def file_io_path_write_len_bytes(self) -> int:
        """
        Write the length (in bytes) of the file path or directory to access.

        Register: FILE_IO_PATH_WRITE_LEN_BYTES

        Address: 60640
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_PATH_WRITE_LEN_BYTES)  # type: ignore

    @file_io_path_write_len_bytes.setter
    @locked
    def file_io_path_write_len_bytes(self, value: int) -> None:
        """
        Write the length (in bytes) of the file path or directory to access.

        Register: FILE_IO_PATH_WRITE_LEN_BYTES

        Address: 60640
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.FILE_IO_PATH_WRITE_LEN_BYTES, value)

    @property
    @locked
    def file_io_path_read_len_bytes(self) -> int:
        """
        Read the length (in bytes) of the next file path or directory to access.

        Register: FILE_IO_PATH_READ_LEN_BYTES

        Address: 60642
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.FILE_IO_PATH_READ_LEN_BYTES)  # type: ignore

    @property
    @locked
    def file_io_path_write(self) -> int:
        """
        Write the desired file path. Must first write the length of the file path string (in bytes) to
        FILE_IO_PATH_WRITE_LEN_BYTES. File paths should be null terminated. This register is a buffer.

        Register: FILE_IO_PATH_WRITE

        Address: 60650
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.FILE_IO_PATH_WRITE)  # type: ignore

    @file_io_path_write.setter
    @locked
    def file_io_path_write(self, value: int) -> None:
        """
        Write the desired file path. Must first write the length of the file path string (in bytes) to
        FILE_IO_PATH_WRITE_LEN_BYTES. File paths should be null terminated. This register is a buffer.

        Register: FILE_IO_PATH_WRITE

        Address: 60650
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.FILE_IO_PATH_WRITE, value)

    @property
    @locked
    def file_io_path_read(self) -> int:
        """
        Read the next file path in the CWD. Length of the string (in bytes) determined by
        FILE_IO_PATH_READ_LEN_BYTES. File paths will be null terminated.

        This register is a buffer.
        Underrun behavior - fill with zeros.

        Register: FILE_IO_PATH_READ

        Address: 60652
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.FILE_IO_PATH_READ)  # type: ignore

    @property
    @locked
    def file_io_write(self) -> int:
        """
        Unimplemented. This register is a buffer.

        Register: FILE_IO_WRITE

        Address: 60654
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.FILE_IO_WRITE)  # type: ignore

    @file_io_write.setter
    @locked
    def file_io_write(self, value: int) -> None:
        """
        Unimplemented. This register is a buffer.

        Register: FILE_IO_WRITE

        Address: 60654
        """
        self.client.write_holding_register_sbint8(RegisterAddresses.FILE_IO_WRITE, value)

    @property
    @locked
    def file_io_read(self) -> int:
        """
        Read the contents of a file. Must first write to FILE_IO_OPEN. Size of the file (in bytes) determined by
        FILE_IO_SIZE_BYTES.

        This register is a buffer.
        Underrun behavior - throws an error.

        Register: FILE_IO_READ

        Address: 60656
        """
        return self.client.read_holding_register_sbint8(RegisterAddresses.FILE_IO_READ)  # type: ignore

    @property
    @locked
    def watchdog_enable_default(self) -> int:
        """
        Write a 1 to enable the watchdog or a 0 to disable. The watchdog must be disabled before writing any of the
        other watchdog registers (except for WATCHDOG_STRICT_CLEAR).

        Register: WATCHDOG_ENABLE_DEFAULT

        Address: 61600
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_ENABLE_DEFAULT)  # type: ignore

    @watchdog_enable_default.setter
    @locked
    def watchdog_enable_default(self, value: int) -> None:
        """
        Write a 1 to enable the watchdog or a 0 to disable. The watchdog must be disabled before writing any of the
        other watchdog registers (except for WATCHDOG_STRICT_CLEAR).

        Register: WATCHDOG_ENABLE_DEFAULT

        Address: 61600
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_advanced_default(self) -> int:
        """
        A single binary-encoded value where each bit is an advanced option. If bit 0 is set,
        IO_CONFIG_SET_CURRENT_TO_FACTORY will be done on timeout. If bit 1 is set, IO_CONFIG_SET_CURRENT_TO_DEFAULT
        will be done on timeout.

        Register: WATCHDOG_ADVANCED_DEFAULT

        Address: 61602
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_ADVANCED_DEFAULT)  # type: ignore

    @watchdog_advanced_default.setter
    @locked
    def watchdog_advanced_default(self, value: int) -> None:
        """
        A single binary-encoded value where each bit is an advanced option. If bit 0 is set,
        IO_CONFIG_SET_CURRENT_TO_FACTORY will be done on timeout. If bit 1 is set, IO_CONFIG_SET_CURRENT_TO_DEFAULT
        will be done on timeout.

        Register: WATCHDOG_ADVANCED_DEFAULT

        Address: 61602
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_ADVANCED_DEFAULT, value)

    @property
    @locked
    def watchdog_timeout_s_default(self) -> int:
        """
        When the device receives any communication over USB/Ethernet/WiFi, the watchdog timer is cleared. If the
        watchdog timer is not cleared within the timeout period, the enabled actions will be done.

        Register: WATCHDOG_TIMEOUT_S_DEFAULT

        Address: 61604
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_TIMEOUT_S_DEFAULT)  # type: ignore

    @watchdog_timeout_s_default.setter
    @locked
    def watchdog_timeout_s_default(self, value: int) -> None:
        """
        When the device receives any communication over USB/Ethernet/WiFi, the watchdog timer is cleared. If the
        watchdog timer is not cleared within the timeout period, the enabled actions will be done.

        Register: WATCHDOG_TIMEOUT_S_DEFAULT

        Address: 61604
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_TIMEOUT_S_DEFAULT, value)

    @property
    @locked
    def watchdog_startup_delay_s_default(self) -> int:
        """
        This specifies the initial timeout period at device bootup. This is used until the first time the watchdog
        is cleared or timeout ... after that the normal timeout is used.

        Register: WATCHDOG_STARTUP_DELAY_S_DEFAULT

        Address: 61606
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.WATCHDOG_STARTUP_DELAY_S_DEFAULT
        )

    @watchdog_startup_delay_s_default.setter
    @locked
    def watchdog_startup_delay_s_default(self, value: int) -> None:
        """
        This specifies the initial timeout period at device bootup. This is used until the first time the watchdog
        is cleared or timeout ... after that the normal timeout is used.

        Register: WATCHDOG_STARTUP_DELAY_S_DEFAULT

        Address: 61606
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_STARTUP_DELAY_S_DEFAULT, value)

    @property
    @locked
    def watchdog_strict_enable_default(self) -> int:
        """
        Set to 1 to enable strict mode.

        Register: WATCHDOG_STRICT_ENABLE_DEFAULT

        Address: 61610
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.WATCHDOG_STRICT_ENABLE_DEFAULT
        )

    @watchdog_strict_enable_default.setter
    @locked
    def watchdog_strict_enable_default(self, value: int) -> None:
        """
        Set to 1 to enable strict mode.

        Register: WATCHDOG_STRICT_ENABLE_DEFAULT

        Address: 61610
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_STRICT_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_strict_key_default(self) -> int:
        """
        When set to strict mode, this is the value that must be written to the clear register.

        Register: WATCHDOG_STRICT_KEY_DEFAULT

        Address: 61612
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_STRICT_KEY_DEFAULT)  # type: ignore

    @watchdog_strict_key_default.setter
    @locked
    def watchdog_strict_key_default(self, value: int) -> None:
        """
        When set to strict mode, this is the value that must be written to the clear register.

        Register: WATCHDOG_STRICT_KEY_DEFAULT

        Address: 61612
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_STRICT_KEY_DEFAULT, value)

    @property
    @locked
    def watchdog_strict_clear(self) -> int:
        """
        When running in strict mode, writing the key to this register is the only way to clear the watchdog. Writing
        to this register while not using strict mode will clear the watchdog.

        Register: WATCHDOG_STRICT_CLEAR

        Address: 61614
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_STRICT_CLEAR)  # type: ignore

    @watchdog_strict_clear.setter
    @locked
    def watchdog_strict_clear(self, value: int) -> None:
        """
        When running in strict mode, writing the key to this register is the only way to clear the watchdog. Writing
        to this register while not using strict mode will clear the watchdog.

        Register: WATCHDOG_STRICT_CLEAR

        Address: 61614
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_STRICT_CLEAR, value)

    @property
    @locked
    def watchdog_reset_enable_default(self) -> int:
        """
        Timeout action:
        Set to 1 to enable device-reset on watchdog timeout.

        Register: WATCHDOG_RESET_ENABLE_DEFAULT

        Address: 61620
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.WATCHDOG_RESET_ENABLE_DEFAULT
        )

    @watchdog_reset_enable_default.setter
    @locked
    def watchdog_reset_enable_default(self, value: int) -> None:
        """
        Timeout action:
        Set to 1 to enable device-reset on watchdog timeout.

        Register: WATCHDOG_RESET_ENABLE_DEFAULT

        Address: 61620
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_RESET_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_dio_enable_default(self) -> int:
        """
        Timeout action:
        Set to 1 to enable DIO update on watchdog timeout.

        Register: WATCHDOG_DIO_ENABLE_DEFAULT

        Address: 61630
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_ENABLE_DEFAULT)  # type: ignore

    @watchdog_dio_enable_default.setter
    @locked
    def watchdog_dio_enable_default(self, value: int) -> None:
        """
        Timeout action:
        Set to 1 to enable DIO update on watchdog timeout.

        Register: WATCHDOG_DIO_ENABLE_DEFAULT

        Address: 61630
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_dio_state_default(self) -> int:
        """
        The state high/low of the digital I/O after a Watchdog timeout. See DIO_STATE.

        Register: WATCHDOG_DIO_STATE_DEFAULT

        Address: 61632
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_STATE_DEFAULT)  # type: ignore

    @watchdog_dio_state_default.setter
    @locked
    def watchdog_dio_state_default(self, value: int) -> None:
        """
        The state high/low of the digital I/O after a Watchdog timeout. See DIO_STATE.

        Register: WATCHDOG_DIO_STATE_DEFAULT

        Address: 61632
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_STATE_DEFAULT, value)

    @property
    @locked
    def watchdog_dio_direction_default(self) -> int:
        """
        The direction input/output of the digital I/O after a Watchdog timeout. See DIO_DIRECTION.

        Register: WATCHDOG_DIO_DIRECTION_DEFAULT

        Address: 61634
        """
        return self.client.read_holding_register_ubint32(  # type: ignore
            RegisterAddresses.WATCHDOG_DIO_DIRECTION_DEFAULT
        )

    @watchdog_dio_direction_default.setter
    @locked
    def watchdog_dio_direction_default(self, value: int) -> None:
        """
        The direction input/output of the digital I/O after a Watchdog timeout. See DIO_DIRECTION.

        Register: WATCHDOG_DIO_DIRECTION_DEFAULT

        Address: 61634
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_DIRECTION_DEFAULT, value)

    @property
    @locked
    def watchdog_dio_inhibit_default(self) -> int:
        """
        This register can be used to prevent the watchdog from changing specific IO. See DIO_INHIBIT for more
        information.

        Register: WATCHDOG_DIO_INHIBIT_DEFAULT

        Address: 61636
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_INHIBIT_DEFAULT)  # type: ignore

    @watchdog_dio_inhibit_default.setter
    @locked
    def watchdog_dio_inhibit_default(self, value: int) -> None:
        """
        This register can be used to prevent the watchdog from changing specific IO. See DIO_INHIBIT for more
        information.

        Register: WATCHDOG_DIO_INHIBIT_DEFAULT

        Address: 61636
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DIO_INHIBIT_DEFAULT, value)

    @property
    @locked
    def watchdog_dac0_enable_default(self) -> int:
        """
        Timeout action:
        Set to 1 to enable DAC0 update on watchdog timeout.

        Register: WATCHDOG_DAC0_ENABLE_DEFAULT

        Address: 61640
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_DAC0_ENABLE_DEFAULT)  # type: ignore

    @watchdog_dac0_enable_default.setter
    @locked
    def watchdog_dac0_enable_default(self, value: int) -> None:
        """
        Timeout action:
        Set to 1 to enable DAC0 update on watchdog timeout.

        Register: WATCHDOG_DAC0_ENABLE_DEFAULT

        Address: 61640
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DAC0_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_dac0_default(self) -> float:
        """
        The voltage of DAC0 after a Watchdog timeout.

        Register: WATCHDOG_DAC0_DEFAULT

        Address: 61642
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.WATCHDOG_DAC0_DEFAULT)  # type: ignore

    @watchdog_dac0_default.setter
    @locked
    def watchdog_dac0_default(self, value: float) -> None:
        """
        The voltage of DAC0 after a Watchdog timeout.

        Register: WATCHDOG_DAC0_DEFAULT

        Address: 61642
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.WATCHDOG_DAC0_DEFAULT, value)

    @property
    @locked
    def watchdog_dac1_enable_default(self) -> int:
        """
        Timeout action:
        Set to 1 to enable DAC1 update on watchdog timeout.

        Register: WATCHDOG_DAC1_ENABLE_DEFAULT

        Address: 61650
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.WATCHDOG_DAC1_ENABLE_DEFAULT)  # type: ignore

    @watchdog_dac1_enable_default.setter
    @locked
    def watchdog_dac1_enable_default(self, value: int) -> None:
        """
        Timeout action:
        Set to 1 to enable DAC1 update on watchdog timeout.

        Register: WATCHDOG_DAC1_ENABLE_DEFAULT

        Address: 61650
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.WATCHDOG_DAC1_ENABLE_DEFAULT, value)

    @property
    @locked
    def watchdog_dac1_default(self) -> float:
        """
        The voltage of DAC1 after a Watchdog timeout.

        Register: WATCHDOG_DAC1_DEFAULT

        Address: 61652
        """
        return self.client.read_holding_register_bfloat(RegisterAddresses.WATCHDOG_DAC1_DEFAULT)  # type: ignore

    @watchdog_dac1_default.setter
    @locked
    def watchdog_dac1_default(self, value: float) -> None:
        """
        The voltage of DAC1 after a Watchdog timeout.

        Register: WATCHDOG_DAC1_DEFAULT

        Address: 61652
        """
        self.client.write_holding_register_bfloat(RegisterAddresses.WATCHDOG_DAC1_DEFAULT, value)

    @property
    @locked
    def internal_flash_key(self) -> int:
        """
        Sets the region of internal flash to which access is allowed.

        Register: INTERNAL_FLASH_KEY

        Address: 61800
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_KEY)  # type: ignore

    @internal_flash_key.setter
    @locked
    def internal_flash_key(self, value: int) -> None:
        """
        Sets the region of internal flash to which access is allowed.

        Register: INTERNAL_FLASH_KEY

        Address: 61800
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_KEY, value)

    @property
    @locked
    def internal_flash_read_pointer(self) -> int:
        """
        The address in internal flash that reads will start from.

        Register: INTERNAL_FLASH_READ_POINTER

        Address: 61810
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_READ_POINTER)  # type: ignore

    @internal_flash_read_pointer.setter
    @locked
    def internal_flash_read_pointer(self, value: int) -> None:
        """
        The address in internal flash that reads will start from.

        Register: INTERNAL_FLASH_READ_POINTER

        Address: 61810
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_READ_POINTER, value)

    @property
    @locked
    def internal_flash_read(self) -> int:
        """
        Data read from internal flash. Read size must be an even number of registers.

        Register: INTERNAL_FLASH_READ

        Address: 61812
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_READ)  # type: ignore

    @property
    @locked
    def internal_flash_erase(self) -> int:
        """
        Erases a 4k section of internal flash starting at the specified address. This register is a buffer.

        Register: INTERNAL_FLASH_ERASE

        Address: 61820
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_ERASE)  # type: ignore

    @internal_flash_erase.setter
    @locked
    def internal_flash_erase(self, value: int) -> None:
        """
        Erases a 4k section of internal flash starting at the specified address. This register is a buffer.

        Register: INTERNAL_FLASH_ERASE

        Address: 61820
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_ERASE, value)

    @property
    @locked
    def internal_flash_write_pointer(self) -> int:
        """
        Address in internal flash where writes will begin.

        Register: INTERNAL_FLASH_WRITE_POINTER

        Address: 61830
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_WRITE_POINTER)  # type: ignore

    @internal_flash_write_pointer.setter
    @locked
    def internal_flash_write_pointer(self, value: int) -> None:
        """
        Address in internal flash where writes will begin.

        Register: INTERNAL_FLASH_WRITE_POINTER

        Address: 61830
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_WRITE_POINTER, value)

    @property
    @locked
    def internal_flash_write(self) -> int:
        """
        Data written here will be written to internal flash. Write size must be an even number of registers. This
        register is a buffer.

        Register: INTERNAL_FLASH_WRITE

        Address: 61832
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_WRITE)  # type: ignore

    @internal_flash_write.setter
    @locked
    def internal_flash_write(self, value: int) -> None:
        """
        Data written here will be written to internal flash. Write size must be an even number of registers. This
        register is a buffer.

        Register: INTERNAL_FLASH_WRITE

        Address: 61832
        """
        self.client.write_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_WRITE, value)

    @property
    @locked
    def internal_flash_calculate_crc(self) -> int:
        """
        Calculates the checksum of a 4096 byte flash page.

        Register: INTERNAL_FLASH_CALCULATE_CRC

        Address: 61842
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.INTERNAL_FLASH_CALCULATE_CRC)  # type: ignore

    @property
    @locked
    def device_reset_dbg_reg(self) -> int:
        """
        Bitmask register that indicates why a device was last re-set.
        bit0:
        Power-on reset,
        bit1:
        Brown-out,
        bit2:
        Wake from idle,
        bit3:
        Wake from sleep,
        bit4:
        watchdog timer time-out,
        bit5:
        reserved,
        bit6:
        Software reset,
        bit7:
        external reset (MCLR),
        bit8:
        Voltage regulator standby,
        bit9:
        Configuration mismatch.

        Register: DEVICE_RESET_DBG_REG

        Address: 60014
        """
        return self.client.read_holding_register_ubint32(RegisterAddresses.DEVICE_RESET_DBG_REG)  # type: ignore
